C     FILEIO COMMON
C
C     Purpose: Common definition for FILE Input/Output routines.
C
C     History: - Creation (18.12.96, MK)
C                         (08.09.08, GG)
C                         (19.06.18, MA)
C                         (08.07.19, RDV)
C                         (25.10.19, FL)
C                         (27.11.19, AMK)
C                         (20.01.20, ADLL)
C
C     ******************************************************************
C
C     List of tapes:
C
C     a) ACCESS = 'SEQUENTIAL', FORM = 'FORMATTED'
C
C     AUX : Auxiliary function input file (AUXIS).
C     BAS : Basis set input file (BASIS).
C     ECP : ECP input file (ECPS).
C     MCP : MCP input file (MCPS).
C     PRM : Parameter file (FFDS).
C     INP : Input file (deMon.inp).
C     OUT : Output file (deMon.out).
C     GEO : Geometry file (deMon.geo).
C     NEW : New input file (deMon.new).
C     TRJ : Trajectory file (deMon.trj).
C     MOL : Molden input file (deMon.mol).
C     QMM : QM/MM interface file (deMon.qmm).
C     XRY : XAS/XES analysis file (deMon.xry).
C     NBO : NBO analysis input file (deMon.nbo).
C     PIE : Plot import/export file (deMon.pie).
C     CUB : User defined embedding cube (deMon.cub).
C     LAT : User defined external lattice (deMon.lat).
C     EDU : Input file for educt structure (deMon.edu).
C     PRO : Input file for product structure (deMon.pro).
C     C6D3: Empirical dispersion D3 parameter file (D3CFS).
C     PT  : Output file for ptmd exchange informations (deMon.pt).
C
C     b) ACCESS = 'SEQUENTIAL', FORM = 'UNFORMATTED'
C
CTODO NMR11: NMR grid tape.
CTODO NMR70: Dimension informations.
CTODO NMR71: Pointer informations.
CTODO NMR72: Molecule informations.
C     MEM  : Memory file (deMon.mem).
C     RST  : Restart file (deMon.rst).
C
C     c) ACCESS = 'DIRECT', STATUS = 'KEEP'
C
C     IOPLOT: Plot output file.
C
C     d) ACCESS = 'DIRECT', STATUS = 'DELETE'
C
C     IOADP: Auxiliary density perturbation scratch tape.
CTODO IOAOA: TDDFT AO scratch tape.
C     IOCDF: Charge Density Fit scratch tape.
C     IODE2: Second derivative scratch tape.
C     IODIS: MO DIIS scratch tape.
C     IODIR: Direct SCF scratch tape.
C     IOERI: ERI scratch tape.
C     IOETM: Constrained DFT (ET module) scratch tape.
C     IOEXX: EXX DIIS scratch tape.
CTODO IOFFD: Force field definition scratch tape.
C     IOFFP: Finit field perturbation scratch tape.
C     IOGRD: Grid scratch tape.
C     IOIRC: IRC path scratch tape.
C     IOISO: Isosurface scratch tape.
C     IOLRC: Linear response scratch tape.
C     IOMAG: Magnetic scratch tape.
C     IOMIX: Mixing scratch tape.
CTODO IOMOA: TDDFT MO scratch tape.
C     IOOPT: Optimization scratch tape.
C     IOPMR: Preconditioner (MINRES) scratch tape.
C     IORPA: TDDFT scratch tape.
C     IOSCF: SCF scratch tape.
C     IOSCP: SCP scratch tape.
C     IOTRJ: Trajectory scratch tape.
CTODO IOVEC: TDDFT scratch tape.
C
C     e) ACCESS = 'SEQUENTIAL', STATUS = 'DELETE'
C
C     IOPCG: Preconditioner (CG) scratch tape.
C     IOPRO: Promolecular density scratch tape.
C     IOTOP: Critical point scratch tape.
C
C     List of global input/output variables:
C
C     NREC(IO) : Number of records on tape IO.
C     RECNO(IO): Record number of tape IO.
C
C     List of tape position indices:
C
C     ADPIDX: Auxiliary density perturbation tape index.
C     FFPIDX: Finit field perturbation tape index.
C
C     ------------------------------------------------------------------
C
C     *** Input/Output parameters ***
C
      INTEGER AUX,BAS,CUB,C6D3,ECP,EDU,GEO,INP,IOADP,IOAOA,IOCDF,IODE2,
     $        IODIS,IODIR,IOERI,IOETM,IOEXX,IOFFD,IOFFP,IOGRD,IOIRC,
     $        IOISO,IOLRC,IOMAG,IOMIX,IOMOA,IOOPT,IOPCG,IOPLOT,IOPMR,
CCC ADLL ADD IORTP PDB RSTRT RTP
     $        IOPRO,IORPA,IOSCF,IOSCP,IOTOP,IOTRJ,LAT,MAXIO,MAXIDX,
     $        MAXSEQ,MCP,MEM,MINIO,MINSEQ,MOL,NBO,NEW,OUT,PIE,QMM,PRM,
     $        PRO,PT,RST,TRJ,XRY,IOVEC,NMR11,NMR70,NMR71,NMR72,IORTP,
     $        PDB,RSTRT,RTP,
     $        MLDAT,FFM,FFTOP
C
C     *** Input and output tapes ***
C
      PARAMETER (MINSEQ = 1,
     $           AUX    = 1,
     $           BAS    = 2,
     $           ECP    = 3,
     $           MCP    = 4,
     $           PRM    = 5,
     $           INP    = 6,
     $           OUT    = 7,
     $           GEO    = 8,
     $           NEW    = 9,
     $           TRJ    = 10,
     $           MOL    = 11,
     $           QMM    = 12,
     $           XRY    = 13,
     $           NBO    = 14,
     $           PIE    = 15,
     $           MEM    = 16,
     $           RST    = 17,
     $           CUB    = 18,
     $           LAT    = 19,
     $           EDU    = 20,
     $           PRO    = 21,
     $           C6D3   = 22,
     $           PT     = 23,
     $           RTP    = 24,
     $           PDB    = 25,
     $           RSTRT  = 26,
     $           MAXSEQ = 26,
     $           NMR11  = 27,
     $           NMR70  = 28,
     $           NMR71  = 29,
     $           NMR72  = 30,
     $           MLDAT  = 31,
     $           FFM    = 32,
     $           FFTOP  = 33)
C
C     *** Scratch tapes ***
C
      PARAMETER (IOPLOT = 49,
     $           MINIO  = 50,
     $           IOADP  = 50,
     $           IOAOA  = 51,
     $           IOCDF  = 52,
     $           IODE2  = 53,
     $           IODIS  = 54,
     $           IODIR  = 55,
     $           IOERI  = 56,
     $           IOETM  = 57,
     $           IOEXX  = 58,
     $           IOFFD  = 59,
     $           IOFFP  = 60,
     $           IOGRD  = 61,
     $           IOIRC  = 62,
     $           IOISO  = 63,
     $           IOLRC  = 64,
     $           IOMAG  = 65,
     $           IOMIX  = 66,
     $           IOMOA  = 67,
     $           IOOPT  = 68,
     $           IOPCG  = 69,
     $           IOPMR  = 70,
     $           IOPRO  = 71,
     $           IORPA  = 72,
     $           IOSCF  = 73,
     $           IOSCP  = 74,
     $           IOTOP  = 75,
     $           IOTRJ  = 76,
     $           IOVEC  = 77,
     $           IORTP  = 78,
     $           MAXIO  = 78)
C
C     *** Maximum tape index bound ***
C
      PARAMETER (MAXIDX = 25)
C
C     *** Common blocks for FILE Input/Output variables ***
C
      CHARACTER*20 TAPENAME(1:MAXIO),TAPEROOT(1:MAXIO)
C
      COMMON /CFILEIO/ TAPENAME,TAPEROOT
C
      INTEGER NREC(1:MAXIO),RECNO(1:MAXIO)
      INTEGER ADPIDX(0:MAXIDX),DE2IDX(0:MAXIDX),FFPIDX(0:MAXIDX),
     $        LRCIDX(0:MAXIDX),MAGIDX(0:MAXIDX),SCPIDX(0:MAXIDX)
C
      COMMON /IFILEIO/ ADPIDX,DE2IDX,FFPIDX,LRCIDX,MAGIDX,NREC,RECNO,
     $                 SCPIDX
C
C     ------------------------------------------------------------------
