C     GRID COMMON
C
C     Purpose: Common block definition for the grid generation.
C
C     History: - Creation (01.11.95, MK)
C                         (03.12.04, RFM)
C                         (14.10.10, GG)
C                         (06.12.17, AMK)
C
C     ******************************************************************
C
C     List of logical variables:
C
C     PRUNED: If true, grid pruning is enabled.
C     REFGRD: If true, use a reference grid.
C     REFTOP: If true, use a reference grid in topological analysis.
C
C     List of integer variables:
C
C     LLGP(IATOM)   : Number of the first grid point of atom IATOM.
C     LRADIAL       : L quantum numbers of radial shells.
C     LSCODE(IATOM) : Local symmetry code of atom IATOM.
C     NATOMBUF      : Number of atoms is the current buffer (record).
C     NEIGH(IATOM)  : List of neighbors of atom IATOM.
C     NGPATOM       : Number of grid points for an atom.
C     NGPBUF        : Number of grid points in buffer.
C     NGPFAIL       : Number of failed grid points in search or Gxc.
C     NGPGXC        : Number of grid points visited in Gxc calculation.
C     NGPSPH        : Number of grid points for an sphere.
C     NGPTOT        : Total number of grid points.
C     NGPVEC(IATOM) : Vector with number of grid points for IATOM.
C     NGPZERO       : Number of grid points with zero density.
C     NNEIGH        : Number of neighbors.
C     NPGPATOM      : Number of progrid points for an atom.
C     NPGPTOT       : Total number of progrid points.
C     NRADIAL(IATOM): Number of radial shells for atom IATOM.
C     NRADGRD       : User defined radial shells for SCF grid.
C     NRADTOP       : User defined radial shells for topological grid.
C     ULGP(IATOM)   : Number of the last grid point of atom IATOM.
C
C     List of real variables:
C
C     ATOMRAD(IATOM): Maximum radius of atom.
C     AUXSETRAD     : Maximum radius of auxiliary function set.
C     AUXSHLRAD     : Maximum radius of auxiliary function shell.
C     CUSPRAD(IATOM): Maximum radius of atom cusp.
C     G98RAD        : Atomic radii for G98 grids.
C     GRDVEC(1,IGP) : X coordinate of grid point IGP.
C     GRDVEC(2,IGP) : Y coordinate of grid point IGP.
C     GRDVEC(3,IGP) : Z coordinate of grid point IGP.
C     GRDVEC(4,IGP) : Weight of grid point IGP.
C     SHLRAD        : Maximum radius of orbital shell.
C     TOPRAD        : Maximum topological radius.
C
C     ------------------------------------------------------------------
C
C     *** Common block for LOGICAL grid variables ***
C
      LOGICAL PRUNED,REFGRD,REFTOP
C
      COMMON /LGRID/ PRUNED,REFGRD,REFTOP
C
C     *** Common block for INTEGER grid variables ***
C
      INTEGER NATOMBUF,NGPATOM,NGPBUF,NGPFAIL,NGPGXC,NGPSPH,NGPTOT,
     $        NGPZERO,NNEIGH,NPGPATOM,NPGPTOT,NRADGRD,NRADTOP
CC ADLL GRID
     $        ,NRADTGRD
CCC ADLL GRID
C
      INTEGER LLGP(MAXATOM),LRADIAL(MAXATOM,MAXRAD),LSCODE(MAXATOM),
     $        NEIGH(0:MAXATOM),NGPVEC(MAXATOM),NRADIAL(MAXATOM),
     $        ULGP(MAXATOM)
C
      COMMON /IGRID/ LLGP,LRADIAL,LSCODE,NATOMBUF,NEIGH,NGPATOM,
     $               NGPBUF,NGPFAIL,NGPGXC,NGPSPH,NGPTOT,NGPVEC,
     $               NGPZERO,NNEIGH,NPGPATOM,NPGPTOT,NRADIAL,
     $               NRADGRD,NRADTOP,ULGP
CCC ADLL GRID
     $        ,NRADTGRD
CCC ADLL GRID
C
C     *** Common block for REAL grid variables ***
C
      REAL ATOMRAD(MAXATOM),AUXSETRAD(MAXAUXSET),AUXSHLRAD(MAXAUXSHL),
     $     CUSPRAD(MAXATOM),G98RAD(111),GRDVEC(4,0:MAXGP),
     $     SHLRAD(MAXSHL),TOPRAD(MAXATOM)
C
      COMMON /RGRID/ ATOMRAD,AUXSETRAD,AUXSHLRAD,CUSPRAD,G98RAD,
     $               GRDVEC,SHLRAD,TOPRAD
C
C     ------------------------------------------------------------------
