C     MM COMMON
C
C     Purpose: Common definition for molecular mechanics (mm) global
C              variables.
C
C     History: - Creation (10.11.08, AG) 
C                         (14.07.10, AG)
C                         (13.05.14, AMK)
C                         (13.11.14, GG)
C
C     ******************************************************************
C
C     List of variables:
C
C     EBANG   : Molecular mechanics bond angle energy.
C     EBOND   : Molecular mechanics bond energy.
C     EMM     : Molecular mechanics total energy.
C     EQQMM   : Molecular mechanics Coulomb energy.
C     ETORS   : Molecular mechanics torsion energy.
C     EIMPTORS: Molecular mechanics improper torsion energy.
C     EUREY   : Molecular mechanics Urey-Bradley energy.
C     EVDWMM  : Molecular mechanics van der Waals energy.
C     LINKMM  : Link information for molecular mechanics atoms.
C     CBOND   : Connector list for molecular mechanics bonds.
C     NBOND   : Number of molecular mechanics bonds.
C     TYPEMM  : Type of molecular mechanics atoms.
C     VDWINDX : Indexing type for vdW parameter.
CCC ADLL LA
C     TYPLNA: MM types of two atoms involved in link atom.
C     NTYPLA: Number of MM types of QM atoms involved in QM/MM frontier
C     NLBANG: Number of MM BANGS terms involving QM atoms.
C     LBOND : Connector list for MM bonds involving QM atom.
C             Entry(1): QM atom.
C             Entry(2): MM atom.
C     LBANGS: Connector list for MM angle involving QM atom.
C             Entry(1): QM atom.
C             Entry(2): MM atom.
C             Entry(3): MM atom.
C     LTANGS: Connector list for MM torsions involving QM atom.
C     KLBOND: Force constant for MM bonds involing one QM atom.
C     LBOND0: Referece value for MM bonds involving one QM atom.
C     KLANG : Force constant for MM angles involving one QM atom.
C     LANG0 : Reference value for MM angles involving one QM atom.
CCC ADLL LA
CCC ADLL MMPOL
C    INDTHR : Threshold controlling convergence of dipoles.
C    EIND   : Induction energy.
C    APOLMM : Atomic polarizabilities.
C    DPOLMM : Thole damping factors.
C    MUMM   : MM induced dipoles.
CCC ADLL MMPOL
	

CTODO
CTODO
CTODO
C
C     List of variables:
C
C     TYPEQM: Type of each QM atom.
C     NTYPES: Number of different MM Types (from deMon.prm).
C     TYPES : Types of MM atoms used.
C     MMIMPTANG: Points on the improper torsion angle atoms.
C
C     ------------------------------------------------------------------
C
C     *** Common block for INTEGER molecular mechanics variables ***
C
      INTEGER NBOND
C
      INTEGER CBOND(MAXLINK*MAXATMM,2),LINKMM(MAXATOM_ALL,0:MAXLINK),
     $        TYPEMM(MAXATOM_ALL)
CCC ADLL LA
     $        ,LBOND(MAXLNKAT,2),LBANGS(MAXLNKAT*MAXLINK,3),
     $        LTANGS(MAXTANG,4)
CCC ADLL LA
C
      COMMON /IMM_CAMK/ CBOND,
CCC ADLL LA
     $         LBOND,LBANGS,LTANGS,
CCC ADLL LA
     $        LINKMM,NBOND,TYPEMM
C
C     *** Common block for REAL molecular mechanics variables ***
C
      REAL EBANG,EBOND,EIMPTORS,EMM,EQQMM,ETORS,EUREY,EVDWMM
CCC ADLL MMPOL
     $     ,EIND,INDTHR
CCC ADLL MMPOL
C

C
      COMMON /RMM_CAMK/ EBANG,EBOND,EIMPTORS,EMM,EQQMM,ETORS,EUREY,
     $                  EVDWMM
CCC ADLL MMPOL
     $     ,EIND,INDTHR
CCC ADLL MMPOL
C
CTODO
CTODO
CTODO
C     *** Common block for Logical variables ***
C
      LOGICAL MMFILE
CCC ADLL MMPOL
     $       ,INDPSCF
CCC ADLL MMPOL

      COMMON /LOGMM/ MMFILE
CCC ADLL MMPOL
     $       ,INDPSCF
CCC ADLL MMPOL
C
C     *** Common block for CHARACTER variables ***
C
CTODO
CTODO Move FFIELD to molecule.h
CTODO
C
      CHARACTER*10 FFIELD,VDWINDX
C
      COMMON /CMM_OLD/ FFIELD,VDWINDX
C
C     *** Common block for REAL variables ***
C
      REAL ANGLE0(MAXBANG),
     $     BOND0(MAXBOND),
CCC ADLL LA
     $     LBOND0(MAXLNKAT),LANG0(MAXLNKAT*MAXLINK),
CCC ADLL LA
     $      CHGMM(MAXATMM),
CCC ADLL MMPOL
     $     APOLMM(MAXATMM),DPOLMM(MAXATMM),MUMM(MAXATMM,3,10),
     $     EFLDX_OLD(MAXATMM),EFLDY_OLD(MAXATMM),EFLDZ_OLD(MAXATMM),
CCC ADLL MMPOL
     $     IMPTANG1(MAXTYPE,2),IMPTANG2(MAXTYPE,2),IMPTANG3(MAXTYPE,2),
     $     IMPTORS1(MAXTANG,2),IMPTORS2(MAXTANG,2),IMPTORS3(MAXTANG,2),
     $     KANGLE(MAXBANG),KBOND(MAXBOND),
CCC ADLL LA
     $     KLBOND(MAXLNKAT),KLANG(MAXLNKAT*MAXLINK),LTORS1(MAXTANG,2),
     $     LTORS2(MAXTANG,2),LTORS3(MAXTANG,2),
CCC ADLL LA
     $     KUREYB(MAXBANG),
     $     REDVDW(MAXATMM),TORS1(MAXTANG,2),
     $     TORS2(MAXTANG,2),TORS3(MAXTANG,2),UREYB0(MAXBANG),
     $     VLJEPS(MAXATMM),VLJRAD(MAXATMM)
     $     ,tors4(maxtang,2)
CCC ADLL
     $     ,MUMOL(MAXATMM,3)
CCC ADLL
C
      COMMON /RMM/ ANGLE0,BOND0,
CCC ADLL MMPOL
     $             APOLMM,DPOLMM,MUMM,
     $             EFLDX_OLD,EFLDY_OLD,EFLDZ_OLD,
CCC ADLL MMPOL
CCC ADLL LA
     $             LBOND0,LANG0,KLBOND,KLANG,
CCC ADLL LA
     $             CHGMM,IMPTANG1,IMPTANG2,IMPTANG3,
     $             IMPTORS1,IMPTORS2,IMPTORS3,KANGLE,
     $             KBOND,
CCC ADLL LA
     $             LTORS1,LTORS2,LTORS3,
CCC ADLL LA
     $             KUREYB,REDVDW,
     $             TORS1,TORS2,TORS3,UREYB0,VLJEPS,VLJRAD
     $             ,tors4
CCC ADLL
     $     ,MUMOL
CCC ADLL

C
C     *** Common block for INTEGER variables ***
C
      INTEGER NIMPTS,NIMPTORS,NBANGS,NTANGS,NTYPES,
CCC ADLL LA
     $        NTYPLA,NLBANG,NLTANG,NLTANG1,
CCC ADLL LA
     $        NTYPLNK,NUREYB,
     $        NIMTORS,MBANGS,MTANGS
C
      INTEGER BANGS(MAXBANG,3),
     $        IMPT(MAXATMM,4),
     $        IREDVDW(MAXATMM),
     $        MMCLASS(MAXATMM),
     $        NVALMM(MAXATMM),
     $        TANGS(MAXTANG,4),
CCC ADLL LA
     $        TYPLA(MAXLNKAT,2),
CCC ADLL LA
     $        TYPES(MAXATMM),TYPLNK(MAXBOND,2),
     $        UREY(MAXBANG,2),
     $        MMIMPTANG(MAXTYPE,4),
     $        IIMTORS(MAXTANG,4)
C
      COMMON /IMM_OLD/ BANGS,IMPT,IREDVDW,
     $             MMCLASS,NBANGS,MBANGS,
     $             NIMPTS,NIMPTORS,
     $             NTANGS,MTANGS,
     $             NTYPES,
CCC ADLL LA
     $             NTYPLA,NLBANG,NLTANG,NLTANG1,
CCC ADLL LA
     $             NTYPLNK,NUREYB,NVALMM,TANGS,
     $             TYPES,
CCC ADLL LA
     $             TYPLA,
CCC ADLL LA
     $             TYPLNK,UREY,
     $             NIMTORS,
     $             MMIMPTANG,
     $             IIMTORS
C
C     ------------------------------------------------------------------
