C     MOLECULE COMMON
C
C     Purpose: Common definition for molecule data.
C
C     History: - Creation (16.01.96, AMK)
C                         (24.06.00, AMK)
C                         (19.06.18, MA)
C
C     ******************************************************************
C
C     List of variables:
C
C     ACCEL     : Nuclear accelerations.
C     ANGBOR    : Unit conversion factor.
C                 Entry 1: Length conversion.
C                 Entry 2: Velocity conversion.
C                 Entry 3: Acceleration conversion.
C     ATOMAUX   : Atomic auxiliary function label.
C     ATOMBAS   : Atomic basis set label.
C                 Entry 1: Number of atom.
C                 Entry 2: 1 --> Basis set.
C                          2 --> Basis set for projection.
C                          3 --> Augmented basis set.
C     ATOMECP   : Atomic ECP labels.
C     ATOMLAB   : Atomic label.
C     ATOMMAP   : Atomic mapping vector.
C                 Entry 1 maps: CCSET = 1 --> CCSET = 3.
C                 Entry 2 maps: CCSET = 3 --> CCSET = 1.
C                 Entry 3 maps: CCSET = 4 --> CCSET = 3.
C                 Entry 4 maps: CCSET = 3 --> CCSET = 4.
C     ATOMMCP   : Atomic MCP labels.
C     ATOMNO    : Atomic numbers.
C     CCSET     : Number of the currently used coordinate set.
C                 CCSET = 1, Cartesian (Z-Matrix).
C                 CCSET = 2, Cartesian standard orientation.
C                 CCSET = 3, Internal (Z-matrix).
C                 CCSET = 4, Cartesian input.
C                 CCSET = 5, Work coordinates.
C                 CCSET = 6, Step coordinates.
C                 CCSET = 7, Cartesian reference coordinates.
C     CHARGE    : Molecular charge.
C     C6REF     : C6 coefficients from reference D3CFS file.
C     CNREF     : Coordination numbers from reference D3CFS file.
C     COORD     : Atomic coordinates.
C     CPRIMS    : Connector list of primitive coordinates.
C                 Entry 1: First atom of primitive coordinate.
C                 Entry 2: Second atom of primitive coordinate.
C                 Entry 3: Third atom of primitive coordinate.
C                 Entry 4: Fourth atom of primitive coordinate.
C                 Entry 5: Indicates if coordinate is constant.
C                 Entry 6: Indicates status of coordinate sets.
C                 Entry 7: Indicates deleted primitive coordinate.
C
C                 Status          CPRIMS(5)         CPRIMS(6)
C                 Variable           0                 0
C                 Constant        -99999            -99999
C                 Equivalence     -99999        # of first in set
C     C6ATOM    : Atomic C6 coefficients for dispersion energy.
C     GCC       : Gaussian contraction coefficients.
C     ECPGCC    : Gaussian contraction coefficients of ECP.
C     ECPZET    : Gaussian exponents of ECP.
C     EPES      : Energy of single-point on potential energy surface.
C     FIXCORE   : Exponent limit for freezing option CORE.
C     FIXCUSP   : Exponent limit for freezing option CUSP.
C     FIXNONE   : Exponent limit for freezing option NONE.
C     FIXVALENCE: Exponent limit for freezing option VALENCE.
C     LAUXMAX   : Maximum L for free auxiliary sets of atoms.
C     LAUXMAX(0): Maximum L for free auxiliary sets of the molecule.
C     LAXIS     : Local atomic coordinate system.
C     LBASMAX   : Maximum L for orbital shells of atoms.
C     LBASMAX(0): Maximum L for orbital shells of the molecule.
C     LECPMAX   : Maximum L for ECP functions of atoms.
C     LECPMAX(0): Maximum L for ECP functions of the molecule.
C     LFIXMAX   : Maximum L for fixed auxiliary sets of atoms.
C     LFIXMAX(0): Maximum L for fixed auxiliary sets of the molecule.
C     LMCPMAX   : Maximum L for MCP core functions of atoms.
C     LMCPMAX(0): Maximum L for MCP core functions of the molecule.
C     MATOM     : Atomic mass.
C     MCPGCC    : Gaussian contraction coefficients of MCP.
C     MCPZET    : Gaussian exponents of MCP.
C     MCPEW     : MCP Energy Weights for the projector.
C     MMOLECULE : Total mass of the molecule.
C     MULTIP    : Spin multiplicity.
C     NADB      : Number of elements in atomic diagonal blocks.
C     NALL      : Number of all (free and fixed) auxiliary functions.
C     NALLSET   : Number of all auxiliary function sets.
C     NALLSHL   : Number of all auxiliary function shells.
C     NATMM     : Number of MM atoms.
C     NATOM_ALL : Number of atoms QM + MM.
C     NATOM     : Number of QM atoms.
C     NATXYZ    : Entry 1: Number of Cartesian atoms in mixed input.
C                 Entry 2: Number of generated Cartesian dummy atoms.
C                 Entry 3: Number of all atoms in mixed input.
C                 Entry 4: Number of Cartesian input atoms.
C     NAUX      : Number of auxiliary functions.
C     NAUXSET   : Number of auxiliary function sets.
C     NAUXSHL   : Number of auxiliary function shells.
C     NA        : Connectivity description for length.
C     NB        : Connectivity description for angle.
C     NC        : Connectivity description for dihedral angle.
C     NCND3     : Number of coordination numbers from reference D3CFS file.
C     NCAUX     : Normalization constant for auxiliary functions.
C     NCMCPSTO  : Normalization constant for MCP Slater typ orbitals.
C     NCODE     : Flag for angle/dihedral angle substitution.
C     NCSTO     : Normalization constant for Slater typ orbitals.
C     NDUMMY    : Number of dummy atoms.
C     NECPGTO   : Number of different ECP Gaussians.
C     NECPSHL   : Number of ECP shells.
C     NELEC     : Number of electrons.
C     NFIX      : Number of fixed auxiliary functions.
C     NFIXSET   : Number of fixed auxiliary function sets.
C     NFIXSHL   : Number of fixed auxiliary function shells.
C     NGTO      : Number of different Gaussians (without L).
C     NMCPGTO   : Number of different MCP Gaussians.
C     NMCPSHL   : Number of MCP shells.
C     NMCPSTO   : Number of MCP orbitals.
C     NORB      : Number of orbitals (MOs) used in the SCF iteration.
C     NORTH     : Number of orthonormal basis functions.
C     NORTH_TB  : Number of orthonormal tight-binding basis functions.
C     NPRIMS    : Number of primitive length and angle connectors.
C     NSHL      : Number of shells.
C     NSPH      : Number of spherical orbitals.
C     NSTO      : Number of orbitals.
C     NXAS      : Number of extended XAS virtual orbitals.
C     OPTFLAG   : Z-Matrix optimization flags for internal coordinates.
C     OPTVAR    : Z-Matrix optimization variable and constant names.
C     OPTXYZ    : Cartesian freezing string for coordinates.
C     PRILAB    : Primitive internal coordinate label.
C     QAUX      : Charge moments of auxiliary function.
C     QPRIMS    : Primitive internal coordinates.
C     RAB(I)    : Interatomic coordinates between A and B.
C     RAC(I)    : Interatomic coordinates between A and C.
C     RBC(I)    : Interatomic coordinates between B and C.
C     RSQ       : Interatomic square distance between A and B.
C     VELOS     : Nuclear velocities.
C     ZATOM     : Nuclear charges.
C     ZCORE     : Core charge.
C     ZET       : Gaussian exponents.
C     ZETFIX    : Exponent limit for fixed auxiliary functions.
C
C     ------------------------------------------------------------------
C
C     *** Common block for input character data ***
C
      CHARACTER*3 NCODE(MAXATOM_ALL),OPTXYZ(MAXATOM_ALL)
      CHARACTER*10 ATOMLAB(MAXATOM_ALL),OPTVAR(MAXATOM_ALL,3)
      CHARACTER*11 OPTFLAG(MAXATOM_ALL,3),PRILAB(MAXPRI)
      CHARACTER*80 ATOMAUX(0:MAXATOM),ATOMBAS(0:MAXATOM,3),
     $             ATOMECP(0:MAXATOM),ATOMMCP(0:MAXATOM)
C
      COMMON /CMOLECULE/ ATOMAUX,ATOMBAS,ATOMECP,ATOMLAB,ATOMMCP,
     $                   NCODE,OPTFLAG,OPTVAR,OPTXYZ,PRILAB
C
C     *** Common block for molecule specific data ***
C
      INTEGER CCSET,CHARGE,MULTIP,NADB,NALL,NALLSET,NALLSHL,NATMM,
     $        NATOM,NATOM_ALL,NAUX,NAUXSET,NAUXSHL,NDUMMY,NECPGTO,
     $        NECPSHL,NELEC,NFIX,NFIXSET,NFIXSHL,NGTO,NMCPGTO,NMCPSHL,
     $        NMCPSTO,NORB,NORTH,NORTH_TB,NSHL,NSPH,NSTO,NXAS
CCC ADLL
     $        ,NMOL
CCC ADLL
C
      INTEGER ATOMMAP(MAXATOM_ALL,4),ATOMNO(MAXATOM_ALL),
     $        CPRIMS(MAXPRI,7),NATXYZ(4),LAUXMAX(0:MAXATOM),
     $        LFIXMAX(0:MAXATOM),LBASMAX(0:MAXATOM),LECPMAX(0:MAXATOM),
     $        LMCPMAX(0:MAXATOM),NA(MAXATOM_ALL),NB(MAXATOM_ALL),
     $        NC(MAXATOM_ALL),NCND3(MAXD3ATM),NPRIMS(-3:3),
     $        ZATOM(MAXATOM_ALL),ZCORE(MAXATOM_ALL)
CCC ADLL
     $       ,FRAGMOL(MAXATOM_ALL)
CCC ADLL
C
      COMMON /IMOLECULE/ ATOMMAP,ATOMNO,CCSET,CHARGE,CPRIMS,LAUXMAX,
     $                   LBASMAX,LECPMAX,LFIXMAX,LMCPMAX,MULTIP,NA,
     $                   NADB,NALL,NALLSET,NALLSHL,NATMM,NATOM,
     $                   NATOM_ALL,NATXYZ,NAUX,NAUXSET,NAUXSHL,NB,NC,
     $                   NCND3,NDUMMY,NECPGTO,NECPSHL,NELEC,NFIX,
     $                   NFIXSET,NFIXSHL,NGTO,NMCPGTO,NMCPSHL,NMCPSTO,
     $                   NORB,NORTH,NORTH_TB,NPRIMS,NSHL,NSPH,NSTO,
     $                   NXAS,ZATOM,ZCORE
CCC ADLL
     $                  ,NMOL,FRAGMOL
CCC ADLL
C
C     *** Common block for geometry and Gaussians ***
C
      REAL EPES,FIXCORE,FIXCUSP,FIXNONE,FIXVALENCE,MMOLECULE,RSQ
CCC ADLL
     $    ,NELA,NELB,NELCD
CCC ADLL
C
      REAL ACCEL(3,MAXATOM_ALL),ANGBOR(3),COORD(3,MAXATOM_ALL,7),
     $     C6ATOM(MAXATOM),C6REF(MAXD3ATM,MAXD3ATM,MAXD3CN,MAXD3CN),
     $     CNREF(MAXD3ATM,MAXD3CN),ECPGCC(MAXECPGTO),ECPZET(MAXECPGTO),
     $     GCC(MAXGTO),LAXIS(0:MAXATOM_ALL,3,3),MATOM(MAXATOM_ALL),
     $     MCPEW(MAXMCPSHL),MCPGCC(MAXMCPGTO),MCPZET(MAXMCPGTO),
     $     NCAUX(MAXAUX),NCMCPSTO(MAXMCPSTO),NCSTO(MAXSTO),QAUX(MAXAUX),
     $     QPRIMS(MAXPRI,2),RAB(3),RAC(3),RBC(3),VELOS(3,MAXATOM_ALL),
     $     ZET(MAXGTO + MAXAUXSET),ZETFIX(0:MAXATOM)
C
      COMMON /RMOLECULE/ ACCEL,ANGBOR,COORD,C6ATOM,C6REF,CNREF,ECPGCC,
     $                   ECPZET,EPES,FIXCORE,FIXCUSP,FIXNONE,FIXVALENCE,
     $                   GCC,LAXIS,MATOM,MCPEW,MCPGCC,MCPZET,MMOLECULE,
     $                   NCAUX,NCMCPSTO,NCSTO,QAUX,QPRIMS,RAB,RAC,RBC,
     $                   RSQ,VELOS,ZET,ZETFIX
CCC ADLL
     $    ,NELA,NELB,NELCD
CCC ADLL
C
C     ------------------------------------------------------------------
