      SUBROUTINE EBOUNDS(H,PA,PB,CDC,CDJ,DSTO)
C
C     Purpose: Calculation of SCF energy bounds.
C
C     History: - Creation (30.07.02, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     CDC   : Charge density coefficients.
C     CDJ   : Charge density Coulomb vector.
C     ECORE : Core SCF energy.
C     ELOWER: Lower SCF energy bound.
C     EUPPER: Upper SCF energy bound.
C     H     : Core Hamiltonian.
C     PA    : Alpha density matrix.
C     PB    : Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
CCC ADLL
      INCLUDE 'projectile.h'
      INCLUDE 'qmmm.h'
CCC ADLL
      INCLUDE 'scf.h'
C
      INTEGER DSTO
      REAL H(DSTO,DSTO),PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      REAL CDC(MAXAUX),CDJ(MAXAUX)
C
      INTEGER IAUX
      REAL ECORE,TRACE
CCC ADLL
      LOGICAL STATED
CCC ADLL
C
C     ------------------------------------------------------------------
C
C     *** Load core Hamiltonian ***
C
      CALL SCFIO(H,DSTO,'READ','CORE')
C
C     *** Calculation of core energy ***
C
      ECORE = TRACE(PA,H,DSTO,NSTO,'LOWERA')
C
      IF (OKS) ECORE = ECORE + TRACE(PB,H,DSTO,NSTO,'LOWERA')
CCC ADLL
C     *** Calculation of QM/MM interaction energy ***
C
      IF (QMMM) THEN
        CALL SCFIO(H,DSTO,'READ','PCHGMM')
        EPCQM = TRACE(PA,H,DSTO,NSTO,'LOWERA')
        IF (OKS) EPCQM = EPCQM + TRACE(PB,H,DSTO,NSTO,'LOWERA')
      END IF
CCC ADLL
C
CCC ADLL
C     *** Calculation of interaction with projectile ***
C
      ERADIA(:) = 0.0
      IF (ADDRAD) THEN
        IF ((.NOT.(STATED('ATOM'))).AND.(STATUS.EQ.'RTDFT')) THEN
          CALL GETEPROJ(PA,PB,H,DSTO)
        END IF
      END IF
C
CCC ADLL
C     *** Add contribution of fixed auxiliary functions ***
C
      IF (FITTYP.EQ.'VALENCE') ECORE = ECORE + EFIX
C
C     *** Build total charge density for lower energy bound ***
C
      DO IAUX=1,NAUX
        CDC(IAUX) = CDCA(IAUX)
        CDJ(IAUX) = CDJA(IAUX) - 0.5*GCDCA(IAUX)
      END DO
C
      IF (OKS) THEN
        DO IAUX=1,NAUX
          CDC(IAUX) = CDC(IAUX) + CDCB(IAUX)
          CDJ(IAUX) = CDJ(IAUX) + CDJB(IAUX) - 0.5*GCDCB(IAUX)
        END DO
      END IF
C
C     *** Calculation of lower energy bound ***
C
      ELOWER = ECORE
C
      DO IAUX=1,NAUX
        ELOWER = ELOWER + CDC(IAUX)*CDJ(IAUX)
      END DO
C
C     *** Build total charge density for upper energy bound ***
C
      DO IAUX=1,NAUX
        CDC(IAUX) = FCDA(IAUX)
        CDJ(IAUX) = CDJA(IAUX) - 0.5*GFCDA(IAUX)
      END DO
C
      IF (OKS) THEN
        DO IAUX=1,NAUX
          CDC(IAUX) = CDC(IAUX) + FCDB(IAUX)
          CDJ(IAUX) = CDJ(IAUX) + CDJB(IAUX) - 0.5*GFCDB(IAUX)
        END DO
      END IF
C
C     *** Calculation of upper energy bound ***
C
      EUPPER = ECORE
C
      DO IAUX=1,NAUX
        EUPPER = EUPPER + CDC(IAUX)*CDJ(IAUX)
      END DO
C
C     *** Save nucleus (core) and Coulomb energy ***
C
      ENUCLEUS = ECORE
      ECOULOMB = EUPPER - ECORE
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE EBOUNDS ***
C
      END
