      SUBROUTINE AUTOLEB(ATOM,SPHNUM,SPHRAD,WLEB,LSTART,PA,PB,DSTO)
C
C     Purpose: Generating of Lebedev grids for the automatic 
C              numerical integrator.
C
C     History: - Creation (19.11.95, MK)
C                         (28.01.02, AMK)
C                         (02.06.05, FJ)
C
C     ******************************************************************
C
C     List of local variables:
C
C     ATOM  : Atom at which radial integration shell is located.
C     ERROR : Maximum difference between two integrations.
C     LEND  : Maximum L value.
C     LSTART: L value of the first angular grid.
C     PA    : Alpha density matrix.
C     PB    : Beta density matrix.
C     SPHNUM: Number of radial shell.
C     SPHRAD: Radius of the actual sphere (atomic coordinate system).
C     WLEB  : Lebedev grid weight times volume element.
C
C     List of local dynamical fields:
C
C     NEWVEC: Generating function values of current cycle.
C     OLDVEC: Generating function values of last cycle.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'lebedev.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'pointer.h'
      INCLUDE 'tolerance.h'
C
      INTEGER DSTO
      REAL PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      INTEGER ATOM,SPHNUM
C
      CHARACTER*12 TRUNCATED
      INTEGER ILG,L,LEND,LGNUMOF,LSTART
      INTEGER CYCLE,INEIGH,IATOM,IAUX,IAUXSET,IAUXSHL,IFIX,IFIXSET
      INTEGER IFIXSHL,ISTO
      INTEGER OLDNGP,STREXT
      REAL ERROR,SPHRAD,WLEB
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: NEWVEC(:),OLDVEC(:)
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      CYCLE = 0
C
      TRUNCATED = ' '
C
C     *** Allocate local work vectors ***
C
CCC ADLL
C 
      IF (STATUS.EQ.'ESA'.AND.TOPCD) THEN
        ALLOCATE(NEWVEC(NALL),OLDVEC(NALL),STAT=ALLOCATION)
      ELSE IF (STATUS.EQ.'ESA') THEN
        ALLOCATE(NEWVEC(NSTO),OLDVEC(NSTO),STAT=ALLOCATION)
      ELSE IF (CDXC) THEN
CCC ADLL
        ALLOCATE(NEWVEC(NALL),OLDVEC(NALL),STAT=ALLOCATION)
      ELSE
        ALLOCATE(NEWVEC(NSTO),OLDVEC(NSTO),STAT=ALLOCATION)
      END IF
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('AUTOLEB','ALLOCATION FAILED',1)
      END IF
C
C     *** Define upper limit for the angular grid size ***
C
      LEND = MIN(5 - 6*NINT(LOG10(GRDTOL)),LGL(MAXLG))
C
C     *** Loop over growing angular Lebedev grids ***
C
      DO 100 ILG=LGNUMOF(LSTART),LGNUMOF(LEND)
C
        CYCLE = CYCLE + 1
C
C     *** Initialize grid point counters ***
C
        NGPBUF = 0
        NGPGXC = 0
        NGPTOT = 0
C
C     *** Load angular quantum number L of actual Lebedev grid ***
C
        L = LGL(ILG)
C
C     *** Generate angular Lebedev grid ***
C
        CALL GETLEB(ATOM,L,SPHRAD,WLEB)
C
C     *** Integration over the actual grid for the atomic sphere ***
C
        IF (MPMASTER) THEN
CCC ADLL
          IF (STATUS.EQ.'ESA') THEN
            IF (TOPCD) THEN
              CALL SDIAGRD(PA,PB,NEWVEC,DSTO,NALL)
            ELSE
              CALL SDIAGRD(PA,PB,NEWVEC,DSTO,NSTO)
            END IF
          ELSE
            IF (CDXC) THEN
              CALL XCVGRD(PA,PB,NEWVEC,DSTO,NALL)
            ELSE 
              CALL XCVGRD(PA,PB,NEWVEC,DSTO,NSTO)
            END IF
          END IF
CCC ADLL
        END IF
C
        IF (MPP) THEN
          IF (STATUS.EQ.'ESA') THEN
            IF (TOPCD)  THEN
              CALL MP_SEND_ALL(NEWVEC,NALL,MASTER,MP_REAL)
            ELSE
              CALL MP_SEND_ALL(NEWVEC,NSTO,MASTER,MP_REAL)
            END IF
          ELSE 
            IF (CDXC) THEN
              CALL MP_SEND_ALL(NEWVEC,NALL,MASTER,MP_REAL)
            ELSE
              CALL MP_SEND_ALL(NEWVEC,NSTO,MASTER,MP_REAL)
            END IF
          END IF
        END IF 
C
C     *** Error estimation for theta convergence ***
C
        IF (CYCLE.GT.1) THEN
C
          ERROR = 0.0
C
CCC ADLL
          IF (STATUS.EQ.'ESA') THEN
            IF (TOPCD) THEN
              DO 75 INEIGH=1,NNEIGH
                IATOM = NEIGH(INEIGH)
                DO 35 IAUXSET=LLAUXSET(IATOM),ULAUXSET(IATOM)
                  DO 25 IAUXSHL=LLAUXSHL(IAUXSET),ULAUXSHL(IAUXSET)
                    DO 15 IAUX=LLAUX(IAUXSHL),ULAUX(IAUXSHL)
                      ERROR = MAX(ERROR,ABS(OLDVEC(IAUX)- NEWVEC(IAUX)))
   15               CONTINUE
   25             CONTINUE
   35           CONTINUE
                IF (.NOT.FIXAUXIS(IATOM)) GO TO 75
                DO 65 IFIXSET=LLFIXSET(IATOM),ULFIXSET(IATOM)
                  DO 55 IFIXSHL=LLAUXSHL(IFIXSET),ULAUXSHL(IFIXSET)
                    DO 45 IFIX=LLAUX(IFIXSHL),ULAUX(IFIXSHL)
                      ERROR = MAX(ERROR,ABS(OLDVEC(IFIX)- NEWVEC(IFIX)))
   45               CONTINUE
   55             CONTINUE
   65           CONTINUE
   75         CONTINUE
           ELSE
             DO 95 INEIGH=1,NNEIGH
               IATOM = NEIGH(INEIGH)
               DO 85 ISTO=LL(IATOM),UL(IATOM)
                 ERROR = MAX(ERROR,ABS(OLDVEC(ISTO) - NEWVEC(ISTO)))
   85          CONTINUE
   95         CONTINUE
           END IF
CCC ADLL
C
          ELSE 
            IF (CDXC) THEN
              DO 70 INEIGH=1,NNEIGH
                IATOM = NEIGH(INEIGH)
                DO 30 IAUXSET=LLAUXSET(IATOM),ULAUXSET(IATOM)
                  DO 20 IAUXSHL=LLAUXSHL(IAUXSET),ULAUXSHL(IAUXSET)
                    DO 10 IAUX=LLAUX(IAUXSHL),ULAUX(IAUXSHL)
                      ERROR = MAX(ERROR,ABS(OLDVEC(IAUX)- NEWVEC(IAUX)))
   10               CONTINUE
   20             CONTINUE
   30           CONTINUE
                IF (.NOT.FIXAUXIS(IATOM)) GO TO 70
                DO 60 IFIXSET=LLFIXSET(IATOM),ULFIXSET(IATOM)
                  DO 50 IFIXSHL=LLAUXSHL(IFIXSET),ULAUXSHL(IFIXSET)
                    DO 40 IFIX=LLAUX(IFIXSHL),ULAUX(IFIXSHL)
                      ERROR = MAX(ERROR,ABS(OLDVEC(IFIX)- NEWVEC(IFIX)))
   40               CONTINUE
   50             CONTINUE
   60           CONTINUE
   70         CONTINUE
            ELSE
              DO 90 INEIGH=1,NNEIGH
                IATOM = NEIGH(INEIGH)
                DO 80 ISTO=LL(IATOM),UL(IATOM)
                  ERROR = MAX(ERROR,ABS(OLDVEC(ISTO) - NEWVEC(ISTO)))
   80           CONTINUE
   90         CONTINUE
            END IF
          END IF
C
          IF (ERROR.LE.GRDTOL/10.0) GO TO 500
C
        END IF
C
C     *** Store last errors ***
C
        OLDVEC(:) = NEWVEC(:)
C
        OLDNGP = NGPSPH
C
C     *** Next Lebedev grid, if angular grid is not converged ***
C
  100 CONTINUE
C
      ILG = LGNUMOF(LEND) + 1
C
      TRUNCATED = ' (TRUNCATED)'
C
C     *** Automatic generation of angular ***
C     *** grid converged for L sequence   ***
C
  500 CONTINUE
C
C     *** Set L value of a actual sphere ***
C
      LRADIAL(ATOM,SPHNUM) = LGL(MAX(1,ILG-1))
C
      NGPSPH = OLDNGP
C
      IF (PRTGRD.AND.(STATUS.EQ.'SCF')) THEN
        IF (STREXT(TRUNCATED).GT.0) THEN
          WRITE (OUT,5000) SPHNUM,SPHRAD,LRADIAL(ATOM,SPHNUM),CYCLE,
     $                     ERROR,NGPSPH,TRUNCATED
 5000     FORMAT (T2,I6,7X,G11.4,2X,I6,5X,I6,5X,G10.3,2X,I6,A12)
        ELSE
          WRITE (OUT,5010) SPHNUM,SPHRAD,LRADIAL(ATOM,SPHNUM),CYCLE,
     $                     ERROR,NGPSPH
 5010     FORMAT (T2,I6,7X,G11.4,2X,I6,5X,I6,5X,G10.3,2X,I6)
        END IF
      END IF
C
C     *** Deallocate local work vectors ***
C
      DEALLOCATE(NEWVEC,OLDVEC,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('AUTOLEB','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE AUTOLEB ***
C
      END
