      SUBROUTINE AUXRAD(T)
C
C     Purpose: Generate auxiliary function set, shell and cusp radii.
C
C     History: - Creation (04.07.97, AMK)
C                         (04.12.04, RFM)
C                         (02.06.05, FJ)
C                         (23.10.18, AMK)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     ALPHA: Primitive exponent.
C     L    : Angular momentum quantum number of current shell.
C     T    : Treshold for radius.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'pointer.h'
      INCLUDE 'scf.h'
C
      REAL T
C
      INTEGER IATOM,IAUXSET,IAUXSHL,L
      REAL ALPHA,SETRAD
C
      REAL GTORAD
C
C     ------------------------------------------------------------------
C
C     *** Loop over all atoms ***
C
      DO 10 IATOM=1,NATOM
C
C     *** Set atomic cusp radius for auxiliary density ***
C
        IF (ZATOM(IATOM).EQ.0) THEN
          CUSPRAD(IATOM) = -0.1
        ELSE
          CUSPRAD(IATOM) = 0.1
        END IF
C
   10 CONTINUE
C
C     *** Loop over all auxiliary function sets ***
C
      DO 30 IAUXSET=1,NALLSET
C
        AUXSETRAD(IAUXSET) = 0.0
        ALPHA = ZET(NGTO+IAUXSET)
C
C     *** Loop over all shells of IAUXSET ***
C
        DO 20 IAUXSHL=LLAUXSHL(IAUXSET),ULAUXSHL(IAUXSET)
C
          L = AUXSHLPTR(IAUXSHL,3)
C
C     *** Get the radi ***
C
          AUXSHLRAD(IAUXSHL) = GTORAD(ALPHA,1.0,L,T)
          SETRAD = MAX(AUXSETRAD(IAUXSET),AUXSHLRAD(IAUXSHL))
          AUXSETRAD(IAUXSET) = SETRAD
C
   20   CONTINUE
C
   30 CONTINUE
C
      IF (PRTGRD.AND.(STATUS.EQ.'SCF').AND.(SCFCYC.EQ.1)) THEN
        CALL MESSAGE(0,'AUXILIARY FUNCTION SHELL RADII [A.U.]',2,1)
        WRITE (OUT,5000) (IAUXSHL,SQRT(AUXSHLRAD(IAUXSHL)),
     $                    IAUXSHL=1,NAUXSHL)
 5000   FORMAT (T2,'SHELL',4X,'MAX. RADIUS',/,(T2,I4,3X,F12.6))
        CALL MESSAGE(0,'AUXILIARY FUNCTION SET RADII [A.U.]',2,1)
        WRITE (OUT,5010) (IAUXSET,SQRT(AUXSETRAD(IAUXSET)),
     $                    IAUXSET=1,NAUXSET)
 5010   FORMAT (T2,'  SET',4X,'MAX. RADIUS',/,(T2,I4,3X,F12.6))
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE AUXRAD ***
C
      END
