      SUBROUTINE CANORT(MATRIX,DMAT,N,THRESH)
C
C     Purpose: Calculation of the CANonical ORThogonalization matrix
C              X=U*D**(-1/2). The matrix is truncated if linear
C              dependences occure.
C
C     Lit.: A. Szabo, N.S. Ostlund, Modern Quantum Chemistry pp.143,
C           Dover Mineola (1989)
C
C     History: - Creation (29.09.05, FJ)
C                         (25.09.06, GG)
C                         (15.09.15, XJ, check for EGIS)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DMAT: Dimension of MATRIX.
C
C     List of local variables:
C
C     MATRIX: On entry the overlap matrix, on exit the canonical
C             orthogonalization matrix.
C     N     : Number of rows of MATRIX.
C     THRESH: Threshold for quenching the eigenvalues.
C     EGIS  : Eigenvalues after quenching.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
CCC ADLL 
      INCLUDE 'rtdft.h'
CCC ADLL
      INCLUDE 'scf.h'
C
      INTEGER DMAT
      REAL THRESH
      REAL MATRIX(DMAT,DMAT)
C
      CHARACTER*80 PRTSTR,STRCOMP
      INTEGER IEIGVAL,JEIGVAL,INULLMOS,N
C
      INTEGER ALLOCATION
      INTEGER,ALLOCATABLE :: NEIGVEC(:)
      REAL,ALLOCATABLE :: EIGVAL(:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(EIGVAL(N),NEIGVEC(N),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'CANORT','ALLOCATION FAILED')
C
C     *** Initialization ***
C
      NNULLMOS = 0
C
      EIGVAL(:) = 0.0
      EGIS(:) = 0.0
C
C     *** Diagonalize matrix ***
C
      USEMPF = MPP
      CALL DIAMAT(MATRIX,DMAT,N,MATDIA,EIGVAL) 
      USEMPF = .FALSE.
C
      IF (MPSLAVE) GO TO 100
C
C     *** Check for small eigenvalues ***
C
      NEIGVEC(:) = 0
C
      DO IEIGVAL=1,N
        IF (EIGVAL(IEIGVAL).LT.THRESH) THEN
          NNULLMOS = NNULLMOS + 1
          NEIGVEC(NNULLMOS) = IEIGVAL
        END IF
      END DO
C
C     *** Calculate number of orthonormal atomic orbitals ***
C
      NORTH = N - NNULLMOS
C
CCC ADLL
      IF (((STATUS.EQ.'SCF GUESS').AND.HUECKEL).OR.
     $    ((STATUS.EQ.'OPT GUESS').AND.HUECKEL)) THEN
CCC ADLL
        NORTH_TB = N - NNULLMOS
      END IF
C  
C     *** Eliminate eigenvectors with small eigenvalues ***
C
      DO INULLMOS=NNULLMOS,1,-1
        IEIGVAL = NEIGVEC(INULLMOS)
        DO JEIGVAL=IEIGVAL+1,N
          EIGVAL(JEIGVAL-1) = EIGVAL(JEIGVAL)
          MATRIX(1:N,JEIGVAL-1) = MATRIX(1:N,JEIGVAL) 
        END DO
      END DO
C
      DO IEIGVAL=NORTH+1,N
        EIGVAL(IEIGVAL) = 100.0 + REAL(IEIGVAL)
        MATRIX(1:N,IEIGVAL) = 0.0
      END DO
C
C     *** Calculate canonical orthogonalization matrix ***
C
      DO IEIGVAL=1,NORTH
        MATRIX(1:N,IEIGVAL) = MATRIX(1:N,IEIGVAL)/SQRT(EIGVAL(IEIGVAL))
CCC ADLL 
        EGIS(IEIGVAL) = EIGVAL(IEIGVAL)
CCC ADLL
      END DO
C
      IF ((NNULLMOS.GT.0).AND.(STATUS.EQ.'SCF')) THEN
        WRITE (PRTSTR,5000) NNULLMOS
 5000   FORMAT (I5,' OVERLAP EIGENVALUES ARE QUENCHED')
        CALL ERRMSG('CANORT',STRCOMP(PRTSTR),0)
      ELSE IF ((NNULLMOS.GT.0).AND.(STATUS.EQ.'XRAY')) THEN
        WRITE (PRTSTR,5010) NNULLMOS
 5010   FORMAT (I5,' OVERLAP EIGENVALUES FROM EXTENDED',
     $             ' SPACE ARE QUENCHED')
        CALL MESSAGE(0,STRCOMP(PRTSTR),1,1)
      ELSE IF ((NNULLMOS.GT.0).AND.(STATUS.EQ.'SCF GUESS')
     $         .AND.(HUECKEL)) THEN
        WRITE (PRTSTR,5020) NNULLMOS
 5020   FORMAT (I5,' TB OVERLAP EIGENVALUES ARE QUENCHED')
        CALL ERRMSG('CANORT',STRCOMP(PRTSTR),0)
      END IF
C
  100 CONTINUE
C
CCC ADLL
      CALL MP_SEND_ALL(EGIS(1:N),N,MASTER,MP_REAL)
CCC ADLL
      CALL MP_SEND_ALL(NORTH,1,MASTER,MP_INT)
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(EIGVAL,NEIGVEC,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'CANORT','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE CANORT ***
C
      END 
