      SUBROUTINE ATOMSORT(OPTION)
C
C     Purpose: Set up atomic mapping vectors and sort atoms.
C
C     History: - Creation (23.04.97, BZ)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of variables
C
C     ATOMMAP: Atomic mapping vector.
C     ATOMNO : Atomic numbers.
C     NATOM  : Number of atoms in molecule.
C     NATOMS : Number of atoms in Z-Matrix (NATOM+NDUMMY).
C     NDUMMY : Number of dummy atoms.
C     OPTION : a) OPTION = ALL, sort all atomic fields.
C              b) OPTION = MAP, sort atomic mapping vector.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'cdft.h'
      INCLUDE 'molecule.h' 
C
      CHARACTER*(*) OPTION
C
      INTEGER ICNSTR,NATOMS
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      NATOMS = NATOM_ALL + NDUMMY
C
C     *** Generate new atomic mapping vector ***
C
      IF ((OPTION.EQ.'ALL').OR.(OPTION.EQ.'MAP')) THEN
        CALL MAPATOM(ATOMMAP,ATOMNO,ZATOM,NATOM,NATMM,NDUMMY,0,'ZMAT')
      END IF
C
C     *** Sort coordinates according to QM, MM and dummy atoms ***
C
      IF ((NATMM.NE.0).OR.(NDUMMY.NE.0)) THEN
C
C     *** Sort x,y,z-coordinates ***
C
        CALL REALSORT(COORD(1,:,CCSET),ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
        CALL REALSORT(COORD(2,:,CCSET),ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
        CALL REALSORT(COORD(3,:,CCSET),ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
C
C     *** Sort x,y,z-velocities ***
C
        CALL REALSORT(VELOS(1,:),ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
        CALL REALSORT(VELOS(2,:),ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
        CALL REALSORT(VELOS(3,:),ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
C
C     *** Sort atomic masses, numbers and charges ***
C
        IF (OPTION.EQ.'ALL') THEN 
          CALL REALSORT(MATOM,ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
          CALL INTSORT(ATOMNO,ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
          CALL INTSORT(ZATOM,ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
          CALL INTSORT(ZCORE,ATOMMAP(:,2),MAXATOM_ALL,NATOMS)
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE ATOMSORT ***
C
      END
