      SUBROUTINE READCART
C
C     Purpose: Read Cartesian coordinates.
C
C     History: - Creation (03.05.97, AMK)
C                         (21.02.00, FJ)
C                         (03.10.02, UR) 
C                         (19.02.10, GG)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     LINE  : Input line.
C     NATOMS: Number of input atoms (NATQM+NATMM+NDUMMY).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
      INCLUDE 'opt.h'
      INCLUDE 'physic.h'
CCC ADLL LA
      INCLUDE 'qmmm.h'
CCC ADLL LA
C
CCC ADLL LA
      LOGICAL SETLINAT
      INTEGER INF
CCC ADLL LA
      LOGICAL BLOCKEND,ERROR,SUCCESS
      CHARACTER*80 DTS,LINE,LTU,PRTSTR,STRCOMP,STRING,STRTYP*7
      INTEGER I,IATOM,ITOKEN,IVALUE,NATOMS,NTOKEN,STREXT,STRLEN
      REAL RVALUE
C
      CHARACTER*80 TOKEN(80)
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      NATMM = 0
      NDUMMY = 0
CCC ADLL LA
      NLATOM = 0
CCC ADLL LA

C
CCC ADLL LA
      LINKLA(:,:) = 0
CCC ADLL LA

      TYPEMM(:) = 0
      LINKMM(:,:) = 0

CCC ADLL LA
      SETLINAT = .TRUE.
CCC ADLL LA
C
      ERROR = .FALSE.
C
CCC ADLL QMMM
      IF (FGEO) THEN
        INF = GEO
      ELSE
        INF = INP
      END IF
C
      IF (FGEO) THEN
        REWIND(INF)
 100    CONTINUE
        READ (INF,'(A)',ERR=99980,END=99970) LINE
        IF (INDEX(LINE,'GEOME').EQ.0) GO TO 100
      END IF
CCC ADLL QMMM

C     *** Prepare logic field for geometry optimization ***
C
      OPTFLAG(1:MAXATOM_ALL,1:3) = 'UNDEFINED'
      OPTXYZ(1:MAXATOM_ALL) = ' '
C
C     *** Read Cartesian input ***
C
      IATOM = 0
C
  500 CONTINUE
C
        READ (INF,'(A)',ERR=99980,END=1000) LINE  
        STRING = STRCOMP(LTU(DTS(LINE)))
        IF (BLOCKEND(STRING)) GO TO 1000
        IATOM = IATOM + 1
C
C     *** Tokenize input line ***
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        IF (((INPTYP.EQ.'CARTESIAN').AND.(NTOKEN.LT.4)).OR. 
     $      ((INPTYP.EQ.'MIXED').AND.(NTOKEN.LT.2))) THEN 
          WRITE (PRTSTR,5000) IATOM
 5000     FORMAT ('DEFINITION OF ATOM ',I4,' INCOMPLETE')
          CALL MESSAGE (1,STRCOMP(PRTSTR),1,0)
          GO TO 99990
        END IF
C
C     *** Check end of Cartesian coordinates for mixed input ***
C
        IF (INPTYP.EQ.'MIXED') THEN
          CALL GETNUM(TOKEN(2),STRTYP,RVALUE,IVALUE)
          IF (STRTYP.NE.'FLOAT') THEN
            BACKSPACE (INP)
            IATOM = IATOM - 1
            GO TO 1000
          END IF
        END IF
C
C     *** Check atomic label, assign atomic number and store label ***
C
        IF (STREXT(TOKEN(1)).GT.MAXLAB) THEN
          WRITE (PRTSTR,5010) TOKEN(1)(1:STREXT(TOKEN(1)))
 5010     FORMAT ('LABEL < ',A,' > TOO LONG (MAX. 10 CHRS.)')
          GO TO 99990
        END IF
C
        CALL CHKLABEL(TOKEN(1),ATOMNO(IATOM),ERROR)
        IF (ERROR) GO TO 99990
        MATOM(IATOM) = STDMATOM(ATOMNO(IATOM))
        ZATOM(IATOM) = ATOMNO(IATOM)
        ZCORE(IATOM) = 0
        TYPEMM(IATOM) = -1
        ATOMLAB(IATOM) = TOKEN(1)
C
CCC ADLL LA
        IF (INDEX('HLFL',ATOMLAB(IATOM)(1:2)).NE.0) THEN
          SETLINAT = .FALSE.
          NLATOM = NLATOM + 1
          LINKLA(NLATOM,0) = IATOM
        END IF
CCC ADLL LA
        IF (ZATOM(IATOM).EQ.0) NDUMMY = NDUMMY + 1
C
C     *** Read Cartesian coordinates ***
C
        DO I=1,3
          STRLEN = STREXT(TOKEN(I+1))
          CALL GETNUM(TOKEN(I+1),STRTYP,COORD(I,IATOM,CCSET),
     $                IVALUE)
          IF (STRLEN.GT.MAXNUM) THEN
            WRITE (PRTSTR,5020) TOKEN(I+1)(1:STRLEN)
 5020       FORMAT ('COORDINATE < ',A, ' > TOO LONG (MAX. 15 CHRS.)')
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          ELSE IF (STRTYP.NE.'FLOAT') THEN
            CALL MESSAGE(1,'COORDINATES MUST BE REAL',1,0)
            GO TO 99990
          END IF
        END DO
C
C     *** Read ZATOM, MATOM or TYPEMM if set for this atom ***
C
        IF (NTOKEN.GE.5) THEN
          STRLEN = STREXT(TOKEN(5))
          CALL GETNUM(TOKEN(5),STRTYP,RVALUE,IVALUE)
          IF (STRLEN.GT.MAXVAR) THEN
            IF (STRTYP.EQ.'INTEGER') THEN
              WRITE (PRTSTR,5030) TOKEN(5)(1:STRLEN)
 5030         FORMAT ('ZATOM STRING < ',A,' > TOO LONG (MAX. 10 CHRS.)')
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              WRITE (PRTSTR,5040) TOKEN(5)(1:STRLEN)
 5040         FORMAT ('MATOM STRING < ',A,' > TOO LONG (MAX. 10 CHRS.)')
            ELSE
              PRTSTR = 'ERROR IN POSITION 5'
            END IF
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          IF (STRTYP.EQ.'INTEGER') THEN
            IF (IVALUE.LT.0) THEN
              NATMM = NATMM + 1
              TYPEMM(IATOM) = ABS(IVALUE)
              ZATOM(IATOM) = -ZATOM(IATOM)
            ELSE
              ZATOM(IATOM) = IVALUE
            END IF
          ELSE IF (STRTYP.EQ.'FLOAT') THEN
            MATOM(IATOM) = RVALUE
          ELSE
            CALL MESSAGE(1,'ERROR IN POSITION 5',1,0)
            GO TO 99990
          END IF
        END IF
C
C     *** Read ZATOM, MATOM, TYPEMM or LINKMM if set for this atom ***
C
        IF (NTOKEN.GE.6) THEN
          STRLEN = STREXT(TOKEN(6))
          CALL GETNUM(TOKEN(6),STRTYP,RVALUE,IVALUE)
          IF (STRLEN.GT.MAXVAR) THEN
            IF (STRTYP.EQ.'INTEGER') THEN
              WRITE (PRTSTR,5030) TOKEN(6)(1:STRLEN)
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              WRITE (PRTSTR,5040) TOKEN(6)(1:STRLEN)
            ELSE
              PRTSTR = 'ERROR IN POSITION 6'
            END IF
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          IF (STRTYP.EQ.'INTEGER') THEN
            IF (IVALUE.LT.0) THEN
              NATMM = NATMM + 1
              TYPEMM(IATOM) = ABS(IVALUE)
              ZATOM(IATOM) = -ZATOM(IATOM)
            ELSE IF (ZATOM(IATOM).LT.0) THEN
              I = LINKMM(IATOM,0) + 1
              LINKMM(IATOM,I) = IVALUE
              LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
            ELSE
              ZATOM(IATOM) = IVALUE
            END IF
          ELSE IF (STRTYP.EQ.'FLOAT') THEN
            MATOM(IATOM) = RVALUE
          ELSE
            CALL MESSAGE(1,'ERROR IN POSITION 6',1,0)
            GO TO 99990
          END IF
        END IF
C
C     *** Read remaining LINKMMs if set for this atom ***
C
        DO ITOKEN=7,NTOKEN
          STRLEN = STREXT(TOKEN(ITOKEN))
          CALL GETNUM(TOKEN(ITOKEN),STRTYP,RVALUE,IVALUE)
          IF (ZATOM(IATOM).GE.0) THEN
            WRITE (PRTSTR,5050) IATOM
 5050       FORMAT ('TOO MANY ENTRIES IN COORDINATE LINE ',I5)
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          ELSE IF (STRTYP.NE.'INTEGER') THEN
            WRITE (PRTSTR,5060) ITOKEN
 5060       FORMAT ('ERROR IN POSITION ',I2,'; ENTRY MUST BE INTEGER')
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          ELSE IF (STRLEN.GT.MAXVAR) THEN
            WRITE (PRTSTR,5070) TOKEN(ITOKEN)(1:STRLEN)
 5070       FORMAT ('STRING < ',A,' > TOO LONG (MAX. 10 CHRS.)')
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          I = LINKMM(IATOM,0) + 1
          IF (I.GT.MAXLINK) THEN
            WRITE (PRTSTR,5080) IATOM,I,MAXLINK
 5080       FORMAT ('ATOM ',I5,' HAS ',I2,' LINKS > MAXLINK = ',I2)
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          LINKMM(IATOM,I) = IVALUE
          LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
        END DO
C
        GO TO 500
C
 1000 CONTINUE
C
C     *** Set number of atoms ***
C
      NATOMS = IATOM
      NATXYZ(1) = NATOMS
      NATXYZ(3) = NATXYZ(1)
      NATXYZ(4) = NATXYZ(1)
      NATOM_ALL = NATOMS - NDUMMY
      NATOM = NATOM_ALL - NATMM
C
CCC ADLL LA
C     *** Check presence of covalent QM/MM borders ***
C
      CALL CHKQMMMBRD
      IF (LINKAT) THEN
        IF (SETLINAT) THEN
          CALL SETLAPOS('SET')
          NATOMS = NATOMS + NLATOM
        ELSE
C          CALL SETLAPOS('POSITION')
        END IF
      END IF
      NATOMS = NATOMS
      NATXYZ(1) = NATOMS
      NATXYZ(3) = NATXYZ(1)
      NATXYZ(4) = NATXYZ(1)
      NATOM_ALL = NATOMS - NDUMMY
      NATOM = NATOM_ALL - NATMM
C
CCC ADLL LA
C
C     *** Change to atomic units ***
C
      COORD(1:3,1:NATOMS,CCSET) = COORD(1:3,1:NATOMS,CCSET)/ANGBOR(1)
C
C     *** Check atomic distances ***
C
      CALL DISTCHK('PLAIN')
C
C     *** Initialize ATOMMAP and check atomic distances ***
C
      IF (INPTYP.NE.'MIXED') THEN
        CALL MAPATOM(ATOMMAP,ATOMNO,ZATOM,NATOM,NATMM,NDUMMY,0,'ZMAT')
      END IF
C
C     *** Return, if syntax check ***
C
      IF (STATUS.EQ.'CHECK') RETURN
C
C     *** Check number of atoms ***
C
      IF (NATOMS.EQ.0) THEN
        PRTSTR = 'NO ATOMS FOUND IN THE INPUT FILE'
        CALL ERRMSG('READCART',PRTSTR,1)
      ELSE IF ((.NOT.OPT).AND.(.NOT.IRC).AND.(.NOT.SADDLE)) THEN
        OPTTYP = 'UNDEFINED'
      ELSE IF ((NATOMS.EQ.1).AND.(INPTYP.NE.'MIXED')) THEN
        OPTTYP = 'UNDEFINED'
      END IF
C
C     *** Set degrees of freedom ***
C
      NDEG = 3*NATOMS
      NXYZ = 3*NATOM_ALL
C
C     *** Read Cartesian constrains ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('CONSTANTS',SUCCESS)
      IF (MD.AND.SUCCESS) THEN
        CALL READMDCON
      ELSE IF (SUCCESS) THEN
        CALL READCON(NATOMS)
      END IF
C
C     *** Read constrained DFT regions *** 
C
C      SUCCESS = .FALSE.
C      CALL FINDKEY('CNSDFT',SUCCESS)
C      IF (SUCCESS) CALL READCNS(NATOMS)
C
C     *** Reset input ***
C
      IF (INPTYP.EQ.'MIXED') THEN
        SUCCESS = .FALSE.
        CALL FINDKEY('GEOMETRY',SUCCESS)
        DO IATOM=1,NATOMS
          READ (INF,'(A)',ERR=99990) LINE
        END DO
      END IF 
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
CCC ADLL
99970 CONTINUE
      WRITE (PRTSTR,99975)
99975 FORMAT ('NO GEOMETRY BLOCK FOUND IN deMon.geo FILE')
      CALL ERRMSG('READCART',STRCOMP(PRTSTR),1)
C
CCC ADLL
99980 CONTINUE
      WRITE (PRTSTR,99985) IATOM+1
99985 FORMAT ('ERROR IN CARTESIAN COORDINATE LINE ',I4)
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READCART',STRCOMP(PRTSTR),1)
C
99990 CONTINUE
      WRITE (PRTSTR,99995) IATOM
99995 FORMAT ('ERROR IN CARTESIAN COORDINATE LINE ',I4)
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READCART',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE READCART ***
C
      END
