      SUBROUTINE READCNS(NATOMS)
C
C     Purpose: Read the definitions of the CDFT constrains.
C
C     History: - Creation (09.30.09, ADLL)
C
C     ATMCNS  : ATOMS subjected to by each constrain. 
C               =  0 if IATOM is not constrained.
C               =  1 if IATOM constrained.
C               =  1 if constrained donor atom (ET module).
C               = -1 if constrained acceptor atom (ET module).
C     TYPECNS : Type of constraint (CHARGE or SPIN DENSITY). 
C     NCNSTR  : Number of constrains.
C
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'cdft.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
C
      INTEGER NATOMS
C
CCC ADLL JUL22
      INTEGER ICNS,IVALUE,NTOKEN
CCC ADLL JUL22
      REAL RVALUE,VAL(2)
      CHARACTER*80 DTS,LINE,LTU,STRCOM,STRCOMP,STRING,STRTYP*7
      CHARACTER*80 TOKEN(500)
C
C    -------------------------------------------------------------------
C
C     *** Initiliaze *** 
C
      NCNSTR = 0
      ETMOD = .FALSE.
      ATMCNS(:,:) = 0
      ATMMHW(:) = 1.0
      LMCDFT(:,:) = 0.0
C
 50   CONTINUE
C
      READ (INP,'(A)',ERR=99990,END=500) LINE
      STRING = STRCOM(STRCOMP(LTU(DTS(LINE))))
C
C      *** Get type and value to be achieved ***
C
      CALL STRTOK(STRING,NTOKEN,TOKEN)
C
      CALL GETNUM(TOKEN(1),STRTYP,RVALUE,IVALUE)
C
      IF (STRTYP.EQ.'FLOAT') THEN
        VAL(1) = RVALUE
      ELSE
        GO TO 500 
      END IF
C
      CALL GETNUM(TOKEN(2),STRTYP,RVALUE,IVALUE)
C
      IF (STRTYP.EQ.'STRING') THEN 
C
C     *** Constraint on charge or spin charge ***  
C
        IF ((TOKEN(2).EQ.'CHARGE').OR.(TOKEN(2).EQ.'SPIN')) THEN
          NCNSTR = NCNSTR + 1
          QCDFT(NCNSTR) = VAL(1)
          CDFTYPE(NCNSTR) = TOKEN(2)  
          CALL GETCNS(TOKEN,NTOKEN,3,NATOMS,'STANDARD')
C
C     *** Electron donor in ET module *** 
C
        ELSE IF (TOKEN(2).EQ.'DONOR') THEN
          NCNSTR = NCNSTR + 1
          ETMOD = .TRUE.
          QDGRP = VAL(1)
          QCDFT(NCNSTR) = VAL(1)
          CDFTYPE(NCNSTR) = 'CHARGE'
          CALL GETCNS(TOKEN,NTOKEN,3,NATOMS,'DONOR')
C
C     *** Electron acceptor in ET module *** 
C
        ELSE IF (TOKEN(2).EQ.'ACCEPTOR') THEN 
          NCNSTR = NCNSTR + 1
          ETMOD = .TRUE.
          QAGRP = VAL(1)
          QCDFT(NCNSTR) = VAL(1)
          CDFTYPE(NCNSTR) = 'CHARGE'
          CALL GETCNS(TOKEN,NTOKEN,3,NATOMS,'ACCEPTOR')
C
C     *** Specific definition of Hirshfeld weights *** 
C
        ELSE IF ((TOKEN(2).EQ.'MODHW')) THEN
          MODHIR = .TRUE.
          CALL GETMHW(TOKEN,NTOKEN,NATOMS,VAL)
C
        ELSE 
          GO TO 99980
        END IF
C
      ELSE IF (STRTYP.EQ.'INTEGER') THEN 
C
        IF (TOKEN(3).EQ.'DONOR') THEN
          NCNSTR = NCNSTR + 1
          ETMOD = .TRUE.
          QDGRP = VAL(1)
          SDGRP = RVALUE
          QCDFT(NCNSTR) = VAL(1)
          CDFTYPE(NCNSTR) = 'CHASD'
          CALL GETCNS(TOKEN,NTOKEN,4,NATOMS,'DONOR')
        ELSE IF (TOKEN(3).EQ.'DONOR') THEN
          NCNSTR = NCNSTR + 1
          ETMOD = .TRUE.
          QAGRP = VAL(1)
          SAGRP = RVALUE
          QCDFT(NCNSTR) = VAL(1)
          CDFTYPE(NCNSTR) = 'CHASD'
          CALL GETCNS(TOKEN,NTOKEN,4,NATOMS,'ACCPETOR')
        ELSE
          GO TO 99990
        END IF
      ELSE
        GO TO 99990
      END IF
C
      GO TO 50
C
C     *** All constrains are now read ***
C
 500  CONTINUE
C
C     *** Reorder ATMCNS arrays if ET module activated ***
C
      IF (ETMOD) CALL REORCNS(NATOMS)
C
CCC ADLL JUL22
      DO ICNS=1,MAXCNS
        CALL INTSORT(ATMCNS(:,ICNS),ATOMMAP(:,3),MAXATOM_ALL,NATOMS)
      END DO 
CCC ADLL JUL22
C
      RETURN
C
C   ***  Handling of errors ***
C
99980 CONTINUE
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READCNS','UNKNOWN DENSITY CONSTRAINT TYPE',1)
C
99990 CONTINUE
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READCNS','ERROR WHEN READING CDFT PARAMETERS',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE READCNS ***
C
      END
