      SUBROUTINE DIVIDE(NICUBE,NJCUBE,NKCUBE,OPTION)
C
C     Purpose: Divide plot lattice in sublattices.
C
C     History: - Creation (09.12.96, ML)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DSTEP : Step sizes along lattice basis vectors.
C     NICUBE: Number of cubes along i lattice vector.
C     NJCUBE: Number of cubes along j lattice vector.
C     NKCUBE: Number of cubes along k lattice vector.
C     OPTION: a) OPTION = CUBES, sublattice division for cubes.
C             b) OPTION = STEPS, sublattice division for steps.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'marching.h'
      INCLUDE 'plot.h'
C
      CHARACTER*(*) OPTION
      INTEGER I,J,NICUBE,NJCUBE,NKCUBE
C
C     ------------------------------------------------------------------
C
C     *** Calculate step sizes ***
C
      CALL SETMAT(DSTEP,3,3,3,'TOTAL',0.0)
C
      DO I=1,3
        DO J=1,3
          IF (NLATTICE(I).LE.1) THEN
            DSTEP(I,J) = 0.0
          ELSE
            DSTEP(I,J) = (LATTICE(J,I) - LATTICE(J,0))/(NLATTICE(I)-1)
          END IF
        END DO
      END DO
C
C     *** Sublattice division for cubes ***
C
      IF (OPTION.EQ.'CUBES') THEN
C
        NICUBE = NLATTICE(1) - 1
        NJCUBE = NLATTICE(2) - 1
        NKCUBE = NLATTICE(3) - 1
C
C     *** Sublattice division for steps ***
C
      ELSE IF (OPTION.EQ.'STEPS') THEN
C
C     *** Calculate number of points per sublattice ***
C     *** 1. One dimensional lattice ***
C
        IF (EUCLID.EQ.1) THEN
C
          CUBED = MAXGPB
          NICUBE = NLATTICE(1)/CUBED
          IF (MOD(NLATTICE(1),CUBED).NE.0) NICUBE = NICUBE + 1
          NJCUBE = 1
          NKCUBE = 1
C
C     *** 2. Two dimensional lattice ***
C
        ELSE IF (EUCLID.EQ.2) THEN
C
          CUBED = INT(SQRT(REAL(MAXGPB)))
          NICUBE = NLATTICE(1)/CUBED
          IF (MOD(NLATTICE(1),CUBED).NE.0) NICUBE = NICUBE + 1
          NJCUBE = NLATTICE(2)/CUBED
          IF (MOD(NLATTICE(2),CUBED).NE.0) NJCUBE = NJCUBE + 1
          NKCUBE = 1
C
C     *** 3. Three dimensional lattice ***
C
        ELSE IF (EUCLID.EQ.3) THEN
C
          CUBED = INT(REAL(MAXGPB)**(1.0/3.0))
          NICUBE = NLATTICE(1)/CUBED
          IF (MOD(NLATTICE(1),CUBED).NE.0) NICUBE = NICUBE + 1
          NJCUBE = NLATTICE(2)/CUBED
          IF (MOD(NLATTICE(2),CUBED).NE.0) NJCUBE = NJCUBE + 1
          NKCUBE = NLATTICE(3)/CUBED
          IF (MOD(NLATTICE(3),CUBED).NE.0) NKCUBE = NKCUBE + 1
C
        END IF
C
CCC ADLL
C     *** Calculate number of points per slices ***
C
      ELSE IF (OPTION.EQ.'SLICES') THEN
C
          CUBED = MAXGPB
          NKCUBE = NLATTICE(3)/CUBED
          IF (MOD(NLATTICE(3),CUBED).NE.0) NKCUBE = NKCUBE + 1
          NJCUBE = 1
          NICUBE = 1
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DIVIDE ***
C
      END
