      PROGRAM deMon
C
C     Version: 6.0.8 (Developer Version)
C
C     Purpose: Program for ab-initio LCGTO-SCF calculations.
C
C     Gatekeepers: Gerald Geudtner & Andreas M. Koester (since 2006)
C                  Bernardo Zuniga-Gutierrez (since 2017)
C
C     Authors: Andreas M. Koester        (AMK)
C              Gerald Geudtner           (GG)
C              Aurelio Alvarez-Ibarra    (AAI)
C              Patrizia Calaminici       (PC)
C              Mark E. Casida            (MEC)
C              Javier Carmona-Espindola  (JCE)
C              Francisco A. Delesma      (FAD)
C              Rogelio Delgado Venegas   (RDV)
C              Victor D. Dominguez       (VDD)
C              Roberto Flores-Moreno     (RFM)
C              Gabriel U. Gamboa         (GUG)
C              Annick Goursot            (AG)
C              Thomas Heine              (TH)
C              Andrei Ipatov             (AI)
C              Florian Janetzko          (FJ)
C              Aurelien de la Lande      (ADLL)
C              Jorge Martin del Campo    (JMC)
C              Lars G.M. Pettersson      (LGM)
C              Nain Pedroza Montero      (NPM)
C              Daniel Mejia Rodriguez    (DMR)
C              J. Ulises Reveles         (UR)
C              Dennis R. Salahub         (DRS)
C              J. Manuel Vasquez Perez   (JMVP)
C              Alberto Vela              (AVA)
C              Bernardo Zuniga-Gutierrez (BZG)
C
C     Contributors: Samuel B. Trickey       (SBT, since 2010)
C                   Sourav Pal              (SP,  since 2011)
C                   Tzonka Mineva           (TM,  since 2016)
C                   Adrian A.M. Carranza    (AC,  since 2017)
C                   Morteza C. Amirani      (MA,  since 2018)
C                   Fernand Louisnard       (FL,  since 2019)
C
C     Former Contributors: Marcin Dulak     (MD,  2004 - 2006)
C                          Matthias Krack   (MK,  1996 - 1998)
C                          Martin Leboeuf   (ML,  1996 - 1998)
C                          Victor Medel     (VM,  2008 - 2010)
C                          Sapna Shedge     (SVS, 2011 - 2012) 
C                          Bernd Zimmermann (BZ,  1996 - 1999)
C                          Rui Zhang        (RZ,  2012 - 2014)
C                          Yue Zhang        (YZ,  2006 - 2007)
C
C     Copyright (C) The international deMon developer community.
C
C     Authors emails:
C
C     Andreas M. Koster         (AMK)  akoster@cinvestav.mx
C     Gerald Geudtner           (GG)   geudtner@cinvestav.mx
C     Aurelio Alvarez-Ibarra    (AAI)  aurelioai@gmail.com
C     Patrizia Calaminici       (PC)   pcalamin@cinvestav.mx
C     Mark E. Casida            (MEC)  mark.casida@ujf-grenoble.fr
C     Javier Carmona Espindola  (JCE)  jcarmona_26@yahoo.com.mx
C     Francisco A. Delesma      (FAD)  francisco.delesma@cinvestav.mx
C     Rogelio Delgado Venegas   (RDV)  rdelgado@cinvestav.mx
C     Victor D. Dominguez       (VDD)  danieldaso@yahoo.com.mx
C     Roberto Flores-Moreno     (RFM)  roberto.floresmoreno.qt@gmail.com
C     Gabriel Ulises Gamboa     (GUG)  gabrielulisesg@gmail.com
C     Annick Goursot            (AG)   annick.goursot@gmail.com
C     Thomas Heine              (TH)   t.heine@jacobs-university.de
C     Andrei Ipatov             (AI)   andrei_ipatov@mail.ru
C     Florian Janetzko          (FJ)   f_janetzko@web.de
C     Aurelien de la Lande      (ADLL) aurelien.de-la-lande@u-psud.fr
C     Jorge Martin del Campo    (JMC)  jmdelc@unam.mx
C     Nain Pedroza Montero      (NPM)  npedroza@cinvestav.mx
C     Lars G.M. Pettersson      (LGM)  lgm@fysik.su.se
C     Daniel Mejia Rodriguez    (DMR)  dmejia@cinvestav.mx
C     J. Ulises Reveles         (UR)   jureveles@gmail.com
C     Dennis R. Salahub         (DRS)  dennis.salahub@ucalgary.ca
C     J. Manuel Vasquez Perez   (JMVP) gsmanuel@gmail.com
C     Alberto Vela              (AVA)  avela@cinvestav.mx
C     Bernardo Zuniga-Gutierrez (BZG)  bzuniga51@gmail.com
C
C     Contributors emails:
C
C     Samuel B. Trickey       (SBT) trickey@qtp.ufl.edu
C     Sourav Pal              (SP)  s.pal@ncl.res.in
C     Tzonka Mineva           (TM)  tmineva@gmail.com
C     Adrian A.M. Carranza    (AC)  amartinezcar@cinvestav.mx
C     Morteza C. Amirani      (MA)  morteza.amirani@gmail.com
C     Fernand Louisnard       (FL)  flouisnard@irsamc.ups-tlse.fr
C
C     ******************************************************************
C
C     History: - Creation 6.0.1 (06.04.18, AMK)
C                         6.0.2 (23.07.18, GG)
C                         6.0.3 (14.12.18, AMK)
C                         6.0.4 (14.01.19, GG)
C                         6.0.5 (02.03.19, AMK)
C                         6.0.6 (23.07.19, AMK)
C                         6.0.7 (09.09.19, AMK)
C                         6.0.8 (01.11.19, GG)
C
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'align.h'
      INCLUDE 'auxis.h'
      INCLUDE 'cfp.h'
      INCLUDE 'cpks.h'
      INCLUDE 'custom.h'
      INCLUDE 'diis.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'embed.h'
      INCLUDE 'en.h'
      INCLUDE 'eri.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'freq.h'
      INCLUDE 'grid.h'
      INCLUDE 'interface.h'
      INCLUDE 'iteration.h'
      INCLUDE 'keywords.h'
      INCLUDE 'lebedev.h'
      INCLUDE 'machine.h'
      INCLUDE 'marching.h'
      INCLUDE 'math.h'
      INCLUDE 'minres.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmnrgy.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'mmscale.h'
      INCLUDE 'molden.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'mp_system.h'
      INCLUDE 'nmr.h'
      INCLUDE 'octree.h'
      INCLUDE 'opt.h'
      INCLUDE 'pcg.h'
      INCLUDE 'physic.h'
      INCLUDE 'plot.h'
      INCLUDE 'pointer.h'
      INCLUDE 'pointgroup.h' 
      INCLUDE 'polarizability.h'
      INCLUDE 'primitive.h'
      INCLUDE 'prm.h'
      INCLUDE 'project.h'
      INCLUDE 'property.h'
      INCLUDE 'ptmd.h'
      INCLUDE 'qmmm.h'
      INCLUDE 'roothaan.h'
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'strings.h'
      INCLUDE 'symmetry.h'
      INCLUDE 'tbauxis.h'
      INCLUDE 'tbgroup.h'
      INCLUDE 'tbmolecule.h'
      INCLUDE 'tddft.h'
      INCLUDE 'thermo.h'
      INCLUDE 'threshold.h'
      INCLUDE 'timer.h'
      INCLUDE 'tolerance.h'
      INCLUDE 'trafo.h'
      INCLUDE 'xc.h'
C
C     ------------------------------------------------------------------
C
C     *** Initialize parallel MPI run ***
C
      CALL MP_START
C
C     *** Initialize machine bounds ***
C
      CALL MACHINE
C
C     *** Open permanent files (input/output files) ***
C
      CALL FILEIO('OPEN PERMANENT FILES')
C
C     *** Write input in the standard deMon format ***
C
      CALL INPUT
C
C     *** Prepare a MD calculation with parallel tempering ***
C
      CALL PTMDPREP
C
C     *** Distribute input for parallel MPI run ***
C
      CALL MYINPUT
C
C     *** Print copyright header ***
C
      CALL COPYRIGHT
C
C     *** Initialize timer ***
C
      CALL TIMER('PRINT TOTAL','FIRST TIMING')
C
C     *** Initialization ***
C
      CALL INIDRV
C
C     *** Read and write input information ***
C
      STATUS = 'READ IN'
      CALL READIN
C
C     *** Read generated input for PES scan ***
C
      IF (SCANPES) THEN
        STATUS = 'SCAN'
        CALL MYINPUT
        CALL READIN
      END IF
C
C     *** Open external output files ***
C
      CALL FILEIO('OPEN EXTERNAL OUTPUTS')
C
C     *** Open temporary files (scratch files) ***
C
      CALL FILEIO('OPEN TEMPORARY FILES')
C
C     *** Modify trajector file ***
C
      CALL MODTRJ
C
C     *** Modify IRC path file ***
C
      CALL MODIRC
C
C     *** Geometry alingment ***
C
      STATUS = 'ALIGN'
      CALL ALIDRV
C
C     *** SCF symmetry analysis ***
C
      STATUS = 'SCF'
      CALL SYMDRV
      CALL GEODRV
C
C     *** Modify initial restart file ***
C
      IF (RESTART) CALL MODRST('INITIAL')
C
C     *** Generate SCF start density ***
C
      STATUS = 'SCF GUESS'
      CALL GUESSDRV
C
C     *** Single point SCF ***
C
      STATUS = 'SCF'
      CALL PESDRV
      CALL FORMMDRV
      CALL NEWDRV('NEW INPUT FILE')
C
C     *** Saddle interpolation ***
C
      STATUS = 'SAD'
      CALL SADDRV
C
C     *** Potential energy surface scan ***
C
      STATUS = 'SCAN'
      CALL SCANDRV
C
C     *** Structure optimization ***
C
      STATUS = 'OPT'
      CALL OPTDRV
C
C     *** Molecular dynamics simulation ***
C
      STATUS = 'BOMD'
      CALL MDDRV
C
C     *** Frequency analysis ***
C
      STATUS = 'FREQ'
      CALL FREQDRV
C
C     *** IRC path calculation ***
C
      STATUS = 'IRC'
      CALL IRCDRV
C
C     *** Polarizability calculation ***
C
      STATUS = 'POLAR'
      CALL POLDRV
C
C     *** Excited states calculation ***
C
      STATUS = 'TDDFT'
      CALL TDDRV
C
CCC ADLL
C     *** Electron dynamics simulations ***
C
      STATUS = 'RTDFT'
      CALL RTDRV
CCC ADLL 
C
C     *** Magnetic property calculation ***
C
      STATUS = 'MAGNETIC'
      CALL MAGDRV
C
C     *** X-ray property calculation ***
C
      STATUS = 'XRAY'
      CALL XRAYDRV
C
C     *** Electronic structure analysis ***
C
      STATUS = 'ESA'
      CALL ESADRV
C
C     *** Fukui reactivity analysis ***
C
      STATUS = 'FUKUI'
      CALL FKIDRV
C
C     *** Plotting or critical point search ***
C
      STATUS = 'PLOT'
      CALL PLOTDRV
C
C     *** Topological structure analysis ***
C
      STATUS = 'TOPO'
      CALL TOPDRV
C
C     *** Molecular property calculation ***
C
      STATUS = 'PROP'
      CALL PROPDRV
C
C     *** Write new MOLDEN file ***
C
      CALL NEWDRV('NEW MOLDEN FILE')
C
C     *** Close temporary files (scratch files) ***
C
      CALL FILEIO('CLOSE TEMPORARY FILES')
C
C     *** Print total program execution time and timing statistics ***
C
      CALL TIMER('STATISTICS','FINAL TIMING')
C
C     *** Close permanent files (input/output files) ***
C
      CALL FILEIO('CLOSE PERMANENT FILES')
C
C     *** End parallel MPI run ***
C
      CALL MP_END
C
C     ------------------------------------------------------------------
C
C     *** End of PROGRAM deMon ***
C
      END
