      SUBROUTINE EMBEDMM
C
C     Purpose: Embedding of the QM core Hamiltonian by MM part.
C
C     History: - Creation (04.08.13, AG) 
C                         (13.11.14, GG)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     H   : Core Hamiltonian -> Embedded core Hamiltonian.
C     WORK: Work matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
C
      CHARACTER*80 PRTSTR,TIMSTR
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: H(:,:),WORK(:,:)
C
C     ------------------------------------------------------------------
C
      IF (MPP) THEN
        TIMSTR = 'MPI QM/MM CALCULATION'
      ELSE
        TIMSTR = 'QM/MM CALCULATION'
      END IF
      CALL TIMER('START',TIMSTR)
C
C     *** Allocate local fields for embedding integrals ***
C
      ALLOCATE(H(NSTO,NSTO),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'EMBEDMM','ALLOCATION FAILED')
C
CCC ADLL COMMENT THIS HERE
C     *** Load core Hamiltonian ***
C
C      IF (MPMASTER.AND.((STATUS.EQ.'MAGNETIC').OR.
C     $                  (STATUS.EQ.'FFP'))) THEN
C        CALL FFPIO(H,NSTO,'READ','CORE')
C      ELSE IF (MPMASTER) THEN
C        CALL SCFIO(H,NSTO,'READ','CORE')
C      ELSE
C        H(:,:) = 0.0
C      END IF
      H(:,:) = 0.0
CCC ADLL
C
CCC ADLL LA
C     *** Scale down MM charges near link atoms ***
C
      CALL SCLCHGMM('SCALE')
C
CCC ADLL LA
C
C     *** Calculate far-field QM/MM integrals and add to H ***
C
      CALL FARQMMM(H,NSTO)
C
C     *** Calculate near-field QM/MM integrals and add to H ***
C
      CALL NEARQMMM(H,NSTO)
C
C     *** Collecting of the results on the master node ***
C
      CALL MP_COLLECT(H,NSTO*NSTO,MASTER,(/1.0/),.FALSE.)
C
      IF (MPMASTER) THEN
C
C     *** Symmetrization of core Hamiltonian ***
C
        CALL SYMMAT(H,NSTO,NSTO,'UPLOW')
C
CCC ADLL
C     *** Store embedding Hamiltonian on tape ***
C
        CALL SCFIO(H,NSTO,'WRITE','PCHGMM')
C
        ALLOCATE(WORK(NSTO,NSTO),STAT=ALLOCATION)
C
        IF ((STATUS.EQ.'MAGNETIC').OR.(STATUS.EQ.'FFP')) THEN
          CALL FFPIO(WORK,NSTO,'READ','CORE')
        ELSE 
          CALL SCFIO(WORK,NSTO,'READ','CORE')
        END IF
C
        H(:,:) = H(:,:) + WORK(:,:)
C
        DEALLOCATE (WORK,STAT=ALLOCATION)
C
CCC ADLL
C     *** Store embedded core Hamiltonian matrix on tape ***
C
        IF ((STATUS.EQ.'FFP').OR.(STATUS.EQ.'MAGNETIC')) THEN
          CALL FFPIO(H,NSTO,'WRITE','CORE')
        ELSE
          CALL SCFIO(H,NSTO,'WRITE','CORE')
        END IF
C
        IF (ERITYP.EQ.'DIRECT') THEN
          CALL SCFIO(H,NSTO,'WRITE','KSALPHA')
          IF (OKS) CALL SCFIO(H,NSTO,'WRITE','KSBETA')
        END IF
C
C     *** Print MM embedded core Hamiltonian matrix ***
C
        IF (PRTKS.AND.(STATUS.EQ.'SCF')) THEN
          PRTSTR = 'MM EMBEDDED CORE HAMILTONIAN'
          IF (SPHORB) THEN
            ALLOCATE(WORK(NSTO,NSTO))
            CALL CMTOSM(H,WORK,NSTO)
            H(:,:) = WORK(:,:)
            DEALLOCATE(WORK)
            CALL PRISMAT(H,NSTO,1,NSPH,1,NSPH,3,5,OUT,'BASIS','TOTAL',
     $                   PRTSTR)
          ELSE
            CALL PRIMAT(H,NSTO,1,NSTO,1,NSTO,3,5,OUT,'BASIS','TOTAL',
     $                  PRTSTR)
          END IF
        END IF
C
      END IF
C
CCC ADLL LA
C     *** Restore orginal charges ***
C
      CALL SCLCHGMM('RESTORE')
C
CCC ADLL LA
C     *** Deallocate local fields for embedding integrals ***
C
      DEALLOCATE(H,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'EMBEDMM','DEALLOCATION FAILED')
C
      CALL TIMER('STOP',TIMSTR)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE EMBEDMM ***
C
      END
