      SUBROUTINE FILEIO(OPTION)
C
C     Purpose: Driver routine for file Input/Output.
C
C     History: - Creation (07.01.98, MK)
C                         (09.02.17, AMK)
C                         (07.12.17, GG)
C                         (19.06.18, MA)
C                         (09.07.19, RDV)
C                         (25.10.19, FL)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     OPTION: a) OPEN PERMANENT FILES      = Open input/output files.
C             b) OPEN REACTION FILES       = Open educt/product files.
C             c) OPEN EXTERNAL INPUTS      = Open external input files.
C             d) OPEN EXTERNAL OUTPUTS     = Open external output files.
C             e) OPEN TEMPORARY FILES      = Open scratch files.
C             f) OPEN TEMPORARY SCF FILES  = Open SCF scratch files.
C             g) OPEN PLOT FILES           = Open plot files.
C             h) OPEN ISO FILES            = Open isosurface files.
C             i) CLOSE ISO FILES           = Close isosurface files.
C             j) CLOSE PLOT FILES          = Close plot files.
C             k) CLOSE PERMANENT FILES     = Close input/output files.
C             l) CLOSE TEMPORARY FILES     = Close scratch files.
C             m) CLOSE TEMPORARY SCF FILES = Close SCF scratch files.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'cdft.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'opt.h'
      INCLUDE 'plot.h'
C
      CHARACTER OPTION*(*)
C
      LOGICAL EMPTYIO,EX,NMD,OD
      CHARACTER*80 LTU,NAME*20,PRTSTR,STRCOMP
      CHARACTER DIRECT*3,MYFILE*4,SEQUENTIAL*3
      INTEGER DIMTRJ,MPGSAV,STREXT,TAPE
C
      REAL RDUMMY(1)
C
C     ------------------------------------------------------------------
C
C     *** Ensure that unique file names can be created ***
C
      IF (OPTION(1:4).EQ.'OPEN') THEN
        MPGSAV = MPCURGRP
        CALL MP_GROUP(MPGALL,0,'SET')
      END IF
C
C     *** Open input/output files ***
C
      IF (OPTION.EQ.'OPEN PERMANENT FILES') THEN
C
        OPEN (AUX,FILE='AUXIS',STATUS='UNKNOWN',POSITION='REWIND',
     $        ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=99930)
        OPEN (BAS,FILE='BASIS',STATUS='UNKNOWN',POSITION='REWIND',
     $        ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=99940)
        OPEN (ECP,FILE='ECPS',STATUS='UNKNOWN',POSITION='REWIND',
     $        ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=99950)
        OPEN (MCP,FILE='MCPS',STATUS='UNKNOWN',POSITION='REWIND',
     $        ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=99960)
        OPEN (PRM,FILE='FFDS',STATUS='UNKNOWN',POSITION='REWIND',
     $        ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=99970)
        OPEN (C6D3,FILE='D3CFS',STATUS='UNKNOWN',POSITION='REWIND',
     $        ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=99980)
C
C     *** Multiple input/output files for parallel processing ***
C
        IF (MPP.AND.(ME.GT.0)) THEN
C
          OPEN (GEO,FILE='deMon.geo'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='FORMATTED')
          OPEN (INP,FILE='deMon.inp'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='FORMATTED')
          OPEN (NEW,FILE='deMon.new'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='FORMATTED')
          OPEN (OUT,FILE='deMon.out'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='FORMATTED')
          OPEN (TRJ,FILE='deMon.trj'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='FORMATTED')
          OPEN (MEM,FILE='deMon.mem'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='UNFORMATTED')
          OPEN (RST,FILE='deMon.rst'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='UNFORMATTED')
CCC ADLL
          OPEN (RSTRT,FILE='deMon.rstrt'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',
     $          FORM='UNFORMATTED')
CCC ADLL
C
        ELSE
C
          OPEN (GEO,FILE='deMon.geo',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED') 
          OPEN (INP,FILE='deMon.inp',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
          OPEN (NEW,FILE='deMon.new',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
          OPEN (OUT,FILE='deMon.out',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
          OPEN (TRJ,FILE='deMon.trj',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
          OPEN (MEM,FILE='deMon.mem',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
          OPEN (RST,FILE='deMon.rst',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
CCC ADLL
          OPEN (RSTRT,FILE='deMon.rstrt',STATUS='UNKNOWN',
     $         POSITION='REWIND',ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
CCC ADLL
        END IF
C
C     *** Define temporary scratch file names ***
C
        IF (MPP) THEN
C
          TAPENAME(IOADP) = 'ioadp.'//MYFILE(ME)
          TAPENAME(IOAOA) = 'ioaoa.'//MYFILE(ME)
          TAPENAME(IOCDF) = 'iocdf.'//MYFILE(ME)
          TAPENAME(IODE2) = 'iode2.'//MYFILE(ME)
          TAPENAME(IODIR) = 'iodir.'//MYFILE(ME)
          TAPENAME(IODIS) = 'iodis.'//MYFILE(ME)
          TAPENAME(IOERI) = 'ioeri.'//MYFILE(ME)
          TAPENAME(IOETM) = 'ioetm.'//MYFILE(ME)
          TAPENAME(IOEXX) = 'ioexx.'//MYFILE(ME)
          TAPENAME(IOFFD) = 'ioffd.'//MYFILE(ME)
          TAPENAME(IOFFP) = 'ioffp.'//MYFILE(ME)
          TAPENAME(IOGRD) = 'iogrd.'//MYFILE(ME)
          TAPENAME(IOIRC) = 'ioirc.'//MYFILE(ME)
          TAPENAME(IOISO) = 'ioiso.'//MYFILE(ME)
          TAPENAME(IOLRC) = 'iolrc.'//MYFILE(ME)
          TAPENAME(IOMAG) = 'iomag.'//MYFILE(ME)
          TAPENAME(IOMIX) = 'iomix.'//MYFILE(ME)
          TAPENAME(IOMOA) = 'iomoa.'//MYFILE(ME)
          TAPENAME(IOOPT) = 'ioopt.'//MYFILE(ME)
          TAPENAME(IOPCG) = 'iopcg.'//MYFILE(ME)
          TAPENAME(IOPMR) = 'iopmr.'//MYFILE(ME)
          TAPENAME(IOPRO) = 'iopro.'//MYFILE(ME)
          TAPENAME(IORPA) = 'iorpa.'//MYFILE(ME)
          TAPENAME(IOSCF) = 'ioscf.'//MYFILE(ME)
          TAPENAME(IOSCP) = 'ioscp.'//MYFILE(ME)
          TAPENAME(IOTOP) = 'iotop.'//MYFILE(ME)
          TAPENAME(IOTRJ) = 'iotrj.'//MYFILE(ME)
          TAPENAME(IOVEC) = 'iovec.'//MYFILE(ME)
          TAPENAME(IOFFD) = 'ioffd.'//MYFILE(ME)
CCC ADLL 
          TAPENAME(IORTP) = 'iortp.'//MYFILE(ME)
CCC ADLL
C
        ELSE
C
          TAPENAME(IOADP) = 'ioadp.scr'
          TAPENAME(IOAOA) = 'ioaoa.scr'
          TAPENAME(IOCDF) = 'iocdf.scr'
          TAPENAME(IODE2) = 'iode2.scr'
          TAPENAME(IODIR) = 'iodir.scr'
          TAPENAME(IODIS) = 'iodis.scr'
          TAPENAME(IOERI) = 'ioeri.scr'
          TAPENAME(IOETM) = 'ioetm.scr'
          TAPENAME(IOEXX) = 'ioexx.scr'
          TAPENAME(IOFFD) = 'ioffd.scr'
          TAPENAME(IOFFP) = 'ioffp.scr'
          TAPENAME(IOGRD) = 'iogrd.scr'
          TAPENAME(IOIRC) = 'ioirc.scr'
          TAPENAME(IOISO) = 'ioiso.scr'
          TAPENAME(IOLRC) = 'iolrc.scr'
          TAPENAME(IOMAG) = 'iomag.scr'
          TAPENAME(IOMIX) = 'iomix.scr'
          TAPENAME(IOMOA) = 'iomoa.scr'
          TAPENAME(IOOPT) = 'ioopt.scr'
          TAPENAME(IOPCG) = 'iopcg.scr'
          TAPENAME(IOPMR) = 'iopmr.scr'
          TAPENAME(IOPRO) = 'iopro.scr'
          TAPENAME(IORPA) = 'iorpa.scr'
          TAPENAME(IOSCF) = 'ioscf.scr'
          TAPENAME(IOSCP) = 'ioscp.scr'
          TAPENAME(IOTOP) = 'iotop.scr'
          TAPENAME(IOTRJ) = 'iotrj.scr'
          TAPENAME(IOVEC) = 'iovec.scr'
          TAPENAME(IOFFD) = 'ioffd.scr'
CCC ADLL
          TAPENAME(IORTP) = 'iotrp.scr'
CCC ADLL
C
        END IF
C
C     *** Initialize file roots ***
C
        TAPEROOT(:) = 'UNDEFINED'
C
C     *** Define external file roots ***
C
        TAPEROOT(AUX) = 'EXTERNAL'
        TAPEROOT(BAS) = 'EXTERNAL'
        TAPEROOT(ECP) = 'EXTERNAL'
        TAPEROOT(MCP) = 'EXTERNAL'
        TAPEROOT(PRM) = 'EXTERNAL'
        TAPEROOT(C6D3) = 'EXTERNAL'
C
C     *** Define input/output file roots ***
C
        TAPEROOT(GEO) = 'I/O'
        TAPEROOT(INP) = 'I/O'
        TAPEROOT(NEW) = 'I/O'
        TAPEROOT(OUT) = 'I/O'
        TAPEROOT(TRJ) = 'I/O'
        TAPEROOT(MEM) = 'I/O'
        TAPEROOT(RST) = 'I/O'
CCC ADLL
        TAPEROOT(RSTRT) = 'I/O'
CCC ADLL
C
C     *** Define temporary scratch file roots ***
C
        TAPEROOT(IOADP) = 'SCP'
        TAPEROOT(IOAOA) = 'SCP'
        TAPEROOT(IOCDF) = 'SCF'
        TAPEROOT(IODE2) = 'SCP'
        TAPEROOT(IODIR) = 'SCF'
        TAPEROOT(IODIS) = 'SCF'
        TAPEROOT(IOERI) = 'SCF'
        TAPEROOT(IOETM) = 'ETM'
        TAPEROOT(IOEXX) = 'SCF'
        TAPEROOT(IOFFD) = 'PRM'
        TAPEROOT(IOFFP) = 'SCP'
        TAPEROOT(IOGRD) = 'SCF'
        TAPEROOT(IOIRC) = 'TRJ'
        TAPEROOT(IOISO) = 'ISO'
        TAPEROOT(IOLRC) = 'SCP'
        TAPEROOT(IOMAG) = 'SCP'
        TAPEROOT(IOMIX) = 'SCF'
        TAPEROOT(IOMOA) = 'SCP'
        TAPEROOT(IOOPT) = 'OPT'
        TAPEROOT(IOPCG) = 'SCF'
        TAPEROOT(IOPMR) = 'SCF'
        TAPEROOT(IOPRO) = 'POP'
        TAPEROOT(IORPA) = 'SCP'
        TAPEROOT(IOSCF) = 'SCF'
        TAPEROOT(IOSCP) = 'SCP'
        TAPEROOT(IOTOP) = 'POP'
        TAPEROOT(IOTRJ) = 'TRJ'
        TAPEROOT(IOVEC) = 'SCP'
        TAPEROOT(IOFFD) = 'PRM'
CCC ADLL 
        TAPEROOT(IORTP) = 'RTP'
CCC ADLL
C
C     *** Initialize position indices ***
C
        ADPIDX(:) = 0
        DE2IDX(:) = 0
        FFPIDX(:) = 0
        LRCIDX(:) = 0
        MAGIDX(:) = 0
        SCPIDX(:) = 0
C
C     *** Open MM parameter scratch file ***
C
        OPEN (IOFFD,FILE=TAPENAME(IOFFD),STATUS='UNKNOWN',
     $        ACCESS='SEQUENTIAL',FORM='FORMATTED')
C
C     *** Open educt/product files ***
C
      ELSE IF (OPTION.EQ.'OPEN REACTION FILES') THEN
C
        IF (MPP.AND.(ME.GT.0)) THEN
C
          OPEN (EDU,FILE='deMon.edu'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',FORM='FORMATTED')
          OPEN (PRO,FILE='deMon.pro'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',FORM='FORMATTED')
C
        ELSE
C
          OPEN (EDU,FILE='deMon.edu',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
          OPEN (PRO,FILE='deMon.pro',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
C
        END IF
C
C     *** Define educt/product file roots ***
C
        TAPEROOT(EDU) = 'REACTION'
        TAPEROOT(PRO) = 'REACTION'
C
C     *** Open external input files ***
C
      ELSE IF (OPTION.EQ.'OPEN EXTERNAL INPUTS') THEN
C
        INQUIRE (UNIT=LAT,OPENED=OD)
        IF (LATINP.EQ.'EXTERNAL BINARY') THEN
          TAPENAME(LAT) = 'LAT.bin'
          IF (.NOT.OD) CALL SCRDIR(RDUMMY,3,3,1,LAT,'OPEN')
        ELSE IF (LATINP.EQ.'EXTERNAL ASCII') THEN
          IF (OD) THEN
            REWIND (LAT)
          ELSE
            OPEN (LAT,FILE='LAT.asc',ACCESS='SEQUENTIAL',
     $            POSITION='REWIND',FORM='FORMATTED')
          END IF
        END IF
C
C     *** Open external output files ***
C
      ELSE IF (OPTION.EQ.'OPEN EXTERNAL OUTPUTS') THEN
C
C     *** Open external Molden/Molekel/WFN input file ***
C
        IF (MPP.AND.(ME.GT.0).AND.(MOLDEN.NE.'NONE')) THEN
          OPEN (MOL,FILE='deMon.mol'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',FORM='FORMATTED')
        ELSE IF (MPP.AND.(ME.GT.0).AND.(MOLEKEL.NE.'NONE')) THEN
          OPEN (MOL,FILE='deMon.mkl'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',FORM='FORMATTED')
        ELSE IF (MPP.AND.(ME.GT.0).AND.(WFNFILE.NE.'NONE')) THEN
          OPEN (MOL,FILE='deMon.wfn'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',FORM='FORMATTED')
        ELSE IF (MOLDEN.NE.'NONE') THEN
          OPEN (MOL,FILE='deMon.mol',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        ELSE IF (MOLEKEL.NE.'NONE') THEN
          OPEN (MOL,FILE='deMon.mkl',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        ELSE IF (WFNFILE.NE.'NONE') THEN
          OPEN (MOL,FILE='deMon.wfn',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        END IF
C
C     *** Open external CHARMM QM/MM interface file ***
C
        IF (MPMASTER.AND.CHARMM) THEN
          OPEN (QMM,FILE='deMon.qmm',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        END IF
C
C     *** Open external XAS/XES analysis file ***
C
        IF (MPMASTER.AND.(XAS.OR.XES)) THEN
          OPEN (XRY,FILE='deMon.xry',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        END IF
C
C     *** Open external NBO input file ***
C
        IF (MPP.AND.(ME.GT.0).AND.NPA) THEN
          OPEN (NBO,FILE='deMon.nbo'//MYFILE(ME),STATUS='UNKNOWN',
     $          POSITION='REWIND',ACCESS='SEQUENTIAL',FORM='FORMATTED')
        ELSE IF (NPA) THEN
          OPEN (NBO,FILE='deMon.nbo',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        END IF
C
C     *** Open external PT file to record the replicas temperature ***
C
        IF ((MPGME(MPGALL) == MPGMASTER(MPGALL)).AND.PTMD) THEN
          OPEN (PT,FILE='deMon.pt',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        END IF
CCC ADLL
C     *** Open external RTDFT analysis file ***
C
        IF (MPMASTER.AND.RTDFT) THEN
          OPEN (RTP,FILE='deMon.rtp',STATUS='UNKNOWN',POSITION='REWIND',
     $          ACCESS='SEQUENTIAL',FORM='FORMATTED')
        END IF
CCC ADLL
C
C     *** Open temporary scratch files ***
C
      ELSE IF (OPTION.EQ.'OPEN TEMPORARY FILES') THEN
C
        IF ((XTYP.NE.'NOFUN').OR.(CTYP.NE.'NOFUN')) THEN
          IF (.NOT.DIRGRD) CALL GRDIO(1,'OPEN')
        END IF
C
        IF (EXX.OR.(ERITYP.EQ.'DIRECT')) THEN
          CALL SCFIO(RDUMMY,NSTO,'OPEN','KSALPHA')
        END IF
C
        IF (EXX.OR.OMA.OR.(.NOT.CDXC)) THEN
          CALL MIXIO(RDUMMY,NSTO,'OPEN','PALPHA')
        END IF
C
        IF (DIIS) THEN
          IF (EXX) CALL DIISIO(RDUMMY,NSTO,'OPEN','EXXALPHA',1)
          IF (.NOT.CDXC) CALL DIISIO(RDUMMY,NSTO,'OPEN','PALPHA',1)
        END IF
C
        IF ((FITTING.EQ.'NUMERICAL').AND.USESVD) THEN
          CALL PCGIO(RDUMMY,1,1,'OPEN')
        ELSE IF ((FITTING.EQ.'NUMERICAL').AND.USEMIN) THEN
          CALL PMRIO(RDUMMY,NADB,'OPEN')
        END IF
C
        IF (OPT.OR.IRC.OR.SADTS) THEN
          CALL OPTIO(RDUMMY,NDEG**2,'OPEN','HESSIAN')
        END IF
C
        IF (IRC) THEN
          CALL IRCIO(RDUMMY,NDEG**2+3,'OPEN')
        END IF
C
        IF (MD) THEN
          CALL TRJIO(RDUMMY,DIMTRJ(TRJLEN),'OPEN')
        END IF
C
        IF ((FREQTYP.EQ.'ANALYTIC').OR.POLARIS.OR.TDDFT) THEN
          CALL ADPIO(RDUMMY,0,NAUX,'OPEN','RMATRIX','ALPHA')
          ADPIDX(:) = 0
        END IF
C
        IF (FREQTYP.EQ.'ANALYTIC') THEN
          CALL DE2IO(RDUMMY,NSTO,'OPEN','KSXA')
          CALL LRCIO(RDUMMY,0,NAUX,'OPEN','LRXCD0','ALPHA')
          DE2IDX(:) = 0
          LRCIDX(:) = 0
        END IF
C
        IF (MXI.OR.NMR.OR.NQR) THEN
          CALL MAGIO(RDUMMY,NSTO,'OPEN','CORE')
          MAGIDX(:) = 0
        END IF
C
        IF (DEFORM.OR.USEDD) THEN
          CALL PROIO(RDUMMY,NSTO,'OPEN')
        END IF
CCC ADLL
        IF (RTDFT) THEN
          CALL RTPIO(RDUMMY,RDUMMY,NSTO,NAUX,'OPEN')
        END IF
CCC ADLL
C
        IF (.NOT.MMONLY) THEN
          CALL CDFIO(RDUMMY,NAUX,1,'OPEN','CDERI')
          CALL FFPIO(RDUMMY,NSTO,'OPEN','CORE')
          CALL SCFIO(RDUMMY,NSTO,'OPEN','OVERLAP')
          CALL SCPIO(RDUMMY,NSTO,'OPEN','CALPHA')
          FFPIDX(:) = 0
          SCPIDX(:) = 0
        END IF
C
        IF (ETMOD) THEN
          CALL ETMIO(RDUMMY,NSTO,'OPEN','CALPHA1')
        END IF
C
C     *** Open temporary SCF scratch files ***
C
      ELSE IF (OPTION.EQ.'OPEN TEMPORARY SCF FILES') THEN
C
        IF ((XTYP.NE.'NOFUN').OR.(CTYP.NE.'NOFUN')) THEN
          IF (.NOT.DIRGRD) CALL GRDIO(1,'OPEN')
        END IF
C
        IF (EXX.OR.OMA.OR.(.NOT.CDXC)) THEN
          CALL MIXIO(RDUMMY,NSTO,'OPEN','PALPHA')
        END IF
C
        IF (DIIS) THEN
          IF (EXX) CALL DIISIO(RDUMMY,NSTO,'OPEN','EXXALPHA',1)
          IF (.NOT.CDXC) CALL DIISIO(RDUMMY,NSTO,'OPEN','PALPHA',1)
        END IF
C
        IF (EXX.OR.(ERITYP.EQ.'DIRECT')) THEN
          CALL SCFIO(RDUMMY,NSTO,'OPEN','KSALPHA')
        END IF
C
        IF ((FITTING.EQ.'NUMERICAL').AND.USESVD) THEN
          CALL PCGIO(RDUMMY,1,1,'OPEN')
        ELSE IF ((FITTING.EQ.'NUMERICAL').AND.USEMIN) THEN
          CALL PMRIO(RDUMMY,NADB,'OPEN')
        END IF
C
        CALL CDFIO(RDUMMY,NAUX,1,'OPEN','CDERI')
        CALL SCFIO(RDUMMY,NSTO,'OPEN','OVERLAP')
C
C     *** Open plot files ***
C
      ELSE IF (OPTION.EQ.'OPEN PLOT FILES') THEN
C
        IF (PLOTTYP.EQ.'CPSEARCH') THEN
          CALL SCRSEQ(RDUMMY,1,IOTOP,'OPEN')
          CALL BINARY(RDUMMY,1,3,1,IOPLOT,PLOTNAME(0),'OPEN')
        ELSE
          CALL BINARY(RDUMMY,1,1,1,IOPLOT,PLOTNAME(0),'OPEN')
        END IF
C
        IF ((PLOT.EQ.'ASCII').OR.(PLOT.EQ.'CUBE')) THEN
          PRTSTR = PLOTNAME(0)
          PRTSTR(STREXT(PRTSTR)-2:) = 'asc'
          IF (MPMASTER) THEN
            OPEN (PIE,FILE=PRTSTR,ACCESS='SEQUENTIAL',FORM='FORMATTED')
          END IF
CCC ADLL
        ELSE IF (PLOT.EQ.'CUBE') THEN
          PRTSTR = PLOTNAME(0)
          PRTSTR(STREXT(PRTSTR)-2:) = 'cube'
          IF (MPMASTER) THEN
            OPEN (PIE,FILE=PRTSTR,ACCESS='SEQUENTIAL',FORM='FORMATTED')
          END IF
CCC ADLL
        ELSE IF (PLOT.EQ.'BINARY') THEN
          IF (MPMASTER) THEN
            OPEN (PIE,FILE='deMon.pie',ACCESS='SEQUENTIAL',
     $            FORM='FORMATTED')
          END IF
        END IF
C
C     *** Open isosurface files ***
C
      ELSE IF (OPTION.EQ.'OPEN ISO FILES') THEN
C
        CALL BINARY(RDUMMY,1,1,1,IOPLOT,PLOTNAME(0),'OPEN')
        CALL ISOIO('OPEN')
C
        IF (PLOT.EQ.'ASCII') THEN
          OPEN (LAT,FILE='deMon.lat',ACCESS='SEQUENTIAL',
     $          FORM='FORMATTED')
        ELSE IF (PLOT.EQ.'BINARY') THEN
          IF (MPMASTER) THEN
            OPEN (PIE,FILE='deMon.pie',ACCESS='SEQUENTIAL',
     $            FORM='FORMATTED')
            CALL BINARY(RDUMMY,3,3,1,LAT,'LAT.bin','OPEN')
          END IF
        END IF
C
C     *** Close isosurface files ***
C
      ELSE IF (OPTION.EQ.'CLOSE ISO FILES') THEN
C
        CALL BINARY(RDUMMY,1,1,1,IOPLOT,PLOTNAME(0),'DELETE')
        CALL ISOIO('DELETE')
C
C     *** Close plot files ***
C
      ELSE IF (OPTION.EQ.'CLOSE PLOT FILES') THEN
C
        IF ((PLOT.EQ.'BINARY').AND.(PLOTTYP.NE.'CPSEARCH')) THEN
          CALL BINARY(RDUMMY,1,1,1,IOPLOT,PLOTNAME(0),'CLOSE')
        ELSE IF (PLOTTYP.EQ.'CPSEARCH') THEN
          CALL SCRSEQ(RDUMMY,1,IOTOP,'DELETE')
          CALL BINARY(RDUMMY,1,3,1,IOPLOT,PLOTNAME(0),'DELETE')
        ELSE
          CALL BINARY(RDUMMY,1,1,1,IOPLOT,PLOTNAME(0),'DELETE')
        END IF
C
C     *** Close all input/output files ***
C
      ELSE IF (OPTION.EQ.'CLOSE PERMANENT FILES') THEN
C
        DO TAPE=MINSEQ,MAXSEQ
          INQUIRE (UNIT=TAPE,NAMED=NMD,OPENED=OD,SEQUENTIAL=SEQUENTIAL)
          IF (LTU(SEQUENTIAL).EQ.'YES') THEN
            IF (TAPEROOT(TAPE).EQ.'REACTION') THEN
              CLOSE (UNIT=TAPE,STATUS='DELETE')
            ELSE IF (NMD.AND.OD) THEN
              REWIND (TAPE,ERR=99990)
              IF (EMPTYIO(TAPE)) THEN
                CLOSE (UNIT=TAPE,STATUS='DELETE')
              ELSE
                CLOSE (UNIT=TAPE,STATUS='KEEP')
              END IF
            ELSE IF (OD) THEN
              CLOSE (UNIT=TAPE,STATUS='DELETE')
            END IF
          ELSE
            CLOSE (UNIT=TAPE,STATUS='KEEP')
          END IF
        END DO
C
C     *** Close all temporary scratch files ***
C
      ELSE IF (OPTION.EQ.'CLOSE TEMPORARY FILES') THEN
C
        DO TAPE=MINIO,MAXIO
          NAME = TAPENAME(TAPE)
          INQUIRE (FILE=NAME,EXIST=EX,OPENED=OD)
          INQUIRE (FILE=NAME,DIRECT=DIRECT,SEQUENTIAL=SEQUENTIAL)
          IF (OD) THEN
            CLOSE (UNIT=TAPE,STATUS='DELETE')
          ELSE IF (EX) THEN
            IF (LTU(DIRECT).EQ.'YES') THEN
              OPEN (UNIT=TAPE,FILE=NAME,STATUS='UNKNOWN',
     $             ACCESS='DIRECT',FORM='UNFORMATTED',RECL=1)
            ELSE IF (LTU(SEQUENTIAL).EQ.'YES') THEN
              OPEN (UNIT=TAPE,FILE=NAME,STATUS='UNKNOWN',
     $              ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
            END IF
            CLOSE (UNIT=TAPE,STATUS='DELETE')
          END IF
        END DO
C
C     *** Close temporary SCF scratch files ***
C
      ELSE IF (OPTION.EQ.'CLOSE TEMPORARY SCF FILES') THEN
C
        DO TAPE=MINIO,MAXIO
          IF (TAPEROOT(TAPE).EQ.'SCF') THEN
            NAME = TAPENAME(TAPE)
            INQUIRE (FILE=NAME,EXIST=EX,OPENED=OD)
            INQUIRE (FILE=NAME,DIRECT=DIRECT,SEQUENTIAL=SEQUENTIAL)
            IF (OD) THEN
              CLOSE (UNIT=TAPE,STATUS='DELETE')
            ELSE IF (EX) THEN
              IF (LTU(DIRECT).EQ.'YES') THEN
                OPEN (UNIT=TAPE,FILE=NAME,STATUS='UNKNOWN',
     $                ACCESS='DIRECT',FORM='UNFORMATTED',RECL=1)
              ELSE IF (LTU(SEQUENTIAL).EQ.'YES') THEN
                OPEN (UNIT=TAPE,FILE=NAME,STATUS='UNKNOWN',
     $                ACCESS='SEQUENTIAL',FORM='UNFORMATTED')
              END IF
              CLOSE (UNIT=TAPE,STATUS='DELETE')
            END IF
          END IF
        END DO
C
C     *** Error handling ***
C
      ELSE
C
        PRTSTR = 'UNKNOWN OPTION '//OPTION//' SPECIFIED'
        CALL ERRMSG('FILEIO',STRCOMP(PRTSTR),2)
C
      END IF
C
      IF (OPTION(1:4).EQ.'OPEN') THEN
        CALL MP_GROUP(MPGSAV,0,'SET')
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Error handling ***
C
99930 CONTINUE
      CALL ERRMSG('FILEIO','OPEN FOR FILE AUXIS FAILED',1)
C
99940 CONTINUE
      CALL ERRMSG('FILEIO','OPEN FOR FILE BASIS FAILED',1)
C
99950 CONTINUE
      CALL ERRMSG('FILEIO','OPEN FOR FILE ECPS FAILED',1)
C
99960 CONTINUE
      CALL ERRMSG('FILEIO','OPEN FOR FILE MCPS FAILED',1)
C
99970 CONTINUE
      CALL ERRMSG('FILEIO','OPEN FOR FILE FFDS FAILED',1)
C
99980 CONTINUE
      CALL ERRMSG('FILEIO','OPEN FOR FILE C6D3 FAILED',1)
C
99990 CONTINUE
      CLOSE (UNIT=TAPE,STATUS='KEEP')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE FILEIO ***
C
      END
