      SUBROUTINE NEARFIELD(ATOM,NEAR,NCTR,OPTION)
C
C     Purpose: Get near-field relation between ATOM and all other
C              QM atoms, MM atoms or embedding points.
C
C     History: - Creation (06.12.13, AAI)
C                         (18.02.15, AMK)
C
C     ******************************************************************
C
C     List of local variables:
C
C     NCTR  : Number of centers.
C     NEAR  : Logical field for near-field relation.
C     OPTION: a) OPTION = ATOMS, near-field check between atoms.
C             b) OPTION = QM/MM, near-field check between QM & MM atoms.
C             c) OPTION = POINTS, near-field check between atom & point.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'embed.h'
      INCLUDE 'eri.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
CCC ADLL  
      INCLUDE 'projectile.h'
CCC ADLL
C
      INTEGER NCTR
      LOGICAL NEAR(NCTR)
C
      CHARACTER*(*) OPTION
CCC ADLL
      INTEGER ATOM,IATOM,IEMBED,IRAD
CCC ADLL
      REAL RADIUS,RSQUARE
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      NEAR(:) = .TRUE.
C
C     *** Calculate near-field relation between ATOM and QM atoms ***
C
      IF (OPTION.EQ.'ATOMS') THEN
C
        IF (.NOT.MULTIPOLE) RETURN
C
        DO 10 IATOM=1,NATOM
          RSQUARE = (COORD(1,ATOM,CCSET) - COORD(1,IATOM,CCSET))**2 +
     $              (COORD(2,ATOM,CCSET) - COORD(2,IATOM,CCSET))**2 +
     $              (COORD(3,ATOM,CCSET) - COORD(3,IATOM,CCSET))**2
          RADIUS = ERIRAD(ATOM,1) + ERIRAD(IATOM,2)
          IF (SQRT(RSQUARE).GE.RADIUS) NEAR(IATOM) = .FALSE.
   10   CONTINUE
C
C     *** Calculate near-field relation between ATOM and MM atoms ***
C
      ELSE IF (OPTION.EQ.'QM/MM') THEN
C
        IF (.NOT.MONOPOLE) RETURN
C
        DO 20 IATOM=NATOM+1,NATOM_ALL
          RSQUARE = (COORD(1,ATOM,CCSET) - COORD(1,IATOM,CCSET))**2 +
     $              (COORD(2,ATOM,CCSET) - COORD(2,IATOM,CCSET))**2 +
     $              (COORD(3,ATOM,CCSET) - COORD(3,IATOM,CCSET))**2
          RADIUS = ERIRAD(ATOM,1) + COVRAD(ATOMNO(IATOM))
          IF (SQRT(RSQUARE).GE.RADIUS) NEAR(IATOM-NATOM) = .FALSE.
   20   CONTINUE
C
C     *** Calculate near-field relation between ATOM and points ***
C
      ELSE IF (OPTION.EQ.'POINTS') THEN
C
        IF (.NOT.MONOPOLE) RETURN
C
        DO 30 IEMBED=1,NEMBED
          RSQUARE = (COORD(1,ATOM,CCSET) - CEMBED(1,IEMBED,CCSET))**2 +
     $              (COORD(2,ATOM,CCSET) - CEMBED(2,IEMBED,CCSET))**2 +
     $              (COORD(3,ATOM,CCSET) - CEMBED(3,IEMBED,CCSET))**2
          RADIUS = ERIRAD(ATOM,1) + REMBED(IEMBED)
          IF (SQRT(RSQUARE).GE.RADIUS) NEAR(IEMBED) = .FALSE.
   30   CONTINUE
C
CCC ADLL
C     *** Calculate near-field relation between ATOM and points ***
C
      ELSE IF (OPTION.EQ.'RADIATION') THEN
C
        IF (.NOT.MONOPOLE) RETURN
C
        DO 40 IRAD=1,NRADP
          RSQUARE = (COORD(1,ATOM,CCSET) - CRAD(1,IRAD,CCSET))**2 +
     $              (COORD(2,ATOM,CCSET) - CRAD(2,IRAD,CCSET))**2 +
     $              (COORD(3,ATOM,CCSET) - CRAD(3,IRAD,CCSET))**2
          RADIUS = ERIRAD(ATOM,1) + RRAD(IRAD)
          IF (SQRT(RSQUARE).GE.RADIUS) NEAR(IRAD) = .FALSE.
   40   CONTINUE
CCC ADLL
C
C     *** Error handling ***
C
      ELSE
C
        CALL ERRMSG('NEARFIELD','UNKWNOWN OPTION',1)
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NEARFIELD ***
C
      END
