      SUBROUTINE PESDRV
C
C     Purpose: Driver routine for potential energy surface calculation.
C
C     History: - Creation (03.07.14, AMK)
C                         (13.11.14, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C     MMONLY: If true, a pure MM calculation is performed.
C     QMMM  : If true, a mixed QM/MM calculation is performed.
C     QMONLY: If true, a pure QM calculation is performed.
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'iteration.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmnrgy.h'
      INCLUDE 'molecule.h'
      INCLUDE 'scf.h'
C
C     ------------------------------------------------------------------
C
CCC ADLL
C     *** Skip SCF if restarting a RT-TDDFT simulation ***
C
      IF (RTRST) RETURN
C
CCC ADLL
C     *** Potential energy surface point for QM calculation ***
C
      IF (QMONLY) THEN
C
        CALL SCFDRV
        EPES = ESCF
        IF (SCF.AND.(STATUS.EQ.'SCF')) THEN
          CALL TIMER('PRINT STEP','SCF ITERATION')
        END IF
C
C     *** Potential energy surface point for MM calculation ***
C
      ELSE IF (MMONLY) THEN
C
        CALL MMDRV
        EPES = EMM
C
C     *** Potential energy surface point for QM/MM calculation ***
C
      ELSE IF (QMMM) THEN
C
        CALL SCFDRV
        CALL EQMMM
        CALL MMDRV
        EPES = ESCF + EQMMMLJ + EMM
        IF ((STATUS.EQ.'SCF').OR.
     $     ((STATUS.EQ.'OPT').AND.OPTIMIZED).OR.
     $     ((STATUS.EQ.'BOMD').AND.OPTIMIZED)) THEN
          WRITE (OUT,5000) EPES
 5000     FORMAT (T2,'TOTAL ENERGY                = ',F20.9,/)
        END IF
        IF (SCF.AND.(STATUS.EQ.'SCF')) THEN
          CALL TIMER('PRINT STEP','SCF ITERATION')
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE PESDRV ***
C
      END
