      SUBROUTINE PLOTBOX
C
C     Purpose: Automatic generation of a plot box.
C
C     History: - Creation (07.07.02, GM)
C                         (27.01.03, AMK)
C
C     ******************************************************************
C
C     List of local variables:
C
C     EUCLID  : Plot lattice dimensionality.
C     LATTICE : PLot lattice basis vectors.
C     NLATTICE: Number of interval steps of lattice basis vector.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
CCC ADLL
      INCLUDE 'flags.h'
CCC ADLL
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'plot.h'
C
      REAL EPS
      PARAMETER (EPS = 1.0)
C
      INTEGER IATOM
      REAL XA,XB,XMAX,XMIN,YA,YB,YMAX,YMIN,ZA,ZB,ZMAX,ZMIN
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      EUCLID = 3
      LATTICE = 0.0
C
      XMAX = -EPS
      YMAX = -EPS
      ZMAX = -EPS
      XMIN = EPS
      YMIN = EPS
      ZMIN = EPS
C
C     *** Get standard orientation coordinates ***
C
      IF (NATOM.GT.1) THEN
        CALL SYMINI
        CALL MOLSYM
      END IF
C
C     *** Find plot box in standard orientation ***
C
      DO IATOM=1,NATOM
        IF (BOXATOM(IATOM)) THEN
          XA = COORD(1,IATOM,2) + BOXSIZE*MAX(1.0,COVRAD(ATOMNO(IATOM)))
          XB = COORD(1,IATOM,2) - BOXSIZE*MAX(1.0,COVRAD(ATOMNO(IATOM)))
          YA = COORD(2,IATOM,2) + BOXSIZE*MAX(1.0,COVRAD(ATOMNO(IATOM)))
          YB = COORD(2,IATOM,2) - BOXSIZE*MAX(1.0,COVRAD(ATOMNO(IATOM)))
          ZA = COORD(3,IATOM,2) + BOXSIZE*MAX(1.0,COVRAD(ATOMNO(IATOM)))
          ZB = COORD(3,IATOM,2) - BOXSIZE*MAX(1.0,COVRAD(ATOMNO(IATOM)))
          IF (XMAX.LT.XA) XMAX = XA
          IF (XMIN.GT.XB) XMIN = XB
          IF (YMAX.LT.YA) YMAX = YA
          IF (YMIN.GT.YB) YMIN = YB
          IF (ZMAX.LT.ZA) ZMAX = ZA
          IF (ZMIN.GT.ZB) ZMIN = ZB
        END IF
      END DO
C
C     *** Plot box origin ***
C
      LATTICE(1,0) = XMIN
      LATTICE(2,0) = YMIN
      LATTICE(3,0) = ZMIN
C
C     *** Plot box vectors ***
C
      NLATTICE(1) = INT((XMAX - XMIN)/BOXMESH) + 1
      LATTICE(1,1) = XMAX
      LATTICE(2,1) = YMIN
      LATTICE(3,1) = ZMIN
C
      NLATTICE(2) = INT((YMAX - YMIN)/BOXMESH) + 1
      LATTICE(1,2) = XMIN
      LATTICE(2,2) = YMAX
      LATTICE(3,2) = ZMIN
C
      NLATTICE(3) = INT((ZMAX - ZMIN)/BOXMESH) + 1
      LATTICE(1,3) = XMIN
      LATTICE(2,3) = YMIN
      LATTICE(3,3) = ZMAX
C
      NLATTICE(0) = NLATTICE(1)*NLATTICE(2)*NLATTICE(3)
C
C     *** Re-order plot box vectors ***
C
CCC ADLL
      IF (PLOT.NE.'CUBE') CALL BOXSORT
CCC ADLL
C
C     *** Transform plot box in input orientation ***
C
      CALL TRABOX('INPUT')
C
C     *** Check plot box input ***
C
      CALL CHKBOX
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE PLOTBOX ***
C
      END
