      SUBROUTINE PLOTSCF
C
C     Purpose: Driver routine for SCF plot function calculation.
C
C     History: - Creation (29.07.96, ML)
C                         (02.08.06, GG)
C                         (07.12.12, AMK)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     CA: Alpha MO coefficient matrix.
C     CB: Beta MO coefficient matrix.
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C
C     External functions:
C
C     WELF   : Calculate electron localization function (Wrapper).
C     WEFIDS : Calculate EFI x dS (Wrapper).
C     WLAPRHO: Calculate molecular density Laplacian (Wrapper).
C     WSGGA  : Calculate dimensionless density gradient (Wrapper).
C
C     WEFG   : Calculate molecular electric field gradient (Wrapper).
C     WEFI   : Calculate molecular electric field (Wrapper).
C     WESP   : Calculate molecular electrostatic potential (Wrapper).
C     WD1ESP : Calculate electrostatic potential gradient (Wrapper).
C     WD2ESP : Calculate potential 2nd derivative (Wrapper).
C
C     WPSI   : Calculate molecular orbitals (Wrapper).
C     WD1PSI : Calculate molecular orbital gradients (Wrapper).
C     WD2PSI : Calculate molecular orbital 2nd derivative (Wrapper).
C     WRHO   : Calculate molecular density (Wrapper).
C     WD1RHO : Calculate molecular density gradient (Wrapper).
C     WD2RHO : Calculate molecular density 2nd derivative (Wrapper).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'plot.h'
      INCLUDE 'scfvec.h'
CCC ADLL
      INCLUDE 'symmetry.h'
CCC ADLL
C
      REAL CDC(MAXAUX)
C
      INTEGER ALLOCATION
CCC ADLL
      INTEGER CCSET_SAVE
CCC ADLL
      REAL,ALLOCATABLE :: CA(:,:),CB(:,:),PA(:,:),PB(:,:)
CCC ADLL
      REAL,ALLOCATABLE :: WORK(:,:)
CCC ADLL
C
      EXTERNAL WELF,WEFIDS,WCAP,WLAPRHO,WSGGA
      EXTERNAL WEFG,WEFI,WESP,WD1ESP,WD2ESP
      EXTERNAL WPSI,WD1PSI,WD2PSI,WRHO,WD1RHO,WD2RHO
C
C     ------------------------------------------------------------------
C
C     *** Calculate dipole matrices for MO localization ***
C
      IF (USELMO) CALL DIPOLES
C
C     *** Normalization of fitting coefficients ***
C
      IF (USEDD) CALL NORMCDC(CDCA,CDCB,NAUX)
C
C     *** Quick return for MPI slave nodes ***
C
      IF (MPSLAVE) RETURN
C
C     *** Select MOs for plotting ***
C
CCC ADLL NUMBER OF ARG IS CHANGED
      CALL GETOMOS(EIGVALA,HFOMOA,NPLOTFUN,PLOTMOA,PLOTMOSA)
      IF (OKS) CALL GETOMOS(EIGVALB,HFOMOB,NPLOTFUN,PLOTMOB,PLOTMOSB)
CCC ADLL
C
      CALL CHKPLOT('ORBITALS')
C
C     *** Set plot function names ***
C
      CALL PIEDRV('PIENAME')
C
C     *** Allocate local fields ***
C
      ALLOCATE(CA(NSTO,NSTO),CB(NSTO,NSTO),STAT=ALLOCATION)
      ALLOCATE(PA(NSTO,NSTO),PB(NSTO,NSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PLOTSCF','ALLOCATION FAILED',1)
      END IF
C
C     *** Build alpha plot MOs and density matrix ***
C
      CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $           MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
      IF (USELMO.AND.FBLMO) THEN
        CALL LOCALIZE(CA,PA,PB,NSTO,'FB','ALL','LOOSE','ALPHA')
      ELSE IF (USELMO.AND.PMLMO) THEN
        CALL LOCALIZE(CA,PA,PB,NSTO,'PM','ALL','LOOSE','ALPHA')
      ELSE IF (USELMO) THEN
        CALL LOCALIZE(CA,PA,PB,NSTO,'CH','ALL','LOOSE','ALPHA')
      END IF
C
CCC ADLL 
      CCSET_SAVE = CCSET
      IF ((CCSET.NE.2).AND.(PLOT.EQ.'CUBE')) THEN
        CCSET = 2
        COORD(1:3,1:NATOM,5) = COORD(1:3,1:NATOM,CCSET)
        CALL PLOTBOX
        CALL TRABOX('SYMMETRY')
C
        ALLOCATE (WORK(NSTO,NSTO),STAT=ALLOCATION)
        CALL MODMOS(CA,WORK,INPTOSTD,NSTO,'ROTATE')
        CA(:,:) = WORK(:,:)
        DEALLOCATE (WORK)
      END IF
CCC ADLL
C
      CALL BLDPLOT(CA,PA,NSTO,OCCNA,HFOMOA,PLOTMOSA)
C
C     *** Build beta plot MOs and density matrix ***
C
      IF (OKS) THEN
        IF (ROKS) THEN
          CB(:,:) = CA(:,:)
        ELSE
          CALL RSTIO(CB,EIGVALB,PB,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $               MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
          IF (USELMO.AND.FBLMO) THEN
            CALL LOCALIZE(CB,PB,CA,NSTO,'FB','ALL','LOOSE','BETA')
          ELSE IF (USELMO.AND.PMLMO) THEN
            CALL LOCALIZE(CB,PB,CA,NSTO,'PM','ALL','LOOSE','BETA')
          ELSE IF (USELMO) THEN
            CALL LOCALIZE(CB,PB,CA,NSTO,'CH','ALL','LOOSE','BETA')
          END IF
        END IF
CCC ADLL
        IF (PLOT.EQ.'CUBE') THEN
          ALLOCATE (WORK(NSTO,NSTO),STAT=ALLOCATION)
          CALL MODMOS(CB,WORK,INPTOSTD,NSTO,'ROTATE')
          CB(:,:) = WORK(:,:)
          DEALLOCATE (WORK)
        END IF
CCC ADLL
        CALL BLDPLOT(CB,PB,NSTO,OCCNB,HFOMOB,PLOTMOSB)
      END IF
C
ccc ADLL This part is in development or in tests
      IF (USELMO) THEN
        CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $             MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'WRITE','ALPHA')
        IF (OKS) THEN
            CALL RSTIO(CB,EIGVALB,PB,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $                 MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'WRITE','BETA')
        END IF

      END IF
C
C     *** Build deformation density, if requested ***
C
      IF (USEDD) CALL BLDDRHO(PA,PB,NSTO)
C
C     ------------------------------------------------------------------
C
C     *** Orbital plot ***
C
      IF (PLOTFNC.EQ.'PSI') THEN
C
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WPSI)
        ELSE IF (PLOTDER.EQ.1) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD1PSI)
        ELSE IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD2PSI)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID ORBITAL DERIVATIVE',1)
        END IF
C
C     *** Density plot ***
C
      ELSE IF (PLOTFNC.EQ.'RHO') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) CDC(:) = CDCA(:) + CDCB(:)
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WRHO)
        ELSE IF (PLOTDER.EQ.1) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD1RHO)
        ELSE IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD2RHO)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID DENSITY DERIVATIVE',1)
        END IF
C
C     *** Density laplacian plot ***
C
      ELSE IF (PLOTFNC.EQ.'LAP') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) CDC(:) = CDCA(:) + CDCB(:)
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WLAPRHO)
        ELSE IF (PLOTDER.EQ.1) THEN
          CALL ERRMSG('PLOTSCF','INVALID LAPLACIAN DERIVATIVE',1)
        END IF
C
C     *** Spin density plot ***
C
      ELSE IF (PLOTFNC.EQ.'SPIN') THEN
C
        PA(:,:) = PA(:,:) - PB(:,:)
        CDC(:) = CDCA(:) - CDCB(:)
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WRHO)
        ELSE IF (PLOTDER.EQ.1) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD1RHO)
        ELSE IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD2RHO)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID SPIN DENSITY DERIVATIVE',1)
        END IF
C
C     *** Molecular electrostatic potential plot ***
C
      ELSE IF (PLOTFNC.EQ.'ESP') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) THEN
          CDC(:) = CDCA(:) + CDCB(:)
          CALL BLDCDC(CDC,'NORMALIZE')
        ELSE
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'TOTAL',2.0)
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'M = N',0.5)
        END IF 
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WESP)
        ELSE IF (PLOTDER.EQ.1) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD1ESP)
        ELSE IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WD2ESP)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID ESP DERIVATIVE',1)
        END IF
C
C     *** Molecular electric field ***
C
      ELSE IF (PLOTFNC.EQ.'EFI') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) THEN
          CDC(:) = CDCA(:) + CDCB(:)
          CALL BLDCDC(CDC,'NORMALIZE')
        ELSE
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'TOTAL',2.0)
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'M = N',0.5)
        END IF
        IF (PLOTDER.EQ.1) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WEFI)
        ELSE IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WEFG)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID ELECTRIC FIELD DERIVATIVE',1)
        END IF
C
C     *** Molecular electric field gradient ***
C
      ELSE IF (PLOTFNC.EQ.'EFG') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) THEN
          CDC(:) = CDCA(:) + CDCB(:)
          CALL BLDCDC(CDC,'NORMALIZE')
        ELSE
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'TOTAL',2.0)
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'M = N',0.5)
        END IF
        IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WEFG)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID EFG DERIVATIVE',1)
        END IF
C
C     *** Molecular anion shape (nabla ESP x dS) ***
C
      ELSE IF (PLOTFNC.EQ.'EFIDS') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) THEN
          CDC(:) = CDCA(:) + CDCB(:)
          CALL BLDCDC(CDC,'NORMALIZE')
        ELSE
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'TOTAL',2.0)
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'M = N',0.5)
        END IF
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WEFIDS)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID EFIDS DERIVATIVE',1)
        END IF
C
C     *** Dimensionless density gradient ***
C
      ELSE IF (PLOTFNC.EQ.'SGGA') THEN
C
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WSGGA)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID SGGA DERIVATIVE',1)
        END IF
C
C     *** Electron Localisation Function ***
C
      ELSE IF (PLOTFNC.EQ.'ELF') THEN
C
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WELF)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID ELF DERIVATIVE',1)
        END IF
C
CCC ADLL 
C     *** Complex absorbing potential ***
C
      ELSE IF (PLOTFNC.EQ.'ABS') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(CA,CB,PA,PB,CDC,NSTO,WCAP)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID ABS DERIVATIVE',1)
        END IF
CCC ADLL
C
C     *** No more properties ***
C
      ELSE
C
        CALL ERRMSG('PLOTSCF','INVALID PLOT FUNCTION',1)
C
      END IF
C
CCC ADLL
C
C     *** Restore Cartesian Coordinates SET ***
C
      CCSET = CCSET_SAVE
C
CCC ADLL
C     *** Deallocate local fields ***
C
      DEALLOCATE(CA,CB,PA,PB,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PLOTSCF','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE PLOTSCF ***
C
      END
