      SUBROUTINE PLOTSCP
C
C     Purpose: Driver routine for SCP plot function calculation.
C
C     History: - Creation (25.07.08, AMK)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     P0: Unperturbed density matrix.
C     PX: Perturbed (x field or Fukui^-) density matrix.
C     PY: Perturbed (y field or Fukui^+) density matrix.
C     PZ: Perturbed (z field or Fukui sum) density matrix.
C
C     External functions:
C
C     WFUKUI: Calculate Fukui functions (Wrapper).
C     WRHO  : Calculate molecular density (Wrapper).
C     WPRHO : Calculate perturbed molecular density (Wrapper).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'plot.h'
      INCLUDE 'scfvec.h'
C
      REAL CDC(MAXAUX)
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: P0(:,:),PX(:,:),PY(:,:),PZ(:,:)
C
      EXTERNAL WFUKUI,WPRHO,WRHO
C
C     ------------------------------------------------------------------
C
C     *** Quick return for MPI slave nodes ***
C
      IF (MPSLAVE) RETURN
C
C     *** Select MOs for plotting ***
C
CCC ADLL THE NUMBER OF ARG CHANGED 
      CALL GETOMOS(EIGVALA,HFOMOA,NPLOTFUN,PLOTMOA,PLOTMOSA)
      IF (OKS) CALL GETOMOS(EIGVALB,HFOMOB,NPLOTFUN,PLOTMOB,PLOTMOSB)
CCC ADLL
C
      CALL CHKPLOT('ORBITALS')
C
C     *** Set plot function names ***
C
      CALL PIEDRV('PIENAME')
C
C     *** Allocate local fields ***
C
      ALLOCATE(P0(NSTO,NSTO),PX(NSTO,NSTO),PY(NSTO,NSTO),
     $         PZ(NSTO,NSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PLOTSCP','ALLOCATION FAILED',1)
      END IF
C
C     *** Build unperturbed alpha density matrix ***
C
      CALL RSTIO(PX,EIGVALA,P0,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $           MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
      CALL BLDPLOT(PX,P0,NSTO,OCCNA,HFOMOA,PLOTMOSA)
C
C     *** Build unperturbed beta density matrix ***
C
      IF ((OKS).AND.(.NOT.ROKS)) THEN
        CALL RSTIO(PX,EIGVALB,PY,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $             MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
      END IF
      CALL BLDPLOT(PX,PY,NSTO,OCCNB,HFOMOB,PLOTMOSB)
C
C     *** Add unperturbed alpha and beta density matrix ***
C
      P0(:,:) = P0(:,:) + PY(:,:)
C
C     *** Load Fukui function matrices ***
C
      IF (PLOTFNC.EQ.'FUKUI') THEN
C
        CALL FFPIO(PX,NSTO,'READ','F-')
        CALL FFPIO(PY,NSTO,'READ','F+')
        CALL FFPIO(PZ,NSTO,'READ','F#')
C
C     *** Load perturbed density matrices ***
C
      ELSE
C
        CALL FFPIO(PX,NSTO,'READ','PX')
        CALL FFPIO(PY,NSTO,'READ','PY')
        CALL FFPIO(PZ,NSTO,'READ','PZ')
C
      END IF
C
C     *** Perturbed density plot ***
C
      IF (PLOTFNC.EQ.'FUKUI') THEN
        CALL PLOTTER(P0,PX,PY,PZ,CDC,NSTO,WFUKUI)
      ELSE IF (PLOTFNC.EQ.'PRHO') THEN
        CALL PLOTTER(P0,PX,PY,PZ,CDC,NSTO,WPRHO)
      ELSE IF (PLOTFNC.EQ.'PRHOX') THEN
        CALL PLOTTER(PX,PX,PX,PX,CDC,NSTO,WRHO)
      ELSE IF (PLOTFNC.EQ.'PRHOY') THEN
        CALL PLOTTER(PY,PY,PY,PY,CDC,NSTO,WRHO)
      ELSE IF (PLOTFNC.EQ.'PRHOZ') THEN
        CALL PLOTTER(PZ,PZ,PZ,PZ,CDC,NSTO,WRHO)
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(P0,PX,PY,PZ,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PLOTSCP','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE PLOTSCP ***
C
      END
