      SUBROUTINE POPINI
C
C     Purpose: Initialize population analysis by numerical integration
C              on a grid.
C
C     History: - Creation (30.08.12, AMK)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DIRTOP: If true, a direct population integration is requested.
C     G98TOP: Flag for G98 population analysis fixed grid type.
C     REFTOP: If true, reference setting for population integration.
C     
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'numint.h'
      INCLUDE 'property.h'
C
C     ------------------------------------------------------------------
C
C     *** Initialize population analysis grid ***
C
      RETGRD = RETGRD_GST(TOPGSN)
      CALL GRDDEF(TOPGSN,'LOAD-TYPE')
      CALL GRDTHRS
C
C     *** Initialize atomic properties ***
C
      NBASINA(:) = 0.0
      NBASINB(:) = 0.0
      MBASIN(:,:) = 0.0
C
C     *** Assign reference fitting coefficients, if needed ***
C
      IF (MPP) THEN
        CALL MP_SEND_ALL(CDCA,NAUX,MASTER,MP_REAL)
        CALL MP_SEND_ALL(CDCB,NAUX,MASTER,MP_REAL)
        CALL MP_SEND_ALL(REFCA,NAUX,MASTER,MP_REAL)
        CALL MP_SEND_ALL(REFCB,NAUX,MASTER,MP_REAL)
      END IF
C
      IF (HPA.AND.(.NOT.ITHPA).AND.(.NOT.SPHPA)) THEN
        CALL NORMCDC(CDCA,CDCB,NAUX)
        REFCA(:) = CDCA(:)
        REFCB(:) = CDCB(:)
      END IF
C
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE POPINI ***
C
      END
