      SUBROUTINE RSTIO(C,EIGVAL,P,DSTO,DSTO_ME,OCCN,AOCCN,ELCFG,
     $                 MOEXC,NMOEXC,LFOMO,HFOMO,OPTION,SPIN)
C
C     Purpose: Driver routine for ReSTart file Input/Output.
C
C     History: - Creation (02.10.97, MK)
C                         (30.05.01, AMK)
C                         (19.05.04, GG)
C                         (11.08.17, AC)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO   : Dimension of C and P in the calling subroutine.
C     DSTO_ME: Dimension of P on node ME.
C
C     List of local variables:
C
C     AOCCN : Fixed AO occupation numbers.
C     C     : Alpha/Beta MO coefficients.
C     EIGVAL: Vector with the MO eigenvalues.
C     HFOMO : Highest Fractional Occupied MO.
C     LFOMO : Lowest Fractional Occupied MO.
C     MOEXC : Vector with the numbers of the MOs to be exchanged.
C     NDE2  : Number of Hessian variables.
C     NMOEXC: Number of MO exchanges.
C     NORTH : Number of orthonormal atomic basis functions.
C     P     : Alpha/Beta density matrix.
C     RWSTO : Reading or writing STO dimension.
C     SPIN  : Keyword for alpha or beta spin.
C     OCCN  : Vector with the MO occupation numbers.
C     OPTION: a) OPTION = READ, read MO data from restart file.
C             b) OPTION = RESTART, read SCF data from restart file.
C             c) OPTION = WRITE, write MO data to restart file.
C             d) OPTION = READ H, read Hessian from restart file.
C             e) OPTION = WRITE H, write Hessian to restart file.
C
C     List of local dynamical fields:
C
C     WORK: Work matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'freq.h'
      INCLUDE 'iteration.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'opt.h'
      INCLUDE 'pointer.h'
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'symmetry.h'
C
      INTEGER DSTO,DSTO_ME
      REAL C(DSTO,DSTO),P(DSTO_ME,DSTO_ME)
C
      CHARACTER*(*) OPTION,SPIN
      INTEGER HFOMO,LFOMO,NDE2,NMOEXC
C
      INTEGER ELCFG(0:*),MOEXC(2,*)
      REAL AOCCN(0:(MAXLBAS+1)**2,*),EIGVAL(MAXSTO),OCCN(MAXSTO)
C
      CHARACTER*80 CSCKEY,FIXKEY,KEY,PRTSTR,STRCOMP
      INTEGER ISTO,JSTO,RWSTO
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: WORK(:,:)
C
C     ------------------------------------------------------------------
C
      KEY = 'C'//SPIN
      KEY = STRCOMP(KEY)
C
      FIXKEY = 'COLD'//SPIN
      FIXKEY = STRCOMP(FIXKEY)
C
      CSCKEY = 'CSC'//SPIN
      CSCKEY = STRCOMP(CSCKEY)
C
C     *** Rewind restart tape ***
C
      IF (SPIN.EQ.'ALPHA') REWIND (RST)
C
C     *** Load MO information ***
C
      IF (MPMASTER.AND.(OPTION.EQ.'READ')) THEN
C
        READ (RST,ERR=99960,END=99970) RWSTO
        READ (RST,ERR=99960,END=99970) ((C(JSTO,ISTO),JSTO=1,RWSTO),
     $                                                ISTO=1,RWSTO)
        READ (RST,ERR=99960,END=99970) (EIGVAL(ISTO),ISTO=1,RWSTO)
C
C     *** Write MO information ***
C
      ELSE IF (MPMASTER.AND.(OPTION.EQ.'WRITE')) THEN
C
        IF (ROKS.AND.(SPIN.EQ.'BETA')) RETURN
C
        IF (MOBLK) THEN
          CALL BLKSORT(C,EIGVAL,DSTO,'MATRIX',SPIN,'AUFBAU')
          CALL BLKSORT(C,EIGVAL,DSTO,'VECTOR',SPIN,'AUFBAU')
          CALL BLKSORT(C,OCCN,DSTO,'VECTOR',SPIN,'AUFBAU')
        END IF
C
        WRITE (RST,ERR=99980) DSTO
        WRITE (RST,ERR=99980) ((C(JSTO,ISTO),JSTO=1,DSTO),
     $                                       ISTO=1,DSTO)
        WRITE (RST,ERR=99980) (EIGVAL(ISTO),ISTO=1,DSTO)
C
C     *** Load SCF restart information ***
C
      ELSE IF (OPTION.EQ.'RESTART') THEN
C
        IF (ROKS.AND.(SPIN.EQ.'BETA')) THEN
C
C     *** Build the ROKS beta density matrix ***
C
          IF (MPMASTER) CALL BLDPMAT(C,P,DSTO,OCCN,LFOMO,HFOMO)
C
        ELSE
C
          IF (MPMASTER.AND.SPHORB) THEN
            READ (RST,ERR=99960,END=99970) RWSTO
            READ (RST,ERR=99960,END=99970) ((P(JSTO,ISTO),JSTO=1,RWSTO),
     $                                                    ISTO=1,RWSTO)
            IF (LOCO) CALL DOLOCO(P,C,DSTO,SPIN)
            CALL CCTOSC(P,C,DSTO)
          ELSE IF (MPMASTER) THEN
            READ (RST,ERR=99960,END=99970) RWSTO
            READ (RST,ERR=99960,END=99970) ((C(JSTO,ISTO),JSTO=1,RWSTO),
     $                                                    ISTO=1,RWSTO)
            IF (LOCO) CALL DOLOCO(C,P,DSTO,SPIN)
          END IF
C
C     *** Complete tight-binding MO coefficients ***
C
          CALL MP_SEND_ALL(HUECKEL,1,MASTER,MP_LOG)
C
          IF (HUECKEL) THEN
            CALL NULLSPACE(C,P,DSTO,DSTO_ME,NORB,NORTH,NORTH_TB)
          END IF
C
C     *** Build additional transformation matrix, if level ***
C     *** shifting, MO blocking and/or ROKS is requested   ***
C
          IF (LSHIFT.OR.MOBLK.OR.ROKS) THEN
            ALLOCATE(WORK(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
            CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
            IF (ALLOCATION.GT.0) THEN
              CALL ERRMSG('RSTIO','ALLOCATION FAILED',1)
            END IF
            IF (MPMASTER) THEN
              IF (MOBLK) THEN
                CALL BLKSORT(C,EIGVAL,DSTO,'MATRIX',SPIN,'BLOCKS')
              END IF
              CALL SCFIO(C,DSTO,'WRITE',KEY)
              IF (SPHORB) THEN
                CALL SCFIO(P,DSTO,'READ','OVERLAP')
                CALL CMTOSM(P,WORK,DSTO)
              ELSE
                CALL SCFIO(WORK,DSTO,'READ','OVERLAP')
              END IF
            END IF
            CALL MPMULMAT(C,WORK,P,DSTO_ME,NORB,'TRANSA')
            CALL MPMULMAT(P,C,WORK,DSTO_ME,NORB,'NORMAL')
            CALL SVDMAT(WORK,DSTO,DSTO_ME,NORTH,-0.5,1.E-14)
            IF (MPMASTER) CALL SCFIO(WORK,DSTO,'WRITE',CSCKEY)
            DEALLOCATE(WORK,STAT=ALLOCATION)
            CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
            IF (ALLOCATION.GT.0) THEN
              CALL ERRMSG('RSTIO','DEALLOCATION FAILED',1)
            END IF
          END IF
C
          IF (MPSLAVE) GO TO 100
C
C     *** Now read the old MO eigenvalues ***
C
          READ (RST,ERR=99960,END=99970) (EIGVAL(ISTO),ISTO=1,RWSTO)
          IF (MOBLK) CALL BLKSORT(C,EIGVAL,DSTO,'VECTOR',SPIN,'BLOCKS')
C
C     *** Generate the requested atomic configuration ***
C
          IF (ROKS) THEN
            CALL ATOMCONF(C,P,DSTO,EIGVALA,OCCNB,AOCCNB,ELCFGB,LFOMOB,
     $                    HFOMOB)
          END IF
C
          CALL ATOMCONF(C,P,DSTO,EIGVAL,OCCN,AOCCN,ELCFG,LFOMO,HFOMO)
CCC ADLL
C     *** Localize MOs ***
C
          IF (USERLMO) ALLOCATE(WORK(DSTO,DSTO))
          IF (USERLMO.AND.FBRMO) THEN
            CALL LOCALIZE(C,P,WORK,DSTO,'FB','ALL','LOOSE',SPIN)
          ELSE IF (USERLMO.AND.PMRMO) THEN
            CALL LOCALIZE(C,P,WORK,DSTO,'PM','ALL','LOOSE',SPIN)
          ELSE IF (USERLMO) THEN
            CALL LOCALIZE(C,P,WORK,DSTO,'CH','ALL','LOOSE',SPIN)
          END IF
          IF (USERLMO) DEALLOCATE(WORK)
C
CCC ADLL
C
C     *** Exchange molecular orbitals ***
C
          CALL EXCMOS(C,DSTO,EIGVAL,MOEXC,NMOEXC)
C
C     *** Smearing of molecular orbital occupation ***
C
          IF (ROKS.AND.HUECKEL) THEN
            CALL SMRDEN(EIGVALA,OCCNB,MOMODB,NMOMODB,NORTH_TB,NOMOB,
     $                  LFOMOB,HFOMOB)
          ELSE IF (ROKS) THEN
            CALL SMRDEN(EIGVALA,OCCNB,MOMODB,NMOMODB,NORTH,NOMOB,
     $                  LFOMOB,HFOMOB)
          ELSE IF ((SPIN.EQ.'ALPHA').AND.HUECKEL) THEN
            CALL SMRDEN(EIGVALA,OCCNA,MOMODA,NMOMODA,NORTH_TB,NOMOA,
     $                  LFOMOA,HFOMOA)
          ELSE IF ((SPIN.EQ.'BETA').AND.HUECKEL) THEN
            CALL SMRDEN(EIGVALB,OCCNB,MOMODB,NMOMODB,NORTH_TB,NOMOB,
     $                  LFOMOB,HFOMOB)
          ELSE IF (SPIN.EQ.'ALPHA') THEN
            CALL SMRDEN(EIGVALA,OCCNA,MOMODA,NMOMODA,NORTH,NOMOA,
     $                  LFOMOA,HFOMOA)
          ELSE IF (SPIN.EQ.'BETA') THEN
            CALL SMRDEN(EIGVALB,OCCNB,MOMODB,NMOMODB,NORTH,NOMOB,
     $                  LFOMOB,HFOMOB)
          ELSE
            CALL ERRMSG('RSTIO','RESTART MO SMEARING FAILED',1)
          END IF
C
C     *** Print the first set of MOs ***
C
          IF (PRTMOS.AND.(.NOT.OPTIMIZED)) THEN
            IF (LOCO) THEN
              PRTSTR = 'LOCALIZED MO COEFFICIENTS'
            ELSE IF (HUECKEL) THEN
              PRTSTR = 'TIGHT-BINDING MO COEFFICIENTS'
            ELSE 
              PRTSTR = 'RESTART MO COEFFICIENTS'
            END IF
            CALL PRISCF(C,DSTO,LLMOS,ULMOS,SPIN,STRCOMP(PRTSTR))
          ELSE IF (PRTMOE) THEN
            IF (HUECKEL) THEN
              PRTSTR = 'TIGHT-BINDING MO ENERGIES'
            ELSE
              PRTSTR = 'RESTART MO ENERGIES'
            END IF
            CALL PRISCF(C,DSTO,LLMOS,ULMOS,SPIN,STRCOMP(PRTSTR))
          END IF
C
C     *** Write modified configuration to tape ***
C
CCC ADLL
          IF ((FIXCFG.AND.(SCFCYC.GE.FIXCYC)).OR.RTDFT) THEN
C          IF (FIXCFG.AND.(SCFCYC.GE.FIXCYC)) THEN
CCC ADLL
            CALL SCFIO(C,DSTO,'WRITE',FIXKEY)
          END IF
C
C     *** Backtransformation of MOs to Cartesian representation ***
C
          IF (SPHORB) THEN
            CALL SCTOCC(C,P,DSTO)
            C(:,:) = P(:,:)
          END IF
C
C     *** Build new density matrix ***
C
          CALL BLDPMAT(C,P,DSTO,OCCN,LFOMO,HFOMO)
C
  100     CONTINUE
C
        END IF
C
        IF (MPSLAVE) GO TO 200
C
C     *** Print density matrix ***
C
        IF (PRTPMAT) THEN
          IF (HUECKEL) THEN
            PRTSTR = 'TIGHT-BINDING START DENSITY MATRIX'
          ELSE
            PRTSTR = 'RESTART DENSITY MATRIX'
          END IF
          CALL SCALMAT(P,DSTO,NSTO,NSTO,'UPPWD',0.5)
          CALL SYMMAT(P,DSTO,NSTO,'UPLOW')
          IF (SPHORB) THEN
            ALLOCATE(WORK(DSTO,DSTO))
            CALL CPTOSP(P,WORK,DSTO)
            CALL PRISCF(WORK,DSTO,1,NORB,SPIN,STRCOMP(PRTSTR))
            DEALLOCATE(WORK)
          ELSE
            CALL PRISCF(P,DSTO,1,NORB,SPIN,STRCOMP(PRTSTR))
          END IF
          CALL SCALMAT(P,DSTO,NSTO,NSTO,'UPPWD',2.0)
          CALL SYMMAT(P,DSTO,NSTO,'UPLOW')
        END IF
C
C     *** Set RESTART for SCF ***
C
        IF (HUECKEL) THEN
          IF ((.NOT.OKS).OR.(OKS.AND.(SPIN.EQ.'BETA'))) THEN
            HUECKEL = .FALSE.
            RESTART = .FALSE.
          END IF
        ELSE IF ((.NOT.OKS).OR.(OKS.AND.(SPIN.EQ.'BETA'))) THEN
          RESTART = .TRUE.
        END IF
C
C     *** Read restart Hessian (optimization) and store on tape ***
C
        IF ((HESSIAN.EQ.'RESTART').AND.
     $      ((.NOT.OKS).OR.(SPIN.EQ.'BETA'))) THEN
          ALLOCATE(WORK(NDEG,NDEG))
          WORK(:,:) = 0.0
          READ (RST,ERR=99960,END=99990) NVARRST,NDE2
          READ (RST,ERR=99960,END=99990) ((WORK(JSTO,ISTO),JSTO=1,NDE2),
     $                                                     ISTO=1,NDE2)
          CALL OPTIO(WORK,NDEG**2,'WRITE','HESSIAN')
          DEALLOCATE(WORK)
        END IF
C
  200   CONTINUE
C
        CALL MP_SEND_ALL(HUECKEL,1,MASTER,MP_LOG)
        CALL MP_SEND_ALL(RESTART,1,MASTER,MP_LOG)
C
C     *** Read Hessian from restart file ***
C
      ELSE IF (MPMASTER.AND.(OPTION.EQ.'READ H')) THEN
C
        C(:,:) = 0.0
C
C     *** Read electronic structure header of restart file ***
C
        REWIND (RST)
        READ (RST,ERR=99960,END=99970) NDE2
        READ (RST,ERR=99960,END=99970) C(1,1)
        READ (RST,ERR=99960,END=99970) C(1,1)
        IF (OKS.AND.(.NOT.ROKS)) THEN
          READ (RST,ERR=99960,END=99970) NDE2
          READ (RST,ERR=99960,END=99970) C(1,1)
          READ (RST,ERR=99960,END=99970) C(1,1)
        END IF
C
C     *** Read Hessian matrix from restart file ***
C
        READ (RST,ERR=99960,END=99990) NVARRST,NDE2
        READ (RST,ERR=99960,END=99990) ((C(JSTO,ISTO),JSTO=1,NDE2),
     $                                                ISTO=1,NDE2)
C
C     *** Write Hessian to restart file ***
C
      ELSE IF (MPMASTER.AND.(OPTION.EQ.'WRITE H')) THEN
C
        WRITE (RST,ERR=99980) NVARRST,DSTO
        WRITE (RST,ERR=99980) ((C(JSTO,ISTO),JSTO=1,DSTO),
     $                                       ISTO=1,DSTO)
C
C     *** Print error message for an unknown option key ***
C
      ELSE IF (MPMASTER) THEN
C
        PRTSTR = 'UNKNOWN OPTION '//OPTION//' WAS SPECIFIED'
        CALL ERRMSG('RSTIO',STRCOMP(PRTSTR),1)
C
      END IF
C
C     *** Flush restart file ***
C
      CALL DEFLUSH(RST)
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of restart read and write errors ***
C
99960 CONTINUE
      CALL ERRMSG('RSTIO','ERROR READING FROM RESTART FILE',1)
C
99970 CONTINUE
      CALL ERRMSG('RSTIO','UNEXPECTED END OF RESTART FILE',1)
C
99980 CONTINUE
      CALL ERRMSG('RSTIO','ERROR WRITING TO RESTART FILE',1)
C
99990 CONTINUE
      CALL ERRMSG('RSTIO','HESSIAN NOT ON RESTART FILE',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE RSTIO ***
C
      END
