      SUBROUTINE SCFIO(MATRIX,MATDIM,OPTION,MATNAME)
C
C     Purpose: Driver routine for SCF Input/Output.
C
C     History: - Creation (09.10.96, AMK)
C
C     ******************************************************************
C
C     List of local variables:
C
C     MATDIM: Matrix (quadratic) dimension.
C     MATRIX: Storage vector for full matrix.
C     OPTION: a) OPTION = OPEN, open SCF tape.
C             b) OPTION = WRITE, write to SCF tape.
C             c) OPTION = READ, read from SCF tape.
C             d) OPTION = CLOSE, close SCF tape.
C             e) OPTION = DELETE, delete SCF tape.
C
C     MATNAME: a) KSALPHA   = Alpha Kohn-Sham matrix.
C              b) PALPHA    = Alpha density matrix.
C              c) KSBETA    = Beta Kohn-Sham matrix.
C              d) PBETA     = Beta density matrix.
C
C              a) OVERLAP   = Overlap matrix.
C              b) CORE      = Core Hamiltonian.
C              c) ORTHO     = Symmetric orthogonalization matrix.
C              d) CALPHA    = Alpha MO coefficients.
C              e) CBETA     = Beta MO coefficients.
C              f) CNOS      = Natural Orbital coefficients.
C              g) COLDALPHA = Alpha MO configuration.
C              h) COLDBETA  = Beta MO configuration.
C              i) CSCALPHA  = (C(alpha)*S*C(alpha))**(-1/2).
C                             C(alpha) represents the old MO vectors
C                             from the restart file and S is the current
C                             overlap matrix.
C              j) CSCBETA   = (C(beta)*S*C(beta))**(-1/2).
C              k) ROALPHA   = RO alpha Kohn-Sham matrix.
C              l) ROBETA    = RO beta Kohn-Sham matrix.
C              o) MOORTHO   = MO orthogonalization matrix.
CCC ADLL
C              p) PCHGMM    = Embedding Hamiltonian.
C              q) KOLDALPR  = Previous Alpha KS Hamiltonian in RT-TDDFT.
C              r) KOLDBETA  = Previous BETA KS Hamiltonian in RT-TDDFT.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
C
      INTEGER MATDIM
      REAL MATRIX(MATDIM**2)
C
      CHARACTER*(*) MATNAME,OPTION
C
      CHARACTER*80 PRTSTR,STRCOMP
      INTEGER IREC,IVEC,MOFF,ROFF,TAPE
C
C     ------------------------------------------------------------------
C
C     *** Direct SCF scratch I/O ***
C
      IF (MATNAME.EQ.'KSALPHA') THEN
        TAPE = IODIR
        RECNO(TAPE) = 1
      ELSE IF (MATNAME.EQ.'PALPHA') THEN
        TAPE = IODIR
        RECNO(TAPE) = 2
      ELSE IF (MATNAME.EQ.'KSBETA') THEN
        TAPE = IODIR
        RECNO(TAPE) = 3
      ELSE IF (MATNAME.EQ.'PBETA') THEN
        TAPE = IODIR
        RECNO(TAPE) = 4
C
C     *** SCF scratch I/O ***
C
      ELSE IF (MATNAME.EQ.'OVERLAP') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 1
      ELSE IF (MATNAME.EQ.'CORE') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 2
      ELSE IF (MATNAME.EQ.'ORTHO') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 3
      ELSE IF (MATNAME.EQ.'CALPHA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 4
      ELSE IF (MATNAME.EQ.'CBETA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 5
      ELSE IF (MATNAME.EQ.'CNOS') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 6
      ELSE IF (MATNAME.EQ.'COLDALPHA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 7
      ELSE IF (MATNAME.EQ.'COLDBETA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 8
      ELSE IF (MATNAME.EQ.'CSCALPHA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 9
      ELSE IF (MATNAME.EQ.'CSCBETA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 10
      ELSE IF (MATNAME.EQ.'ROALPHA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 11
      ELSE IF (MATNAME.EQ.'ROBETA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 12
      ELSE IF (MATNAME.EQ.'MOORTHO') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 13
CCC ADLL
      ELSE IF (MATNAME.EQ.'PCHGMM') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 14
      ELSE IF (MATNAME.EQ.'KOLDALPR') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 15
      ELSE IF (MATNAME.EQ.'KOLDALPI') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 16
      ELSE IF (MATNAME.EQ.'KOLDBETR') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 17
      ELSE IF (MATNAME.EQ.'KOLDBETI') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 18
      ELSE IF (MATNAME.EQ.'CAPA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 19 
      ELSE IF (MATNAME.EQ.'CAPB') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 20
      ELSE IF (MATNAME.EQ.'RADIAF') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 21
      ELSE IF (MATNAME.EQ.'P0ALPHA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 22
      ELSE IF (MATNAME.EQ.'P0BETA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 23
       ELSE IF (MATNAME.EQ.'MOALPHA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 24
      ELSE IF (MATNAME.EQ.'MOBETA') THEN
        TAPE = IOSCF
        RECNO(TAPE) = 25
CCC ADLL
C
C     *** Error handling ***
C
      ELSE
C
        PRTSTR = 'UNKNOWN MATRIX NAME '//MATNAME//' WAS SPECIFIED'
        CALL ERRMSG('SCFIO',STRCOMP(PRTSTR),1)
C
      END IF
C
C     *** Direct access scratch tape operations ***
C
      IF ((MMONLY).AND.(OPTION.EQ.'OPEN')) THEN
        CALL SCRDIR(MATRIX,MATDIM,MAX(MATDIM,1),RECNO(TAPE),TAPE,OPTION)
      ELSE IF ((OPTION.EQ.'OPEN').OR.(OPTION.EQ.'CLOSE').OR.
     $         (OPTION.EQ.'DELETE')) THEN
        IREC = (RECNO(TAPE) - 1)*MATDIM + 1
        CALL SCRDIR(MATRIX,MATDIM,MATDIM,IREC,TAPE,OPTION)
      ELSE
        ROFF = (RECNO(TAPE) - 1)*MATDIM + 1
        DO IVEC=0,MATDIM-1
          IREC = ROFF + IVEC
          MOFF = IVEC*MATDIM + 1
          CALL SCRDIR(MATRIX(MOFF),MATDIM,MATDIM,IREC,TAPE,OPTION)
        END DO
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE SCFIO ***
C
      END
