      SUBROUTINE BLDCORE
C
C     Purpose: Build core Hamiltonian and overlap and Dunlap matrix. 
C
C     History: - Creation (17.05.06, GG)
C                         (01.12.14, AMK)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     ECPS  : If true, add effective core potentials.
C     MCPS  : If true, add model core potentials.
C     EMBED : If true, add point charge embedding potential.
C     EFIELD: If true, add electric field embedding potential.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'cdft.h'
C
      LOGICAL STATED
C
C     ------------------------------------------------------------------
C
C     *** Synchronize parallel processes ***
C
      CALL MP_SYNC
C
C     *** Start timer for core integrals ***
C
      IF (.NOT.STATED('ATOM')) CALL TIMER('START','CORE INTEGRALS')
C
C     *** Calculation of potential expansion cut-off radius ***
C
      IF (.NOT.STATED('ATOM')) CALL CUTRAD((EMBED.OR.QMMM),MULTIPOLE)
C
C     *** Calculation of orthogonalization matrix ***
C
      CALL LOEWDIN
C
C     *** Calculation of auxiliary function fit ***
C
      CALL DUNLAP
C
C     *** Calculation of core Hamiltonian ***
C
      CALL KINETIC
      CALL CORE
C
C     *** Calculation of ECP core Hamiltonian contribution ***
C
      IF (ECPS) CALL ECPCORE
C
C     *** Calculation of MCP core Hamiltonian contribution ***
C
      IF (MCPS) CALL MCPCORE
C
C     *** QMMM core Hamiltonian ***
C
      IF (QMMM) CALL EMBEDMM
C
C     *** Charge embedding of core Hamiltonian ***
C
      IF (EMBED) CALL EMBEDDING
C
C     *** Electric field embedding of core Hamiltonian ***
C
      IF ((EFIELD.AND.EFSCF).AND.(.NOT.STATED('ATOM'))) CALL EMBEDFIELD 
C      IF (EFIELD) CALL EMBEDFIELD
C
C     *** Calculation of constrained DFT weight matrices ***
C
      IF (USECDFT) CALL NUMCDFT
C
C     *** Stop timer for core integrals ***
C
      IF (.NOT.STATED('ATOM')) CALL TIMER('STOP','CORE INTEGRALS')
      IF (STATUS.EQ.'SCF') CALL TIMER('PRINT STEP','CORE INTEGRALS')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE BLDCORE ***
C
      END
