      SUBROUTINE HEADER
C
C     Purpose: Write output header.
C
C     History: - Creation (18.01.96, AMK)
C                         (25.08.16, BZG)
C                         (19.06.18, MA)
C                         (31.10.19, GG)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'align.h'
      INCLUDE 'cdft.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'embed.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'freq.h'
      INCLUDE 'grid.h'
      INCLUDE 'iteration.h'
      INCLUDE 'math.h'
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
      INCLUDE 'opt.h'
      INCLUDE 'physic.h'
      INCLUDE 'plot.h'
      INCLUDE 'polarizability.h'
CCC ADLL
      INCLUDE 'projectile.h'
CCC ADLL
      INCLUDE 'property.h'
      INCLUDE 'ptmd.h'
      INCLUDE 'qmmm.h'
CCC ADLL
      INCLUDE 'rtdft.h'
CCC ADLL
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'strings.h'
      INCLUDE 'symmetry.h'
      INCLUDE 'thermo.h'
      INCLUDE 'threshold.h'
      INCLUDE 'tolerance.h'
      INCLUDE 'xc.h'
C
      CHARACTER*120 PRTSTR,STRCOMP,STRING*25
      INTEGER I,IMOEXC,IMOFIX,IMOMOD,J,L,M,STREXT,TITLEN
CCC ADLL
      REAL BETA,CLAU,MC2,RADIUS,WOMEGA
CCC ADLL
C
      REAL VECTOR(3)
C
C     ------------------------------------------------------------------
C
C     *** Write Title ***
C
      WRITE (OUT,5000)
 5000 FORMAT (/,T2,78('*'),/)
C
      TITLEN = STREXT(TITLE)
C
      IF (TITLEN.GT.0) THEN
        WRITE (OUT,'(T2,A)') TITLE(:TITLEN)
      ELSE
        WRITE (OUT,*)
      END IF
C
      WRITE (OUT,5000)
C
      IF (RESTART) CALL MESSAGE(1,'THE RESTART FILE WILL BE READ',1,1)
C
C     ------------------------------------------------------------------
C
C     *** Write molecular simulation data ***
C
      CALL MESSAGE(1,'MOLECULAR SIMULATION DATA',1,1)
C
      WRITE (PRTSTR,5010) CHARGE
 5010 FORMAT ('MOLECULAR CHARGE: ',I5)
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      WRITE (PRTSTR,5020) NATOM+NATMM
 5020 FORMAT ('NUMBER OF ATOMS: ',I10)
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (QMMM) THEN
        STRING = 'QM/MM'
      ELSE IF (QMONLY) THEN
        STRING = 'QM ONLY'
      ELSE IF (MMONLY) THEN
        STRING = 'MM ONLY'
      ELSE
        STRING = 'UNKNOWN'
      END IF
C
      WRITE (PRTSTR,5030) STRING
 5030 FORMAT ('MOLECULAR SIMULATION TYPE: ',A)
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
C     ------------------------------------------------------------------
C
C     *** Write molecular QM data ***
C
      IF (QMMM.OR.QMONLY) THEN
C
        CALL MESSAGE(1,'MOLECULAR QM DATA',1,1)
C
        WRITE (PRTSTR,5040) MULTIP
 5040   FORMAT ('MULTIPLICITY: ',I5)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,5050) NATOM
 5050   FORMAT ('NUMBER OF QM ATOMS: ',I10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
CCC ADLL LA
        IF (LINKAT) THEN
          WRITE (PRTSTR,5055) NLATOM
 5055     FORMAT ('NUMBER OF LINK ATOMS: ',I10)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
CCC ADLL LA
        WRITE (PRTSTR,5060) NELEC
 5060   FORMAT ('NUMBER OF ELECTRONS: ',I10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      END IF
C
C     *** Write molecular MM data ***
C
      IF (QMMM.OR.MMONLY) THEN
C
        CALL MESSAGE(1,'MOLECULAR MM DATA',1,1)
C
        WRITE (PRTSTR,5070) FFIELD
 5070   FORMAT ('FORCE FIELD: ',A10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,5080) NATMM
 5080   FORMAT ('NUMBER OF MM ATOMS: ',I10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      END IF
C
C     *** Write molecular QM/MM data ***
C
      IF (QMMM) THEN
C
        CALL MESSAGE(1,'MOLECULAR QM/MM DATA',1,1)
C
        WRITE (PRTSTR,5090) NTYPQM
 5090   FORMAT ('QM/MM TYPE PARAMETERS USED: ',I5)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        DO I=1,NTYPQM
          WRITE (OUT,5100) LABTYPQM(I),TYPEQM(I)
 5100     FORMAT (5X,A5,5X,I5)
        END DO
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write ALIGNMENT options ***
C
      IF (ALIGNMENT) THEN
C
        CALL MESSAGE(1,'ALIGNMENT OPTIONS',1,1)
C
        WRITE (PRTSTR,5110) POPTOL
 5110   FORMAT ('POPULATION THRESHOLD: ',I5)
C
        WRITE (PRTSTR,5120) CUBLEV
 5120   FORMAT ('UNIT CUBE SAMPLING DEEPTH: ',I5)
C
        IF (UNIFORM) THEN
          CALL MESSAGE(0,'ALIGN WITH UNIFORM WEIGHTED ATOMS',0,0)
        ELSE
          CALL MESSAGE(0,'ALIGN WITH MASS WEIGHTED ATOMS',0,0)
        END IF
C
        IF (MODMAP) THEN
          CALL MESSAGE(0,'ALIGN WITH BEST PRESERVED CONNECTIVITY',0,0)
        END IF
C
        IF (ENANTIO) THEN
          CALL MESSAGE(0,'PATTERN ENANTIOMERS INCLUDED',0,0)
        END IF
C
        IF (NBLACK.NE.0) THEN
          PRTSTR = 'THE FOLLOWING ATOMS/ELEMENTS ARE EXCLUDED'
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (OUT,5130) (LABLACK(I),I=1,NBLACK)
 5130     FORMAT (T2,5(2X,A10))
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write SCF options ***
C
      CALL MESSAGE(1,'SCF OPTIONS',1,1)
C
      IF (.NOT.SCF) THEN
        PRTSTR = 'SCF THEORY: NOT USED'
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        GO TO 1000
      ELSE IF (GUESSONLY) THEN
        PRTSTR = 'SCF THEORY: GUESS ONLY CALCULATION'
      ELSE IF (SCFMAX.EQ.0) THEN
        PRTSTR = 'SCF THEORY: ENERGY ONLY CALCULATION (NO SCF)'
      ELSE IF (OKS) THEN
        IF (ROKS) THEN
          PRTSTR = 'SCF THEORY: RESTRICTED OPEN KOHN-SHAM (ROKS)'
        ELSE IF (CUKS) THEN
          PRTSTR = 'SCF THEORY: CONSTRAINED UNRESTRICTED KOHN-SHAM'
          PRTSTR = PRTSTR(1:50)//'(CUKS)'
        ELSE
          PRTSTR = 'SCF THEORY: UNRESTRICTED KOHN-SHAM (UKS)'
        END IF
      ELSE
        PRTSTR = 'SCF THEORY: RESTRICTED KOHN-SHAM (RKS)'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      IF (ROKS) THEN
        PRTSTR = 'ROKS MIXING: '//ROKSMODE
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      WRITE (PRTSTR,5140) MATDIA
 5140 FORMAT ('DIAGONALIZER: ',A)
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (HUECKEL) THEN
        PRTSTR = 'START GUESS: HUECKEL DENSITY'
      ELSE IF (FERMI) THEN
        PRTSTR = 'START GUESS: FERMI DENSITY'
      ELSE IF (PROJCT) THEN
        PRTSTR = 'START GUESS: PROJECTION'
      ELSE IF (RESTART) THEN
        PRTSTR = 'START GUESS: RESTART DENSITY'
      ELSE
        PRTSTR = 'START GUESS: CORE DENSITY'
      END IF
      IF (HBOMD) PRTSTR = PRTSTR(1:30)//' (CORE GUESS FOR BOMD)'
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      IF (LOCO) THEN
        WRITE (PRTSTR,5150) LOCOLIST(0)
 5150   FORMAT ('GUESS LOCALIZATION: FIRST ',I5,' MOS')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      WRITE (PRTSTR,5160) SCFTOL
 5160 FORMAT ('SCF TOLERANCE: ',G10.3)
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (SCFADJ.AND.OPT) THEN
        PRTSTR = 'SCF TOLERANCE WILL BE TIGHTEN'
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (CDXC) THEN
        PRTSTR = 'POTENTIAL DENSITY: FITTED DENSITY'
      ELSE 
        PRTSTR = 'POTENTIAL DENSITY: ORBITAL DENSITY'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      WRITE (PRTSTR,5170) SCFMAX
 5170 FORMAT ('NUMBER OF SCF ITERATIONS: ',I10)
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (DIIS) THEN
        PRTSTR = 'DIIS ACCELERATION: ON'
      ELSE
        PRTSTR = 'DIIS ACCELERATION: OFF'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (LSHIFT) THEN
        WRITE (PRTSTR,5180) LSVAL
 5180   FORMAT ('LEVEL-SHIFT: ON  (SHIFT VALUE = ',F7.3,' HARTREE)')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (CTYP.EQ.'P86') THEN
        IF (LCTYP.EQ.'PVWN') THEN
          STRING = 'VWN'//'|'//ADJUSTL(CTYP)
        ELSE
          STRING = ADJUSTR(LCTYP)//'|'//ADJUSTL(CTYP)
        END IF
      ELSE IF (CTYP.EQ.'PVWN') THEN
        STRING = 'VWN'
      ELSE IF (CTYP.EQ.'RVWN') THEN
        STRING = 'VWN (RPA)'
      ELSE IF (CTYP.EQ.'PB3LYP') THEN
        STRING = 'B3LYP'
      ELSE IF (CTYP.EQ.'RB3LYP') THEN
        STRING = 'B3LYP (RPA)'
      ELSE
        STRING = CTYP
      END IF
C
      IF (EXX.AND.(.NOT.HYBRID)) THEN
        PRTSTR = 'XC-POTENTIAL: FOCK - '//STRING
      ELSE IF ((XTYP.EQ.'B3LYP').AND.(PLYP.EQ.1.0)) THEN
        PRTSTR = 'XC-POTENTIAL: BH&H - LYP'
      ELSE IF ((XTYP.EQ.'CAMB88').AND.(CAMALPHA.EQ.0.54)) THEN
        PRTSTR = 'XC-POTENTIAL: CAMQTP00 - '//STRING
      ELSE IF ((XTYP.EQ.'CAMB88').AND.(CAMALPHA.EQ.0.23)) THEN
        PRTSTR = 'XC-POTENTIAL: CAMQTP01 - '//STRING
      ELSE IF ((XTYP.EQ.'HSE').AND.(CAMW.EQ.0.33)) THEN
        PRTSTR = 'XC-POTENTIAL: HSE03 - '//STRING
      ELSE IF ((XTYP.EQ.'HSE').AND.(CAMW.EQ.0.11)) THEN
        PRTSTR = 'XC-POTENTIAL: HSE06 - '//STRING
      ELSE
        PRTSTR = 'XC-POTENTIAL: '//XTYP//' - '//STRING
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (XTYP.EQ.'XALPHA') THEN
        WRITE (PRTSTR,5190) XALPHA
 5190   FORMAT ('ALPHA: ',F8.5)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      PRTSTR = 'GRID TECHNIQUE: '//GRDTYP
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (PRUNED) THEN
        PRTSTR = 'GRID PRUNING: ON'
      ELSE
        PRTSTR = 'GRID PRUNING: OFF'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (DIRGRD) THEN
        PRTSTR = 'GRID METHOD: DIRECT'
      ELSE
        PRTSTR = 'GRID METHOD: CONVENTIONAL'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0) 
C
      IF (GGFUN.NE.'NONE') THEN
        PRTSTR = 'GRID GENERATING FUNCTION: '//GGFUN
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (GRDTYP.EQ.'FIXED') THEN
        PRTSTR = 'GRID TYPE: '//G98TYP
      ELSE IF (GGFUN.EQ.'SCF') THEN
        WRITE (PRTSTR,5200) GRDTOL/10.0
      ELSE IF (GRDTOL.NE.TINY(1.0)) THEN
        WRITE (PRTSTR,5200) GRDTOL
      END IF
 5200 FORMAT ('GRID TOLERANCE: ',G10.3)
      IF (GRDTYP.NE.'NONE') CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (GRDTYP.NE.'NONE') THEN
        PRTSTR = 'ATOMIC WEIGHT FUNCTION: '//WEIGHTS
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        PRTSTR = 'RADIAL QUADRATURE TYPE: '//RADQUA
        IF (NRADGRD.NE.0) PRTSTR = PRTSTR//' (USER DEFINED)'
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (ERITYP.EQ.'DIRECT') THEN
        IF (ERIRAM) THEN
          PRTSTR = 'SCF METHODE: MIXED & DOUBLE ASYMPTOTIC EXPANSION'
        ELSE IF (MULTIPOLE) THEN
          PRTSTR = 'SCF METHODE: DIRECT & DOUBLE ASYMPTOTIC EXPANSION'
        ELSE
          PRTSTR = 'SCF METHODE: DIRECT'
        END IF 
      ELSE
        PRTSTR = 'SCF METHODE: CONVENTIONAL'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (FITTYP.EQ.'ALL') THEN
        PRTSTR = 'FREEZE AUXILIARY FUNCTIONS: OFF'
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      ELSE
        PRTSTR = 'FREEZE AUXILIARY FUNCTIONS: ON'
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (ZETFIX(0).NE.FIXNONE) THEN
          WRITE (PRTSTR,'("GENERAL EXPONENT LIMIT: ",F10.4)') ZETFIX(0)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE
          CALL MESSAGE(0,'GENERAL EXPONENT LIMIT: NONE',0,0)
        END IF
      END IF
C
      IF (USEDIA) THEN
        PRTSTR = 'FIT SOLVER: DIAGONAL SVD'
      ELSE IF (USESVD) THEN
        PRTSTR = 'FIT SOLVER: SVD'
      ELSE IF (USEMIN) THEN
        PRTSTR = 'FIT SOLVER: MINRES'
      ELSE
        PRTSTR = 'FIT SOLVER: ITERATIVE'
      END IF
      IF (FITTING.EQ.'ANALYTICAL') THEN
        PRTSTR = 'ANALYTICAL '//TRIM(PRTSTR)
      ELSE IF (FITTING.EQ.'NUMERICAL') THEN
        PRTSTR = 'NUMERICAL '//TRIM(PRTSTR)
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (USEDIA.OR.USESVD) THEN
        WRITE (PRTSTR,5210) SVDTOL
 5210   FORMAT ('SVD THRESHOLD: ',G10.3)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      WRITE (PRTSTR,5220) ERITHRESH
 5220 FORMAT ('SCF SCREENING: ',G10.3)
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (SYMGEO) THEN
        PRTSTR = 'SYMMETRY USE: ON'
      ELSE IF (SYMMETRY) THEN
        PRTSTR = 'SYMMETRY USE: NONE'
      ELSE
        PRTSTR = 'SYMMETRY USE: OFF'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (RANGRD.OR.ROTGRD) THEN
        PRTSTR = 'ROTATION OF THE ANGULAR GRIDS: ON'
      ELSE
        PRTSTR = 'ROTATION OF THE ANGULAR GRIDS: OFF'
      END IF
      CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      IF (SCFOUT.GE.0) THEN
        WRITE (PRTSTR,5230) SCFOUT
 5230   FORMAT ('SCF PRINTING UNTIL CYCLE ',I6)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (REFGRD) THEN
        PRTSTR = 'GENERATE REFERENCE GRID'
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (SMEAR) THEN
        WRITE (PRTSTR,5240) SMRVAL
 5240   FORMAT ('SMEAR: ON  (SMEAR VALUE = ',F7.3,' HARTREE)')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (CFGATOM) THEN
        PRTSTR = 'ATOMIC CONFIGURATION SELECTED'
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (SPHORB) THEN
          WRITE (OUT,5250) ELCFGA(0)
 5250     FORMAT (T2,'ALPHA ELECTRONS S',/,T18,I1)
          WRITE (OUT,5270) (ELCFGA(L),L=1,3)
          WRITE (OUT,5280) (ELCFGA(L),L=4,8)
          WRITE (OUT,5290) (ELCFGA(L),L=9,15)
          IF (SETAOCC) THEN
            DO 10 L=0,15
              IF (ELCFGA(L).GT.0) THEN
                WRITE (OUT,5300) (AOCCNA(L,M),M=1,ELCFGA(L))
              END IF
   10       CONTINUE
          END IF
          WRITE (OUT,5260) ELCFGB(0)
 5260     FORMAT (T2,'BETA ELECTRONS  S',/,T18,I1)
          WRITE (OUT,5270) (ELCFGB(L),L=1,3)
          WRITE (OUT,5280) (ELCFGB(L),L=4,8)
          WRITE (OUT,5290) (ELCFGB(L),L=9,15)
          IF (SETAOCC) THEN
            DO 20 L=0,15
              IF (ELCFGB(L).GT.0) THEN
                WRITE (OUT,5300) (AOCCNB(L,M),M=1,ELCFGB(L))
              END IF
   20       CONTINUE
          END IF
 5270     FORMAT (T18,'PY,PZ,PX',/,T18,3(I1,2X))
 5280     FORMAT (T18,'D(-2),D(-1),D(0),D(+1),D(+2)',/,
     $            T18,2(I1,5X),I1,4X,2(I1,5X))
 5290     FORMAT (T18,'F(-3),F(-2),F(-1),F(0),F(+1),F(+2),F(+3)',/,
     $            T18,3(I1,5X),I1,4X,3(I1,5X))
 5300     FORMAT (7F10.5)
        ELSE
          WRITE (OUT,5310) (ELCFGA(L),L=0,3)
 5310     FORMAT (T2,'ALPHA ELECTRONS (S,P,D,F):',/,4I5)
          WRITE (OUT,5320) (ELCFGB(L),L=0,3)
 5320     FORMAT (T2,'BETA ELECTRONS (S,P,D,F):',/,4I5)
        END IF
        WRITE (PRTSTR,5330) CFGCYC
 5330   FORMAT (T2,'THE ATOMIC CONFIGURATION WILL BE FIXED FOR',I5,
     $          ' SCF CYCLE(S)')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (MOBLK) THEN
        IF (MOBLKA(0).GT.1) THEN
          IF (OKS) THEN
            CALL MESSAGE(0,'USER DEFINED ALPHA MO BLOCKS',0,0)
          ELSE
            CALL MESSAGE(0,'USER DEFINED MO BLOCKS',0,0)
          END IF
          DO 30 I=1,MOBLKA(0)-1
            WRITE (PRTSTR,'("MOS OF BLOCK: ",I2)') I
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            WRITE (OUT,'(15(2X,I2))') MOBLKA(LLMOBLKA(I):ULMOBLKA(I))
   30     CONTINUE
        END IF
        IF (MOBLKB(0).GT.1) THEN
          CALL MESSAGE(0,'USER DEFINED BETA MO BLOCKS',0,0)
          DO 40 I=1,MOBLKB(0)-1
            WRITE (PRTSTR,'("MOS OF BLOCK: ",I2)') I
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            WRITE (OUT,'(15(2X,I2))') MOBLKB(LLMOBLKB(I):ULMOBLKB(I))
   40     CONTINUE
        END IF
      END IF
C
      IF (MOCC) THEN
        IF (NMOMODA.GT.0) THEN
          IF (OKS) THEN
            CALL MESSAGE(0,'ALPHA MO MODIFICATIONS:',0,0)
          ELSE
            CALL MESSAGE(0,'MO MODIFICATIONS:',0,0)
          END IF
          DO 50 IMOMOD=1,NMOMODA
            IF (OKS) THEN
              WRITE (PRTSTR,5340) INT(MOMODA(1,IMOMOD)),MOMODA(2,IMOMOD)
            ELSE
              WRITE (PRTSTR,5360) INT(MOMODA(1,IMOMOD)),MOMODA(2,IMOMOD)
            END IF
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
   50     CONTINUE
        END IF
        IF (NMOMODB.GT.0) THEN
          IF (OKS) THEN
            CALL MESSAGE(0,'BETA MO MODIFICATIONS:',0,0)
            DO 60 IMOMOD=1,NMOMODB
              WRITE (PRTSTR,5350) INT(MOMODB(1,IMOMOD)),MOMODB(2,IMOMOD)
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
   60       CONTINUE
          END IF
        END IF
 5340   FORMAT ('ALPHA MO',I6,' IS OCCUPIED BY ',F10.5,' ELECTRONS')
 5350   FORMAT ('BETA MO',I6,' IS OCCUPIED BY ',F10.5,' ELECTRONS')
 5360   FORMAT ('MO',I6,' IS OCCUPIED BY ',F10.5,' ELECTRONS')
      END IF
C
      IF (MOEX) THEN
        IF (NMOEXCA.GT.0) THEN
          IF (OKS) THEN
            CALL MESSAGE(0,'ALPHA MO EXCHANGES:',0,0)
          ELSE
            CALL MESSAGE(0,'MO EXCHANGES:',0,0)
          END IF
          DO 70 IMOEXC=1,NMOEXCA
            WRITE (PRTSTR,5370) MOEXCA(1,IMOEXC),MOEXCA(2,IMOEXC)
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
   70     CONTINUE
        END IF
        IF (NMOEXCB.GT.0) THEN
          IF (OKS) THEN
            CALL MESSAGE(0,'BETA MO EXCHANGES:',0,0)
            DO 80 IMOEXC=1,NMOEXCB
              WRITE (PRTSTR,5370) MOEXCB(1,IMOEXC),MOEXCB(2,IMOEXC)
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
   80       CONTINUE
          END IF
        END IF
 5370   FORMAT ('MO',I6,' AND MO',I6,' WILL BE EXCHANGED')
      END IF
C
      IF (FIXCFG) THEN
        IF (FIXLAST) THEN
          IF (FIXCYC.NE.100000) THEN
            WRITE (PRTSTR,5380) FIXCYC
 5380       FORMAT ('FIX THE ELECTRONIC CONFIGURATION ITERATIVELY',
     $              ' BY PROJECTION STARTING WITH SCF CYCLE',I5)
          ELSE
            WRITE (PRTSTR,5390) FIXTOL
 5390       FORMAT ('FIX THE ELECTRONIC CONFIGURATION ',
     $              ' BY PROJECTION FOR SCF ERROR < ',G10.3)
          END IF
        ELSE
          WRITE (PRTSTR,5400) FIXCYC
 5400     FORMAT ('FIX THE ELECTRONIC CONFIGURATION OF SCF CYCLE ',I5,
     $            ' BY PROJECTION')
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (FIXMOA(0)) THEN
        IMOFIX = 0
        DO I=1,NSTO
          IF (FIXMOA(I)) THEN
            IMOFIX = IMOFIX + 1
            EIGVALA(IMOFIX) = I
          END IF
        END DO
        CALL MESSAGE(0,'FIXED ALPHA MOS:',0,0)
        WRITE (OUT,'(10(2X,I5))') INT(EIGVALA(1:IMOFIX))
      END IF
C
      IF (FIXMOB(0)) THEN
        IMOFIX = 0
        DO I=1,NSTO
          IF (FIXMOB(I)) THEN
            IMOFIX = IMOFIX + 1
            EIGVALB(IMOFIX) = I
          END IF
        END DO
        CALL MESSAGE(0,'FIXED BETA MOS:',0,0)
        WRITE (OUT,'(10(2X,I5))') INT(EIGVALB(1:IMOFIX))
      END IF
C
      IF (SPHORB) THEN
        CALL MESSAGE(0,'SPHERICAL ORBITALS WILL BE USED',0,0)
      ELSE
        CALL MESSAGE(0,'CARTESIAN ORBITALS WILL BE USED',0,0)
      END IF
C
      IF (EMBED) THEN
        WRITE (PRTSTR,5410) NEMBED
 5410   FORMAT ('EMBEDDED IN CUBE WITH ',I8,' POINTS')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (MONOPOLE) THEN
          CALL MESSAGE(0,'EMBEDDING INTEGRALS: MULTIPOLE',0,0)
        ELSE
          CALL MESSAGE(0,'EMBEDDING INTEGRALS: ANALYTICAL',0,0)
        END IF
      END IF
C
CCC ADLL
C      IF (EFIELD) THEN
      IF (EFIELD.AND.EFSCF) THEN
CCC ADLL
        WRITE (PRTSTR,5420) (ELFIELD(I),I=1,3)
 5420   FORMAT ('EMBEDDED IN ELECTRIC FIELD ( ',
     $           G10.3,' , ',G10.3,' , ',G10.3,' )')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
      END IF
C
      IF (VDW) THEN
        CALL MESSAGE(0,'EMPIRICAL DISPERSION CONTRIBUTION ADDED',0,0)
        IF (DISPTYP.EQ.'D') THEN 
          CALL MESSAGE(0,'DFT-D DISPERSION PARAMETER:',0,0)
          WRITE (PRTSTR,5430) SSIX
 5430     FORMAT ('S6:',G10.3)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (DISPTYP.EQ.'D3') THEN 
          CALL MESSAGE(0,'DFT-D3(BJ) DISPERSION PARAMETERS:',0,0)
          WRITE (PRTSTR,'(4(A2,": ",G10.3,:,1X))') 'S6',SSIX,'S8',
     $                                             SEIGHT,'A1',AONE,
     $                                             'A2',ATWO
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF  
      END IF
C
      IF (QMMM) THEN
        IF (MONOPOLE) THEN
          CALL MESSAGE(0,'QM/MM INTEGRALS: MULTIPOLE',0,0)
        ELSE
          CALL MESSAGE(0,'QM/MM INTEGRALS: ANALYTICAL',0,0)
        END IF
      END IF
C
C     *** Write constrained DFT options ***
C
      IF (CNSDFT) THEN
C
        CALL MESSAGE(1,'CONSTRAINED DFT OPTIONS',1,1)
C
        WRITE (PRTSTR,5440) NCNSTR
 5440   FORMAT ('NUMBER OF CONSTRAINTS: ',I2)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C       
        IF (FCDFT) WRITE (PRTSTR,5450) 'BECKE FUZZY CELLS'
        IF (HCDFT) WRITE (PRTSTR,5450) 'HIRSHFELD CELLS'
        IF (VCDFT) WRITE (PRTSTR,5450) 'VORONOI CELLS'
 5450   FORMAT ('POPULATION SCHEME: ',A19)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (DEFCDFT) THEN
          WRITE (PRTSTR,*) 'CONSTRAINTS APPLIED ON'//
     $  ' DEFORMATION DENSITY'
        ELSE 
          WRITE (PRTSTR,*) 'CONSTRAINTS APPLIED ON'//
     $  ' ELECTRON DENSITY'
        END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (FCDFTOL) THEN
          WRITE (PRTSTR,5460) CNSTOL
 5460     FORMAT ('CONSTANT CDFT TOLERANCE:',G10.3)
        ELSE
          WRITE (PRTSTR,5470) CNSTOL
 5470     FORMAT ('ADAPTIVE CDFT TOLERANCE:',G10.3)
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
C       *** Print CDFT domains ***
C
        CALL PRTCNS
C
      END IF
C
      IF (ETMOD) THEN
        CALL MESSAGE(0,'ET MODULE ACTIVATED',0,0)
        IF (HDA) THEN
          CALL MESSAGE(0,'TWO DIABATIC ET STATES ARE COMPUTED',0,0)
          IF (PHDA.NE.'NONE') CALL MESSAGE(0,'AN MO PHASE CONVENTION'//
     $                                       'WILL BE APPLIED',0,0)
          IF (DEDIA2) CALL MESSAGE(0,'APPLIED BIAS ON ENERGY GAP',0,0)
          WRITE (PRTSTR,5480) GAPDIA
 5480     FORMAT ('REFERENCE ENERGY GAP (Ha):',G10.3) 
          WRITE (PRTSTR,5490) GAPK
 5490     FORMAT ('HARMONIC FORCE CONSTANT (Ha^-1):',G10.3)
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write intrinsic reaction coordinate options ***
C
      IF (IRC) THEN
C
        CALL MESSAGE(1,'INTRINSIC REACTION COORDINATE OPTIONS',1,1)
C
        IF (IRCTYP.EQ.'READ STEP') THEN
C
          WRITE (PRTSTR,5500) IRCSTEP
 5500     FORMAT ('IRC GEOMETRY OF STEP ',I10,' IS READ')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        ELSE
C
          IF (IRCSIGN.GT.0.0) THEN
            PRTSTR = 'STEP DIRECTION: FORWARD'
          ELSE
            PRTSTR = 'STEP DIRECTION: REVERSE'
          END IF
          IF (IRCTYP.EQ.'EXTEND') THEN
            PRTSTR = PRTSTR(:23)//' (IRC EXTENSION)'
          ELSE IF (IRCTYP.EQ.'RESTART') THEN
            PRTSTR = PRTSTR(:23)//' (IRC RESTART)'
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,5510) IRCDR
 5510     FORMAT ('MAXIMUM IRC STEP: ',G10.3)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,5520) DEVIB
 5520     FORMAT ('INITIAL ENERGY DECREASE: ',G10.3)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (USEMW) THEN
            CALL MESSAGE(0,'IRC PATH IN MASS WEIGHTED COORDINATES',0,0)
          ELSE
            CALL MESSAGE(0,'IRC PATH IN UNWEIGHTED COORDINATES',0,0)
          END IF
C
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write saddle interpolation options ***
C
      IF (SADDLE) THEN
C
        CALL MESSAGE(1,'SADDLE INTERPOLATION OPTIONS',1,1)
C
        WRITE (PRTSTR,5530) SADTOL
 5530   FORMAT ('SADDLE TOLERANCE: ',G10.3)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,5540) SADMAX
 5540   FORMAT ('NUMBER OF INTERPOLATION STEPS: ',I10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,5550) 100.0*SADINT
 5550   FORMAT ('INTERPOLATION DISTANCE REDUCTION [%]: ',F10.2)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        IF (SADALI) THEN
          CALL MESSAGE(0,'REACTANT-PRODUCT ALIGNMENT ENABLED',0,0)
        ELSE
          CALL MESSAGE(0,'REACTANT-PRODUCT ALIGNMENT DISABLED',0,0)
        END IF
C
        IF (SADTS) CALL MESSAGE(0,'HIERACHICAL TS FINDER ENABLED',0,0)
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write potential energy surface scan options ***
C
      IF (SCANPES) THEN
C
        CALL MESSAGE(1,'PES SCAN OPTIONS',1,1)
C
        IF (SCANTYP.EQ.'XYZ') THEN
          WRITE (PRTSTR,5560) SCANLAB(1:STREXT(SCANLAB))
 5560     FORMAT ('CARTESIAN PES SCAN OF ATOM ',A)
        ELSE
          WRITE (PRTSTR,5570) SCANLAB(1:STREXT(SCANLAB))
 5570     FORMAT ('INTERNAL PES SCAN ALONG ',A,' VARIABLE')
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        IF (SCANTYP.EQ.'XYZ') THEN
          WRITE (PRTSTR,5580) UNIT(1)(1:STREXT(UNIT(1))),
     $                        ANGBOR(1)*SCANVAR(1,1:3)
 5580     FORMAT ('FIRST SCAN BOUNDARY [',A,'] : ( ',
     $            F10.5,', ',F10.5,', ',F10.5,' )')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,5590) UNIT(1)(1:STREXT(UNIT(1))),
     $                        ANGBOR(1)*SCANVAR(2,1:3)
 5590     FORMAT ('SECOND SCAN BOUNDARY [',A,'] : ( ',
     $            F10.5,', ',F10.5,', ',F10.5,' )')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (SCANTYP.EQ.'BOND') THEN
          WRITE (PRTSTR,5600) SCANLAB(1:STREXT(SCANLAB)),
     $    ANGBOR(1)*SCANVAR(1,1),ANGBOR(1)*SCANVAR(2,1),UNIT(1)
 5600     FORMAT ('SCAN BOND ',A,' FROM ',F10.3,' TO ',F10.3,1X,A)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE
          WRITE (PRTSTR,5610) SCANLAB(1:STREXT(SCANLAB)),
     $    SCANVAR(1,1)/DEGRAD,SCANVAR(2,1)/DEGRAD
 5610     FORMAT (1X,A,' FROM ',F10.3,' TO ',F10.3,' DEGREE')
          PRTSTR = SCANTYP(1:STREXT(SCANTYP))//PRTSTR(1:STREXT(PRTSTR))
          PRTSTR = 'SCAN '//PRTSTR(1:STREXT(PRTSTR))
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
C
        IF (RELAXING) THEN
          PRTSTR = 'SCAN TECHNIQUE: ADIABATIC'
        ELSE
          PRTSTR = 'SCAN TECHNIQUE: VERTICAL'
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,5620) INT(SCANVAR(1,0))
 5620   FORMAT ('NUMBER OF SCAN STEPS: ',I10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Entry point for reduced header ***
C
      ENTRY REDUCED_HEADER
C
 1000 CONTINUE
C
C     ------------------------------------------------------------------
C
C     *** Write geometry optimization options ***
C
      IF (OPT.OR.(IRC.AND.(IRCTYP.NE.'READ STEP'))) THEN
C
        IF (IRC) THEN
          CALL MESSAGE(1,'IRC PATH OPTIMIZATION OPTIONS',1,1)
        ELSE
          CALL MESSAGE(1,'GEOMETRY OPTIMIZATION OPTIONS',1,1)
        END IF
C
        WRITE (PRTSTR,5630) OPTMAX
 5630   FORMAT ('NUMBER OF OPTIMIZATION STEPS: ',I10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,5640) MAXDR
 5640   FORMAT ('MAXIMUM OPTIMIZATION STEP: ',G10.3)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,5650) OPTTOL
 5650   FORMAT ('OPTMIZATION TOLERANCE: ',G10.3)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        IF (OPTTYP.EQ.'CARTESIAN') THEN
          CALL MESSAGE(0,'COORDINATE SYSTEM: CARTESIAN',0,0)
        ELSE IF (OPTTYP.EQ.'INTERNAL') THEN
          CALL MESSAGE(0,'COORDINATE SYSTEM: INTERNAL Z-MATRIX',0,0)
        ELSE IF (OPTTYP.EQ.'REDUNDANT') THEN
          CALL MESSAGE(0,'COORDINATE SYSTEM: INTERNAL REDUNDANT',0,0)
        END IF
C
        IF (TS) THEN 
          CALL MESSAGE(0,'TRANSITION STATE SEARCH',0,0)
          IF (HMODE.EQ.0) THEN
            WRITE (PRTSTR,5660) 
 5660       FORMAT ('FOLLOWING LOWEST MODE OF HESSIAN')
          ELSE
            WRITE (PRTSTR,5670) HMODE
 5670       FORMAT ('FOLLOWING HESSIAN MODE: ',I5)
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
C
        IF (UPHESS.EQ.'ANALYTIC') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: ANALYTICAL',0,0)
        ELSE IF (UPHESS.EQ.'NUMERIC') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: NUMERICAL',0,0)
        ELSE IF (UPHESS.EQ.'POWELL') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: POWELL',0,0)
        ELSE IF (UPHESS.EQ.'BERNY') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: BERNY',0,0)
        ELSE IF (UPHESS.EQ.'BFGS') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: BFGS',0,0)
        ELSE IF (UPHESS.EQ.'DFP') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: DFP',0,0)
        ELSE IF (UPHESS.EQ.'MSP') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: MSP',0,0)
        ELSE IF (UPHESS.EQ.'SR1') THEN
          CALL MESSAGE(0,'HESSIAN UPDATE: SR1',0,0)
        END IF
C
        IF (STEPTYP.EQ.'P-RFO') THEN
          CALL MESSAGE(0,'STEP TYPE: (P)-RFO',0,0)
        ELSE IF (STEPTYP.EQ.'PES-WALK') THEN
          CALL MESSAGE(0,'STEP TYPE: PES WALKING',0,0)
        ELSE IF (STEPTYP.EQ.'DESCENT') THEN
          CALL MESSAGE(0,'STEP TYPE: STEEPEST DESCENT',0,0)
        ELSE IF (STEPTYP.EQ.'LEVENBERG') THEN
          CALL MESSAGE(0,'STEP TYPE: LEVENBERG-MARQUARDT',0,0)
        END IF
C
        IF (HESSIAN.EQ.'UNITMATRIX') THEN
          PRTSTR = 'THE START HESSIAN IS THE UNIT-MATRIX'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'INTERNAL') THEN
          PRTSTR = 'THE START HESSIAN IS THE INTERNAL UNIT-MATRIX'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'BAKER') THEN
          PRTSTR = 'THE BAKER START HESSIAN IS USED'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'FISCHER') THEN
          PRTSTR = 'THE FISCHER START HESSIAN IS USED'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'LINDH') THEN
          PRTSTR = 'THE LINDH START HESSIAN IS USED'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'READ') THEN
          PRTSTR = 'THE START HESSIAN IS READ FROM THE INPUT FILE'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'ANALYTIC') THEN
          PRTSTR = 'THE START HESSIAN IS ANALYTICALLY CALCULATED'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'NUMERIC') THEN
          PRTSTR = 'THE START HESSIAN IS NUMERICALLY CALCULATED'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HESSIAN.EQ.'RESTART') THEN
          PRTSTR = 'THE START HESSIAN IS READ FROM THE RESTART FILE'
          CALL MESSAGE(0,PRTSTR,0,0)
        END IF
C
        IF (QMONLY.OR.QMMM) THEN
          IF (BWDER) THEN
            CALL MESSAGE(0,'WEIGHT DERIVATIVES WILL BE USED',0,0)
          ELSE
            CALL MESSAGE(0,'NO WEIGHT DERIVATIVES WILL BE USED',0,0)
          END IF
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write molecular dynamics options ***
C
      IF (MD) THEN
C
        CALL MESSAGE(1,'MOLECULAR DYNAMICS OPTIONS',1,1)
C
        IF (TRJTYP.EQ.'RESTART') THEN
          PRTSTR = 'AN OLD MOLECULAR DYNAMICS RUN WILL BE RESTARTED'
          CALL MESSAGE(0,PRTSTR,0,0)
          IF (TRJACC) THEN
            PRTSTR = 'ACCELERATIONS ARE READ FROM THE TRAJECTORY FILE'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          IF (MDCONST) THEN
            PRTSTR = 'MOLECULAR DYNAMICS CONSTRAINTS ARE ENFORCED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
        ELSE IF (TRJTYP.EQ.'ANALYSIS') THEN
          PRTSTR = 'A MOLECULAR DYNAMICS TRAJECTORY WILL BE ANALYSED'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (TRJTYP.EQ.'PROPERTY') THEN
          IF (TRJDIP) THEN
            PRTSTR = 'DIPOLE MOMENTS ALONG TRAJECTORY ARE CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          IF (TRJPOL) THEN
            PRTSTR = 'POLARIZABILITIES ALONG TRAJECTORY ARE CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          IF (TRJNMR) THEN
            PRTSTR = 'NMR SHIELDINGS ALONG TRAJECTORY ARE CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          IF (TRJSSCC) THEN
            PRTSTR = 'NMR COUPLINGS ALONG TRAJECTORY ARE CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          IF (TRJNSR) THEN
            PRTSTR = 'NSR CONSTANTS ALONG TRAJECTORY ARE CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          IF (TRJMAG) THEN
            PRTSTR = 'MAGNETIZABILITIES ALONG TRAJECTORY ARE CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          IF (TRJROG) THEN
            PRTSTR = 'ROT. G-TENSORS ALONG TRAJECTORY ARE CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
        ELSE IF (MDRUN) THEN
          PRTSTR = 'A MOLECULAR DYNAMICS RUN WILL BE PERFORMED'
          CALL MESSAGE(0,PRTSTR,0,0)
          IF (MDCONST) THEN
            PRTSTR = 'MOLECULAR DYNAMICS CONSTRAINTS ARE ENFORCED'
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
        END IF
C
        IF (TRJTYP.EQ.'ANALYSIS') THEN
          WRITE (PRTSTR,5680) SIMINT
 5680     FORMAT (' EVERY ',I9,' TIME STEPS')
          IF (PTRJ.EQ.'SIMILARITY') THEN
            PRTSTR = 'ALIGNMENT'//PRTSTR(1:STREXT(PRTSTR))
          ELSE
            PRTSTR = 'SAMPLING'//PRTSTR(1:STREXT(PRTSTR))
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (TRJTYP.EQ.'PROPERTY') THEN
          WRITE (PRTSTR,5690) SIMINT
 5690     FORMAT ('PROPERTY CALCULATION EVERY ',I9,' TIME STEPS')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (TRJTYP.NE.'READ STEP') THEN
          WRITE (PRTSTR,5700) MDINT
 5700     FORMAT ('OUTPUT AND PLOT FLUSHING EVERY ',I9,' TIME STEPS')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
C
        IF (TRJTYP.EQ.'NEW') THEN
          IF (MDRUN) THEN
            PRTSTR = 'A NEW TRAJECTORY FILE WILL BE GENERATED'
          ELSE
            PRTSTR = 'A TRAJECTORY FILE WILL BE READ'
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (TRJTYP.EQ.'RESTART') THEN
          PRTSTR = 'A RESTART TRAJECTORY FILE WILL BE EXTENDED'
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (TRJTYP.EQ.'PLOT') THEN
          IF ((TRJMIN.EQ.-1).AND.(TRJMAX.EQ.-1)) THEN
            WRITE (PRTSTR,5710) TRJINT
 5710       FORMAT ('FULL TRAJECTORY IS PLOTTED (STEPSIZE: ',I10,' )')
          ELSE
            WRITE (PRTSTR,5720) TRJMIN,TRJMAX,TRJINT
 5720       FORMAT ('A TRAJECTORY PLOT FROM ENTRY ',I10,' TO ',I10,1X,
     $              'WILL BE PERFORMED (STEPSIZE: ',I10,' )')
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (TRJTYP.EQ.'READ STEP') THEN
          WRITE (PRTSTR,5730) TRJMIN
 5730     FORMAT ('TRAJECTORY GEOMETRY ',I10,' IS READ')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (TRJTYP.EQ.'CUT<') THEN
          WRITE (PRTSTR,5740) TRJMIN-1
 5740     FORMAT ('THE FIRST ',I10,' TRAJECTORY ENTRIES ARE CUT')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (TRJTYP.EQ.'CUT>') THEN
          WRITE (PRTSTR,5750) TRJMAX+1
 5750     FORMAT ('THE TRAJECTORY END IS CUTTED FROM ENTRY ',I10,' UP')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
C
C     *** Write trajectory analysis options ***
C
        IF (TRJTYP.EQ.'ANALYSIS') THEN
C
          IF ((TRJMIN.EQ.-1).AND.(TRJMAX.EQ.-1)) THEN
            PRTSTR = 'FULL TRAJECTORY IS ANALYSED'
          ELSE IF (TRJMIN.EQ.-1) THEN
            WRITE (PRTSTR,5760) TRJMAX
 5760       FORMAT ('TRAJECTORY IS ANALYSED UNTIL ENTRY ',I9)
          ELSE IF (TRJMAX.EQ.-1) THEN
            WRITE (PRTSTR,5770) TRJMIN
 5770       FORMAT ('TRAJECTORY IS ANALYSED FROM ENTRY ',I9)
          ELSE
            WRITE (PRTSTR,5780) TRJMIN,TRJMAX
 5780       FORMAT ('TRAJECTORY IS ANALYSED FROM ENTRY ',I9,' TO ',I9)
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
          IF (PTRJ.EQ.'DIPOLE') THEN
            CALL MESSAGE(0,'A DIPOLE ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'POLARIS') THEN
            CALL MESSAGE(0,'A POLARIZABILITY ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'SIMILARITY') THEN
            CALL MESSAGE(0,'A SIMILARITY ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'NMR') THEN
            PRTSTR = 'A NMR SHIELDING ANALYSIS IS PERFORMED'
            CALL MESSAGE(0,PRTSTR,0,0)
            IF (NSRC) THEN
              PRTSTR = 'A NSR CONSTANTS ANALYSIS IS PERFORMED'
              CALL MESSAGE(0,PRTSTR,0,0)
            END IF
          ELSE IF (PTRJ.EQ.'SSCC') THEN
            CALL MESSAGE(0,'A NMR COUPLINGS ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'FULLNMR') THEN
            CALL MESSAGE(0,'A NMR SHIELDING ANALYSIS IS PERFORMED',0,0)
            CALL MESSAGE(0,'A NMR COUPLINGS ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'MAGNET') THEN
            PRTSTR = 'A MAGNETIZABILITY ANALYSIS IS PERFORMED'
            CALL MESSAGE(0,PRTSTR,0,0)
            IF (RGTEN) THEN
              PRTSTR = 'A ROTATIONAL G-TENSOR ANALYSIS IS PERFORMED'
              CALL MESSAGE(0,PRTSTR,0,0)
            END IF
          ELSE IF (PTRJ.EQ.'ANGLE') THEN
            PRTSTR = 'AN AVERAGE BOND ANGLE ANALYSIS IS PERFORMED'
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (PTRJ.EQ.'DIHEDRAL') THEN
            PRTSTR = 'AN AVERAGE DIHEDRAL ANGLE ANALYSIS IS PERFORMED'
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (PTRJ.EQ.'LENGTH') THEN
            PRTSTR = 'AN AVERAGE BOND LENGTH ANALYSIS IS PERFROMED'
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (PTRJ.EQ.'LINDEMANN') THEN
            CALL MESSAGE(0,'A LINDEMANN ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'RADDISTR') THEN
            PRTSTR = 'A RADIAL DISTRIBUTION FUNCTION IS CALCULATED'
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (PTRJ.EQ.'MEANDIS') THEN
            PRTSTR = 'A MEAN INTERATOMIC DISTANCE ANALYSIS IS PERFROMED'
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (PTRJ.EQ.'LPMOMENTA') THEN
            CALL MESSAGE(0,'A LP MOMENT ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'PHASESPACE') THEN
            CALL MESSAGE(0,'A PHASE SPACE ANALYSIS IS PERFORMED',0,0)
          ELSE IF (PTRJ.EQ.'PROLATE') THEN
            PRTSTR = 'A PROLATE DEFORMATION ANALYSIS IS PERFROMED'
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (ETRJ.EQ.'TOTAL') THEN
            CALL MESSAGE(0,'A SYSTEM ENERGY ANALYSIS IS PERFORMED',0,0)
          ELSE
            CALL MESSAGE(0,'A ENERGY ANALYSIS IS PERFORMED',0,0)
          END IF
C
C     *** Write trajectory property calculation options ***
C
        ELSE IF (TRJTYP.EQ.'PROPERTY') THEN
C
          IF ((TRJMIN.EQ.-1).AND.(TRJMAX.EQ.-1)) THEN
            PRTSTR = 'PROPERTIES CALCULATED ALONG FULL TRAJECTORY'
          ELSE IF (TRJMIN.EQ.-1) THEN
            WRITE (PRTSTR,5790) TRJMAX
 5790       FORMAT ('PROPERTIES CALCULATED UNTIL ENTRY ',I9)
          ELSE IF (TRJMAX.EQ.-1) THEN
            WRITE (PRTSTR,5800) TRJMIN
 5800       FORMAT ('PROPERTIES CALCULATED FROM ENTRY ',I9)
          ELSE
            WRITE (PRTSTR,5810) TRJMIN,TRJMAX
 5810       FORMAT ('PROPERTIES CALCULATED FROM ENTRY ',I9,' TO ',I9)
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
C     *** Write trajectory generation options ***
C
        ELSE IF (TRJTYP.NE.'READ STEP') THEN
C
          IF (TRJINT.NE.MDINT) THEN
            WRITE (PRTSTR,5820) TRJINT
 5820       FORMAT ('TRAJECTORY UPDATE EVERY ',I9,' TIME STEPS')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
          IF ((TRJTYP.EQ.'NEW').OR.(TRJTYP.EQ.'RESTART')) THEN
            PRTSTR = 'TRAJECTORY CONTENT: HEADER, R, V'
            IF (TRJACC) PRTSTR = PRTSTR(1:STREXT(PRTSTR))//', A'
            IF (TRJMOS) PRTSTR = PRTSTR(1:STREXT(PRTSTR))//', MOs'
            IF (TRJNHC) PRTSTR = PRTSTR(1:STREXT(PRTSTR))//', NOSE'
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
          IF (BATH) THEN
            IF (MDBATH.EQ.'HOOVER') THEN
              PRTSTR = 'SPECIFIED ENSEMBLE: ~(N,V,T)'
            ELSE IF (MDBATH.EQ.'NOSE') THEN
              PRTSTR = 'SPECIFIED ENSEMBLE: (N,V,T)'
            ELSE
              PRTSTR = 'SPECIFIED ENSEMBLE: NONE'
            END IF
          ELSE
            PRTSTR = 'SPECIFIED ENSEMBLE: (N,V,E)'
          END IF
          CALL MESSAGE(0,PRTSTR,0,0)
C
          IF (MDRADIUS.NE.0.0) THEN
            WRITE (PRTSTR,5830) ANGBOR(1)*MDRADIUS,UNIT(1)
 5830       FORMAT ('CONFINEMENT RADIUS: ',F10.3,1X,A)
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
          IF (MDZERO.EQ.'P') THEN
            WRITE (PRTSTR,'("INITIAL LINEAR MOMENTUM SET TO ZERO")')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          ELSE IF (MDZERO.EQ.'LP') THEN
            WRITE (PRTSTR,'("INITIAL TOTAL MOMENTUM SET TO ZERO")')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
          IF (MDFLP) THEN
            WRITE (PRTSTR,5840) LPTOL
 5840       FORMAT ('LP FORCE CONSERVATION: ',G10.1)
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
          IF (MDVLP) THEN
            WRITE (PRTSTR,'("LP VELOCITY CONSERVATION ON")')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
          WRITE (PRTSTR,5850) MDMAX-MDMIN
 5850     FORMAT ('NUMBER OF MD STEPS: ',I10)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
          WRITE (PRTSTR,5860) MDSTEP/FSEC
 5860     FORMAT ('MD TIME STEP: ',F10.1,' FSEC')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
          IF (BATH) THEN
C
            IF (MDTEMP.EQ.MDTEMP0) THEN
              WRITE (PRTSTR,5870) MDTEMP
 5870         FORMAT ('DESIRED TEMPERATURE: ',F7.1,' KELVIN')
            ELSE
              WRITE (PRTSTR,5880) MDTEMP0,MDTEMP
 5880         FORMAT ('TEMPERATURE CHANGE ',F7.1,' -> ',F7.1,' KELVIN')
            END IF
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
            IF (MDBATH.EQ.'SCALING') THEN
              IF (MDTAU.EQ.1.0) THEN
                WRITE (PRTSTR,'("TEMPERATURE SCALING EVERY TIME STEP")')
                CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              ELSE 
                WRITE (PRTSTR,5890) INT(MDTAU)
 5890           FORMAT ('TEMPERATURE SCALING EVERY ',I6,' TIME STEPS')
                CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              END IF
            ELSE IF (MDBATH.EQ.'BERENDSEN') THEN
              WRITE (PRTSTR,'("BERENDSEN THERMOSTAT IS APPLIED")')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              WRITE (PRTSTR,5900) MDTAU/(1000.0*FSEC)
 5900         FORMAT ('BERENDSEN THERMOSTAT TAU: ',F10.1,' PSEC')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            ELSE IF (MDBATH.EQ.'HOOVER') THEN
              WRITE (PRTSTR,'("HOOVER THERMOSTAT IS APPLIED")')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            ELSE IF (MDBATH.EQ.'NOSE') THEN
              WRITE (PRTSTR,'("NOSE THERMOSTAT IS APPLIED")')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              WRITE (PRTSTR,5910) MDTAU*WAVENUM
 5910         FORMAT ('THERMOSTAT COUPLING FREQUENCY: ',F10.1,' CM^-1')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              IF (NNHC.GT.1) THEN
                WRITE (PRTSTR,5920) NNHC
 5920           FORMAT ('NOSE-HOOVER CHAIN LENGTH: ',I3)
                CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              END IF
            END IF
C
          END IF
C
        END IF
C
        IF (PTMD) THEN
          CALL MESSAGE(0,'PARALLEL TEMPERING WILL BE PERFORMED',0,0)
          PRTSTR = ''
          WRITE (PRTSTR,'("GEOMETRY EXCHANGE EVERY",'//
     $                    'I4," STEPS")') EXSTEP
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          PRTSTR = ''
          WRITE (PRTSTR,'("NUMBER OF REPLICAS: ",I3)') NUMREP
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          PRTSTR = ''
          WRITE (PRTSTR,'("LOWEST TEMPERATURE: ",F6.1,'//
     $                  '" KELVIN")') TMINIMUM
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          PRTSTR = ''
          WRITE (PRTSTR,'("HIGHEST TEMPERATURE: ",F6.1,'//
     $                  '" KELVIN")') TMAXIMUM
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (TEMPLIN) THEN
            CALL MESSAGE(0,'LINERAR TEMPERATURE SCALING SELECTED',0,0)
          END IF
          PRTSTR = ''
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write vibrational analysis options ***
C
      IF (FREQ) THEN
C
        CALL MESSAGE(1,'VIBRATIONAL ANALYSIS OPTIONS',1,1)
C
        IF (FREQRST) THEN
          PRTSTR = 'AN OLD FREQUENCY ANALYSIS WILL BE RESTARTED'
        ELSE IF (FREQFIX(0)) THEN
          PRTSTR = 'A RESTRICTED FREQUENCY ANALYSIS WILL BE PERFORMED'
        ELSE
          PRTSTR = 'A FREQUENCY ANALYSIS WILL BE PERFORMED'
        END IF
        CALL MESSAGE(0,PRTSTR,0,0)
C
        IF (VIBSTEP.NE.1.0) THEN
          WRITE (PRTSTR,5930) 0.01*VIBSTEP*BOHR
 5930     FORMAT ('DISPLACEMENT STEP: ',G10.3,' BOHR')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
C
        IF (RAMAN) THEN
          IF (FREQTYP.EQ.'ANALYTIC') THEN
            WRITE (PRTSTR,5940)
 5940       FORMAT ('SEMI-ANALYTICAL RAMAN INTENSITIES REQUESTED')
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (FREQTYP.EQ.'NUMERIC') THEN
            WRITE (PRTSTR,5950)
 5950       FORMAT ('NUMERICAL RAMAN INTENSITIES REQUESTED')
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          WRITE (PRTSTR,5960) FFRSTEP
 5960     FORMAT ('RAMAN FINITE FIELD STRENGTH: ',F10.5,' [A.U.]')
          CALL MESSAGE(0,PRTSTR,0,0)
        END IF
C
        IF (FREQTYP.EQ.'ANALYTIC') THEN
          CALL MESSAGE(0,'USE APPROXIMATED DENSITY PERTURBATION',0,0)
          IF (FALDA) THEN
            CALL MESSAGE(0,'ALDA KERNEL CALCULATION',0,0)
          ELSE IF (FDFXC) THEN
            CALL MESSAGE(0,'NUMERICAL KERNEL CALCULATION',0,0)
          ELSE
            CALL MESSAGE(0,'ANALYTICAL KERNEL CALCULATION',0,0)
          END IF
          WRITE (PRTSTR,5970) ADPMAX
 5970     FORMAT ('NUMBER OF ADP ITERATIONS: ',I10)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,5980) ADPTOL
 5980     FORMAT ('ADP TOLERANCE: ',G10.3)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (ADPTYP.EQ.'DIRECT') THEN
            PRTSTR = 'ADP METHOD: DIRECT'
          ELSE IF (ADPTYP.EQ.'ADP-I/O') THEN
            PRTSTR = 'ADP METHOD: CONVENTIONAL'
          END IF
        ELSE IF (FREQTYP.EQ.'NUMERIC') THEN
          CALL MESSAGE(0,'USE FINITE FIELD PERTURBATION',0,0)
        END IF
C
        IF (THERMOS) THEN
          WRITE (PRTSTR,5990)
 5990     FORMAT ('THERMODYNAMICAL DATA WILL BE CALCULATED')
          CALL MESSAGE(0,PRTSTR,0,0)
          IF (VIBONLY) THEN
            WRITE (PRTSTR,6000)
 6000       FORMAT ('ONLY FROM VIBRATIONAL PARTITION FUNCTION')
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
        END IF
C
        IF (FREQFIX(0)) CALL PRIFIX
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write polarizability options ***
C
      IF (POLARIS) THEN
C
        CALL MESSAGE(1,'POLARIZABILITY OPTIONS',1,1)
C
        IF (POLTYP.EQ.'ANALYTIC') THEN
          CALL MESSAGE(0,'USE APPROXIMATED DENSITY PERTURBATION',0,0)
          WRITE (PRTSTR,5970) ADPMAX
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,5980) ADPTOL
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (ADPTYP.EQ.'DIRECT') THEN
            PRTSTR = 'ADP METHOD: DIRECT'
          ELSE IF (ADPTYP.EQ.'ADP-I/O') THEN
            PRTSTR = 'ADP METHOD: CONVENTIONAL'
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (POLTYP.EQ.'NIACPKS') THEN
          CALL MESSAGE(0,'USE NON-ITERATIVE APPROACH TO CPKS',0,0)
          WRITE (PRTSTR,6010) FFSTEP
 6010     FORMAT ('KOHN SHAM MATRIX FIELD STRENGTH ',F10.5,' [A.U.]')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (POLTYP.EQ.'NUMERIC') THEN
          CALL MESSAGE(0,'USE FINITE FIELD PERTURBATION',0,0)
          WRITE (PRTSTR,6020) FFSTEP
 6020     FORMAT ('FINITE FIELD STRENGTH ',F10.5,' [A.U.]')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
C
        IF (POLORD.EQ.1) THEN
          IF ((EFISH).AND.(POLKEY.EQ.'DD')) THEN
            PRTSTR = 'DIPOLE-DIPOLE POLARIZABILITIES (EFISH)'
          ELSE IF (EFISH.AND.(POLKEY.EQ.'DQ')) THEN
            PRTSTR = 'DIPOLE-QUADRUPOLE POLARIZABILITIES (EFISH)'
          ELSE IF (POLKEY.EQ.'DD') THEN
            PRTSTR = 'DIPOLE-DIPOLE POLARIZABILITIES (TENSOR)'
          ELSE IF (POLKEY.EQ.'DQ') THEN
            PRTSTR = 'DIPOLE-QUADRUPOLE POLARIZABILITIES (TENSOR)'
          END IF
          IF (POLTYP.EQ.'ANALYTIC') THEN
            IF (WPOLAR(1).NE.0.0) THEN
              IF (UNIT(4).EQ.'CM^-1') THEN
                WRITE (STRING,6030) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
 6030           FORMAT ('W = ',F9.1,' [CM^-1]')
              ELSE IF (UNIT(4).EQ.'EV') THEN
                WRITE (STRING,6040) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
 6040           FORMAT ('W = ',F9.4,' [EV]')
              ELSE IF (UNIT(4).EQ.'NM') THEN
                WRITE (STRING,6050) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
 6050           FORMAT ('W = ',F9.3,' [NM]')
              ELSE
                WRITE (STRING,6060) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
 6060           FORMAT ('W = ',F9.4,' [A.U.]')
              END IF
              PRTSTR = PRTSTR(1:STREXT(PRTSTR))//' '//STRCOMP(STRING)
            END IF
          END IF
        ELSE IF (POLORD.EQ.2) THEN
          IF (EFISH.OR.(POLTYP.EQ.'NUMERIC')) THEN
            PRTSTR = 'FIRST HYPERPOLARIZABILITIES (EFISH)'
          ELSE
            PRTSTR = 'FIRST HYPERPOLARIZABILITIES (TENSOR)'
          END IF
          IF (POLTYP.EQ.'ANALYTIC') THEN
            IF ((WPOLAR(1).NE.0.0).OR.(WPOLAR(2).NE.0.0)) THEN
              IF (UNIT(4).EQ.'CM^-1') THEN
                WRITE (STRING,6070) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
 6070           FORMAT ('W1 = ',F9.1)
                PRTSTR = PRTSTR(1:STREXT(PRTSTR))//' '//STRCOMP(STRING)
                WRITE (STRING,6080) WOMEGA(WPOLAR(2),UNIT(4),'AU->')
 6080           FORMAT ('W2 = ',F9.1,' [CM^-1]')
              ELSE IF (UNIT(4).EQ.'EV') THEN
                WRITE (STRING,6090) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
 6090           FORMAT ('W1 = ',F9.4)
                PRTSTR = PRTSTR(1:STREXT(PRTSTR))//' '//STRCOMP(STRING)
                WRITE (STRING,6100) WOMEGA(WPOLAR(2),UNIT(4),'AU->')
 6100           FORMAT ('W2 = ',F9.4,' [EV]')
              ELSE IF (UNIT(4).EQ.'NM') THEN
                WRITE (STRING,6110) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
 6110           FORMAT ('W1 = ',F9.3)
                PRTSTR = PRTSTR(1:STREXT(PRTSTR))//' '//STRCOMP(STRING)
                WRITE (STRING,6120) WOMEGA(WPOLAR(2),UNIT(4),'AU->')
 6120           FORMAT ('W2 = ',F9.3,' [NM]')
              ELSE
                WRITE (STRING,6090) WOMEGA(WPOLAR(1),UNIT(4),'AU->')
                PRTSTR = PRTSTR(1:STREXT(PRTSTR))//' '//STRCOMP(STRING)
                WRITE (STRING,6130) WOMEGA(WPOLAR(2),UNIT(4),'AU->')
 6130           FORMAT ('W2 = ',F9.4,' [A.U.]')
              END IF
              PRTSTR = PRTSTR(1:STREXT(PRTSTR))//' '//STRCOMP(STRING)
            END IF
          END IF
        ELSE IF (POLORD.EQ.3) THEN
          PRTSTR = 'CALCULATE SECOND HYPERPOLARIZABILITIES'
        END IF
C
        CALL MESSAGE(0,PRTSTR,0,0)
C
        IF (POLTYP.EQ.'ANALYTIC') THEN
          IF (FALDA) THEN
            CALL MESSAGE(0,'ANALYTICAL KERNEL CALCULATION (ALDA)',0,0)
          ELSE IF (GALDA) THEN
            CALL MESSAGE(0,'ANALYTICAL GXC CALCULATION (ALDA)',0,0)
          ELSE IF (FDFXC) THEN
            CALL MESSAGE(0,'NUMERICAL KERNEL CALCULATION',0,0)
            IF (POLORD.GE.2) THEN
              IF (FDGXC) THEN
                PRTSTR = 'NUMERICAL KERNEL DERIVATIVE CALCULATION'
              ELSE
                PRTSTR = 'ANALYTICAL KERNEL DERIVATIVE CALCULATION'
              END IF
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              WRITE (PRTSTR,'("GXC SCREENING: ",G10.3)') GXCTHRESH
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            END IF
          ELSE
            CALL MESSAGE(0,'ANALYTICAL KERNEL CALCULATION',0,0)
          END IF
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write excited state options ***
C
      IF (TDDFT) THEN
C
        CALL MESSAGE(1,'EXCITED STATE OPTIONS',1,1)
        CALL MESSAGE(0,'A TDDFT CALCULATION WILL BE PERFORMED',0,0)
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write magnetic property options ***
C
      IF (MAGOUT.OR.MXI.OR.NMR.OR.NQR) THEN
C
        CALL MESSAGE(1,'MAGNETIC PROPERTY OPTIONS',1,1)
C
        IF (MAGOUT) CALL MESSAGE(0,'WRITE MAG OUTPUT',0,0)
        IF (MXI) CALL MESSAGE(0,'CALCULATE MAGNETIZABILITY',0,0)
        IF (NSRC) CALL MESSAGE(0,'CALCULATE SPIN-ROTATION TENSOR',0,0)
        IF (RGTEN) CALL MESSAGE(0,'CALCULATE ROTATIONAL G-TENSOR',0,0)
        IF (SHIELD) CALL MESSAGE(0,'CALCULATE NMR SHIELDING',0,0)
        IF (SSCC) THEN
          CALL MESSAGE(0,'CALCULATE NMR J-COUPLINGS',0,0)
          IF (FDFXC) THEN
            CALL MESSAGE(0,'NUMERICAL KERNEL CALCULATION',0,0)
          ELSE
            CALL MESSAGE(0,'ANALYTICAL KERNEL CALCULATION',0,0)
          END IF
          WRITE (PRTSTR,5970) ADPMAX
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,5980) ADPTOL
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (ADPTYP.EQ.'DIRECT') THEN
            PRTSTR = 'ADP METHOD: DIRECT'
          ELSE IF (ADPTYP.EQ.'ADP-I/O') THEN
            PRTSTR = 'ADP METHOD: CONVENTIONAL'
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
        IF (NQR.AND.NQRSIG(0)) THEN
          CALL MESSAGE(0,'CALCULATE NQR SPLITTING',0,0)
        ELSE IF (NQR) THEN
          CALL MESSAGE(0,'CALCULATE EFG TENSOR',0,0)
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write x-ray property options ***
C
      IF (XAS.OR.XES) THEN
C
        CALL MESSAGE(1,'X-RAY PROPERTY OPTIONS',1,1)
C
        IF (XAS) THEN
          CALL MESSAGE(0,'CALCULATE X-RAY ABSORPTION SPECTRUM',0,0)
        ELSE IF (XES) THEN
          CALL MESSAGE(0,'CALCULATE X-RAY EMISSION SPECTRUM',0,0)
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write electronic structure analysis options ***
C
      IF (BPA.OR.DOS.OR.FPA.OR.HPA.OR.
     $    LPA.OR.MPA.OR.NPA.OR.PEM.OR.VPA) THEN
C
        CALL MESSAGE(1,'ELECTRONIC ANALYSIS OPTIONS',1,1)
C
        IF (DOS) CALL MESSAGE(0,'CALCULATE DENSITY OF STATES',0,0)
        IF (BPA) CALL MESSAGE(0,'CALCULATE BADER POPULATION',0,0)
        IF (FPA) CALL MESSAGE(0,'CALCULATE BECKE POPULATION',0,0)
        IF (LPA) CALL MESSAGE(0,'CALCULATE LOEWDIN POPULATION',0,0)
        IF (MPA) CALL MESSAGE(0,'CALCULATE MULLIKEN POPULATION',0,0)
        IF (NPA) CALL MESSAGE(0,'GENERATE NBO ANALYSIS INPUT',0,0)
        IF (PEM) CALL MESSAGE(0,'CALCULATE ELECTROSTATIC MOMENTS',0,0)
        IF (VPA) CALL MESSAGE(0,'CALCULATE VORONOI POPULATION',0,0)
C
        IF (ITHFO) THEN
          PRTSTR = 'CALCULATE ITERATIVE FRACTIONAL HIRSHFELD POPULATION'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (ITHPA) THEN
          PRTSTR = 'CALCULATE ITERATIVE HIRSHFELD POPULATION'
          CALL MESSAGE(0,PRTSTR,0,0)
        ELSE IF (HPA) THEN
          PRTSTR = 'CALCULATE HIRSHFELD POPULATION'
          CALL MESSAGE(0,PRTSTR,0,0)
        END IF
CCC ADLL
        PRTSTR = 'GRID TECHNIQUE: '//GRDTTYP
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (GRDTYP.EQ.'FIXED') THEN
          PRTSTR = 'GRID TYPE: '//G98TOP
        ELSE IF (GRDTOL.NE.TINY(1.0)) THEN
          WRITE (PRTSTR,5200) GRDTTOL
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (UPTGRD.AND.RTDFT) THEN
          PRTSTR = 'UPDATE GRID DURING RT-TDDFT SIMULATIONS'
          CALL MESSAGE(0,PRTSTR,0,0)
        END IF
CCC ADLL
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write sigma-pi analysis options ***
C
      IF (SIGPI.OR.USELPI) THEN
C
        CALL MESSAGE(1,'SIGMA-PI ANALYSIS OPTIONS',1,1)
C
        IF (SIGPI) THEN
          CALL MESSAGE(0,'CALCULATE SIGMA-PI ENERGY SEPARATION',0,0)
        END IF
C
        IF (USELPI) THEN
          CALL MESSAGE(0,'LOCALIZE PI ELECTRONS IN SCF CALCULATION',0,0)
          CALL MESSAGE(0,'LOCALIZED PI ELECTRON GROUPS:',0,0)
          DO I=1,NPIGRP
            WRITE (PRTSTR,6140) (ATOMLAB(ATOMMAP(PIGRP(J),1)),
     $             ATOMMAP(PIGRP(J),1),J=LLPIGRP(I),ULPIGRP(I))
 6140       FORMAT (5(A10,'( ',I4,' )',2X))
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END DO
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write Fukui analysis options ***
C
      IF (FUKUI) THEN
C
        CALL MESSAGE(1,'ADPT FUKUI ANALYSIS OPTIONS',1,1)
        WRITE (PRTSTR,5970) ADPMAX
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        WRITE (PRTSTR,5980) ADPTOL
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (ADPTYP.EQ.'DIRECT') THEN
          PRTSTR = 'ADP METHOD: DIRECT'
        ELSE IF (ADPTYP.EQ.'ADP-I/O') THEN
          PRTSTR = 'ADP METHOD: CONVENTIONAL'
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (FALDA) THEN
          CALL MESSAGE(0,'ANALYTICAL KERNEL CALCULATION (ALDA)',0,0)
        ELSE IF (FDFXC) THEN
          CALL MESSAGE(0,'NUMERICAL KERNEL CALCULATION',0,0)
        ELSE
          CALL MESSAGE(0,'ANALYTICAL KERNEL CALCULATION',0,0)
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write plot options ***
C
      IF (PLOTTYP.NE.'OFF') THEN
C
        CALL MESSAGE(1,'PLOT/SEARCH OPTIONS ',1,1)
C
        IF (PLOTTYP.EQ.'FUNCTION') THEN
          IF (PLOTCD) THEN
            IF (USEDD) THEN
              WRITE (PRTSTR,6150) PLOTFNC//' FROM FITTED DENSITY'
            ELSE
              WRITE (PRTSTR,6160) PLOTFNC//' FROM FITTED DENSITY'
            END IF
          ELSE IF (PLOTPER.EQ.'ELECTRIC') THEN
            IF (USEDD) THEN
              WRITE (PRTSTR,6150) PLOTFNC//' FROM ELECTRIC FIELD'
            ELSE
              WRITE (PRTSTR,6160) PLOTFNC//' FROM ELECTRIC FIELD'
            END IF
          ELSE IF (PLOTPER.EQ.'MAGENTIC') THEN
            IF (USEDD) THEN
              WRITE (PRTSTR,6150) PLOTFNC//' FROM MAGNETIC FIELD'
            ELSE
              WRITE (PRTSTR,6160) PLOTFNC//' FROM MAGNETIC FIELD'
            END IF
          ELSE
            IF (USEDD) THEN
              WRITE (PRTSTR,6150) PLOTFNC
            ELSE
              WRITE (PRTSTR,6160) PLOTFNC
            END IF
          END IF 
 6150     FORMAT ('PLOT FUNCTION: DEFORMATION ',A)
 6160     FORMAT ('PLOT FUNCTION: ',A)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (PLOTDER.NE.0) THEN
            WRITE (PRTSTR,6170) PLOTDER
 6170       FORMAT ('PLOT FUNCTION DERIVATIVE: ',I2)
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
          IF (USELMO) THEN
            IF (PMLMO) THEN
              CALL MESSAGE(0,'MO LOCALIZATION: PIPEK-MEZEY',0,0)
            ELSE IF (FBLMO) THEN
              CALL MESSAGE(0,'MO LOCALIZATION: FOSTER-BOYS',0,0)
            ELSE
              CALL MESSAGE(0,'MO LOCALIZATION: CHOLESKY',0,0)
            END IF
          END IF
          IF (PLOTMOA.EQ.'LISTED') THEN
            IF (PLOTMOSA(0).GT.0) THEN
              CALL MESSAGE(0,'ALPHA PLOT MOS:',0,0)
              WRITE (OUT,'(5(1X,I5))') (PLOTMOSA(I),I=1,PLOTMOSA(0))
            END IF
          END IF
          IF (PLOTMOB.EQ.'LISTED') THEN
            IF (PLOTMOSB(0).GT.0) THEN
              CALL MESSAGE(0,'BETA PLOT MOS:',0,0)
              WRITE (OUT,'(5(1X,I5))') (PLOTMOSB(I),I=1,PLOTMOSB(0))
            END IF
          END IF
          IF (PLOTFNC.EQ.'IMF') THEN
            WRITE (PRTSTR,'(3(1X,F5.2))') (HFIELD(I),I=1,3)
            PRTSTR = 'IND. MAG. FIELD: '//PRTSTR(1:STREXT(PRTSTR))
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
        ELSE IF (PLOTTYP.EQ.'ISOSURFACE') THEN
          IF (PLOTCD) THEN
            WRITE (PRTSTR,6180) PLOTFNC//' FROM FITTED DENSITY'
          ELSE IF (PLOTPER.EQ.'ELECTRIC') THEN
            WRITE (PRTSTR,6180) PLOTFNC//' FROM ELECTRIC FIELD'
          ELSE IF (PLOTPER.EQ.'MAGENTIC') THEN
            WRITE (PRTSTR,6180) PLOTFNC//' FROM MAGNETIC FIELD'
          ELSE
            WRITE (PRTSTR,6180) PLOTFNC
          END IF
 6180     FORMAT ('ISOSURFACE FUNCTION: ',A)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (PLOTMOA.EQ.'LISTED') THEN
            IF (PLOTMOSA(0).GT.0) THEN
              CALL MESSAGE(0,'ALPHA PLOT MOS:',0,0)
              WRITE (OUT,'(5I5)') (PLOTMOSA(I),I=1,PLOTMOSA(0))
            END IF
          END IF
          IF (PLOTMOB.EQ.'LISTED') THEN
            IF (PLOTMOSB(0).GT.0) THEN
              CALL MESSAGE(0,'BETA PLOT MOS:',0,0)
              WRITE (OUT,'(5I5)') (PLOTMOSB(I),I=1,PLOTMOSB(0))
            END IF
          END IF
          IF (ISOABS) THEN
            WRITE (PRTSTR,6190) ISOVAL
 6190       FORMAT ('ISOSURFACE VALUE: |',G10.3,'| A.U.')
          ELSE
            WRITE (PRTSTR,6200) ISOVAL
 6200       FORMAT ('ISOSURFACE VALUE: ',G10.3,' A.U.')
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,6210) ISOINT
 6210     FORMAT ('ISOSURFACE INTERPOLATION: ',A)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,6220) ISOTOL
 6220     FORMAT ('DATA REDUCTION PARAMETER: ',G10.3)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (PLOTTYP.EQ.'GEOSURFACE') THEN
          IF (PLOTFNC.EQ.'SPHERE') THEN
            WRITE (PRTSTR,6230) UNIT(1)
 6230       FORMAT ('SPHERICAL SURFACE ( IN ',A,' )')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          ELSE IF (PLOTFNC.EQ.'ELLIPSOID') THEN
            WRITE (PRTSTR,6240) UNIT(1)
 6240       FORMAT ('ELLIPSOID SURFACE ( IN ',A,' )')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
          WRITE (PRTSTR,6250) (ANGBOR(1)*GEORIGIN(I),I=1,3)
 6250     FORMAT ('ORIGIN: ',3(F10.3,1X))
          CALL MESSAGE(0,PRTSTR,0,0)
          IF (PLOTFNC.EQ.'SPHERE') THEN
            RADIUS = SQRT(DOT_PRODUCT(GEOAXES(1,:),GEOAXES(1,:)))
            WRITE (PRTSTR,6260) ANGBOR(1)*RADIUS
 6260       FORMAT ('RADIUS: ',F10.3)
            CALL MESSAGE(0,PRTSTR,0,0)
          ELSE IF (PLOTFNC.EQ.'ELLIPSOID') THEN
            WRITE (PRTSTR,6270) (ANGBOR(1)*GEOAXES(1,I),I=1,3)
 6270       FORMAT ('AXIS 1: ',3(F10.3,1X))
            CALL MESSAGE(0,PRTSTR,0,0)
            WRITE (PRTSTR,6280) (ANGBOR(1)*GEOAXES(2,I),I=1,3)
 6280       FORMAT ('AXIS 2: ',3(F10.3,1X))
            CALL MESSAGE(0,PRTSTR,0,0)
            WRITE (PRTSTR,6290) (ANGBOR(1)*GEOAXES(3,I),I=1,3)
 6290       FORMAT ('AXIS 3: ',3(F10.3,1X))
            CALL MESSAGE(0,PRTSTR,0,0)
          END IF
          WRITE (PRTSTR,6300) ISOINT
 6300     FORMAT ('GEOSURFACE INTERPOLATION: ',A)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,6310) ISOTOL
 6310     FORMAT ('DATA REDUCTION PARAMETER: ',G10.3)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (PLOTTYP.EQ.'CPSEARCH') THEN
          IF (PLOTCD) THEN
            IF (USEDD) THEN
              WRITE (PRTSTR,6320) PLOTFNC//' FROM FITTED DENSITY'
            ELSE
              WRITE (PRTSTR,6330) PLOTFNC//' FROM FITTED DENSITY'
            END IF
          ELSE
            IF (USEDD) THEN
              WRITE (PRTSTR,6320) PLOTFNC
            ELSE
              WRITE (PRTSTR,6330) PLOTFNC
            END IF
          END IF
 6320     FORMAT ('SEARCH MOLECULAR FIELD: DEFORMATION ',A)
 6330     FORMAT ('SEARCH MOLECULAR FIELD: ',A)
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (PLOTMOA.EQ.'LISTED') THEN
            IF (PLOTMOSA(0).GT.0) THEN
              CALL MESSAGE(0,'ALPHA MOS:',0,0)
              WRITE (OUT,'(5I5)') (PLOTMOSA(I),I=1,PLOTMOSA(0))
            END IF
          END IF
          IF (PLOTMOB.EQ.'LISTED') THEN
            IF (PLOTMOSB(0).GT.0) THEN
              CALL MESSAGE(0,'BETA MOS:',0,0)
              WRITE (OUT,'(5I5)') (PLOTMOSB(I),I=1,PLOTMOSB(0))
            END IF
          END IF
        END IF
C
        IF (PLOT.EQ.'TABLE') THEN
          PRTSTR = 'PLOT OUTPUT: FUNCTION TABLE IN OUTPUT FILE'
        ELSE IF (PLOT.EQ.'ASCII') THEN
          PRTSTR = 'PLOT OUTPUT: EXTERNAL ASCII FILE (VU FORMAT)'
        ELSE IF (PLOT.EQ.'BINARY') THEN
          PRTSTR = 'PLOT OUTPUT: EXTERNAL BINARY FILE (VU FORMAT)'
        ELSE IF (PLOT.EQ.'CUBE') THEN
          PRTSTR = 'PLOT OUTPUT: EXTERNAL ASCII FILE (CUBE FORMAT)'
        ELSE
          PRTSTR = 'PLOT OUTPUT: UNDEFINED'
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
C
        IF (LATTYP.EQ.'STRUCTURED') THEN
          WRITE (PRTSTR,6340) EUCLID,UNIT(1)
 6340     FORMAT (I1,'D STRUCTURED PLOT LATTICE USED ( IN ',A,' )')
          CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
          WRITE (PRTSTR,6350) (ANGBOR(1)*LATTICE(I,0),I=1,3)
 6350     FORMAT ('LATTICE ORIGIN : ',3(F10.3,1X)) 
          CALL MESSAGE(0,PRTSTR,0,0)
          DO I=1,EUCLID
            VECTOR(:) = ANGBOR(1)*(LATTICE(:,I)-LATTICE(:,0))
            WRITE (PRTSTR,6360) I,(VECTOR(J),J=1,3),I,NLATTICE(I)
 6360       FORMAT ('LATTICE  AXIS ',I1,': ',3(F10.3,1X),2X,
     $              'N',I1,' = ',I4)
            CALL MESSAGE(0,PRTSTR,0,0)
          END DO
        ELSE IF (LATTYP.EQ.'POINTS') THEN
          WRITE (PRTSTR,6370) UNIT(1)
 6370     FORMAT ('UNSTRUCTURED PLOT LATTICE USED ( IN ',A,' )')
          CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
          IF (LATINP.EQ.'INTERNAL') THEN
            PRTSTR = 'LATTICE INPUT: INPUT FILE (POINTS)'
          ELSE IF (LATINP.EQ.'EXTERNAL ASCII') THEN
            PRTSTR = 'LATTICE INPUT: LAT.asc FILE'
          ELSE IF (LATINP.EQ.'EXTERNAL BINARY') THEN
            PRTSTR = 'LATTICE INPUT: LAT.bin FILE'
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (LATINP.EQ.'EXTERNAL BINARY') THEN
            CALL BINARY(VECTOR,3,3,1,LAT,'LAT.bin','READ')
            WRITE (PRTSTR,6380) INT(VECTOR(1))
 6380       FORMAT ('NUMBER OF GRID POINTS : ',I9)
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            WRITE (PRTSTR,6390) INT(VECTOR(2))
 6390       FORMAT ('NUMBER OF FACETS : ',I9)
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
        END IF
C
      END IF
C
CCC ADLL START 
C     ------------------------------------------------------------------
C
C     *** Write electron dynamics options ***
C
      IF (RTDFT) THEN
        CALL MESSAGE(1,'REAL TIME PROPAGATION OPTIONS',1,1)
C
        IF (RTRST) CALL MESSAGE(0,'A SIMULATION IS RESTARTED',0,0)
        IF (IMTPR) CALL MESSAGE(0,'PROPAGATION WITH IMAGINARY TIME',0,0)
        WRITE (PRTSTR,16280) RTMAX
16280   FORMAT('NUMBER OF PROPAGATION STEPS: ',I10)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        WRITE (PRTSTR,16290) RTSTEP/FSEC
16290   FORMAT ('ELECTRONIC PROPAGATION TIME STEP: ',F10.5' FSEC')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C        
        IF (RTLSTEP.NE.0) THEN
          WRITE (PRTSTR,16295) RTLSTEP
16295     FORMAT ('PROJECTILE DISPLACEMENT PER STEP ',F10.5' BOHR')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        END IF
C
        IF (PROPAGA.EQ.'EULER') THEN
          WRITE (PRTSTR,16300)
16300     FORMAT ('PROPAGATION METHOD: EULER')
        ELSE IF ((PROPAGA.EQ.'MAGNUS').AND.MAGPC) THEN
          WRITE (PRTSTR,16310)
16310     FORMAT ('PROPAGATION METHOD: MAGNUS WITH PREDICTOR-CORRECTOR')
        ELSE IF ((PROPAGA.EQ.'MAGNUS').AND.MAGIT) THEN
          WRITE (PRTSTR,16320)
16320     FORMAT ('PROPAGATION METHOD: SELF CONSISTENT MAGNUS')
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        IF (PROPAGA.NE.'MAGNUS') GO TO 16380
        IF (EXPMAT.EQ.'BCH') THEN
          WRITE (PRTSTR,16330)
16330     FORMAT ('MATRIX EXPONENTATION: BAKER-CAMPBELL-HAUSSDORFF')
        ELSE IF (EXPMAT.EQ.'TAYLOR') THEN
          WRITE (PRTSTR,16340)
16340     FORMAT ('MATRIX EXPONENTATION: TAYLOR EXPANSION')
        ELSE IF (EXPMAT.EQ.'CHEBYSHEV') THEN
          WRITE (PRTSTR,16345)
16345     FORMAT ('MATRIX EXPONENTATION: CHEBYSHEV EXPANSION')
        ELSE IF (EXPMAT.EQ.'DIAGO') THEN
          WRITE (PRTSTR,16350)
16350     FORMAT ('MATRIX EXPONENTATION: DIAGONALIZATION')
        END IF
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF ((EXPMAT.EQ.'BCH').OR.(EXPMAT.EQ.'TAYLOR').OR.
     $      (EXPMAT.EQ.'CHEBYSHEV')) THEN
          IF (NEXPM.LT.0) THEN
           WRITE (PRTSTR,16351) CHEBTOL
16351      FORMAT ('EXPANSION TOLERANCE: ',G10.3)
           CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
           WRITE (PRTSTR,16352) -NEXPM
           CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
16352      FORMAT('MAXIMUM ORDER OF EXPANSION OF THE SERIE: ',I10)
          ELSE
            WRITE (PRTSTR,16360) NEXPM
16360       FORMAT('ORDER OF EXPANSION OF THE SERIE: ',I10)
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
        END IF
      END IF
16380 CONTINUE
C
      IF (EFIELD.AND.EFRTP) THEN
        WRITE (PRTSTR,16390) (ELFIELD(I),I=1,3)
16390   FORMAT ('LASER FIELD COMPONENTS ( ',
     $           G10.3,' , ',G10.3,' , ',G10.3,' ) A.U.')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        IF (FSHAPE.EQ.'KICK') THEN
          WRITE (PRTSTR,16400)
16400     FORMAT('INFINITENLY NARROW ELECTRIC KICK')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        ELSE IF (FSHAPE.EQ.'PULSE') THEN
          WRITE (PRTSTR,16410)
16410     FORMAT('GAUSSIAN SHAPED ELECTRIC PULSE')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,16420) EFCT/FSEC
16420     FORMAT('PULSE CENTERED AT: ',F10.3,' FSEC')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,16430) EFWT/FSEC
16430     FORMAT('PULSE WIDTH: ',F10.3,' FSEC')
          IF (EFPUL.NE.0.0) THEN
            WRITE (PRTSTR,16440) EFPUL*EVOLT
16440       FORMAT('WAVE ENERGY: ',F6.3,' EV')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
        ELSE IF (FSHAPE.EQ.'LRAMP') THEN
          WRITE (PRTSTR,16450)
16450     FORMAT('LINEAR RAMPED ELECTRIC PULSE')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          WRITE (PRTSTR,16460) EFCT/FSEC
16460     FORMAT('MAXIMUM FIELD STRENGTH AT: ',F10.3,' FSEC')
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (EFPUL.NE.0.0) THEN
            WRITE (PRTSTR,16470) EFPUL*EVOLT
16470       FORMAT('WAVE ENERGY: ',F6.3,' EV')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
        END IF
      END IF
C
      IF (ADDRAD) THEN
        CALL MESSAGE(1,'PROJECTILE OPTIONS',1,1)
        IF (RTDFT.AND.(.NOT.MD)) THEN
          CALL MESSAGE(0,'COLLISION TYPE: INELASTIC',0,0)
        END IF
        IF (USELWP) THEN
          CALL MESSAGE(0,'A LIENARD-WIECHERT POTENTIAL WILL BE USED'
     $                 ,0,0)
        ELSE
          CALL MESSAGE(0,'A COULOMB POTENTIAL WILL BE USED',0,0)
        END IF
        MC2 = MRAD*AMU*(CLIGHT/(ABOHR*FSEC*1.E15))**2
        CLAU = CLIGHT/(ABOHR*FSEC*1.E15)
        BETA = SQRT(1 - (KINRAD/MC2 + 1)**(-2)) 
C
        WRITE (PRTSTR,16500) KINRAD*EVOLT/10**6,BETA
16500   FORMAT ('PROJECTILE KINETIC ENERGY (MeV):',F9.5,' , ',F8.6,' c')
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        WRITE (PRTSTR,16510) MRAD
16510   FORMAT ('PROJECTILE MASS:',F8.3)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
        WRITE (PRTSTR,16520) QIORAD
16520   FORMAT ('PROJECTILE CHARGE:',F8.3)
        CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
        IF ((.NOT.USELWP).AND.(BETA.GE.0.1)) THEN
          PRTSTR = 'WARNING : THE PROJECTILE SPEED APPROACHES THE SPEED
     $ OF LIGHT !! IT IS ADVISED TO USE A LIENAR WIERCHET POTENTIAL'
          CALL MESSAGE(2,STRCOMP(PRTSTR),1,0)
        END IF
      END IF
C
CCC ADLL END 
C     ------------------------------------------------------------------
C
C     *** Write print options ***
C
      IF (PRTAMAT.OR.PRTAUX.OR.PRTBAS.OR.PRTC6.OR.PRTCIS.OR.
     $    PRTCON.OR.PRTDBG.OR.PRTDE2.OR.PRTECP.OR.PRTEMBED.OR.
     $    PRTERIS.OR.PRTFIT.OR.PRTFUK.OR.PRTGRD.OR.PRTGTO.OR.
     $    PRTKS.OR.PRTLMAT.OR.PRTMAG.OR.PRTMCP.OR.PRTMD.OR.
     $    PRTMOE.OR.PRTMOS.OR.PRTNMR.OR.PRTNQR.OR.PRTOPT.OR.
     $    PRTPMAT.OR.PRTPOP.OR.PRTPRI.OR.PRTSAD.OR.PRTSCF.OR.
     $    PRTRAM.OR.PRTRMAT.OR.PRTSMAT.OR.PRTSTO.OR.PRTSYM.OR.
     $    PRTTB.OR.PRTTD.OR.PRTTMAT.OR.PRTXCE.OR.PRTXCV.OR.PRTXRAY) THEN
C
        IF (PRTDBG) THEN
          CALL MESSAGE(1,'DEBUG OUTPUT',1,1)
        ELSE
          CALL MESSAGE(1,'PRINT OPTIONS',1,1)
        END IF
C
        IF (PRTGTO) THEN
          CALL MESSAGE(0,'PRINT GTO TABLE',0,0)
        END IF
        IF (PRTSTO) THEN
          CALL MESSAGE(0,'PRINT STO TABLE',0,0)
        END IF
        IF (PRTECP) THEN
          CALL MESSAGE(0,'PRINT ECP TABLE',0,0)
        END IF
        IF (PRTMCP) THEN
          CALL MESSAGE(0,'PRINT MCP TABLE',0,0)
        END IF
        IF (PRTAUX) THEN
          CALL MESSAGE(0,'PRINT AUXILIARY FUNCTION TABLE',0,0)
        END IF
        IF (PRTC6) THEN
          CALL MESSAGE(0,'PRINT C6 COEFFICIENTS',0,0)
        END IF
        IF (PRTSMAT) THEN
          CALL MESSAGE(0,'PRINT OVERLAP MATRIX',0,0)
        END IF
        IF (PRTLMAT) THEN
          CALL MESSAGE(0,'PRINT ORTHOGONALIZATION MATRIX',0,0)
        END IF
        IF (PRTAMAT) THEN
          CALL MESSAGE(0,'PRINT FITTED DENSITY ERI MATRIX',0,0)
        END IF
        IF (PRTFIT) THEN
          CALL MESSAGE(0,'PRINT ITERATIVE FITTING',0,0)
        END IF
        IF (PRTSCF) THEN
          CALL MESSAGE(0,'PRINT SCF CONVERGENCE',0,0)
        END IF
        IF (PRTTMAT) THEN
          CALL MESSAGE(0,'PRINT KINETIC ENERGY MATRIX',0,0)
        END IF
        IF (PRTKS) THEN
          CALL MESSAGE(0,'PRINT KOHN-SHAM MATRIX',0,0)
        END IF
        IF (PRTERIS) THEN
          CALL MESSAGE(0,'PRINT ERI MATRICES',0,0)
        END IF
        IF (PRTMOS) THEN
          CALL MESSAGE(0,'PRINT MOLECULAR ORBITALS',0,0)
        END IF
        IF (PRTMOE) THEN
          CALL MESSAGE(0,'PRINT MOLECULAR ORBITAL ENERGIES',0,0)
        END IF
        IF (PRTPMAT) THEN
          CALL MESSAGE(0,'PRINT DENSITY MATRIX',0,0)
        END IF
        IF (PRTGRD) THEN
          CALL MESSAGE(0,'PRINT GRID INFORMATION',0,0)
        END IF
        IF (PRTXCE) THEN
          CALL MESSAGE(0,'PRINT EXCHANGE-CORRELATION ENERGY',0,0)
        END IF
        IF (PRTXCV) THEN
          PRTSTR = 'PRINT EXCHANGE-CORRELATION POTENTIAL MATRIX'
          IF (SCFOUT.GT.0) CALL MESSAGE(0,PRTSTR,0,0)
        END IF
        IF (EMBED.AND.PRTEMBED) THEN
          CALL MESSAGE(0,'PRINT EMBEDDING CUBE COORDINATES',0,0)
        END IF
        IF (PRTSYM) THEN
          CALL MESSAGE(0,'PRINT SYMMETRY INFORMATION',0,0)
        END IF
        IF (PRTBAS) THEN
          CALL MESSAGE(0,'PRINT BASIS SET CONTRACTION',0,0)
        END IF
        IF (PRTTB) THEN
          CALL MESSAGE(0,'PRINT TIGHT-BINDING INFORMATION',0,0)
        END IF
        IF (PRTRAM) THEN
          CALL MESSAGE(0,'PRINT RAM ALLOCATION',0,0)
        END IF
        IF (PRTPRI) THEN
          CALL MESSAGE(0,'PRINT PRIMITIVE COORDINATES',0,0)
        END IF
        IF (PRTOPT.AND.PRTMD) THEN
          CALL MESSAGE(0,'PRINT MOLECULAR DYNAMICS DEBUG OUTPUT',0,0)
        ELSE IF (PRTMD) THEN
          CALL MESSAGE(0,'PRINT MOLECULAR DYNAMICS INFORMATION',0,0)
        ELSE IF (PRTOPT) THEN
          CALL MESSAGE(0,'PRINT OPTIMIZATION INFORMATION',0,0)
        END IF
        IF (PRTDE2) THEN
          CALL MESSAGE(0,'PRINT HESSIAN',0,0)
        END IF
        IF (PRTSAD) THEN
          CALL MESSAGE(0,'PRINT SADDLE DEBUG OUTPUT',0,0)
        END IF
        IF (PRTCON) THEN
          CALL MESSAGE(0,'PRINT MOLECULAR DYNAMICS CONSTRAINTS',0,0)
        END IF
        IF (PRTRMAT) THEN
          CALL MESSAGE(0,'PRINT RESPONSE MATRIX',0,0)
        END IF
        IF (PRTTD) THEN
          CALL MESSAGE(0,'PRINT TDDFT DEBUG OUTPUT',0,0)
        END IF
        IF (PRTCIS) THEN
          CALL MESSAGE(0,'PRINT ADPT CIS DEBUG OUTPUT',0,0)
        END IF
        IF (PRTMAG) THEN
          CALL MESSAGE(0,'PRINT MAGNETIZABILITY DEBUG OUTPUT',0,0)
        END IF
        IF (PRTNMR) THEN
          CALL MESSAGE(0,'PRINT NMR DEBUG OUTPUT',0,0)
        END IF
        IF (PRTNQR) THEN
          CALL MESSAGE(0,'PRINT NQR DEBUG OUTPUT',0,0)
        END IF
        IF (PRTXRAY) THEN
          CALL MESSAGE(0,'PRINT XAS/XES DEBUG OUTPUT',0,0)
        END IF
        IF (PRTPOP) THEN
          CALL MESSAGE(0,'PRINT POPULATION DENSITY MATRIX',0,0)
        END IF
        IF (PRTFUK) THEN
          CALL MESSAGE(0,'PRINT FUKUI FUNCTION MATRIX',0,0)
        END IF
C
      END IF
C
      IF (.NOT.PRTGEO) THEN
        CALL MESSAGE(0,'GEOMETRY PRINTING IS SUPPRESSED',0,0)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Write output options ***
C
      IF (PRTFIT.OR.PRTSAD.OR.PRTSCF) THEN
C
        CALL MESSAGE(1,'OUTPUT OPTIONS',1,1)
C
        IF (PRTFIT) THEN
          CALL MESSAGE(0,'ITERATIVE FIT OUTPUT NOT REMOVED',0,0)
        END IF
        IF (PRTSAD) THEN
          CALL MESSAGE(0,'SADDLE OPTIMIZATIONS NOT REMOVED',0,0)
        END IF
        IF (PRTSCF) THEN
          CALL MESSAGE(0,'SCF CYCLES NOT REMOVED',0,0)
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Activated interfaces ***
C
      IF (PORT) THEN
C
        CALL MESSAGE(1,'ACTIVATED INTERFACE PORTS',1,1)
C
        IF (MOLDEN.NE.'NONE') THEN
          CALL MESSAGE(0,'FILE deMon.mol WILL BE WRITTEN',0,0)
        END IF
C
        IF (MOLEKEL.NE.'NONE') THEN
          CALL MESSAGE(0,'FILE deMon.mkl WILL BE WRITTEN',0,0)
        END IF
C
        IF (WFNFILE.NE.'NONE') THEN
          CALL MESSAGE(0,'FILE deMon.wfn WILL BE WRITTEN',0,0)
        END IF
C
        IF (PLOT.EQ.'BINARY') THEN
          CALL MESSAGE(0,'FILE deMon.pie WILL BE WRITTEN',0,0)
        END IF
C
        IF (CHARMM) THEN
          CALL MESSAGE(0,'FILE deMon.qmm WILL BE WRITTEN',0,0)
        END IF
C
        IF (NPA) THEN
          CALL MESSAGE(0,'FILE deMon.nbo WILL BE WRITTEN',0,0)
        END IF
C
      END IF
C
C     -----------------------------------------------------------------
C
C     *** LibXC functional information ***
C
      IF (LXC) THEN
        CALL MESSAGE(1,'LIBXC VERSION AND FUNCTIONAL INFORMATION',1,1)
        CALL LXC_PRINTXCF()
      END IF
C
C     ------------------------------------------------------------------
C
C     *** No molecular geometry output ***
C
      IF ((IRCTYP.EQ.'READ STEP').OR.(TRJTYP.EQ.'READ STEP')) THEN
C
C     *** Flush output ***
C
        CALL DEFLUSH(OUT)
C
C     *** Write molecular geometry ***
C
      ELSE IF ((TRJTYP.EQ.'NEW').OR.(TRJTYP.EQ.'RESTART')) THEN
C
        CALL MESSAGE(1,'GEOMETRY',2,0)
C
C     *** Cartesian coordinate input ***
C
        IF (INPTYP.EQ.'CARTESIAN') THEN
C
C     *** Optimization in redundant coordinates ***
C
          IF (OPTTYP.EQ.'REDUNDANT') THEN
C
            CALL PRICAR('ORIGINAL INPUT ORIENTATION',4)
            IF (EMBED.AND.PRTEMBED) THEN
              PRTSTR = 'ORIGINAL INPUT ORIENTATION OF EMBEDDING CUBE'
              CALL PRICUB(PRTSTR,4)
            END IF
            CALL PRIZMAT('GENERATED Z-MATRIX',3)
            CALL PRICAR('GENERATED Z-MATRIX ORIENTATION',1)
            IF (EMBED.AND.PRTEMBED) THEN
              PRTSTR = 'GENERATED ORIENTATION OF EMBEDDING CUBE'
              CALL PRICUB(PRTSTR,1)
            END IF
C
C     *** No optimization or optimization in Cartesian coordinates ***
C
          ELSE
C
            CALL PRICAR('INPUT ORIENTATION',1)
            IF (EMBED.AND.PRTEMBED) THEN
              PRTSTR = 'INPUT ORIENTATION OF EMBEDDING CUBE'
              CALL PRICUB(PRTSTR,1)
            END IF
C
          END IF
C
C     *** Internal coordinate input ***
C
        ELSE IF (INPTYP.EQ.'INTERNAL') THEN
C
          CALL PRIZMAT('Z-MATRIX INPUT',3)
          CALL PRICAR('Z-MATRIX INPUT ORIENTATION',1)
          IF (ZBUILD) THEN
            CALL PRIZMAT('GENERATED Z-MATRIX',3)
            CALL PRICAR('GENERATED Z-MATRIX ORIENTATION',1)
          END IF
C
C     *** Mixed coordinate input ***
C
        ELSE IF (INPTYP.EQ.'MIXED') THEN
C
C     *** Optimization in generated redundant coordinates ***
C
          IF (ZBUILD.AND.(OPTTYP.EQ.'REDUNDANT')) THEN
C
            CALL PRICAR('ORIGINAL INPUT ORIENTATION',4)
            IF (EMBED.AND.PRTEMBED) THEN
              PRTSTR = 'ORIGINAL INPUT ORIENTATION OF EMBEDDING CUBE'
              CALL PRICUB(PRTSTR,4)
            END IF
            CALL PRIZMAT('GENERATED Z-MATRIX',3)
            CALL PRICAR('GENERATED Z-MATRIX ORIENTATION',1)
            IF (EMBED.AND.PRTEMBED) THEN
              PRTSTR = 'GENERATED ORIENTATION OF EMBEDDING CUBE'
              CALL PRICUB(PRTSTR,1)
            END IF
C
C     *** Optimization in internal or redundant coordinates ***
C
          ELSE IF ((OPTTYP.EQ.'INTERNAL').OR.
     $             (OPTTYP.EQ.'REDUNDANT')) THEN
C
            CALL PRICAR('INPUT ORIENTATION',1)
            IF (EMBED.AND.PRTEMBED) THEN
              PRTSTR = 'INPUT ORIENTATION OF EMBEDDING CUBE'
              CALL PRICUB(PRTSTR,1)
            END IF
            CALL PRIZMAT('GENERATED Z-MATRIX',3)
C
C     *** No optimization or optimization in Cartesian coordinates ***
C
          ELSE
C
            CALL PRICAR('INPUT ORIENTATION',1)
            IF (EMBED.AND.PRTEMBED) THEN
              PRTSTR = 'INPUT ORIENTATION OF EMBEDDING CUBE'
              CALL PRICUB(PRTSTR,1)
            END IF
C
          END IF
C
        END IF
C
        IF (PRTSYM) CALL PRIDIST('INPUT DISTANCE MATRIX')
C
CCC ADLL QMMM
        IF (PRTMMTOP) CALL PRTMM
CCC ADLL LA
C
C     *** Print summary of QM/MM frontier treatment ***
C
        IF (QMMM.AND.LINKAT) CALL PRTQMMMBRD
CCC ADLL LA
C
C     *** Flush output ***
C
        CALL DEFLUSH(OUT)
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE HEADER ***
C
      END
