      SUBROUTINE G98GRDABS(GRID)
C
C     Purpose: Generate fixed grids of G98 with aggressive pruning
C
C     Lit.: P.M.W. Gill et al., Chem. Phys. Lett. 209, 506 (1993)
C           M. Krack, A.M. Koster, J. Chem. Phys. 108, 3226 (1998)
C
C     History: - Creation (14.11.96, MK)
C                         (13.10.15, AMK)
C
C     ******************************************************************
C
C     SPHRAD: Radius of actual sphere.
C     U     : Abscissas of radial quadrature.
C     WR    : Weights of radial quadrature.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'rtdft.h'
C
      CHARACTER*(*) GRID
C
      CHARACTER*80 PRTSTR,STRCOMP
      REAL SPHRAD
      INTEGER IATOM,IRAD
C
      REAL U(MAXRAD),WR(MAXRAD)
C
C     ------------------------------------------------------------------
C
      IF (GRID.EQ.'(50,194)p') THEN
C
C     *** Pruned (50,194) grid of G98 (SG-1) ***
C
        IF (PRTGRD.AND.(STATUS.EQ.'SCF')) THEN
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL MESSAGE(1,'PRUNED (50,194) GRID OF G98 (COARSE)',1,0)
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MESSAGE(1,'PRUNED (50,194) MURA-KNOWLES GRID',1,0)
          END IF
        END IF
C
        DO IATOM=1,NATOM
C
          NRADIAL(IATOM) = 50
C
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL EULMAC(U,WR,NRADIAL(IATOM))
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MURKNO(IATOM,U,WR,NRADIAL(IATOM))
          ELSE
            PRTSTR = 'INVALID RADIAL QUADRATURE FOR FIXED GRID'
            CALL ERRMSG('G98GRD',STRCOMP(PRTSTR),1)
          END IF
C
          CALL LOCAL(IATOM)
C
          DO IRAD=1,NRADIAL(IATOM)
C
            IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
              SPHRAD = G98RAD(ATOMNO(IATOM))*
     $                 (U(IRAD)/(1.0 - U(IRAD)))**2
            ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
              SPHRAD = U(IRAD)
            END IF
C
            IF ((SPHRAD.LT.P3ABS).AND.(ZATOM(IATOM).EQ.0)) THEN
              LRADIAL(IATOM,IRAD) = 0
            ELSE IF (ATOMNO(IATOM).LE.2) THEN
              IF (SPHRAD.LE.(0.25*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 3
              ELSE IF (SPHRAD.LE.(0.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 9
              ELSE IF (SPHRAD.LE.G98RAD(ATOMNO(IATOM))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(4.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE
                LRADIAL(IATOM,IRAD) = 15
              END IF
            ELSE IF (ATOMNO(IATOM).LE.10) THEN
              IF (SPHRAD.LE.(0.1667*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 3
              ELSE IF (SPHRAD.LE.(0.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 9
              ELSE IF (SPHRAD.LE.(0.9*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(3.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE
                LRADIAL(IATOM,IRAD) = 15
              END IF
            ELSE IF (ATOMNO(IATOM).LE.18) THEN
              IF (SPHRAD.LE.(0.1*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 3
              ELSE IF (SPHRAD.LE.(0.4*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 9
              ELSE IF (SPHRAD.LE.(0.8*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(2.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE
                LRADIAL(IATOM,IRAD) = 15
              END IF
            ELSE IF (ATOMNO(IATOM).LE.36) THEN
              IF (SPHRAD.LE.(0.02*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 3
              ELSE IF (SPHRAD.LE.(0.1*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 9
              ELSE IF (SPHRAD.LE.(0.2*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(3.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE
                LRADIAL(IATOM,IRAD) = 15
              END IF
            ELSE
              LRADIAL(IATOM,IRAD) = 23
            END IF
C
          END DO
C
        END DO
C
      ELSE IF (GRID.EQ.'(75,302)p') THEN
C
C     *** Pruned (75,302) default grid of G98 (FineGrid) ***
C
        IF (PRTGRD.AND.(STATUS.EQ.'SCF')) THEN
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL MESSAGE(1,'PRUNED (75,302) GRID OF G98 (MEDIUM)',1,0)
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MESSAGE(1,'PRUNED (75,302) MURA-KNOWLES GRID',1,0)
          END IF
        END IF
C
        DO IATOM=1,NATOM
C
          NRADIAL(IATOM) = 75
C
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL EULMAC(U,WR,NRADIAL(IATOM))
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MURKNO(IATOM,U,WR,NRADIAL(IATOM))
          ELSE
            PRTSTR = 'INVALID RADIAL QUADRATURE FOR FIXED GRID'
            CALL ERRMSG('G98GRD',STRCOMP(PRTSTR),1)
          END IF
C
          CALL LOCAL(IATOM)
C
          DO IRAD=1,NRADIAL(IATOM)
C
            IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
              SPHRAD = G98RAD(ATOMNO(IATOM))*
     $                 (U(IRAD)/(1.0 - U(IRAD)))**2
            ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
              SPHRAD = U(IRAD)
            END IF
C
            IF ((SPHRAD.LT.P3ABS).AND.(ZATOM(IATOM).EQ.0)) THEN 
              LRADIAL(IATOM,IRAD) = 0
            ELSE IF (ATOMNO(IATOM).LE.2) THEN
              IF (SPHRAD.LE.(0.25*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 7
              ELSE IF (SPHRAD.LE.(0.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.G98RAD(ATOMNO(IATOM))) THEN
                LRADIAL(IATOM,IRAD) = 19
              ELSE IF (SPHRAD.LE.(4.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE
                LRADIAL(IATOM,IRAD) = 19
              END IF
            ELSE IF (ATOMNO(IATOM).LE.10) THEN
              IF (SPHRAD.LE.(0.1667*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 7
              ELSE IF (SPHRAD.LE.(0.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(0.9*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 19
              ELSE IF (SPHRAD.LE.(3.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE
                LRADIAL(IATOM,IRAD) = 19
              END IF
            ELSE IF (ATOMNO(IATOM).LE.18) THEN
              IF (SPHRAD.LE.(0.1*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 7
              ELSE IF (SPHRAD.LE.(0.4*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(0.8*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 19
              ELSE IF (SPHRAD.LE.(2.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE
                LRADIAL(IATOM,IRAD) = 19
              END IF
            ELSE IF (ATOMNO(IATOM).LE.36) THEN
              IF (SPHRAD.LE.(0.02*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 7
              ELSE IF (SPHRAD.LE.(0.1*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(0.2*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 19
              ELSE IF (SPHRAD.LE.(3.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE
                LRADIAL(IATOM,IRAD) = 19
              END IF
            ELSE
              LRADIAL(IATOM,IRAD) = 29
            END IF
C
          END DO
C
        END DO
C
      ELSE IF (GRID.EQ.'(99,590)p') THEN
C
C     *** Pruned (99,590) grid of G98 (UltraFineGrid) ***
C
        IF (PRTGRD.AND.(STATUS.EQ.'SCF')) THEN
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL MESSAGE(1,'PRUNED (99,590) GRID OF G98 (FINE)',1,0)
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MESSAGE(1,'PRUNED (99,590) MURA-KNOWLES GRID',1,0)
          END IF
        END IF
C
        DO IATOM=1,NATOM
C
          NRADIAL(IATOM) = 99
C
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL EULMAC(U,WR,NRADIAL(IATOM))
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MURKNO(IATOM,U,WR,NRADIAL(IATOM))
          ELSE
            PRTSTR = 'INVALID RADIAL QUADRATURE FOR FIXED GRID'
            CALL ERRMSG('G98GRD',STRCOMP(PRTSTR),1)
          END IF
C
          CALL LOCAL(IATOM)
C
          DO IRAD=1,NRADIAL(IATOM)
C
            IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
              SPHRAD = G98RAD(ATOMNO(IATOM))*
     $                 (U(IRAD)/(1.0 - U(IRAD)))**2
            ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
              SPHRAD = U(IRAD)
            END IF
C
            IF ((SPHRAD.LT.P3ABS).AND.(ZATOM(IATOM).EQ.0)) THEN
              LRADIAL(IATOM,IRAD) = 0
            ELSE IF (ATOMNO(IATOM).LE.2) THEN
              IF (SPHRAD.LE.(0.25*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(0.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE IF (SPHRAD.LE.G98RAD(ATOMNO(IATOM))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE IF (SPHRAD.LE.(4.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 39
              ELSE
                LRADIAL(IATOM,IRAD) = 29
              END IF
            ELSE IF (ATOMNO(IATOM).LE.10) THEN
              IF (SPHRAD.LE.(0.1667*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(0.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE IF (SPHRAD.LE.(0.9*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE IF (SPHRAD.LE.(3.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 39
              ELSE
                LRADIAL(IATOM,IRAD) = 29
              END IF
            ELSE IF (ATOMNO(IATOM).LE.18) THEN
              IF (SPHRAD.LE.(0.1*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(0.4*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE IF (SPHRAD.LE.(0.8*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE IF (SPHRAD.LE.(2.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 39
              ELSE
                LRADIAL(IATOM,IRAD) = 29
              END IF
            ELSE IF (ATOMNO(IATOM).LE.36) THEN
              IF (SPHRAD.LE.(0.02*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 15
              ELSE IF (SPHRAD.LE.(0.1*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 23
              ELSE IF (SPHRAD.LE.(0.2*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 29
              ELSE IF (SPHRAD.LE.(3.5*G98RAD(ATOMNO(IATOM)))) THEN
                LRADIAL(IATOM,IRAD) = 39
              ELSE
                LRADIAL(IATOM,IRAD) = 29
              END IF
            ELSE
              LRADIAL(IATOM,IRAD) = 39
            END IF
C
          END DO
C
        END DO
C
      ELSE IF (GRID.EQ.'(200,1202)') THEN
C
C     *** Unpruned (200,1202) reference grid ***
C
        IF (PRTGRD.AND.(STATUS.EQ.'SCF')) THEN
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL MESSAGE(1,'UNPRUNED (200,1202) GRID (REFERENCE)',1,0)
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MESSAGE(1,'UNPRUNED (200,1202) MURA-KNOWLES GRID',1,0)
          END IF
        END IF
C
        DO IATOM=1,NATOM
C
          NRADIAL(IATOM) = 200
C
          IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
            CALL EULMAC(U,WR,NRADIAL(IATOM))
          ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
            CALL MURKNO(IATOM,U,WR,NRADIAL(IATOM))
          ELSE
            PRTSTR = 'INVALID RADIAL QUADRATURE FOR FIXED GRID'
            CALL ERRMSG('G98GRD',STRCOMP(PRTSTR),1)
          END IF
C
          CALL LOCAL(IATOM)
C
          DO IRAD=1,NRADIAL(IATOM)

            IF (RADQUA.EQ.'EULER-MACLAURIN') THEN
              SPHRAD = G98RAD(ATOMNO(IATOM))*
     $                 (U(IRAD)/(1.0 - U(IRAD)))**2
            ELSE IF (RADQUA.EQ.'MURA-KNOWLES') THEN
              SPHRAD = U(IRAD)
            END IF

C
            IF ((SPHRAD.LT.P3ABS).AND.(ZATOM(IATOM).EQ.0)) THEN 
              LRADIAL(IATOM,IRAD) = 0
            ELSE IF (ATOMNO(IATOM).LE.2) THEN
              LRADIAL(IATOM,IRAD) = 59
            END IF
C
          END DO
C
        END DO
C
      ELSE
C
        PRTSTR = 'UNKNOWN FIXED GRID TYPE '//GRID
        CALL ERRMSG('G98GRD',STRCOMP(PRTSTR),1)
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE G98GRD ***
C
      END
