      SUBROUTINE GETCXC(EIGAUX,VXCDCA,VXCDCB,XCDCA,XCDCB,XCD0A,XCD0B)
C
C     Purpose: Transformation of approximated exchange-correlation
C              potential vector.
C
C     History: - Creation (24.09.00, AMK)
C                         (02.11.05, FJ)
C                         (25.04.18, NPM)
C                         (08.11.18, AMK)
C                         (24.01.20, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C     EIGAUX: Inverse eigenvalues of auxiliary function ERI matrix.
C     VXCDCA: Alpha exchange-correlation potential vector.
C     VXCDCB: Beta exchange-correlation potential vector.
C     XCDCA : Transformed alpha exchange-correlation potential vector.
C     XCDCB : Transformed beta exchange-correlation potential vector.
C     XCD0A : Alpha exchange-correlation potential start vector.
C     XCD0B : Beta exchange-correlation potential start vector.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
C
CNP
      INCLUDE 'fileio.h'
CNP
      REAL EIGAUX(MAXAUX),VXCDCA(MAXAUX),VXCDCB(MAXAUX)
      REAL XCDCA(MAXAUX),XCDCB(MAXAUX),XCD0A(MAXAUX),XCD0B(MAXAUX)
C
      LOGICAL STATED
      CHARACTER*80 TIMSTR
      INTEGER IAUX
      REAL NUMRND
C
C     ------------------------------------------------------------------
C
C     *** Start timer for auxiliary function fit ***
C
      IF (MPP) THEN
        TIMSTR = 'MPI AUXILIARY FUNCTION FIT'
      ELSE
        TIMSTR = 'AUXILIARY FUNCTION FIT'
      END IF
C
      IF (.NOT.STATED('ATOM')) THEN
        CALL TIMER('START',TIMSTR)
      END IF
C
      USEMPF = (MPP.AND.(.NOT.STATED('ATOM')).AND.(NAUX.GE.NCPU))
C
C     *** Round exchange-correlation potential vector ***
C
      DO IAUX=1,NAUX
        VXCDCA(IAUX) = NUMRND(VXCDCA(IAUX),1.E16)
        IF (OKS) VXCDCB(IAUX) = NUMRND(VXCDCB(IAUX),1.E16)
      END DO
C
C     *** Calculate exchange-correlation potential fit coefficients ***
C
      IF (NAUX.GT.0) THEN
C
C     *** Do the fitting analytical ***
C
        IF (FITTING.EQ.'ANALYTICAL') THEN
C
          CALL ANACXC(EIGAUX,VXCDCA,VXCDCB,XCDCA,XCDCB)
C
C     *** Do the fitting numerical ***
C
        ELSE IF (FITTING.EQ.'NUMERICAL') THEN
C
C     *** Use pre-conditioned CG solver ***
C
          IF (STATED('ATOM').OR.USESVD) THEN

            CALL NUMCXC(VXCDCA,VXCDCB,XCDCA,XCDCB,XCD0A,XCD0B)
C
C     *** Use pre-conditioned MINRES solver ***
C
          ELSE IF (USEMIN) THEN
C
            CALL MINCXC(VXCDCA,VXCDCB,XCDCA,XCDCB,XCD0A,XCD0B)
C
C     *** Error handling for numerical fitting ***
C
          ELSE
C
            CALL ERRMSG('GETCXC','UNKNOWN NUMERICAL FITTING OPTION',1)
C
          END IF
C
        END IF
C
      END IF 
C
      USEMPF = .FALSE.
C
      IF (FITTYP.EQ.'VALENCE') THEN
        XCDCA(NAUX+1:NALL) = 0.0
        IF (OKS) XCDCB(NAUX+1:NALL) = 0.0
      END IF
C
C     *** Stop timer for auxiliary function fit ***
C
      IF (.NOT.STATED('ATOM')) THEN
        CALL TIMER('STOP',TIMSTR)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE GETCXC ***
C
      END
