      SUBROUTINE HIRSHFELD(HTO,WATOM,LLGPB,NGPB,NSPIN)
C
C     Purpose: Generating of Hirshfeld weights.
C
C     Lit.: F.L. Hirshfeld, Theor. Chim. Acta 44, 129 (1977)
C
C     History: - Creation (09.02.10, ADLL)
C                         (10.11.17, AMK)
C                         (05.08.19, GG)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     WATOM: Atomic Hirshfeld weights of the current grid points.
C
C     List of local dynamical fields:
C
C     RHOA0: Reference atom densities for alpha spin.
C     RHOB0: Reference atom densities for beta spin.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
C
      REAL EPS
      PARAMETER (EPS = 1.E-6)
C
      INTEGER NSPIN
      REAL HTO(MAXGPB,NALL),WATOM(MAXGPB,NATOM,NSPIN)
C
      INTEGER IATOM,IGP,INEIGH,LLGPB,NGPB
      REAL RHOTOTA,RHOTOTB
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: RHOA0(:,:),RHOB0(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(RHOA0(MAXGPB,NATOM),RHOB0(MAXGPB,NATOM),STAT=ALLOCATION)
      IF (ALLOCATION.NE.0) THEN
        CALL ERRMSG('HIRSHFELD','ALLOCATION FAILED',1)
      END IF
C
C     *** Initialization ***
C
      RHOA0(:,:) = 0.0
      RHOB0(:,:) = 0.0
      WATOM(1:NGPB,1:NATOM,1:NSPIN) = 0.0 
C
C     *** Calculate reference densities for Hirshfeld weights ***
C
      IF (.NOT.TOPCD) CALL TAU(HTO,NALL,LLGPB,NGPB)
      CALL PHI0(HTO,NALL,RHOA0,RHOB0,LLGPB,NGPB)
C
C     *** Calculate alpha Hirshfeld weights for current grid points ***
C
      DO IGP=1,NGPB
C
C     *** Calculate total reference density ***
C
        RHOTOTA = 0.0
        DO INEIGH=1,NNEIGH
          IATOM = NEIGH(INEIGH)
          RHOTOTA = RHOTOTA + RHOA0(IGP,IATOM)
        END DO
C
C     *** Loop over all neighbor atoms ***
C
        DO INEIGH=1,NNEIGH
C
          IATOM = NEIGH(INEIGH)
C
C     *** Assign Hirshfeld weights for alpha density ***
C
          IF (ABS(RHOTOTA).GT.EPS) THEN
            WATOM(IGP,IATOM,1) = RHOA0(IGP,IATOM)/RHOTOTA
          END IF
C
        END DO
C
      END DO
C
      IF (ITHFO) THEN
C
C     *** Calculate beta Hirshfeld weights for current grid points ***
C
        DO IGP=1,NGPB
C
C     *** Calculate total reference density ***
C
          RHOTOTB = 0.0
          DO INEIGH=1,NNEIGH
            IATOM = NEIGH(INEIGH)
            RHOTOTB = RHOTOTB + RHOB0(IGP,IATOM)
          END DO
C
C     *** Loop over all neighbor atoms ***
C
          DO INEIGH=1,NNEIGH
C
            IATOM = NEIGH(INEIGH)
C
C     *** Assign Hirshfeld weights for beta density ***
C
            IF (ABS(RHOTOTB).GT.EPS) THEN
              WATOM(IGP,IATOM,2) = RHOB0(IGP,IATOM)/RHOTOTB
            END IF
C
          END DO
C
        END DO
C
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(RHOA0,RHOB0,STAT=ALLOCATION)
      IF (ALLOCATION.NE.0) THEN
        CALL ERRMSG('HIRSHFELD','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE HIRSHFELD ***
C
      END
