      SUBROUTINE ICHOLESKY(MATRIX,DMAT,TOL,RANK,OPTION)
C     
C     Purpose: Incomplete Cholesky decomposition of matrix A.
C
C     Lit.: http://www.netlib.org
C
C     History: - Creation (26.07.11, DMR)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DMAT: Dimension of matrix A and B in the calling subroutine.
C
C     List of local variables:
C
C     MATRIX: Input matrix to be decomposed.
C     INFO  : Cholesky decomposition error information.
C     OPTION: a) L  = Perform the decomposition P(T)*A*P = L*L(T).
C             b) PL = Backtransform to A = P*L*L(T)*P(T).
C     RANK  : Rank of matrix A obtained by decomposition.
C     TOL   : Stop decomposition at this tolerance.
C     
C     List of local fields:
C
C     PVEC: Permutation vector pointer.
C     WORK: Work field for decomposition.
C
C     -----------------------------------------------------------------
C
      IMPLICIT NONE
C
      INTEGER DMAT
      REAL MATRIX(DMAT,DMAT)
C
      CHARACTER*(80) OPTION*(*),PRTSTR,STRCOMP
      INTEGER INFO,IMAT,JMAT,RANK
      REAL TOL
C     
      INTEGER ALLOCATION
      INTEGER,ALLOCATABLE :: PVEC(:)
      REAL,ALLOCATABLE :: WORK(:)
C
C     -----------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE (PVEC(DMAT),WORK(2*DMAT),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('ICHOLESKY','ALLOCATION FAILED',1)
      END IF
C
C     *** Do Cholesky decomposition: P(T)*A*P = L*L(T) ***
C
C       call primat(MATRIX,dmat,1,DMAT,1,dmat,4,10,700,
C     $            'NONE','TOTAL','P_R 1')

      CALL DPSTRF('L',DMAT,MATRIX,DMAT,PVEC,RANK,TOL,WORK,INFO)
C       call primat(MATRIX,dmat,1,DMAT,1,dmat,4,10,700,
C     $            'NONE','TOTAL','P_R 2')
C        write(700,*) INFO,RANK
C
C     *** Check Cholesky decomposition ***
C
      IF (INFO.LT.0) THEN
        PRTSTR = 'INCOMPLETE CHOLESKY DECOMPOSITION FAILED'
        CALL ERRMSG('ICHOLESKY',PRTSTR,1)
      END IF
C
C     *** Set upper triangular part equal to zero ***
C
      CALL SCALMAT(MATRIX,DMAT,DMAT,DMAT,'UPPWD',0.0)
C       call primat(MATRIX,dmat,1,DMAT,1,dmat,4,10,700,
C     $            'NONE','TOTAL','P_R 3')
C
C     *** Build backtransformed decomposed matrix ***
C
      IF (OPTION.EQ.'PL') THEN
C
        DO IMAT=1,DMAT
          DO JMAT=IMAT,DMAT
            IF (PVEC(JMAT).EQ.IMAT) THEN
              IF (JMAT.EQ.IMAT) EXIT
              WORK(1:DMAT) = MATRIX(IMAT,1:DMAT)
              MATRIX(IMAT,1:DMAT) = MATRIX(JMAT,1:DMAT)
              MATRIX(JMAT,1:DMAT) = WORK(1:DMAT)
              PVEC(JMAT) = PVEC(IMAT)
              EXIT
            END IF
          END DO
        END DO
C
C       call primat(MATRIX,dmat,1,DMAT,1,dmat,4,10,700,
C     $            'NONE','TOTAL','P_R 4')
C     *** Error handling ***
C
      ELSE IF (OPTION.NE.'L') THEN
C
        WRITE (PRTSTR,5500) OPTION
 5500   FORMAT ('UNKNOWN INCOMPLETE CHOLESKY OPTION: ',A)
        CALL ERRMSG('ICHOLESKY',STRCOMP(PRTSTR),1)
C
      END IF
C
C     *** Deallocate local dynamical fields ***
C
      DEALLOCATE(WORK,PVEC,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('ICHOLESKY','DEALLOCATION FALED',1)
      END IF
C
C     -----------------------------------------------------------------
C
C     END OF SUBROUTINE ICHOLESKY
C
      END
