      REAL FUNCTION DFXNCAP(RHO,DRHO)
C
C     Purpose: Calculate the exchange enhancement factor for the
C              NCAP GGA exchange functional.
C
C     Lit.: J. Carmona-Espindola et al.
C
C     History: - Creation (26.09.16, JCE)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DRHO: Norm of the gradient of the electron density.
C     RHO : Electron density.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'threshold.h'
      INCLUDE 'xc.h'
      include 'math.h'
C
      REAL DRHO,RHO
C
      REAL KF,S
      REAL MU,B,A,Z,MYF,ARSINH,SECH
C
C     ------------------------------------------------------------------
C
C     *** Threshold density ***
C
      IF ((RHO.LT.RHOTHRESH).OR.(DRHO.LT.RHOTHRESH)) THEN
        DFXNCAP = 0.0
        RETURN
      END IF
C
C     *** Define parameters ***
C
      MU = 0.2195149727645171
      MYF = 0.5
      Z = 1.0
      B = 0.017982946634292535
      A = (4.0*PI)/3.0*B/MU
C
C     *** Calculate DFx for the NCAP exchange ***
C
      KF = CKF*(2.0*RHO)**EXP13
C
      S = DRHO/(2.0*KF*RHO)
C
      DFXNCAP = -((MU*ARSINH(S)*(1.0 + A*(MYF*S*Z + (1.0 - MYF)*S*
     $            LOG(1.0 + S)))*TANH(S)*(B*ARSINH(S)*
     $            SECH(S)**2.0 + (B*TANH(S))/SQRT(1.0 + S**2)))/
     $            (1.0 + B*ARSINH(S)*TANH(S))**2) + (MU*ARSINH(S)*
     $            (1.0 + A*(MYF*S*Z + (1.0 - MYF)*S*
     $            LOG(1.0 + S)))*SECH(S)**2)/(1.0 + B*ARSINH(S)*
     $            TANH(S)) + (A*MU*ARSINH(S)*(((1.0 - MYF)*S)/
     $            (1.0 + S) + MYF*Z + (1.0 - MYF)*LOG(1.0 + S))*
     $            TANH(S))/(1.0 + B*ARSINH(S)*TANH(S)) + (MU*(1.0
     $            + A*(MYF*S*Z + (1.0 - MYF)*S*LOG(1.0 + S)))*
     $            TANH(S))/(SQRT(1.0 + S**2)*(1.0 + B*ARSINH(S)*
     $            TANH(S)))
C
C     ------------------------------------------------------------------
C
C     *** End of REAL FUNCTION DFXNCAP ***
C
      END

