      REAL FUNCTION FXNCAP(RHO,DRHO)
C
C     Purpose: Calculate the exchange enhancement factor for the
C              NCAP GGA exchange functional.
C
C     Lit.: J. Carmona-Espindola et al.
C
C     History: - Creation (26.09.16, JCE)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DRHO: Norm of the gradient of the electron density.
C     RHO : Electron density.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'threshold.h'
      INCLUDE 'xc.h'
      include 'math.h'
C
      REAL DRHO,RHO
C
      REAL KF,S
      REAL MU,B,A,Z,MYF,ARSINH
C
C     ------------------------------------------------------------------
C
C     *** Threshold density ***
C
      IF ((RHO.LT.RHOTHRESH).OR.(DRHO.LT.RHOTHRESH)) THEN
        FXNCAP = 0.0
        RETURN
      END IF
C
C     *** Define parameters ***
C
      MU = 0.2195149727645171
      MYF = 0.5
      Z = 1.0
      B = 0.017982946634292535
      A = (4.0*PI)/3.0*B/MU
C
C     *** Calculate Fx for the NCAP exchange ***
C
      KF = CKF*(2.0*RHO)**EXP13
C
      S = DRHO/(2.0*KF*RHO)
C
      FXNCAP = 1.0 + MU*TANH(S)*ARSINH(S)*(1.0 + A*(
     $           (1.0 - MYF)*S*LOG(1.0 + S) + Z*MYF*S))/(1.0 +
     $           B*TANH(S)*ARSINH(S))
C
C     ------------------------------------------------------------------
C
C     *** End of REAL FUNCTION FXNCAP ***
C
      END

