      SUBROUTINE VXNCAP(RHO,DRHO,VX,VXG)
C
C     Purpose: Calculate the NCAP GGA exchange potential.
C
C     Lit.: J. Carmona-Espindola et al., 
C
C     History: - Creation (26.09.16, JCE)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DRHO: Norm of the gradient of the alpha/beta electron density.
C     RHO : Alpha/Beta electron density value.
C     VX  : Alpha/Beta exchange potential value.
C     VXG : Gradient correction of the alpha/beta exchange potential.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'threshold.h'
      INCLUDE 'xc.h'
C
      REAL DRHO,RHO,VX,VXG
C
      REAL DFX,FX,KF,S
      REAL FXNCAP,DFXNCAP,TAU,TMF
C
C     ------------------------------------------------------------------
C
C     *** Threshold density and gradients ***
C
      IF ((RHO.LT.RHOTHRESH).OR.(DRHO.LT.RHOTHRESH)) RETURN
C
C     *** Calculate the exchange potential of NCAP ***
C
      KF = CKF*(2.0*RHO)**EXP13
C
      S = DRHO/(2.0*KF*RHO)
C
      FX = FXNCAP(RHO,DRHO)
C
      DFX = DFXNCAP(RHO,DRHO)
C
C     *** Cutoff, different to the original NCAP form ***
C
      IF (S.GT.10.0) THEN
        TAU = EXP(0.25 - 0.05*S)
        TMF = TAU*(1.0 - FX)
        FX  = 1.0 - TMF
        DFX = 0.05*TMF + TAU*DFX
      END IF
C
      VX = CVLSD*(FX - S*DFX)*(2.0*RHO)**EXP13
C
      VXG = CXVG*DFX/DRHO
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE VXNCAP ***
C
      END

