      SUBROUTINE NUMPOP(PA,PB,DSTO)
C
C     Purpose: Numerical calculation of atom populations by weighted
C              density integration.
C
C     History: - Creation (30.08.12, AMK)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'property.h'
C
      INTEGER DSTO
      REAL PA(DSTO,DSTO),PB(DSTO,DSTO)
C
C     ------------------------------------------------------------------
C
C     *** Prepare density matrices for numerical integration ***
C
      IF (.NOT.TOPCD) THEN
        CALL SCALMAT(PA,DSTO,NSTO,NSTO,'UPPWD',0.5)
        CALL SETMAT(PA,DSTO,NSTO,NSTO,'LOWER',0.0)
        IF (OKS) THEN
          CALL SCALMAT(PB,DSTO,NSTO,NSTO,'UPPWD',0.5)
          CALL SETMAT(PB,DSTO,NSTO,NSTO,'LOWER',0.0)
        END IF
      END IF
C
C     *** Perform a direct numerical integration ***
C
      IF (POPMO) THEN 
        CALL NUMPOPDIR_SCREENED(PA,PB,DSTO) 
      ELSE 
         CALL NUMPOPDIR(PA,PB,DSTO)
      END IF
C
C     *** Restore density matrices to original form ***
C
      IF (.NOT.TOPCD) THEN
        CALL SCALMAT(PA,DSTO,NSTO,NSTO,'UPPWD',2.0)
        CALL SYMMAT(PA,DSTO,NSTO,'UPLOW')
        IF (OKS) THEN
          CALL SCALMAT(PB,DSTO,NSTO,NSTO,'UPPWD',2.0)
          CALL SYMMAT(PB,DSTO,NSTO,'UPLOW')
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NUMPOP ***
C
      END
