      SUBROUTINE ORTHOMOZ(C,S,PR,PI,DSTO,DSTO_ME,SPIN)
C
C     Purpose: Orthogonalize complex MOs by Gram-Schmidt method.
C
C     History: (Creation 02.09.20, ADLL)
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'scfvec.h'
      INCLUDE 'mp_demon.h'
C
      INTEGER DSTO,DSTO_ME
      REAL C(DSTO,DSTO),S(DSTO,DSTO)
      REAL PR(DSTO_ME,DSTO_ME),PI(DSTO_ME,DSTO_ME)
      CHARACTER*(*) SPIN
C
      INTEGER ALLOCATION,NOMO,IMO
      REAL NORM
C
      COMPLEX (KIND=8) EYE
C
C     -----------------------------------------------------------------
C
C     *** Start timer for topological analysis ***
C
      IF (MPP) THEN
        CALL TIMER('START','MPI GS ORTHOGONALIZATION')
      ELSE
        CALL TIMER('START','GS ORTHOGONALIZATION')
      END IF

C     *** Load overlap matrix ***
C
      IF (SPIN.EQ.'ALPHA') NOMO = NOMOA
      IF (SPIN.EQ.'BETA') NOMO = NOMOB
C
      IF (MPMASTER) THEN
        CALL SCALMAT(PR,DSTO,DSTO,DSTO,'M = N',0.5)
        CALL SCALMAT(PR,DSTO,DSTO,DSTO,'TOTWD',0.5/2)
        C(:,:) = PR(:,:)! + EYE*PI(:,:)
C        
        CALL CHLMOS(C,DSTO,NOMO,'OCC')
        CALL SCFIO(S,DSTO,'READ','OVERLAP')
      END IF
C
      IF (MPP) THEN
        CALL MP_SEND_ALL(NOMO,1,MASTER,MP_INT)
        CALL MP_SEND_ALL(S,DSTO**2,MASTER,MP_REAL) 
        CALL MP_SEND_ALL(C,DSTO**2,MASTER,MP_REAL)
      END IF
C
C     *** Loop over MOs and orthogonilize ***
C
      DO IMO=1,NOMO
C        CALL MP_SYNC
        CALL GRAMMOZ(C,S,IMO,DSTO)
      END DO
C
      S(:,:) = 0.0
      IF (MPMASTER) CALL BLDCOMMAT(C,S,PR,PI,DSTO,OCCNA,NOMO,NOMO)
C
      IF (MPP) THEN
        CALL TIMER('STOP','MPI GS ORTHOGONALIZATION')
      ELSE
        CALL TIMER('STOP','GS ORTHOGONALIZATION')
      END IF
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE ORTHOMOZ ***
C
      END 

