      SUBROUTINE RHO_SCREENED(P,STO,DSTO,RHOS,LLGPB,NGPB)
C
C     Purpose: Numerical calculation of the density at the grid point
C              of the current batch of grid points.
C
C     History: - Creation (12.08.96, MK)
C                         (11.01.00, AMK)
C                         (15.08.15, DMR)
C
C     ******************************************************************
C
C     List of local variables:
C
C     NGPB: Number of grid points in the current batch.
C     P   : Alpha/Beta density matrix.
C     PSTO: Work vector (P(mu,nu)*Chi(nu)).
C     RHOS: Density values of the current batch of grid points.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
      INCLUDE 'property.h'
      INCLUDE 'threshold.h'
C
      INTEGER DSTO
      REAL P(DSTO,DSTO),STO(MAXGPB,DSTO)
C
      INTEGER IGP,LLGPB,NGPA,NGPB
      INTEGER IATOM,INEIGH,ISHL,ISTO,JATOM,JNEIGH,JSTO
      REAL R,XAG,YAG,ZAG
C
      INTEGER IGPPTR(MAXGPB)
      REAL DAG(MAXGPB),PSTO(MAXGPB),RHOS(MAXGPB)
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      RHOS(1:NGPB) = 0.0
C
C     *** Loop over all neighbors ***
C
      DO INEIGH=1,NNEIGH
C
        IATOM = NEIGH(INEIGH)
C
        DO IGP=1,NGPB
          XAG = GRDVEC(1,LLGPB+IGP-1) - COORD(1,IATOM,CCSET)
          YAG = GRDVEC(2,LLGPB+IGP-1) - COORD(2,IATOM,CCSET)
          ZAG = GRDVEC(3,LLGPB+IGP-1) - COORD(3,IATOM,CCSET)
          DAG(IGP) = XAG**2 + YAG**2 + ZAG**2
        END DO
C
        DO ISHL=LLSHL(IATOM),ULSHL(IATOM)
C
C     *** Select active grid points of current shell ***
C
          NGPA = 0
          DO 100 IGP=1,NGPB
            DO JNEIGH=0,NNEIGH
              JATOM = NEIGH(JNEIGH)
              R = (GRDVEC(1,LLGPB+IGP-1) - COORD(1,JATOM,CCSET))**2 +
     $            (GRDVEC(2,LLGPB+IGP-1) - COORD(2,JATOM,CCSET))**2 +
     $            (GRDVEC(3,LLGPB+IGP-1) - COORD(3,JATOM,CCSET))**2
              R = SQRT(R)
              IF (R.LE.VDWRAD(ATOMNO(JATOM))) GO TO 100  
            END DO
            IF (DAG(IGP).LE.SHLRAD(ISHL)) THEN
              NGPA = NGPA + 1
              IGPPTR(NGPA) = IGP
            END IF
 100    CONTINUE
C
          IF (NGPA.EQ.0) CYCLE
C
C     *** Build Kohn-Sham density ***
C
          DO ISTO=LLSTO(ISHL),ULSTO(ISHL)
            PSTO(1:NGPA) = 0.0
            DO JNEIGH=1,NNEIGH
              JATOM = NEIGH(JNEIGH)
              DO JSTO=LL(JATOM),UL(JATOM)
                IF (ISTO.LE.JSTO) THEN
                  IF (ABS(P(ISTO,JSTO)).GT.PTHRESH) THEN
                    IF (NGPA.EQ.NGPB) THEN
                      PSTO(1:NGPB) = PSTO(1:NGPB) + P(ISTO,JSTO)*
     $                               STO(1:NGPB,JSTO)
                    ELSE
                      PSTO(1:NGPA) = PSTO(1:NGPA) + P(ISTO,JSTO)*
     $                               STO(IGPPTR(1:NGPA),JSTO)
                    END IF
                  END IF
                ELSE
                  IF (ABS(P(JSTO,ISTO)).GT.PTHRESH) THEN
                    IF (NGPA.EQ.NGPB) THEN
                      PSTO(1:NGPB) = PSTO(1:NGPB) + P(JSTO,ISTO)*
     $                               STO(1:NGPB,JSTO)
                    ELSE
                      PSTO(1:NGPA) = PSTO(1:NGPA) + P(JSTO,ISTO)*
     $                               STO(IGPPTR(1:NGPA),JSTO)
                    END IF
                  END IF
                END IF
              END DO
            END DO
            RHOS(IGPPTR(1:NGPA)) = RHOS(IGPPTR(1:NGPA)) +
     $                             PSTO(1:NGPA)*STO(IGPPTR(1:NGPA),ISTO)
          END DO
C
        END DO
      END DO
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE RHO ***
C
      END
