      SUBROUTINE BACKCST(DRSTEP,U,DACT,DDEG,DPRI,DVAR)
C
C     Purpose: Expand Cartesian displacement to full space.
C
C     History: - Creation (30.05.06, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DACT: Number of active coordiantes.
C     DDEG: Number of degrees of freedom of the system.
C     DPRI: Dimension of primitive internal coordinates.
C     DVAR: Number of internal degrees of freedom.
C
C     List of local variables:
C
C     DRSTEP: Internal displacement step.
C     U     : Transformation matrix for active variables.
C
C     List of local dynamical fields:
C
C     DRCST: Constrained Cartesian displacement step.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
C
      INTEGER DACT,DDEG,DPRI,DVAR
      REAL DRSTEP(DDEG),U(DPRI,DVAR)
C
      INTEGER IATOM,IDEG,IPOS
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: DRCST(:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(DRCST(DDEG),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('BACKCST','ALLOCATION FAILED',1)
      END IF
C
C     *** Initialize constrained Cartesian step ***
C
      DRCST(:) = 0.0
      DRCST(1:DACT) = DRSTEP(1:DACT)
C
C     *** Expand Cartesian displacement step ***
C
      DRSTEP(:) = 0.0
      DRSTEP(1:DDEG) = MATMUL(U(1:DDEG,1:DACT),DRCST(1:DACT))
C
C     *** Calculate new Cartesian coordinates ***
C
      IDEG = 0
      DO IATOM=1,NATOM_ALL
        DO IPOS=1,3
          IDEG = IDEG + 1
          COORD(IPOS,IATOM,CCSET) = COORD(IPOS,IATOM,CCSET) + 
     $                              DRSTEP(IDEG)
        END DO
      END DO
C
CCC ADLL LA
C     *** Set link atom position ***
C
      CALL SETLAPOS('MD')
C
CCC ADLL LA
C
      IF (PRTOPT) THEN
        CALL PRIDR('DISPLACEMENT STEP',DRSTEP)
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(DRCST,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('BACKCST','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE BACKCST ***
C
      END
