      SUBROUTINE CLOUT(KSA,PA,DSTO,DSTO_ME)
C
C     Purpose: Closed shell SCF output.
C
C     History: - Creation (09.09.99, AMK)
C                         (27.06.08, AG)
C                         (08.02.18, GG)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables:
C
C     KSA   : MO coefficients.
C     NUCREP: Nuclear repulsion.
C     PA    : Density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'cdft.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'iteration.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
CCC ADLL
      INCLUDE 'qmmm.h'
CCC ADLL
      INCLUDE 'physic.h'
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'xc.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KSA(DSTO,DSTO),PA(DSTO_ME,DSTO_ME)
C
      LOGICAL STATED
      CHARACTER*80 PRTSTR,STRCOMP
      INTEGER IMO,STREXT
      REAL NUCEXT,NUCREP
C
      REAL RDUMMY(1)
C
C     ------------------------------------------------------------------
C
C     *** Save restart information ***
C
      IF (((STATUS.EQ.'SCF').OR.(STATUS.EQ.'SADSCF').OR.
     $     (STATUS.EQ.'GRID').OR.(STATUS.EQ.'BOMD').OR.
     $     (STATUS.EQ.'OPT').OR.(STATUS.EQ.'SAD').OR.
     $     (STATUS.EQ.'IRC').OR.STATED('END').OR.STATED('SCAN')).AND.
     $     (.NOT.FREQRST)) THEN
        CALL RSTIO(KSA,EIGVALA,PA,DSTO,DSTO_ME,OCCNA,AOCCNA,ELCFGA,
     $             MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'WRITE','ALPHA')
      END IF
C
      IF (MPSLAVE) GO TO 100
C
C     *** Write energy and basis into MOLDEN, MOLEKEL or WFN files ***
C
      IF ((STATUS.EQ.'SCF').AND.(.NOT.OPT).OR.
     $    (STATUS.EQ.'GRID').AND.(.NOT.OPT)) THEN
        IF (MOLDEN.EQ.'FULL') THEN
          CALL MOLDRV(RDUMMY,RDUMMY,RDUMMY,RDUMMY,1,1,'BAS')
          CALL MOLDRV(EIGVALA,KSA,RDUMMY,RDUMMY,DSTO,DSTO,'ALPHA')
        ELSE IF (MOLEKEL.EQ.'FULL') THEN
          CALL MKLDRV(RDUMMY,RDUMMY,1,'BAS','UNDEFINED')
          CALL MKLDRV(EIGVALA,KSA,DSTO,'MOS','ALPHA')
          CALL MKLDRV(EIGVALB,KSA,DSTO,'MOS','BETA')
        ELSE IF (WFNFILE.EQ.'FULL') THEN
          CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'REWIND')
          CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'HEADER')
          CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'BASIS')
          CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'ALPHA')
          CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'END')
        END IF
      ELSE IF (((STATUS.EQ.'OPT').AND.OPTIMIZED).OR.
     $         ((STATUS.EQ.'BOMD').AND.OPTIMIZED)) THEN
        CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'REWIND')
        CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'HEADER')
        CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'BASIS')
        CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'ALPHA')
        CALL WFNDRV(EIGVALA,OCCNA,KSA,NSTO,'END')
      END IF
C
C     *** Calculate converged energy ***
C
      ESCF = EHARTREE + EXC + EVDW + NUCREP()
C
C      *** Add energy from external field ***
C
      IF (EFIELD.AND.THZ) ESCF = ESCF + NUCEXT()
C
C     *** Decompose core energy ***
C
      IF (PRTDEC) CALL DECCORE(EKINETIC,ENUCLEUS,PA,PA,DSTO)
C
C     *** Write SCF output ***
C
      IF ((STATUS.EQ.'SCF').OR.(STATUS.EQ.'GRID')) THEN
        IF (STATUS.EQ.'GRID') THEN
          IF (.NOT.BREAK) CALL RMSCFOUT
          WRITE (PRTSTR,5000) SCFCYC
 5000     FORMAT ('SCF GRID GENERATED IN ',I5,' CYCLES')
          IF (RANGRD) PRTSTR = 'RANDOMIZED '//PRTSTR(1:STREXT(PRTSTR))
          CALL MESSAGE(0,STRCOMP(PRTSTR),1,1)
        END IF
C
        IF (USECDFT) CALL CDFTOUT
C
CCC ADLL MMPOL
CCCC COMMENTS: FOR QM/MM THE DEFINITION OF CORE ENERGY IN THE REPORT IS
CCCC           MODIFIED WRT THE MASTER VERSION. THE PRINTED CORE ENERGY
CCCC           DO NOT INLUCDE ELECTROSTATIC EMBEDDING ENERRY FROM MM
CCCC           CHARGES AND INDUCED DIPOLES (if DOINDIP).
        IF (QMMM) THEN
C          IF (DOINDIP) THEN
C            WRITE (OUT,5005) ENUCLEUS-EDENCH-EDENIND,EDENCH,EDENIND,
C     $                       ECOULOMB,EHARTREE
C 5005   FORMAT (/,T2,'ELECTRONIC CORE ENERGY      = ',F20.9,
C     $          /,T2,'PERM. MLTPOLE EMBED. ENERGY = ',F20.9,
C     $          /,T2,'IND. DIPOLE EMBED. ENERGY   = ',F20.9,
C     $          /,T2,'ELECTRONIC COULOMB ENERGY   = ',F20.9,
C     $          /,T2,'ELECTRONIC HARTREE ENERGY   = ',F20.9)
C          ELSE
            WRITE (OUT,5008) ENUCLEUS-EPCQM,EPCQM,ECOULOMB,EHARTREE
 5008   FORMAT (/,T2,'ELECTRONIC CORE ENERGY      = ',F20.9,
     $          /,T2,'EMBEDDING CHARGES ENERGY    = ',F20.9,
     $          /,T2,'ELECTRONIC COULOMB ENERGY   = ',F20.9,
     $          /,T2,'ELECTRONIC HARTREE ENERGY   = ',F20.9)
C          END IF
        ELSE
          WRITE (OUT,5009) ENUCLEUS,ECOULOMB,EHARTREE
 5009   FORMAT (/,T2,'ELECTRONIC CORE ENERGY      = ',F20.9,
     $          /,T2,'ELECTRONIC COULOMB ENERGY   = ',F20.9,
     $          /,T2,'ELECTRONIC HARTREE ENERGY   = ',F20.9)
        END IF
CCC ADLL MMPOL

        IF (PRTDEC) THEN
          WRITE (OUT,5010) EKINETIC,ENUCLEUS
 5010     FORMAT (/,T2,'ELECTRONIC KINETIC ENERGY   = ',F20.9,
     $            /,T2,'ELECTRONIC NUCLEAR ENERGY   = ',F20.9)
        ELSE 
          WRITE (OUT,5020) ENUCLEUS
 5020     FORMAT (/,T2,'ELECTRONIC CORE ENERGY      = ',F20.9)
        END IF
C
        WRITE (OUT,5030) ECOULOMB,EHARTREE
 5030   FORMAT (T2,'ELECTRONIC COULOMB ENERGY   = ',F20.9,
     $          /,T2,'ELECTRONIC HARTREE ENERGY   = ',F20.9)
C
        WRITE (OUT,5040) EXSUM,ECSUM,EXC
 5040   FORMAT (/,T2,'EXCHANGE ENERGY             = ',F20.9,
     $          /,T2,'CORRELATION ENERGY          = ',F20.9,
     $          /,T2,'EXCHANGE-CORRELATION ENERGY = ',F20.9)
C
        WRITE (OUT,5050) EHARTREE+EXC
 5050   FORMAT (/,T2,'ELECTRONIC SCF ENERGY       = ',F20.9)
C
        IF (EVDW.NE.0.0) THEN
          WRITE (OUT,5060) EVDW
 5060     FORMAT (T2,'EMPIRICAL DISPERSION ENERGY = ',F20.9)
        END IF
C
        IF (EFIELD.AND.THZ) THEN
          WRITE (OUT,5070) NUCEXT()
 5070     FORMAT (T2,'NUCLEAR EXTERNAL ENERGY     = ',F20.9)
        END IF
C
        WRITE (OUT,5080) NUCREP()
 5080   FORMAT (T2,'NUCLEAR-REPULSION ENERGY    = ',F20.9)
C
        IF (QMONLY) THEN
          WRITE (OUT,5090) ESCF
 5090     FORMAT (T2,'TOTAL ENERGY                = ',F20.9,/)
        ELSE
          WRITE (OUT,5100) ESCF
 5100     FORMAT (T2,'TOTAL SCF ENERGY            = ',F20.9,/)
        END IF
C
      ELSE IF (((STATUS.EQ.'OPT').AND.OPTIMIZED).OR.
     $         ((STATUS.EQ.'BOMD').AND.OPTIMIZED)) THEN
        IF (.NOT.(BREAK.OR.PRTMOE.OR.PRTMOS.OR.PRTOPT)) CALL RMSCFOUT
        WRITE (OUT,5020) ENUCLEUS
        WRITE (OUT,5030) ECOULOMB,EHARTREE
        WRITE (OUT,5040) EXSUM,ECSUM,EXC
        WRITE (OUT,5050) EHARTREE+EXC
        IF (EVDW.NE.0.0) WRITE (OUT,5060) EVDW
        IF (EFIELD.AND.THZ) WRITE (OUT,5070) NUCEXT()
        WRITE (OUT,5080) NUCREP()
        IF ((.NOT.QMONLY).OR.(EVDW.NE.0.0).OR.THZ) THEN
          WRITE (OUT,5090) ESCF
        ELSE
          WRITE (OUT,5100) ESCF
        END IF
      ELSE 
        IF (.NOT.BREAK) CALL RMSCFOUT
      END IF
C
C     *** Write additional output for failed SCF ***
C
      IF (BREAK) THEN
        IF (STATUS.EQ.'SCF') CALL NEWDRV('NEW INPUT FILE')
        CALL MESSAGE(0,'MO OCCUPATIONS AND ENERGIES [eV]',1,1)
        WRITE (OUT,'(T2,"   MO"," OCCUPATION "," ENERGY [eV] ")')
        DO IMO=MAX(1,NOMOA-5),MIN(NORB,NOMOA+5)
          WRITE (OUT,5110) IMO,OCCNA(IMO),EIGVALA(IMO)*EVOLT
 5110     FORMAT (T2,I5,2X,F7.4,3X,F10.3)
        END DO
      END IF
C
  100 CONTINUE
C
C     *** Flush buffer ***
C
      CALL DEFLUSH(OUT)
C
C     *** Stop program, if SCF is not converged ***
C
      IF (BREAK) CALL ERRMSG('CLOUT','PROGRAM STOP',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE CLOUT ***
C
      END
