      SUBROUTINE D2QMMMCHG(DE2,DXYZ)
C
C     Purpose: Second derivative QM nuclear - MM Charge energy 
C              calculation.
C
C     History: - Creation (13.01.11, AG) 
C
C     ******************************************************************
C
C     List of local variables:
C
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
C
      INTEGER DXYZ
      REAL DE2(DXYZ,DXYZ)
C
      INTEGER I,IATOM,ICOORD,J,JATOM,JCOORD,LLAT_ME,ULAT_ME
      REAL DER,DER2,HESSE,R,R2,XX,YY,ZZ
C
      REAL RQM(3)
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: X(:),XQM(:),Y(:),YQM(:),Z(:),ZQM(:)
C
C     ------------------------------------------------------------------
C
CCC ADLL LA
C     *** Scale down MM charges near link atoms ***
C
      CALL SCLCHGMM('SCALE')
C
CCC ADLL LA
CC     *** Calculate parallel intervals over MM atoms ***
C
      CALL MPLIMITS(NCPU,ME,1,1,NATMM,LLAT_ME,ULAT_ME,(/0/),0)
C
C     *** Allocate local fields ***
C
      ALLOCATE(X(LLAT_ME:ULAT_ME),XQM(NATOM),Y(LLAT_ME:ULAT_ME),
     $         YQM(NATOM),Z(LLAT_ME:ULAT_ME),ZQM(NATOM),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'D2QMMMCHG','ALLOCATION FAILED')
C
C     *** Initialize parameters ***
C
      DO I=1,NATOM
        XQM(I) = COORD(1,I,CCSET)/BOHR
        YQM(I) = COORD(2,I,CCSET)/BOHR
        ZQM(I) = COORD(3,I,CCSET)/BOHR
      END DO
C
      DO I=LLAT_ME,ULAT_ME
        X(I) = COORD(1,I+NATOM,CCSET)/BOHR
        Y(I) = COORD(2,I+NATOM,CCSET)/BOHR
        Z(I) = COORD(3,I+NATOM,CCSET)/BOHR
      END DO
C
C     *** Electrostatic energy from a double loop ***
C
      DO IATOM=LLAT_ME,ULAT_ME
C
        DO JATOM=1,NATOM         
C
          XX = X(IATOM) - XQM(JATOM)
          YY = Y(IATOM) - YQM(JATOM)
          ZZ = Z(IATOM) - ZQM(JATOM)
C
          R2 = XX*XX + YY*YY + ZZ*ZZ
          R = SQRT(R2)
          DER = CHGMM(IATOM)*ZATOM(JATOM)/R2
          DER2 = DER/R 
C
          RQM(:) = SQRT(3.0)*(/ XX/R, YY/R, ZZ/R /)
C
          DO I=1,3
            ICOORD = 3*(IATOM+NATOM) - 3
            DO J=1,3
              JCOORD = 3*JATOM - 3
              HESSE = RQM(I)*RQM(J)*DER2
              IF (I.EQ.J) THEN
                HESSE = HESSE - DER2
              END IF
              DE2(ICOORD+I,JCOORD+J) = DE2(ICOORD+I,JCOORD+J) - HESSE
              DE2(ICOORD+I,ICOORD+J) = DE2(ICOORD+I,ICOORD+J) + HESSE
              DE2(JCOORD+I,JCOORD+J) = DE2(JCOORD+I,JCOORD+J) + HESSE
            END DO
          END DO
C
        END DO
C
      END  DO
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(X,XQM,Y,YQM,Z,ZQM,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'D2QMMMCHG','DEALLOCATION FAILED')
C
CCC ADLL LA
C
C     *** Restore orginal charges ***
C
      CALL SCLCHGMM('RESTORE')
C
CCC ADLL LA
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE D2QMMMCHG ***
C
      END
