      SUBROUTINE D2QMMMLJ(DE2,DXYZ)
C
C     Purpose: Second derivative Lennard-Jones  QM - MM calculations
C
C     History: - Creation (16.01.11, AG) 
C                         (13.11.14, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'qmmm.h'
C
      INTEGER DXYZ
      REAL DE2(DXYZ,DXYZ)
C
      INTEGER ALLOCATION,CNTATM,I,IA,IATOM,ICOORD,J,JATOM,
     $        JCOORD
      REAL DER,DER2,EPS,E6,E12,HESSE,R,RED,RMIN,RMIN6,R2,R6,VDWCUT2,
     $     XIA,RX,YIA,RY,ZIA,RZ
      REAL RQM(3)
      REAL,ALLOCATABLE :: XR(:),YR(:),ZR(:)
C
      REAL EPSQMMM,RADQMMM
CCC ADLL LA
      LOGICAL LAPAIR
CCC ADLL LA
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(XR(NATMM),YR(NATMM),ZR(NATMM),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'D2QMMMLJ','ALLOCATION FAILED')
C
C     *** Initialize parameters ***
C 
      VDWCUT2 = VDWCUT*VDWCUT
C
C     *** Reduction factors for MM coordinates ***
C
      DO IA=1,NATMM
        IATOM = IA + NATOM
        JATOM = IREDVDW(IA) + NATOM
        RED = REDVDW(IA)
        XR(IA) = COORD(1,IATOM,CCSET)*RED +
     $           COORD(1,JATOM,CCSET)*(1.0 - RED)
        YR(IA) = COORD(2,IATOM,CCSET)*RED +
     $           COORD(2,JATOM,CCSET)*(1.0 - RED)
        ZR(IA) = COORD(3,IATOM,CCSET)*RED +
     $           COORD(3,JATOM,CCSET)*(1.0 - RED)
      END DO
C
C     *** Convert to angstrom ***
C
      XR(:) = XR(:)/BOHR
      YR(:) = YR(:)/BOHR
      ZR(:) = ZR(:)/BOHR
C
C     *** Van der Vaals energy from a double loop ***
C
      CNTATM = 0
C
      DO IATOM=1,NATOM
C
C       *** Skip calculation for QM atoms with no VDW parameter ***
C
        IF (VLJRADQM(IATOM) == 0.0) CYCLE
C
        XIA = COORD(1,IATOM,CCSET)/BOHR
        YIA = COORD(2,IATOM,CCSET)/BOHR
        ZIA = COORD(3,IATOM,CCSET)/BOHR
C
        DO JATOM=1,NATMM
C
C         *** Skip calculation for MM atoms with no VDW parameter ***
C
          IF (VLJRAD(JATOM) == 0.0) CYCLE
C
          IF (VLJRAD(JATOM) == 0.0) CYCLE
CCC ADLL LA
C
C        *** Skip calculation for atoms at QM/MM covelent frontier ***
C
          IF (LAPAIR(IATOM,JATOM+NATOM)) CYCLE
CCC ADLL LA
          CNTATM = CNTATM + 1
          IF (MOD(CNTATM,NCPU).NE.ME) CYCLE
C
          RX = XIA - XR(JATOM)
          RY = YIA - YR(JATOM)
          RZ = ZIA - ZR(JATOM)
C
          R2 = RX*RX + RY*RY + RZ*RZ
C
          IF (R2.LE.VDWCUT2) THEN
            RMIN = RADQMMM(IATOM,JATOM)
            EPS = EPSQMMM(IATOM,JATOM)
C
            RMIN6 = RMIN**6
            R6 = R2**3
C
            E6 = RMIN6/R6    
            E12 = E6*E6
          ELSE
            CALL ERRMSG('D2QMMMLJ',
     $                  'DISTANCE R LARGER THAN VDW CUTOFF',1)
          END IF
C
          R = SQRT(R2)
          DER = EPS*(-12.0/R)*(E12 - E6)
          DER2 = EPS*(12.0/(R**2))*(13.0*E12 - 7.0*E6)
          RQM(1) = RX/R
          RQM(2) = RY/R
          RQM(3) = RZ/R
C
          DO I=1,3
            ICOORD = 3*IATOM - 3
            DO J=1,3
              JCOORD = 3*(JATOM+NATOM) - 3
              HESSE = RQM(I)*RQM(J)*(DER2-DER/R)
              IF (I.EQ.J) THEN
                HESSE = HESSE + DER/R
              END IF
              DE2(ICOORD+I,JCOORD+J) = DE2(ICOORD+I,JCOORD+J) - HESSE
              DE2(ICOORD+I,ICOORD+J) = DE2(ICOORD+I,ICOORD+J) + HESSE
              DE2(JCOORD+I,JCOORD+J) = DE2(JCOORD+I,JCOORD+J) + HESSE
            END DO
          END DO
        END DO
      END DO
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(XR,YR,ZR,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'D2QMMMLJ','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE D2QMMMLJ ***
C
      END
