      SUBROUTINE LINKGEO
C
C     Purpose: Link molecular geometry for molecular mechanics 
C              force field calculations.
C
C     History: - Creation (14.03.09, AG)
C                         (02.06.14, AMK)
C                         (13.11.14, GG)
C                         (27.01.17, TM)
C
C     ******************************************************************
C
C     List of local variables:
C
C     List of local dynamical fiels:
C
C     LNKVEC: Atomic link vector.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
CCC ADLL
      INCLUDE 'fileio.h'
CCC ADLL
      INCLUDE 'flags.h'
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
CCC ADLL LA for link atoms
      INCLUDE 'qmmm.h'
CCC ADLL LA

C
      CHARACTER*80 PRTSTR,STRCOMP
      INTEGER IATOM,JATOM
CCC ADLL
      INTEGER ILA,ILINK,NDANG
C
      INTEGER ALLOCATION
      INTEGER,ALLOCATABLE :: LNKVEC(:)
C
C     ------------------------------------------------------------------
C
C     *** Quick return, if linking of geometry is not requested ***
C
      IF (QMONLY) RETURN
C
C     *** Initialization ***
C
       CALL TIMER('START','LINK GEOMETRY ')
      NBOND = 0
C
C     *** Allocate local field ***
C
      ALLOCATE(LNKVEC(NATOM_ALL),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'LINKGEO','ALLOCATION FAILED')
C
C    *** Link atoms for molecular mechanics bonds ***
C
      DO 20 IATOM=NATOM+1,NATOM_ALL
C
        IF (TYPEMM(IATOM).LE.0) GO TO 20
        IF ((LINKMM(IATOM,0).EQ.1).AND.(LINKMM(IATOM,1).EQ.0)) GO TO 20
C
C     *** Build atomic link vector ***
C
        CALL BLDLNK(LNKVEC,IATOM,NDANG)
C
C     *** Reset atomic link information ***
C
        LINKMM(IATOM,:) = 0
C
C     *** Loop over all other MM atoms ***
C
        ILINK = 0
        DO 10 JATOM=NATOM+1,NATOM_ALL
C
          IF (TYPEMM(IATOM).LE.0) GO TO 10
C
          IF (LNKVEC(JATOM).GT.0) THEN
            ILINK = ILINK + 1
            IF (ILINK.GT.MAXLINK) THEN
              WRITE (PRTSTR,5000) ATOMMAP(IATOM,3),ILINK,MAXLINK
 5000         FORMAT ('ATOM ',I5,' HAS ',I2,' LINKS > MAXLINK = ',I2)
              CALL ERRMSG('LINKGEO',STRCOMP(PRTSTR),1)
            END IF
C
            LINKMM(IATOM,ILINK) = JATOM
C
C     *** Add molecular mechanics bond ***
C
            IF (JATOM.GT.IATOM) THEN
              NBOND = NBOND + 1
              CBOND(NBOND,1) = IATOM
              CBOND(NBOND,2) = JATOM
            END IF
          END IF
C
   10   CONTINUE
CCC ADLL LA
        DO 15 JATOM=1,NATOM
          IF (LNKVEC(JATOM).GT.0) THEN
            ILINK = ILINK + 1
            IF (ILINK.GT.MAXLINK) THEN
              WRITE (PRTSTR,5000) ATOMMAP(IATOM,3),ILINK,MAXLINK
              CALL ERRMSG('LINKGEO',STRCOMP(PRTSTR),1)
            END IF
            LINKMM(IATOM,ILINK) = JATOM
          END IF
   15   CONTINUE
C
        LINKMM(IATOM,0) = ILINK + NDANG
CCC ADLL 
C
      CALL DEFLUSH (OUT)
   20 CONTINUE
C
CCC   *** Link atoms for cut covalent QM/MM bonds (Link atoms) ***
C
      IF (LINKAT) THEN
C
        LINKLA(:,:) = 0
C
C       *** Identify atoms involved in covalent QM/MM frontiers ***
C
        CALL CHKQMMMBRD
C
        DO ILA=1,NLATOM
C
          IATOM = LATQM(ILA,1)
          JATOM = LATQM(ILA,2)
          LBOND(ILA,1) = IATOM
          LBOND(ILA,2) = JATOM
C
C        *** Build connectivity of IATOM on QM side **
C
          CALL BLDCONNECLA(IATOM,ILA)
C
        END DO
      END IF
CCC ADLL LA
C
      CALL TIMER('STOP','LINK GEOMETRY ')
C     *** Deallocate local matrix ***
C
      DEALLOCATE(LNKVEC,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'LINKGEO','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE LINKGEO *** 
C
      END

