      SUBROUTINE READQMPARAM
C
C     Purpose: Read VDW parameters from the prepared file FFDS
C              for the QM atoms in a QM-MM calculation.
C
C     History: - Creation (28.07.10, AG)
C                         (13.11.14, GG)
C                         (06.12.16, TM)
C
C     ******************************************************************
C
C     List of local variables:
C
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
CCC ADLL LA
      INCLUDE 'prm.h'
CCC ADLL LA
      INCLUDE 'qmmm.h'
C
      LOGICAL ERROR
      CHARACTER*80 DTS,EXTENS1,EXTENS2,EXTENS3,LINE,LTU,PRTSTR,STRCOMP,
     $             STRING,STRTYP*7
      CHARACTER*10 SYMBQM
      CHARACTER*80 TOKEN(80)
      INTEGER I,ICLASS,II,IVALUE,ITYPES,J,KLINE,LAST,NN,NO,NTOKEN,TYP,
     $        TYPM1,ZAT
CCC ADLL LA
      INTEGER CLASSLA(MAXTYPE,2),NCLASSLA,I1,I2,IJ,J1,J2,JJ,K,L
      REAL    BB,BR,FK
CCC ADLL LA
      REAL MASSAT,RED,RVALUE
C
C     ------------------------------------------------------------------
C
C     *** RETURN IF QM Params are not those of the deMon.prm file ***
C
CCC ADLL LA LA
      IF ((QMMMFF.NE.FFIELD).AND.(.NOT.LINKAT)) RETURN
CCC ADLL LA LA
C
C     *** Initialization ***
C
      NQMCLASS = 0
      CLASSOFTYP2(:) = 0
      CLASS2 = 0
C
C     *** Read the atom parameters for the needed types ***
C
      DO 500 ITYPES=1,NTYPQM 
        REWIND (IOFFD)
C
C     *** Go to the ATOM keyword lines ***
C
        DO J=1,KEYWLINES
          READ (IOFFD,'(A80)',ERR=99990) LINE
        END DO
C
        KLINE = KEYWLINES
        DO 50 I=1,ATOMLINES
          READ (IOFFD,'(A80)',ERR=99990) LINE
          STRING = STRCOMP(LTU(DTS(LINE)))
          CALL STRTOK(STRING,NTOKEN,TOKEN)
C
          CALL CUT(STRING,1,LAST)
          EXTENS1 = STRING(1:LAST)
          EXTENS2 = STRING(LAST+2:)
          CALL CUT(EXTENS2,1,LAST)
          EXTENS3 = EXTENS2(LAST+2:)
          EXTENS2 = EXTENS2(1:LAST)
C
          STRING = EXTENS1
          CALL STRTOK(STRING,NTOKEN,TOKEN)
C
C     *** Read Atom type of FF atoms ***
C
          CALL GETNUM(TOKEN(2),STRTYP,RVALUE,TYP)
          IF (STRTYP.NE.'INTEGER') THEN
            CALL MESSAGE(1,'MM ATOM TYPE MUST BE INTEGER',1,0)
            GO TO 99990
          ELSE IF (TYP.GT.MAXTYPE) THEN
            CALL MESSAGE(1,'NUMBER OF TYPES MUST BE < MAXTYPE',1,0)
            GO TO 99990
          END IF
C
C     *** Check if TYP fits with a TYPEMM value ***
C
          IF (TYP.NE.TYPEQM(ITYPES)) CYCLE
C
C     *** Read Atom Class of FF atoms ***
C
          CALL GETNUM(TOKEN(3),STRTYP,RVALUE,CLASSOFTYP2(TYP))
          IF (STRTYP.NE.'INTEGER') THEN
            CALL MESSAGE(1,'ATOM CLASS MUST BE INTEGER',1,0)
            GO TO 99990
          ELSE IF (CLASSOFTYP2(TYP).GT.MAXCLASS) THEN
            CALL MESSAGE(1,'CLASS NUMBER MUST BE < MAXCLASS',1,0)
            GO TO 99990
          END IF
C
C     *** Read Atom SYMBOL and Identification of FF atoms ***
C
          SYMBQM = TOKEN(4)
C
C     *** Read Atom Number and Mass of FF atoms ***
C
          STRING = STRCOMP(LTU(DTS(EXTENS3)))
          CALL STRTOK(STRING,NTOKEN,TOKEN)
          CALL GETNUM(TOKEN(1),STRTYP,RVALUE,ZAT)
          IF (STRTYP.NE.'INTEGER') THEN
            CALL MESSAGE(1,'TYPE ZVALUE MUST BE INTEGER',1,0)
            GO TO 99990
          END IF
          CALL GETNUM(TOKEN(2),STRTYP,MASSAT,IVALUE)
          IF (STRTYP.EQ.'INTEGER') THEN
            CALL MESSAGE(1,'MASS TYPE MUST BE REAL',1,0)
            GO TO 99990
          END IF
          CALL CHKLABEL(SYMBQM,NO,ERROR)
C
C     *** Read Number of Atoms linked to Atom Type ***
C
          IF (NTOKEN.GT.2) THEN
            CALL GETNUM(TOKEN(3),STRTYP,RVALUE,NN)
            IF (STRTYP.NE.'INTEGER') THEN
              CALL MESSAGE(1,'NUMBER OF NEIGHBOURS MUST BE INTEGER',1,0)
              GO TO 99990
            END IF
          END IF
C
   50   CONTINUE
C
  500 CONTINUE
C
C     *** Determine the number of QM atom classes in the QM system ***
C
      CLASS2(1) = CLASSOFTYP2(TYPEQM(1))
      NQMCLASS = 1
      DO ITYPES= 2,NTYPQM
        TYP = TYPEQM(ITYPES)
        TYPM1 = TYPEQM(ITYPES-1)
        IF (CLASSOFTYP2(TYP).NE.CLASSOFTYP2(TYPM1)) THEN
          NQMCLASS = NQMCLASS + 1
          CLASS2(NQMCLASS) = CLASSOFTYP2(TYP)
        END IF
      END DO
C
C     *** VDW keywords are read using the previous parameters ***
C
      KLINE = KEYWLINES + ATOMLINES
C
      IF (VDWLINES.NE.0) THEN
C
        DO 60 I=1,VDWLINES
C
          READ (IOFFD,'(A80)',ERR=99990) LINE
          STRING = STRCOMP(LTU(DTS(LINE)))
          CALL STRTOK(STRING,NTOKEN,TOKEN)
C
          CALL GETNUM(TOKEN(2),STRTYP,RVALUE,II)
          IF (STRTYP.NE.'INTEGER') THEN
            CALL MESSAGE(1,'MM VDW CLASS MUST BE INTEGER',1,0)
            GO TO 99990
          END IF
C
C     *** Read vdW parameter by TYPE ***
C
          IF (VDWINDX.EQ.'TYPE') THEN
C
            DO ITYPES=1,NTYPQM
              IF (II.EQ.TYPEQM(ITYPES)) THEN
                IF (TOKEN(1)(1:4).EQ.'VDW ') THEN
                  CALL GETNUM(TOKEN(3),STRTYP,VRADQM(ITYPES),IVALUE)
                  IF (STRTYP.EQ.'INTEGER') THEN
                    CALL MESSAGE(1,'VDWRAD MUST BE REAL',1,0)
                    GO TO 99990
                  END IF
                  CALL GETNUM(TOKEN(4),STRTYP,VEPSQM(ITYPES),IVALUE)
                  IF (STRTYP.EQ.'INTEGER') THEN
                    CALL MESSAGE(1,'VDW EN. MUST BE REAL',1,0)
                    GO TO 99990
                  END IF
                  IF (NTOKEN.GT.4) THEN
                    CALL GETNUM(TOKEN(4),STRTYP,RED,IVALUE)
                    IF (STRTYP.EQ.'INTEGER') THEN
                      CALL MESSAGE(1,'VDW FACTOR. MUST BE REAL',1,0)
                      GO TO 99990
                    END IF
                  END IF
                END IF
              END IF
            END DO
C
C     *** Read vdW parameter by CLASS ***
C
          ELSE IF (VDWINDX.EQ.'CLASS') THEN
C
            DO ICLASS=1,NQMCLASS
              IF (II.EQ.CLASS2(ICLASS)) THEN
                IF (TOKEN(1)(1:4).EQ.'VDW ') THEN
                  CALL GETNUM(TOKEN(3),STRTYP,VRADQM(ICLASS),IVALUE)
                  IF (STRTYP.EQ.'INTEGER') THEN
                    CALL MESSAGE(1,'VDWRAD MUST BE REAL',1,0)
                    GO TO 99990
                  END IF
                  CALL GETNUM(TOKEN(4),STRTYP,VEPSQM(ICLASS),IVALUE)
                  IF (STRTYP.EQ.'INTEGER') THEN
                    CALL MESSAGE(1,'VDW EN. MUST BE REAL',1,0)
                  GO TO 99990
                  END IF
                  IF (NTOKEN.GT.4) THEN
                    CALL GETNUM(TOKEN(4),STRTYP,RED,IVALUE)
                    IF (STRTYP.EQ.'INTEGER') THEN
                      CALL MESSAGE(1,'VDW FACTOR. MUST BE REAL',1,0)
                      GO TO 99990
                    END IF
                  END IF
                END IF
              END IF
            END DO
C
C     *** Error handling ***
C
          ELSE
C
            CALL ERRMSG('READQMPARAM','UNKNOWN VDW-INDEXING OPTION: '//
     $                  TRIM(VDWINDX),1)
C
          END IF
C
   60   CONTINUE
C
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
99990 CONTINUE
      WRITE (PRTSTR,'("ERROR IN IOFFD FILE LINE: ",I10)') KLINE+I
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READQMPARAM',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE READQMPARAM ***
C
      END
