      SUBROUTINE SYNCHK
C
C     Purpose: Syntax checker for the deMon input.
C
C     History: - Creation (13.03.03, AMK)
C
C     ******************************************************************
C
C     List of local variables:
C
C     INPLIST: Input list.
C     MULLIST: Multiple appearence list.
C     NLIST  : Number of list elements.
C     STRING : Read string.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'keywords.h'
      INCLUDE 'molecule.h'
C
      LOGICAL IRCPATH,SKIPLINE,SUCCESS
      CHARACTER*500 DTS,LTU,STRCOMP,STRING
      CHARACTER*80 LINE,NEXTLINE*81,PRTSTR,RESET*10
      INTEGER ILINE,IOPTION,LINENO,LINES,NLIST,NOPTION,STREXT
C
CCC ADLL QMMM
      LOGICAL EXTFILE
CCC ADLL QMMM
      LOGICAL MULLIST(MAXKEY)
      CHARACTER*80 OPTIONS(MAXOPT)
      CHARACTER*32 INPLIST(MAXKEY+4*MAXATOM_ALL)
C
C     ------------------------------------------------------------------
C
C     *** Initialize checker status ***
C
      RESET = STATUS
      STATUS = 'CHECK'
C
C     ------------------------------------------------------------------
C
C     *** Check line length ***
C
      REWIND (INP)
C
      LINENO = 0
C
  500 CONTINUE
C
        LINENO = LINENO + 1
        READ (INP,'(A)',ERR=99960,END=600) STRING
        IF (SKIPLINE(LTU(DTS(STRING)))) GO TO 500
        IF (STREXT(STRING).GT.MAXCHAR) THEN
          CALL MESSAGE(0,'INPUT ERROR DETECTED',1,0)
          CALL MESSAGE(0,'--> '//TRIM(STRING),1,0)
          WRITE (PRTSTR,5000) LINENO,MAXCHAR
 5000     FORMAT ('LINE ',I4,' HAS MORE THAN ',I4,' CHARACTER')
          CALL ERRMSG('SYNCHK',STRCOMP(PRTSTR),1)
        ELSE IF ((STREXT(STRING).EQ.1).AND.(STRING(1:1).EQ.' ')) THEN
          CALL MESSAGE(0,'INPUT ERROR DETECTED',1,0)
          WRITE (PRTSTR,5010) LINENO
 5010     FORMAT ('LINE ',I4,' IS EMPTY! PLEASE DELETE OR COMMENT')
          CALL ERRMSG('SYNCHK',STRCOMP(PRTSTR),1)
        END IF
C
        GO TO 500
C
  600 CONTINUE
C
C     ------------------------------------------------------------------
C
C     *** Check maximum number of continuation lines ***
C
      REWIND (INP)
C
      LINENO = 0
C
  700 CONTINUE
C
        LINES = 1
        LINENO = LINENO + 1
        READ (INP,'(A80)',ERR=99990,END=800) LINE
        IF (SKIPLINE(LTU(DTS(LINE)))) GO TO 700
        STRING = STRCOMP(LTU(DTS(LINE)))
        DO WHILE (LINE(STREXT(LINE):STREXT(LINE)).EQ.NEWLINE)
          READ (INP,'(A80)',ERR=99990,END=800) LINE
          NEXTLINE = ' '//STRCOMP(LTU(DTS(LINE)))
          STRING = STRCOMP(STRING(:STREXT(STRING)-1)//NEXTLINE)
          LINES = LINES + 1
          LINENO = LINENO + 1
        END DO
C
        IF (LINES.GT.MAXLINE) THEN
          DO ILINE=1,LINES
            BACKSPACE (INP)
          END DO
          READ (INP,'(A80)',ERR=99990,END=800) LINE
          CALL MESSAGE(0,'INPUT ERROR DETECTED',1,0)
          CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
          WRITE (PRTSTR,5020) LINENO-LINES+1,MAXLINE
 5020     FORMAT ('LINE ',I4,' HAS MORE THAN ',I4,' CONTINUATION LINES')
          CALL ERRMSG('SYNCHK',STRCOMP(PRTSTR),1)
        END IF
C
        GO TO 700
C
  800 CONTINUE
C
C     ------------------------------------------------------------------
C
C     *** Check REACTANT and PRODUCT keywords ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('GEOMETRY',SUCCESS)
      IF (.NOT.SUCCESS) THEN
        CALL FINDKEY('REACTANT',SUCCESS)
        IF (.NOT.SUCCESS) THEN
          CALL MESSAGE(0,'INPUT ERROR DETECTED',1,0)
          CALL FINDKEY('SADDLE',SUCCESS)
          IF (SUCCESS) THEN
            CALL MESSAGE(0,'KEYWORD REACTANT MUST BE SPECIFIED',1,0)
          ELSE
            CALL MESSAGE(0,'KEYWORD GEOMETRY MUST BE SPECIFIED',1,0)
          END IF
          CALL ERRMSG('SYNCHK','MAYBE THE INPUT FILE IS EMPTY',1)
        ELSE
          CALL FINDKEY('PRODUCT',SUCCESS)
          IF (.NOT.SUCCESS) THEN
            CALL MESSAGE(0,'INPUT ERROR DETECTED',1,0)
            PRTSTR = 'REACTANT AND PRODUCT GEOMETRY MUST BE SPECIFIED'
            CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
            CALL ERRMSG('SYNCHK','MAYBE THE INPUT FILE IS EMPTY',1)
          END IF
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Initialize input & multiple appearence list with keywords ***
C
      NLIST = MAXKEY
      INPLIST(1:NLIST) = KEYWORDS(1:NLIST)%FULL_KEYWORD
      MULLIST(1:NLIST) = KEYWORDS(1:NLIST)%MULTIPLE_APPEARENCE
C
C     *** Read and check geometry input ***
C
      IRCPATH = .FALSE.
      SUCCESS = .FALSE.
      CALL FINDKEY('IRC',IRCPATH)
      CALL FINDKEY('GEOMETRY',SUCCESS)
      CALL FINDOPT('GEOMETRY',OPTIONS,NOPTION)
      CALL READUNITS(OPTIONS,NOPTION,'GEOMETRY')
C
C     *** Check GEOMETRY input ***
C
      IF (SUCCESS) THEN
C
        SUCCESS = .FALSE.
        CALL CHKOPT('ZMATR',OPTIONS,NOPTION,SUCCESS)
        CALL CHKOPT('Z-MAT',OPTIONS,NOPTION,SUCCESS)
CCC ADLL QMM
        EXTFILE = .FALSE.
        CALL CHKOPT('GFILE',OPTIONS,NOPTION,EXTFILE)
        FGEO = .FALSE.
        IF (EXTFILE) FGEO = .TRUE.
CCC ADLL QMMM
C
        IF (SUCCESS) THEN
          INPTYP = 'INTERNAL'
          CALL CHKZMAT
          IF (IRCPATH) CALL READIRC('ZMATRIX')
        ELSE
          CALL CHKOPT('MIXED',OPTIONS,NOPTION,SUCCESS)
          IF (SUCCESS) THEN
            INPTYP = 'MIXED'
            CALL CHKCART
            CALL CHKZMAT
            IF (IRCPATH) CALL READIRC('MIXED')
          ELSE
            INPTYP = 'CARTESIAN'
            CALL CHKCART
            IF (IRCPATH) CALL READIRC('CARTESIAN')
          END IF
        END IF
C
C     *** Check REACTANT and PRODUCT input ***
C
      ELSE
C
        CALL FINDKEY('REACTANT',SUCCESS)
        CALL FINDOPT('REACTANT',OPTIONS,NOPTION)
        CALL READUNITS(OPTIONS,NOPTION,'REACTANT')
C
C     *** Check REACTANT input ***
C
        IF (SUCCESS) THEN
C
          CALL CHKEDU(OPTIONS,NOPTION)
C
        ELSE
C
          CALL MESSAGE(0,'GEOMETRY OR REACTANT DEFINITION MISSING',1,1)
          CALL ERRMSG('SYNCHK','GEOMETRY DEFINITION PROBLEM OCCURED',1)
C
        END IF
C
        CALL FINDKEY('PRODUCT',SUCCESS)
        CALL FINDOPT('PRODUCT',OPTIONS,NOPTION)
        CALL READUNITS(OPTIONS,NOPTION,'PRODUCT')
C
C     *** Check PRODUCT input ***
C
        IF (SUCCESS) THEN
C
          CALL CHKPRO(OPTIONS,NOPTION)
          CALL CHKUNITS(OPTIONS,NOPTION)
C
        ELSE
C
          CALL MESSAGE(0,'GEOMETRY OR PRODUCT DEFINITION MISSING',1,1)
          CALL ERRMSG('SYNCHK','GEOMETRY DEFINITION PROBLEM OCCURED',1)
C
        END IF
C
      END IF
C
C     *** Add variables and constants to input list ***
C
      CALL ADDLIST(INPLIST,NLIST,'VARIABLES')
C
C     *** Add atomic labels to input list ***
C
      CALL ADDLIST(INPLIST,NLIST,'LABELS')
C
C     *** Add atomic symbols to input list ***
C
      CALL ADDLIST(INPLIST,NLIST,'SYMBOLS')
C
C     *** Check input list ***
C
      CALL CHKLIST(INPLIST,MULLIST,NLIST)
C
C     ------------------------------------------------------------------
C
C     *** Check title input ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('TITLE',SUCCESS)
C
      IF (SUCCESS) THEN
        BACKSPACE (INP)
        READ (INP,'(A80)',ERR=99970) LINE
        IF (LINE(STREXT(LINE):STREXT(LINE)).EQ.NEWLINE) THEN
          CALL ERRMSG('SYNCHK','CONTINUATION OF TITLE NOT PERMITTED',1)
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check simulation input ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('SIMULATION',SUCCESS)
C
      IF (SUCCESS) THEN
        CALL FINDOPT('SIMULATION',OPTIONS,NOPTION)
        SUCCESS = .FALSE.
        DO IOPTION=1,NOPTION
          IF (OPTIONS(IOPTION)(1:5).EQ.'ANALY') SUCCESS = .TRUE.
          IF (OPTIONS(IOPTION)(1:5).EQ.'CALCU') SUCCESS = .TRUE.
        END DO
        IF (.NOT.SUCCESS) THEN
          WRITE (PRTSTR,5030)
 5030     FORMAT ('< SIMULATION > MUST HAVE OPTION ANALYSIS OR ',
     $            'CALCULATE SPECIFIED!')
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check intrinsic reaction coordinate and Hessian input ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('IRC',SUCCESS)
C
      IF (SUCCESS) THEN
        IRCPATH = .TRUE.
        CALL FINDOPT('IRC',OPTIONS,NOPTION)
        DO IOPTION=1,NOPTION
          IF (OPTIONS(IOPTION)(1:4).EQ.'READ') IRCPATH = .FALSE.
        END DO
        IF (IRCPATH) THEN
          SUCCESS = .FALSE.
          CALL FINDKEY('HESSIAN',SUCCESS)
          IF ((.NOT.SUCCESS).AND.(RESET.EQ.'READ IN')) THEN
            WRITE (PRTSTR,5040)
 5040       FORMAT ('USE < IRC > ONLY IN COMBINATION WITH ',
     $              '< HESSIan >')
            CALL ERRMSG('SYNCHK',PRTSTR,1)
          END IF
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check thermodynamic data and frequency input ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('THERMO',SUCCESS)
C
      IF (SUCCESS) THEN
        SUCCESS = .FALSE.
        CALL FINDKEY('FREQUENCY',SUCCESS)
        IF (.NOT.SUCCESS) THEN
          WRITE (PRTSTR,5050)
 5050     FORMAT ('USE < THERMo > ONLY IN COMBINATION WITH ',
     $            '< FREQUency >')
          CALL ERRMSG('SYNCHK',PRTSTR,1)
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check plot keyword exclusiveness ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('PLOT',SUCCESS)
C
      IF (SUCCESS) THEN
        SUCCESS = .FALSE.
        CALL FINDKEY('GEOSURFACE',SUCCESS)
        IF (SUCCESS) THEN
          WRITE (PRTSTR,5060)
 5060     FORMAT ('< PLOT > AND < GEOSUrface > KEYWORD ',
     $            'ARE INCOMPATIBLE')
          CALL ERRMSG('SYNCHK',PRTSTR,1)
        END IF
        CALL FINDKEY('ISOSURFACE',SUCCESS)
        IF (SUCCESS) THEN
          WRITE (PRTSTR,5070)
 5070     FORMAT ('< PLOT > AND < ISOSUrface > KEYWORD ',
     $            'ARE INCOMPATIBLE')
          CALL ERRMSG('SYNCHK',PRTSTR,1)
        END IF
      END IF
C
C     *** Check geosurface keyword exclusiveness ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('GEOSURFACE',SUCCESS)
C
      IF (SUCCESS) THEN
        SUCCESS = .FALSE.
        CALL FINDKEY('ISOSURFACE',SUCCESS)
        IF (SUCCESS) THEN
          WRITE (PRTSTR,5080)
 5080     FORMAT ('< GEOSUrface > AND < ISOSUrface > KEYWORD ',
     $            'ARE INCOMPATIBLE')
          CALL ERRMSG('SYNCHK',PRTSTR,1)
        END IF
      END IF
C
C     *** Check box keyword exclusiveness ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('BOX',SUCCESS)
C
      IF (SUCCESS) THEN
        SUCCESS = .FALSE.
        CALL FINDKEY('POINTS',SUCCESS)
        IF (SUCCESS) THEN
          WRITE (PRTSTR,5090)
 5090     FORMAT ('< BOX > AND < POINTs > KEYWORD ',
     $            'ARE INCOMPATIBLE')
          CALL ERRMSG('SYNCHK',PRTSTR,1)
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check isosurface keyword option ***
C
      SUCCESS = .FALSE.
      CALL FINDKEY('ISOSURFACE',SUCCESS)
C
      IF (SUCCESS) THEN
        CALL FINDOPT('ISOSURFACE',OPTIONS,NOPTION)
        SUCCESS = .FALSE.
        DO IOPTION=1,NOPTION
          IF (OPTIONS(IOPTION)(1:4).EQ.'ISO=') THEN
            SUCCESS = .TRUE.
          END IF
        END DO
        IF (.NOT.SUCCESS) THEN
          WRITE (PRTSTR,6000)
 6000     FORMAT ('ISOVALUE < ISO= > IS MANDATORY FOR ',
     $            '< ISOSUrface > KEYWORD')
          CALL ERRMSG('SYNCHK',PRTSTR,1)
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Reset globale variables, fields and rewind input tape ***
C
      NATOM = 0
      NDUMMY = 0
      ATOMNO(:) = 0
      OPTXYZ(:) = ' '
      ATOMLAB(:) = 'UNDEFINED'
      OPTVAR(:,:) = 'UNDEFINED'
      OPTFLAG(:,:) = 'UNDEFINED'
C
      REWIND (INP)
C
      STATUS = RESET
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
99960 CONTINUE
      WRITE (PRTSTR,'("ERROR READING INPUT LINE ",I4)') LINENO
      CALL MESSAGE(0,'--> '//TRIM(STRING),1,0)
      CALL ERRMSG('SYNCHK',STRCOMP(PRTSTR),1)
C
99970 CONTINUE
      WRITE (PRTSTR,'("ERROR READING TITLE ")')
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('SYNCHK',STRCOMP(PRTSTR),1)
C
99990 CONTINUE
      WRITE (PRTSTR,'("ERROR READING INPUT LINE ",I4)') LINENO
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('SYNCHK',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE SYNCHK ***
C
      END
