      SUBROUTINE VERLET
C
C     Purpose: Do a (velocity) verlet time step.
C
C     History: - Creation (27.02.04, AMK)
C
C     Lit.: L. Verlet, Phys. Rev. 159, 98 (1967).
C           W.C. Swope, H.C. Andersen, P.H. Berens, 
C           K.R. Wilson, J. Chem. Phys. 76, 637 (1982)
C
C     ******************************************************************
C
C     List of local variables:
C
C     ACCEL: Nuclear accelerations.
C     COORD: Nuclear coordinates.
C     MDCYC: Molecular dynamics cycle.
C     MDINT: Step interval for MD printing.
C     VELOS: Nuclear velocities.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'dynamics.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
C
      CHARACTER*80 PRTSTR,STRCOMP
      INTEGER IATOM,IXYZ
C
      LOGICAL MDFLUSH
C
C     ------------------------------------------------------------------
C
C     *** Calculate mid-step nuclear velocities ***
C
      DO IXYZ=1,3
        DO IATOM=1,NATOM_ALL
          VELOS(IXYZ,IATOM) = (2.0*THOOVER - 1.0)*VELOS(IXYZ,IATOM) +
     $                        0.5*ACCEL(IXYZ,IATOM)*MDSTEP - 
     $                        0.5*FRICTION*VELOS(IXYZ,IATOM)*MDSTEP
        END DO
      END DO
C
C     *** Apply mid-velocity conditions ***
C
      CALL MIDVEL
      IF (MDVLP) CALL MODVEL
C
C     *** Print mid-step nuclear velocities ***
C
      IF (MDFLUSH(MDCYC,MDINT)) THEN
        IF (PRTMD.AND.PRTOPT) THEN
          WRITE (PRTSTR,5000) MDCYC
 5000     FORMAT ('MID-STEP VELOCITIES OF TIME STEP ',I5)
          CALL PRIVEL(STRCOMP(PRTSTR))
        END IF
      END IF
C
C     *** Calculate/store new/old molecular geometry ***
C
      DO IXYZ=1,3
        DO IATOM=1,NATOM_ALL
          COORD(IXYZ,IATOM,5) = COORD(IXYZ,IATOM,CCSET)
          COORD(IXYZ,IATOM,CCSET) = COORD(IXYZ,IATOM,CCSET) + 
     $                              VELOS(IXYZ,IATOM)*MDSTEP
        END DO
      END DO
C
C     *** Apply geometry constrains ***
C
      CALL SHAKE
C
CCC ADLL LA
C     *** Set link atom position ***
C
      CALL SETLAPOS('MD')
C
CCC ADLL LA
C     *** Calculate new potential energy ***
C
      CALL MDSCF('STEP')
C
C     *** Calculate new nuclear accelerations ***
C
      CALL ACCELERATE
C
C     *** Calculate new velocity conditions ***
C
      CALL NOSE(MDTEMP)
C
C     *** Calculate new nuclear velocities ***
C
      DO IXYZ=1,3
        DO IATOM=1,NATOM_ALL
          VELOS(IXYZ,IATOM) = (VELOS(IXYZ,IATOM) + 
     $                         0.5*ACCEL(IXYZ,IATOM)*MDSTEP)/
     $                         (0.5*FRICTION*MDSTEP + 1.0)
        END DO
      END DO
      IF (MDVLP) CALL MODVEL
C
C     *** Print new nuclear velocities and momenta ***
C
      IF (MDFLUSH(MDCYC,MDINT)) THEN
        IF (PRTMD.AND.PRTOPT) THEN
          WRITE (PRTSTR,5010) MDCYC
 5010     FORMAT ('VELOCITIES OF TIME STEP ',I5)
          CALL PRIVEL(STRCOMP(PRTSTR))
          WRITE (PRTSTR,5020) MDCYC
 5020     FORMAT ('MOMENTUM OF TIME STEP ',I5)
          CALL PRIMOM(STRCOMP(PRTSTR))
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE VERLET ***
C
      END
