      SUBROUTINE D1JCUR(PI,D1STO,D2STO,STO,DSTO,D1J,NGPB)
C
C     Purpose: Compute current density vector on current batch of grid
C              points 
C
C     History: - Creation (13.03.13, ADLL)
C
C     ******************************************************************
C
C     List of wrapper arguments:
C
C     PA    : Alpha MO coefficient matrix.
C     PA2   : Beta MO coefficient matrix.
C     DSTO  : Dimension of Slater typ orbital matrix.
C     J     : Vector with current density
C     LLGPB : Lower limit grid point of current batch.
C     NGPB  : Number of grid points in current batch.
C
C     List of dynamical fields:
C
C     D1STO: First derivatives of the basis functions.
C     STO  : Function values of the basis functions.
C
C     Output list:    Open shell   
C
C     FUNVEC(IGP,2):  Jx 
C     FUNVEC(IGP,3):  Jy
C     FUNVEC(IGP,4):  Jz
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
      INCLUDE 'threshold.h'
C
      INTEGER I,ISTO,JSTO,DSTO,IATOM,JATOM,INEIGH,JNEIGH
      REAL PI(DSTO,DSTO)
C
      REAL D1STO(MAXGPB,3,DSTO),D2STO(MAXGPB,6,DSTO)
      REAL STO(MAXGPB,DSTO),D1J(MAXGPB,3)
C
      INTEGER IGP,LLGPB,NGPB
      REAL FACTOR
      REAL JDSTO(MAXGPB,3),JVSTO(MAXGPB)
C
C     ------------------------------------------------------------------
C
C     *** Calculate alpha density and density gradient norm ***
C
      D1J(:,:) = 0.0
C
      DO INEIGH=1,NNEIGH
        IATOM = NEIGH(INEIGH)
        DO ISTO=LL(IATOM),UL(IATOM)
          JDSTO(1:NGPB,:) = 0.0
          JVSTO(1:NGPB) = 0.0
          DO JNEIGH=1,NNEIGH
            JATOM = NEIGH(JNEIGH)
            DO JSTO=LL(JATOM),UL(JATOM)
              DO I=1,3
                JDSTO(1:NGPB,I) = JDSTO(1:NGPB,I) + 2*PI(JSTO,ISTO)*
     $                            D1STO(1:NGPB,I,JSTO) 
                JVSTO(1:NGPB) = JVSTO(1:NGPB) + 2*PI(JSTO,ISTO)*
     $                         STO(1:NGPB,JSTO)
              END DO
            END DO
          END DO
          DO I=1,3
            D1J(1:NGPB,I) = D1J(1:NGPB,I) + 
     $                    JDSTO(1:NGPB,I)*D1STO(1:NGPB,I,ISTO) 
            IF (I.EQ.1) THEN
              D1J(1:NGPB,I) = D1J(1:NGPB,I) + JVSTO(1:NGPB)*
     $                        D2STO(1:NGPB,1,ISTO)
            ELSE IF (I.EQ.2) THEN
              D1J(1:NGPB,I) = D1J(1:NGPB,I) + JVSTO(1:NGPB)*
     $                        D2STO(1:NGPB,4,ISTO)
            ELSE IF (I.EQ.3) THEN
              D1J(1:NGPB,I) = D1J(1:NGPB,I) + JVSTO(1:NGPB)*
     $                        D2STO(1:NGPB,6,ISTO)
            END IF
          END DO
        END DO
      END DO
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE JCUR ***
C
      END
