      SUBROUTINE DIPCOR(CYC)
C
C     Purpose: Calculate dipole on each molecule and compute the
C              autocorrealation function.
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'property.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'projectile.h'
C
      INTEGER CYC 
C
      INTEGER ALLOCATION,IATOM,I,IMOL
      REAL DPOLE,QPOLE, NORM,SP,TIME
      CHARACTER*80 STRING,SYSTEM
C
      REAL,ALLOCATABLE :: QFR(:),MU(:,:),KIN(:,:),SFR(:)
C
C     -----------------------------------------------------------------
C
C     *** Allocate dynamical arrays ***
C
      ALLOCATE (QFR(NMOL),MU(NMOL,3),KIN(NMOL,3),
     $          SFR(NMOL),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) CALL ERRMSG('DIPCOR','ALLOCATION FAILED',1)
C     
C     *** Loop over MM atoms ***
C
      IF (UNIT(1).EQ.'ANGSTROM') THEN
        DPOLE = ESU(1)
        QPOLE = ESU(2)
        SYSTEM = 'ESU'
      ELSE
        DPOLE = 1.0
        QPOLE = 1.0
        SYSTEM = 'AU'
      END IF

      QFR(:) = 0.0
      SFR(:) = 0.0
      MU(:,:) = 0.0
      KIN(:,:) = 0.0
C
      IF (MMONLY.OR.QMMM) THEN
        DO IATOM=1,NATMM-NRADP
        IMOL = FRAGMOL(IATOM+NATOM)
          MU(IMOL,:) = MU(IMOL,:) + CHGMM(IATOM)*
     $                 COORD(:,IATOM+NATOM,CCSET) 
          QFR(IMOL) =  QFR(IMOL) + CHGMM(IATOM)
        END DO
      END IF
C
C     *** Loop over QM atoms ***
C
      IF (.NOT.MMONLY) THEN
        DO IATOM=1,NATOM
          IMOL = FRAGMOL(IATOM)
          MU(IMOL,:) = MU(IMOL,:) + QATOM(IATOM)*COORD(:,IATOM,CCSET)
          QFR(IMOL) =  QFR(IMOL) + QATOM(IATOM)
          SFR(IMOL) = SFR(IMOL) + QSPIN(IATOM)
          KIN(IMOL,1) = KIN(IMOL,1) + KBASIN(IATOM)
          KIN(IMOL,2) = KIN(IMOL,2) + GBASIN(IATOM)
          KIN(IMOL,3) = KIN(IMOL,3) + LBASIN(IATOM)
          DO I=1,3
            MU(IMOL,I) = MU(IMOL,I) + MBASIN(I+1,IATOM)
          END DO
        END DO
C
      END IF
C
C     *** Print fragment properties to RTP output file ***
C
      STRING = 'FRAGMENT PROPERTIES'
      TIME = 1.0E3*RTT/FSEC
      DO IMOL=1,NMOL
        IF (TDEKIN.AND.OKS) THEN
          WRITE (RTP,4000) TIME,STRING,IMOL,QFR(IMOL),SFR(IMOL),
     $                     KIN(IMOL,1),KIN(IMOL,2),KIN(IMOL,3)
4000  FORMAT (T2,F12.3,1X,A19,1X,I4,1X,2(F8.4,1X),3(ES12.5,1X))
        ELSE IF (TDEKIN) THEN
          WRITE (RTP,4010) TIME,STRING,IMOL,QFR(IMOL),
     $                     KIN(IMOL,1),KIN(IMOL,2),KIN(IMOL,3)
4010  FORMAT (T2,F12.3,1X,A19,1X,I4,1X,F8.4,1X,3(ES12.5,1X))
        ELSE IF (OKS) THEN
          WRITE (RTP,4020) TIME,STRING,IMOL,QFR(IMOL),SFR(IMOL)
4020  FORMAT (T2,F12.3,1X,A19,1X,I4,1X,2(F8.4,1X))
        ELSE
          WRITE (RTP,4030) TIME,STRING,IMOL,QFR(IMOL)
4030  FORMAT (T2,F12.3,1X,A19,1X,I4,1X,F8.4)
        END IF
      END DO
C
      TIME = 1.0E3*RTT/FSEC
      IF (CYC.EQ.0) THEN
C
        SP = 1.0
        DO IMOL=1,NMOL
          NORM = MU(IMOL,1)**2 + MU(IMOL,2)**2 +  MU(IMOL,3)**2
          IF (FULLPA) THEN
            WRITE (RTP,5000) TIME,'FRAGMENT DIPOLE ',IMOL,
     $           (DPOLE*MUMOL(IMOL,I),I=1,3,SQRT(NORM))
          END  IF
        END DO
C
C     *** Update dipole moments ***
C
      ELSE 
C
        DO IMOL=1,NMOL
          NORM = MU(IMOL,1)**2 + MU(IMOL,2)**2 +  MU(IMOL,3)**2
          SP = 0.0
          DO I=1,3
            SP = SP + MUMOL(IMOL,I)*MU(IMOL,I)
          END DO 
C
C      *** Write output ***
C
          IF (FULLPA) THEN
          WRITE (RTP,5000) TIME,'FRAGMENT DIPOLE ',IMOL,
     $                  (DPOLE*MU(IMOL,I),I=1,3),SQRT(NORM)
          END IF
5000  FORMAT (T2,F12.3,1X,A,1X,I4,1X,5(F8.4,1X))
        END DO
      END IF
C
C     *** Deallocate dynamical arrays ***
C
      DEALLOCATE (QFR,MU,KIN,SFR,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) CALL ERRMSG('DIPCOR','DEALLOCATION FAILED',1)
C
C     -------------------------------------------------------------------- 
C
C     *** End of SUBROUTINE DIPCOR ***
C
      END
