      SUBROUTINE DOTDWFN(PAR,PAI,PBR,PBI,DSTO)
C
C     Purpose: write WFN files along RTDFT dynamics. 
C
C     History: - Creation (11.03.13, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'plot.h'
      INCLUDE 'physic.h'
      INCLUDE 'numint.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scfvec.h'
C
      INTEGER DSTO
      REAL PAR(DSTO,DSTO),PAI(DSTO,DSTO),PBR(DSTO,DSTO),PBI(DSTO,DSTO)
C
      INTEGER LLVAR_ME,ULVAR_ME
C
      INTEGER ALLOCATION,ANACYC,ICOOR,IRAD,NARUN,OFFSET,REDCYC
      REAL CE(3),CLAU,D,MC2,SCALFAC
      CHARACTER*30 CHAIN,ELFILE
      CHARACTER*22 TIMSTR 
C
      LOGICAL MDFLUSH
      REAL , ALLOCATABLE :: NCDA(:),NCDB(:),RDUM(:)
      COMPLEX (KIND=8) EYE
      COMPLEX(KIND=8), ALLOCATABLE :: P(:,:)
      COMPLEX(KIND=8), ALLOCATABLE :: WORK1(:,:),WORK2(:,:)
      COMPLEX(KIND=8),ALLOCATABLE :: S(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Quick return if no function plotting requested ***
C
      IF (.NOT.TDWFN) RETURN
C
C     *** Initialization ***
C
      IF (ADDRAD) CALL IONRADINI
      ALLOCATE(NCDA(NAUX),NCDB(NAUX),STAT=ALLOCATION)
      ALLOCATE(P(DSTO,DSTO),STAT=ALLOCATION)
      ALLOCATE(S(DSTO,DSTO),STAT=ALLOCATION)
      ALLOCATE(WORK1(DSTO,DSTO),WORK2(DSTO,DSTO))

      CALL SCFIO(REAL(S),DSTO,'READ','OVERLAP')
C      CALL PRIMAT(REAL(S(:,:)),DSTO,1,DSTO,1,DSTO,3,5,OUT,
C     $     'BASIS','TOTAL','OVER0')
      ALLOCATE (RDUM(DSTO),STAT=ALLOCATION)
      RDUM(:) = 0.0
      s(:,:)=0.0
      work1(:,:) = 0.0
      work2(:,:) = 0.0
C
C     *** Loop over propagation steps ***
C
      NARUN = MIN(RTMAX+FGRTCYC,RTAMAX) - MAX(RTAMIN,FGRTCYC)
      IF (FGRTCYC.EQ.0) NARUN = NARUN + 1
C
      CALL MPLIMITS(NCPU,ME,1,0,NARUN,LLVAR_ME,ULVAR_ME,
     $              (/0/),0)
C
      REWIND(IORTP)
C
      DO 10 RTCYC=1,RTMAX
C
        ANACYC = RTCYC + FGRTCYC
        REDCYC = RTCYC
        IF ((ANACYC.LT.RTAMIN).OR.(ANACYC.GT.RTAMAX)) GO TO 10
        IF ((ANACYC.NE.1).AND.(.NOT.(MDFLUSH(ANACYC,RTPRO)))) GO TO 10
        IF ((REDCYC.LT.LLVAR_ME).OR.(REDCYC.GT.ULVAR_ME)) GO TO 10
 20     CONTINUE
C
C    *** Read density matrix from tape ***
C
        CALL RTPIO(PAR,NCDA,DSTO,NAUX,'READ')
        CALL STOPMAT(PAR,PAR,PAI,DSTO,'EXPAND')
C
        IF (OKS) THEN
          SCALFAC = 1.0
        ELSE
          SCALFAC = 0.5
        END IF
C
C       *** Complex Choleski decomposition on P ***
C
        CALL SCALMAT(PAR,DSTO,DSTO,DSTO,'M = N',SCALFAC)
        CALL SCALMAT(PAR,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
        CALL SCALMAT(PAI,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
        EYE = CMPLX(0,1)
        P(:,:) = PAR(:,:) + EYE*PAI(:,:)
        CALL CHLMOSZ(P,DSTO,NOMOA,'OCC')
        CALL ZGEMM('T','N',DSTO,DSTO,DSTO,1.0,P,DSTO,S,DSTO,0.0,
     $       WORK1,DSTO)
        CALL ZGEMM('N','N',DSTO,DSTO,DSTO,1.0,WORK1,DSTO,P,
     $   DSTO,0.0,WORK2,DSTO)
C        CALL PRIMAT(REAL(WORK2(:,:)),DSTO,1,DSTO,1,DSTO,3,5,OUT,
C     $     'BASIS','TOTAL','OVER') 
 
C        write(OUT,*) "PRINT"
C
C     *** Assign real and imaginary parts ***
C
        PAR(:,:) = REAL(P(:,:))
        PAI(:,:) = AIMAG(P(:,:))
C
        IF (OKS) THEN
          CALL RTPIO(PBR,NCDB,DSTO,NAUX,'READ')
          CALL SCALMAT(PBR,DSTO,DSTO,DSTO,'M = N',SCALFAC)
          CALL SCALMAT(PBR,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
          CALL SCALMAT(PBI,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
          EYE = CMPLX(0,1)
          P(:,:) = PBR(:,:) + EYE*PBI(:,:)
          CALL CHLMOSZ(P,DSTO,NOMOA,'OCC')
          PBR(:,:) = REAL(P(:,:))
          PBI(:,:) = AIMAG(P(:,:))
        END IF
C
C     *** Open binary plot file ***
C
        CLOSE (MOL,STATUS='KEEP')
        WRITE (CHAIN,*) RTCYC+FGRTCYC
        RTT = RTSTEP*(RTCYC+FGRTCYC)
        ELFILE='om_'//ADJUSTL(CHAIN)
        ELFILE=ADJUSTL(ADJUSTR(ELFILE)//'.wfn')
        OPEN (MOL,FILE=ELFILE,STATUS='UNKNOWN',
     $        POSITION='REWIND',ACCESS='SEQUENTIAL',
     $        FORM='FORMATTED')
C
C     *** Write header and geometry ***
C
        WRITE (TIMSTR,'(A4,1X,F12.3,1X,A2)') 'TIME',1.E3*RTT/FSEC,'AS'
        CALL WFNDRV2(RDUM,RDUM,PAR,NSTO,TIMSTR,'REWIND')
        CALL WFNDRV2(RDUM,RDUM,PAR,NSTO,TIMSTR,'HEADER')
C
        IF (ADDRAD) THEN
C
          MC2 = MRAD*AMU*(CLIGHT/(ABOHR*FSEC*1.E15))**2
          CLAU = CLIGHT/(ABOHR*FSEC*1.E15)
          D = CLAU**2*(1-(KINRAD/MC2 + 1)**(-2))
          OFFSET = NATOM_ALL - NRADP
C
          DO IRAD=1,NRADP
            CE(:) = X0IORAD(:,IRAD) + RTT*VIORAD(:,IRAD)*SQRT(D)
            WRITE (MOL,5010) 'He',OFFSET+IRAD+1,OFFSET+IRAD+1,
     $            (CE(ICOOR),ICOOR=1,3),QRAD(IRAD)
          END DO
C
        END IF
C
 5010 FORMAT (2X,A2,I4,4X,'(CENTRE',I3,')',1X,3(F12.8,1X),
     $        '  CHARGE =',F5.1)
C
        CALL WFNDRV2(RDUM,RDUM,PAR,NSTO,TIMSTR,'BASIS')
        CALL WFNDRV2(RDUM,OCCNA,PAR,DSTO,TIMSTR,'ALPHA')
        CALL WFNDRV2(RDUM,OCCNA,PAI,DSTO,TIMSTR,'ALPHA')
        IF (OKS) THEN
          CALL WFNDRV2(RDUM,OCCNB,PBR,DSTO,TIMSTR,'ALPHA')
          CALL WFNDRV2(RDUM,OCCNB,PBI,DSTO,TIMSTR,'ALPHA')
        END IF
        CALL WFNDRV2(RDUM,RDUM,PAR,NSTO,TIMSTR,'END')
         
  10  CONTINUE 
C
      DEALLOCATE(NCDA,NCDB,RDUM,STAT=ALLOCATION)
      DEALLOCATE(P,STAT=ALLOCATION)
      DEALLOCATE (S,WORK1,WORK2)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DOTDWFN ***
C
      END
