      SUBROUTINE JCUR(PI,D1STO,STO,DSTO,J,NGPB)
C
C     Purpose: Compute current density vector on current batch of grid
C              points 
C
C     History: - Creation (13.03.13, ADLL)
C
C     ******************************************************************
C
C     List of wrapper arguments:
C
C     PA    : Alpha MO coefficient matrix.
C     PA2   : Beta MO coefficient matrix.
C     DSTO  : Dimension of Slater typ orbital matrix.
C     J     : Vector with current density
C     NGPB  : Number of grid points in current batch.
C
C     List of dynamical fields:
C
C     D1STO: First derivatives of the basis functions.
C     STO  : Function values of the basis functions.
C
C     Output list:    Open shell   
C
C     FUNVEC(IGP,2):  Jx 
C     FUNVEC(IGP,3):  Jy
C     FUNVEC(IGP,4):  Jz
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
      INCLUDE 'threshold.h'
C
      INTEGER I,ISTO,JSTO,DSTO,IATOM,JATOM,INEIGH,JNEIGH
      REAL PI(DSTO,DSTO)
C
      REAL D1STO(MAXGPB,3,DSTO),STO(MAXGPB,DSTO),J(MAXGPB,3)
C
      INTEGER IGP,NGPB
      REAL FACTOR
      REAL JCSTO(MAXGPB)
C
C     ------------------------------------------------------------------
C
C     *** Calculate alpha density and density gradient norm ***
C
      J(:,:) = 0.0
C
      DO INEIGH=1,NNEIGH
        IATOM = NEIGH(INEIGH)
        DO ISTO=LL(IATOM),UL(IATOM)
          JCSTO(1:NGPB) = 0.0
          DO JNEIGH=1,NNEIGH
            JATOM = NEIGH(JNEIGH)
            DO JSTO=LL(JATOM),UL(JATOM)
              JCSTO(1:NGPB) = JCSTO(1:NGPB) + 
     $                        PI(JSTO,ISTO)*STO(1:NGPB,JSTO)
            END DO
          END DO
          DO I=1,3
            J(1:NGPB,I) = J(1:NGPB,I) + 
     $                    2*JCSTO(1:NGPB)*D1STO(1:NGPB,I,ISTO)
          END DO
        END DO
      END DO
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE JCUR ***
C
      END
