      SUBROUTINE POPGRD_RT(PA,PB,DSTO)
C
C     Purpose: POPulation analysis by numerical integration on a GRiD.
C              Used in RT-TDDFT simulations.
C
C     History: - Creation (03.03.20, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'tolerance.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'numint.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
      INCLUDE 'property.h'
C
      INTEGER DSTO
      REAL PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      CHARACTER*80 STATUS_SAVE
      INTEGER IATOM
      REAL QMOL
C
      LOGICAL NEWGRID
      REAL ERRGRD
C
C     ------------------------------------------------------------------
C
C     *** Initialize grid for numerical population analysis ***
C
      STATUS_SAVE = STATUS
C                                                                         <
      NEWGRID = .FALSE.
C
  100 CONTINUE

      STATUS = 'ESA'
      CALL POPINI
      CALL GRDDRV(PA,PB,DSTO)
      STATUS = STATUS_SAVE
C
C     *** Do numerical population analysis on a grid ***
C
      CALL NUMPOP(PA,PB,DSTO)
C
      IF (MPP) THEN
        CALL MP_SUM(NBASINA,NBASINA,NATOM,MP_REAL)
        CALL MP_SUM(NBASINB,NBASINB,NATOM,MP_REAL)
         IF (TDEKIN) THEN
           CALL MP_SUM(KBASIN,KBASIN,NATOM,MP_REAL)
           CALL MP_SUM(GBASIN,GBASIN,NATOM,MP_REAL)
           CALL MP_SUM(LBASIN,LBASIN,NATOM,MP_REAL)
         END IF
      END IF
C
      QMOL = 0.0
      DO IATOM=1,NATOM
        QMOL = QMOL + NBASINA(IATOM) + NBASINB(IATOM) 
      END DO
C
      IF (MPP) CALL MP_SEND_ALL(QMOL,1,MASTER,MP_REAL) 
C
C     *** Check accuracy and update grid if needed ***
C
      ERRGRD = ABS(QMOL - NELA - NELB)
      IF (UPTGRD.AND.(ERRGRD.GT.GRDTTOL)) THEN
        IF (.NOT.NEWGRID) THEN
          RETGRD_GST(TOPGSN) = .FALSE.
          NEWGRID = .TRUE.
          GO TO 100
        END IF
      END IF
C
 200  CONTINUE
C
C     *** Loop for iterative Hirshefeld population analysis ***
C
      IF (ITHPA) CALL DOHPA(PA,PB,DSTO)
C
      IF (MPP.AND.FULLPA) CALL MP_SUM(MBASIN,MBASIN,10*MAXATOM,MP_REAL)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE POPGRD_RT ***
C
      END
