      SUBROUTINE POPKIN(P,D1STO,D2STO,STO,DSTO,RHOS,D1RHOS,NGPB)
C
C     Purpose: Numerical calculation of topological kinetic energy.
C
C     History: - Creation (18.12.01, AVA)
C                         (27.06.07, AMK)
C
C     ******************************************************************
C
C     D1RHOS : Property values of the current batch of grid points.
C              (1) Kinetic energy density K.
C              (2) Kinetic energy density G.
C              (3) Charge density Laplacian L.
C     NGPB   : Number of grid points in the current batch.
C     P      : Alpha/Beta density matrix.
C     PD1STO : Work array (P(mu,nu)*dChi(nu)).
C     PSTO   : Work vector (P(mu,nu)*Chi(nu)).
C     RHOS   : Density values of the current batch of grid points.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'grid.h'
      INCLUDE 'pointer.h'
      INCLUDE 'threshold.h'
C
      INTEGER DSTO
      REAL P(DSTO,DSTO),D1STO(MAXGPB,3,DSTO),D2STO(MAXGPB,6,DSTO),
     $     STO(MAXGPB,DSTO)
C
      REAL D1RHOS(MAXGPB,3),RHOS(MAXGPB)
C
      INTEGER I,IATOM,IGP,II,INEIGH,ISTO,JATOM,JNEIGH,JSTO,NGPB
C
      REAL PSTO(MAXGPB),PD1STO(MAXGPB,3)
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      RHOS(1:NGPB) = 0.0
      D1RHOS(1:NGPB,1:3) = 0.0
C
C     *** Calculate density and its local properties ***
C
      DO 140 INEIGH=1,NNEIGH
        IATOM = NEIGH(INEIGH)
        DO 130 ISTO=LL(IATOM),UL(IATOM)
          PSTO(1:NGPB) = 0.0
          PD1STO(1:NGPB,1:3) = 0.0
          DO 90 JNEIGH=1,NNEIGH
            JATOM = NEIGH(JNEIGH)
            DO 80 JSTO=LL(JATOM),UL(JATOM)
              IF (ISTO.LE.JSTO) THEN
              IF (ABS(P(ISTO,JSTO)).GT.PTHRESH) THEN
                DO 50 IGP=1,NGPB
                  PSTO(IGP) = PSTO(IGP) + P(ISTO,JSTO)*STO(IGP,JSTO)
   50           CONTINUE
                DO 70 I=1,3
                  DO 60 IGP=1,NGPB
                    PD1STO(IGP,I) = PD1STO(IGP,I) +
     $                              P(ISTO,JSTO)*D1STO(IGP,I,JSTO)
   60             CONTINUE
   70           CONTINUE
              END IF
            ELSE
              IF (ABS(P(JSTO,ISTO)).GT.PTHRESH) THEN
                DO 55 IGP=1,NGPB
                  PSTO(IGP) = PSTO(IGP) + P(JSTO,ISTO)*STO(IGP,JSTO)
   55           CONTINUE
                DO 75 I=1,3
                  DO 65 IGP=1,NGPB
                    PD1STO(IGP,I) = PD1STO(IGP,I) +
     $                              P(JSTO,ISTO)*D1STO(IGP,I,JSTO)
   65             CONTINUE
   75           CONTINUE
              END IF
            END IF

   80       CONTINUE
   90     CONTINUE
C
          DO 100 IGP=1,NGPB
            RHOS(IGP) = RHOS(IGP) + STO(IGP,ISTO)*PSTO(IGP)
  100     CONTINUE
C
          DO 120 I=1,3
            II = D2IND(I,I)
            DO 110 IGP=1,NGPB
              D1RHOS(IGP,1) = D1RHOS(IGP,1) +
     $                        D2STO(IGP,II,ISTO)*PSTO(IGP)
              D1RHOS(IGP,2) = D1RHOS(IGP,2) +
     $                        D1STO(IGP,I,ISTO)*PD1STO(IGP,I)
              D1RHOS(IGP,3) = D1RHOS(IGP,3) +
     $                        D2STO(IGP,II,ISTO)*PSTO(IGP) +
     $                        D1STO(IGP,I,ISTO)*PD1STO(IGP,I)
  110       CONTINUE
  120     CONTINUE
  130   CONTINUE
  140 CONTINUE
C
      DO 150 IGP=1,NGPB
        D1RHOS(IGP,3) = 2.0*D1RHOS(IGP,3)
  150 CONTINUE
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TOPKIN ***
C
      END
