      SUBROUTINE TDOCCN(P,OCCN,HFOMO,DSTO,DSTO_ME,SPIN,OPTION)
C
C     Purpose: Determintation of occupation numbers by projection on
C              ground state KS MOs.
C
C     History: - Creation (15.04.17, ADLL) 
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'mp_demon.h'
      INCLUDE 'fileio.h'
      INCLUDE 'physic.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME,HFOMO
      REAL OCCN(MAXSTO),P(DSTO,DSTO)
      CHARACTER SPIN*(*),OPTION*(*)
C
      INTEGER ALLOCATION,IMO,ISTO,JSTO
      REAL CJMO,NELE,TIME
      REAL, ALLOCATABLE:: C(:,:),WORK(:)
      CHARACTER*80 FIXKEY,STRCOMP
      CHARACTER FORM1*29,FORM2*30,FORM3*31,FORM4*32
C
C     -----------------------------------------------------------------
C
C     *** Allocate local fields **
C
      ALLOCATE(C(DSTO_ME,DSTO_ME),WORK(DSTO),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'TDOCCNUM','ALLOCATION FAILED')
C
      TIME = 1.0E3*RTT/FSEC
      OCCN(:) = 0.0
C
C     *** Load reference MO coefficients ***
C
      FIXKEY = 'COLD'//SPIN
      FIXKEY = STRCOMP(FIXKEY)
C
      IF (MPMASTER) CALL SCFIO(C,DSTO,'READ',FIXKEY)
C
C     *** Transform density matrix in MO basis ***
C
      CALL TIMER('START','AO<->MO BASIS TRANSFORMATIONS')
C
      CALL SCALMAT(P,DSTO,DSTO,DSTO,'TOTWD',0.5) 
      CALL TRFMOA(P,DSTO,'PAO->PMO')
C
      CALL TIMER('STOP','AO<->MO BASIS TRANSFORMATIONS')
C
C     *** Calculate occupation numbers ***
C
      IF (MPMASTER) THEN
        DO IMO=1,DSTO
           WORK(:) = 0.0
           CALL GMULMAT(P,C(:,IMO),WORK,DSTO,DSTO,DSTO,
     $          DSTO,DSTO,1,'NORMAL')
           OCCN(IMO) = DOT_PRODUCT(C(:,IMO),WORK(:))
        END DO
      END IF
C
C     *** Build new density matrix ***
C
      IF ((OPTION.EQ.'BUILDP').AND.MPMASTER) THEN
C
        HFOMO = 0
        DO IMO=1,DSTO
          IF (OCCN(IMO).GT.0) HFOMO = IMO
        END DO
C
        P(:,:) = 0.0
        DO JSTO=1,DSTO
          DO IMO=1,HFOMO
            CJMO = 2.0*OCCN(IMO)*C(JSTO,IMO)
            DO ISTO=1,JSTO-1
              P(ISTO,JSTO) = P(ISTO,JSTO) + CJMO*C(ISTO,IMO)
            END DO
            P(JSTO,JSTO) = P(JSTO,JSTO) + 0.5*CJMO*C(JSTO,IMO)
          END DO
        END DO
C
      ELSE IF ((OPTION.EQ.'PRINT').AND.MPMASTER) THEN
C 
        HFOMO = 0
        DO IMO=1,DSTO
          IF (OCCN(IMO).GT.0) HFOMO = IMO
        END DO
C
        NELE = 0.0
        DO IMO=1,DSTO
          NELE = NELE + OCCN(IMO)
        END DO
        FORM1 = '(T2,F12.3,1X,A,1X,X(F5.3,1X))'
        FORM2 = '(T2,F12.3,1X,A,1X,XX(F5.3,1X))'
        FORM3 = '(T2,F12.3,1X,A,1X,XXX(F5.3,1X))'
        FORM4 = '(T2,F12.3,1X,A,1X,XXXX(F5.3,1X))'
        IF (DSTO.LT.10) THEN
          WRITE(FORM1(19:19),'(I1)') DSTO
          WRITE (RTP,FORM1) TIME,'OCCU NUM',(OCCN(IMO),IMO=1,DSTO)
        ELSE IF (DSTO.LT.100) THEN
          WRITE(FORM2(19:20),'(I2)') DSTO
          WRITE (RTP,FORM2) TIME,'OCCU NUM',(OCCN(IMO),IMO=1,DSTO)
        ELSE IF (DSTO.LT.1000) THEN
          WRITE(FORM3(19:21),'(I3)') DSTO
          WRITE (RTP,FORM3) TIME,'OCCU NUM',(OCCN(IMO),IMO=1,DSTO)
        ELSE IF (DSTO.LT.10000) THEN
          WRITE(FORM4(19:22),'(I4)') DSTO
          WRITE (RTP,FORM4) TIME,'OCCU NUM',(OCCN(IMO),IMO=1,DSTO)
        END IF
        WRITE (RTP,'(T2,F12.3,1X,A,1X,F6.2)') 
     $              TIME,'SUM OF OCCUPATION',NELE
      END IF
C
C     *** Deallocate molecular field ***
C
      DEALLOCATE (C,WORK,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'TDOCCNUM','DEALLOCATION FAILED')
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE TDOCCN ***
C
      END
