      SUBROUTINE WDENJ(PAR,PAI,PBR,PBI,CDC,DSTO,FUNVEC,LLGPB,NGPB)
C
C     Purpose: Wrapper for Time dependent density currents 
C
C     History: - Creation (30.03.17, ADLL)
C 
C     ******************************************************************
C
C     List of wrapper arguments:
C
C     DSTO  : Dimension of Slater typ orbital matrix.
C     FUNVEC: Vector with plot function values.
C     LLGPB : Lower limit grid point of current batch.
C     NGPB  : Number of grid points in current batch.
C     PRE   : Density or spin density matrix.
C     PIM   : ??????????????????????? aai AAI
C
C     List of local variables:
C
C     TPH   : Two particle probability of the uniform electron gas.
C     TP    : Two particle probability.
C
C     List of dynamical fields:
C
C     D1STO: First derivatives of the basis functions.
C     STO  : Function values of the basis functions.
C
C     Output list:    Open shell         Closed shell
C
C     FUNVEC(IGP,1):  Alpha ELF             ELF
C     FUNVEC(IGP,2):  Alpha RHO             RHO
C     FUNVEC(IGP,3):  Beta ELF
C     FUNVEC(IGP,4):  Beta RHO
C     FUNVEC(IGP,5):  |GRAD alpha RHO|^2    |GRAD RHO|^2
C     FUNVEC(IGP,6):  |GRAD beta RHO|^2
C     FUNVEC(IGP,7):  Alpha TRHO            TRHO
C     FUNVEC(IGP,8):  Beta TRHO
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'math.h'
      INCLUDE 'plot.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'threshold.h'
C
      INTEGER DSTO
      REAL PAR(DSTO,DSTO),PAI(DSTO,DSTO)
      REAL PBR(DSTO,DSTO),PBI(DSTO,DSTO)
C
      REAL CDC(MAXAUX),FUNVEC(MAXGPB,MAXPLOT)
C
      INTEGER IGP,IMO,LLGPB,NGPB
      REAL NORM 
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: D1STO(:,:,:),STO(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(D1STO(MAXGPB,3,DSTO),STO(MAXGPB,DSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('WDENJ','ALLOCATION FAILED',1)
      END IF
C
      FUNVEC(:,:) = 0.0
C
C     *** Calculate basis function and derivative values ***
C
      CALL D1CHI(D1STO,STO,DSTO,LLGPB,NGPB)
C
C     *** Calculate density current vector ***
C
      CALL JCUR(PAI,D1STO,STO,DSTO,FUNVEC(:,2:4),NGPB)
      IF (OKS) THEN
        CALL JCUR(PBI,D1STO,STO,DSTO,FUNVEC(:,5:7),NGPB)
        FUNVEC(IGP,2) = FUNVEC(IGP,2) + FUNVEC(IGP,5)
        FUNVEC(IGP,3) = FUNVEC(IGP,3) + FUNVEC(IGP,6)
        FUNVEC(IGP,4) = FUNVEC(IGP,4) + FUNVEC(IGP,7)
      END IF
C
      DO IGP=1,NGPB
        NORM = FUNVEC(IGP,2)**2 + FUNVEC(IGP,3)**2 + FUNVEC(IGP,4)**2  
        FUNVEC(IGP,1) = SQRT(NORM) 
      END DO
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(D1STO,STO,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('WDENJ','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE WTDELF ***
C
      END
