      SUBROUTINE CAPP(P,STO,DSTO,RHOS,LLGPB,NGPB)
C
C     Purpose: Numerical calculation of the complex absorbing potential 
C              of the current batch of grid points.
C
C     History: - Creation (12.04.17, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     NGPB: Number of grid points in the current batch.
C     P   : Alpha/Beta density matrix.
C     PSTO: Work vector (P(mu,nu)*Chi(nu)).
C     RHOS: Density values of the current batch of grid points.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'grid.h'
      INCLUDE 'math.h'
      INCLUDE 'molecule.h'
      INCLUDE 'pointer.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'threshold.h'
C
      INTEGER LLGPB,DSTO
      REAL P(DSTO,DSTO),STO(MAXGPB,DSTO),RHOS(MAXGPB)
C
      INTEGER IATOM,IGP,INEIGH,ISTO,JATOM,JNEIGH,JSTO,NGPB
      REAL FACTOR,R,RMAX,RMIN
C
      REAL PSTO(MAXGPB),ABSVAL(MAXGPB)
C
C     ------------------------------------------------------------------
C
C     *** Initialize density, potentia and boundary values ***
C
      RHOS(1:NGPB) = 0.0
      ABSVAL(1:NGPB) = 0.0
C  
C     *** Calculate absorbing potential ***
C
      RMIN = P3ABS
      RMAX = P2ABS + P3ABS
      DO IATOM=1,NATOM
        DO IGP=1,NGPB
C
          R = SQRT((GRDVEC(1,LLGPB+IGP-1) - COORD(1,IATOM,CCSET))**2 +
     $             (GRDVEC(2,LLGPB+IGP-1) - COORD(2,IATOM,CCSET))**2 +
     $             (GRDVEC(3,LLGPB+IGP-1) - COORD(3,IATOM,CCSET))**2)
C
          IF (R.GT.RMAX) THEN
            ABSVAL(IGP) = ABSVAL(IGP) + P1ABS
          ELSE IF (R.GT.RMIN) THEN
            FACTOR = PI*(R - RMIN)/(2*P2ABS)
            FACTOR = (SIN(FACTOR))**2
            ABSVAL(IGP) = ABSVAL(IGP) + P1ABS*FACTOR
          END IF
        END DO
      END DO
C  
C     *** Calculate density values after absorption ***
C
      DO INEIGH=1,NNEIGH
        IATOM = NEIGH(INEIGH)
        DO ISTO=LL(IATOM),UL(IATOM)
          PSTO(1:NGPB) = 0.0
          DO JNEIGH=1,NNEIGH
            JATOM = NEIGH(JNEIGH)
            DO JSTO=LL(JATOM),UL(JATOM)
              IF (ISTO.LE.JSTO) THEN
                PSTO(1:NGPB) = PSTO(1:NGPB) + 
     $                         P(ISTO,JSTO)*STO(1:NGPB,JSTO)
              ELSE
                PSTO(1:NGPB) = PSTO(1:NGPB) + 
     $                         P(JSTO,ISTO)*STO(1:NGPB,JSTO)
              END IF
            END DO
          END DO
          RHOS(1:NGPB) = RHOS(1:NGPB) + ABSVAL(1:NGPB)*PSTO(1:NGPB)*
     $                                   STO(1:NGPB,ISTO)
        END DO
      END DO
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE CAPP ***
C
      END
