      SUBROUTINE NUMCAPV(ATOM,LLGPB,NGPB)
C
C     Purpose: Numerical calculation of absorbing potential at grid
C              points. 
C
C     History: - Creation (23.03.17, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     NGPB         : Number of grid points in the current batch.
C     WGRID        : Numerical grid weight value.
C     P1/2/3ABS    : Parameters defining CAP in real space.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'grid.h'
      INCLUDE 'math.h'
      INCLUDE 'molecule.h'
      INCLUDE 'rtdft.h'
C
      INTEGER ATOM,LLGPB,NGPB
C
      INTEGER IATOM,IGP,INEIGH
      REAL DSQMAX,FACTOR,R,RMAX,RMIN
      REAL ABSVAL(MAXGPB),WGRID(MAXGPB)
C
C     ------------------------------------------------------------------
C
C     *** Initialize local fields ***
C
      ABSVAL(1:NGPB) = 0.0
      RMAX = P3ABS + P2ABS
C
C     *** Load grid weights of the current batch ***
C
      DO 50 IGP=1,NGPB
        WGRID(IGP) = GRDVEC(4,LLGPB+IGP-1)
50    CONTINUE
C
      DO 40 IGP=1,NGPB
        RMIN = P3ABS**2
C
        R = ((GRDVEC(1,LLGPB+IGP-1) - COORD(1,ATOM,CCSET))**2 +
     $       (GRDVEC(2,LLGPB+IGP-1) - COORD(2,ATOM,CCSET))**2 +
     $       (GRDVEC(3,LLGPB+IGP-1) - COORD(3,ATOM,CCSET))**2)
C
        IF ((ZATOM(ATOM).GT.0).AND.(R.LE.RMIN)) GO TO 40

        DSQMAX=99999.9999
        DO 30 IATOM=1,NATOM
          IF (IATOM.EQ.ATOM) GO TO 30
          IF (ZATOM(IATOM).EQ.0) GO TO 30
C
          R = ((GRDVEC(1,LLGPB+IGP-1) - COORD(1,IATOM,CCSET))**2 +
     $         (GRDVEC(2,LLGPB+IGP-1) - COORD(2,IATOM,CCSET))**2 +
     $         (GRDVEC(3,LLGPB+IGP-1) - COORD(3,IATOM,CCSET))**2)
C
          IF (R.LE.RMIN) THEN
            GO TO 40 
          ELSE IF (R.LT.DSQMAX) THEN 
            DSQMAX = R
          END IF
 30     CONTINUE 

        R = SQRT(DSQMAX)
        RMIN = SQRT(RMIN)
        IF (R.GT.RMAX) THEN
          ABSVAL(IGP) = ABSVAL(IGP) + P1ABS
        ELSE IF (R.GT.RMIN) THEN
          FACTOR = PI*(R-RMIN)/(2*P2ABS)
          FACTOR = SIN(FACTOR)**2
          ABSVAL(IGP) = ABSVAL(IGP) + P1ABS*FACTOR
        END IF
 40   CONTINUE
C      
C     *** Build vector with the absorbing potential values ***
C
      VABS(1:NGPB) = WGRID(1:NGPB)*ABSVAL(1:NGPB)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NUMCAPV **
C
      END
