      SUBROUTINE WCAP(CA,CB,PA,PB,CDC,DSTO,FUNVEC,LLGPB,NGPB)
C
C     Purpose: Wrapper for complex absorbing potential
C
C     History: - Creation (12.04.17, ADLL)
C
C     ******************************************************************
C
C     List of wrapper arguments:
C
C     CA    : Alpha MO coefficient matrix (not used).
C     CB    : Beta MO coefficient matrix (not used).
C     CDC   : Charge density coefficients.
C     DSTO  : Dimension of Slater typ orbital matrix.
C     FUNVEC: Vector with plot function values.
C     LLGPB : Lower limit grid point of current batch.
C     NGPB  : Number of grid points in current batch.
C     PA    : Density matrix.
C     PB    : Beta density matrix (not used).
C
C     List of local dynamical fields:
C
C     STO: Function values of the basis functions.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
C
      INTEGER DSTO
      REAL CA(DSTO,DSTO),CB(DSTO,DSTO),PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      REAL CDC(MAXAUX),FUNVEC(MAXGPB,MAXPLOT)
C
      INTEGER LLGPB,NGPB
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: STO(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(STO(MAXGPB,DSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('WRHO','ALLOCATION FAILED',1)
      END IF
C
C     *** Calculate STO and CAP values on the grid ***
C
      CALL CHI(STO,DSTO,LLGPB,NGPB)
      CALL CAPP(PA,STO,DSTO,FUNVEC(:,1),LLGPB,NGPB)
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(STO,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('WRHO','ALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE WCAP ***
C
      END
