      SUBROUTINE CLRT(KR,KI,PR,PI,DSTO,DSTO_ME)
C
C     Purpose:  Propagate MO density matrix/matrices. 
C
C     History: - Creation (21.10.15, XJ)
C                         (02.03.20, ADLL)
C
C     ******************************************************************
C     List of local variables: 
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables:
C
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real parts of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real parts of density matrix.
C 
C     KIA   : imaginary parts of Kohn-Sham matrix of step before
C     KRA   : real parts of Kohn-Sham matrix of step before
C
C 
C     ------------------------------------------------------------------
C
      use iso_c_binding
      use magma
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'embed.h'
      INCLUDE 'flags.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'slp.h'
      INCLUDE 'xc.h'
      INCLUDE 'numint.h'
      INCLUDE 'tolerance.h'
      INCLUDE 'iteration.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME)
C
      INTEGER ALLOCATION
      LOGICAL MDFLUSH
C
      external blacs_get
      external blacs_gridinit
      external blacs_gridexit
      external blacs_exit
C
C     ------------------------------------------------------------------
C
C     *** Write header in output file ***
C
      CALL RTOUT(PR,PR,DSTO_ME,'HEADER')
C
C     *** Distribute global counters of RTDFT cycles ***
C
      CALL MP_SEND_ALL(GRTCYC,1,MASTER,MP_INT)
      CALL MP_SEND_ALL(FGRTCYC,1,MASTER,MP_INT)
C
C     *** Initialize Scalapack grid or Magma
C
      IF (USESCALA) THEN
C     setting the BLACS environment and the grid definitions
        call blacs_get( 0, 0, ictxt )
        ictxt_0 = ictxt
        call blacs_gridinit( ictxt, 'All', nprow, npcol )
        call blacs_gridinit( ictxt_0, 'All', 1, 1 )
      ELSE IF (USEMAGMA) THEN
C     setting the MAGMA environment
      END IF
C
C     *** Perform electronic propagation *** 
C
      DO RTCYC=1,RTMAX
C
        GRTCYC = GRTCYC + 1
        IF (PROPAGA.EQ.'EULER') THEN
          CALL PEULER(KR,KI,KR,KI,PR,PI,PR,PI,DSTO,DSTO_ME)
        ELSE IF (PROPAGA.EQ.'MAGNUS') THEN
          IF (MAGPC) THEN
            CALL PMAGPC(KR,KI,KR,KI,PR,PI,PR,PI,DSTO,DSTO_ME)
          ELSE IF (MAGIT) THEN
            CALL PMAGIT(KR,KI,KR,KI,PR,PI,PR,PI,DSTO,DSTO_ME)
          END IF
        END IF
C
        RTT = RTT + RTSTEP
C
C     *** Print data on output file ***
C
        IF (MDFLUSH(GRTCYC,RTINT)) CALL RTOUT(PR,PR,DSTO_ME,'STEP')
C
C     *** Save data in restarting file ***
C
        IF (MDFLUSH(GRTCYC,RTSAV)) THEN
          CALL RTRESTART(KR,KI,PR,PI,DSTO,DSTO_ME,'SAVE','ALPHA')
        END IF
C
C     *** Property calculations ***
C
        CALL RTANA(PR,PI,PR,PI,DSTO,DSTO_ME)
C
C     ***  Load XC grid type ***
C
        CALL GRDDEF(XCGSN,'LOAD-TYPE')
        IF (GGFUN.EQ.'SCF') THEN
          IF (RANGRD) ROTGRD = .TRUE.
          GRDTOL = GRDTOL/10.0
        END IF
C
      END DO
C
      IF (USESCALA) THEN
        call blacs_gridexit( ictxt )
      ELSE IF (USEMAGMA) THEN
      END IF
C
C     *** Generate restart file ***
C
      CALL RTRESTART(KR,KI,PR,PI,DSTO,DSTO_ME,'SAVE','ALPHA')
C
C     ------------------------------------------------------------------

C     *** End of SUBROUTINE CLRT ***
C
      END
