      SUBROUTINE CLRTINI(KR,KI,PR,PI,DSTO,DSTO_ME)
C
C     Purpose: Initialize RTDFT calculation for closed-shell system.
C
C     History: - Creation (09.09.15, XJ)
C                         (02.03.20, ADLL)
C
C     ******************************************************************
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables: 
C     
C     CI    : imaginary MO coefficient matrix
C     CR    : real MO coefficient matrix
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real parts of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real parts of density matrix.
C 
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
C
      INCLUDE 'fileio.h'
      INCLUDE 'auxis.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'eri.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'iteration.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'numint.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'tolerance.h'
C
      INTEGER DSTO,DSTO_ME
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME)
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
C
      INTEGER ALLOCATION
      REAL TIME
      REAL, ALLOCATABLE ::  WORK(:,:)
C
C     ------------------------------------------------------------------
C
c     *** Initialize closed shell SCF ***
C
      CALL RTCINI('CLOSED SHELL')
C
C     *** Restart from previous ED simuation ***
C
      IF (RTRST) THEN
C
        CALL RTRESTART(KR,KI,PR,PI,DSTO,DSTO_ME,'RESTART','ALPHA')
C
C     *** Distribute auxiliary vectors for direct scheme ***
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(RTT,1,MASTER,MP_REAL)
          CALL MP_SEND_ALL(GRTCYC,1,MASTER,MP_INT)
          CALL MP_SEND_ALL(FGRTCYC,1,MASTER,MP_INT)
          IF (ERITYP.EQ.'DIRECT') THEN
            CALL MP_SEND_ALL(CDCOLDA(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(CDCOLDB(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(CDJOLDA(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(CDJOLDB(1:NAUX),NAUX,MASTER,MP_REAL)
          END IF
C
          IF (TOPCD.AND.DEFORM) THEN
            CALL MP_SEND_ALL(MFCDA(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(MFCDB(1:NAUX),NAUX,MASTER,MP_REAL)
          END IF
C
          CALL GRDDEF(XCGSN,'SEND_ALL')
C
        END IF
C
        RETGRD = .TRUE.
        INITLEB = .TRUE.
        INITGRD = .TRUE.
        CALL GRDINI
        CALL GRDDEF(XCGSN,'LOAD-TYPE')
        IF (GGFUN.EQ.'SCF') THEN
          IF (RANGRD) ROTGRD = .TRUE.
          GRDTOL = GRDTOL/10.0
        END IF
        IF (.NOT.DIRGRD) CALL GRDRET
C
C     *** Save promolecular density on each node ***
C
        IF (MPP) THEN
          ALLOCATE(WORK(DSTO,DSTO),STAT=ALLOCATION)
          IF (ALLOCATION.GT.0) THEN
            CALL ERRMSG('CLRTINI','ALLOCATION FAILED',1)
          END IF
          IF (MPMASTER) CALL SCFIO(WORK,DSTO,'READ','P0ALPHA')
          CALL MP_SEND_ALL(WORK,DSTO**2,MASTER,MP_REAL)
          CALL SCFIO(WORK,DSTO,'WRITE','P0ALPHA')
C
          DEALLOCATE(WORK,STAT=ALLOCATION)
          IF (ALLOCATION.GT.0) THEN
            CALL ERRMSG('CLRTINI','DEALLOCATION FAILED',1)
          END IF
        END IF
C
C     *** Launch RT-TDDFT simulation from SCF solution ***
C
      ELSE
C
        CALL RSTIO(KR,EIGVALA,PR,DSTO,DSTO_ME,OCCNA,AOCCNA,ELCFGA,
     $           MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
        KI(:,:) = 0.0
C
        IF (MPSLAVE) GO TO 100
C
C     *** Localize MOs ***
C
        IF (USERLMO) ALLOCATE(WORK(DSTO,DSTO))
        IF (USERLMO.AND.FBRMO) THEN
          CALL LOCALIZE(KR,PR,WORK,DSTO,'FB','ALL','TIGHT','ALPHA')
        ELSE IF (USERLMO.AND.PMRMO) THEN
          CALL LOCALIZE(KR,PR,WORK,DSTO,'PM','ALL','TIGHT','ALPHA')
        ELSE IF (USERLMO) THEN
          CALL LOCALIZE(KR,PR,WORK,DSTO,'CH','ALL','TIGHT','ALPHA')
        END IF
        IF (USERLMO) DEALLOCATE(WORK)
C
C     *** Exchange molecular orbitals ***
C
        CALL EXCMOS(KR,DSTO,EIGVALA,MOEXCA,NMOEXCA)
C       
C     *** Save reference coefficients in orthonormal basis  ***
C
100     CONTINUE
C
        IF (MPMASTER) THEN
          CALL SCFIO(KI,DSTO,'READ','OVERLAP')
          CALL GMULMAT(KI,KR,PI,DSTO,DSTO,DSTO,NSTO,NSTO,NSTO,
     $                 'NORMAL')
          CALL SCFIO(KI,DSTO,'READ','ORTHO')
          CALL GMULMAT(KI,PI,PR,DSTO,DSTO,DSTO,NSTO,NORTH,NSTO,
     $                 'TRANSA')
          KI(:,:) = PR(:,:)
        END IF
        IF (MPP) CALL MP_SEND_ALL(KI,DSTO**2,MASTER,MP_REAL) 
        CALL SCFIO(KI,DSTO,'WRITE','COLDALPHA')
        KI(:,:) = 0.0
        PI(:,:) = 0.0
        PR(:,:) = 0.0
C
C     *** Build complex density matrix ***
C
        IF (MPSLAVE) GO TO 200
C 
        CALL BLDCOMMAT(KR,KI,PR,PI,DSTO,OCCNA,LFOMOA,HFOMOA)
C  
        KR(:,:) = PR(:,:)
        KI(:,:) = PI(:,:)
C
 200    CONTINUE
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(KR,DSTO**2,MASTER,MP_REAL)
          CALL MP_SEND_ALL(KI,DSTO**2,MASTER,MP_REAL)
        END IF
C
        CALL SCFIO(KR,DSTO,'WRITE','P0ALPHA')
C       
        IF (MPMASTER) CALL MOMENT(PR,1,DSTO)
C
C     *** Build charge density Coulomb vector ***
C
        IF (ERITYP.EQ.'DIRECT') THEN
          CALL BLDCDJ(KR,KI,PR,PR,DSTO,DSTO_ME)
        ELSE
          CALL GETCDJ(PR,PR,DSTO)
        END IF
C
C     *** Calculate charge density fit coefficients ***
C
        CALL GETCDC
C
C     *** Initialize deformation density analysis ***
C
        IF (DEFORM) THEN
          MFCDA(:) = FCDA(:)
        END IF
        IF (MPP) THEN
          CALL MP_SEND_ALL(MFCDA,NAUX,MASTER,MP_REAL)
        END IF
C
        IF (ADDRAD) THEN
          TIME = 0.0
          CALL SETPROJ(TIME,'PRTOUT')
          CALL EMBEDRAD
        END IF
C
C     *** Calculation of energy bounds ***
C
        IF (MPMASTER) CALL EBOUNDS(KR,PR,PR,QCDCA,QCDCB,DSTO)
C
        IF (MPMASTER) CALL MIXING(1.0,PR,PR,PR,PR,DSTO)
C
        IF (MPMASTER) THEN
          CALL STOPMAT(KI,PR,PI,DSTO_ME,'REDUCE') 
          CALL SCFIO(KI,DSTO,'WRITE','KOLDALPR') 
        END IF
C
C     *** Build Complex Absorbing Potential ***
C
        IF (ABSPOT) 
     $    CALL BLDCAP(KR,EIGVALA(1:DSTO),DSTO,DSTO_ME,'ALPHA')
C
        IF (EXX) THEN 
          CALL FOCK(KR,KR,PR,PR,DSTO,DSTO_ME) 
          CALL MIXIO(KR,DSTO,'WRITE','EXXALPHA')
        END IF
        IF (MPMASTER) CALL SCFIO(KI,DSTO,'READ','KOLDALPR')
C
C     *** Build numerical exchange-correlation matrix ***
C
        CALL NUMXC(KR,KR,PR,PR,DSTO,DSTO_ME)
C
C     *** Build Kohn-Sham matrix ***
C
        CALL BLDKSMAT(KR,KR,PR,PR,DSTO,DSTO_ME)
C
C     *** Recover density matrix ***
C
        IF (MPMASTER) THEN
          CALL STOPMAT(KI,PR,PI,DSTO_ME,'EXPAND')
          KI(:,:) = 0.0
C
C     *** Save current KS matrix as OLD (since this is RTCYC=0) ***
C
          CALL SCFIO(KR,DSTO,'WRITE','KOLDALPR')
          IF (DOIKS) CALL SCFIO(KI,DSTO,'WRITE','KOLDALPI')
C   
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE CLRTINI ***
C
      END
