      SUBROUTINE EMBEDRAD
C
C     Purpose: Embedding of the Hamiltonian by a charged particle. 
C
C     History: Creation (01.01.18, AAI)
C                       (02.03.20, ADLL)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     WORK: Work matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'

      INCLUDE 'physic.h'
      INCLUDE 'projectile.h'
C
      CHARACTER*80 PRTSTR
C
      INTEGER ALLOCATION,HDIM,I
      REAL,ALLOCATABLE :: H(:,:),WORK(:,:)
      LOGICAL STATED
      LOGICAL MONOPOLE_SAVED
C
C     ------------------------------------------------------------------
C
      IF (.NOT.STATED('ATOM'))
     $     CALL TIMER('START','EMBEDDING BY PROJECTILE')

C     *** Allocate local fields for embedding integrals ***
C
      ALLOCATE(H(NSTO,NSTO),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'EMBEDDING','ALLOCATION FAILED')
C
      H(:,:) = 0.0
C
C     *** Calculate far-field embedding integrals and add to H ***
C
      MONOPOLE_SAVED = MONOPOLE
      MONOPOLE = .FALSE.
C
C     *** Calculate near-field embedding integrals and add to H ***
C
      CALL NEAREMBEDRAD(H,NSTO)
C
      MONOPOLE = MONOPOLE_SAVED
C
C     *** Collecting of the results on the master node ***
C
      CALL MP_COLLECT(H,NSTO*NSTO,MASTER,(/999.999/),.FALSE.)
C
      IF (MPMASTER) THEN
C
C     *** Symmetrization of core Hamiltonian ***
C
        CALL SYMMAT(H,NSTO,NSTO,'UPLOW')
C
C     *** Store embedded core Hamiltonian matrix on tape ***
C
        CALL SCFIO(H,NSTO,'WRITE','RADIAF')
C
      END IF
C
C     *** Deallocate local fields for embedding integrals ***
C
      DEALLOCATE(H,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'EMBEDDING','DEALLOCATION FAILED')
C
      IF (.NOT.STATED('ATOM'))
     $     CALL TIMER('STOP','EMBEDDING BY PROJECTILE')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE EMBEDRAD ***
C
      END
