      SUBROUTINE FIELDRT(H,DSTO,X,X0,SIGMA,OMEGA,OPTION)
C
C     Purpose: Add electric field embedding to Hamiltonian matrix.
C
C     History: - Creation (30.10.15, XJ)
C                         (02.03.20, ADLL)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'embed.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'math.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'strings.h'
C
      CHARACTER*80 LTU*1,PRTSTR
      INTEGER IFIELD,DSTO
      REAL X,X0,SIGMA,OMEGA
      CHARACTER OPTION*(*)
C
      INTEGER ALLOCATION
      REAL FMOD1,FMOD2,TRACE,UFID
      REAL H(DSTO,DSTO)
      REAL,ALLOCATABLE :: WORK(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Quick return if no electric field applied ***
C
      IF (.NOT.EFRTP) RETURN
C
      IF (MPSLAVE) GOTO 500
C
C     *** Calculate modulation factor ***
C
      CALL MODEFIELD(FMOD1,FMOD2,X,X0,SIGMA,OMEGA)
C
      FMOD1 = FMOD1*FMOD2
C
      IF (OPTION.EQ.'REM') FMOD1 = -FMOD1
      IF ((FMOD1/FMOD2).LT.1.0E-10) RETURN
C
      IF (PRTRT.AND.(OPTION.EQ.'ADD')) THEN
        IF (FSHAPE.NE.'KICK') THEN
          WRITE(OUT,5000) X*1.E3/FSEC,FMOD1,FMOD2,FMOD1/FMOD2
 5000     FORMAT (T2,'EFIELD : ',F10.3,1X,3(F12.5,X)) 
        ELSE IF (FMOD1.NE.0.0) THEN 
          WRITE(OUT,5010) FMOD1
 5010     FORMAT (T2,'ELECTRIC FIELD KICK APPLIED',X,F12.5)
        END IF
      END IF
C
C     *** Allocate local fields for embedding integrals ***
C
      ALLOCATE(WORK(DSTO,DSTO),STAT=ALLOCATION)
C
C     *** Loop over electric field components ***
C
      DO 10 IFIELD=1,3
C
C     *** Load dipole moment matrix ***
C
        CALL FFPIO(WORK,DSTO,'READ',LTU(XYZ(IFIELD)))
C
C     *** Build field Hamiltonian ***
C
        WORK(:,:) = WORK(:,:)*ELFIELD(IFIELD)*FMOD1
        H(:,:) = H(:,:) + WORK(:,:)
C
   10 CONTINUE
C
C     *** Symmetrization of core Hamiltonian ***
C
      CALL SYMMAT(H,DSTO,DSTO,'UPLOW')
C
C     *** Deallocate local fields for embedding integrals ***
C
      DEALLOCATE(WORK,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('FIELDRT','DEALLOCATION FAILED',1)
      END IF
C
 500  CONTINUE
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE FIELDRT ***
C
      END
