      SUBROUTINE IONRADINI 
C
C     Purpose: Define initial conditions of ionizing radiation. 
C
C     History (17.06.17, ADLL)
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'dynamics.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mm.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'projectile.h'
C
      INTEGER ATOM,IATOM,JATOM,NATOMS
      INTEGER I,J,IRAD,OFFSET
      REAL NORM,D,DMAX,VEL,FACTOR
      REAL CM1(3),CM2(3),MATRIX(3,3),R(3),RMATRIX(3,3)
      CHARACTER*500 PRTSTR
      LOGICAL STATED
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: M(:),X(:,:),Y(:,:)
C
C     -----------------------------------------------------------------
C
C     *** Quick return if ATOM status ***
C
      IF (STATED('ATOM')) RETURN
C
C     *** Particle properties ***
C
      DO IRAD=1,NRADP
        QRAD(IRAD) = QIORAD
        RRAD(IRAD) = COVRAD(0)
C
C     *** Passing info from RADIATION to MM fields ***
C
        IF (MD) THEN
          OFFSET = NATOM_ALL-NRADP
          CHGMM(OFFSET-NATMM+IRAD) = QIORAD
          MATOM(OFFSET+IRAD) = MRAD
        END IF
      END DO
C
C     *** Quick return for MD (after charge and mass were initialized) ***
C
ccc aai AAI
ccc The return must work if I already started the EhrMD (whereas I am
ccc   in a subsequent step or in a restarted dynamics). This is
ccc   because the rad. particle position and velocity are dictated by 
ccc   verlet.f when a MD is on, unlike the RTDFT case where the particle
ccc   location is dictated by the RTT value and/or the input information.
ccc It must be noticed that the charge and mass of the rad. particle are
ccc   always read from the input (section "Particle properties" above)
ccc   so even if I am in a started dynamics I must declare them there.
ccc aai AAI
      IF (DYNTYP.EQ.'RTDFT') THEN
        IF (MDCYC.GT.0) RETURN !!!!!!!!!!!
        IF (TRJTYP.EQ.'RESTART') THEN
          DO IRAD=1,NRADP
            OFFSET = NATOM_ALL-NRADP
            CRAD(1:3,IRAD,CCSET) = COORD(1:3,OFFSET+IRAD,CCSET)
          END DO
          RETURN !!!!!!!!!!!!!!!
        END IF
      END IF
ccc I do not know what happens with the PDB fragment definition in case
ccc   of the RETURN. It is supposed to be unimportant to me since at this
ccc   moment I already initialized the particle position and velocity, and
ccc   after that the trajectory is given by verlet.f. Technically, I do not
ccc   need the fragment reference anymore since it is only used to define 
cccc  the particle location and direction.
C
C     *** Determine number of atoms in PDB fragments ***
C
      NATOMS = 0
      IF (NMOL.EQ.0) THEN
        NATOMS = NATOM
      ELSE 
        DO IATOM=1,NATOM
          IF (FRAGMOL(IATOM).EQ.FRARAD) NATOMS = NATOMS + 1
        END DO
      END IF
C
C     *** Calculate center of mass reference coordinates ***
C
      ALLOCATE(M(NATOM),X(3,NATOM),Y(3,NATOM),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('IONRADINI','ALLOCATION FAILED',1)
      END IF
C
      M(1:NATOM) = MATOM(1:NATOM)
      X(1:3,1:NATOM) = COORD(1:3,1:NATOM,CCSET)
      CALL GETCM(NATOM,X,M,CM1)
      CM2(:) = 0.0
      IF (NMOL.NE.0) THEN 
        JATOM=0
        DO IATOM=1,NATOM
          IF (FRAGMOL(IATOM).EQ.FRARAD) THEN
            JATOM = JATOM + 1
            M(JATOM) = MATOM(IATOM)
            X(1:3,JATOM) = COORD(1:3,IATOM,CCSET)
          END IF
        END DO
        CALL GETCM(NATOMS,X,M,CM2)
      END IF
C
C
C     *** Define particle initial position w.r.t. to target ***
C
      DO IRAD=1,NRADP
        IF (NMOL.EQ.0) THEN
          X0IORAD(:,IRAD) = CM1(:) - VIORAD(:,IRAD)*DISRAD(IRAD)
        ELSE 
          X0IORAD(:,IRAD) = CM2(:) - VIORAD(:,IRAD)*DISRAD(IRAD)
        END IF
        X0IORAD(:,IRAD) = X0IORAD(:,IRAD) + DPLRAD(:,IRAD)
C
C     *** Setting initial conditions for RAD in MMATM list  ***
C
ccc aai AAI
ccc Overwriting particle coordinates from DECODE statements
ccc   (see, for example, readvelos.f, ranvelos.f, veloscale.f). It
ccc   assumes only one particle and it is the last "atom" defined.
ccc aai AAI
        IF (MD) THEN
          OFFSET = NATOM_ALL-NRADP
          COORD(1:3,OFFSET+IRAD,CCSET) =  X0IORAD(1:3,IRAD)
          VEL = 2*KINRAD/(MRAD*AMU)
          VEL = SQRT(VEL)
          VELOS(1:3,OFFSET+IRAD) = VEL*VIORAD(1:3,IRAD)
        END IF
      END DO
C
C      *** Calculate largest atom distance from COM ***
C
      DMAX = TINY(0.0)
C
      DO IATOM=1,NATOMS
        R(1:3) = COORD(1:3,IATOM,CCSET) - CM1(1:3)
        D = R(1)**2  + R(2)**2 + R(3)**2
        IF (D.GT.DMAX) THEN
          DMAX = D
          ATOM = IATOM
        END IF 
      END DO
C
      D = SQRT(DMAX)
C
      DO IRAD=1,NRADP
        IF ((DISRAD(IRAD).GT.0).AND.(DISRAD(IRAD).LT.D)) THEN
          WRITE(PRTSTR,5000) DISRAD(IRAD)/BOHR
          CALL EXTMSG(1,PRTSTR,1,0,OUT) 
          WRITE(PRTSTR,5001) D/BOHR 
          CALL EXTMSG(1,PRTSTR,0,1,OUT) 
        END IF
      END DO
5000  FORMAT ('IONIZING PARTICLE SET AT ',F9.5,' ANG. FROM COM')
5001  FORMAT ('WHILE THE MOST REMOTE ATOM IS ',F9.5,' ANG. FROM COM')
C
      DEALLOCATE(M,X,Y,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('IONRADINI','DEALLOCATION FAILED',1)
      END IF
C
C     ----------------------------------------------------------------- 
C
C     *** End of IONRADINI ***
C     
      END
