      SUBROUTINE MODEFIELD(FMOD1,FMOD2,X,X0,SIGMA,OMEGA)
C
C     Purpose: Calculate electric field modulation factor.
C
C     History: - Creation (21.07.22, ADLL)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'embed.h'
      INCLUDE 'flags.h'
      INCLUDE 'math.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'rtdft.h'
C
      REAL FMOD1,FMOD2,X,X0,SIGMA,OMEGA
C
      INTEGER ALLOCATION
C
C     ------------------------------------------------------------------
C
C     *** Calculate electric field strength ***
C
      IF (FSHAPE.EQ.'PULSE') THEN 
C
        FMOD1 = EXP(-(X-X0)**2/(2*SIGMA**2))
        FMOD2 = COS(OMEGA*X)
C
      ELSE IF (FSHAPE.EQ.'SQRCOS') THEN
        IF (X.GT.2*X0) THEN
          FMOD1 = 0.0
        ELSE 
          FMOD1 = COS(PI*(X-X0)/(2*X0))
          FMOD1 = FMOD1*FMOD1
        END IF
        FMOD2 = SIN(OMEGA*X)
C
      ELSE IF (FSHAPE.EQ.'LRAMP') THEN
C
        IF (X.LT.X0) THEN
          FMOD1 = X/X0
        ELSE
          FMOD1 = 1.0
        END IF 
        FMOD2 = COS(OMEGA*X)
C
      ELSE IF (FSHAPE.EQ.'KICK') THEN
C
        FMOD1 = 0.0 
        FMOD2 = 1.0
        IF (GRTCYC.EQ.KICKCYC) THEN
          FMOD1 = 1.0
        END IF
C
      ELSE 
C
       FMOD1 = 1.0
       FMOD2 = 1.0
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MODEFIELD ***
C
      END
