      SUBROUTINE READPROJ
C
C     Purpose: Read propagation vectors for projectiles.
C
C     History: - Creation (17.06.17, ADLL)
C
C     ******************************************************************
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'physic.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
C
      INTEGER I,NTOKEN
      REAL CLAU,MC2,NORM,RVALUE,VECT(3),VELSQ
      CHARACTER*80 DTS,LINE,LTU,PRTSTR,STRCOMP,STRING
      CHARACTER*80 TOKEN(80)
      CHARACTER*7 STRTYP
C
C     ------------------------------------------------------------------
C
C     *** Initialize ***
C
      VIORAD(:,:) = 0.0
      DPLRAD(:,:) = 0.0
C
 100  CONTINUE
C
      READ(INP,'(A80)',ERR=99990,END=99970) LINE
      STRING = STRCOMP(LTU(DTS(LINE)))
      CALL STRTOK(STRING,NTOKEN,TOKEN)
C      
C     *** Read vector components ***
C
      CALL GETNUM(TOKEN(1),STRTYP,RVALUE,I)
C
      IF (STRTYP.NE.'FLOAT') GO TO 99970
      VECT(1) = RVALUE
      CALL GETNUM(TOKEN(2),STRTYP,RVALUE,I)
      IF (STRTYP.NE.'FLOAT') GO TO 99990
      VECT(2) = RVALUE
      CALL GETNUM(TOKEN(3),STRTYP,RVALUE,I)
      IF (STRTYP.NE.'FLOAT') GO TO 99990
      VECT(3) = RVALUE
C
      IF (INDEX(TOKEN(4),'DIR').NE.0) THEN
C
C       *** Normalize vector ***
C
        NORM = VECT(1)**2 + VECT(2)**2 + VECT(3)**2
        IF (NORM.EQ.0.0) GO TO 99980
C
        VIORAD(:,NRADP) = VECT(:)/SQRT(NORM)
C 
      ELSE IF (INDEX(TOKEN(4),'DISP').NE.0) THEN
C
        DPLRAD(:,NRADP) = VECT(:)*BOHR
C
      ELSE 
        GO TO 99996 
      END IF
C
      GO TO 100
C
99970 CONTINUE
C
C     *** If RT-DFT step defined by projectile speed ***
C
      IF (RTLSTEP.NE.0.0) THEN
        MC2 = MRAD*AMU*(CLIGHT/(ABOHR*FSEC*1.E15))**2
        CLAU = CLIGHT/(ABOHR*FSEC*1.E15)
        VELSQ = CLAU**2*(1-(KINRAD/MC2 + 1)**(-2))
        RTSTEP = RTLSTEP/SQRT(VELSQ) 
      END IF
C
      RETURN
C
C     *** Handling of read errors ***
C
99980 CONTINUE
      WRITE (PRTSTR,99985)
99985 FORMAT ('NORM OF DIRECTION VECTOR EQUALS ZERO')
      CALL ERRMSG('READIONRAD',STRCOMP(PRTSTR),1)
99990 CONTINUE
      WRITE (PRTSTR,99995) 
99995 FORMAT ('ERROR READING VELOCITY VECTOR COMPONENTS')
      CALL ERRMSG('READIONRAD',STRCOMP(PRTSTR),1)
99996 CONTINUE
      WRITE (PRTSTR,99997)
99997 FORMAT ('UNKNOWN RADIATION SPECIFICATIONS')
      CALL ERRMSG('READIONRAD',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE READPROJ *** 
C
      END
C
