      SUBROUTINE BCH(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
C
C     Purpose: Matrix exponentiation using Baker–Campbell–Hausdorff
C              formula.
C
C     History: - Creation (16.11.15, XJ)
C
C     ******************************************************************
C
C     List of local variables: 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO),DT
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME)
C
      REAL DTR,DTI,M,N
      INTEGER ALLOCATION,IBCH,J,MAXLOOP,NBCH
      REAL, ALLOCATABLE :: CR(:,:),CI(:,:) 
      LOGICAL CHECKCV
C
C     -----------------------------------------------------------------
C
C     *** Start timer for TAYLOR PROPAGATION ***
C
      IF (MPMASTER) WRITE(OUT,*) 'START BCH ITERATIONS'
C
      CHECKCV = .FALSE.
      IF (NEXPM.LT.0) THEN
        CHECKCV = .TRUE.
        MAXLOOP = ABS(NEXPM)
      ELSE
        MAXLOOP = NEXPM
      END IF
C
C     *** Allocation ***
C
      ALLOCATE(CR(DSTO_ME,DSTO_ME),CI(DSTO_ME,DSTO_ME),
     $         STAT=ALLOCATION)
C
      CR(:,:) = PR(:,:) 
      CI(:,:) = PI(:,:)
C
      N = 1.0
      IBCH = 0
50    CONTINUE
      IBCH = IBCH + 1

      CALL COMCPLX(KR,KI,CR,CI,DSTO,DSTO_ME,'REPLACEB')
      N = N*IBCH
      M = 1/N
      DTR = 0.0
      DTI = 0.0
C
      IF(MOD(IBCH+3,4).EQ.0) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (CR(J,J)*(DT**IBCH)*M)**2
          DTR = DTR + (CI(J,J)*(DT**IBCH)*M)**2
        END DO
        PI(:,:) = PI(:,:) - CR(:,:)*(DT**IBCH)*M
        PR(:,:) = PR(:,:) + CI(:,:)*(DT**IBCH)*M
      ELSE IF (MOD(IBCH+3,4).EQ.1) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (CI(J,J)*(DT**IBCH)*M)**2
          DTR = DTR + (CR(J,J)*(DT**IBCH)*M)**2
        END DO
        PI(:,:) = PI(:,:) - CI(:,:)*(DT**IBCH)*M
        PR(:,:) = PR(:,:) - CR(:,:)*(DT**IBCH)*M
      ELSE IF (MOD(IBCH+3,4).EQ.2) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (CR(J,J)*(DT**IBCH)*M)**2
          DTR = DTR + (CI(J,J)*(DT**IBCH)*M)**2
        END DO
        PI(:,:) = PI(:,:) + CR(:,:)*(DT**IBCH)*M
        PR(:,:) = PR(:,:) - CI(:,:)*(DT**IBCH)*M
      ELSE IF (MOD(IBCH+3,4).EQ.3) THEN 
        DO J=1,DSTO_ME
          DTI = DTI + (CI(J,J)*(DT**IBCH)*M)**2
          DTR = DTR + (CR(J,J)*(DT**IBCH)*M)**2
        END DO
        PI(:,:) = PI(:,:) + CI(:,:)*(DT**IBCH)*M
        PR(:,:) = PR(:,:) + CR(:,:)*(DT**IBCH)*M
      END IF 
C
        DTR = SQRT(DTR+DTI)
C
C     *** Check convergence ***
C
      WRITE(OUT,5000) IBCH,DTR
5000  FORMAT(T2,'BCH ITERATIONS',1X,I4,1X,ES12.5)
C
      IF (CHECKCV) THEN
        IF(MPP) CALL MP_SEND_ALL(DTR,1,MASTER,MP_REAL)
        IF (DTR.LT.CHEBTOL) GO TO 100
        IF (IBCH.GE.MAXLOOP) CALL ERRMSG('BCH','UNABLE TO CONVERGE 
     $BCH EXPANSION',1)
        GO TO 50
      ELSE IF (IBCH.LT.MAXLOOP) THEN
        GO TO 50
      ELSE
        GO TO 100
      END IF
C
 100  CONTINUE
C
      CALL RMCHEBOUT(IBCH)
C
C    *** Deallocation ***
C
      DEALLOCATE(CI,CR,STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('BCH','DEALLOCATION FAILED',1)
      END IF
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE BCH ***
C
      END 
