      SUBROUTINE EULER(KR,KI,PR,PI,DSTO,DSTO_ME)
C
C     Purpose:  Propagate MO density matrices by Euler method
C
C     History: - Creation (29.10.15, XJ)
C
C     ******************************************************************
C
C     List of local variables: 
C
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real parts of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real parts of density matrix.
C      
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'embed.h'
      INCLUDE 'flags.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'scf.h'
      INCLUDE 'rtdft.h'  
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME)
C 
C     -----------------------------------------------------------------
C
C     *** Start timer for basis transformation ****
C
      CALL TIMER('START','AO<->MO BASIS TRANSFORMATIONS')
C
C     *** Scale density matrices in AO basis *** 
C
      CALL SCALMAT(PR,DSTO,NSTO,NSTO,'TOTWD',0.5)
      CALL SCALMAT(PI,DSTO,NSTO,NSTO,'TOTWD',0.5)
C
C     *** Transform density matrices to MO basis ***
C
      CALL TRFMOA(PR,DSTO,'PAO->PMO')
      CALL TRFMOA(PI,DSTO,'PAO->PMO')
      CALL TRFMOA(KR,DSTO,'KAO->KMO')
C
C     *** Start timer for TAYLOR PROPAGATION ***
C
      CALL TIMER('SUSPEND','AO<->MO BASIS TRANSFORMATIONS')
C
C     *** Calculate complex commutator [K,P] ***
C
      CALL COMCPLX(KR,KI,PR,PI,DSTO,DSTO_ME,'REPLACEA')
C
C     *** Perform propagation *** 
C 
      PI(:,:) = PI(:,:) - KR(:,:)*RTSTEP
      PR(:,:) = PR(:,:) + KI(:,:)*RTSTEP
C
C
C     *** Back transformation and scaling of density matrices ***
C
      CALL TIMER('CONTINUE','AO<->MO BASIS TRANSFORMATIONS')
C
      CALL TRFMOA(PI,DSTO,'PMO->PAO')
      CALL TRFMOA(PR,DSTO,'PMO->PAO')
      CALL SCALMAT(PR,DSTO,NSTO,NSTO,'TOTWD',2.0)
      CALL SCALMAT(PI,DSTO,NSTO,NSTO,'TOTWD',2.0)
C
C     ----------------------------------------------------------------
C    
C     *** End of SUBROUTINE EULER ***
C
      END 
