      SUBROUTINE MAGNUS(KR,KI,PR,PI,DSTO,DSTO_ME,DT,SPIN)
C
C     Purpose: Perform one propagation step with a split operator. 
C
C     Litterature: Cheng. et al. Phys. Rev. B 2006, 74, 155112.
C
C     History: - Creation (16.11.15, XJ)
C                         (31.08.16, ADLL)
C
C     ******************************************************************
C
C     List of local variables: 
C
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real part of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real part of density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'     
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scfvec.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME)
      CHARACTER*(*) SPIN
C
      REAL DT
C
C     -----------------------------------------------------------------
C
C     *** Start timer for basis transformation***
C
      CALL TIMER('START','AO<->MO BASIS TRANSFORMATIONS')
C
C     *** Scale density matrix and transform to MO basis ***
C
      IF (MPMASTER) THEN
        CALL SCALMAT(PR,DSTO,DSTO,DSTO,'TOTWD',0.5)
        CALL SCALMAT(PI,DSTO,DSTO,DSTO,'TOTWD',0.5)
      END IF
C
      IF (USESCALA) THEN
        CALL TRFMOASCALA(PI,DSTO,DSTO_ME,'PAO->PMO')
        CALL TRFMOASCALA(PR,DSTO,DSTO_ME,'PAO->PMO')
        CALL TRFMOASCALA(KR,DSTO,DSTO_ME,'KAO->KMO')
        IF (DOIKS) CALL TRFMOASCALA(KI,DSTO,DSTO_ME,'KAO->KMO')
      ELSE IF (USEMAGMA) THEN
        IF (MPMASTER) THEN
          CALL TRFMOAMAGMA(PI,DSTO,DSTO_ME,'PAO->PMO')
          CALL TRFMOAMAGMA(PR,DSTO,DSTO_ME,'PAO->PMO')
          CALL TRFMOAMAGMA(KR,DSTO,DSTO_ME,'KAO->KMO')
          IF (DOIKS) CALL TRFMOAMAGMA(KI,DSTO,DSTO_ME,'KAO->KMO')
        END IF
      ELSE
        CALL TRFMOA(PI,DSTO_ME,'PAO->PMO')
        CALL TRFMOA(PR,DSTO_ME,'PAO->PMO')
        CALL TRFMOA(KR,DSTO,'KAO->KMO')
        IF (DOIKS) CALL TRFMOA(KI,DSTO,'KAO->KMO')
      END IF
C
      IF (MPMASTER.AND.(CAPTYP.EQ.'ENERGY')) THEN
        CALL ADDCAP(KI,KI,DSTO,'ADD')
      END IF
C
C     *** Start timer for MATRIX EXPON ***
C
      CALL TIMER('SUSPEND','AO<->MO BASIS TRANSFORMATIONS')
      CALL TIMER('START','MATRIX EXPONENTIATION')
C
C     *** Matrix exponentiation ***
C
      IF (USESCALA) THEN
        IF (EXPMAT.EQ.'TAYLOR') THEN
          CALL TAYLORSCALA(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'DIAGO') THEN
          CALL DIAEXP(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'BCH') THEN
          CALL BCH(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'CHEBYSHEV') THEN
          CALL CHEBYSHEV(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        END IF
      ELSE IF (USEMAGMA) THEN
        IF (EXPMAT.EQ.'TAYLOR') THEN
          IF (MONOGPU) THEN
            IF (MPMASTER) THEN
              CALL TAYLORMAGMA(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
            END IF
          ELSE IF (MULTIGPU) THEN
            CALL TAYLORMAGMA_MULTI(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
          END IF
        ELSE IF (EXPMAT.EQ.'DIAGO') THEN
          CALL DIAEXP(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'BCH') THEN
          CALL BCH(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'CHEBYSHEV') THEN
          CALL CHEBYSHEV(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        END IF
      ELSE
        IF (EXPMAT.EQ.'DIAGO') THEN
          CALL DIAEXP(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'BCH') THEN
          CALL BCH(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'TAYLOR') THEN
          CALL TAYLOR(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        ELSE IF (EXPMAT.EQ.'CHEBYSHEV') THEN
          CALL CHEBYSHEV(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
        END IF                         
      END IF          
C
C     *** Stop timer for MATRIX EXPON ***
C
      CALL TIMER('STOP','MATRIX EXPONENTIATION')
C
C     *** Back transformation and scaling of density matrices *** 
C
      CALL TIMER('CONTINUE','AO<->MO BASIS TRANSFORMATIONS')
C
      IF (USESCALA) THEN
        CALL TRFMOASCALA(PI,DSTO,DSTO_ME,'PMO->PAO')
        CALL TRFMOASCALA(PR,DSTO,DSTO_ME,'PMO->PAO')
      ELSE IF (USEMAGMA) THEN
        IF (MPMASTER) THEN
          CALL TRFMOAMAGMA(PI,DSTO,DSTO_ME,'PMO->PAO')
          CALL TRFMOAMAGMA(PR,DSTO,DSTO_ME,'PMO->PAO')
        END IF
      ELSE
        CALL TRFMOA(PI,DSTO_ME,'PMO->PAO')
        CALL TRFMOA(PR,DSTO_ME,'PMO->PAO')
      END IF
C
      IF (MPMASTER) THEN
        CALL SCALMAT(PR,DSTO,DSTO,DSTO,'TOTWD',2.0)
        CALL SCALMAT(PI,DSTO,DSTO,DSTO,'TOTWD',2.0)
      END IF
C
C     *** Perform Gram-Schmidt ortogonalization  ***
C
      IF (IMTPR) CALL ORTHOMOZ(KR,KI,PR,PI,DSTO,DSTO_ME,SPIN)
C
C     *** Stop timer for basis transformation ****
C
      CALL TIMER('STOP','AO<->MO BASIS TRANSFORMATIONS')
C
C     ----------------------------------------------------------------
C    
C     *** End of SUBROUTINE MAGNUS ***
C
      END
