      SUBROUTINE PEULER(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
C
C     Purpose:  Propagate MO density matrices using Euler method
C
C     History: - Creation (29.10.15, XJ)
C
C     ******************************************************************
C     List of local variables: 
C
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real parts of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real parts of density matrix.
C      
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'embed.h'
      INCLUDE 'flags.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'scf.h'
      INCLUDE 'rtdft.h'  
C
      INTEGER DSTO,DSTO_ME
      REAL KAR(DSTO,DSTO),KAI(DSTO,DSTO)
      REAL KBR(DSTO,DSTO),KBI(DSTO,DSTO)
      REAL PAI(DSTO_ME,DSTO_ME),PAR(DSTO_ME,DSTO_ME)
      REAL PBI(DSTO_ME,DSTO_ME),PBR(DSTO_ME,DSTO_ME)
C 
      INTEGER ALLOCATION
      REAL, ALLOCATABLE :: WORKA(:,:),WORKB(:,:)
C
C     -----------------------------------------------------------------
C
C     *** Allocate dynamical arrays ***
C
      ALLOCATE(WORKA(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PEULER','ALPHA ALLOCATION FAILED',1)
      END IF
C
      IF (OKS) THEN
        ALLOCATE(WORKB(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
        CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
        IF (ALLOCATION.GT.0) THEN
          CALL ERRMSG('PEULER','BETA ALLOCATION FAILED',1)
        END IF
      END IF
C
C     *** Add electric field *** 
C
      CALL FIELDRT(KAR,DSTO,RTT,EFCT,EFWT,EFPUL,'ADD')
      IF (OKS)  THEN
        CALL FIELDRT(KBR,DSTO,RTT,EFCT,EFWT,EFPUL,'ADD')
      END IF
C
      CALL EULER(KAR,KAI,PAR,PAI,DSTO,DSTO_ME)
      WORKA(:,:) = PAR(:,:)
      IF (OKS) THEN
        CALL EULER(KBR,KBI,PBR,PBI,DSTO,DSTO_ME)
        WORKB(:,:) = PBR(:,:)
      END IF 
C
C     *** Build new Kohn-Sham matrix ***
C
      IF (OKS) THEN
        CALL RTKSMAT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
      ELSE 
        CALL RTKSMAT(KAR,KAI,KAR,KAI,PAR,PAI,PAR,PAI,DSTO,DSTO_ME) 
      END IF 
C
      PAR(:,:) = WORKA(:,:)
      IF (OKS) PBR(:,:) = WORKB(:,:) 
C         
      DEALLOCATE(WORKA,STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PEULER','ALPHA DEALLOCATION FAILED',1)
      END IF
      IF (OKS) DEALLOCATE(WORKB,STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PEULER','BETA DEALLOCATION FAILED',1)
      END IF
C
C     ----------------------------------------------------------------
C    
C     *** End of SUBROUTINE PRO_EULER ***
C
        END 
