      SUBROUTINE PMAGIT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
C
C     Purpose:  Propagate density matrix using by iterative Magnus
C               scheme.
C
C     Litterature: Cheng. et al. Phys. Rev. B 2006, 74, 155112.
C
C     History: - Creation (16.11.15, XJ)
C                         (31.08.16, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     K{A/B}I  : Alpha/beta Kohn-Sham matrix.
C     K{A/B}R  : Alpha/beta Kohn-Sham matrix.
C     P{A/B}I  : Alpha/beta Density matrix.
C     P{A/B}R  : Alpha/beta Density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'embed.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scf.h'
      INCLUDE 'xc.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KAR(DSTO,DSTO),KAI(DSTO,DSTO)
      REAL PAI(DSTO_ME,DSTO_ME),PAR(DSTO_ME,DSTO_ME)
      REAL KBR(DSTO,DSTO),KBI(DSTO,DSTO)
      REAL PBI(DSTO_ME,DSTO_ME),PBR(DSTO_ME,DSTO_ME)
C
      INTEGER ALLOCATION,NCYCMAG
      INTEGER OFFSET,IRAD
      REAL DT,TIME
      LOGICAL CVMAGIT
C
      REAL, ALLOCATABLE :: WORK1(:,:),WORK2(:,:)
C
C     -----------------------------------------------------------------
C
C     *** Allocate dynamical arrays ***
C
      ALLOCATE(WORK1(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
      IF (OKS) ALLOCATE(WORK2(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
C
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PMAGIT','ALLOCATION FAILED',1)
      END IF
C
C     *** Step 1: Extapolate H(T+DT/2) from H(T-DT) and H(T) ***
C
      IF (MPMASTER) THEN
        CALL SCFIO(WORK1,DSTO,'READ','KOLDALPR')
        CALL SCFIO(KAR,DSTO,'WRITE','KOLDALPR')
        KAR(:,:) = 1.5*KAR(:,:) - 0.5*WORK1(:,:)
        IF (DOIKS) THEN
          CALL SCFIO(WORK1,DSTO,'READ','KOLDALPI')
          CALL SCFIO(KAI,DSTO,'WRITE','KOLDALPI')
          KAI(:,:) = 1.5*KAI(:,:) - 0.5*WORK1(:,:) 
        END IF
        IF (OKS) THEN
          CALL SCFIO(WORK1,DSTO,'READ','KOLDBETR')
          CALL SCFIO(KBR,DSTO,'WRITE','KOLDBETR')
          KBR(:,:) = 1.5*KBR(:,:) - 0.5*WORK1(:,:)
          IF (DOIKS) THEN
            CALL SCFIO(WORK1,DSTO,'READ','KOLDBETI')
            CALL SCFIO(KBI,DSTO,'WRITE','KOLDBETI')
            KBI(:,:) = 1.5*KBI(:,:) - 0.5*WORK1(:,:)
          END IF
        END IF
C
      END IF
C
C     *** Calculate potential from projectile at time T + DT/2 ***
C
      IF (ADDRAD) THEN
        TIME = RTT + RTSTEP/2.0
        DT = RTSTEP/2.0
        IF (MPP) CALL MP_SEND_ALL(TIME,1,MASTER,MP_REAL)
        IF (MPP) CALL MP_SEND_ALL(DT,1,MASTER,MP_REAL)
C
        CALL SETPROJ(TIME,'PRTOUT')
        CALL EMBEDRAD
C
      END IF
C
C     *** Store density matrice at time T ***
C
      CALL STOPMAT(WORK1,PAR,PAI,DSTO_ME,'REDUCE')
      IF (OKS) CALL STOPMAT(WORK2,PBR,PBI,DSTO_ME,'REDUCE')
C
C     *** Initialize Magnus iterations ***
C
      NCYCMAG = 0
      CVMAGIT = .FALSE.
C
C     *** Enter iteration cycles ***
C
 500  CONTINUE
C
      IF (NCYCMAG.EQ.RTSMAX) THEN
        WRITE(OUT,5000) NCYCMAG,ERRCDC
        CALL ERRMSG('PMAGIT','NON-CONVERGENT '//
     $                       'ITERATIVE MAGNUS PROCEDURE',1)
      END IF
C
      NCYCMAG = NCYCMAG + 1
C
C     *** Add electric field generated by photons or projectiles ***
C
      IF (MPMASTER) THEN
        TIME = RTT + RTSTEP/2.0
        DT = RTSTEP/2.0
        CALL FIELDRT(KAR,DSTO,TIME,EFCT,EFWT,EFPUL,'ADD')
        IF (OKS) CALL FIELDRT(KBR,DSTO,TIME,EFCT,EFWT,EFPUL,'ADD')
C
        IF (ADDRAD) THEN
          CALL SCFIO(PAI,DSTO,'READ','RADIAF')
          KAR(:,:) = KAR(:,:) + PAI(:,:)
          IF (OKS) KBR(:,:) = KBR(:,:) + PAI(:,:)
        END IF
      END IF
C
C     *** Add spatial absorbing potential ***
C
      IF (CAPTYP.EQ.'SPACE') CALL ADDCAP(KAI,KBI,DSTO,'ADD')
C
C     *** Step 2: Load and Propagate P(T) to P(T + DT) ***
C
      DT = RTSTEP
      TIME = RTT + RTSTEP
C
C
      CALL STOPMAT(WORK1,PAR,PAI,DSTO_ME,'EXPAND')
      CALL MAGNUS(KAR,KAI,PAR,PAI,DSTO,DSTO_ME,DT,'ALPHA')
      IF (OKS) THEN
        CALL STOPMAT(WORK2,PBR,PBI,DSTO_ME,'EXPAND')
        CALL MAGNUS(KBR,KBI,PBR,PBI,DSTO,DSTO_ME,DT,'BETA')
      END IF
C
C     ***  At convergence store density matrix ***
C
      IF (CVMAGIT) THEN
        CALL STOPMAT(WORK1,PAR,PAI,DSTO_ME,'REDUCE')
        IF (OKS) CALL STOPMAT(WORK2,PBR,PBI,DSTO_ME,'REDUCE')
      END IF
C
C     *** Step 3: build KS(t+dt) from P(t+dt) ****
C
      IF (OKS) THEN
        CALL RTKSMAT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
      ELSE
        CALL RTKSMAT(KAR,KAI,KAR,KAI,PAR,PAI,PAR,PAI,DSTO,DSTO_ME)
      END IF
C
C     *** Remove spatial absorbing potential ***
C
      IF (CAPTYP.EQ.'SPACE') CALL ADDCAP(KAI,KBI,DSTO,'REM')
C
      IF (CVMAGIT) THEN
        CALL STOPMAT(WORK1,PAR,PAI,DSTO_ME,'EXPAND')
        IF (OKS) CALL STOPMAT(WORK2,PBR,PBI,DSTO_ME,'EXPAND')
        GO TO 800
      END IF
C
C     *** Check convergence ***
C
      ERRSCF = EUPPER - ELOWER
      IF ((ERRSCF.LE.RTETOL).AND.(ERRCDC.LE.RTCDTOL)) CVMAGIT = .TRUE.
C
      IF (MPP) CALL MP_SEND_ALL(ERRCDC,1,MASTER,MP_REAL)
      IF (MPP) CALL MP_SEND_ALL(CVMAGIT,1,MASTER,MP_INT)
      IF (MPP) CALL MP_SEND_ALL(NCYCMAG,1,MASTER,MP_INT)
C
      IF (PRTRT.AND.CVMAGIT) WRITE(OUT,5000) NCYCMAG,ERRCDC
 5000 FORMAT (T2,'MAGNUS P. CONVERGENCE IN : ',I3,
     $              ' CYCLES WITH ERRCDC = ',E20.13)
C
C     *** Interpolate H(T+DT/2) from H(T+DT) and H(T) ***
C
      IF (MPMASTER) THEN
        CALL SCFIO(PAR,DSTO,'READ','KOLDALPR')
        KAR(:,:) = 0.5*(KAR(:,:) + PAR(:,:))
        IF (DOIKS) THEN
          CALL SCFIO(PAI,DSTO,'READ','KOLDALPI')
          KAI(:,:) = 0.5*(KAI(:,:) + PAI(:,:))
        END IF
        IF (OKS) THEN
          CALL SCFIO(PBR,DSTO,'READ','KOLDBETR')
          KBR(:,:) = 0.5*(KBR(:,:) + PBR(:,:))
          IF (DOIKS) THEN
            CALL SCFIO(PBI,DSTO,'READ','KOLDBETI')
            KBI(:,:) = 0.5*(KBI(:,:) + PBI(:,:))
          END IF
        END IF
      END IF
C
      GO TO 500
C
 800  CONTINUE
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(WORK1,STAT=ALLOCATION)
      IF (OKS) DEALLOCATE(WORK2,STAT=ALLOCATION)
C
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PMAGIT','DEALLOCATION FAILED',1)
      END IF
C
C     ----------------------------------------------------------------
C
C     *** End of SUBROUTINE PMAGIT ***
C
      END
