      SUBROUTINE PMAGPC(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
C
C     Purpose:  Propagate density matrices by 
C               Predictor-Corrector 2nd order Magnus scheme.
C
C     Litterature: Cheng. et al. Phys. Rev. B 2006, 74, 155112.
C
C     History: - Creation (16.11.15, XJ)
C                         (31.08.16, ADLL)
C
C     ******************************************************************
C     List of local variables: 
C
C     List of local variables: 
C
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real parts of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real parts of density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'     
      INCLUDE 'rtdft.h'
      INCLUDE 'auxis.h'
      INCLUDE 'xc.h'
      INCLUDE 'scf.h'
      INCLUDE 'embed.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'projectile.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KAR(DSTO,DSTO),KAI(DSTO,DSTO)
      REAL PAI(DSTO_ME,DSTO_ME),PAR(DSTO_ME,DSTO_ME)
      REAL KBR(DSTO,DSTO),KBI(DSTO,DSTO)
      REAL PBI(DSTO_ME,DSTO_ME),PBR(DSTO_ME,DSTO_ME)
      INTEGER OFFSET,IRAD
C
      INTEGER ALLOCATION
      REAL DT,TIME
    
      REAL, ALLOCATABLE :: WORK1(:,:),WORK2(:,:),WORK3(:,:)
C
C     -----------------------------------------------------------------
C
C     *** Allocate dynamical arrays *** 
C
      ALLOCATE(WORK1(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
      IF (OKS) ALLOCATE(WORK2(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
C
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PMAGPC','ALLOCATION FAILED',1)
      END IF
C
C     *** Initialize Kohn-Sham matrices ***
C
      IF (MPMASTER) THEN
        CALL SCFIO(WORK1,DSTO,'READ','KOLDALPR')
        CALL SCFIO(KAR,DSTO,'WRITE','KOLDALPR')
        KAR(:,:) = 1.75*KAR(:,:) - 0.75*WORK1(:,:)
        IF (DOIKS) THEN
          CALL SCFIO(WORK1,DSTO,'READ','KOLDALPI')
          CALL SCFIO(KAI,DSTO,'WRITE','KOLDALPI')
          KAI(:,:) = 1.75*KAI(:,:) - 0.75*WORK1(:,:)
        END IF
        IF (OKS) THEN
          CALL SCFIO(WORK1,DSTO,'READ','KOLDBETR')
          CALL SCFIO(KBR,DSTO,'WRITE','KOLDBETR')
          KBR(:,:) = 1.75*KBR(:,:) - 0.75*WORK1(:,:)
          IF (DOIKS) THEN
            CALL SCFIO(WORK1,DSTO,'READ','KOLDBETI')
            CALL SCFIO(KBI,DSTO,'WRITE','KOLDBETI')
            KBI(:,:) = 1.75*KBI(:,:) - 0.75*WORK1(:,:)
          END IF
        END IF 
C
C     *** Add electric field contributions to KS matrix ***
C
        TIME = RTT + RTSTEP/4.0
        DT = RTSTEP/4.0
        CALL FIELDRT(KAR,DSTO,TIME,EFCT,EFWT,EFPUL,'ADD')
        IF (OKS) CALL FIELDRT(KBR,DSTO,TIME,EFCT,EFWT,EFPUL,'ADD')
C
        IF (CAPTYP.EQ.'SPACE') CALL ADDCAP(KAI,KBI,DSTO,'ADD')
C
      END IF
C
C     *** Add radiation particle field contributions to KS matrix ***
C
      DT = RTSTEP/4.0
      TIME = RTT + RTSTEP/4.0
      IF (ADDRAD) THEN
        IF (MPP) CALL MP_SEND_ALL(TIME,1,MASTER,MP_REAL) 
        IF (MPP) CALL MP_SEND_ALL(DT,1,MASTER,MP_REAL) 
C
C     *** Interpolate particle position **
C
        CALL SETPROJ(TIME,'NOPRT')
C
        CALL EMBEDRAD
C
        IF (MPMASTER) THEN
          CALL SCFIO(WORK1,DSTO,'READ','RADIAF')
          KAR(:,:) = KAR(:,:) + WORK1(:,:)
          IF (OKS) KBR(:,:) = KBR(:,:) + WORK1(:,:)
        END IF
      END IF
C
C     *** Store real and imaginary parts of density matrix ***
C
      CALL STOPMAT(WORK1,PAR,PAI,DSTO_ME,'REDUCE')
      IF (OKS) CALL STOPMAT(WORK2,PBR,PBI,DSTO_ME,'REDUCE') 
C
C     *** Perform Predictor step ***
C
      DT = RTSTEP/2.0
      TIME = RTT + RTSTEP/2.0

      CALL MAGNUS(KAR,KAI,PAR,PAI,DSTO,DSTO_ME,DT,'ALPHA')
      IF (OKS) CALL MAGNUS(KBR,KBI,PBR,PBI,DSTO,DSTO_ME,DT,'BETA')
C
C     *** Build new Kohn-Sham matrix ***
C
      IF (MPP) CALL MP_SEND_ALL(TIME,1,MASTER,MP_REAL) 
      IF (MPP) CALL MP_SEND_ALL(DT,1,MASTER,MP_REAL) 
      IF (OKS) THEN
        CALL RTKSMAT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
      ELSE
        CALL RTKSMAT(KAR,KAI,KAR,KAI,PAR,PAI,PAR,PAI,DSTO,DSTO_ME)
      END IF 
C 
      IF (MPMASTER) THEN
        TIME = RTT + RTSTEP/2.0
        DT = RTSTEP/2.0
        CALL FIELDRT(KAR,DSTO,TIME,EFCT,EFWT,EFPUL,'ADD')
        IF (OKS) CALL FIELDRT(KBR,DSTO,TIME,EFCT,EFWT,EFPUL,'ADD')
      END IF
C
C     *** Perform corrector step ***
C
      TIME = RTT + RTSTEP/2.0
      DT = RTSTEP/2.0
      CALL STOPMAT(WORK1,PAR,PAI,DSTO_ME,'EXPAND')
C
C     *** Load and add radiation particle contributions to KS matrix ***
C
      IF (ADDRAD) THEN
        IF (MPP) CALL MP_SEND_ALL(TIME,1,MASTER,MP_REAL)
        IF (MPP) CALL MP_SEND_ALL(DT,1,MASTER,MP_REAL)
C
C     *** Interpolate particle position **
C
        CALL SETPROJ(TIME,'PRTOUT')
C
        CALL EMBEDRAD
C
        IF (MPMASTER) THEN
          CALL SCFIO(WORK1,DSTO,'READ','RADIAF')
          KAR(:,:) = KAR(:,:) + WORK1(:,:)
        END IF
C
      END IF
C
      ALLOCATE(WORK3(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
C
      IF (MPMASTER) THEN
        WORK1(:,:) = KAR(:,:)
        WORK3(:,:) = KAI(:,:)
      END IF
      DT = RTSTEP
      TIME = RTT + RTSTEP !! for consistency
      CALL MAGNUS(KAR,KAI,PAR,PAI,DSTO,DSTO_ME,DT,'ALPHA')
C
      IF (MPMASTER) THEN
        KAR(:,:) = WORK1(:,:)
        KAI(:,:) = WORK3(:,:)
        CALL FIELDRT(KAR,DSTO,TIME,EFCT,EFWT,EFPUL,'REM')
        IF (ADDRAD) THEN
          CALL SCFIO(WORK1,DSTO,'READ','RADIAF')
          KAR(:,:) = KAR(:,:) - WORK1(:,:)
        END IF
      END IF
C
C     *** Perform corrector step for beta matrices ***
C
      IF (OKS) THEN
        CALL STOPMAT(WORK2,PBR,PBI,DSTO_ME,'EXPAND')
        IF (ADDRAD.AND.MPMASTER) THEN
          KBR(:,:) = KBR(:,:) + WORK1(:,:)
        END IF
C
        IF (MPMASTER) THEN
          WORK2(:,:) = KBR(:,:)
          WORK3(:,:) = KBI(:,:)
        END IF
C
        CALL MAGNUS(KBR,KBI,PBR,PBI,DSTO,DSTO_ME,DT,'BETA')
        IF (MPMASTER) THEN
          KBR(:,:) = WORK2(:,:)
          KBI(:,:) = WORK3(:,:)
          CALL FIELDRT(KBR,DSTO,TIME,EFCT,EFWT,EFPUL,'REM')
          IF (ADDRAD) KBR(:,:) = KBR(:,:) - WORK1(:,:) 
        END IF
      END IF
C
      IF (CAPTYP.EQ.'SPACE') CALL ADDCAP(KAI,KBI,DSTO,'REM') 
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(WORK1,WORK3,STAT=ALLOCATION)
      IF (OKS) DEALLOCATE(WORK2,STAT=ALLOCATION)
C
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PMAGPC','ALLOCATION FAILED',1)
      END IF
C
C     ----------------------------------------------------------------
C    
C     *** End of SUBROUTINE PMAGPC ***
C
      END 
