      SUBROUTINE CHLMOSZ(P,DSTO,NOMO,OPTION)
C
C     Purpose: Calculate CHoLesky localized complex molecular orbitals.
C
C     History: - Creation (20.02.19, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions
C
C     DSTO: Dimension of the Slater type orbital matrix.
C
C     List of local variables
C
C     NOMO  : Number of occupied molecular orbitals.
C     OPTION: a) OPTION = OCC, occupied space localization.
C             b) OPTION = UNO, unoccupied space localization.
C     PR    : The real part (occ/uno) density matrix.
C     PR    : The imaginary part (occ/uno) density matrix.
C     RANK  : Rank of the density matrix obtained by the decomposition.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INTEGER DSTO
      COMPLEX(KIND=8) P(DSTO,DSTO) 
C
      CHARACTER*(*) OPTION
      INTEGER COL,IUMO,NOMO,RANK
      INTEGER ALLOCATION,I,J
C
C     -----------------------------------------------------------------
C
C     *** Start timer for Cholesky localiztion of MOs ***
C
      CALL TIMER('START','CHOLESKY MO LOCALIZATION')
C
C     *** Do incomplete Cholesky decomposition ***
C
      IF (OPTION.EQ.'OCC') THEN
C        CALL TESTSP_CPLX(P,DSTO)
        CALL ICHOLESKYZ(P,DSTO,1.0E-12,RANK,'PL')
        NOMO = RANK
      ELSE IF (OPTION.EQ.'UNO') THEN
        CALL ICHOLESKYZ(P,DSTO,1.0E-6,RANK,'PL')
      ELSE
        CALL ERRMSG('CHLMOSZ','UNKNOWN OPTION FOR CHLMOS',1)
      END IF
C
C     *** Arrange unoccupied molecular orbitals coefficients ***
C
      IF (OPTION.EQ.'UNO') THEN
        IUMO = NOMO + RANK + 1
        DO COL=RANK,1,-1
          IUMO = IUMO - 1
          P(:,IUMO) = P(:,COL)
        END DO
        P(:,NOMO+RANK+1:) = 0.0
      END IF
C
C     *** Stop timer for Cholesky localization of MOs ***
C
      CALL TIMER('STOP','CHOLESKY MO LOCALIZATION')
C
C     -----------------------------------------------------------------
C
C     END OF SUBROUTINE CHLMOSZ
C
      END
