      SUBROUTINE READFRAGPDB 
C
C     Purpose: Read definition of molecular fragments based on an
C              external PDB file.
C
C     History: Creation (03.03.20, ADLL)
C
C     -----------------------------------------------------------------
C
C     List of variables:
C
C     NMOL   : Number of molecular fragments.
C     FRAGMOL: Pointer relating each atom for a molecular fragment.
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'fileio.h'
C
      INTEGER I,IDUM,NAT,NTOKEN,NSEGID,RESID,RESID_PREV
      REAL RDUM
      CHARACTER*5 CDUM,SEGID,NAME_SEG(25)
      CHARACTER*80 LINE,TOKEN(80),STRING,LTU,DTS,STRCOMP,STRTYP
      CHARACTER*80 PRTSTR
      LOGICAL NEWSEG,NEWRES
C
C     -----------------------------------------------------------------
C
      IF (MPSLAVE) GO TO 900
C
C     *** Initilialization ***
C
      NMOL = 0
C
      OPEN(PDB ,FILE='PDB',STATUS='UNKNOWN',POSITION='REWIND',
     $     ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=99900)
C
      FRAGMOL(:) = 0 
      NMOL = 0
      NAT = 0
      NSEGID = 0
      RESID_PREV = 99999 
      SEGID = 'XXXX'
C
500   CONTINUE
      READ (PDB,'(A80)',ERR=99990,END=800) LINE
      STRING = STRCOMP(LTU(DTS(LINE)))
C
C     *** Check correct string lenght ***
C
      IF (LEN(STRING).GT.80) THEN
        CALL ERRMSG('READFRAGPDB','STRING LENGTH GREATER THAN 80',1)
      END IF
C
      CALL STRTOK(STRING,NTOKEN,TOKEN)
      IF (TOKEN(1).NE.'ATOM') GO TO 500
C
      SEGID = TOKEN(NTOKEN)
      CALL GETNUM(TOKEN(5),STRTYP,RDUM,RESID)
      NAT = NAT + 1
      IF (NAT.GT.NATOM_ALL) THEN
        NAT = NAT -1 
        GO TO 800 
      END IF
C
C    *** Determine if new fragment ***
C
      NEWSEG = .TRUE.
C      
      DO I=1,NSEGID
        IF (NAME_SEG(I).EQ.SEGID) NEWSEG = .FALSE.
      END DO
C
      IF (NEWSEG) THEN
        NSEGID = NSEGID + 1
        NAME_SEG(NSEGID) = SEGID
      END IF
C
      IF ((RESID.NE.RESID_PREV).OR.
     $    ((RESID.EQ.RESID_PREV).AND.NEWSEG)) THEN
         NMOL = NMOL + 1  
         RESID_PREV = RESID
      END IF
C
      FRAGMOL(NAT) = NMOL
C
      GO TO 500
C
 800  CONTINUE
C
      IF (NAT.NE.NATOM_ALL) THEN
        WRITE (PRTSTR,5000) NAT,NATOM_ALL
5000    FORMAT('NUMBER OF ATOMS IN PDB FILE (',
     $          I6,') LOWER THAN IN GEOMETRY (',I6,')')
        CALL MESSAGE(1,STRCOMP(PRTSTR),1,1) 
      END IF
C
      CLOSE (PDB)
 900  CONTINUE
C
      IF (MPP) THEN
        CALL MP_SEND_ALL(FRAGMOL,MAXATOM_ALL,MASTER,MP_INT)
        CALL MP_SEND_ALL(NMOL,1,MASTER,MP_INT)
      END IF
C
      RETURN
C
C    *** Handling errors ***
C
99900 CONTINUE
      CALL ERRMSG('READPDB','OPEN FOR FILE PDB FAILED',1)
C
99990 CONTINUE
      CALL ERRMSG('READPDB','PROBLEM TO READ PDB FILE',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE READFRAGPDB ***
C
      END
