      SUBROUTINE RTKSMAT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
C
C     Purpose: Build new KS potential with current density. 
C
C     History: - Creation (29.10.15, XJ)
C                         (02.03.20. ADLL)
C
C     ******************************************************************
C
C     List of local variables: 
C
C     K{A/B}I  : Alpha/beta Kohn-Sham matrix.
C     K{A/B}R  : Alpha/beta Kohn-Sham matrix.
C     P{A/B}I  : Alpha/beta Density matrix.
C     P{A/B}R  : Alpha/beta Density matrix.
C      
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'embed.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'projectile.h'
      INCLUDE 'scf.h'
      INCLUDE 'rtdft.h'  
C
      INTEGER DSTO,DSTO_ME
      REAL KAR(DSTO,DSTO),KAI(DSTO,DSTO)
      REAL PAI(DSTO_ME,DSTO_ME),PAR(DSTO_ME,DSTO_ME)
      REAL KBR(DSTO,DSTO),KBI(DSTO,DSTO)
      REAL PBI(DSTO_ME,DSTO_ME),PBR(DSTO_ME,DSTO_ME)
C
      REAL TIME,MIX
C
C     -----------------------------------------------------------------
C
C     *** Initialize timer for KS potential calculation ***
C
      CALL TIMER('START','POTENTIAL CALCULATION IN RT-DFT')
C
C     *** Calculate Coulomb vector ***
C
      IF (ERITYP.EQ.'DIRECT') THEN
        CALL BLDCDJ(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
      ELSE
        CALL GETCDJ(PAR,PBR,DSTO)
      END IF
C
C     *** Calculate charge density coefficients ***
C
      CALL GETCDC
C
C     *** Update charge density coefficients ***
C
      CDCA(:NAUX) = FCDA(:NAUX)
      IF (OKS) CDCB(:NAUX) = FCDB(:NAUX)
      MIX=1.0
      CDJMA(:NAUX) = CDJMA(:NAUX) + MIX*(CDJA(:NAUX) - CDJMA(:NAUX))
      GCDCA(:NAUX) = GCDCA(:NAUX) + MIX*(GFCDA(:NAUX) - GCDCA(:NAUX))
      IF (OKS) THEN
        CDJMB(:NAUX) = CDJMB(:NAUX) + MIX*(CDJB(:NAUX) - CDJMB(:NAUX))
        GCDCB(:NAUX) = GCDCB(:NAUX) + MIX*(GFCDB(:NAUX) - GCDCB(:NAUX))
      END IF
C
      IF (MPMASTER) CALL EBOUNDS(KAR,PAR,PBR,QCDCA,QCDCB,DSTO)
C
C     *** Build new Kohn-Sham matrix ***
C
      KAI(:,:) = 0.0
      KBI(:,:) = 0.0
C
C     *** Add spatial absorbing potential ***
C
      IF (MPMASTER.AND.(CAPTYP.EQ.'SPACE')) THEN 
        CALL ADDCAP(KAI,KBI,DSTO,'ADD')
      END IF 
C 
C     *** Add XC contributions ***
C
      IF (EXX) THEN
        CALL FOCK(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
        CALL MIXIO(KAR,DSTO,'WRITE','EXXALPHA')
        IF (OKS) CALL MIXIO(KBR,DSTO,'WRITE','EXXBETA')
      END IF
C
      CALL NUMXC(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
C
      CALL BLDKSMAT(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
C
C     *** Scale open shell Kohn-Sham matrices ***
C
      IF (OKS) THEN
        CALL SCALMAT(KAR,DSTO,NSTO,NSTO,'TOTAL',2.0)
        CALL SCALMAT(KBR,DSTO,NSTO,NSTO,'TOTAL',2.0)
      END IF
C
C     *** Stop timer for KS potential calculation ***
C
      CALL TIMER('STOP','POTENTIAL CALCULATION IN RT-DFT')
C
C     ----------------------------------------------------------------
C    
C     *** End of SUBROUTINE RTKSMAT ***
C
      END 
