      SUBROUTINE RTRESTART(KR,KI,PR,PI,DSTO,DSTO_ME,OPTION,SPIN)
C
C     Purpose : Save and load data for restarting RT-TDDFT simulations. 
C
C     History: (creation 04.05.17, ADLL)
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'numint.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PR(DSTO_ME,DSTO_ME),PI(DSTO_ME,DSTO_ME)
      CHARACTER *(*) OPTION,SPIN
C
      INTEGER ALLOCATION,DAUX,IATOM,IAUX,IRAD,ISTO,JSTO,RWSTO,RWAUX
      REAL MAT(1,1),VEC(1)
      REAL,ALLOCATABLE :: WORK(:,:)
      REAL,ALLOCATABLE :: DUMVEC(:)
      INTEGER,ALLOCATABLE :: DUMMAT(:,:,:)
      INTEGER DIMA,DIMB,DIMC,DIMV
C
C     -----------------------------------------------------------------
C
C     *** Allocate dynamical arrays ***
C
      ALLOCATE(WORK(DSTO,DSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('RTRST','ALLOCATION FAILED',1)
      END IF
C
C     *** Save Data ***
C
      IF (SPIN.EQ.'ALPHA') REWIND (RSTRT)
C
      IF (OPTION.EQ.'SAVE') THEN
C
C     *** Jump for slave processors ***
C
        IF (MPSLAVE) GO TO 500
C
C     *** Save current time and cycle ***
C
        WRITE (RSTRT,ERR=99980) RTT
        WRITE (RSTRT,ERR=99980) GRTCYC
C
C     *** Compact complex density matrix ***
C 
        CALL STOPMAT(WORK,PR,PI,DSTO_ME,'REDUCE')
C
C     *** Save compacted, complex density matrix ***
C 
        WRITE (RSTRT,ERR=99980) DSTO
        WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                            ISTO=1,DSTO) 
C
C     *** Save real KS matrix ***
C
        WRITE (RSTRT,ERR=99980) ((KR(JSTO,ISTO),JSTO=1,DSTO),
     $                                          ISTO=1,DSTO)
C
C     *** Load old real KS matrix from SCF tape ***
C
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,DSTO,'READ','KOLDALPR')
        ELSE
          CALL SCFIO(WORK,DSTO,'READ','KOLDBETR')
        END IF
C
C     *** Save old real KS matrix ***
C
        WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                          ISTO=1,DSTO)
        IF (DOIKS) THEN
C
C     *** Save imaginary KS matrix ***
C
          WRITE (RSTRT,ERR=99980) ((KI(JSTO,ISTO),JSTO=1,DSTO),
     $                                        ISTO=1,DSTO)
C
C     *** Load old imaginary KS matrix from SCF tape ***
C
          IF (SPIN.EQ.'ALPHA') THEN
            CALL SCFIO(WORK,DSTO,'READ','KOLDALPI')
          ELSE
            CALL SCFIO(WORK,DSTO,'READ','KOLDBETI')
          END IF
C
C     *** Save imaginary KS matrix ***
C
          WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                            ISTO=1,DSTO)
        END IF
C
C      *** Load promolecular density from SCF tape ***
C
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,DSTO,'READ','P0ALPHA')
        ELSE 
          CALL SCFIO(WORK,DSTO,'READ','P0BETA') 
        END IF
C
        WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                          ISTO=1,DSTO)
C
C       *** Save inital MO coef in MO basis (for TDOCC)
C
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,DSTO,'READ','COLDALPHA')
        ELSE
          CALL SCFIO(WORK,DSTO,'READ','COLDBETA')
        END IF
C
        WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                          ISTO=1,DSTO)

C     *** Save absorbing potential matrix on tape ***
C
        IF (.NOT.ABSPOT) GO TO 400
C
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,DSTO,'READ','CAPA')
        ELSE IF (SPIN.EQ.'BETA') THEN
          CALL SCFIO(WORK,DSTO,'READ','CAPB')
        END IF
C
        WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                          ISTO=1,DSTO)
C
 400    CONTINUE
C
C      *** Save auxiliary density ***
C
        IF (SPIN.EQ.'ALPHA') THEN
          WRITE (RSTRT,ERR=99980) NAUX
          WRITE (RSTRT,ERR=99980) (REFCA(IAUX),IAUX=1,NAUX),
     $                            (REFCB(IAUX),IAUX=1,NAUX)
        END IF
C  
        IF (TOPCD.AND.DEFORM.AND.(SPIN.EQ.'ALPHA')) THEN
          WRITE (RSTRT,ERR=99980) (MFCDA(IAUX),IAUX=1,NAUX),
     $                            (MFCDB(IAUX),IAUX=1,NAUX)
        END IF
C
C     *** Save grid parameters ***
C
        IF (SPIN.EQ.'ALPHA') THEN
          WRITE (RSTRT,ERR=99980)(LLGP_GST(IATOM,XCGSN),IATOM=1,NATOM)
          WRITE (RSTRT,ERR=99980) ((LRADIAL_GST(IATOM,IRAD,XCGSN),
     $                                  IATOM=1,NATOM),IRAD=1,MAXRAD)
          WRITE (RSTRT,ERR=99980) (NRADIAL_GST(IATOM,XCGSN),
     $                                               IATOM=1,NATOM)
          WRITE (RSTRT,ERR=99980)(ULGP_GST(IATOM,XCGSN),IATOM=1,NATOM)
        END IF
C
C     *** Read direct KS matrix from SCF tape ***
C
        IF (ERITYP.EQ.'DIRECT') THEN

          IF (SPIN.EQ.'ALPHA') THEN
            CALL SCFIO(WORK,DSTO,'READ','KSALPHA')
          ELSE
            CALL SCFIO(WORK,DSTO,'READ','KSBETA')
          END IF
C
C     *** Save direct KS matrix ***
C
          WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                            ISTO=1,DSTO)
C
C     *** Read direct P matrix from SCF tape ***
C
          IF (SPIN.EQ.'ALPHA') THEN
            CALL SCFIO(WORK,DSTO,'READ','PALPHA')
          ELSE
            CALL SCFIO(WORK,DSTO,'READ','PBETA')
          END IF
C
C     *** Save direct P matrix ***
C
          WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                            ISTO=1,DSTO)
C
C     *** Save J vector ***
C
          IF (SPIN.EQ.'ALPHA') THEN
            WRITE (RSTRT,ERR=99980) (CDCOLDA(IAUX),IAUX=1,NAUX),
     $                              (CDCOLDB(IAUX),IAUX=1,NAUX),
     $                              (CDJOLDA(IAUX),IAUX=1,NAUX),
     $                              (CDJOLDB(IAUX),IAUX=1,NAUX)
          END IF
        END IF
C
C     *** Save orthogonalization matrix ***
C
          CALL SCFIO(WORK,DSTO,'READ','MOORTHO')
          WRITE (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,DSTO),
     $                                              ISTO=1,DSTO)
C
 500  CONTINUE
C
C     *** Read restart informations ***
C
      ELSE IF (OPTION.EQ.'RESTART') THEN
C
C     *** Jump for slave processors ***
C
        IF (MPSLAVE) GO TO 600
C
C     *** Load time and cycle information ***
C
        READ (RSTRT,ERR=99960) RTT
        READ (RSTRT,ERR=99960) GRTCYC
        FGRTCYC = GRTCYC
C
C     *** Load compacted, complex density matrix ***
C
        READ (RSTRT,ERR=99960) RWSTO 
        READ (RSTRT,ERR=99960) ((KR(JSTO,ISTO),JSTO=1,RWSTO),
     $                                         ISTO=1,RWSTO) 
C
C     *** Separate complex density matrix in Re and Im parts ***
C
        CALL STOPMAT(KR,PR,PI,RWSTO,'EXPAND')
C
C     *** Load real KS matrix ***
C 
        READ(RSTRT,ERR=99960) ((KR(JSTO,ISTO),JSTO=1,RWSTO),
     $                              ISTO=1,RWSTO)
C
C     *** Load old real KS matrix ***
C 
        READ (RSTRT,ERR=99960) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                           ISTO=1,RWSTO)
C
C     *** Save old real KS matrix to SCF tape ***
C 
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,RWSTO,'WRITE','KOLDALPR')
        ELSE
          CALL SCFIO(WORK,RWSTO,'WRITE','KOLDBETR')
        END IF
C
C     *** Load imaginary KS matrix ***
C 
        IF (DOIKS) THEN
          READ (RSTRT,ERR=99960) ((KI(JSTO,ISTO),JSTO=1,RWSTO),
     $                                             ISTO=1,RWSTO)
C
C     *** Load old imaginary KS matrix ***
C 
          READ (RSTRT,ERR=99960) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                         ISTO=1,RWSTO)
C
C     *** Save old imaginary KS matrix to SCF tape ***
C 
          IF (SPIN.EQ.'ALPHA') THEN
            CALL SCFIO(WORK,RWSTO,'WRITE','KOLDALPI')
          ELSE 
            CALL SCFIO(WORK,RWSTO,'WRITE','KOLDBETI')
          END IF
        END IF
C
C     *** Load promolecular densities ***
C
        READ (RSTRT,ERR=99960) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                          ISTO=1,RWSTO)
C
 600    CONTINUE
C
C     *** Send promolecular density and cycle information ***
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(RWSTO,1,MASTER,MP_INT)
          CALL MP_SEND_ALL(WORK,RWSTO**2,MASTER,MP_REAL)
          CALL MP_SEND_ALL(GRTCYC,1,MASTER,MP_INT)
          CALL MP_SEND_ALL(FGRTCYC,1,MASTER,MP_INT)
        END IF
C
C     *** Save promolecular density to SCF tape ***
C
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,RWSTO,'WRITE','P0ALPHA')
        ELSE
          CALL SCFIO(WORK,RWSTO,'WRITE','P0BETA')
        END IF
C
C     *** Jump to end for slave processors ***
C
        IF (MPSLAVE) GO TO 700
C
C     *** Load old MO coefficients ***
C 
        READ (RSTRT,ERR=99960) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                          ISTO=1,RWSTO)
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,RWSTO,'WRITE','COLDALPHA')
        ELSE
          CALL SCFIO(WORK,RWSTO,'WRITE','COLDBETA')
        END IF
C
C     *** Load absorbing poetntials
C
      IF (.NOT.ABSPOT) GO TO 650
C
        READ (RSTRT,ERR=99960) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                          ISTO=1,RWSTO)
        IF (SPIN.EQ.'ALPHA') THEN
          CALL SCFIO(WORK,RWSTO,'WRITE','CAPA')
        ELSE
          CALL SCFIO(WORK,RWSTO,'WRITE','CAPB')
        END IF
C
 650    CONTINUE
C
C     *** Load auxiliary density ***
C
        IF (SPIN.EQ.'ALPHA') THEN
          READ (RSTRT,ERR=99960) RWAUX 
          READ (RSTRT,ERR=99960) (REFCA(IAUX),IAUX=1,RWAUX),
     $                           (REFCB(IAUX),IAUX=1,RWAUX)
        END IF
C
        IF (TOPCD.AND.DEFORM.AND.(SPIN.EQ.'ALPHA')) THEN
          READ (RSTRT,ERR=99960) (MFCDA(IAUX),IAUX=1,RWAUX),
     $                           (MFCDB(IAUX),IAUX=1,RWAUX)
        END IF
C
C     *** Load grid parameters ***
C
        IF (SPIN.EQ.'ALPHA') THEN
          LLGP_GST(:,:) = 0
          LRADIAL_GST(:,:,:) = 0
          NRADIAL_GST(:,:) = 0
          ULGP_GST(:,:) = 0

          READ (RSTRT,ERR=99960) (LLGP_GST(IATOM,XCGSN),IATOM=1,NATOM)
          READ (RSTRT,ERR=99960) ((LRADIAL_GST(IATOM,IRAD,XCGSN),
     $                                   IATOM=1,NATOM),IRAD=1,MAXRAD)
          READ (RSTRT,ERR=99960) (NRADIAL_GST(IATOM,XCGSN),
     $                                       IATOM=1,NATOM)
          READ (RSTRT,ERR=99960) (ULGP_GST(IATOM,XCGSN),IATOM=1,NATOM)
        END IF
C
C     *** Transfer grid parameters to XC fields ***
C
        CALL GRDDEF(XCGSN,'LOAD')
C
C     *** Read direct KS matrix ***
C
        IF (ERITYP.EQ.'DIRECT') THEN
          READ (RSTRT,ERR=99960) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                             ISTO=1,RWSTO)
C
C     *** Save direct KS matrix to SCF tape ***
C
          IF (SPIN.EQ.'ALPHA') THEN
            CALL SCFIO(WORK,RWSTO,'WRITE','KSALPHA')
          ELSE
            CALL SCFIO(WORK,RWSTO,'WRITE','KSBETA')
          END IF
C
C     *** Read direct P matrix ***
C
          READ (RSTRT,ERR=99960) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                             ISTO=1,RWSTO)
C
C     *** Save direct P matrix to SCF tape ***
C
          IF (SPIN.EQ.'ALPHA') THEN
            CALL SCFIO(WORK,RWSTO,'WRITE','PALPHA')
          ELSE
            CALL SCFIO(WORK,RWSTO,'WRITE','PBETA')
          END IF
C
C     *** Read J vector ***
C
          IF (SPIN.EQ.'ALPHA') THEN
            READ (RSTRT,ERR=99960)  (CDCOLDA(IAUX),IAUX=1,RWAUX),
     $                              (CDCOLDB(IAUX),IAUX=1,RWAUX),
     $                              (CDJOLDA(IAUX),IAUX=1,RWAUX),
     $                              (CDJOLDB(IAUX),IAUX=1,RWAUX)
          END IF
        END IF
C
C     *** Recover previous orthogonalization matrix ***
C
         READ (RSTRT,ERR=99980) ((WORK(JSTO,ISTO),JSTO=1,RWSTO),
     $                                            ISTO=1,RWSTO)
         CALL SCFIO(WORK,RWSTO,'WRITE','MOORTHO')
      END IF
C
C
 700  CONTINUE
C
C     *** Deallocate dynamical arrays ***
C
      DEALLOCATE(WORK,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('RTRST','DEALLOCATION FAILED',1)
      END IF

C     *** Flush restart file ***
C
      CALL DEFLUSH(RSTRT)
C
      RETURN
C
C     *** Handling error messages ***
 
99960 CONTINUE
      CALL ERRMSG('RTRESTART','ERROR READING FROM RESTART FILE',1)
C
99970 CONTINUE
      CALL ERRMSG('RTRESTART','UNEXPECTED END OF RESTART FILE',1)
C
99980 CONTINUE
      CALL ERRMSG('RTRESTART','ERROR WRITING TO RESTART FILE',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE RTRESTART ***
C
      END

