      SUBROUTINE TRFMOA(MATRIX,DSTO_ME,OPTION)
C
C     Purpose: Perform AO <-> MO transformations.
C
C     History: - Creation (16.09.15, XJ)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of MATRIX.
C
C     List of local variables:
C    
C     MATRIX : Matrix to be transformed -> The transfored matrix MO(OA)  
C     OPTION : a) PAO->PMO, transform density matrix from AO to MO basis
C              b) PMO->PAO, transform density matrix from MO to AO basis
C              c) KAO->KMO, transform Fock matrix from AO to MO basis
C     S      : Overlap matrix -> Canonical orthogonalization matrix.
C     Y      : Y = S*eigenvalue
C     C      : Work matrix. 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C     
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'molecule.h'
      INCLUDE 'rtdft.h' 
      INCLUDE 'tolerance.h'
C
      CHARACTER*(*) OPTION
      INTEGER ALLOCATION,DSTO_ME,I
      REAL MATRIX(DSTO_ME,DSTO_ME)
C
      REAL,ALLOCATABLE ::S(:,:),Y(:,:)
      REAL,ALLOCATABLE ::C(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocation ***
C
      ALLOCATE(S(DSTO_ME,DSTO_ME),C(DSTO_ME,DSTO_ME),Y(DSTO_ME,DSTO_ME)
     $         ,STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('TRFMOA','ALLOCATION FAILED',1)
      END IF

      Y(:,:) = 0.0
      C(:,:) = 0.0
      S(:,:) = 0.0
C
C     *** Get transformation and eigenvalue eigenvalue matrices ***
C
      IF (MPMASTER) CALL SCFIO(S,DSTO_ME,'READ','ORTHO')
C
C     *** Transformation PAO -> PMO ***
C
      IF (OPTION.EQ.'PAO->PMO') THEN
        IF (MPSLAVE) GO TO 200
        DO I=1,NORTH
          Y(1:DSTO_ME,I) = S(1:DSTO_ME,I)*EGIS(I)
        END DO
 200    CONTINUE
       
        CALL MPMULMAT(Y,MATRIX,C,DSTO_ME,NORB,'TRANSA')
        CALL MPMULMAT(C,Y,MATRIX,DSTO_ME,NORB,'NORMAL')
C
C     *** Transformation PMO -> PAO ***
C
      ELSE IF (OPTION.EQ.'PMO->PAO') THEN
C
        CALL MPMULMAT(S,MATRIX,C,DSTO_ME,NORB,'NORMAL')
        CALL MPMULMAT(C,S,MATRIX,DSTO_ME,NORB,'TRANSB')
C  
C     *** Transformation KAO -> KMO ***
C
      ELSE IF (OPTION.EQ.'KAO->KMO') THEN
C
        CALL MPMULMAT(S,MATRIX,C,DSTO_ME,NORB,'TRANSA')
        CALL MPMULMAT(C,S,MATRIX,DSTO_ME,NORB,'NORMAL')
C
      ELSE
C
        CALL ERRMSG('TRFMOA','UNKNOWN OPTION',1)
C
      END IF
C
C     *** Deallocation ***
C
      DEALLOCATE(S,C,Y,STAT=ALLOCATION)
C
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('TRFMOA','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TRFMOA ***
C
      END

