      SUBROUTINE TRFMOASCALA(MATRIX,DSTO,DSTO_ME,OPTION)
C
C     Purpose: Perform AO <-> MO transformations with ScaLAPACK.
C
C     History: - Creation (11.07.19, KH)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of MATRIX.
C
C     List of local variables:
C    
C     MATRIX : Matrix to be transformed -> The transfored matrix MO(OA)  
C     OPTION : a) PAO->PMO, transform density matrix from AO to MO basis
C              b) PMO->PAO, transform density matrix from MO to AO basis
C              c) KAO->KMO, transform Fock matrix from AO to MO basis
C     S      : Overlap matrix -> Canonical orthogonalization matrix.
C     Y      : Y = S*eigenvalue
C     C      : Work matrix. 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C     
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'molecule.h'
      INCLUDE 'rtdft.h' 
      INCLUDE 'tolerance.h'
      INCLUDE 'slp.h'
C
      CHARACTER*(*) OPTION
      INTEGER ALLOCATION,I
      INTEGER(kind=4) DSTO,DSTO_ME
      REAL MATRIX(DSTO_ME,DSTO_ME)

C     MPI and BLACS variables 
      integer(kind=4)               :: nb_procs,rank
C     local row and column index for the blacs grid
C     context variable for the blacs environment 
      !integer(kind=4)               :: ictxt=0
      !integer(kind=4)               :: ictxt_0=-1
C     the process row and column parameters over which the first row of
C     the matrix is distributed
      integer(kind=4)               :: rsrc,csrc
C     blacs descriptor for the local array (9 parameters)
      integer(kind=4), dimension(9) :: desc_0, desc_loc
C     the leading dimension of the local array storing the local blocks
C     of the distributed matrix
      integer(kind=4)               :: lld
C     Global row and column indexes which points to the beginning of the
C     submatrices
      integer(kind=4)               :: i_matrix, j_matrix
      integer(kind=4)               :: i_s, j_s
      integer(kind=4)               :: i_y, j_y
      integer(kind=4)               :: i_c, j_c
C     dimension of the local sub-blocs
      integer(kind=4)               :: dim_row, dim_col
C     factor for the pdgemm subroutine such as C = alpha * A * B + beta
C     * C
      real(kind=8)                  :: alpha,beta
C     exit error info 
      integer(kind=4)               :: info
C
      real(kind=8),allocatable      :: MATRIX_grid(:,:)
      real(kind=8),allocatable      :: S(:,:), S_grid(:,:)
      real(kind=8),allocatable      :: Y(:,:), Y_grid(:,:)
      real(kind=8),allocatable      :: C_grid(:,:)

      external blacs_pinfo
      external blacs_gridinfo
      external pdgemr2d
      external pdgemm
      integer, external             :: numroc

C     setting the BLACS and PBLAS variables
      rsrc = 0
      csrc = 0
      i_matrix = 1
      j_matrix = 1
      i_s = 1
      j_s = 1
      i_y = 1
      j_y = 1
      i_c = 1
      j_c = 1
      alpha = 1.0
      beta  = 0.0
    
C     getting information about the grid definition and local processes
      call blacs_pinfo(rank, nb_procs)
      call blacs_gridinfo( ictxt, nprow, npcol, myrow, mycol )

C     calculation of lld and the dimension of the local sub matrix
      lld = numroc(DSTO, mbsub, myrow, rsrc, nprow )
      dim_row = numroc(DSTO, mbsub, myrow, rsrc, nprow )
      dim_col = numroc(DSTO, nbsub, mycol, rsrc, npcol )

C     initialization of the local descriptor
      call descinit(desc_loc, DSTO, DSTO, mbsub, nbsub,
     $     rsrc, csrc, ictxt, lld, info)

C     initialization of the descriptor for the global matrix
      desc_0(:) = 0
      desc_0(2) = -1
      if(rank == 0) then
         call descinit(desc_0,
     $        DSTO, DSTO, DSTO, DSTO,
     $        rsrc, csrc, ictxt_0, DSTO, info)
C
C     *** Get transformation and eigenvalue eigenvalue matrices ***
C
         allocate(S(DSTO,DSTO))
         S(:,:) = 0.0
         call SCFIO(S,DSTO_ME,'READ','ORTHO')
      end if
C
C     ------------------------------------------------------------------
C
C     *** Allocation ***
C
      allocate(MATRIX_grid(dim_row,dim_col))
      allocate(C_grid(dim_row,dim_col))

      call pdgemr2d(DSTO, DSTO, MATRIX, 1, 1, desc_0,
     $     MATRIX_grid, 1, 1, desc_loc, ictxt)

C
C     *** Transformation PAO -> PMO ***
C
      IF (OPTION.EQ.'PAO->PMO') THEN
 
        allocate(Y_grid(dim_row,dim_col))
 
        !print *,"PAO->PMO SCALA",DSTO_ME
        IF (rank == 0) THEN
          allocate(Y(DSTO,DSTO))
          Y(:,:) = 0.0
          DO I=1,NORTH
            Y(1:DSTO_ME,I) = S(1:DSTO_ME,I)*EGIS(I)
          END DO
        END IF
      
        call pdgemr2d(DSTO, DSTO, Y, 1, 1, desc_0,
     $       Y_grid, 1, 1, desc_loc, ictxt)
      
        call pdgemm('T','N',DSTO,DSTO,DSTO,alpha,Y_grid,
     $       i_y, j_y, desc_loc, MATRIX_grid, i_matrix, j_matrix,
     $       desc_loc, beta, C_grid, i_c, j_c, desc_loc)

        call pdgemm('N','N',DSTO,DSTO,DSTO,alpha,C_grid,
     $       i_c, j_c, desc_loc, Y_grid, i_y, j_y,
     $       desc_loc, beta, MATRIX_grid, i_matrix, j_matrix, desc_loc)

        if (rank == 0) then
          deallocate(Y,STAT=ALLOCATION)
        end if
        deallocate(Y_grid,STAT=ALLOCATION)
C
C     *** Transformation PMO -> PAO ***
C
      ELSE IF (OPTION.EQ.'PMO->PAO') THEN
C
        allocate(S_grid(dim_row,dim_col))
        call pdgemr2d(DSTO, DSTO, S, 1, 1, desc_0,
     $       S_grid, 1, 1, desc_loc, ictxt)
 
        call pdgemm('N','N',DSTO,DSTO,DSTO,alpha,S_grid,
     $       i_s, j_s, desc_loc, MATRIX_grid, i_matrix, j_matrix,
     $       desc_loc, beta, C_grid, i_c, j_c, desc_loc)

        call pdgemm('N','T',DSTO,DSTO,DSTO,alpha,C_grid,
     $       i_c, j_c, desc_loc, S_grid, i_s, j_s,
     $       desc_loc, beta, MATRIX_grid, i_matrix, j_matrix, desc_loc)

        deallocate(S_grid,STAT=ALLOCATION)

C  
C     *** Transformation KAO -> KMO ***
C
      ELSE IF (OPTION.EQ.'KAO->KMO') THEN
C
        allocate(S_grid(dim_row,dim_col))
        call pdgemr2d(DSTO, DSTO, S, 1, 1, desc_0,
     $       S_grid, 1, 1, desc_loc, ictxt)
 
        call pdgemm('T','N',DSTO,DSTO,DSTO,alpha,S_grid,
     $       i_s, j_s, desc_loc, MATRIX_grid, i_matrix, j_matrix,
     $       desc_loc, beta, C_grid, i_c, j_c, desc_loc)

        call pdgemm('N','N',DSTO,DSTO,DSTO,alpha,C_grid,
     $       i_c, j_c, desc_loc, S_grid, i_s, j_s,
     $       desc_loc, beta, MATRIX_grid, i_matrix, j_matrix, desc_loc)

        deallocate(S_grid,STAT=ALLOCATION)

C
      ELSE
C
        CALL ERRMSG('TRFMOA','UNKNOWN OPTION',1)
C
      END IF



C    distribute the local sub matrices MATRIX with a 2D Block Cyclic
C    Distribution to the global matrix MATRIX 

      call pdgemr2d(DSTO, DSTO, MATRIX_grid,
     $     1, 1, desc_loc, MATRIX, 1, 1, desc_0, ictxt)

C
C     *** Deallocation ***
C
      if(rank == 0) then
        deallocate(S,STAT=ALLOCATION)
      end if        
      deallocate(MATRIX_grid,C_grid,STAT=ALLOCATION)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TRFMOASCALA ***
C
      END
C
