      SUBROUTINE WFNBAS
C
C     Purpose: Write basis set information for WFN file output.
C
C     History: - Creation (15.01.09, ADLL)
C                         (18.03.09, AMK)
C                         (20.10.17, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C     NPTO: Number of primitive type orbitals.
C
C     List of local dynamical fields:
C
C     WFNPTR(K): WFN file pointer.
C                (1) Atomic center of primitive orbital.
C                (2) Angular momentum index assignment.
C     ZETA     : Exponent of primitive orbital.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'molecule.h'
      INCLUDE 'pointer.h'
C
      INTEGER AX,AY,AZ,IATOM,IGTO,IPTO,ISHL,ISTO,L,NPTO
C
      INTEGER ALLOCATION,DIMPTO
      INTEGER,ALLOCATABLE :: WFNPTR(:,:)
      REAL,ALLOCATABLE :: ZETA(:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      NPTO = DIMPTO(NSHL,NCO,SHLPTR)
C
      ALLOCATE (WFNPTR(NPTO,2),ZETA(NPTO),STAT=ALLOCATION)
      IF (ALLOCATION.NE.0) CALL ERRMSG('WFNBAS','ALLOCATION FAILED',1)
C
C     *** Loop over all primitive GTOs ***
C
      NPTO = 0
      DO 40 IATOM=1,NATOM
        DO 30 ISHL=LLSHL(IATOM),ULSHL(IATOM)
          L = SHLPTR(ISHL,3)
          DO 20 ISTO=LLSTO(ISHL),ULSTO(ISHL)
            AX = STOPTR(ISTO,3)
            AY = STOPTR(ISTO,4)
            AZ = STOPTR(ISTO,5)
            DO 10 IGTO=LLGTO(ISHL),ULGTO(ISHL)
              NPTO = NPTO + 1
              ZETA(NPTO) = ZET(IGTO)
              WFNPTR(NPTO,1) = IATOM
              IF (L.EQ.0) THEN
                WFNPTR(NPTO,2) = 1
              ELSE IF (L.EQ.1) THEN
                IF (AX.EQ.1) THEN
                  WFNPTR(NPTO,2) = 2
                ELSE IF (AY.EQ.1) THEN
                  WFNPTR(NPTO,2) = 3
                ELSE IF (AZ.EQ.1) THEN
                  WFNPTR(NPTO,2) = 4
                END IF
              ELSE IF (L.EQ.2) THEN
                IF (AX.EQ.2) THEN
                  WFNPTR(NPTO,2) = 5
                ELSE IF (AY.EQ.2) THEN
                  WFNPTR(NPTO,2) = 6
                ELSE IF (AZ.EQ.2) THEN
                  WFNPTR(NPTO,2) = 7
                ELSE IF ((AX.EQ.1).AND.(AY.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 8
                ELSE IF ((AX.EQ.1).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 9
                ELSE IF ((AY.EQ.1).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 10
                END IF
              ELSE IF (L.EQ.3) THEN
                IF (AX.EQ.3) THEN
                  WFNPTR(NPTO,2) = 11 
                ELSE IF (AY.EQ.3) THEN
                  WFNPTR(NPTO,2) = 12
                ELSE IF (AZ.EQ.3) THEN
                  WFNPTR(NPTO,2) = 13
                ELSE IF ((AX.EQ.1).AND.(AY.EQ.2)) THEN
                  WFNPTR(NPTO,2) = 14
                ELSE IF ((AX.EQ.2).AND.(AY.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 15
                ELSE IF ((AX.EQ.2).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 16
                ELSE IF ((AX.EQ.1).AND.(AZ.EQ.2)) THEN
                  WFNPTR(NPTO,2) = 17
                ELSE IF ((AY.EQ.1).AND.(AZ.EQ.2)) THEN
                  WFNPTR(NPTO,2) = 18
                ELSE IF ((AY.EQ.2).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 19
                ELSE IF ((AX.EQ.1).AND.(AY.EQ.1).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 20
                END IF
              ELSE IF (L.EQ.4) THEN
                IF (AZ.EQ.4) THEN
                  WFNPTR(NPTO,2) = 21
                ELSE IF ((AY.EQ.1).AND.(AZ.EQ.3)) THEN
                  WFNPTR(NPTO,2) = 22 
                ELSE IF ((AY.EQ.2).AND.(AZ.EQ.2)) THEN
                  WFNPTR(NPTO,2) = 23
                ELSE IF ((AY.EQ.3).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 24
                ELSE IF (AY.EQ.4) THEN
                  WFNPTR(NPTO,2) = 25
                ELSE IF ((AX.EQ.1).AND.(AZ.EQ.3)) THEN
                  WFNPTR(NPTO,2) = 26
                ELSE IF ((AX.EQ.1).AND.(AY.EQ.1).AND.(AZ.EQ.2)) THEN
                  WFNPTR(NPTO,2) = 27
                ELSE IF ((AX.EQ.1).AND.(AY.EQ.2).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 28
                ELSE IF ((AX.EQ.1).AND.(AY.EQ.3)) THEN
                  WFNPTR(NPTO,2) = 29
                ELSE IF ((AX.EQ.2).AND.(AZ.EQ.2)) THEN
                  WFNPTR(NPTO,2) = 30
                ELSE IF ((AX.EQ.2).AND.(AY.EQ.1).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 31
                ELSE IF ((AX.EQ.2).AND.(AY.EQ.2)) THEN
                  WFNPTR(NPTO,2) = 32
                ELSE IF ((AX.EQ.3).AND.(AZ.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 33
                ELSE IF ((AX.EQ.3).AND.(AY.EQ.1)) THEN
                  WFNPTR(NPTO,2) = 34
                ELSE IF (AX.EQ.4) THEN
                  WFNPTR(NPTO,2) = 35
                END IF
              ELSE
                CALL ERRMSG('WFNBAS','ONLY L <= 4 IN WFN FORMAT',0)
              END IF
   10       CONTINUE
   20     CONTINUE
   30   CONTINUE
   40 CONTINUE
C
C     *** Write basis set information to WFN file ***
C
      WRITE (MOL,5000) (WFNPTR(IPTO,1),IPTO=1,NPTO)
      WRITE (MOL,5010) (WFNPTR(IPTO,2),IPTO=1,NPTO)
      WRITE (MOL,5020) (ZETA(IPTO),IPTO=1,NPTO)
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(WFNPTR,ZETA,STAT=ALLOCATION)
      IF (ALLOCATION.NE.0) CALL ERRMSG('WFNBAS','DEALLOCATION FAILED',1)
C
C     ------------------------------------------------------------------
C
C     *** Format statements ***
C
 5000 FORMAT ('CENTER ASSIGNMENTS ',15I4)
 5010 FORMAT ('TYPE ASSIGNMENTS   ',15I4)
 5020 FORMAT ('EXPONENTS ',5D14.7)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE WFNBAS ***
C
      END 
