      SUBROUTINE WPSIZ(CAR,CBR,CAI,CBI,CDC,DSTO,FUNVEC,LLGPB,NGPB)
C
C     Purpose: Wrapper for molecular orbital calculation.
C
C     History: - Creation (07.01.00, AMK)
C
C     ******************************************************************
C
C     List of wrapper arguments:
C
C     CA{R/I}   : {Real/Imaginary} part of Alpha MO coefficient matrix.
C     CB{R/I}   : {Real/Imaginary} part of Beta MO coefficient matrix.
C     CDC       : Charge density coefficients (not used).
C     DSTO      : Dimension of Slater typ orbital matrix.
C     FUNVEC    : Vector with plot function values.
C     LLGPB     : Lower limit grid point of current batch.
C     NGPB      : Number of grid points in current batch.
C
C     List of local dynamical fields:
C
C     STO: Function values of the basis functions.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INTEGER DSTO
      REAL CAR(DSTO,DSTO),CBR(DSTO,DSTO),CAI(DSTO,DSTO),CBI(DSTO,DSTO)
C
      REAL CDC(MAXAUX),FUNVEC(MAXGPB,MAXPLOT)
C
      INTEGER LLGPB,NGPB
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: STO(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local field ***

      ALLOCATE(STO(MAXGPB,DSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) CALL ERRMSG('WPSI','ALLOCATION FAILED',1)
C
C     *** Calculate MO values ***
C
      CALL CHI(STO,DSTO,LLGPB,NGPB)
      CALL PSIZ(CAR,CBR,CAI,CBI,STO,DSTO,FUNVEC,NGPB)
C
C     *** Deallocate local field ***
C
      DEALLOCATE(STO,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) CALL ERRMSG('WPSI','DEALLOCATION FAILED',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE WPSI ***
C
      END
