      SUBROUTINE XCINI
C
C     Purpose: Initilization routine for the calculation of the
C              exchange-correlation energy and potential.
C
C     History: - Creation (17.06.95, MK)
C                         (13.11.03, AVA)
C                         (15.08.15, DMR)
C                         (01.04.18, FAD)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'math.h'
      INCLUDE 'numint.h'
      INCLUDE 'xc.h'
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      IF (INITXC) THEN
C
C     *** Exponents for exchange correlation energy and potential ***
C
        EXP13 = 1.0/3.0
        EXP16 = 1.0/6.0
        EXP76 = 7.0/6.0
        EXP37 = 3.0/7.0
        EXP23 = 2.0*EXP13
        EXP43 = 4.0*EXP13
        EXP53 = 5.0*EXP13
        EXP73 = 7.0*EXP13
        EXP83 = 8.0*EXP13
        EXP512 = 5.0/12.0
        EXP113 = 11.0*EXP13
C
C     ------------------------------------------------------------------
C
C     *** Factors for LDA exchange energy and potential ***
C
C     *** LSD approximation ***
C
        CELSD = -(3.0/4.0)*(3.0/PI)**EXP13
        CVLSD = -(3.0/PI)**EXP13
        CXVG = -3.0/(8.0*PI)
        CKF = (3.0*PI**2)**EXP13
C
C     *** X-alpha ***
C
        CEXALPHA = (3.0/2.0)*XALPHA
C
C     ------------------------------------------------------------------
C
C     *** Factors for GGA exchange energy and potential ***
C
C     *** Perdew & Wang 86 exchange ***
C
        APW86 = 15.0*0.0864
        BPW86 = 14.0
        CPW86 = 0.2
        MPW86 = 1.0/15.0
C
C     *** Becke 88 exchange ***
C
        BETAB88 = 0.0042
        AB88 = BETAB88*16.0*PI*(2.0*PI*PI/9.0)**EXP13
        BB88 = BETAB88*12.0*(6.0*PI*PI)**EXP13
        CB88 = 2.0*(6.0*PI*PI)**EXP13
        AVB88 = BETAB88*(2.0/3.0)*(4.0*PI/3.0)**EXP13
        BVB88 = BETAB88*(4.0/3.0)*(4.0*PI/3.0)**EXP13
        B6B88 = 6.0*BETAB88
        B3B88 = 3.0*BETAB88
C
C     *** Perdew & Wang 91 exchange ***
C
        APW91 = 7.7956
        A1PW91 = 0.19645
        A2PW91 = 0.27430
        A3PW91 = 0.15084
        B1PW91 = 0.00400
C
C     *** Engel, Chevary, Macdonald & Vosko 92 exchange ***
C
        A1ECMV =  0.3402
        A2ECMV =  5.9955
        B1ECMV = 27.5026
        B2ECMV =  5.7728
C
C     *** Engel & Vosko 93 exchange ***
C
        A1EV93 = 0.123456
        A2EV93 = 0.612889
        A3EV93 = 0.006117
        B1EV93 = 1.523671
        B2EV93 = 0.367229
        B3EV93 = 0.011282
C
C     *** Perdew, Burke & Ernzerhof 96 exchange ***
C
        KPBE96 = 0.804
        BETAPBE96 = 0.066725
        ONEPKPBE96 = 1.0 + KPBE96
        MUPBE96 = BETAPBE96*PI**2/3.0
        MUOVKPBE96 = MUPBE96/KPBE96
C
C     *** Zhang & Yang modified PBE 98 exchange ***
C
        KPBE98 = 1.245
        ONEPKPBE98 = 1.0 + KPBE98
        MUOVKPBE98 = MUPBE96/KPBE98
C
C     *** PBE exchange for solids (PBEsol) ***
C
        MUPBESOL = 10.0/81.0
        MUOVKPBESOL = MUPBESOL/KPBE96
C
C     *** OPTX Handy & Cohen exchange ***
C
        GAMAHC = 0.006
        A1HC =  1.05151
        A2HC = -1.43169/(2.0**EXP13*CELSD)
        A3HC =  (4.0*(6.0*PI**2.0)**EXP23)*GAMAHC
        A4HC =  4.0*A2HC/A3HC
C
C      *** Keal & Tozer type exchange ***
C
        ALPHAKT1 =  1.0
        DELTAKT1 =  0.1
        GAMMAKT1 = -0.006
C
        ALPHAKT2 =  1.07173
        DELTAKT2 =  0.1
        GAMMAKT2 = -0.006
C
        ALPHAKT3 =  1.092
        DELTAKT3 =  0.1
        EPSILKT3 = -0.925452
        GAMMAKT3 = -0.004
C
C      *** Vela, Medel & Trickey exchange ***
C
        MUVMT = BETAPBE96*PI**2/3.0
        MUVMTSOL = 10.0/81.0
        AVMT = 0.002762
        AVMTSOL = 0.001553
C
C     *** Peverati, Zhao & Truhlar SO11 exchange ***
C
        AXSO11(0) =  5.0E-1
        AXSO11(1) = -2.95535
        AXSO11(2) =  1.57974E1
        AXSO11(3) = -9.11804E1
        AXSO11(4) =  9.62030E1
        AXSO11(5) =  1.86830E-1
        BXSO11(0) =  5.0E-1
        BXSO11(1) =  3.50743
        BXSO11(2) = -1.29523E1
        BXSO11(3) =  4.97870E1
        BXSO11(4) = -3.32545E1
        BXSO11(5) = -1.11396E1
C
        KSO11 = AXSO11(1) + BXSO11(1)
        MUSO11 = 10.0/(81.0*KSO11)
C
C     *** Peverati & Truhlar N12 exchange ***
C
        GAMMAN12 = 0.004
        OMEGAN12 = 2.5
C
        AN12(0,0) =    1.0000000
        AN12(1,0) =    0.5078800
        AN12(2,0) =    0.1682330
        AN12(3,0) =    0.1288870
        AN12(0,1) =    0.0860211
        AN12(1,1) =  -17.1008000
        AN12(2,1) =   65.0814000
        AN12(3,1) =  -70.1726000
        AN12(0,2) =   -0.3907550
        AN12(1,2) =   51.3392000
        AN12(2,2) = -166.2200000
        AN12(3,2) =  142.7380000
        AN12(0,3) =    0.4036110
        AN12(1,3) =  -34.4631000
        AN12(2,3) =   76.1661000
        AN12(3,3) =   -2.4183400
C
C     ------------------------------------------------------------------
C
C     *** Factors for CAM-GGA exchange energy and potential ***
C
        PI12 = SQRT(PI)
        KLDA = 3.0*(3.0/(4.0*PI))**EXP13
        C6PI12 = 6.0*PI12
C
        CG(1) = 1.0/36.0
        CG(2) = 1.0/960.0
        CG(3) = 1.0/26880.0
        CG(4) = 7.0/829440.0
        CG(5) = 1.0/28385280.0
C
        DCG(1) = 1.0/18.0
        DCG(2) = 1.0/240.0
        DCG(3) = 1.0/4480.0
        DCG(4) = 2.0/207360.0
        DCG(5) = 0.0
C
C
C     *** Carmona-Espindola, Gazquez, Vela & Trickey CAP exchange ***
C
        MUCAP = BETAPBE96*PI**2/3.0
        CCAP = 3.0*MUCAP/(4.0*PI)
C
C     *** Yu, Zhang, Verma, He & Truhlar GAM exchange ***
C
        AGAM(0,0) =   1.3273000
        AGAM(0,1) =  -0.7860180
        AGAM(0,2) =   0.8025750
        AGAM(0,3) =  -0.1423310
        AGAM(1,0) =   0.8861020
        AGAM(1,1) =  -4.7878700
        AGAM(1,2) =  14.4363000
        AGAM(1,3) = -13.4598000
        AGAM(2,0) =  -5.7383300
        AGAM(2,1) =   3.9098900
        AGAM(2,2) =   8.4273500
        AGAM(2,3) =   1.5235500
        AGAM(3,0) =   8.6019700
        AGAM(3,1) =  -2.1161100
        AGAM(3,2) =  -6.2155200
        AGAM(3,3) = -10.0530000
C
C     ------------------------------------------------------------------
C
C     *** Factors for meta-GGA exchange energy and potential ***
C
C     *** Voohris & Scuseria 98 exchange ***
C
        AXVS98 = CELSD*2.0**EXP13
        CFVS98 = (3.0/5.0)*(6.0*PI*PI)**EXP23
        GGVS98 = 0.00186726
C
        XVS98(1,1) = -9.800683E-01
        XVS98(2,1) = -3.556788E-03
        XVS98(3,1) =  6.250326E-03
        XVS98(4,1) = -2.354518E-05
        XVS98(5,1) = -1.282732E-04
        XVS98(6,1) =  3.574822E-04
        XVS98(1,2) =  6.012244E-01*AXVS98
        XVS98(2,2) =  4.748822E-03*AXVS98
        XVS98(3,2) = -8.635108E-03*AXVS98
        XVS98(4,2) = -9.308062E-06*AXVS98
        XVS98(5,2) =  4.482811E-05*AXVS98
        XVS98(6,2) =  0.000000E+00
        XVS98(1,3) = -1.179732E-01*AXVS98
        XVS98(2,3) = -2.500000E-03*AXVS98
        XVS98(3,3) = -1.180065E-02*AXVS98
        XVS98(4,3) =  0.000000E+00
        XVS98(5,3) =  0.000000E+00
        XVS98(6,3) =  0.000000E+00
        XVS98(1,4) =  1.422057E-01*AXVS98
        XVS98(2,4) =  7.370319E-04*AXVS98
        XVS98(3,4) = -1.601373E-02*AXVS98
        XVS98(4,4) =  0.000000E+00
        XVS98(5,4) =  0.000000E+00
        XVS98(6,4) =  0.000000E+00
C
C     *** Perdew, Kurth, Zupan & Blaha 99 exchange ***
C
        DPKZB = 0.113
        KPKZB = 0.804
        C0PKZB = 10.0/81.0
        C1PKZB = 2126.0/10125.0
        C2PKZB = 2263.0/145800.0+DPKZB+C0PKZB**2/KPKZB
        C3PKZB = -146.0/375.0
        C4PKZB = 584.0/225.0
        C5PKZB = -2336.0/2025.0
        C6PKZB = 73.0/5000.0
C
C     *** Tao, Perdew, Staroverov & Scuseria 03 exchange ***
C
        BTPSS = 0.40
        ETPSS = 1.537
        KTPSS = 0.804
        MTPSS = 0.21951
        C0TPSS = 9.0/20.0
        C1TPSS = 10.0/81.0
        C2TPSS = 1.59096
        C3TPSS = 146.0/2025.0
        C4TPSS = 73.0/(405.0*SQRT(2.0))
        C5TPSS = 9.0/25.0
        C6TPSS = 4.0/45.0*SQRT(ETPSS)
        C7TPSS = MTPSS*ETPSS
        C8TPSS = SQRT(ETPSS)
C
C     *** Zhao & Truhlar M06 exchange ***
C
        XM06(1,1)  =  3.987756E-01
        XM06(2,1)  =  2.548219E-01
        XM06(3,1)  =  3.923994E-01
        XM06(4,1)  = -2.103655E+00
        XM06(5,1)  = -6.302147E+00
        XM06(6,1)  =  1.097615E+01
        XM06(7,1)  =  3.097273E+01
        XM06(8,1)  = -2.318489E+01
        XM06(9,1)  = -5.673480E+01
        XM06(10,1) =  2.160364E+01
        XM06(11,1) =  3.421814E+01
        XM06(12,1) = -9.049762E+00
        XM06(1,2)  =  1.179732E-01
        XM06(2,2)  = -1.066708E+00
        XM06(3,2)  = -1.462405E-01
        XM06(4,2)  =  7.481848E+00
        XM06(5,2)  =  3.776679E+00
        XM06(6,2)  = -4.436118E+01
        XM06(7,2)  = -1.830962E+01
        XM06(8,2)  =  1.003903E+02
        XM06(9,2)  =  3.864360E+01
        XM06(10,2) = -9.806018E+01
        XM06(11,2) = -2.557716E+01
        XM06(12,2) =  3.590404E+01
        XM06(1,3)  =  5.877943E-01
        XM06(2,3)  = -1.371776E-01
        XM06(3,3)  =  2.682367E-01
        XM06(4,3)  = -2.515898E+00
        XM06(5,3)  = -2.978892E+00
        XM06(6,3)  =  8.710679E+00
        XM06(7,3)  =  1.688195E+01
        XM06(8,3)  = -4.489724E+00
        XM06(9,3)  = -3.299983E+01
        XM06(10,3) = -1.449050E+01
        XM06(11,3) =  2.043747E+01
        XM06(12,3) =  1.256504E+01
        XM06(1,4)  =  4.600000E-01
        XM06(2,4)  = -2.206052E-01
        XM06(3,4)  = -9.431788E-02
        XM06(4,4)  =  2.164494E+00
        XM06(5,4)  = -2.556466E+00
        XM06(6,4)  = -1.422133E+01
        XM06(7,4)  =  1.555044E+01
        XM06(8,4)  =  3.598078E+01
        XM06(9,4)  = -2.722754E+01
        XM06(10,4) = -3.924093E+01
        XM06(11,4) =  1.522808E+01
        XM06(12,4) =  1.522227E+01
C
C     *** Peverati & Truhlar M11 exchange ***
C
        KM11 = 0.552
        MUM11 = 10.0/81.0
        MUOVKM11 = MUM11/KM11
C
        XM11(0,1,1) =    0.8121131
        XM11(1,1,1) =   17.3812400
        XM11(2,1,1) =    1.1540070
        XM11(3,1,1) =   68.6955600
        XM11(4,1,1) =  101.6864000
        XM11(5,1,1) =   -5.8874670
        XM11(6,1,1) =   45.1740900
        XM11(7,1,1) =   -2.7731490
        XM11(8,1,1) =  -26.1721100
        XM11(0,2,1) =    0.1878869
        XM11(1,2,1) =  -16.5387700
        XM11(2,2,1) =    0.6755753
        XM11(3,2,1) =  -75.6757200
        XM11(4,2,1) = -104.0272000
        XM11(5,2,1) =   18.3185300
        XM11(6,2,1) =  -55.7335200
        XM11(7,2,1) =   -3.5202100
        XM11(8,2,1) =   37.2427600
C
        XM11(0,1,2) =   -0.4386615
        XM11(1,1,2) = -121.4016000
        XM11(2,1,2) = -139.3573000
        XM11(3,1,2) =   -2.0466490
        XM11(4,1,2) =   28.0409800
        XM11(5,1,2) =  -13.1225800
        XM11(6,1,2) =   -6.3618190
        XM11(7,1,2) =   -0.8055758
        XM11(8,1,2) =    3.7365510
        XM11(0,2,2) =    1.4386620
        XM11(1,2,2) =  120.9465000
        XM11(2,2,2) =  132.8252000
        XM11(3,2,2) =   12.9635500
        XM11(4,2,2) =    5.8548660
        XM11(5,2,2) =   -3.3781620
        XM11(6,2,2) =  -44.2339300
        XM11(7,2,2) =    6.8444750
        XM11(8,2,2) =   19.4954100
C
C     *** Peverati & Truhlar MN12 exchange ***
C
        AMN12(0:3,0:3,0:5) = 0.0
C
        AMN12(0,0,0) =  6.73598E-01
        AMN12(0,0,1) = -2.27060
        AMN12(0,0,2) = -2.61371
        AMN12(0,0,3) =  3.99361
        AMN12(0,0,4) =  4.63557
        AMN12(0,0,5) =  1.25068
        AMN12(0,1,0) =  8.44492E-01
        AMN12(0,1,1) = -1.30117E+01
        AMN12(0,1,2) = -1.77773E+01
        AMN12(0,1,3) = -4.62721
        AMN12(0,1,4) =  5.97660
        AMN12(0,2,0) =  1.14290
        AMN12(0,2,1) = -2.04023E+01
        AMN12(0,2,2) = -2.38284E+01
        AMN12(0,2,3) =  7.11911
        AMN12(0,3,0) = -2.33573E+01
        AMN12(0,3,1) = -1.62263E+01
        AMN12(0,3,2) =  1.48273E+01
        AMN12(1,0,0) =  1.44928
        AMN12(1,0,1) =  1.02060E+01
        AMN12(1,0,2) =  4.40745
        AMN12(1,0,3) = -2.00819E+01
        AMN12(1,0,4) = -1.25356E+01
        AMN12(1,1,0) = -5.43503
        AMN12(1,1,1) =  1.65674E+01
        AMN12(1,1,2) =  2.00023E+01
        AMN12(1,1,3) = -2.51311
        AMN12(1,2,0) =  9.65844
        AMN12(1,2,1) = -3.82528
        AMN12(1,2,2) = -2.50000E+01
        AMN12(2,0,0) = -2.07008
        AMN12(2,0,1) = -9.95191
        AMN12(2,0,2) =  8.73121E-01
        AMN12(2,0,3) =  2.21089E+01
        AMN12(2,1,0) =  8.82263
        AMN12(2,1,1) =  2.49995E+01
        AMN12(2,1,2) =  2.50000E+01
        AMN12(3,0,0) =  6.85169E-01
        AMN12(3,0,1) = -7.40695E-02
        AMN12(3,0,2) = -6.78800E-01
C
C     ------------------------------------------------------------------
C
C     *** Factors for LDA correlation energy and potential ***
C
C     *** VWN local correlation ***
C
        F2ZERO = 4.0/(9.0*(2.0**EXP13 - 1.0))
        SVWN = 2.0*(2.0**EXP13 - 1.0)
C
C     *** Parameters of the Pade approximation ***
C     *** of the local VWN correlation energy  ***
C
C     PADE{A,B,C,X0}(1): Paramagnetic case.
C     PADE{A,B,C,X0}(2): Ferromagnetic case.
C     PADE{A,B,C,X0}(3): Zeta fit between 0 and 1.
C
        PADEA(1) =   0.0621814
        PADEA(2) =   0.0310907
        PADEA(3) =  -0.0337737
        PADEB(1) =   3.7274400
        PADEB(2) =   7.0604200
        PADEB(3) =   1.1310710
        PADEC(1) =  12.9352000
        PADEC(2) =  18.0578000
        PADEC(3) =  13.0045000
        PADEX0(1) = -0.1049800
        PADEX0(2) = -0.3250000
        PADEX0(3) = -0.0047584
C
C     RPA{A,B,C,X0}(1): Paramagnetic case.
C     RPA{A,B,C,X0}(2): Ferromagnetic case.
C
        RPAA(1) =   0.0621814
        RPAA(2) =   0.0310907
        RPAB(1) =  13.0720000
        RPAB(2) =  20.1231000
        RPAC(1) =  42.7198000
        RPAC(2) = 101.5780000
        RPAX0(1) = -0.4092860
        RPAX0(2) = -0.7432940
C
C     *** Perdew & Wang 92 local correlation ***
C
        SPW92 = 2.0**EXP43 - 2.0
C
C     *** Parameters of the G-function approximation of ***
C     *** the local Perdew & Wang 92 correlation energy ***
C
C     PW{ALP,A,B1,B2,B3,B4}(1): Paramagnetic case.
C     PW{ALP,A,B1,B2,B3,B4}(2): Ferromagnetic case.
C     PW{ALP,A,B1,B2,B3,B4}(3): Zeta fit between 0 and 1.
C
        PWALP(1) = 0.21370
        PWALP(2) = 0.20548
        PWALP(3) = 0.11125
        PWA(1) = 0.031091
        PWA(2) = 0.015545
        PWA(3) = 0.016887
        PWB1(1) = 7.5957
        PWB1(2) = 14.1189
        PWB1(3) = 10.3570
        PWB2(1) = 3.5876
        PWB2(2) = 6.1977
        PWB2(3) = 3.6231
        PWB3(1) = 1.6382
        PWB3(2) = 3.3662
        PWB3(3) = 0.88026
        PWB4(1) = 0.49294
        PWB4(2) = 0.62517
        PWB4(3) = 0.49671
C
C     *** Parameters for the local PZ81 correlation ***
C     *** The parameters correspond to the values   ***
C     *** Table XII from Ceperley and Alder         ***
C
        PZGAM(1) = -0.1423
        PZBE1(1) = 1.0529
        PZBE2(1) = 0.3334
        PZA(1) = 0.0311
        PZB(1) = -0.0480
        PZC(1) = 0.0020
        PZD(1) = -0.0116
        PZGAM(2) = -0.0843
        PZBE1(2) = 1.3981
        PZBE2(2) = 0.2611
        PZA(2) = 0.01555
        PZB(2) = -0.0269
        PZC(2) = 0.0007
        PZD(2) = -0.0048
C
C     ------------------------------------------------------------------
C
C     *** Factors for GGA correlation energy and potential ***
C
C     *** Perdew 86 correlation ***
C
        P86(1) = (3.0/(4.0*PI))**EXP13
        P86(2) = (9.0*PI)**(1.0/6.0)
        SP86 = SQRT(2.0)
C
C     *** Lee, Yang & Parr 88 correlation ***
C
        LYP(1) = 2.0**EXP113*0.3*(3.0*PI**2)**EXP23
        LYP(2) = 47.0/18.0
        LYP(3) = 7.0/18.0
        LYP(4) = 5.0/2.0
        LYP(5) = 1.0/18.0
C
C     *** Perdew & Wang 91 correlation ***
C
        PW91(1) = 400.0*(2.0/(3.0*PI**2))**EXP23
        PW91(2) = 0.09
        PW91(3) = 4.235E-3
        PW91(4) = 1.667E-3
        PW91(5) = -16.0*CVLSD*PW91(3)
        PW91(6) = PW91(2)/PW91(5)
        PW91(7) = PW91(2)/PW91(5)**2
C
C     *** Perdew, Burke & Ernzenhof 96 (and PBEsol) correlation ***
C
        CKS = 3.0/PI
        BETAPBESOL = 0.046
        GAMMAPBE = (1.0 - LOG(2.0))/(PI**2)
C
C     *** Keal & Tozer 03/04 correlation ***
C
        BETAKT2 = 0.576727
        BETAKT3 = 0.864409
C
C     *** Peverati, Zhao & Truhlar SO11 correlation ***
C
        BETASO11 = 0.066725
        CSO11 = CKF*PI
C
        ACSO11(0) =  5.0E-1
        ACSO11(1) = -4.62334
        ACSO11(2) =  8.00410
        ACSO11(3) = -1.30226E2
        ACSO11(4) =  3.82685E1
        ACSO11(5) =  6.95599E1
C
        BCSO11(0) =  5.0E-1
        BCSO11(1) =  3.62334
        BCSO11(2) =  9.36393
        BCSO11(3) =  3.45114E1
        BCSO11(4) = -1.85684E1
        BCSO11(5) = -1.65195E-1
C
C     *** Peverati & Truhlar N12 correlation ***
C
        OSGAMMAN12 = 0.003
        SSGAMMAN12 = 0.200
C
        BN12(0) =   1.0
        BN12(1) =   3.24511
        BN12(2) = -25.2893
        BN12(3) =  14.4407
        BN12(4) =  19.6870
C
        CN12(0) =   1.0
        CN12(1) =  -5.53170
        CN12(2) =  30.7958
        CN12(3) = -56.4196
        CN12(4) =  32.1250
C
C     *** Yu, Zhang, Verma, He & Truhlar GAM correlation ***
C
        BGAM(0) =   0.860548
        BGAM(1) =  -2.94135
        BGAM(2) =  15.4176
        BGAM(3) =  -5.99825
        BGAM(4) = -23.4119
C
        CGAM(0) =  0.231765
        CGAM(1) =  0.575592
        CGAM(2) = -3.43391
        CGAM(3) = -5.77281
        CGAM(4) =  9.52448
C
C     ------------------------------------------------------------------
C
C     *** Factors for meta-GGA correlation energy and potential ***
C
C     *** Voohris & Scuseria 98 correlation ***
C
        GABVS98 = 0.00304966
        GCCVS98 = 0.00515088
        PVS98   = 3.0/(4.0*PI)
C
        CVS98(1,1) =  7.035010E-01
        CVS98(2,1) =  7.694574E-03
        CVS98(3,1) =  5.152765E-02
        CVS98(4,1) =  3.394308E-05
        CVS98(5,1) = -1.269420E-03
        CVS98(6,1) =  1.296118E-03
        CVS98(1,2) =  3.957626E-01
        CVS98(2,2) = -5.614546E-01
        CVS98(3,2) =  1.403963E-02
        CVS98(4,2) =  9.831442E-04
        CVS98(5,2) = -3.577176E-03
        CVS98(6,2) =  0.000000E+00
        CVS98(1,3) = -6.746338E-01
        CVS98(2,3) = -1.534002E-01
        CVS98(3,3) = -9.021521E-02
        CVS98(4,3) = -1.292037E-03
        CVS98(5,3) = -2.352983E-04
        CVS98(6,3) =  0.000000E+00
        CVS98(1,4) = -2.741539E+00
        CVS98(2,4) = -6.720113E-01
        CVS98(3,4) = -7.932688E-02
        CVS98(4,4) =  1.918681E-03
        CVS98(5,4) = -2.032902E-03
        CVS98(6,4) =  0.000000E+00
        CVS98(1,5) =  1.166404E-01
        CVS98(2,5) = -9.120847E-02
        CVS98(3,5) = -6.726189E-02
        CVS98(4,5) =  6.720580E-05
        CVS98(5,5) =  8.448011E-04
        CVS98(6,5) =  0.000000E+00
C
        VS98SS(1,1) =  3.270912E-01
        VS98SS(2,1) = -3.228915E-02
        VS98SS(3,1) = -2.942406E-02
        VS98SS(4,1) =  2.134222E-03
        VS98SS(5,1) = -5.451559E-03
        VS98SS(6,1) =  1.577575E-02
        VS98SS(1,2) =  4.650534E-01
        VS98SS(2,2) =  1.617589E-01
        VS98SS(3,2) =  1.833657E-01
        VS98SS(4,2) =  4.692100E-04
        VS98SS(5,2) = -4.990573E-03
        VS98SS(6,2) =  0.000000E+00
        VS98SS(1,3) =  8.976746E-01
        VS98SS(2,3) = -2.345830E-01
        VS98SS(3,3) =  2.368173E-01
        VS98SS(4,3) = -9.913890E-04
        VS98SS(5,3) = -1.146165E-02
        VS98SS(6,3) =  0.000000E+00
        VS98SS(1,4) =  4.905945E-01
        VS98SS(2,4) = -1.437348E-01
        VS98SS(3,4) =  2.357824E-01
        VS98SS(4,4) =  1.871015E-03
        VS98SS(5,4) = -3.788963E-03
        VS98SS(6,4) =  0.000000E+00
        VS98SS(1,5) =  6.902145E-01
        VS98SS(2,5) =  9.847204E-02
        VS98SS(3,5) =  2.214797E-01
        VS98SS(4,5) = -1.968264E-03
        VS98SS(5,5) = -6.775479E-03
        VS98SS(6,5) =  0.000000E+00
C
C     *** Perdew, Kurth, Zupan & Blaha 99 correlation ***
C
        CPKZB = 0.53
C
C     *** Tao, Perdew, Staroverov & Scuseria 03 correlation ***
C
        DTPSS = 2.8
        TPSS(1) = 0.53
        TPSS(2) = 0.87
        TPSS(3) = 0.5
        TPSS(4) = 2.26
C
C     *** Zhao & Truhlar M06 correlation ***
C
        CPPM06 = 0.0031
        CSSM06 = 0.06
C
        CM06(1,1) =  6.042374E-01
        CM06(2,1) =  1.776783E+02
        CM06(3,1) = -2.513252E+02
        CM06(4,1) =  7.635173E+01
        CM06(5,1) = -1.255699E+01
        CM06(1,2) =  1.674634E+00
        CM06(2,2) =  5.732017E+01
        CM06(3,2) =  5.955416E+01
        CM06(4,2) = -2.311007E+02
        CM06(5,2) =  1.255199E+02
        CM06(1,3) =  3.741539E+00
        CM06(2,3) =  2.187098E+02
        CM06(3,3) = -4.531252E+02
        CM06(4,3) =  2.936479E+02
        CM06(5,3) = -6.287470E+01
        CM06(1,4) =  8.833596E-01
        CM06(2,4) =  3.357972E+01
        CM06(3,4) = -7.043548E+01
        CM06(4,4) =  4.978271E+01
        CM06(5,4) = -1.852891E+01
C
        M06SS(1,1) =  5.349466E-01
        M06SS(2,1) =  5.396620E-01
        M06SS(3,1) = -3.161217E+01
        M06SS(4,1) =  5.149592E+01
        M06SS(5,1) = -2.919613E+01
        M06SS(1,2) =  1.023254E-01
        M06SS(2,2) = -2.453783E+00
        M06SS(3,2) =  2.913180E+01
        M06SS(4,2) = -3.494358E+01
        M06SS(5,2) =  2.315955E+01
        M06SS(1,3) =  5.094055E-01
        M06SS(2,3) = -1.491085E+00
        M06SS(3,3) =  1.723922E+01
        M06SS(4,3) = -3.859018E+01
        M06SS(5,3) =  2.845044E+01
        M06SS(1,4) =  3.097855E-01
        M06SS(2,4) = -5.528642E+00
        M06SS(3,4) =  1.347420E+01
        M06SS(4,4) = -3.213623E+01
        M06SS(5,4) =  2.846742E+01
C
C     *** Peverati & Truhlar M11 correlation ***
C
        CM11(0,1) =   1.0000000
        CM11(1,1) =   0.0000000
        CM11(2,1) =   2.7508800
        CM11(3,1) = -15.6228700
        CM11(4,1) =   9.3633810
        CM11(5,1) =  21.4102400
        CM11(6,1) = -14.2497500
        CM11(7,1) = -11.3471200
        CM11(8,1) =  10.2236500
        CM11(0,2) =   1.0000000
        CM11(1,2) =  -9.0820600
        CM11(2,2) =   6.1346820
        CM11(3,2) = -13.3321600
        CM11(4,2) = -14.6411500
        CM11(5,2) =  17.1314300
        CM11(6,2) =   2.4807380
        CM11(7,2) = -10.0703600
        CM11(8,2) =  -0.1117521
C
C     *** Peverati & Truhlar MN12 correlation ***
C
        CMN12(0,1) =  8.84461E-01
        CMN12(1,1) = -2.20228E-01
        CMN12(2,1) =  5.70147
        CMN12(3,1) = -2.56238
        CMN12(4,1) = -9.64683E-01
        CMN12(5,1) =  1.98218E-01
        CMN12(6,1) =  1.01998E+01
        CMN12(7,1) =  9.78935E-01
        CMN12(8,1) = -1.51272
        CMN12(0,2) =  5.32395E-01
        CMN12(1,2) = -5.83191
        CMN12(2,2) =  3.88239
        CMN12(3,2) =  5.87849
        CMN12(4,2) =  1.49323E+01
        CMN12(5,2) = -1.37464E+01
        CMN12(6,2) = -8.49233
        CMN12(7,2) = -2.48655
        CMN12(8,2) = -1.82235E+01
C
C     ------------------------------------------------------------------
C
C     *** Recognize the performed initialization ***
C
        INITXC = .FALSE.
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Initialize summation variables ***
C
      ECSUM  = 0.0
      EXSUM  = 0.0
      RHOSUM = 0.0
      VXCDCA = 0.0
      VXCDCB = 0.0
C
C     -----------------------------------------------------------------
C
CCC ADLL
C     *** Load XC numerical grid parameters ***
C
      CALL GRDDEF(XCGSN,'LOAD')
      CALL GRDTHRS
CCC ADLL
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE XCINI ***
C
      END
