C     MP_DEMON COMMON (for the parallel version)
C
C     Purpose: Common and parameter definition for MPI deMon version.
C
C     History: - Creation (15.07.03, GG)
C                         (09.02.15, AMK)
C                         (21.01.20, GG)
C
C     ******************************************************************
C
C     List of parameters:
C
C     MAXCHUNK: Maximum chunk size in MegaByte.
C     MB2INT4 : MegaByte -> Word conversion factor for INTEGER4.
C     MB2INT8 : MegaByte -> Word conversion factor for INTEGER8.
C     MB2REAL8: MegaByte -> Word conversion factor for REAL8.
C
C     MP_AIMS: MPI index for AIM analysis branch.
C     MP_CHAR: Coding of character variables.
C     MP_CORE: MPI index for core integral branch.
C     MP_DCOR: MPI index for core integral derivative branch.
C     MP_DERI: MPI index for ERI derivative branch.
C     MP_DFAR: MPI index for far-field ERI derivative branch.
C     MP_DXCE: MPI index for XC derivative branch.
C     MP_EMBF: MPI index for far-field emebedding NAI.
C     MP_EMBN: MPI index for near-field emebedding NAI.
C     MP_ERIS: MPI index for three-center ERI branch.
C     MP_EXXP: MPI index for exact exchange (EXX) branch.
C     MP_FARJ: MPI index for far-field J vector branch.
C     MP_FARS: MPI index for far-field ERI branch.
C     MP_INT : Coding of integer variables.
C     MP_JVEC: MPI index for Coulomb (J) vector branch.
C     MP_LOG : Coding of logical variables.
C     MP_MAXI: Maximum number of MPI indices.
C     MP_REAL: Coding of real variables.
C     MP_XCVE: MPI index for exchange-correlation (XC) branch.
C
C     MPGALL : Group including all processors.
C     MPGPT  : Group for parallel tempering MD.
C     MPGPTM : Group of masters only from parallel tempering MD.
C
C     List of variables:
C
C     D1CAL     : Array index for the calculation of derivatives.
C     ERICAL    : Array index for the calculation of Coulomb integrals.
C     JCAL      : Array index for the calculation of Coulomb (J) vector.
C     LLMEM     : Lower limit of CPU for ERI/grid RAM storage.
C     LLROW     : Lower row limit for distributed matrix.
C     LLSET     : Lower auxiliary function set limit.
C     MASTER    : Number of the master process.
C     ME        : Number of local process.
C     MGRPMEM   : True, if process is on the same node as the master.
C     MGRPNUM   : Number of processes on the same node as the master.
C     MPCOMMACT : Active communicator.
C     MPCURGRP  : The currect active group.
C     MPFRQMEM  : Inverse G matrix storage in parallel frequency run.
C     MPFRQTYP  : Frequency parallelization type.
C     MPKRNTYP  : Kernel parallelization type.
C     MPGCOL    : Save for color for each group.
C     MPGCOMM   : Save for communicator for each group.
C     MPGMASTER : Save for MASTER for each group.
C     MPGME     : Save for ME for each group.
C     MPGNCPU   : Save for NCPU for each group.
C     MPINGRP   : True, if local process is member of the current group.
C     MPINI     : True, if MPI load balancing is initialized.
C     MPINIGRP  : Save for MPINI for each group.
C     MPMASTER  : True, if local process is the master process.
C     MPNOCOLOR : Color to use to not include a processor in a group.
C     MPP       : True for the parallel deMon version.
C     MP_PART   : Partitioning factor for MPI load balancing.
C     MP_PARTGRP: Save for MP_PART for each group.
C     MPSLAVE   : True, if local process is a slave process.
C     MP_SFC    : Scaling factor depending on the number of processes.
C     NCPU      : Total number of CPUs in parallel MPI run.
C     NMEM      : Number of CPUs for ERI/grid RAM storage.
C     ULMEM     : Upper limit of CPU for ERI/grid RAM storage.
C     ULROW     : Upper row limit for distributed matrix.
C     ULSET     : Upper auxiliary function set limit.
C     USEDMAT   : True, if an already distributed matrix is used.
C     USEMPF    : True, if the parallel part of a routine is used.
C
C     ------------------------------------------------------------------
C
C     *** Maximum chunk size in MegaByte ***
C
      INTEGER*8 MAXCHUNK
C
      PARAMETER (MAXCHUNK = 128)
C
C     Conversion factors MegaByte -> Word ***
C
      INTEGER*8 MB2INT4,MB2INT8,MB2REAL8
C
      PARAMETER (MB2INT4  = 262144,
     $           MB2INT8  = 131072,
     $           MB2REAL8 = 131072)
C
C     *** MPI variable type parameters ***
C
      CHARACTER*4 MP_CHAR,MP_INT,MP_LOG,MP_REAL
C
      PARAMETER (MP_CHAR = 'CHAR',
     $           MP_INT  = 'INTE',
     $           MP_LOG  = 'LOGI',
     $           MP_REAL = 'REAL')
C
C     *** MPI field index parameters ***
C
      INTEGER*8 MP_MAXI,MP_AIMS,MP_CORE,MP_DCOR,MP_DERI,MP_DFAR,MP_DXCE,
     $          MP_EMBF,MP_EMBN,MP_ERIS,MP_EXXP,MP_FARJ,MP_FARS,MP_JVEC,
     $          MP_XCVE
C
      PARAMETER (MP_MAXI = 14,
     $           MP_EMBN = 14,
     $           MP_EMBF = 13,
     $           MP_DCOR = 12,
     $           MP_CORE = 11,
     $           MP_AIMS = 10,
     $           MP_DFAR = 9,
     $           MP_DERI = 8,
     $           MP_DXCE = 7,
     $           MP_FARS = 6,
     $           MP_FARJ = 5,
     $           MP_EXXP = 4,
     $           MP_ERIS = 3,
     $           MP_JVEC = 2,
     $           MP_XCVE = 1)
C
C     *** MPI group parameters ***
C
      INTEGER*8 MAXDISTYP,MAXMPG,MPGALL,MPGPT,MPGPTM
C
      PARAMETER (MAXDISTYP = 2,
     $           MAXMPG = 3,
     $           MPGALL = 1,
     $           MPGPT  = 2,
     $           MPGPTM = 3)
C
C     ------------------------------------------------------------------
C
C     *** Common block for INTEGER ***
C
      INTEGER*4 MPCOMMACT,MBSUB,NBSUB,ICTXT,ICTXT_0,
     $          GPUPNODE,COREPGPU
      INTEGER*8 LLMEM,LLROW,LLSET,MASTER,ME,MGRPNUM,MPCURCOL,MPCURGRP,
     $          MPNOCOLOR,NCPU,NMEM,ULMEM,ULROW,ULSET
C
      INTEGER*8 MPGCOL(MAXMPG),MPGCOMM(MAXMPG),MPGMASTER(MAXMPG),
     $          MPGME(MAXMPG),MPGNCPU(MAXMPG),
     $          SLPCONTXT(MAXDISTYP,MAXMPG)
C
      INTEGER*4 MAGMA_DEVICE_CODE
      INTEGER*8 MAGMA_QUEUE
      INTEGER*4 MPI_COMM_GPU,GPUGRP,NGPU,ISTART_GPU,IEND_GPU,SLICE_GPU
C
      COMMON /IPARALLEL/ LLMEM,LLROW,LLSET,MASTER,ME,MPCURCOL,MPCURGRP,
     $                   MPGCOL,MPGCOMM,MPGMASTER,MPGME,MPGNCPU,MGRPNUM,
     $                   MPNOCOLOR,NCPU,NMEM,SLPCONTXT,ULMEM,ULROW,
     $                   ULSET,
     $                   MAGMA_QUEUE,MAGMA_DEVICE_CODE,
     $                   GPUPNODE,COREPGPU,MPI_COMM_GPU,GPUGRP,
     $                   NGPU,ISTART_GPU,IEND_GPU,SLICE_GPU,
     $                   MPCOMMACT,MBSUB,NBSUB,ICTXT,ICTXT_0
C
C     *** Common block for REAL ***
C
      REAL MP_SFC
C
      REAL MP_PART(MP_MAXI),MP_PARTGRP(MP_MAXI,MAXMPG)
C
      COMMON /RPARALLEL/ MP_PART,MP_PARTGRP,MP_SFC
C
C     *** Common block for LOGICAL ***
C
      LOGICAL*8 MGRPMEM,MPP,MPMASTER,MPSLAVE,USEDMAT,USEMPF
C
      LOGICAL*8 MPINGRP(MAXMPG),MPINI(MP_MAXI),MPINIGRP(MP_MAXI,MAXMPG)
C
      COMMON /LPARALLEL/ MPINGRP,MPINI,MPINIGRP,MGRPMEM,MPMASTER,MPP,
     $                   MPSLAVE,USEDMAT,USEMPF
C
C     *** Common block for CHARACTER ***
C
      CHARACTER*3 MPFRQMEM,MPFRQTYP,MPKRNTYP
C
      COMMON /CPARALLEL/ MPFRQMEM,MPFRQTYP,MPKRNTYP
C
C     ------------------------------------------------------------------
