C     PROPERTY COMMON
C
C     Purpose: Common definition for properties.
C
C     History: - Creation (05.08.96, ML)
C                         (24.01.09, AMK)
C
C     ******************************************************************
C
C     List of property variables:
C
C     ABASIN : Assignment of atomic basin.
C     BARNS  : Unit factor for nuclear quadrupole moments.
C     CPSCALE: Function scaling parameter for CP search.
C     CPSMAX : Cutoff distance for ending CP search.
C     CPTOL  : CP search tolerance tightening.
C     CURVE  : Curve length of current bondpath.
C     GBASIN : G kinetic energy of atomic basin.
C     HPACYC : Number of iterative Hirshfeld population analysis cycles.
C     KBASIN : K kinetic energy of atomic basin.
C     LBASIN : Charge density Laplacian of atomic basin.
C     LLPIGRP: Lower limit of a localized pi group.
C     MBASIN : Moments of density basin.
C     MODALP : If true, alpha MO occupation is modified.
C     MODBET : If true, beta MO occupation is modified.
C     NATT   : Number of basin attractors.
C     NBASIN : Number of electrons in density basin.
C     NBONDS : Number of generated bondpaths.
C     NCPS   : Number of characterized critical points.
C     NMRSIG : List of atoms for NMR signal calculation.
C     NQRSIG : List of atoms for NQR signal calculation.
C     NOCPS  : Number of critical points with given rank and signature.
C     NPIGRP : Number and size of defined pi groups.
C     NPOINTS: Current number of bondpath support points.
C     PIGRP  : Atom list of localized pi groups.
C     POINTS : Bondpath support points.
C     QATOM  : Atomic net charge.
C     QMIX   : Mixing value for iterative Hirshfeld analysis.
C     QSPIN  : Atomic spin charge.
C     SIGTEN : NMR shielding (sigma) tensors.
C     SRCTEN : Spin-rotation coupling tensors.
C     ULPIGRP: Upper limit of a localized pi group.
C
C     ------------------------------------------------------------------
C
C     *** Common block for LOGICAL property variables ***
C
      LOGICAL MODALP,MODBET,POPMO
C
      LOGICAL NMRSIG(0:MAXATOM),NQRSIG(0:MAXATOM)
C
      COMMON /LPROPERTY/ MODALP,MODBET,NMRSIG,NQRSIG,POPMO
C
C     *** Common block for INTEGER property variables ***
C
      INTEGER HPACYC,NATT,NBONDS,NCPS,NPIGRP,NPOINTS
C
      INTEGER ABASIN(MAXATOM),LLPIGRP(MAXATOM),NOFCPS(0:3,-3:3),
     $        PIGRP(MAXATOM),ULPIGRP(MAXATOM)
C
      COMMON /IPROPERTY/ ABASIN,HPACYC,LLPIGRP,NATT,NBONDS,NCPS,
     $                   NOFCPS,NPIGRP,NPOINTS,PIGRP,ULPIGRP
C
C     *** Common block for REAL property variables ***
C
      REAL BARNS,CPSCALE,CPSMAX,CPTOL,CURVE,QMIX
C
      REAL GBASIN(MAXATOM),KBASIN(MAXATOM),LBASIN(MAXATOM),
     $     MBASIN(10,MAXATOM),NBASINA(MAXATOM),NBASINB(MAXATOM),
     $     QATOM(MAXATOM),QSPIN(MAXATOM)
C
      REAL POINTS(1:3,-MAXGPB:MAXGPB),SIGTEN(3,3,MAXATOM),
     $     SRCTEN(3,3,MAXATOM)
C
      COMMON /RPROPERTY/ BARNS,CPSCALE,CPSMAX,CPTOL,CURVE,GBASIN,
     $                   KBASIN,LBASIN,MBASIN,NBASINA,NBASINB,POINTS,
     $                   QATOM,QMIX,QSPIN,SIGTEN,SRCTEN
C
C     ------------------------------------------------------------------
