      SUBROUTINE BIN2ASC
C
C     Purpose: Convert binary plot data to ascii.
C
C     History: - Creation (06.03.96, ML)
C
C     ******************************************************************
C
C     List of local variables:
C
C     FUN  : Plot function value.
C     X/Y/Z: Plot function coordinates.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'plot.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
C
      LOGICAL SUCCESS
      CHARACTER*80 PRTSTR,STRCOMP
      INTEGER ALLOCATION,FILE,I,J,K,IFUN,IVAL,IREC,RECLAT,TAPE
      REAL X,Y,Z,NORM
C
      CHARACTER*37 FORM2
      CHARACTER*36 FORM1
      REAL,ALLOCATABLE :: RESALL(:,:)
      REAL TIME,FUNVAL(1),VECTOR(3)
C
C     ------------------------------------------------------------------
C
C     *** No conversion for isosurface generation ***
C
      IF (PLOTTYP.EQ.'ISOSURFACE') RETURN
C
C     *** Select output file ***
C
CCC ADLL
      IF (STATUS.EQ.'RTDFT') THEN
        FILE = RTP
        TIME = 1.0E3*RTT/FSEC
      ELSE IF (PLOT.EQ.'ASCII') THEN
        FILE = PIE
      ELSE IF (PLOT.EQ.'TABLE') THEN
        FILE = OUT
      END IF
CCC ADLL
C
C     *** Structured lattice ***
C
      IF (LATTYP.EQ.'STRUCTURED') THEN
C
        WRITE (PRTSTR,5000) UNIT(1)
        CALL EXTMSG(0,STRCOMP(PRTSTR),1,0,FILE)
        DO IFUN=1,NPLOTFUN
          WRITE (FILE,5010) PLOTNAME(IFUN)
          IVAL = 0
          DO K=1,NLATTICE(3)
            DO J=1,NLATTICE(2)
              DO I=1,NLATTICE(1)
                IVAL = IVAL + 1
                IREC = IVAL + (IFUN - 1)*NLATTICE(0)
                X = ANGBOR(1)*(LATTICE(1,0) + DSTEP(1,1)*(I-1) +
     $                                        DSTEP(2,1)*(J-1) +
     $                                        DSTEP(3,1)*(K-1))
                Y = ANGBOR(1)*(LATTICE(2,0) + DSTEP(1,2)*(I-1) +
     $                                        DSTEP(2,2)*(J-1) +
     $                                        DSTEP(3,2)*(K-1))
                Z = ANGBOR(1)*(LATTICE(3,0) + DSTEP(1,3)*(I-1) +
     $                                        DSTEP(2,3)*(J-1) +
     $                                        DSTEP(3,3)*(K-1))
                CALL BINARY(FUNVAL,1,1,IREC,IOPLOT,PLOTNAME(0),'READ')
                WRITE (FILE,5020) X,Y,Z,FUNVAL(1)
              END DO
            END DO
          END DO
        END DO
C
C     *** Unstructured lattice ***
C
      ELSE IF (LATTYP.EQ.'POINTS') THEN
C
CCC ADLL
        IF (STATUS.EQ.'RTDFT') THEN
          ALLOCATE (RESALL(NPLOTFUN+1,NLATTICE(0)),STAT=ALLOCATION)
          IF (ALLOCATION.GT.0) THEN
            CALL ERRMSG('BIN2ASC','ALLOCATION FAILED',1)
          END IF
          FORM1 = '(T2,F12.3,1X,A13,I5,1X,X(E13.6,1X))'
          FORM2 = '(T2,F12.3,1X,A13,I5,1X,XX(E13.6,1X))'
        END IF
CCC ADLL
C
        WRITE (PRTSTR,5000) UNIT(1)
CCC ADLL
        IF (STATUS.NE.'RTDFT') CALL EXTMSG(0,STRCOMP(PRTSTR),1,0,FILE)
CCC ADLL
        IREC = 0
        DO IFUN=1,NPLOTFUN
CCC ADLL
          IF (STATUS.NE.'RTDFT') WRITE (FILE,5010) PLOTNAME(IFUN)
CCC ADLL
          IF (LATINP.EQ.'INTERNAL') THEN
            CALL FINDKEY('POINTS',SUCCESS)
            IF (.NOT.SUCCESS) GO TO 99980
            TAPE = INP
          ELSE IF (LATINP.EQ.'EXTERNAL ASCII') THEN
            REWIND (LAT)
            READ (LAT,*,ERR=99980,END=99980) NLATTICE(0)
            TAPE = LAT
          ELSE IF (LATINP.EQ.'EXTERNAL BINARY') THEN
            RECLAT = 1
          END IF
          DO IVAL=1,NLATTICE(0)
            IREC = IVAL + (IFUN-1)*NLATTICE(0)
            IF (LATINP.EQ.'EXTERNAL BINARY') THEN
              RECLAT = RECLAT + 1
              CALL BINARY(VECTOR,3,3,RECLAT,LAT,'LAT.bin','READ')
            ELSE
              READ (TAPE,*,ERR=99980,END=99990) (VECTOR(I),I=1,3)
            END IF
            CALL MODPOINT(VECTOR)
            CALL BINARY(FUNVAL,1,1,IREC,IOPLOT,PLOTNAME(0),'READ')
            X = VECTOR(1)
            Y = VECTOR(2)
            Z = VECTOR(3)
CCC ADLL
            IF (STATUS.EQ.'RTDFT') THEN
              RESALL(IFUN,IVAL) = FUNVAL(1)
            ELSE
              WRITE (FILE,5030) X,Y,Z,FUNVAL(1)
            END IF
CCC ADLL
          END DO
        END DO
CCC ADLL
C     *** Print values in deMon.rtp file ***
C
        IF (STATUS.EQ.'RTDFT') THEN
          IF (PLOTFNC.EQ.'EFI') NPLOTFUN = NPLOTFUN + 1
          DO IVAL=1,NLATTICE(0)
            IF (PLOTFNC.EQ.'EFI') THEN
              NORM = RESALL(2,IVAL)**2 + RESALL(3,IVAL)**2 +
     $               RESALL(4,IVAL)**2 
              RESALL(NPLOTFUN,IVAL) = SQRT(NORM)
            END IF
            IF (NPLOTFUN.LE.9) THEN
              WRITE(FORM1(24:24),'(I1)') NPLOTFUN
              WRITE (FILE,FORM1) TIME,'POINT NUMBER ',IVAL,
     $                              (RESALL(IFUN,IVAL),IFUN=1,NPLOTFUN)
            ELSE
              FORM2 = '(T2,F12.3,1X,A12,1X,I5,XX(ES13.6,1X))'
              WRITE (FILE,FORM2) TIME,'POINT NUMBER ',IVAL,
     $                              (RESALL(IFUN,IVAL),IFUN=1,NPLOTFUN)
            END IF
          END DO
          IF (PLOTFNC.EQ.'EFI') NPLOTFUN = NPLOTFUN - 1
          DEALLOCATE (RESALL)
        END IF
C
      CALL DEFLUSH(FILE)
C
CCC ADLL
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Format statements ***
C
 5000 FORMAT ('PLOT COORDINATES IN ',A)
 5010 FORMAT (/,9X,'X',15X,'Y',15X,'Z',12X,A10)
 5020 FORMAT (4(3X,E13.6))
 5030 FORMAT (4(3X,E13.6))
C
C     ------------------------------------------------------------------
C
C     *** Handling of errors ***
C
99980 CONTINUE
      CALL ERRMSG('BIN2ASC','I/O ERROR READING UNSTRUCTURED LATTICE',1)
C
99990 CONTINUE
      CALL ERRMSG('BIN2ASC','EOF WHILE READING UNSTRUCTURED LATTICE',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE BIN2ASCII ***
C
      END
