      SUBROUTINE BIN2CUBE
C
C     Purpose: Convert binary plot data to cube format. 
C
C     History: - Creation (16.09.17, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     FUN  : Plot function value.
C     X/Y/Z: Plot function coordinates.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'embed.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'plot.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'symmetry.h'
C
      LOGICAL SUCCESS
      CHARACTER*240 PRTSTR,STRCOMP,STRING1,STRING2 
      CHARACTER*240 STRCOMPL,STRINGL             
      CHARACTER*10 FORMAT1                       
      CHARACTER*11 FORMAT2                       
      CHARACTER*16 FORMAT3                       
      CHARACTER*17 FORMAT4                       
      INTEGER FILE,I,J,K,IFUN,IVAL,IREC,RECLAT,TAPE,IATOM,ICOORD,ICOUNT
      INTEGER IRAD
      INTEGER FILE2,FILE3
      INTEGER ALLOCATION
      REAL VEL,X,Y,Z
C
      REAL CE(3),FUNVAL(1),VECTOR(3)
      REAL, ALLOCATABLE:: BUFFER(:,:,:,:)
      CHARACTER*1 PSIFLG
C
C     ------------------------------------------------------------------
C
C     *** Define file indices ***
C
      FILE = PIE 
      FILE2 = PIE + 500
      FILE3 = PIE + 501
C
C     *** Allocate dynamic fields ***
C
      ALLOCATE (BUFFER(NLATTICE(1),NLATTICE(2),NLATTICE(3),
     $            NPLOTFUN),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0)
     $    CALL ERRMSG('BIN2ASC','ALLOCATION FAILED',1)
C
      BUFFER(:,:,:,:) = 0.0
C
      WRITE (FILE,'(a)') 'CUBE FILE GENERATED BY DEMON'
C
C     *** Write time value if RTTDDFT calculation is requested  ***
C
      IF (STATUS.EQ.'RTDFT') THEN
        WRITE (FILE2,'(a)') 'CUBE FILE GENERATED BY DEMON'
        WRITE (FILE,4999) NPLOTFUN,PLOTFNC,1.E3*RTT/FSEC
        IF (PLOTFNC.EQ.'ELF') WRITE (FILE2,4999)
     $    NPLOTFUN,'RHO',1.E3*RTT/FSEC
      ELSE
        WRITE (FILE,5001) NPLOTFUN,PLOTFNC
      END IF
C
      IF (ADDRAD) NATOM = NATOM + NRADP
C
C     *** Header of plot file (number of atoms and lattice interval) ***
C
      IF (NPLOTFUN.GT.1.OR.(PLOTFNC.EQ.'PSI')) THEN
        IF ((STATUS.EQ.'RTDFT').AND.(PLOTFNC.EQ.'ELF')) THEN
          WRITE (FILE,5005) NATOM,(LATTICE(I,0),I=1,3)
          WRITE (FILE2,5005) NATOM,(LATTICE(I,0),I=1,3)
        ELSE
          WRITE (FILE,5005) -NATOM,(LATTICE(I,0),I=1,3)
        END IF
      ELSE 
        WRITE (FILE,5005) NATOM,(LATTICE(I,0),I=1,3)
      END IF
C
      IF (ADDRAD) NATOM = NATOM - NRADP
C
C     *** Write step size along lattice vectors ***
C
      DO I=1,3
        WRITE (FILE,5006) NLATTICE(I),(DSTEP(I,J),J=1,3) 
        IF ((STATUS.EQ.'RTDFT').AND.(PLOTFNC.EQ.'ELF')) THEN
          WRITE (FILE2,5006) NLATTICE(I),(DSTEP(I,J),J=1,3)
        END IF
      END DO
C
C     *** Write QM atoms positions ***
C
      WRITE (FILE,5007) (ATOMNO(IATOM),
     $  (COORD(ICOORD,IATOM,CCSET),ICOORD=1,3),IATOM=1,NATOM)
      IF ((STATUS.EQ.'RTDFT').AND.(PLOTFNC.EQ.'ELF')) THEN
        WRITE (FILE2,5007) (ATOMNO(IATOM),
     $    (COORD(ICOORD,IATOM,CCSET),ICOORD=1,3),IATOM=1,NATOM)
      END IF
      I = 2
C
C     *** Write radiation particle position ***
C
      IF (ADDRAD) THEN 
        DO IRAD=1,NRADP
          VEL = 2*KINRAD/(MRAD*AMU)
          CE(:) = X0IORAD(:,IRAD) + RTT*VIORAD(:,IRAD)*SQRT(VEL)
          CALL TRACOO(CE,CE,1,INPTOSTD,3,SHIFT,'SHIFTROTATE')
          WRITE (FILE,5007) I, (CE(ICOORD),ICOORD=1,3)
        END DO
        IF ((STATUS.EQ.'RTDFT').AND.(PLOTFNC.EQ.'ELF')) THEN
          WRITE (FILE2,5007) I, (CE(ICOORD),ICOORD=1,3)
        END IF
      END IF

 4999 FORMAT ('MOLECULAR FIELD : ',I2,2X,A10,2X,' TIME(AS)=',2X,F12.4)
 5001 FORMAT ('MOLECULAR FIELD : ',I2,2X,A10)
 5005 FORMAT (I4,2X,3(F12.6,X))
 5006 FORMAT ((I4,2X,3(F12.6,X)))
 5007 FORMAT ((I4,2x,(3X,'0.000000',X,3(F12.6,3X))))
C
      IF (NPLOTFUN.GT.1.AND.(PLOTFNC.EQ.'PSI')) THEN
        STRING1 = ''
        STRING2 = ''
        STRINGL = ''
        FORMAT1 = '(Y(I4,1X))'
        FORMAT2 = '(YY(I4,1X))'
        FORMAT3 = '(Y(I4,1X,I4,1X))'
        FORMAT4 = '(YY(I4,1X,I4,1X))'
        IF (.NOT.RTDFT) THEN
            IF (PLOTMOSA(0).LE.8) THEN
              WRITE(FORMAT1(2:2),'(I1)') PLOTMOSA(0) + 1
              WRITE(STRING1,FORMAT1) (PLOTMOSA(I),I=1,PLOTMOSA(0))
            ELSE IF (PLOTMOSA(0).LE.98) THEN
              WRITE(FORMAT2(2:3),'(I2)') PLOTMOSA(0) + 1
              WRITE(STRING1,FORMAT2) (PLOTMOSA(I),I=1,PLOTMOSA(0))
            END IF
            IF (OKS) THEN
              IF (PLOTMOSB(0).LE.8) THEN
                WRITE(FORMAT1(2:2),'(I1)') PLOTMOSB(0) + 1
                WRITE(STRING2,FORMAT1) (PLOTMOSB(I),I=1,PLOTMOSB(0))
              ELSE IF (PLOTMOSB(0).LE.98) THEN
                WRITE(FORMAT2(2:3),'(I2)') PLOTMOSB(0) + 1
                WRITE(STRING2,FORMAT2) (PLOTMOSB(I),I=1,PLOTMOSB(0))
              END IF
            END IF
        ELSE
          IF (2*PLOTMOSA(0).LE.7) THEN
            WRITE(FORMAT3(2:2),'(I1)') 2*PLOTMOSA(0) + 2
            WRITE(STRING1,FORMAT3) (PLOTMOSA(I),PLOTMOSA(I),
     $                              I=1,PLOTMOSA(0))
          ELSE IF (2*PLOTMOSA(0).LE.97) THEN
            WRITE(FORMAT4(2:3),'(I2)') PLOTMOSA(0) + 2
            WRITE(STRING1,FORMAT4) (PLOTMOSA(I),PLOTMOSA(I),
     $                              I=1,PLOTMOSA(0))
          END IF
          IF (OKS) THEN
            IF (2*PLOTMOSB(0).LE.7) THEN
              WRITE(FORMAT3(2:2),'(I1)') 2*PLOTMOSB(0) + 2
              WRITE(STRING2,FORMAT3) (PLOTMOSB(I),PLOTMOSB(I),
     $                                I=1,PLOTMOSB(0))
            ELSE IF (2*PLOTMOSB(0).LE.97) THEN
              WRITE(FORMAT4(2:3),'(I2)') 2*PLOTMOSB(0) + 2
              WRITE(STRING2,FORMAT4) (PLOTMOSB(I),PLOTMOSB(I),
     $                                 I=1,PLOTMOSB(0))
            END IF
          END IF
        END IF
        WRITE(STRINGL,'(I3,X,A,X,A)') NPLOTFUN,TRIM(ADJUSTL(STRING1)),
     $                           TRIM(ADJUSTL(STRING2))
        WRITE(FILE,*) TRIM(ADJUSTL(STRCOMP(STRINGL)))
      END IF
C
C     *** Read field values from binary file ***
C
      DO IFUN=1,NPLOTFUN
        IVAL = 0
        DO K=1,NLATTICE(3)
          DO J=1,NLATTICE(2)
            DO I=1,NLATTICE(1)
              IVAL = IVAL + 1
              IREC = IVAL + (IFUN - 1)*NLATTICE(0)
              CALL BINARY(FUNVAL,1,1,IREC,IOPLOT,PLOTNAME(0),'READ')
              BUFFER(I,J,K,IFUN) = FUNVAL(1)
            END DO
          END DO
        END DO
      END DO
C
C     *** Write field values to cube file ***
C
      IF (RTDFT.AND.(PLOTFNC.EQ.'ELF')) THEN
         DO I=1,NLATTICE(1)
           DO J=1,NLATTICE(2)
             WRITE (FILE,5009)((BUFFER(I,J,K,1)),K=1,NLATTICE(3)) 
             WRITE (FILE2,5009)((BUFFER(I,J,K,2)),K=1,NLATTICE(3))
C
             X = LATTICE(1,0) + DSTEP(1,1)*(I-1)
     $                        + DSTEP(2,1)*(J-1)
     $                        + DSTEP(3,1)*(K-1)
             Y = LATTICE(2,0) + DSTEP(1,2)*(I-1)
     $                        + DSTEP(2,2)*(J-1)
     $                        + DSTEP(3,2)*(K-1)

             DO K=1,NLATTICE(3)
               Z = LATTICE(3,0) + DSTEP(1,3)*(I-1)
     $                          + DSTEP(2,3)*(J-1)
     $                          + DSTEP(3,3)*(K-1)

                WRITE (FILE3,5009) X,Y,Z,BUFFER(I,J,K,3),
     $                             BUFFER(I,J,K,4),BUFFER(I,J,K,5)
             END DO
           END DO
         END DO
      ELSE
        DO I=1,NLATTICE(1)
          DO J=1,NLATTICE(2)
            WRITE (FILE,5009) 
     $      ((BUFFER(I,J,K,IFUN),IFUN=1,NPLOTFUN),K=1,NLATTICE(3))
          END DO
        END DO
      END IF
 5009 FORMAT ((6(E13.5)))
      DEALLOCATE (BUFFER,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) 
     $    CALL ERRMSG('BIN2ASC','DEALLOCATION FAILED',1)
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Format statements ***
C
 5000 FORMAT ('PLOT COORDINATES IN ',A)
 5010 FORMAT (/,9X,'X',15X,'Y',15X,'Z',12X,A10)
 5020 FORMAT (4(3X,E13.6))
C
C     ------------------------------------------------------------------
C
C     *** Handling of errors ***
C
99980 CONTINUE
      CALL ERRMSG('BIN2ASC','I/O ERROR READING UNSTRUCTURED LATTICE',1)
C
99990 CONTINUE
      CALL ERRMSG('BIN2ASC','EOF WHILE READING UNSTRUCTURED LATTICE',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE BIN2ASCII ***
C
      END
