      SUBROUTINE BLDCDJ(DPA,DPB,PA,PB,DSTO,DSTO_ME)
C
C     Purpose: Build charge density Coulomb vector.
C
C     History: - Creation (19.02.15, AMK)
C                         (25.08.16, BZG)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables:
C
C     DPA: Difference alpha density matrix.
C     DPB: Difference beta density matrix.
C     PA : Alpha density matrix.
C     PB : Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'math.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'opt.h'
      INCLUDE 'pointer.h'
      INCLUDE 'scf.h'
      INCLUDE 'rtdft.h'



      include 'dynamics.h'
      include 'iteration.h'
C
      INTEGER DSTO,DSTO_ME
      REAL DPA(DSTO,DSTO),DPB(DSTO,DSTO)
      REAL PA(DSTO_ME,DSTO_ME),PB(DSTO_ME,DSTO_ME)
C
      INTEGER KATOM,KAUX,KSET,KSHL,KX,KY,KZ,OFFSET
      REAL ERISIGN,NORM,SIGNUS
C
C     ------------------------------------------------------------------
C
C     *** Quick return, if all auxiliary functions are fixed ***
C
      IF (NAUX.EQ.0) RETURN
C
C     *** Synchronize parallel processes ***
C
      CALL MP_SYNC
C
C     *** Suspend external timers for J vector calculation ***
C
      IF (STATUS.EQ.'FFP') THEN
        CALL TIMER('SUSPEND','FFP ITERATION')
      ELSE IF (STATUS.EQ.'BOMD') THEN
        CALL TIMER('SUSPEND','MD SIMULATION')
      ELSE IF (STATUS.EQ.'POLAR') THEN
        CALL TIMER('SUSPEND','POLARIZABILITY')
      ELSE IF (STATUS.EQ.'FREQ') THEN
        CALL TIMER('SUSPEND','FORCE CONSTANTS')
      ELSE IF (STATUS.EQ.'ESA') THEN
        CALL TIMER('SUSPEND','ELECTRONIC STRUCTURE ANALYSIS')
      ELSE IF (STATUS.EQ.'SCF ITERATION') THEN
        CALL TIMER('SUSPEND','SCF ITERATION')
      ELSE IF ((STATUS.EQ.'HESSIAN').OR.(STATUS.EQ.'UPDATE')) THEN
        IF (SADDLE) THEN
          CALL TIMER('SUSPEND','SADDLE INTERPOLATION')
        ELSE
          CALL TIMER('SUSPEND','OPTIMIZATION CYCLE')
        END IF
CCC ADLL
      ELSE IF (STATUS.EQ.'RTDFT') THEN
        CALL TIMER('SUSPEND','ELECTRON DYNAMICS SIMULATION')
CCC ADLL
      END IF
C
C     *** Build difference density matrix ***
C
      IF (MPMASTER) THEN
C
CCC ADLL
        IF ( (SCFCYC.EQ.1).OR.
     $      ((STATUS.EQ.'RTDFT').AND.(RTCYC.EQ.0))) THEN
CCC ADLL
          DPA(:,:) = PA(:,:)
          IF (OKS) DPB(:,:) = PB(:,:)
        ELSE
          CALL SCFIO(DPA,DSTO,'READ','PALPHA')
          DPA(:,:) = PA(:,:) - DPA(:,:)
          IF (OKS) THEN
            CALL SCFIO(DPB,DSTO,'READ','PBETA')
            DPB(:,:) = PB(:,:) - DPB(:,:)
          END IF
        END IF
C
C     *** Store density matrix on tape ***
C
        IF (ERITYP.EQ.'DIRECT') THEN
          CALL SCFIO(PA,DSTO,'WRITE','PALPHA')
          IF (OKS) CALL SCFIO(PB,DSTO,'WRITE','PBETA')
        END IF
C
      END IF
C
C     *** Initialize charge density Coulomb vector ***
C
      CDJA(:) = CDJOLDA(:)
      CDJB(:) = CDJOLDB(:)
C
      CDJOLDA(:) = 0.0
      CDJOLDB(:) = 0.0
C
      IF (NCPU.GT.1) THEN
        CALL SCALMAT(CDJA,NAUX,NAUX,1,'TOTAL',MP_SFC)
        CALL SCALMAT(CDJB,NAUX,NAUX,1,'TOTAL',MP_SFC)
      END IF
C
      CALL MP_SEND_ALL(DPA,DSTO*NSTO,MASTER,MP_REAL)
      IF (OKS) CALL MP_SEND_ALL(DPB,DSTO*NSTO,MASTER,MP_REAL)
C
C     *** Build new J vector from mixed ERIs ***
C
      IF (ERIRAM) THEN
C
C     *** Load near-field ERIs and build J vector ***
C
        CALL RAMCDJ(DPA,DPB,DSTO)
C
C     *** Calculate far-field ERIs and build J vector ***
C
        CALL FARCDJ(DPA,DPB,DSTO)
C
C     *** Build new J vector from direct ERIs ***
C
      ELSE
C
C     *** Calculate near-field ERIs and build J vector ***
C
        CALL NEARCDJ(DPA,DPB,DSTO)
C
C     *** Calculate far-field ERIs and build J vector ***
C
        CALL FARCDJ(DPA,DPB,DSTO)
C
      END IF
C
C     *** Add atomic multipole contributions to Coulomb vector ***
C
      IF (MULTIPOLE) THEN
        CALL MP_SUM(CDJOLDA,CDJOLDA,NAUX,MP_REAL)
        CALL MP_SUM(CDJOLDB,CDJOLDB,NAUX,MP_REAL)
      END IF
C
      IF (MPMASTER.AND.MULTIPOLE) THEN
C
        DO 40 KATOM=1,NATOM
          OFFSET = LLAUXM(KATOM) - 1
          DO 30 KSET=LLAUXSET(KATOM),ULAUXSET(KATOM)
            NORM = (PI/ZET(NGTO+KSET))**1.5
            DO 20 KSHL=LLAUXSHL(KSET),ULAUXSHL(KSET)
              DO 10 KAUX=LLAUX(KSHL),ULAUX(KSHL)
                KX = AUXPTR(KAUX,3)
                KY = AUXPTR(KAUX,4)
                KZ = AUXPTR(KAUX,5)
                CDJA(KAUX) = CDJA(KAUX) + 
     $                       NORM*CDJOLDA(OFFSET+COP(KX,KY,KZ))
                IF (OKS) THEN
                  CDJB(KAUX) = CDJB(KAUX) +
     $                         NORM*CDJOLDB(OFFSET+COP(KX,KY,KZ))
                END IF
   10         CONTINUE
   20       CONTINUE
   30     CONTINUE
   40   CONTINUE
C
      END IF
C
      CALL MP_SUM(CDJA,CDJA,NAUX,MP_REAL)
      CALL MP_SUM(CDJB,CDJB,NAUX,MP_REAL)
C
C     *** Save and normalize Coulomb vector ***
C
      DO 50 KAUX=1,NAUX
        CDJOLDA(KAUX) = CDJA(KAUX)
        CDJOLDB(KAUX) = CDJB(KAUX)
        SIGNUS = ERISIGN(AUXPTR(KAUX,3)+AUXPTR(KAUX,4)+AUXPTR(KAUX,5))
        CDJA(KAUX) = SIGNUS*NCAUX(KAUX)*CDJA(KAUX)
        CDJB(KAUX) = SIGNUS*NCAUX(KAUX)*CDJB(KAUX)
   50 CONTINUE
C
C     *** Add fixed Coulomb vector contribution ***
C
      CDJA(1:NAUX) = CDJA(1:NAUX) - FIXCDJA(1:NAUX)
C
      IF (OKS) THEN
        CDJB(1:NAUX) = CDJB(1:NAUX) - FIXCDJB(1:NAUX)
      ELSE
        CDJA(1:NAUX) = CDJA(1:NAUX) - FIXCDJB(1:NAUX)
      END IF
C
C     *** Continue external timers after J vector calculation ***
C
      IF (STATUS.EQ.'FFP') THEN
        CALL TIMER('CONTINUE','FFP ITERATION')
      ELSE IF (STATUS.EQ.'BOMD') THEN
        CALL TIMER('CONTINUE','MD SIMULATION')
      ELSE IF (STATUS.EQ.'POLAR') THEN
        CALL TIMER('CONTINUE','POLARIZABILITY')
      ELSE IF (STATUS.EQ.'FREQ') THEN
        CALL TIMER('CONTINUE','FORCE CONSTANTS')
      ELSE IF (STATUS.EQ.'ESA') THEN
        CALL TIMER('CONTINUE','ELECTRONIC STRUCTURE ANALYSIS')
      ELSE IF (STATUS.EQ.'SCF ITERATION') THEN
        CALL TIMER('CONTINUE','SCF ITERATION')
      ELSE IF ((STATUS.EQ.'HESSIAN').OR.(STATUS.EQ.'UPDATE')) THEN
        IF (SADDLE) THEN
          CALL TIMER('CONTINUE','SADDLE INTERPOLATION')
        ELSE
          CALL TIMER('CONTINUE','OPTIMIZATION CYCLE')
        END IF
CCC ADLL
      ELSE IF (STATUS.EQ.'RTDFT') THEN
        CALL TIMER('CONTINUE','ELECTRON DYNAMICS SIMULATION')
CCC ADLL
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE BLDCDJ ***
C
      END
