      SUBROUTINE READZMAT
C
C     Purpose: Read Z-MATRIX input.
C
C     History: - Creation (05.09.96, BZ) 
C                         (03.05.97, AMK)
C                         (23.02.00, FJ)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     LINE    : Input line.
C     NA,NB,NC: Connectivity description for atoms.
C     NATOMS  : Number of atoms in Z-Matrix (NATQM+NATMM+NDUMMY).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mm.h'
      INCLUDE 'physic.h'
C
      LOGICAL BLOCKEND,ERROR,SUCCESS
      CHARACTER*80 DTS,LINE,LTU,PRTSTR,STRCOMP,STRING,STRTYP*7
      INTEGER I,IATOM,IPOS,ITOKEN,IVALUE,NATOMS,NTOKEN,STREXT,STRLEN
      REAL RVALUE
C
      CHARACTER*80 TOKEN(80)
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      ERROR = .FALSE.
C
C     *** Move mixed input to first Z-Matrix line ***
C
      IF (INPTYP.EQ.'MIXED') THEN
        CALL FINDKEY('GEOMETRY',SUCCESS)
        DO IATOM=1,NATXYZ(4)
          READ (INP,'(A)',ERR=99980,END=1000) LINE
        END DO
        IATOM = NATXYZ(1) + NATXYZ(2)
        GO TO 500
      END IF
C
C     *** Set connectivity index for unused fields ***
C
      NA(1) = 0
      NB(1) = 0 
      NC(1) = 0
      NB(2) = 0
      NC(2) = 0
      NC(3) = 0
C
C     *** Initialize pure Z-Matrix input ***
C
      NATMM = 0
      NDUMMY = 0
C
      TYPEMM(:) = 0
      LINKMM(:,:) = 0
C
C     *** Read Z-Matrix input ***
C
      IATOM = 0
C
  500 CONTINUE
C
        READ (INP,'(A)',ERR=99980,END=1000) LINE
        STRING = STRCOMP(LTU(DTS(LINE)))
        IF (BLOCKEND(STRING)) GO TO 1000
        IATOM = IATOM + 1
C
        NCODE(IATOM) = 'RAD'
C
C     *** Tokenize Z-Matrix line ***
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        IF (STREXT(TOKEN(1)).GT.MAXLAB) THEN
          WRITE (PRTSTR,5000) TOKEN(1)(1:STREXT(TOKEN(1)))
 5000     FORMAT ('LABEL < ',A,' > TOO LONG (MAX. 10 CHRS.)')
          CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
          GO TO 99990
        END IF
C
C     *** Check atomic label, assign atomic number and store label ***
C
        CALL CHKLABEL(TOKEN(1),ATOMNO(IATOM),ERROR)
        IF (ERROR) GO TO 99990
        MATOM(IATOM) = STDMATOM(ATOMNO(IATOM))
        ZATOM(IATOM) = ATOMNO(IATOM)
        ZCORE(IATOM) = 0
        ATOMLAB(IATOM) = TOKEN(1)
        IF (ATOMNO(IATOM).EQ.0) NDUMMY = NDUMMY + 1
C
C     *** Initialize variable name ***
C
        OPTVAR(IATOM,1) = ' '
        OPTVAR(IATOM,2) = ' '
        OPTVAR(IATOM,3) = ' '
C
C     *** Read ZATOM, MATOM, TYPEMM or LINKMM if set for first atom ***
C
        IF (IATOM.EQ.1) THEN
C
          IF (NTOKEN.GE.2) THEN
            STRLEN = STREXT(TOKEN(2))
            CALL GETNUM(TOKEN(2),STRTYP,RVALUE,IVALUE)
            IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(2)(1:STRLEN)
 5010         FORMAT ('STRING < ',A,' > TOO LONG (MAX. 10 CHRS.)')
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            IF (STRTYP.EQ.'INTEGER') THEN
              IF (IVALUE.LT.0) THEN
                NATMM = NATMM + 1
                TYPEMM(IATOM) = ABS(IVALUE)
                ZATOM(IATOM) = -ZATOM(IATOM)
              ELSE
                ZATOM(IATOM) = IVALUE
              END IF
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              MATOM(IATOM) = RVALUE
            ELSE
              CALL MESSAGE(1,'ERROR IN POSITION 2',1,0)
              GO TO 99990
            END IF
          END IF
C
          IF (NTOKEN.GE.3) THEN
            STRLEN = STREXT(TOKEN(3))
            CALL GETNUM(TOKEN(3),STRTYP,RVALUE,IVALUE)
            IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(3)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            IF (STRTYP.EQ.'INTEGER') THEN
              IF (IVALUE.LT.0) THEN
                NATMM = NATMM + 1
                TYPEMM(IATOM) = ABS(IVALUE)
                ZATOM(IATOM) = -ZATOM(IATOM)
              ELSE IF (ZATOM(IATOM).LT.0) THEN
                I = LINKMM(IATOM,0) + 1
                LINKMM(IATOM,I) = IVALUE
                LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
              ELSE
                ZATOM(IATOM) = IVALUE
              END IF
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              MATOM(IATOM) = RVALUE
            ELSE
              CALL MESSAGE(1,'ERROR IN POSITION 3',1,0)
              GO TO 99990
            END IF
          END IF
C
C     *** Read remaining LINKMMs if set for first atom ***
C
          DO ITOKEN=4,NTOKEN
            STRLEN = STREXT(TOKEN(ITOKEN))
            CALL GETNUM(TOKEN(ITOKEN),STRTYP,RVALUE,IVALUE)
            IF (ZATOM(IATOM).GE.0) THEN
              PRTSTR = 'TOO MANY ENTRIES IN Z-MATRIX LINE 1'
              CALL ERRMSG('READZMAT',PRTSTR,1)
            ELSE IF (STRTYP.NE.'INTEGER') THEN
              WRITE (PRTSTR,5020) ITOKEN
 5020         FORMAT ('ERROR IN POSITION ',I2,'; ENTRY MUST BE INTEGER')
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            ELSE IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(ITOKEN)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            I = LINKMM(IATOM,0) + 1
            IF (I.GT.MAXLINK) THEN
              WRITE (PRTSTR,5030) IATOM,I,MAXLINK
 5030         FORMAT ('ATOM ',I5,' HAS ',I2,' LINKS > MAXLINK = ',I2)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            LINKMM(IATOM,I) = IVALUE
            LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
          END DO
C
          GO TO 500
C
        END IF
C
C     *** Read connectivity for length ***
C
        CALL GETNUM(TOKEN(2),STRTYP,RVALUE,NA(IATOM))
        IF (STRTYP.EQ.'FLOAT') THEN
          CALL MESSAGE(1,'CONNECTOR FOR LENGTH CANNOT BE REAL',1,0)
          GO TO 99990
        END IF
        IF (STRTYP.EQ.'STRING') THEN
          DO 10 IPOS=1,IATOM-1
            IF (TOKEN(2).EQ.ATOMLAB(IPOS)) GO TO 20
   10     CONTINUE
          CALL MESSAGE(1,'WRONG CONNECTOR FOR LENGTH IN Z-MATRIX',1,0)
          GO TO 99990
   20     CONTINUE
          NA(IATOM) = IPOS
        ELSE IF (STRTYP.EQ.'INTEGER') THEN
          IF (NA(IATOM).GE.IATOM) THEN
            CALL MESSAGE(1,'WRONG CONNECTOR FOR LENGTH IN Z-MATRIX',1,0)
            GO TO 99990
          END IF
        END IF
        IF ((INPTYP.EQ.'MIXED').AND.(STRTYP.EQ.'INTEGER')) THEN
          IF (NA(IATOM).LE.NATXYZ(1)) THEN
            NA(IATOM) = ATOMMAP(NA(IATOM),3)
          ELSE
            NA(IATOM) = NA(IATOM) + NATXYZ(2)
          END IF
        END IF
C
C     *** Read value for length or variable ***
C
        CALL GETZSYM(TOKEN(3),COORD(1,IATOM,3),OPTVAR(IATOM,1),STRTYP)
        IF (MDRUN.AND.(STRTYP.EQ.'FLOAT')) THEN
          WRITE (PRTSTR,5040) TOKEN(3)(1:STREXT(TOKEN(3)))
 5040     FORMAT ('DEFINE CONSTANT < ',A,' > BY MD CONSTRAINT')
          CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
          GO TO 99990
        END IF
C
C     *** Read ZATOM, MATOM, TYPEMM or LINKMM if set for second atom ***
C
        IF (IATOM.EQ.2) THEN
C
          IF (NTOKEN.GE.4) THEN
            STRLEN = STREXT(TOKEN(4))
            CALL GETNUM(TOKEN(4),STRTYP,RVALUE,IVALUE)
            IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(4)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            IF (STRTYP.EQ.'INTEGER') THEN
              IF (IVALUE.LT.0) THEN
                NATMM = NATMM + 1
                TYPEMM(IATOM) = ABS(IVALUE)
                ZATOM(IATOM) = -ZATOM(IATOM)
              ELSE
                ZATOM(IATOM) = IVALUE
              END IF
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              MATOM(IATOM) = RVALUE
            ELSE
              CALL MESSAGE(1,'ERROR IN POSITION 4',1,0)
              GO TO 99990
            END IF
          END IF
C
          IF (NTOKEN.GE.5) THEN
            STRLEN = STREXT(TOKEN(5))
            CALL GETNUM(TOKEN(5),STRTYP,RVALUE,IVALUE)
            IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(5)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            IF (STRTYP.EQ.'INTEGER') THEN
              IF (IVALUE.LT.0) THEN
                NATMM = NATMM + 1
                TYPEMM(IATOM) = ABS(IVALUE)
                ZATOM(IATOM) = -ZATOM(IATOM)
              ELSE IF (ZATOM(IATOM).LT.0) THEN
                I = LINKMM(IATOM,0) + 1
                LINKMM(IATOM,I) = IVALUE
                LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
              ELSE
                ZATOM(IATOM) = IVALUE
              END IF
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              MATOM(IATOM) = RVALUE
            ELSE
              CALL MESSAGE(1,'ERROR IN POSITION 5',1,0)
              GO TO 99990
            END IF
          END IF
C
C     *** Read remaining LINKMMs if set for second atom ***
C
          DO ITOKEN=6,NTOKEN
            STRLEN = STREXT(TOKEN(ITOKEN))
            CALL GETNUM(TOKEN(ITOKEN),STRTYP,RVALUE,IVALUE)
            IF (ZATOM(IATOM).GE.0) THEN
              PRTSTR = 'TOO MANY ENTRIES IN Z-MATRIX LINE 2'
              CALL ERRMSG('READZMAT',PRTSTR,1)
            ELSE IF (STRTYP.NE.'INTEGER') THEN
              WRITE (PRTSTR,5020) ITOKEN
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            ELSE IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(ITOKEN)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            I = LINKMM(IATOM,0) + 1
            IF (I.GT.MAXLINK) THEN
              WRITE (PRTSTR,5030) IATOM,I,MAXLINK
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            LINKMM(IATOM,I) = IVALUE
            LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
          END DO
C
          GO TO 500
C
        END IF
C
C     *** Read connectivity for angle ***
C
        CALL GETNUM(TOKEN(4),STRTYP,RVALUE,NB(IATOM))
        IF (STRTYP.EQ.'FLOAT') THEN
          CALL MESSAGE(1,'CONNECTOR FOR ANGLE CANNOT BE REAL',1,0)
          GO TO 99990
        END IF
        IF (STRTYP.EQ.'STRING') THEN
          DO 30 IPOS=1,IATOM-1
            IF (TOKEN(4).EQ.ATOMLAB(IPOS)) GO TO 40
   30     CONTINUE
          CALL MESSAGE(1,'WRONG CONNECTOR FOR ANGLE IN Z-MATRIX',1,0)
          GO TO 99990
   40     CONTINUE
          IF (TOKEN(4).EQ.TOKEN(2)) THEN
            PRTSTR = 'DUBLICATE DEFINITION OF CONNECTORS'
            CALL MESSAGE(1,PRTSTR,1,0)
            GO TO 99990
          END IF
          NB(IATOM) = IPOS
        ELSE IF (STRTYP.EQ.'INTEGER') THEN
          IF (NB(IATOM).GE.IATOM) THEN
            CALL MESSAGE(1,'WRONG CONNECTOR FOR ANGLE IN Z-MATRIX',1,0)
            GO TO 99990
          END IF
        END IF
        IF ((INPTYP.EQ.'MIXED').AND.(STRTYP.EQ.'INTEGER')) THEN
          IF (NB(IATOM).LE.NATXYZ(1)) THEN
            NB(IATOM) = ATOMMAP(NB(IATOM),3)
          ELSE
            NB(IATOM) = NB(IATOM) + NATXYZ(2)
          END IF
        END IF
C
C     *** Read value for angle or variable ***
C
        CALL GETZSYM(TOKEN(5),COORD(2,IATOM,3),OPTVAR(IATOM,2),STRTYP)
        IF (MDRUN.AND.(STRTYP.EQ.'FLOAT')) THEN
          WRITE (PRTSTR,5050) TOKEN(5)(1:STREXT(TOKEN(5)))
 5050     FORMAT ('DEFINE CONSTANT < ',A,' > BY MD CONSTRAINT')
          CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
          GO TO 99990
        END IF
C
C     *** Read NCODE, ZATOM, MATOM, TYPEMM or LINKMM for third atom ***
C
        IF (IATOM.EQ.3) THEN
C
          IF (NTOKEN.GE.6) THEN
            STRLEN = STREXT(TOKEN(6))
            CALL GETNUM(TOKEN(6),STRTYP,RVALUE,IVALUE)
            IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(6)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            IF (STRTYP.EQ.'INTEGER') THEN
              IF (IVALUE.LT.0) THEN
                NATMM = NATMM + 1
                TYPEMM(IATOM) = ABS(IVALUE)
                ZATOM(IATOM) = -ZATOM(IATOM)
              ELSE
                ZATOM(IATOM) = IVALUE
              END IF
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              MATOM(IATOM) = RVALUE
            ELSE IF (STRTYP.EQ.'STRING') THEN
              NCODE(IATOM) = TOKEN(6)(:3)
            ELSE
              CALL MESSAGE(1,'ERROR IN POSITION 6',1,0)
              GO TO 99990
            END IF
          END IF
C
          IF (NTOKEN.GE.7) THEN
            STRLEN = STREXT(TOKEN(7))
            CALL GETNUM(TOKEN(7),STRTYP,RVALUE,IVALUE)
            IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(7)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            IF (STRTYP.EQ.'INTEGER') THEN
              IF (IVALUE.LT.0) THEN
                NATMM = NATMM + 1
                TYPEMM(IATOM) = ABS(IVALUE)
                ZATOM(IATOM) = -ZATOM(IATOM)
              ELSE IF (ZATOM(IATOM).LT.0) THEN
                I = LINKMM(IATOM,0) + 1
                LINKMM(IATOM,I) = IVALUE
                LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
              ELSE
                ZATOM(IATOM) = IVALUE
              END IF
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              MATOM(IATOM) = RVALUE
            ELSE IF (STRTYP.EQ.'STRING') THEN
              NCODE(IATOM) = TOKEN(7)(:3)
            ELSE
              CALL MESSAGE(1,'ERROR IN POSITION 7',1,0)
              GO TO 99990
            END IF
          END IF
C
          IF (NTOKEN.GE.8) THEN
            STRLEN = STREXT(TOKEN(8))
            CALL GETNUM(TOKEN(8),STRTYP,RVALUE,IVALUE)
            IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(7)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            IF (STRTYP.EQ.'INTEGER') THEN
              IF (IVALUE.LT.0) THEN
                NATMM = NATMM + 1
                TYPEMM(IATOM) = ABS(IVALUE)
                ZATOM(IATOM) = -ZATOM(IATOM)
              ELSE IF (ZATOM(IATOM).LT.0) THEN
                I = LINKMM(IATOM,0) + 1
                LINKMM(IATOM,I) = IVALUE
                LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
              ELSE
                ZATOM(IATOM) = IVALUE
              END IF
            ELSE IF (STRTYP.EQ.'FLOAT') THEN
              MATOM(IATOM) = RVALUE
            ELSE IF (STRTYP.EQ.'STRING') THEN
              NCODE(IATOM) = TOKEN(8)(:3)
            ELSE
              CALL MESSAGE(1,'ERROR IN POSITION 8',1,0)
              GO TO 99990
            END IF
          END IF
C
C     *** Read remaining LINKMMs if set for third atom ***
C
          DO ITOKEN=9,NTOKEN
            STRLEN = STREXT(TOKEN(ITOKEN))
            CALL GETNUM(TOKEN(ITOKEN),STRTYP,RVALUE,IVALUE)
            IF (ZATOM(IATOM).GE.0) THEN
              PRTSTR = 'TOO MANY ENTRIES IN Z-MATRIX LINE 3'
              CALL ERRMSG('READZMAT',PRTSTR,1)
            ELSE IF (STRTYP.NE.'INTEGER') THEN
              WRITE (PRTSTR,5020) ITOKEN
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            ELSE IF (STRLEN.GT.MAXVAR) THEN
              WRITE (PRTSTR,5010) TOKEN(ITOKEN)(1:STRLEN)
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            I = LINKMM(IATOM,0) + 1
            IF (I.GT.MAXLINK) THEN
              WRITE (PRTSTR,5030) IATOM,I,MAXLINK
              CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
              GO TO 99990
            END IF
            LINKMM(IATOM,I) = IVALUE
            LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
          END DO
C
          GO TO 500
C
        END IF
C
C     *** Read connectivity for dihedral angle ***
C
        CALL GETNUM(TOKEN(6),STRTYP,RVALUE,NC(IATOM))
        IF (STRTYP.EQ.'FLOAT') THEN
          PRTSTR = 'CONNECTOR FOR DIHEDRAL ANGLE CANNOT BE REAL'
          CALL MESSAGE(1,PRTSTR,1,0)
          GO TO 99990
        END IF
        IF (STRTYP.EQ.'STRING') THEN
          DO 50 IPOS=1,IATOM-1
            IF (TOKEN(6).EQ.ATOMLAB(IPOS)) GO TO 60
   50     CONTINUE
          PRTSTR = 'WRONG CONNECTOR FOR DIHEDRAL ANGLE IN Z-MATRIX'
          CALL MESSAGE(1,PRTSTR,1,0)
          GO TO 99990
   60     CONTINUE
          IF ((TOKEN(6).EQ.TOKEN(2)).OR.(TOKEN(6).EQ.TOKEN(4))) THEN
            PRTSTR = 'DUBLICATE DEFINITION OF CONNECTORS'
            CALL MESSAGE(1,PRTSTR,1,0)
            GO TO 99990
          END IF
          NC(IATOM) = IPOS
        ELSE IF (STRTYP.EQ.'INTEGER') THEN
          IF (NA(IATOM).GE.IATOM) THEN
            PRTSTR = 'WRONG CONNECTOR FOR DIHEDRAL ANGLE IN Z-MATRIX'
            CALL MESSAGE(1,PRTSTR,1,0)
            GO TO 99990
          END IF
        END IF
        IF ((INPTYP.EQ.'MIXED').AND.(STRTYP.EQ.'INTEGER')) THEN
          IF (NC(IATOM).LE.NATXYZ(1)) THEN
            NC(IATOM) = ATOMMAP(NC(IATOM),3)
          ELSE
            NC(IATOM) = NC(IATOM) + NATXYZ(2)
          END IF
        END IF
C
C     *** Read value for dihedral angle or variable ***
C
        CALL GETZSYM(TOKEN(7),COORD(3,IATOM,3),OPTVAR(IATOM,3),STRTYP)
        IF (MDRUN.AND.(STRTYP.EQ.'FLOAT')) THEN
          WRITE (PRTSTR,5060) TOKEN(7)(1:STREXT(TOKEN(7)))
 5060     FORMAT ('DEFINE CONSTANT < ',A,' > BY MD CONSTRAINT')
          CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
          GO TO 99990
        END IF
C
C     *** Read NCODE, ZATOM, MATOM, TYPEMM or LINKMM if set ***
C
        IF (NTOKEN.GE.8) THEN
          STRLEN = STREXT(TOKEN(8))
          CALL GETNUM(TOKEN(8),STRTYP,RVALUE,IVALUE)
          IF (STRLEN.GT.MAXVAR) THEN
            WRITE (PRTSTR,5010) TOKEN(8)(1:STRLEN)
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          IF (STRTYP.EQ.'INTEGER') THEN
            IF (IVALUE.LT.0) THEN
              NATMM = NATMM + 1
              TYPEMM(IATOM) = ABS(IVALUE)
              ZATOM(IATOM) = -ZATOM(IATOM)
            ELSE
              ZATOM(IATOM) = IVALUE
            END IF
          ELSE IF (STRTYP.EQ.'FLOAT') THEN
            MATOM(IATOM) = RVALUE
          ELSE IF (STRTYP.EQ.'STRING') THEN
            NCODE(IATOM) = TOKEN(8)(:3)
          ELSE
            CALL MESSAGE(1,'ERROR IN POSITION 8',1,0)
            GO TO 99990
          END IF
        END IF
C
        IF (NTOKEN.GE.9) THEN
          STRLEN = STREXT(TOKEN(9))
          CALL GETNUM(TOKEN(9),STRTYP,RVALUE,IVALUE)
          IF (STRLEN.GT.MAXVAR) THEN
            WRITE (PRTSTR,5010) TOKEN(9)(1:STRLEN)
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          IF (STRTYP.EQ.'INTEGER') THEN
            IF (IVALUE.LT.0) THEN
              NATMM = NATMM + 1
              TYPEMM(IATOM) = ABS(IVALUE)
              ZATOM(IATOM) = -ZATOM(IATOM)
            ELSE IF (ZATOM(IATOM).LT.0) THEN
              I = LINKMM(IATOM,0) + 1
              LINKMM(IATOM,I) = IVALUE
              LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
            ELSE
              ZATOM(IATOM) = IVALUE
            END IF
          ELSE IF (STRTYP.EQ.'FLOAT') THEN
            MATOM(IATOM) = RVALUE
          ELSE IF (STRTYP.EQ.'STRING') THEN
            NCODE(IATOM) = TOKEN(9)(:3)
          ELSE
            CALL MESSAGE(1,'ERROR IN POSITION 9',1,0)
            GO TO 99990
          END IF
        END IF
C
        IF (NTOKEN.GE.10) THEN
          STRLEN = STREXT(TOKEN(10))
          CALL GETNUM(TOKEN(10),STRTYP,RVALUE,IVALUE)
          IF (STRLEN.GT.MAXVAR) THEN
            WRITE (PRTSTR,5010) TOKEN(10)(1:STRLEN)
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          IF (STRTYP.EQ.'INTEGER') THEN
            IF (IVALUE.LT.0) THEN
              NATMM = NATMM + 1
              TYPEMM(IATOM) = ABS(IVALUE)
              ZATOM(IATOM) = -ZATOM(IATOM)
            ELSE IF (ZATOM(IATOM).LT.0) THEN
              I = LINKMM(IATOM,0) + 1
              LINKMM(IATOM,I) = IVALUE
              LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
            ELSE
              ZATOM(IATOM) = IVALUE
            END IF
          ELSE IF (STRTYP.EQ.'FLOAT') THEN
            MATOM(IATOM) = RVALUE
          ELSE IF (STRTYP.EQ.'STRING') THEN
            NCODE(IATOM) = TOKEN(10)(:3)
          ELSE
            CALL MESSAGE(1,'ERROR IN POSITION 10',1,0)
            GO TO 99990
          END IF
        END IF
C
C     *** Read remaining LINKMMs if set ***
C
        DO ITOKEN=11,NTOKEN
          STRLEN = STREXT(TOKEN(ITOKEN))
          CALL GETNUM(TOKEN(ITOKEN),STRTYP,RVALUE,IVALUE)
          IF (ZATOM(IATOM).GE.0) THEN
            WRITE (PRTSTR,5070) IATOM
 5070       FORMAT ('TOO MANY ENTRIES IN Z-MATRIX LINE ',I5)
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          ELSE IF (STRTYP.NE.'INTEGER') THEN
            WRITE (PRTSTR,5020) ITOKEN
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          ELSE IF (STRLEN.GT.MAXVAR) THEN
            WRITE (PRTSTR,5010) TOKEN(ITOKEN)(1:STRLEN)
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          I = LINKMM(IATOM,0) + 1
          IF (I.GT.MAXLINK) THEN
            WRITE (PRTSTR,5030) IATOM,I,MAXLINK
            CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
            GO TO 99990
          END IF
          LINKMM(IATOM,I) = IVALUE
          LINKMM(IATOM,0) = LINKMM(IATOM,0) + 1
        END DO
C
      GO TO 500
C
 1000 CONTINUE
C
C     *** Set number of atoms in the Z-Matrix ***
C
      NATOMS = IATOM
      NATOM_ALL = NATOMS - NDUMMY
      NATOM = NATOM_ALL - NATMM
C
C     *** Initialize ATOMMAP ***
C
      CALL MAPATOM(ATOMMAP,ATOMNO,ZATOM,NATOM,NATMM,NDUMMY,0,'ZMAT')
C
C     *** Extend ATOMMAP of mixed input ***
C
      IF (INPTYP.EQ.'MIXED') THEN
        CALL MAPATOM(ATOMMAP,ATOMNO,ZATOM,NATOM,NATMM,NDUMMY,NATXYZ(3),
     $               'MIXED')
      END IF
C
C     *** Return, if syntax check ***
C
      IF (STATUS.EQ.'CHECK') RETURN
C
C     *** Check number of atoms ***
C
      IF (NATOMS.EQ.0) THEN
        PRTSTR = 'NO ATOMS FOUND IN THE INPUT FILE'
        CALL ERRMSG('READZMAT',PRTSTR,1)
      ELSE IF (.NOT.OPT) THEN
        OPTTYP = 'UNDEFINED'
      ELSE IF (NATOMS.EQ.1) THEN
        OPTTYP = 'UNDEFINED'
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
99980 CONTINUE
      WRITE (PRTSTR,99985) IATOM+1
99985 FORMAT ('ERROR IN Z-MATRIX LINE ',I4)
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READZMAT',STRCOMP(PRTSTR),1)
C
99990 CONTINUE
      WRITE (PRTSTR,99995) IATOM
99995 FORMAT ('ERROR IN Z-MATRIX LINE ',I4)
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READZMAT',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE READZMAT ***
C
      END
