      SUBROUTINE CDSDD(CDC,OSD,HTO,DAUX,LLGPB,NGPB)
C
C     Purpose: Add the contribution of the current batch of grid
C              points to the approximated density vector.
C
C     History: - Creation (23.03.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     NGPB: Number of grid points in the current batch.
C     OSD : Density values at grid points. 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'pointer.h'
C
      INTEGER DAUX
      REAL OSD(NAUX),HTO(MAXGPB,DAUX)
      REAL CDC(MAXAUX)
C
      INTEGER IGP,LLGPB,NGPA,NGPB
      INTEGER IATOM,INEIGH,IAUX,IAUXSET,IFIX,IFIXSET
      REAL XAG,YAG,ZAG
C
      INTEGER IGPPTR(MAXGPB)
      REAL DAG(MAXGPB),WGRID(MAXGPB)
C
C     -----------------------------------------------------------------
C
      DO IGP=1,NGPB
        WGRID(IGP) = GRDVEC(4,LLGPB+IGP-1)
      END DO

C     *** Loop over all neighbors ***
C
      DO INEIGH=1,NNEIGH
C
        IATOM = NEIGH(INEIGH)
C
        DO IGP=1,NGPB
          XAG = GRDVEC(1,LLGPB+IGP-1) - COORD(1,IATOM,CCSET)
          YAG = GRDVEC(2,LLGPB+IGP-1) - COORD(2,IATOM,CCSET)
          ZAG = GRDVEC(3,LLGPB+IGP-1) - COORD(3,IATOM,CCSET)
          DAG(IGP) = XAG**2 + YAG**2 + ZAG**2
        END DO
C
        DO IAUXSET=LLAUXSET(IATOM),ULAUXSET(IATOM)
C
C     *** Select active grid points of current set ***
C
          NGPA = 0
          DO IGP=1,NGPB
            IF (DAG(IGP).LE.AUXSETRAD(IAUXSET)) THEN
              NGPA = NGPA + 1
              IGPPTR(NGPA) = IGP
            END IF
          END DO
C
          IF (NGPA.EQ.0) CYCLE
C
C     *** Contribution of the free auxiliary functions ***
C
          DO IAUX=LLAUX(LLAUXSHL(IAUXSET)),ULAUX(ULAUXSHL(IAUXSET))
            DO IGP=1,NGPA
              OSD(IAUX) = OSD(IAUX) + CDC(IAUX)*HTO(IGPPTR(IGP),IAUX)*
     $                    WGRID(IGPPTR(IGP))
            END DO
          END DO
        END DO
C
C     *** Add fixed approximated density contribution, if requested ***
C
        IF (FIXAUXIS(IATOM)) THEN
C
          DO IFIXSET=LLFIXSET(IATOM),ULFIXSET(IATOM)
C
C     *** Select active grid points of current fixed set ***
C
            NGPA = 0
            DO IGP=1,NGPB
              IF (DAG(IGP).LE.AUXSETRAD(IFIXSET)) THEN
                NGPA = NGPA + 1
                IGPPTR(NGPA) = IGP
              END IF
            END DO
C
            IF (NGPA.EQ.0) CYCLE
C
C     *** Contribution of the fixed auxiliary functions ***
C
            DO IFIX=LLAUX(LLAUXSHL(IFIXSET)),ULAUX(ULAUXSHL(IFIXSET))
              OSD(IFIX) = OSD(IFIX) + CDC(IFIX)*HTO(IGPPTR(IGP),IFIX)*
     $                    WGRID(IGPPTR(IGP))

            END DO
C
          END DO
C
        END IF

      END DO
C
      END
