      SUBROUTINE CHKPCG(RA,RB,DAUX)
C
C     Purpose: Check pre-conditioned conjugated gradient convergence.
C
C     History: - Creation (26.06.07, AMK)
C
C     ******************************************************************
C
C     List of local dimension:
C
C     DAUX: Dimension of auxiliary function matrix.
C
C     List of local variables:
C
C     RA  : Alpha residual vector.
C     RB  : Beta residual vector.
C     RMSQ: Root mean square of residual.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'iteration.h'
      INCLUDE 'molecule.h'
      INCLUDE 'pcg.h'
CCC ADLL
      INCLUDE 'rtdft.h'
CCC ADLL
      INCLUDE 'scf.h'
      INCLUDE 'tolerance.h'
C
      INTEGER DAUX
      REAL RA(DAUX),RB(DAUX)
C
      INTEGER I
      REAL FACTOR,RMSSIX,RMSQ
C
C     ------------------------------------------------------------------
C
C     *** Calculate RMSQ of residual vectors ***
C
      RMSQ = SQRT(DOT_PRODUCT(RA,RA)/REAL(NAUX))
      IF (OKS) RMSQ = RMSQ + SQRT(DOT_PRODUCT(RB,RB)/NAUX)
C
C     *** Calculate rounding threshold ***
C
      IF (RMSQ.LT.TINIEST) THEN
        PCGRND = 1.E40 
      ELSE
        PCGRND = 1.0/MIN(RMSQ/10.0,1.0/PCGRND)
      END IF
C
C     *** Initialize mean RMSQ values ***
C
      IF (PCGCYC.EQ.0) PCGRMS(:) = 10.0
C
C     *** Calculate six steps RMSQ deviations ***
C
      RMSSIX = SQRT(SUM((RMSQ - PCGRMS(:))**2))
C
C     *** Update mean RMSQ values ***
C
      DO I=6,2,-1
        PCGRMS(I) = PCGRMS(I-1)
      END DO
      PCGRMS(1) = RMSQ
C 
C     *** PCG cycle output ***
C
      WRITE (OUT,5000) PCGCYC,PCGTOL,RMSQ
 5000 FORMAT (/,T2,'CYCLE: ',I4,4X,'EPSI = ',1PE15.5,
     $          4X,'RMSQ = ',1PE15.5)
C
      WRITE (OUT,5010) PCGCYC,PCGSIX,RMSSIX
 5010 FORMAT (T2,'CYCLE: ',I4,4X,'EPS6 = ',1PE15.5,
     $        4X,'RMS6 = ',1PE15.5)
C
      CALL DEFLUSH(OUT)
C
C     *** Check RMSQ residual convergence ***
C
      IF (PCGCYC.EQ.0) THEN
C
        CONVERGED = .FALSE.
        PCGCYC = PCGCYC + 1
C
CCC ADLL
        IF (STATUS.EQ.'RTDFT') THEN
          PCGSIX = 1.0E-10
          FACTOR = ABS(MAX(ERRSCF,TINIEST))/SCFTOL
          PCGTOL = PCGTOL/(10.0-MIN(LOG10(FACTOR),9.0))
          PCGTOL = MAX(PCGTOL,TINIEST)
        ELSE IF (SCFCYC.EQ.1) THEN
          PCGTOL=1.0E-5
          PCGSIX=1.0E-8
CCC ADLL
        ELSE IF (SCFCYC.GE.3) THEN
          PCGSIX = 1.0E-9
          FACTOR = ABS(MAX(ERRSCF,TINIEST))/SCFTOL
          PCGTOL = PCGTOL/(10.0-MIN(LOG10(FACTOR),9.0))
          PCGTOL = MAX(PCGTOL,TINIEST)
        ELSE IF (SCFCYC.GE.5) THEN
          PCGSIX = 1.0E-10
        END IF
C
C     ** Converged by residual RMSQ ***
C
      ELSE IF (RMSQ.LT.PCGTOL) THEN
C
        CONVERGED = .TRUE.
        PCGCYC = PCGCYC + 1
C
C     *** Converged by RMSQ averages ***
C
      ELSE IF (RMSSIX.LT.PCGSIX) THEN
C
        CONVERGED = .TRUE.
        PCGCYC = PCGCYC + 1
C
C     *** Convergence failed ***
C
      ELSE IF (PCGCYC.GE.PCGMAX) THEN
C
        BREAK = .TRUE.
        CONVERGED = .TRUE.
        PCGCYC = PCGCYC + 1
        CALL MESSAGE(1,'PCG NOT CONVERGED',1,1)
C
C     *** Continue iteration ***
C
      ELSE
C
        CONVERGED = .FALSE.
        PCGCYC = PCGCYC + 1
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE CHKPCG ***
C
      END
