      SUBROUTINE CUBUS(ICUBE,JCUBE,KCUBE,NGPC,OPTION)
C
C     Purpose: Store points of sublattice in grid vector GRDVEC.
C
C     History: - Creation (12.12.96, ML)
C                         (19.03.03, AMK)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DSTEP     : Step sizes along lattice basis vectors.
C     I/J/KEND  : End point of sublattice.
C     I/J/KSTART: Start point of sublattice.
C     NGPC      : Number of points in sublattice.
C     OPTION    : a) OPTION = CUBES, sublattice cube coordinates.
C                 b) OPTION = STEPS, sublattice step coordinates.
CCC ADLL
C                 c) OPTION = SLICES, sliced sublattice coordinates.
CCC ADLL
C
C     Definition of sublattice cubes:
C
C                      k
C                      |
C                      |
C                      |
C                      5-----------8
C                     /|          /|
C                    / |         / |
C                   /  |        /  |
C                  6-----------7   |
C                  |   |       |   |
C                  |   |       |   |
C                  |   1-------|---4 --- j
C                  |  /        |  /
C                  | /         | /
C                  |/          |/
C                  2-----------3
C                 /
C                /
C               i
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'grid.h'
      INCLUDE 'marching.h'
      INCLUDE 'plot.h'
C
      CHARACTER*(*) OPTION
      INTEGER I,IC,J,K,NGPC
      INTEGER ICUBE,IEND,ISTART,JCUBE,JEND,JSTART,KCUBE,KEND,KSTART
C
C     ------------------------------------------------------------------
C
C     *** Fill grid vector with sublattice cube coordinates ***
C
      IF (OPTION.EQ.'CUBES') THEN
C
C     *** Set loop index to work w.r.t. global lattice ***
C
        ISTART = ICUBE
        IEND = MAX(MIN(ICUBE+1,NLATTICE(1)-1),1)
        JSTART = JCUBE
        JEND = MAX(MIN(JCUBE+1,NLATTICE(2)-1),1)
        KSTART = KCUBE
        KEND = MAX(MIN(KCUBE+3,NLATTICE(3)-1),1)
C
C     *** Fill grid vector GRDVEC ***
C
        NGPC = 0
        DO K=KSTART,KEND
          DO J=JSTART,JEND
            DO I=ISTART,IEND
              DO IC=1,3
                GRDVEC(IC,NGPC+1) = LATTICE(IC,0) + DSTEP(1,IC)*(I-1)
     $                                            + DSTEP(2,IC)*(J-1)
     $                                            + DSTEP(3,IC)*(K-1)
                GRDVEC(IC,NGPC+2) = LATTICE(IC,0) + DSTEP(1,IC)*I
     $                                            + DSTEP(2,IC)*(J-1)
     $                                            + DSTEP(3,IC)*(K-1)
                GRDVEC(IC,NGPC+3) = LATTICE(IC,0) + DSTEP(1,IC)*I
     $                                            + DSTEP(2,IC)*J
     $                                            + DSTEP(3,IC)*(K-1)
                GRDVEC(IC,NGPC+4) = LATTICE(IC,0) + DSTEP(1,IC)*(I-1)
     $                                            + DSTEP(2,IC)*J
     $                                            + DSTEP(3,IC)*(K-1)
                GRDVEC(IC,NGPC+5) = LATTICE(IC,0) + DSTEP(1,IC)*(I-1)
     $                                            + DSTEP(2,IC)*(J-1)
     $                                            + DSTEP(3,IC)*K
                GRDVEC(IC,NGPC+6) = LATTICE(IC,0) + DSTEP(1,IC)*I
     $                                            + DSTEP(2,IC)*(J-1)
     $                                            + DSTEP(3,IC)*K
                GRDVEC(IC,NGPC+7) = LATTICE(IC,0) + DSTEP(1,IC)*I
     $                                            + DSTEP(2,IC)*J
     $                                            + DSTEP(3,IC)*K
                GRDVEC(IC,NGPC+8) = LATTICE(IC,0) + DSTEP(1,IC)*(I-1)
     $                                            + DSTEP(2,IC)*J
     $                                            + DSTEP(3,IC)*K
              END DO
              NGPC = NGPC + 8
            END DO
          END DO
        END DO
C
C     *** Fill grid vector with sublattice step coordinates ***
C
      ELSE IF (OPTION.EQ.'STEPS') THEN
C
C     *** Set loop index to work w.r.t. global lattice ***
C
        ISTART = CUBED*(ICUBE-1)+1
        IEND = MAX(MIN(CUBED*ICUBE,NLATTICE(1)),1)
        JSTART = CUBED*(JCUBE-1)+1
        JEND = MAX(MIN(CUBED*JCUBE,NLATTICE(2)),1)
        KSTART = CUBED*(KCUBE-1)+1
        KEND = MAX(MIN(CUBED*KCUBE,NLATTICE(3)),1)
C
C     *** Fill grid vector GRDVEC ***

        NGPC = 0
        DO K=KSTART,KEND
          DO J=JSTART,JEND
            DO I=ISTART,IEND
              NGPC = NGPC + 1
              DO IC=1,3
                GRDVEC(IC,NGPC) = LATTICE(IC,0) + DSTEP(1,IC)*(I-1)
     $                                          + DSTEP(2,IC)*(J-1)
     $                                          + DSTEP(3,IC)*(K-1)
              END DO
            END DO
          END DO
        END DO
C
CCC ADLL
C     *** Fill grid vector with sublattice step coordinates ***
C
      ELSE IF (OPTION.EQ.'SLICES') THEN
C
C     *** Set loop index to work w.r.t. global lattice ***
C
        KSTART = CUBED*(KCUBE-1)+1
        KEND = MAX(MIN(CUBED*KCUBE,NLATTICE(3)),1)
C
C     *** Fill grid vector GRDVEC ***

        NGPC = 0
        DO K=KSTART,KEND
          NGPC = NGPC + 1
          DO IC=1,3
            GRDVEC(IC,NGPC) = LATTICE(IC,0) + DSTEP(1,IC)*(ICUBE-1)
     $                                      + DSTEP(2,IC)*(JCUBE-1)
     $                                      + DSTEP(3,IC)*(K-1)
          END DO
        END DO
C
CCC ADLL
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE CUBUS ***
C
      END
