      SUBROUTINE DOSCF(KS,ORTHO,DSTO,DSTO_ME,EIGVAL,OCCN,NOMO,AOCCN,
     $                 ELCFG,MOEXC,NMOEXC,MOMOD,NMOMOD,LFOMO,HFOMO,SPIN)
C
C     Purpose: DO SCF iteration step.
C
C     History: - Creation (17.03.97, MK)
C                         (03.11.05, FJ)
C                         (10.09.07, GG)
C                         (18.07.12, AMK)
C                         (11.08.17, AC)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables:
C
C     EIGVAL: Eigenvalues of alpha/beta Kohn-Sham matrix (input).
C     FSCYC : Number of the first SCF cycle with level shifting.
C     KS    : Alpha/Beta Kohn-Sham (input) -> MO (output) matrix.
C     ORTHO : (Symmetric) orthogonalization matrix.
C
C     List of local dynamical fields:
C
C     MO: Intermediate alpha/beta MO coefficients.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'pointer.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'symmetry.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KS(DSTO,DSTO),ORTHO(DSTO_ME,DSTO_ME)
C
      INTEGER ELCFG(0:*),MOEXC(2,*)
      REAL EIGVAL(MAXSTO),OCCN(MAXSTO)
      REAL AOCCN(0:(MAXLBAS+1)**2,*),MOMOD(2,*)
C
      CHARACTER SPIN*(*)
      INTEGER HFOMO,LFOMO,NMOEXC,NMOMOD,NOMO
C
      CHARACTER*80 MOKEY,FIXKEY,KEY,STRCOMP
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: MO(:,:)
C
C     ------------------------------------------------------------------
C
      KEY = 'C'//SPIN
      KEY = STRCOMP(KEY)
C
      FIXKEY = 'COLD'//SPIN
      FIXKEY = STRCOMP(FIXKEY)

      MOKEY = 'MO'//SPIN
      MOKEY = STRCOMP(MOKEY)
C
C     *** Level-shift ***
C
      IF (MPMASTER) CALL HILLIER(KS,DSTO,EIGVAL,OCCN,'SHIFT')
C
C     *** Diagonalization of orthogonalized Kohn-Sham matrix ***
C
      USEMPF = MPP
      IF (MOBLK) THEN
        CALL DIABLK(KS,DSTO,EIGVAL(:DSTO),SPIN)
        CALL SORTOCC(EIGVAL,OCCN)
      ELSE
        CALL DIAMAT(KS,DSTO,NORTH,MATDIA,EIGVAL(:DSTO))
      END IF
      USEMPF = .FALSE.
C
C     *** Correct MO eigenvalues, if level-shift is used ***
C
      IF (MPMASTER) CALL HILLIER(KS,DSTO,EIGVAL,OCCN,'UNSHIFT')
C
      IF (MPMASTER.AND.OKS) THEN
        EIGVAL(1:NORTH) = 2.0*EIGVAL(1:NORTH)
      END IF
C
      IF (MPMASTER.AND.ABSPOT.AND.(CAPTYP.EQ.'ENERGY')) THEN
        CALL SCFIO(KS,DSTO,'WRITE',MOKEY)
      END IF
C     ------------------------------------------------------------------
C
C     *** Allocate local fields for backtransformation ***
C
      ALLOCATE(MO(MIN(DSTO,DSTO_ME),MIN(DSTO,DSTO_ME)),STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('DOSCF','ALLOCATION FAILED',1)
      END IF
C
C     *** Backtransformation of the orthonormalized MO coefficients ***
C
      MO(:,:) = 0.0
      CALL MPMULMAT(ORTHO,KS,MO,DSTO_ME,NORB,'NORMAL')
C
      IF (RESTART.AND.(SCFCYC.EQ.FSCYC).AND.
     $   (LSHIFT.OR.MOBLK.OR.ROKS)) THEN
        IF (MPMASTER) CALL SCFIO(ORTHO,DSTO,'READ',KEY)
        CALL MPMULMAT(ORTHO,MO,KS,DSTO_ME,NORB,'NORMAL')
      ELSE IF (MPMASTER) THEN
        KS(:,:) = MO(:,:)
      END IF
C
      IF (MPSLAVE) GO TO 100
C
C     *** Apply phase convention to backtransformed MO coefficients ***
C
      CALL PHASE(KS,DSTO,NORB)
C
C     *** Write current MO coefficients to tape ***
C
      IF (FERMI.OR.LSHIFT.OR.MOBLK.OR.ROKS) THEN
        IF (.NOT.ROMO) CALL SCFIO(KS,DSTO,'WRITE',KEY)
      END IF
C
C     *** Generate the requested atomic configuration ***
C
      IF (ROKS) THEN
        CALL ATOMCONF(KS,MO,DSTO,EIGVALA,OCCNB,AOCCNB,ELCFGB,
     $                LFOMOB,HFOMOB)
      END IF
C
      CALL ATOMCONF(KS,MO,DSTO,EIGVAL,OCCN,AOCCN,ELCFG,LFOMO,HFOMO)
C
C     *** Exchange molecular orbitals ***
C
      CALL EXCMOS(KS,DSTO,EIGVAL,MOEXC,NMOEXC)
C
C     *** Maintain the last electronic ***
C     *** configuration by projection  ***
C
      IF (ROMO) THEN
        CALL PRJMOS(KS,MO,ORTHO,DSTO,EIGVAL,'COLDALPHA')
      ELSE
        CALL PRJMOS(KS,MO,ORTHO,DSTO,EIGVAL,FIXKEY)
      END IF
C
C     *** Sort molecular orbitals according to Aufbau principle ***
C
      IF (ROKS) THEN
        CALL SORTMOS(KS,DSTO,EIGVAL,OCCN,HFOMO)
      ELSE
        CALL SORTMOS(KS,DSTO,EIGVAL,OCCN,NOMO)
      END IF
C
C     *** Smear occupation numbers ***
C
      CALL SMRDEN(EIGVAL,OCCN,MOMOD,NMOMOD,NORTH,NOMO,LFOMO,HFOMO)
C
C     *** Print the new set of MOs ***
C
      IF (PRTMOS) THEN
        CALL PRISCF(KS,DSTO,LLMOS,ULMOS,SPIN,'MO COEFFICIENTS')
      ELSE IF (PRTMOE) THEN
        CALL PRISCF(KS,DSTO,LLMOS,ULMOS,SPIN,'MO ENERGIES')
      END IF
C
C     *** Write modified configuration to tape ***
C
      IF (FIXCFG) THEN
        IF ((FIXLAST.AND.(SCFCYC.GE.FIXCYC)).OR.
     $      (.NOT.FIXLAST.AND.(SCFCYC.EQ.FIXCYC))) THEN
          IF (.NOT.ROMO) CALL SCFIO(KS,DSTO,'WRITE',FIXKEY)
        END IF
      END IF
C
  100 CONTINUE
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(MO,STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('DOSCF','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DOSCF ***
C
      END
