      SUBROUTINE GRDDRV(PA,PB,DSTO)
C
C     Purpose: Driver routine for grid generation and grid retrieve.
C
C     Lit.: P.J. Davis, P. Rabinowitz, Methods of Numerical
C           Integration, Academic Press, New York (1975).
C
C     History: - Creation (14.04.96, MK)
C                         (18.05.04, FJ)
C                         (24.06.07, AMK)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     GRDVEC(1,IGP): X component of grid point IGP.
C     GRDVEC(2,IGP): Y component of grid point IGP.
C     GRDVEC(3,IGP): Z component of grid point IGP.
C     GRDVEC(4,IGP): Weight of grid point IGP.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'numint.h'
C
      INTEGER DSTO
      REAL PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      LOGICAL STATED
C
C     ------------------------------------------------------------------
C
C     *** Suspend timer for FFP and SCF iteration ***
C
      IF ((.NOT.STATED('ATOM')).AND.(.NOT.STATED('FERMI'))) THEN
        IF (STATUS.EQ.'FFP') CALL TIMER('SUSPEND','FFP ITERATION')
        IF (STATUS.EQ.'SCF') CALL TIMER('SUSPEND','SCF ITERATION')
      END IF
C
C     *** Build fixed grid for topological (population) analysis ***
C
      IF ((STATUS.EQ.'ESA').OR.(STATUS.EQ.'TOPO')) THEN
C
C     *** Initialize grid generation ***
C
        CALL GRDINI
C
CCC ADLL
C     *** Prepare density matrices ***
C
        IF (.NOT.TOPCD) THEN
          CALL SCALMAT(PA,DSTO,NSTO,NSTO,'UPPWD',0.5)
          CALL SYMMAT(PA,DSTO,NSTO,'UPLOW')
          IF (OKS) THEN
            CALL SCALMAT(PB,DSTO,NSTO,NSTO,'UPPWD',0.5)
            CALL SYMMAT(PB,DSTO,NSTO,'UPLOW')
          END IF
        END IF
CCC ADLL
C
C     *** Generate topological grid ***
C
        CALL GRDGEN(PA,PB,DSTO)
        RETGRD = .TRUE.
C
C     *** Print direct topological grid ***
C
        IF (PRTGRD) CALL GRDPRT
C
CCC ADLL
C     *** Restore density matrices ***
       IF (.NOT.TOPCD) THEN
          CALL SCALMAT(PA,DSTO,NSTO,NSTO,'UPPWD',2.0)
          CALL SYMMAT(PA,DSTO,NSTO,'UPLOW')
          IF (OKS) THEN
            CALL SCALMAT(PB,DSTO,NSTO,NSTO,'UPPWD',2.0)
            CALL SYMMAT(PB,DSTO,NSTO,'UPLOW')
          END IF
        END IF
CCC ADLL
C
C     *** Store grid set ***
C
        CALL GRDDEF(TOPGSN,'STORE')
C
C     *** Load or build grid for exchange-correlation potential ***
C
      ELSE IF ((XTYP.NE.'NOFUN').OR.(CTYP.NE.'NOFUN')) THEN
C
        IF ((STATUS.EQ.'GRID').OR.(.NOT.RETGRD)) THEN
C
C     *** Initialize grid generation ***
C
          CALL GRDINI
C
C     *** Initialization for xc potential calculation ***
C
          CALL XCINI
C
C     *** Prepare density matrices ***
C
          IF (.NOT.CDXC) THEN
            CALL SCALMAT(PA,DSTO,NSTO,NSTO,'UPPWD',0.5)
            CALL SYMMAT(PA,DSTO,NSTO,'UPLOW')
            IF (OKS) THEN
              CALL SCALMAT(PB,DSTO,NSTO,NSTO,'UPPWD',0.5)
              CALL SYMMAT(PB,DSTO,NSTO,'UPLOW')
            END IF
          END IF
C
C     *** Generate new grid ***
C
          CALL GRDGEN(PA,PB,DSTO)
          RETGRD = .TRUE.
C
C     *** Reset density matrices ***
C
          IF (.NOT.CDXC) THEN
            CALL SCALMAT(PA,DSTO,NSTO,NSTO,'UPPWD',2.0)
            CALL SYMMAT(PA,DSTO,NSTO,'UPLOW')
            IF (OKS) THEN
              CALL SCALMAT(PB,DSTO,NSTO,NSTO,'UPPWD',2.0)
              CALL SYMMAT(PB,DSTO,NSTO,'UPLOW')
            END IF
          END IF
C
C     *** Store grid set ***
C
          CALL GRDDEF(XCGSN,'STORE')
C
        END IF
C
C     *** Print direct grid ***
C
        IF (DIRGRD.AND.PRTGRD) THEN
          IF ((STATUS.EQ.'SCF').OR.(STATUS.EQ.'GRID')) THEN
            CALL GRDPRT
          END IF
        END IF
C
C     *** Retrieve current (conventional) grid ***
C
        IF (.NOT.DIRGRD) THEN
          IF (RETGRD) THEN
            CALL GRDDEF(XCGSN,'LOAD')
            CALL GRDTHRS
          END IF
          CALL GRDRET
          CALL GRDDEF(XCGSN,'STORE')
        END IF
C
CCC ADLL 
C
C     *** Build fixed grid for absorbing potetial  ***
C
      ELSE IF (STATUS.EQ.'BLDCAP') THEN
C
C     *** Initialize grid generation ***
C
        CALL GRDINI
C
C     *** Generate topological grid ***
C
        CALL GRDGEN(PA,PB,DSTO)
        RETGRD = .TRUE.
C
C     *** Print direct topological grid ***
C
        IF (DIRGRD.AND.PRTGRD) CALL GRDPRT
C
        CALL GRDDEF(CAPGSN,'STORE')
C
CCC ADLL
      END IF
C
C     *** Continue timer for FFP and SCF iteration ***
C
      IF ((.NOT.STATED('ATOM')).AND.(.NOT.STATED('FERMI'))) THEN
        IF (STATUS.EQ.'FFP') CALL TIMER('CONTINUE','FFP ITERATION')
        IF (STATUS.EQ.'SCF') CALL TIMER('CONTINUE','SCF ITERATION')
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE GRDDRV ***
C
      END
