      SUBROUTINE INPCHK
C
C     Purpose: Input check.
C
C     History: - Creation (18.01.96, AMK)
C                         (22.04.17, GG)
C                         (11.06.19, RDV)
C
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'align.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'eri.h'
      INCLUDE 'flags.h'
      INCLUDE 'freq.h'
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'opt.h'
      INCLUDE 'pointer.h'
      INCLUDE 'polarizability.h'
      INCLUDE 'property.h'
      INCLUDE 'thermo.h'
      INCLUDE 'xc.h'
C
      CHARACTER*80 STRCOMP,STRING
      INTEGER IESA,ISHL
C
C     ==================================================================
C
C     *** Check general inputs settings ***
C
C     ==================================================================
C
C     *** Check input and optimization option consistency ***
C
      IF ((INPTYP.EQ.'CARTESIAN').AND.(OPTTYP.EQ.'INTERNAL')) THEN
        STRING = 'INTERNAL OPTIMIZATION ONLY WITH Z-MATRIX POSSIBLE'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
      IF (MD.AND.OPT.AND.(TRJTYP.NE.'READ STEP')) THEN
        STRING = 'MOLECULAR DYNAMICS AND OPTIMIZATION ARE INCOMPATIBLE'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
      IF ((INPTYP.NE.'CARTESIAN').AND.(TRJTYP.EQ.'READ STEP')) THEN
        STRING = 'TRAJECTORY STEP INPUT MUST BE CARTESIAN'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
      IF ((OPT.OR.FREQ).AND.(NATOM_ALL.EQ.1)) THEN
        STRING = 'NO OPTIMIZATION OR FREQUENCY ANALYSIS FOR 1 ATOM!'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Exclude simultaneous specification of MD and IRC keywords ***
C
      IF (IRC.AND.MD) THEN
        STRING = 'MOLECULAR DYNAMICS AND IRC PATH ARE INCOMPATIBLE'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check molecular dynamics bath and velocity consistency ***
C
      IF ((VELOTYP.EQ.'READIN').AND.(MDZERO.NE.'NONE')) THEN
        WRITE (STRING,5000) MDZERO
 5000   FORMAT ('READ IN VELOCITIES ALTERED FOR ',A10,' = 0 SETTING')
        CALL ERRMSG('INPCHK',STRCOMP(STRING),0)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check molecular dynamics trajectory options ***
C
      IF ((PTRJ.EQ.'PHASESPACE').AND.(NATOM.NE.2)) THEN
        STRING = 'PHASE SPACE ANALYSIS IS ONLY AVAILABLE FOR DIMERS'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
      IF (TRJGEO.AND.ALIGNMENT) THEN
        STRING = 'ALIGNMENT AND GEOMETRICAL ANALYSIS ARE INCOMPATIBLE'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Check temperature range for thermodynamical data ***
C
      IF (THERMOS) THEN
        IF (TUPPER.LT.TLOWER) THEN
          CALL ERRMSG('INPCHK','TUPPER MUST BE LARGER THAN TLOWER',1)
        ELSE IF (TDELTA.LE.0) THEN
          CALL ERRMSG('INPCHK','TDELTA MUST BE LARGER THAN ZERO',1)
        ELSE IF (TLOWER.LT.0) THEN
          CALL ERRMSG('INPCHK','TLOWER MUST BE LARGER THAN ZERO',1)
        END IF
      END IF
C
C     ==================================================================
C
C     *** Check QM input settings ***
C
      IF (QMONLY.OR.QMMM) THEN
C
C     ==================================================================
C
C     *** Check basic counters ***
C
        IF (NATOM.EQ.0) THEN
          STRING = 'NATOM COUNTER IS ZERO!'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        ELSE IF (NAUX.EQ.0) THEN
          STRING = 'NAUX COUNTER IS ZERO!'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        ELSE IF (NSTO.EQ.0) THEN
          STRING = 'NSTO COUNTER IS ZERO!'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check basis and auxiliary functions ***
C
        IF (STOPTR(NSTO,1).LT.NATOM) THEN
          STRING = 'THERE ARE ATOMS WITHOUT ORBITALS'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),0)
        ELSE IF (AUXPTR(NAUX,1).LT.NATOM) THEN
          STRING = 'THERE ARE ATOMS WITHOUT AUXILIARY FUNCTIONS'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),0)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check order of shells with respect to atoms ***
C
        DO ISHL=2,NSHL
          IF (SHLPTR(ISHL,1).LT.SHLPTR(ISHL-1,1)) THEN
            STRING = 'ORDER OF SHELLS AND ATOMS DOES NOT MATCH'
            CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
          END IF
        END DO
C
C     ------------------------------------------------------------------
C
C     *** Check multiplicity and electron numbers ***
C
        IF (OKS) THEN
          IF (MULTIP.LT.1) THEN
            CALL ERRMSG('INPCHK','MULTIPLICITY CANNOT BE 0',1)
          ELSE IF ((MULTIP - 1).GT.NELEC) THEN
            CALL ERRMSG('INPCHK','MULTIPLICITY CANNOT BE FORMED '//
     $                  'BY THE GIVEN NUMBER OF ELECTRONS',1)
          ELSE IF ((MOD(MULTIP,2).EQ.0).AND.(MOD(NELEC,2).EQ.0)) THEN
            CALL ERRMSG('INPCHK','AN EVEN MULTIPLICITY IMPLIES '//
     $                  'AN ODD NUMBER OF ELECTRONS',0)
          ELSE IF ((MOD(MULTIP,2).NE.0).AND.(MOD(NELEC,2).NE.0)) THEN
            CALL ERRMSG('INPCHK','AN ODD MULTIPLICITY IMPLIES '//
     $                  'AN EVEN NUMBER OF ELECTRONS',0)
          ELSE IF (NELEC.GT.2*NSTO) THEN
            CALL ERRMSG('INPCHK','NOT ENOUGH ORBITAL FUNCTIONS',1)
          ELSE IF (NELEC.GT.2*NAUX) THEN
            CALL ERRMSG('INPCHK','NOT ENOUGH AUXILIARY FUNCTIONS',1)
          END IF
        ELSE
          IF (MULTIP.NE.1) THEN
            CALL ERRMSG('INPCHK','CLOSED SHELL MULTIPLICITY IS 1',1)
          ELSE IF (MOD(NELEC,2).NE.0) THEN
            CALL ERRMSG('INPCHK','ODD NUMBER OF ELECTRONS IMPOSSIBLE',1)
          ELSE IF (NELEC.GT.2*NSTO) THEN
            CALL ERRMSG('INPCHK','NOT ENOUGH ORBITAL FUNCTIONS',1)
          ELSE IF (NELEC.GT.2*NAUX) THEN
            CALL ERRMSG('INPCHK','NOT ENOUGH AUXILIARY FUNCTIONS',1)
          END IF
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check grid definition for numerical integration ***
C
        IF (((XTYP.NE.'NOFUN').OR.(CTYP.NE.'NOFUN'))
     $                        .AND.(GRDTYP.EQ.'NONE')) THEN
          CALL ERRMSG('INPCHK','GRID IS NOT DEFINED',1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check for meta-GGAs or hybrids and AUXIS ***
C
        IF (CDXC.AND.LAP) THEN
          STRING = 'META-GGAS NOT AVAILABLE FOR VXCTYPE OPTION AUXIS'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
        IF (HUECKEL.AND.(EXX.OR.LAP)) THEN
          STRING = 'START GUESS IS CALCULATED WITH VXCTYPE PBE AUXIS'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),0)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check for range-separated hybrids and ERIs ***
C
        IF (HYBRID.AND.(CAMW.GT.0.0)) THEN
          IF (ERIRAM) THEN
            STRING = 'MIXED ERIS NOT AVAILABLE FOR RS HYBRIDS'
            CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
          ELSE IF (ERITYP.EQ.'NOT DIRECT') THEN
            STRING = 'CONVENTIONAL SCF NOT AVAILABLE FOR RS HYBRIDS'
            CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
          END IF
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check compatibility between MATINV and ADPTYPE option ***
C
        IF (USEMIN.AND.(ADPTYP.EQ.'ADP-I/O')) THEN
          STRING = 'MINRES ONLY COMPATIBLE WITH ADPTYPE DIRECT'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check molecular dynamics trajectory options ***
C
        IF ((LLTMOS.LT.0).OR.(ULTMOS.GT.NORB)) THEN
          WRITE (STRING,5010) LLTMOS,ULTMOS
 5010     FORMAT ('TRAJECTORY MO RANGE ',I5,' TO ',I5,' IS NOT VALID')
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
        IF (TRJNMR.AND.NICS) THEN
          STRING = 'NICS FOR BOMD SIMULATION NOT POSSIBLE'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
        IF (TRJNMR.AND.(.NOT.NMRSIG(TRJATOM))) THEN
          WRITE (STRING,5020) TRJATOM
 5020     FORMAT ('NMR SHIELDING NOT ACTIVATED FOR ATOM ',I5)
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check frequency input options ***
C
        IF (FREQ) THEN
          IF ((LAP.OR.EXX).AND.(FREQTYP.EQ.'ANALYTIC')) THEN
            CALL ERRMSG('INPCHK','NO ANALYTIC FREQUENCIES '//
     $                  'FOR META-GGAS AND HYBRID FUNCTIONALS',1)
          ELSE IF ((.NOT.CDXC).AND.(FREQTYP.EQ.'ANALYTIC')) THEN
            CALL ERRMSG('INPCHK','NO ANALYTIC FREQUENCIES FOR '//
     $                  'VXCTYPE OPTION BASIS',1)
          ELSE IF (EMBED.AND.(FREQTYP.EQ.'ANALYTIC')) THEN
            CALL ERRMSG('INPCHK','NO ANALYTIC FREQUENCIES FOR '//
     $                  'EMBEDDED SYSTEMS YET',1)
          END IF
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check polarizability input options ***
C
        IF (POLARIS) THEN
          IF ((POLORD.GT.2).AND.(POLTYP.EQ.'ANALYTIC')) THEN
            CALL ERRMSG('INPCHK','NO ANALYTIC SECOND '//
     $                  'HYPERPOLARIZABILITIES ARE AVAILABLE',1)
          ELSE IF ((LAP.OR.EXX).AND.(POLTYP.EQ.'ANALYTIC')) THEN
            CALL ERRMSG('INPCHK','NO ANALYTIC POLARIZABILITIES '//
     $                  'FOR META GGA AND HYBRID FUNCTIONALS',1)
          ELSE IF ((POLORD.GT.1).AND.(POLTYP.EQ.'NIACPKS')) THEN
            CALL ERRMSG('INPCHK','NO NIA-CPKS HYPERPOLARIZABILITIES '//
     $                  'ARE AVAILABLE',1)
          ELSE IF ((POLKEY.EQ.'DQ').AND.(POLTYP.EQ.'NUMERIC')) THEN
            CALL ERRMSG('INPCHK','NO NUMERIC DIPOLE-QUADRUPOLE '//
     $                  'POLARIZABILITIES ARE AVAILABLE',1)
          END IF
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check NMR input options ***
C
        IF ((SSCC.OR.SHIELD).AND.OKS) THEN
          STRING = 'NMR PROPERTY CALCULATION IS NOT IMPLEMENTED '//
     $             'FOR OPEN SHELL SYSTEMS'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        ELSE IF ((SSCC.OR.SHIELD).AND.EXX) THEN
          STRING = 'NMR PROPERTY CALCULATION NOT AVAILABLE FOR '//
     $             'EXACT EXCHANGE CONTAINING FUNCTIONALS'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        ELSE IF (SSCC.AND.LAP) THEN
          STRING = 'NSSCC FOR META-GGAS NOT AVAILABLE'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check topological input options ***
C
        IF (TOPCD.AND.BPA) THEN
          CALL ERRMSG('INPCHK','BADER ANALYSIS OF AUXILIARY '//
     $                         'DENSITY IS NOT AVAILABLE',1)
        END IF
C
        IF (HPA.AND.(FITTYP.NE.'ALL')) THEN
          CALL ERRMSG('INPCHK','HIRSHFELD ANALYSIS ONLY WITH '//
     $                         'FULL AUXILIARY DENSITY AVAILABLE',1)
        END IF
CCC ADLL
C     *** Avoid incompatible population analysis schemes ***
C
      IF (RTDFT.AND.(LPA.OR.MPA.OR.NPA.OR.PEM)) THEN
        STRING = 'INCOMPATIBLE POPULATION SCHEME FOR RT-TDDFT'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
      IF (RTDFT.AND.HYBRID) THEN
        STRING = 'RT-TDDFT SIMULATION ARE INCOMPATIBLE WITH HYBRIDS'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
      IF (RTDFT.AND.SPHORB) THEN
        STRING = 'SPHERICAL ORBITALS AND RT-TDDFT NOT COMPATIBLE'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
      IF (TOPCD.AND.TDEKIN) THEN
        STRING = 'TOPOLOGY AUXIS OPTION AND KINETIC NOT COMPATIBLE'
        CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
      END IF
C
CCC ADLL
C     ------------------------------------------------------------------
C
C     *** Check electronic structure analysis input options ***
C
        IESA = 0
        IF (PEM) IESA = IESA + 1
        IF (LPA) IESA = IESA + 1
        IF (MPA) IESA = IESA + 1
        IF (NPA) IESA = IESA + 1
        IF (FPA) IESA = IESA + 1
        IF (HPA) IESA = IESA + 1
        IF (VPA) IESA = IESA + 1
C
        IF (IESA.GT.1) THEN
          CALL ERRMSG('INPCHK','ONLY ONE POPULATION OR '//
     $                         'DIPOLE OPTION POSSIBLE PER RUN',1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check lattice definition for isosurface generation ***
C
        IF ((PLOTTYP.EQ.'ISOSUPPORT').AND.(LATTYP.NE.'STRUCTURED')) THEN
          STRING = 'ISOSURFACE ONLY ON STRUCTURED LATTICE'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
C     *** Check lattice definition for geometric support generation ***
C
        IF ((PLOTTYP.EQ.'GEOSUPPORT').AND.(LATTYP.NE.'STRUCTURED')) THEN
          STRING = 'GEOSUPPORT ONLY ON STRUCTURED LATTICE'
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check print options ***
C
        IF (PRTERIS.AND.(ERITYP.EQ.'DIRECT')) THEN
          STRING = 'ERI MATRICES ONLY FOR CONVENTIONAL SCF AVAILABLE'
          CALL ERRMSG('INPCHK',STRING,1)
        END IF
C
C     *** Check print range for MOs ***
C
        IF ((LLMOS.LT.0).OR.(ULMOS.GT.NORB)) THEN
          WRITE (STRING,5030) LLMOS,ULMOS
 5030     FORMAT ('MO PRINT RANGE FROM ',I5,' TO ',I5,' IS NOT VALID')
          CALL ERRMSG('INPCHK',STRCOMP(STRING),1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check for parallelization exceptions ***
C
        IF (NCPU.GT.1) THEN
          IF (NATOM.EQ.1) THEN
            STRING = 'ATOM CALCULATIONS ARE NOT PARALLELIZED!'
            CALL ERRMSG('INPCHK',STRING,1)
          END IF
C
          IF (TDDFT) THEN
            STRING = 'TD-DFT BRANCH NOT YET PARALLELIZED'
            CALL ERRMSG('INPCHK',STRING,1)
          END IF
        END IF
C
      END IF
C
C     ==================================================================
C
C     *** Check QM/MM input settings ***
C
      IF (QMMM) THEN
C
C     ==================================================================
C
C     *** Check MM force-field setting ***
C
        IF (FFIELD.EQ.'UNKNOWN') THEN
          STRING = 'FORCE-FIELD KEYWORD MISSING'
          CALL ERRMSG('INPCHK',STRING,1)
        END IF
C
      END IF
C
C     ==================================================================
C
C     *** Check MM input settings ***
C
      IF (MMONLY) THEN
C
C     ==================================================================
C
C     *** Check MM force-field setting ***
C
        IF (FFIELD.EQ.'UNKNOWN') THEN
          STRING = 'FORCE-FIELD KEYWORD MISSING'
          CALL ERRMSG('INPCHK',STRING,1)
        END IF
C
C     ------------------------------------------------------------------
C
C     *** Check Hessian setting ***
C
        IF (HESSIAN.EQ.'RESTART') THEN
          STRING = 'NO HESSIAN RESTART IN MM CALCULATIONS'
          CALL ERRMSG('INPCHK',STRING,1)
        END IF
C
      END IF
C
C     ==================================================================
C
C     *** Check RAM input settings ***
C
      IF (RAMMAX.GT.MAXRAM) THEN
        STRING = 'RAM REQUEST LARGER THAN MAXRAM PARAMETER'
        CALL ERRMSG('INPCHK',STRING,0)
      END IF
C
C     ==================================================================
C
C     *** End of SUBROUTINE INPCHK ***
C
      END
