      SUBROUTINE MOMENT(P,MOMMAX,DSTO)
C
C     Purpose: Calculation of electrostatic MOMENTs.
C
C     History: - Creation (29.07.96, ML)
C                         (02.05.01, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DCOP: Dimension of Cartesian orbitals.
C     DMOM: Dimension of multipole moments.
C     DSHL: Dimension of shell blocks.
C     DSPP: Dimension of scaling parameters.
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     DEBYE    : Dipole moment vector in Debye.
C     L{A/B}   : Orbital L quantum number.
C     LC       : Angular quantum number of moment operator.
C     MOMMAX   : Electrostatic moment maximum rank.
C     MX,MY,MZ : Angular momentum index of multipole moment.
C     NGTO{A/B}: Number of contractions.
C     P        : Density matrix.
C     SKIPSHL  : Skipping of shell combination, if true.
C
C     List of local dynamical fields:
C
C     HRRINT: Work field for horizontal recurrence relation.
C     MOMBLK: Diatomic moment integral shell block.
C     MOMSYM: Moment symbol.
C     PBLK  : Diatomic density block.
C     PEME  : Permanent electrostatic moments (electronic part).
C     PEMN  : Permanent electrostatic moments (nuclear part).
C     PEMT  : Permanent electrostatic moments (total).
C     SHLBLK: Diatomic integral shell block.
C     VRRINT: Work field for vertical recurrence relation.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'dynamics.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
C
      INTEGER DSTO
      REAL P(DSTO,DSTO)
C
      LOGICAL SKIPSHL
      CHARACTER*80 PRTSTR,STRCOMP,SYSTEM*4
      INTEGER IATOM,ISHL,JATOM,JSHL,LA,LB,LC,MX,MY,MZ,NGTOA,NGTOB
      REAL FACTOR,NUMRND,TWONPOLE
C
CCC ADLL
      REAL DEBYE(3),NUMRD
CCC ADLL
C
      INTEGER ALLOCATION,DCOP,DIMCOP,DIMSOP,DMOM,DSHL,DSPP,MOMMAX
      CHARACTER*9,ALLOCATABLE :: MOMSYM(:)
      REAL,ALLOCATABLE :: HRRINT(:,:),MOMBLK(:,:,:),PBLK(:,:),PEME(:),
     $                    PEMN(:),PEMT(:),SHLBLK(:,:),VRRINT(:,:)
C
C     ------------------------------------------------------------------
C
C     *** DATA block for multipole moment labels ***
C
      CHARACTER*12 LABEL(4)
C
      DATA LABEL /'      DIPOLE',
     $            '  QUADRUPOLE',
     $            '    OCTUPOLE',
     $            'HEXADECAPOLE'/
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      DCOP = DIMCOP(2*LBASMAX(0)+MOMMAX)
      DMOM = DIMCOP(MOMMAX)
      DSHL = DIMSOP(LBASMAX(0)+MOMMAX)
      DSPP = DIMSOP(2*LBASMAX(0)+MOMMAX)
C
      ALLOCATE(HRRINT(DCOP,DCOP),MOMBLK(DSHL,DSHL,DMOM),MOMSYM(DMOM),
     $         PBLK(DSHL,DSHL),PEME(DMOM),PEMN(DMOM),PEMT(DMOM),
     $         SHLBLK(DSHL,DSHL),VRRINT(DCOP,DSPP),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('MOMENT','ALLOCATION FAILED',1)
      END IF
C
C     *** Initialize multipole moments ***
C
      PEME(:) = 0.0
      PEMN(:) = 0.0
      PEMT(:) = 0.0
C
C     *** Initialize integral recurrence fields ***
C
      VRRINT(:,:) = 0.0
C
C     *** Generate multipole moment symbols ***
C
      CALL GENMS(MOMSYM,MOMMAX,DMOM)
C
C     *** Calculate electronic multipole moments ***
C
      DO 50 IATOM=1,NATOM
        DO 40 JATOM=IATOM,NATOM
          CALL ATOMDIS(IATOM,JATOM,RAB,RSQ)
          CALL ORGDIS(IATOM,JATOM,RAC,RBC)
          DO 30 ISHL=LLSHL(IATOM),ULSHL(IATOM)
            DO 20 JSHL=MAX(ISHL,LLSHL(JATOM)),ULSHL(JATOM)
              LA = SHLPTR(ISHL,3)
              LB = SHLPTR(JSHL,3)
              LC = MOMMAX
              CALL OSETUP(LA,LB,NGTOA,NGTOB,ISHL,JSHL)
              CALL GTOPRO(NGTOA,NGTOB,SKIPSHL)
              IF (SKIPSHL) GO TO 20
              CALL BLDPBLK(ISHL,JSHL,PBLK,P,DSHL,DSTO)
              CALL OCONT(LA,LB,LC,NGTOA,NGTOB,0,VRRINT,DCOP,DSPP)
              CALL OVRR(LA,LB,LC,HRRINT,VRRINT,DCOP,DSPP)
C
C     *** Build moment integral from basic overlap integrals ***
C
              MOMBLK(:,:,:) = 0.0
              DO 10 LC=0,MOMMAX
                CALL MHRR(LA,LB,LC,HRRINT,SHLBLK,DCOP,DSHL)
                CALL MSRR(LA,LB,LC,MOMMAX,MOMBLK,SHLBLK,DMOM,DSHL)
   10         CONTINUE
              CALL BLDMOM(LA,LB,MOMMAX,MOMBLK,PBLK,PEME,DMOM,DSHL)
C
   20       CONTINUE
   30     CONTINUE
   40   CONTINUE
   50 CONTINUE
C
C     *** Calculate nuclear multipole moments ***
C
      DO 90 LC=1,MAX(1,MOMMAX)
        DO 80 MX=0,LC
          DO 70 MY=0,LC-MX
            MZ = LC - MX - MY
            DO 60 IATOM=1,NATOM
C
              FACTOR = 1.0
C
              IF (MX.NE.0) THEN
                FACTOR = FACTOR*COORD(1,IATOM,CCSET)**MX
              END IF 
C
              IF (MY.NE.0) THEN
                FACTOR = FACTOR*COORD(2,IATOM,CCSET)**MY
              END IF
C
              IF (MZ.NE.0) THEN
                FACTOR = FACTOR*COORD(3,IATOM,CCSET)**MZ
              END IF
C
              FACTOR = ZATOM(IATOM)*FACTOR
              PEMN(COP(MX,MY,MZ)) = PEMN(COP(MX,MY,MZ)) + FACTOR
C
   60       CONTINUE
C
C     *** Calculate total multipole moments ***
C
CCC ADLL
            IF (STATUS.EQ.'RTDFT') THEN
              NUMRD = 1.E8
            ELSE 
              NUMRD = 1.E5
            END IF
CCC ADLL
            PEME(COP(MX,MY,MZ)) = -NUMRND(PEME(COP(MX,MY,MZ)),NUMRD)
            PEMN(COP(MX,MY,MZ)) = NUMRND(PEMN(COP(MX,MY,MZ)),NUMRD)
            PEMT(COP(MX,MY,MZ)) = PEMN(COP(MX,MY,MZ)) +
     $                            PEME(COP(MX,MY,MZ))
C
   70     CONTINUE
   80   CONTINUE
   90 CONTINUE
C
C     *** Store moments in molecular dynamics variables ***
C
CCC ADLL
      IF ((STATUS.EQ.'BOMD').OR.(STATUS.EQ.'RTDFT')) THEN
CCC ADLL
C
        DO 120 LC=1,MAX(1,MOMMAX)
          DO 110 MX=LC,0,-1
            DO 100 MY=LC-MX,0,-1
              MZ = LC - MX - MY
              LA = COP(MX,MY,MZ) - 1
              MDDIP(LA) = PEMT(COP(MX,MY,MZ))
  100       CONTINUE
  110     CONTINUE
  120   CONTINUE
C
C     *** Print electrostatic moments ***
C
      ELSE
C
C     *** Select unit system for electrostatic moments ***
C
        IF (UNIT(1).EQ.'ANGSTROM') THEN
          SYSTEM = 'ESU'
        ELSE
          SYSTEM = 'A.U.'
          FACTOR = 1.0
        END IF
C
C     *** Print electrostatic moment header ***
C
        WRITE (PRTSTR,5000) SYSTEM
 5000   FORMAT ('ELECTROSTATIC MOMENTS IN ',A4)
        CALL MESSAGE(1,STRCOMP(PRTSTR),0,1)
        IF (CCSET.NE.2) THEN
          CALL MESSAGE(1,'ATTENTION: NO STANDARD ORIENTATION!',0,1)
        END IF
        WRITE (OUT,5010)
 5010   FORMAT (T2,'AXIS',8X,'NUCLEAR PART',3X,'ELECTRONIC PART',
     $                    3X,'TOTAL MULTIPOLE',3X,'TOTAL TRACELESS')
        WRITE (OUT,'(T2,78("-"))')
C
C     *** Loop over electrostatic moments ***
C
        DO 150 LC=1,MAX(1,MOMMAX)
          IF (LC.GT.4) THEN
            WRITE (PRTSTR,'(I7," POLE")') 2**LC
          ELSE
            WRITE (PRTSTR,'(A)') LABEL(LC)
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
C     *** Set conversion factor for ESU system ***
C
          IF (SYSTEM.EQ.'ESU') FACTOR = ESU(LC)
C
C     *** Loop over electrostatic moment compounds ***
C
          DO 140 MX=LC,0,-1
            DO 130 MY=LC-MX,0,-1
              MZ = LC - MX - MY
C
C     *** Print electrostatic moment compounds ***
C
              WRITE (OUT,'(T2,A9,1X,E14.7,4X,E14.7,4X,E14.7,4X,E14.7)')
     $              MOMSYM(COP(MX,MY,MZ)),
     $              PEMN(COP(MX,MY,MZ))*FACTOR,
     $              PEME(COP(MX,MY,MZ))*FACTOR,
     $              PEMT(COP(MX,MY,MZ))*FACTOR,
     $              TWONPOLE(MX,MY,MZ,PEMT)*FACTOR
C
  130       CONTINUE 
  140     CONTINUE
  150   CONTINUE
C
        WRITE (OUT,'(/)')
C
C     *** Write dipole moment [Debye] into Molekel file ***
C
        IF (MOLEKEL.EQ.'FULL') THEN
          DEBYE(1) = TWONPOLE(1,0,0,PEMT)*ESU(1)
          DEBYE(2) = TWONPOLE(0,1,0,PEMT)*ESU(1)
          DEBYE(3) = TWONPOLE(0,0,1,PEMT)*ESU(1)
          CALL MKLDRV(DEBYE,DEBYE,3,'DIPOLE','NONE')
        END IF
C
        CALL DEFLUSH(OUT)
C
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(HRRINT,MOMBLK,MOMSYM,PBLK,PEME,PEMN,PEMT,
     $           SHLBLK,VRRINT,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('MOMENT','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MOMENT ***
C
      END
