      SUBROUTINE MPMULMAT(A,B,C,DMAT,N,OPTION)
C
C     Purpose: MPi MULtiplication of real quadratic MATrices.
C
C              C := A * B
C
C     History: - Creation (18.04.2003, GG)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DMAT: Dimension of matrices in the calling subroutine.
C
C     List of local variables:
C
C     A     : Left side matrix (N x N).
C     B     : Right side matrix (N x N).
C     C     : Product matrix (N x N).
C     N     : Number of rows or columns of matrix.
C     OPTION: a) NORMAL = A*B.
C             b) TRANSA = TRANSPOSE(A)*B.
C             c) TRANSB = A*TRANSPOSE(B).
CCC ADLL 
C             d) NORMALADD --> C = C + A*B
C             e) TRANSBADD --> C = C + A*B^T
CCC ADLL
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'mp_demon.h'
C
      INTEGER DMAT
      REAL A(DMAT,DMAT),B(DMAT,DMAT),C(DMAT,DMAT)
C
      CHARACTER OPTION*(*)
      INTEGER LLA,LLAK,LLB,LLBK,ULAK,ULB,ULBK
      INTEGER I,IDUM,J,K,LDAK,LDBK,LDG,LDLA,LDLB,N,NCYC
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: A_ME(:),B_ME(:),C_ME(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Serial multiplication for small matrices ***
C
      IF (.NOT.MPP.OR.(N.LT.NCPU)) THEN
        IF (MPSLAVE.AND.(N.LT.NCPU)) RETURN
        CALL MULMAT(A,B,C,DMAT,N,OPTION)
        RETURN
      END IF
C
C     *** Synchronize parallel processes ***
C
      CALL MP_SYNC
C
C     *** Start timer for parallel matrix multiplication ***
C
      CALL TIMER('START','MPI MATRIX MULTIPLICATION')
C
C     *** Allocate local vectors for matrix multiplication ***
C
      IDUM = 0
      CALL MPGETINFO(N,1,IDUM,LDG,IDUM,IDUM,IDUM)
      ALLOCATE(A_ME(N*LDG),B_ME(N*LDG),C_ME(LDG,LDG),STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.NE.0) THEN
        CALL ERRMSG('MPMULMAT','ALLOCATION FAILED',1)
      END IF
C
C     *** Distribution of matrix A and B over the processors ***
C
CCC ADLL
      IF ((OPTION.EQ.'NORMAL').OR.(OPTION.EQ.'NORMALADD')) THEN
CCC ADLL
        CALL MPDISTM(A_ME,A,LDG,DMAT,N,MASTER, 2)
        CALL MPDISTM(B_ME,B,LDG,DMAT,N,MASTER,-2)
      ELSE IF (OPTION.EQ.'TRANSA') THEN
        CALL MPDISTM(A_ME,A,LDG,DMAT,N,MASTER,-2)
        CALL MPDISTM(B_ME,B,LDG,DMAT,N,MASTER,-2)
CCC ADLL
      ELSE IF ((OPTION.EQ.'TRANSB').OR.(OPTION.EQ.'TRANSBADD')) THEN
CCC ADLL
        CALL MPDISTM(A_ME,A,LDG,DMAT,N,MASTER, 2)
        CALL MPDISTM(B_ME,B,LDG,DMAT,N,MASTER, 2)
      END IF
C
      CALL MPGETINFO(N,3,ME,LDLA,LLA,LLB,IDUM)
      LLA = LLA - 1
C
      DO NCYC=0,NCPU-1       
C
        CALL MPGETINFO(N,3,MOD(ME-NCYC+NCPU,NCPU),LDLB,LLB,ULB,IDUM)
C
C     *** BLAS level three Matrix Matrix product ***
C
CCC ADLL
        IF ((OPTION.EQ.'NORMAL').OR.(OPTION.EQ.'NORMALADD')) THEN 
CCC ADLL
          CALL DGEMM('N','N',LDLA,LDLB,N,1.0,A_ME,LDG,B_ME,N,0.0,
     $               C_ME,LDG)
        ELSE IF (OPTION.EQ.'TRANSA') THEN
          CALL DGEMM('T','N',LDLA,LDLB,N,1.0,A_ME,N,B_ME,N,0.0,
     $               C_ME,LDG)
CCC ADLL
        ELSE IF ((OPTION.EQ.'TRANSB').OR.(OPTION.EQ.'TRANSBADD')) THEN
CCC ADLL
          CALL DGEMM('N','T',LDLA,LDLB,N,1.0,A_ME,LDG,B_ME,LDG,0.0,
     $               C_ME,LDG)
        ELSE
          CALL ERRMSG('MPMULMAT','UNKNOWN OPTION FOR MPMULMAT',1)
        END IF
C
C     *** Collect local results ***
C
        IF (MPMASTER) THEN
          LLB = LLB - 1
          DO I = 1,LDLB
            DO J = 1,LDLA
CCC ADLL
              IF ((OPTION.EQ.'NORMALADD').OR.
     $            (OPTION.EQ.'TRANSBADD')) THEN
                C(J+LLA,I+LLB) = C(J+LLA,I+LLB) + C_ME(J,I)
              ELSE 
                C(J+LLA,I+LLB) = C_ME(J,I)
              END IF
CCC ADLL
            END DO
          END DO
        END IF
C
        DO K=0,NCPU-1
C
          IF (K.NE.MASTER) THEN
C
            IF (ME.EQ.K) THEN
              CALL MP_SEND(C_ME,LDG*LDG,MASTER,MP_REAL)
            ELSE IF (ME.EQ.MASTER) THEN
              CALL MP_GET(C_ME,LDG*LDG,K,MP_REAL)
              CALL MPGETINFO(N,3,K,LDAK,LLAK,ULAK,IDUM)
              CALL MPGETINFO(N,3,MOD(K-NCYC+NCPU,NCPU),
     $                       LDBK,LLBK,ULBK,IDUM)
              LLAK = LLAK - 1
              LLBK = LLBK - 1
              DO I=1,LDBK
                DO J=1,LDAK
CCC ADLL
                  IF ((OPTION.EQ.'NORMALADD').OR.
     $                (OPTION.EQ.'TRANSBADD')) THEN
                    C(J+LLAK,I+LLBK) = C(J+LLAK,I+LLBK) + C_ME(J,I)
                  ELSE
                    C(J+LLAK,I+LLBK) = C_ME(J,I)
                  END IF
CCC ADLL
                END DO
              END DO
            END IF
C
          END IF
C
        END DO
C
C     *** Cycle local parts of matrix B between processors ***
C
        IF (NCYC.LT.(NCPU-1)) CALL MPCYCMAT(B_ME,N*LDG)
C
      END DO
C
      DEALLOCATE(A_ME,B_ME,C_ME,STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.NE.0) THEN
        CALL ERRMSG('MPMULMAT','DEALLOCATION FAILED',1)
      END IF
C
C     *** Stop timer for parallel matrix multiplication ***
C
      CALL TIMER('STOP','MPI MATRIX MULTIPLICATION')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MPMULMAT ***
C
      END
