      SUBROUTINE NORMCDC(NCDCA,NCDCB,DAUX)
C
C     Purpose: Normalize charge density fitting coefficients.
C
C     History: - Creation (05.09.12, AMK)
C
C     ******************************************************************
C
C     List of local dimension:
C
C     DAUX: Dimension of auxiliary function matrix.
C
C     List of local variables:
C
C     NALPHA   : Number of alpha electrons.
C     NBETA    : Number of beta electrons.
C     NCDC{A/B}: Charge density coefficients to be normalized.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
CCC ADLL
      INCLUDE 'rtdft.h'
CCC ADLL
      INCLUDE 'scfvec.h'
C
      INCLUDE 'auxis.h'
C
      INTEGER DAUX
      REAL NCDCA(DAUX),NCDCB(DAUX)
C
      INTEGER IMO
      REAL NALPHA,NBETA
C
      LOGICAL STATED
C
C     ------------------------------------------------------------------
C
C     *** Quick return, if no fit coefficients exist ***
C
      IF (NAUX.EQ.0) RETURN
C
C     *** Calculate number of alpha and beta electrons ***
C
CCC ADLL
      IF (STATUS.EQ.'RTDFT') THEN
C
        NALPHA = NELA
        NBETA = NELB
C
      ELSE

        NALPHA = 0.0
        DO IMO=1,HFOMOA
          NALPHA = NALPHA + OCCNA(IMO)
        END DO
C
        NBETA = 0.0
        DO IMO=1,HFOMOB
          NBETA = NBETA + OCCNB(IMO)
        END DO
      END IF
CCC ADLL 
C
      USEMPF = (MPP.AND.(.NOT.STATED('ATOM')))
C
C     *** Normalize fitting coefficients ***
C
      IF (FITTING.EQ.'ANALYTICAL') THEN
C
C     *** Normalize analytical fitting coefficients ***
C
        CALL ANANORM(NCDCA,NCDCB,NALPHA,NBETA,DAUX)
C
      ELSE IF (FITTING.EQ.'NUMERICAL') THEN
C
C     *** Normalize numerical fitting coefficients ***
C
        CALL NUMNORM(NCDCA,NCDCB,NALPHA,NBETA,DAUX)
C
      ELSE
C
C     *** Unknown density fitting option ***
C
        CALL ERRMSG('NORMCDC','UNKNOWN DENSITY FITTING OPTION',1)
C
      END IF
C
CCC ADLL
      NALPHA = 0.0
      DO IMO=1,NAUX
        NALPHA = NALPHA + FCDA(IMO)*QAUX(IMO)
      END DO
CCC ADLL
      USEMPF = .FALSE.
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NORMCDC ***
C
      END
