      SUBROUTINE NOSCF(KSA,KSB,PA,PB,DSTO,DSTO_ME)
C
C     Purpose: Assign energies without SCF iteration or stop program
C              for start density only calculation.
C
C     History: - Creation (08.05.03, AMK)
C                         (27.06.08, AG)
C                         (08.02.18, GG)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables:
C
C     KSA: Alpha Kohn-Sham matrix.
C     KSB: Beta Kohn-Sham matrix.
C     PA : Alpha density matrix.
C     PB : Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'iteration.h'
      INCLUDE 'scf.h'
      INCLUDE 'xc.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KSA(DSTO,DSTO),KSB(DSTO,DSTO)
      REAL PA(DSTO_ME,DSTO_ME),PB(DSTO_ME,DSTO_ME)
C
      REAL NUCEXT,NUCREP
CCC ADLL
      LOGICAL SUCCESS
      INTEGER NOPTION
C
      CHARACTER*80 OPTIONS(MAXOPT)
CCC ADLL
C
C
C     ------------------------------------------------------------------
C
C     *** Stop program if only a start guess was requested ***
C
      IF (GUESSONLY) THEN
C
C     *** Store start density on memory file ***
C
        CALL NEWDRV('NEW INPUT FILE')
C
C     *** Stop timer for SCF iteration ***
C
        IF (STATUS.EQ.'SCF') CALL TIMER('STOP','SCF ITERATION')
C
C     *** Close temporary files (scratch files) ***
C
        CALL FILEIO('CLOSE TEMPORARY FILES')
C
C     *** Print total program execution time ***
C
        CALL TIMER('STATISTICS','FINAL TIMING')
C
C     *** Close permanent files (input/output files) ***
C
        CALL FILEIO('CLOSE PERMANENT FILES')
C
C     *** Stop program execution (normal program termination) ***
C
        CALL MP_END
C
        STOP 'Normal program termination'
C
      END IF
C
C     *** Hartree energy ***
C
      EHARTREE = EUPPER
C
C     *** Fock exchange energy ***
C
      CALL EXXSCF(KSA,KSB,PA,PB,DSTO,DSTO_ME)
C
C     *** Set flags ***
C
      SCF = .FALSE.
      GGFUN = 'GUESS'
      CONVERGED = .TRUE.
C
CCC ADLL
      IF (RTDFT) THEN
        CALL FINDKEY('MOEXCHANGE',SUCCESS)
        CALL DEFAULT('MOEXCHANGE',SUCCESS)
        IF (SUCCESS) THEN
          CALL FINDOPT('MOEXCHANGE',OPTIONS,NOPTION)
          CALL DECODE('MOEXCHANGE',OPTIONS,NOPTION)
        END IF
      END IF
CCC ADLL
C
C     *** Calculate restart energy ***
C
      ESCF = EHARTREE + EXC + EVDW + NUCREP()
C
C      *** Add energy from external field ***
C
      IF (EFIELD.AND.THZ) ESCF = ESCF + NUCEXT()
C
C     *** Write energy only output ***
C
      WRITE (OUT,5000)
 5000 FORMAT (/,T2,'ENERGY CALCULATION FROM RESTART MOS (NO SCF):')
      WRITE (OUT,5010) ENUCLEUS,ECOULOMB,EHARTREE
 5010 FORMAT (/,T2,'ELECTRONIC CORE ENERGY      = ',F20.9,
     $        /,T2,'ELECTRONIC COULOMB ENERGY   = ',F20.9,
     $        /,T2,'ELECTRONIC HARTREE ENERGY   = ',F20.9)
      WRITE (OUT,5020) EXSUM,ECSUM,EXC
 5020 FORMAT (/,T2,'EXCHANGE ENERGY             = ',F20.9,
     $        /,T2,'CORRELATION ENERGY          = ',F20.9,
     $        /,T2,'EXCHANGE-CORRELATION ENERGY = ',F20.9)
      WRITE (OUT,5030) EHARTREE+EXC,NUCREP()
 5030 FORMAT (/,T2,'ELECTRONIC SCF ENERGY       = ', F20.9,
     $        /,T2,'NUCLEAR-REPULSION ENERGY    = ', F20.9)
      IF (EVDW.NE.0.0) THEN
        WRITE (OUT,5040) EVDW
 5040   FORMAT (T2,'EMPIRICAL DISPERSION ENERGY = ', F20.9)
      END IF
      IF (EFIELD.AND.THZ) THEN
        WRITE (OUT,5050) NUCEXT()
 5050   FORMAT (T2,'NUCLEAR EXTERNAL ENERGY     = ', F20.9)
      END IF
      WRITE (OUT,5060) ESCF
 5060 FORMAT (T2,'TOTAL ENERGY                = ',F20.9,/)
C
C     *** Flush buffer ***
C
      CALL DEFLUSH(OUT)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NOSCF ***
C
      END
