      SUBROUTINE NUMXC(KSA,KSB,PA,PB,DSTO,DSTO_ME)
C
C     Purpose: Numerical integration of the exchange-correlation 
C              potential matrix and energy.
C
C     History: - Creation (26.11.04, AMK)
C                         (19.04.07, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C     KSA  : Alpha work matrix -> Alpha Kohn-Sham matrix.
C     KSB  : Beta work matrix -> Beta Kohn-Sham matrix.
C     PA   : Alpha density matrix -> Alpha exchange-correlation matrix.
C     PB   : Beta density matrix -> Beta exchange-correlation matrix.
C     XCDCA: Alpha exchange-correlation potential vector.
C     XCDCB: Beta exchange-correlation potential vector.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'xc.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KSA(DSTO,DSTO),KSB(DSTO,DSTO)
      REAL PA(DSTO_ME,DSTO_ME),PB(DSTO_ME,DSTO_ME)
C
C     ------------------------------------------------------------------
C
C     *** Calculate DFT exchange-correlation contribution ***
C
      IF ((XTYP.NE.'NOFUN').OR.(CTYP.NE.'NOFUN')) THEN
C
C     *** Initialize Kohn-Sham matrices ***
C
        KSA(:,:) = 0.0
        KSB(:,:) = 0.0
C
C     *** Build charge density coefficients ***
C
        IF (MPMASTER) THEN
          CALL BLDCDC(XCDCA,'BUILD ALPHA')
          IF (OKS) CALL BLDCDC(XCDCB,'BUILD BETA')
        END IF
C
        IF (MPP.AND.CDXC) THEN
          CALL MP_SEND_ALL(XCDCA,NALL,MASTER,MP_REAL)
          IF (OKS) CALL MP_SEND_ALL(XCDCB,NALL,MASTER,MP_REAL)
        END IF
C
C     *** Initialization of exchange-correlation functionals ***
C
        CALL XCINI
C
C     *** Prepare density matrices for numerical integration ***
C
        IF (MPMASTER.AND.(.NOT.CDXC)) THEN
          KSA(:,:) = PA(:,:)
          CALL SCALMAT(KSA,DSTO,NSTO,NSTO,'UPPWD',0.5)
          CALL SETMAT(KSA,DSTO,NSTO,NSTO,'LOWER',0.0)
          IF (OKS) THEN
            KSB(:,:) = PB(:,:)
            CALL SCALMAT(KSB,DSTO,NSTO,NSTO,'UPPWD',0.5)
            CALL SETMAT(KSB,DSTO,NSTO,NSTO,'LOWER',0.0)
          END IF
        END IF
C
C     *** Send density matrices to slave nodes ***
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(KSA,DSTO*DSTO,MASTER,MP_REAL)
          IF (OKS) CALL MP_SEND_ALL(KSB,DSTO*DSTO,MASTER,MP_REAL)
        END IF
C
C     *** Perform a direct numerical integration ***
C
        IF (DIRGRD) THEN
C
          CALL NUMXCDIR(KSA,KSB,XCDCA,XCDCB,DSTO)
C
C     *** Perform a conventional numerical integration ***
C
        ELSE IF (MPMASTER) THEN
C
          CALL NUMXCCON(KSA,KSB,XCDCA,XCDCB,DSTO)
C
        END IF
C
C     *** Transform approximated exchange-correlation potential ***
C
        IF (CDXC) THEN
C
          CALL GETCXC(EIGAUX,VXCDCA,VXCDCB,XCDCA,XCDCB,XCD0A,XCD0B)
C
CCC ADLL
C          IF (MPMASTER.AND.(STATUS.EQ.'SCF')) THEN
          IF (MPMASTER.AND.
     $       ((STATUS.EQ.'SCF').OR.(STATUS.EQ.'RTDFT'))) THEN
CCC ADLL
            CALL XCOUT(KSA,KSB,PA,DSTO)
          END IF
C
          PA(:,:) = 0.0
          IF (OKS) PB(:,:) = 0.0
C
C     *** Symmetrize exchange-correlation potential matrix ***
C
        ELSE IF (MPMASTER) THEN
C
          CALL SYMMAT(KSA,DSTO,NSTO,'LOWUP')
          IF (OKS) CALL SYMMAT(KSB,DSTO,NSTO,'LOWUP')
C
CCC ADLL
C          IF (STATUS.EQ.'SCF') CALL XCOUT(KSA,KSB,PA,DSTO)
          IF ((STATUS.EQ.'SCF').OR.(STATUS.EQ.'RTDFT')) THEN
            CALL XCOUT(KSA,KSB,PA,DSTO)
          END IF
CCC ADLL
C
          PA(:,:) = KSA(:,:)
          IF (OKS) PB(:,:) = KSB(:,:)
C
        END IF
C
C     *** Build hybrid exchange-correlation potential matrix ***
C
        IF (MPMASTER.AND.EXX) THEN
          CALL MIXIO(KSA,DSTO,'READ','EXXALPHA')
          KSA(:,:) = KSA(:,:) + PA(:,:)
          PA(:,:) = KSA(:,:)
          IF (OKS) THEN
            CALL MIXIO(KSB,DSTO,'READ','EXXBETA')
            KSB(:,:) = KSB(:,:) + PB(:,:)
            PB(:,:) = KSB(:,:)
          END IF
        END IF
C
C     *** No DFT exchange-correlation contribution requested ***
C
      ELSE IF (MPMASTER) THEN
C
        IF (EXX) THEN
          PA(:,:) = KSA(:,:)
          PB(:,:) = KSB(:,:)
        ELSE
          PA(:,:) = 0.0
          PB(:,:) = 0.0
          KSA(:,:) = 0.0
          KSB(:,:) = 0.0
        END IF
C
        XCDCA(:) = 0.0
        XCDCB(:) = 0.0
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NUMXC ***
C
      END
