      SUBROUTINE VICINITY(ICUBE,JCUBE,KCUBE,POINTS,OPTION)
C
C     Purpose: Build neighborhood of sublattice.
C
C     History: - Creation (09.12.96, ML)
C                         (19.05.97, ML)
C
C     ******************************************************************
C
C     List of local variables:
C
C     CENTER: Center of current cube.
C     DIST  : Distance between ORIGIN and IATOM.
C     ORIGIN: Origin of current cube.
C     POINT : Point coordinates.
C     RAYON : Radius of sphere containing current cube.
C     OPTION: a) OPTION = CUBES, neighborhood for sublattice cubes.
C             b) OPTION = POINTS, neighborhood for sublattice point.
C             c) OPTION = STEPS, neighborhood for sublattice steps.
CCC ADLL
C             d) OPTION = SLICES, neighborhood for sublattice slices. 
C             e) OPTION = ALL, neighborhood with all atoms.
CCC ADLL
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'grid.h'
      INCLUDE 'marching.h'
      INCLUDE 'molecule.h'
      INCLUDE 'plot.h'
C
      REAL POINTS(4,MAXGP)
C
      CHARACTER*(*) OPTION
      INTEGER IEND,ISTART,JEND,JSTART,KEND,KSTART
      INTEGER I,IATOM,ICUBE,IPOINT,JCUBE,KCUBE
      REAL DIST,ICENTER,JCENTER,KCENTER,RAYON
C
      REAL CENTER(3),ORIGIN(3)
C
C     ------------------------------------------------------------------
C
C     *** Calculate neighborhood for cubes and steps ***
C
CCC ADLL
      IF ((OPTION.EQ.'CUBES').OR.
     $    (OPTION.EQ.'STEPS').OR.(OPTION.EQ.'SLICES')) THEN
CCC ADLL
C
C     *** Set loop index to work w.r.t. global mesh ***
C
        IF (OPTION.EQ.'CUBES') THEN
          ISTART = ICUBE
          IEND = MAX(MIN(ICUBE+1,NLATTICE(1)-1),1)
          JSTART = JCUBE
          JEND = MAX(MIN(JCUBE+1,NLATTICE(2)-1),1)
          KSTART = KCUBE
          KEND = MAX(MIN(KCUBE+3,NLATTICE(3)-1),1)
          ICENTER = (ISTART + IEND)/2.0 + 0.5
          JCENTER = (JSTART + JEND)/2.0 + 0.5
          KCENTER = (KSTART + KEND)/2.0 + 0.5
        ELSE IF (OPTION.EQ.'STEPS') THEN
          ISTART = CUBED*(ICUBE-1)+1
          IEND = MAX(MIN(CUBED*ICUBE,NLATTICE(1)),1)
          JSTART = CUBED*(JCUBE-1)+1
          JEND = MAX(MIN(CUBED*JCUBE,NLATTICE(2)),1)
          KSTART = CUBED*(KCUBE-1)+1
          KEND = MAX(MIN(CUBED*KCUBE,NLATTICE(3)),1)
          ICENTER = (ISTART + IEND)/2.0
          JCENTER = (JSTART + JEND)/2.0
          KCENTER = (KSTART + KEND)/2.0
        END IF
C
C     *** Calculate origin and center of cube ***
C
        DO I=1,3
          CENTER(I) = LATTICE(I,0) + DSTEP(1,I)*(ICENTER - 1) +
     $                               DSTEP(2,I)*(JCENTER - 1) +
     $                               DSTEP(3,I)*(KCENTER - 1)
          ORIGIN(I) = LATTICE(I,0) + DSTEP(1,I)*(ISTART - 1) +
     $                               DSTEP(2,I)*(JSTART - 1) +
     $                               DSTEP(3,I)*(KSTART - 1)
        END DO
C
C     *** Calculate radius of sphere embedding cube ***
C
        RAYON = SQRT((CENTER(1) - ORIGIN(1))**2 +
     $               (CENTER(2) - ORIGIN(2))**2 +
     $               (CENTER(3) - ORIGIN(3))**2)
C
C     *** Get neighbors ***
C
        NNEIGH = 0
C
        DO IATOM=1,NATOM
C
C     *** Calculate distance between atom and center of cube ***
C
          DIST = SQRT((COORD(1,IATOM,CCSET) - CENTER(1))**2 +
     $                (COORD(2,IATOM,CCSET) - CENTER(2))**2 +
     $                (COORD(3,IATOM,CCSET) - CENTER(3))**2)
C
C     *** If atom close enough then add it to the neighbors ***
C
          IF (DIST.LE.(RAYON + SQRT(ATOMRAD(IATOM)))) THEN
            NNEIGH = NNEIGH + 1
            NEIGH(NNEIGH) = IATOM
          END IF
C
        END DO
C
C     *** Get neighbors for point ***
C
      ELSE IF (OPTION.EQ.'POINTS') THEN
C
C     *** Calculate center of point distribution ***
C
        CENTER = 0.0
C
        DO I=1,3
          DO IPOINT=1,ICUBE
            CENTER(I) = CENTER(I) + POINTS(I,IPOINT)
          END DO
          CENTER(I) = CENTER(I)/ICUBE
        END DO
C
C     *** Calculate radius of embedding sphere ***
C
        RAYON = 0.0
C
        DO IPOINT=1,ICUBE
          DIST = SQRT((POINTS(1,IPOINT) - CENTER(1))**2 +
     $                (POINTS(2,IPOINT) - CENTER(2))**2 +
     $                (POINTS(3,IPOINT) - CENTER(3))**2)
          IF (DIST.LT.RAYON) RAYON = DIST
        END DO
C
C     *** Get neighbors ***
C
        NNEIGH = 0
C
        DO IATOM=1,NATOM
C
C     *** Calculate distance between atom and point ***
C
          DIST = SQRT((COORD(1,IATOM,CCSET) - CENTER(1))**2 +
     $                (COORD(2,IATOM,CCSET) - CENTER(2))**2 +
     $                (COORD(3,IATOM,CCSET) - CENTER(3))**2)
C
C     *** If atom close enough then add it to the neighbors ***
C
          IF (DIST.LE.(RAYON + SQRT(ATOMRAD(IATOM)))) THEN
            NNEIGH = NNEIGH + 1
            NEIGH(NNEIGH) = IATOM
          END IF
C
        END DO
C
C     *** Set full neighbor list ***
C
      ELSE IF (OPTION.EQ.'ALL') THEN
C
        NNEIGH = NATOM
C
        DO IATOM=1,NATOM
          NEIGH(IATOM) = IATOM
        END DO
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE VICINITY ***
C
      END
