      SUBROUTINE FOCK(FA,FB,PA,PB,DSTO,DSTO_ME)
C
C     Purpose: Calculate analytic Fock exchange potential.
C
C     History: - Creation (09.06.11, DMR & AMK)
C                         (01.04.18, FAD)
C
C     ******************************************************************
C
C     List of local variables:
C
C     FA: Alpha Work matrix -> Alpha Fock potential.
C     FB: Beta work matrix -> Beta Fock potential.
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C      
      INCLUDE 'parameter.h'
C      
      INCLUDE 'flags.h'
      INCLUDE 'mp_demon.h'
CCC ADLL 
      INCLUDE 'scfvec.h'
CCC ADLL
      INCLUDE 'xc.h'
C
      INTEGER DSTO,DSTO_ME
      REAL FA(DSTO,DSTO),FB(DSTO,DSTO)
      REAL PA(DSTO_ME,DSTO_ME),PB(DSTO_ME,DSTO_ME)
C
      CHARACTER*80 TIMSTR
      REAL TRACE
C
C     ------------------------------------------------------------------
C
C     *** Quick return if Fock potential is not requested ***
C
      IF (.NOT.EXX) RETURN
C
C     *** Initialize timer for Fock potential calculation ***
C
      IF (MPP) THEN
        TIMSTR = 'MPI EXX POTENTIAL'
      ELSE
        TIMSTR = 'EXX POTENTIAL'
      END IF
C
      CALL TIMER('START',TIMSTR)
C
C     *** Store pure density matrices on tape ***
C
      IF (MPMASTER) THEN
        CALL SCFIO(PA,DSTO,'WRITE','PALPHA')
        IF (OKS) CALL SCFIO(PB,DSTO,'WRITE','PBETA')
      END IF
C
CCC ADLL
C     *** Build real density matrix in case of RT-TD-DFT ***
C
      IF (STATUS.EQ.'RTDFT') THEN
        IF (MPMASTER) FA(:,:) = PA(:,:)
        CALL TDOCCN(FA,OCCNA,HFOMOA,DSTO,DSTO_ME,'ALPHA','BUILDP')
        IF (OKS) THEN
          CALL TDOCCN(FB,OCCNB,HFOMOB,DSTO,DSTO_ME,'BETA','BUILDP')
        END IF
      END IF
C
CCC
C     *** Calculate analytic Fock matrix ***
C
      IF (ERITYP.EQ.'DIRECT') THEN
        IF (ERIRAM) THEN
          CALL FOCKCON(FA,PA,DSTO,DSTO_ME)
          IF (OKS) CALL FOCKCON(FB,PB,DSTO,DSTO_ME)
        ELSE
          CALL FOCKDIR(FA,PA,DSTO,DSTO_ME,.FALSE.)
          IF (OKS) CALL FOCKDIR(FB,PB,DSTO,DSTO_ME,.FALSE.)
        END IF
      ELSE
        CALL FOCKCON(FA,PA,DSTO,DSTO_ME)
        IF (OKS) CALL FOCKCON(FB,PB,DSTO,DSTO_ME)
      END IF
C
      IF (MPMASTER) THEN
C
C     *** Scale Fock matrices for hybrid functionals ***
C
        IF (HYBRID.AND.(PEXX.LT.1.0)) THEN
          CALL SCALMAT(FA,DSTO,DSTO,DSTO,'TOTAL',PEXX)
          IF (OKS) CALL SCALMAT(FB,DSTO,DSTO,DSTO,'TOTAL',PEXX)
        END IF
C
C     *** Load pure density matrices from tape ***
C
        CALL SCFIO(PA,DSTO,'READ','PALPHA')
        IF (OKS) CALL SCFIO(PB,DSTO,'READ','PBETA')
C
C     *** Calculate exact-exchange (Fock) energy ***
C
        EEXX = 0.5*TRACE(PA,FA,DSTO,DSTO,'LOWERA')
        IF (OKS) EEXX = EEXX + 0.5*TRACE(PB,FB,DSTO,DSTO,'LOWERA')
C
      END IF
C
C     *** Stop timer for Fock potential calculation ***
C
      CALL TIMER('STOP',TIMSTR)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE FOCK ***
C
      END
