      SUBROUTINE GRAMMOZ(VECTORS,S,NORT,DSTO)
C
C     Purpose: Modified Gram-Schmidt orthogonalization of VECTORS. The 
C              first NORT entries of VECTORS are already orthogonalized.
C
C     Lit.: G.L.G. Sleijpen, H.A. Vorst, SIAM Review 42, 267 (2000)
C
C     History: - Creation (02.09.20, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of vector space.
C
C     List of local variables:
C
C     NORT   : Column number of vector to be orthogonalized.
C     VECTORS: Field with vectors to be orthogonalized.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'mp_demon.h'
      INCLUDE 'tolerance.h'
C
      REAL EPS
      PARAMETER (EPS = 0.25)
C
      INTEGER DSTO
      REAL S(DSTO,DSTO)
      REAL VECTORS(DSTO,DSTO)
C      COMPLEX (KIND=8) VECTORS(DSTO,DSTO)
C
      INTEGER IORT,ISTO,JSTO,LLORT_ME,NORT,ULORT_ME
      REAL TAU
      REAL NORM, PROD
C
      INTEGER ALLOCATION
C      COMPLEX (KIND=8),ALLOCATABLE :: WORK(:)
      REAL,ALLOCATABLE :: WORK(:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate workng vector ***
C
      ALLOCATE(WORK(DSTO),STAT=ALLOCATION)
C
      WORK(:) = 0.0
C
C     *** Calculate parallel loop limits ***
C
      CALL MPLIMITS(NCPU,ME,1,1,NORT-1,LLORT_ME,ULORT_ME,(/0/),0)
C
C     *** Initialize vector length ***
C
      TAU = 0.0
      DO ISTO=1,DSTO
        DO JSTO=1,DSTO
        TAU = TAU + S(ISTO,JSTO)*
     $             VECTORS(ISTO,NORT)*VECTORS(JSTO,NORT)
        END DO
      END DO
      IF (NORT.EQ.1) THEN
        VECTORS(:,NORT) = VECTORS(:,NORT)/SQRT(TAU)
        IF (MPP) CALL MP_SEND_ALL(VECTORS(:,NORT),DSTO,MASTER,MP_REAL)
        GO TO 100
      END IF
C
C     *** Loop over all orthogonalized vectors ***
C
      DO IORT=LLORT_ME,ULORT_ME
        NORM = 0.0
        PROD = 0.0
        DO ISTO=1,DSTO
          DO JSTO=1,DSTO
           NORM = NORM + S(ISTO,JSTO)*
     $             VECTORS(ISTO,IORT)*VECTORS(JSTO,IORT)
            PROD = PROD + S(ISTO,JSTO)*
     $             VECTORS(ISTO,NORT)*VECTORS(JSTO,IORT)
          END DO
        END DO

        IF (REAL(NORM).GT.TINIEST) THEN
          WORK(:) = WORK(:) - PROD/NORM*VECTORS(:,IORT)
        END IF
      END DO
C
      CALL MP_SUM(WORK,WORK,DSTO,MP_REAL)
C      CALL MP_SUM(REAL(WORK),REAL(WORK),DSTO,MP_REAL)
C      CALL MP_SUM(AIMAG(WORK),AIMAG(WORK),DSTO,MP_REAL) 
C
      IF (MPMASTER) THEN
        VECTORS(:,NORT) = VECTORS(:,NORT) + WORK(:)
C
        NORM = 0.0
        DO ISTO=1,DSTO
          DO JSTO=1,DSTO
          NORM = NORM + S(ISTO,JSTO)*
     $               VECTORS(ISTO,NORT)*VECTORS(JSTO,NORT)
         END DO
        END DO
        NORM = SQRT(NORM)
        VECTORS(:,NORT) = VECTORS(:,NORT)/NORM
      END IF
  
      IF (MPP) THEN
        CALL MP_SEND_ALL(VECTORS(:,NORT),DSTO,MASTER,MP_REAL)
      END IF

C     *** Deallocate workng vector ***
C
100   CONTINUE
C
      DEALLOCATE(WORK,STAT=ALLOCATION)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE GRAM ***
C
      END

