      SUBROUTINE MPI_MAGMA_INIT(DSTO,DSTO_ME)
C
C     History: - Creation (01.09.22, KHA)
C
C     ******************************************************************
C     List of local variables: 
C
C     ------------------------------------------------------------------
C
      use iso_c_binding
      use magma
      IMPLICIT NONE
C
      include 'mp_demon.h'
      include 'parameter.h'
      include 'mp_system.h'

      INTEGER(kind=4) DSTO,DSTO_ME
      INTEGER ALLOCATION
      integer(kind=4)                            :: i 
      integer(kind=4)                            :: code
      integer(kind=4)                            :: rank
      integer(kind=4)                            :: nb_procs
      integer(kind=4)                            :: size_MPI_COMM_GPU
      integer(kind=4), allocatable, dimension(:) :: proc_MPI_COMM_GPU
      integer(kind=4)                            :: world_group
      integer(kind=4)                            :: id_gpu
      integer(kind=4)                            :: id_node
      integer(kind=4)                            :: nb_procspnode

      call MPI_COMM_RANK(MPI_COMM_WORLD,rank,code)
      call MPI_COMM_SIZE(MPI_COMM_WORLD,nb_procs,code)
C
C     number of available GPU for the simulation
C
      ngpu = nb_procs / corepgpu
      size_MPI_COMM_GPU = ngpu
C
C     creates MPI_COMM_GPU communicator
C
      allocate(proc_MPI_COMM_GPU(size_MPI_COMM_GPU))

      do i=1,ngpu
                proc_MPI_COMM_GPU(i) = (i-1) * corepgpu
      end do

      call MPI_COMM_GROUP(MPI_COMM_WORLD, world_group, code)
      call MPI_GROUP_INCL(world_group, size_MPI_COMM_GPU,
     $                    proc_MPI_COMM_GPU, gpugrp, code)
      call MPI_COMM_CREATE(MPI_COMM_WORLD, gpugrp, MPI_COMM_GPU, code)

      if (MPI_COMM_GPU /= MPI_COMM_NULL) then
C
C     within the MPI_COMM_GPU communicator
C     number of MPI processes per node, node ID, GPU ID
C     and stop and start indexes for the columns are calculated
C
              nb_procspnode = corepgpu * gpupnode
              id_node = rank / nb_procspnode
              id_gpu  = (rank / corepgpu) - id_node * gpupnode

              istart_gpu = ( DSTO * rank ) / nb_procs + 1
              if ( rank == (nb_procs - corepgpu) ) then
                      iend_gpu = DSTO
              else
                      iend_gpu = ( (rank + corepgpu) * DSTO ) / nb_procs
              end if
              slice_gpu = iend_gpu - istart_gpu + 1

C
C     MAGMA, GPU and MAGMA queues are set
C
              call magmaf_init()
              call magmaf_setdevice(id_gpu)
              call magmaf_getdevice(id_gpu)
              call magmaf_queue_create(id_gpu, magma_queue)

      end if
C             
C     *** Deallocation ***
C
      deallocate(proc_MPI_COMM_GPU,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('MPI_MAGMA_INIT','ALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MPI_MAGMA_INIT ***
C
      END
