      SUBROUTINE TAYLORMAGMA(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
C
C     History: - Creation (10.08.21, KHA)
C
C     ******************************************************************
C     List of local variables: 
C
C     ------------------------------------------------------------------
C
      use iso_c_binding
      use magma
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'slp.h'

      INTEGER(kind=4) DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME),DT
      INTEGER ALLOCATION

C     square of DTSO
      integer(kind=4) :: DSTO_square
C
C     factor for the magmablasf_zgeadd and magmablasf_zgemm subroutine such as
C     dB = alpha * dA + dB and
C     dC = alpha * dA * dB + beta
      complex(kind=8) :: alpha_add
      complex(kind=8) :: alpha_prod
      complex(kind=8) :: beta_prod
C     exit error info 
      integer(kind=4) :: info
      integer(kind=4) :: m,n
      complex(kind=8) :: In

      complex(kind=8), allocatable, dimension(:,:) :: hKC
      complex(kind=8), allocatable, dimension(:,:) :: hPC
      complex(kind=8), allocatable, dimension(:,:) :: hID
      integer(kind=8)  :: dID
      integer(kind=8)  :: dKC
      integer(kind=8)  :: dPC
      integer(kind=8)  :: dZn
      integer(kind=8)  :: dSEIGC
      integer(kind=8)  :: dTemp
C     setting the variables
      DSTO_square = DSTO * DSTO
      alpha_prod = 1.
      beta_prod = 0.
C
C     *** Allocation ***
C
      allocate(hKC(DSTO,DSTO))
      allocate(hPC(DSTO,DSTO))
      allocate(hID(DSTO,DSTO))
      hID(:,:) = 0
      do m=1,DSTO
         hID(m,m) = 1.0
      enddo
      info = magmaf_zmalloc( dID, DSTO_square )
      info = magmaf_zmalloc( dKC, DSTO_square )
      info = magmaf_zmalloc( dPC, DSTO_square )
      info = magmaf_zmalloc( dZn, DSTO_square )
      info = magmaf_zmalloc( dSEIGC, DSTO_square )
      info = magmaf_zmalloc( dTemp, DSTO_square )

C     Merge and scale real and imaginary parts of the KS matrix 
      hKC(:,:) = cmplx(KI(:,:),-KR(:,:))*DT
C     Merge real and imaginary parts of the propagator matrix 
      hPC(:,:) = cmplx(PR(:,:),PI(:,:))

C     The ID, KC and PC matrices are sent from the host to the GPU
C     device
      call magmaf_zsetmatrix(DSTO, DSTO, hID, DSTO,
     $ dID, DSTO, magma_queue)
      call magmaf_zsetmatrix(DSTO, DSTO, hKC, DSTO,
     $ dKC, DSTO, magma_queue)
      call magmaf_zsetmatrix(DSTO, DSTO, hPC, DSTO,
     $ dPC, DSTO, magma_queue)

C
C     *** Matrix exponentiation ***
C
      call magmablasf_zlacpy('MagmaFull', DSTO, DSTO, dId, DSTO,
     $ dSEIGC, DSTO, magma_queue)
      call magmablasf_zlacpy('MagmaFull', DSTO, DSTO, dId, DSTO,
     $ dTemp, DSTO, magma_queue)

      In = 1
      do n=1,NEXPM
         In = In * n
         call magmablasf_zlacpy('MagmaFull', DSTO, DSTO, dTemp,
     $        DSTO, dZn, DSTO, magma_queue)
         call magmablasf_zgemm(MagmaNoTrans, MagmaNoTrans,
     $        DSTO, DSTO, DSTO, alpha_prod, dKC, DSTO, dZn,
     $        DSTO, beta_prod, dTemp, DSTO, magma_queue)
         alpha_add = 1. / In
         call magmablasf_zgeadd(DSTO, DSTO, alpha_add,
     $        dTemp, DSTO, dSEIGC, DSTO, magma_queue)
      enddo

C      
C     *** parallel matrix multiplication P(t+dt) = U*P(t)*U+ ***
C
      call magmablasf_zgemm(MagmaNoTrans, MagmaConjTrans,
     $ DSTO, DSTO, DSTO, alpha_prod, dPC, DSTO, dSEIGC,
     $ DSTO, beta_prod, dTemp, DSTO, magma_queue)

      call magmablasf_zgemm(MagmaNoTrans, MagmaNoTrans,
     $ DSTO, DSTO, DSTO, alpha_prod, dSEIGC, DSTO, dTemp,
     $ DSTO, beta_prod, dPC, DSTO, magma_queue)

C     The PC is sent back from the GPU device
C     to the host
      call magmaf_zgetmatrix(DSTO, DSTO,
     $ dPC, DSTO, hPC, DSTO, magma_queue)

      PR(:,:) = REAL(hPC(:,:))
      PI(:,:) = AIMAG(hPC(:,:))

C     *** Deallocation ***
C
      deallocate(hKC,hPC,hID,STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('EXPSCALA','ALLOCATION FAILED',1)
      END IF

      info = magmaf_free(dID)
      info = magmaf_free(dKC)
      info = magmaf_free(dPC)
      info = magmaf_free(dZn)
      info = magmaf_free(dSEIGC)
      info = magmaf_free(dTemp)

C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TAYLORMAGMA ***
C
      END

