      SUBROUTINE TAYLORMAGMA_MULTI(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
C
C     History: - Creation (01.09.22, KHA)
C
C     ******************************************************************
C     List of local variables: 
C
C     ------------------------------------------------------------------
C
      use iso_c_binding
      use magma
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      include 'mp_system.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'slp.h'

      INTEGER(kind=4) DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME),DT
      INTEGER ALLOCATION

C     square of DTSO
      integer(kind=4)                            :: DSTO_square
C     DTSO * slice_gpu
      integer(kind=4)                            :: DSTO_slice

      integer(kind=4)                            :: i, k
      integer(kind=4)                            :: code
      integer(kind=4)                            :: rank
      integer(kind=4)                            :: nb_procs
      integer(kind=4), allocatable, dimension(:) :: slices
      integer(kind=4), allocatable, dimension(:) :: offset_gpu
      integer(kind=4), allocatable, dimension(:) :: nb_elements_recv

C     factor for the magmablasf_zgeadd and magmablasf_zgemm subroutine such as
C     dB = alpha * dA + dB and
C     dC = alpha * dA * dB + beta
      complex(kind=8) :: alpha_add
      complex(kind=8) :: alpha_prod
      complex(kind=8) :: beta_prod
C     exit error info 
      integer(kind=4) :: info
      integer(kind=4) :: m,n
      complex(kind=8) :: In

      complex(kind=8), allocatable, dimension(:,:) :: hKC
      complex(kind=8), allocatable, dimension(:,:) :: hPC
      complex(kind=8), allocatable, dimension(:,:) :: hID
      complex(kind=8), allocatable, dimension(:,:) :: hSEIGC
      complex(kind=8), allocatable, dimension(:,:) :: hSEIGC_block
      integer(kind=8)  :: dID
      integer(kind=8)  :: dKC
      integer(kind=8)  :: dPC
      integer(kind=8)  :: dZn
      integer(kind=8)  :: dSEIGC
      integer(kind=8)  :: dSEIGC_block
      integer(kind=8)  :: dTemp

      call MPI_COMM_RANK(MPI_COMM_WORLD,rank,code)
      call MPI_COMM_SIZE(MPI_COMM_WORLD,nb_procs,code)

      if (MPI_COMM_GPU /= MPI_COMM_NULL) then
              allocate(slices(ngpu))
              allocate(nb_elements_recv(ngpu))
              allocate(offset_gpu(ngpu))
C
C     Master process gets the slices size and number fo elements
C     from each MPI_COMM_GPU processes
C
              call mpi_gather(slice_gpu, 1, MPI_INTEGER, slices, 1,
     $                        MPI_INTEGER, 0, MPI_COMM_GPU, code)
              if (rank == 0) then
                      offset_gpu(1) = 0
                      nb_elements_recv(1) = DSTO * slices(1)
                      do k=2,ngpu
                         offset_gpu(k) = offset_gpu(k-1) +
     $                                   nb_elements_recv(k-1)
                         nb_elements_recv(k) = DSTO * slices(k)
                      end do
              end if
C              
C     setting the variables
C
              DSTO_square = DSTO * DSTO
              DSTO_slice  = DSTO * slice_gpu
              alpha_prod = 1.
              beta_prod = 0.
C
C     *** Allocation ***
C
              allocate(hKC(DSTO,DSTO))
              allocate(hID(DSTO,DSTO))
              info = magmaf_zmalloc( dSEIGC_block, DSTO_slice )
              info = magmaf_zmalloc( dKC, DSTO_square )
              info = magmaf_zmalloc( dID, DSTO_slice  )
              info = magmaf_zmalloc( dZn, DSTO_slice  )
              info = magmaf_zmalloc( dTemp, DSTO_slice )
C
C     Merge and scale real and imaginary parts of the KS matrix
C     and then it is sent to all the GPU MPI processes              
C
              if (MPMASTER) then
                      hKC(:,:) = cmplx(KI(:,:),-KR(:,:))*DT
              end if
              call MPI_BCAST(hKC, DSTO_square, MPI_COMPLEX16,
     $         0, MPI_COMM_GPU, code)

              hID(:,:) = 0
              do m=1,DSTO
                 hID(m,m) = 1.0
              enddo
C
C     The ID and KC matrices are sent from the host to the GPU device
C
              call magmaf_zsetmatrix(DSTO, slice_gpu,
     $         hID(:,istart_gpu:iend_gpu),
     $         DSTO, dID, DSTO, magma_queue)
              call magmaf_zsetmatrix(DSTO, DSTO, hKC, DSTO,
     $         dKC, DSTO, magma_queue)
C
C     *** Deallocation ***
C
              deallocate(hKC,hID,STAT=ALLOCATION)
              IF (ALLOCATION.GT.0) THEN
                CALL ERRMSG('EXPSCALA','ALLOCATION FAILED',1)
              END IF
C
C     *** Matrix exponentiation ***
C
              call magmablasf_zlacpy('MagmaFull', DSTO, slice_gpu, dId,
     $         DSTO, dSEIGC_block, DSTO, magma_queue)
              call magmablasf_zlacpy('MagmaFull', DSTO, slice_gpu, dId,
     $         DSTO, dTemp, DSTO, magma_queue)

              In = 1
              do n=1,NEXPM
                 In = In * n
                 call magmablasf_zlacpy('MagmaFull', DSTO, slice_gpu,
     $            dTemp, DSTO, dZn, DSTO, magma_queue)
                 call magmablasf_zgemm(MagmaNoTrans, MagmaNoTrans,
     $            DSTO, slice_gpu, DSTO, alpha_prod, dKC, DSTO, dZn,
     $            DSTO, beta_prod, dTemp, DSTO, magma_queue)
                 alpha_add = 1. / In
                 call magmablasf_zgeadd(DSTO,slice_gpu, alpha_add,
     $            dTemp, DSTO, dSEIGC_block, DSTO, magma_queue)
              enddo
C
C     *** Allocation ***
C
              if (MPMASTER) then
                      allocate(hSEIGC(DSTO,DSTO))
              else
                      allocate(hSEIGC(1,1))
              end if
              allocate(hSEIGC_block(DSTO,slice_gpu))
C
C     The SEIGC matrix is sent from the GPU device to the host
C     and then collected by the master process from each MPI_COMM_GPU
C     processes
C
              call magmaf_zgetmatrix(DSTO, slice_gpu,
     $         dSEIGC_block, DSTO, hSEIGC_block, DSTO, magma_queue)

              call MPI_GATHERV(hSEIGC_block, DSTO_slice, MPI_COMPLEX16,
     $         hSEIGC, nb_elements_recv, offset_gpu, MPI_COMPLEX16,
     $         0, MPI_COMM_GPU, code)
C
C     *** Deallocation ***
C
              deallocate(hSEIGC_block,STAT=ALLOCATION)
              IF (ALLOCATION.GT.0) THEN
                CALL ERRMSG('EXPSCALA','ALLOCATION FAILED',1)
              END IF
              info = magmaf_free(dSEIGC_block)
              info = magmaf_free(dKC)
              info = magmaf_free(dID)
              info = magmaf_free(dZn)
              info = magmaf_free(dTemp)
      end if
C      
C     *** parallel matrix multiplication P(t+dt) = U*P(t)*U+ ***
C
      if (MPMASTER) then
C
C     *** Allocation ***
C
              allocate(hPC(DSTO,DSTO))
              info = magmaf_zmalloc( dPC, DSTO_square )
              info = magmaf_zmalloc( dSEIGC, DSTO_square )
              info = magmaf_zmalloc( dTemp, DSTO_square )
C
C     Merge real and imaginary parts of the propagator matrix
C              
              hPC(:,:) = cmplx(PR(:,:),PI(:,:))

              call magmaf_zsetmatrix(DSTO, DSTO, hPC, DSTO,
     $         dPC, DSTO, magma_queue)

              call magmaf_zsetmatrix(DSTO, DSTO, hSEIGC, DSTO,
     $         dSEIGC, DSTO, magma_queue)

              call magmablasf_zgemm(MagmaNoTrans, MagmaConjTrans,
     $         DSTO, DSTO, DSTO, alpha_prod, dPC, DSTO, dSEIGC,
     $         DSTO, beta_prod, dTemp, DSTO, magma_queue)

              call magmablasf_zgemm(MagmaNoTrans, MagmaNoTrans,
     $         DSTO, DSTO, DSTO, alpha_prod, dSEIGC, DSTO, dTemp,
     $         DSTO, beta_prod, dPC, DSTO, magma_queue)
C
C     The PC is sent back from the GPU device to the host
C
              call magmaf_zgetmatrix(DSTO, DSTO,
     $         dPC, DSTO, hPC, DSTO, magma_queue)

              PR(:,:) = REAL(hPC(:,:))
              PI(:,:) = AIMAG(hPC(:,:))
C
C     *** Deallocation ***
C
              deallocate(hPC,hSEIGC,
     $                   slices,nb_elements_recv,offset_gpu,         
     $                   STAT=ALLOCATION)
              IF (ALLOCATION.GT.0) THEN
                CALL ERRMSG('EXPSCALA','ALLOCATION FAILED',1)
              END IF
              info = magmaf_free(dPC)
              info = magmaf_free(dSEIGC)
              info = magmaf_free(dtemp)
      end if
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TAYLORMAGMA_MULTI ***
C
      END

