      SUBROUTINE MULMAT(A,B,C,DMAT,N,OPTION)
C
C     Purpose: MULtiplication of real quadratic MATrices.
C
C              C := A * B
C
C     History: - Creation (25.03.96, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DMAT: Dimension of matrices in the calling subroutine.
C
C     List of local variables:
C
C     A     : Left side matrix (N x N).
C     B     : Right side matrix (N x N).
C     C     : Product matrix (N x N).
C     N     : Number of rows or columns of matrix.
C     OPTION: a) NORMAL = A*B.
C             b) TRANSA = TRANSPOSE(A)*B.
C             c) TRANSB = A*TRANSPOSE(B).
C             d) NORMALADD --> C = C + A*B
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INTEGER DMAT
      REAL A(DMAT,DMAT),B(DMAT,DMAT),C(DMAT,DMAT)
C
      CHARACTER OPTION*(*)
      INTEGER N
C
C     ------------------------------------------------------------------
C
C     *** Start timer for matrix multiplication ***
C
      CALL TIMER('START','MATRIX MULTIPLICATION')
C
C     *** BLAS level three Matrix Matrix product ***
C
      IF (OPTION.EQ.'NORMAL') THEN
        CALL DGEMM('N','N',N,N,N,1.0,A,DMAT,B,DMAT,0.0,C,DMAT)
      ELSE IF (OPTION.EQ.'TRANSA') THEN
        CALL DGEMM('T','N',N,N,N,1.0,A,DMAT,B,DMAT,0.0,C,DMAT)
      ELSE IF (OPTION.EQ.'TRANSB') THEN
        CALL DGEMM('N','T',N,N,N,1.0,A,DMAT,B,DMAT,0.0,C,DMAT)
CCC ADLL
      ELSE IF (OPTION.EQ.'NORMALADD') THEN
        CALL DGEMM('N','N',N,N,N,1.0,A,DMAT,B,DMAT,1.0,C,DMAT)
      ELSE IF (OPTION.EQ.'TRANSBADD') THEN
        CALL DGEMM('N','T',N,N,N,1.0,A,DMAT,B,DMAT,1.0,C,DMAT)
CCC ADLL
      ELSE
        CALL ERRMSG('MULMAT','UNKNOWN OPTION FOR MULMAT',1)
      END IF
C
C     *** Stop timer for matrix multiplication ***
C
      CALL TIMER('STOP','MATRIX MULTIPLICATION')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MULMAT ***
C
      END
