      SUBROUTINE ELAN
C
C     Purpose: Electrostatic analysis of SCF solution.
C
C     History: - Creation (06.12.96, AMK)
C                         (14.04.05, FJ)
C                         (31.01.08, UR)
C                         (19.08.11, ADLL)
C                         (02.09.16, GG)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     CA: Alpha MO coefficient matrix.
C     CB: Beta MO coefficient matrix.
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C     S : Overlap matrix.
C     V : Atomic valence matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'pointer.h'
      INCLUDE 'property.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'strings.h'
C
      CHARACTER*80 PRTSTR
C
      INTEGER ALLOCATION,NSTO_ME
      INTEGER IATOM,IMO
      REAL,ALLOCATABLE :: CA(:,:),CB(:,:),PA(:,:),PB(:,:),S(:,:),V(:,:)
      REAL,ALLOCATABLE :: MOPOPA(:,:), MOPOPB(:,:)
      REAL,ALLOCATABLE :: SMOPOPA(:,:),SMOPOPB(:)
C
C     ------------------------------------------------------------------
C
C     *** Print molecular orientation for electrostatic analysis ***
C
      IF (STATUS.NE.'BOMD') THEN
        PRTSTR = 'MOLECULE ORIENTATION FOR ELECTROSTATIC ANALYSIS'
        CALL PRICAR(PRTSTR,CCSET)
      END IF
C
C     *** Allocate local fields ***
C
      IF (TOPCD) THEN
        NSTO_ME = 1
      ELSE
        NSTO_ME = NSTO
        IF (MPSLAVE) NSTO_ME = 1
      END IF
C
      ALLOCATE(CA(NSTO_ME,NSTO_ME),PA(NSTO,NSTO),S(NSTO_ME,NSTO_ME),
     $         V(NATOM,NATOM),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'ELAN','ALPHA ALLOCATION FAILED')
      IF (OKS) THEN
        ALLOCATE(CB(NSTO_ME,NSTO_ME),PB(NSTO,NSTO),STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'ELAN','BETA ALLOCATION FAILED')
      END IF
C
C     *** Calculate electrostatic moments ***
C
      IF (PEM) THEN
C
        IF (MPMASTER) THEN
          CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $               MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
          CALL BLDPMAT(CA,PA,NSTO,OCCNA,LFOMOA,HFOMOA)
          IF (OKS) THEN
            IF (ROKS) THEN
              CALL BLDPMAT(CA,PB,NSTO,OCCNB,LFOMOB,HFOMOB)
            ELSE
              CALL RSTIO(CB,EIGVALB,PB,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $                   MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
              CALL BLDPMAT(CB,PB,NSTO,OCCNB,LFOMOB,HFOMOB)
            END IF
            PA(:,:) = PA(:,:) + PB(:,:)
          END IF
          IF (STATUS.EQ.'BOMD') THEN
            CALL MOMENT(PA,1,NSTO)
          ELSE
            CALL MOMENT(PA,MAXMOM,NSTO)
          END IF
        END IF
C
C     *** Orbital population analyses ***
C
      ELSE IF (MPA.OR.LPA) THEN
C
        IF (MPMASTER) THEN
          CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $               MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
          CALL BLDPMAT(CA,PA,NSTO,OCCNA,LFOMOA,HFOMOA)
        END IF
C
        IF (SPHORB) THEN
          CALL POPAN(PA,S,CA,V,LLSPH,ULSPH,NSTO_ME,SPHSYM,'ALPHA')
        ELSE
          CALL POPAN(PA,S,CA,V,LLSTO,ULSTO,NSTO_ME,STOSYM,'ALPHA')
        END IF
C
        IF (OKS) THEN
C
          IF (MPMASTER.AND.ROKS) THEN
            CALL RSTIO(CB,EIGVALA,PB,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $                 MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
          ELSE IF (MPMASTER) THEN
            CALL RSTIO(CB,EIGVALB,PB,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $                 MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
          END IF
C
          IF (MPMASTER) CALL BLDPMAT(CB,PB,NSTO,OCCNB,LFOMOB,HFOMOB)
C
          IF (SPHORB) THEN
            CALL POPAN(PB,S,CB,V,LLSPH,ULSPH,NSTO_ME,SPHSYM,'BETA')
          ELSE
            CALL POPAN(PB,S,CB,V,LLSTO,ULSTO,NSTO_ME,STOSYM,'BETA')
          END IF
C
        END IF
C
C     *** Natural population analysis ***
C
      ELSE IF (NPA) THEN
C
        IF (MPMASTER) THEN
          CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $               MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
          CALL BLDPMAT(CA,PA,NSTO,OCCNA,LFOMOA,HFOMOA)
        END IF
C
        CALL POPMAT(PA,S,CA,NSTO_ME,'ALPHA')
C
        IF (OKS) THEN
C
          IF (MPMASTER.AND.ROKS) THEN
            CALL RSTIO(CB,EIGVALA,PB,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $                 MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
          ELSE IF (MPMASTER) THEN
            CALL RSTIO(CB,EIGVALB,PB,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $                 MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
          END IF
C
          IF (MPMASTER) CALL BLDPMAT(CB,PB,NSTO,OCCNB,LFOMOB,HFOMOB)
C
          CALL POPMAT(PB,S,CB,NSTO_ME,'BETA')
C
        END IF
C
        IF (MPMASTER) THEN
          IF (OKS) THEN
            CALL NBODRV(PA,PB,S,NSTO_ME)
          ELSE
            CALL NBODRV(PA,PA,S,NSTO_ME)
          END IF
        END IF
C
C     *** Topological space division population analyses ***
C
      ELSE IF (FPA.OR.HPA.OR.VPA) THEN
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(FCDA,NAUX,MASTER,MP_REAL)
          CALL MP_SEND_ALL(FCDB,NAUX,MASTER,MP_REAL)
          CALL MP_SEND_ALL(REFCA,NAUX,MASTER,MP_REAL)
          CALL MP_SEND_ALL(REFCB,NAUX,MASTER,MP_REAL)
        END IF
C
        IF (POPMO) THEN
C
          ALLOCATE (SMOPOPA(NSTO,2),MOPOPA(NATOM,NSTO),STAT=ALLOCATION)
C
          MOPOPA(:,:) = 0.0
          SMOPOPA(:,:) = 0.0
             
          MOCC = .TRUE.
          HFOMOA = NOMOA
          LFOMOA = NOMOA + 1
C
          DO IMO=1,NOMOA

            IF (MPMASTER) THEN
              CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $                   MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
              SMOPOPA(:,1) = EIGVALA(1:NSTO)
              OCCNA(1:IMO-1) = 0.0
              OCCNA(IMO+1:) = 0.0 
              OCCNA(IMO) = 2.0
C          
              CALL BLDPMAT(CA,PA,NSTO,OCCNA,LFOMOA,HFOMOA)
C
              IF (DEFORM.AND.OKS) THEN
                CALL BLDPPRO(PA,PB,NSTO,'TWO')
              ELSE IF (DEFORM) THEN
                CALL BLDPPRO(PA,PA,NSTO,'TWO')
              END IF
C
            END IF
C
            IF (MPP) THEN
              CALL MP_SEND_ALL(PA,NSTO**2,MASTER,MP_REAL)
              IF (OKS) CALL MP_SEND_ALL(PB,NSTO**2,MASTER,MP_REAL)
            END IF
C
            IF (OKS) THEN
              CALL POPGRD(PA,PB,NSTO)
            ELSE
              CALL POPGRD(PA,PA,NSTO)
            END IF
            DO IATOM=1,NATOM
              MOPOPA(IATOM,IMO) = NBASINA(IATOM) + NBASINB(IATOM)
              SMOPOPA(IMO,2) = SMOPOPA(IMO,2) + MOPOPA(IATOM,IMO) 
            END DO
          END DO
C
          CALL PRIMAT(MOPOPA,NATOM,1,NATOM,1,NOMOA,3,5,OUT,'NONE',
     $                'TOTAL','ALPHA MO OCCUPATION BY ATOM')
          CALL PRIMAT(SMOPOPA,NSTO,1,NOMOA,1,2,3,5,OUT,'NONE',
     $                'TOTAL','SUM OF ALPHA MO OCCUPATIONS')
          DEALLOCATE (MOPOPA,SMOPOPA,STAT=ALLOCATION)

        ELSE IF (TOPCD) THEN
C
          CALL NORMCDC(FCDA,FCDB,NAUX)
C
          IF (DEFORM.AND.OKS) THEN
            CALL BLDXPRO(FCDA,FCDB,NAUX)
          ELSE IF (DEFORM) THEN
            CALL BLDXPRO(FCDA,FCDA,NAUX)
          END IF
C
          MFCDA(:) = FCDA(:)
          MFCDB(:) = FCDB(:)
C
        ELSE
C
          IF (MPMASTER) THEN
C
            CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $                 MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
            CALL BLDPMAT(CA,PA,NSTO,OCCNA,LFOMOA,HFOMOA)
C
            IF (OKS) THEN
              IF (ROKS) THEN
                CALL RSTIO(CB,EIGVALA,PB,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $                     MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
              ELSE
                CALL RSTIO(CB,EIGVALB,PB,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $                     MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
              END IF
              CALL BLDPMAT(CB,PB,NSTO,OCCNB,LFOMOB,HFOMOB)
            END IF
C
            IF (DEFORM.AND.OKS) THEN
              CALL BLDPPRO(PA,PB,NSTO,'TWO')
            ELSE IF (DEFORM) THEN
              CALL BLDPPRO(PA,PA,NSTO,'TWO')
            END IF
C
          END IF
C
          IF (MPP) THEN
            CALL MP_SEND_ALL(PA,NSTO**2,MASTER,MP_REAL)
            IF (OKS) CALL MP_SEND_ALL(PB,NSTO**2,MASTER,MP_REAL)
          END IF
C
        END IF
C
        IF (OKS) THEN
          CALL POPGRD(PA,PB,NSTO)
        ELSE
          CALL POPGRD(PA,PA,NSTO)
        END IF
C
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(CA,PA,S,V,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'ELAN','ALPHA DEALLOCATION FAILED')
      IF (OKS) THEN
        DEALLOCATE(CB,PB,STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'ELAN','BETA DEALLOCATION FAILED')
      END IF
C
C     *** Flush buffer ***
C
      CALL DEFLUSH(OUT)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE ELAN ***
C
      END
