      SUBROUTINE POPGRD(PA,PB,DSTO)
C
C     Purpose: POPulation analysis by numerical integration on a GRiD.
C
C     History: - Creation (28.08.12, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
      INCLUDE 'property.h'
C
      INTEGER DSTO
      REAL PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      CHARACTER*80 PRTSTR,STRCOMP,SYSTEM*4
      INTEGER I,IATOM,LC,MX,MY,MZ,STREXT,STRLEN
      REAL DPOLE,QMOL,QPOLE,SMOL,TWONPOLE
C
      REAL TOPMOM(10)
C
C     ------------------------------------------------------------------
C
C     *** Initialize grid for numerical population analysis ***
C
      CALL POPINI
      CALL GRDDRV(PA,PB,DSTO)
C
C     *** Do numerical population analysis on a grid ***
C
      CALL NUMPOP(PA,PB,DSTO)
C
      IF (MPP) THEN
        CALL MP_SUM(NBASINA,NBASINA,NATOM,MP_REAL)
        CALL MP_SUM(NBASINB,NBASINB,NATOM,MP_REAL)
      END IF
C
C     *** Loop for iterative Hirshefeld population analysis ***
C
      IF (ITHPA) CALL DOHPA(PA,PB,DSTO)
C
      IF (MPP.AND.FULLPA) CALL MP_SUM(MBASIN,MBASIN,10*MAXATOM,MP_REAL)
C
C     *** Write numerical population analysis header ***
C
      IF (FPA) THEN
        IF (DEFORM) THEN
          PRTSTR = 'BECKE DEFORMATION DENSITY ANALYSIS'
        ELSE
          PRTSTR = 'BECKE POPULATION ANALYSIS'
        END IF
      ELSE IF (HPA) THEN
        IF (DEFORM) THEN
          PRTSTR = 'HIRSHFELD DEFORMATION DENSITY ANALYSIS'
          IF (ITHPA) THEN
            PRTSTR = 'ITERATIVE '//PRTSTR(1:38)
          ELSE IF (.NOT.SPHPA) THEN
             PRTSTR = 'SCF '//PRTSTR(1:38)
          END IF
        ELSE
          PRTSTR = 'HIRSHFELD POPULATION ANALYSIS'
          IF (ITHPA) THEN
            PRTSTR = 'ITERATIVE '//PRTSTR(1:29)
          ELSE IF (.NOT.SPHPA) THEN
            PRTSTR = 'SCF '//PRTSTR(1:29)
          END IF
        END IF
      ELSE IF (VPA) THEN
        IF (DEFORM) THEN
          PRTSTR = 'VORONOI DEFORMATION DENSITY ANALYSIS'
        ELSE
          PRTSTR = 'VORONOI POPULATION ANALYSIS'
        END IF
      ELSE
        CALL ERRMSG('POPGRD','UNKNOWN POPULATION ANALYSIS',1)
      END IF
C
      IF (TOPCD) THEN
        STRLEN = STREXT(PRTSTR)
        PRTSTR = PRTSTR(1:STRLEN)//' OF AUXILIARY DENSITY'
      ELSE
        STRLEN = STREXT(PRTSTR)
        PRTSTR = PRTSTR(1:STRLEN)//' OF KOHN-SHAM DENSITY'
      END IF
C
      CALL RMELSOUT
      CALL MESSAGE(1,STRCOMP(PRTSTR),1,1)
C
      IF (POPMO) RETURN
C
C     *** Write deformation density analysis output ***
C
      IF (DEFORM) THEN
C
        CALL MESSAGE(0,'ATOMIC DEFORMATION CHARGES',0,1)
C
        QMOL = 0.0
        DO IATOM=1,NATOM
          QATOM(IATOM) = -NBASINA(IATOM) - NBASINB(IATOM)
          QMOL = QMOL + QATOM(IATOM)
          WRITE (OUT,'(T2,I4,1X,A,1X,F8.4)') IATOM,
     $          ATOMLAB(ATOMMAP(IATOM,1)),QATOM(IATOM)
        END DO
C
        IF (OKS) THEN
C
          CALL MESSAGE(0,'ATOMIC SPIN DEFORMATION CHARGES',1,1)
C
          SMOL = 0.0
          DO IATOM=1,NATOM
            QSPIN(IATOM) = -NBASINA(IATOM) + NBASINB(IATOM)
            SMOL = SMOL + QSPIN(IATOM)
            WRITE (OUT,'(T2,I4,1X,A,1X,F8.4)') IATOM,
     $            ATOMLAB(ATOMMAP(IATOM,1)),QSPIN(IATOM)
          END DO
C
          WRITE (PRTSTR,'("SPIN CHARGE: ",F8.4)') SMOL
          CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
C
        END IF
C
        CALL MESSAGE(0,'NUMERICAL INTEGRATED MOLECULAR CHARGE',1,0)
        WRITE (PRTSTR,'("TOTAL CHARGE: ",F8.4)') QMOL
        CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
C
C     *** Write numerical population analysis output ***
C
      ELSE
C
        CALL MESSAGE(0,'ATOMIC NET CHARGES',0,1)
C
        QMOL = 0.0
        DO IATOM=1,NATOM
          QATOM(IATOM) = REAL(ZATOM(IATOM)) - NBASINA(IATOM) -
     $                                        NBASINB(IATOM)
          QMOL = QMOL + QATOM(IATOM)
          WRITE (OUT,'(T2,I4,1X,A,1X,F8.4)') IATOM,
     $          ATOMLAB(ATOMMAP(IATOM,1)),QATOM(IATOM)
        END DO
C
        IF (OKS) THEN
C
          CALL MESSAGE(0,'ATOMIC SPIN CHARGES',1,1)
C
          SMOL = 0.0
          DO IATOM=1,NATOM
            QSPIN(IATOM) = NBASINA(IATOM) - NBASINB(IATOM)
            SMOL = SMOL + QSPIN(IATOM)
            WRITE (OUT,'(T2,I4,1X,A,1X,F8.4)') IATOM,
     $            ATOMLAB(ATOMMAP(IATOM,1)),QSPIN(IATOM)
          END DO
C
          WRITE (PRTSTR,'("SPIN CHARGE: ",F8.4)') SMOL
          CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
C
        END IF
C
        CALL MESSAGE(0,'NUMERICAL INTEGRATED MOLECULAR CHARGE',1,0)
        WRITE (PRTSTR,'("TOTAL CHARGE: ",F8.4)') QMOL
        CALL MESSAGE(0,STRCOMP(PRTSTR),1,0)
C
      END IF
C
C     *** Write extended full population analysis output ***
C
      IF (FULLPA) THEN
C
C     *** Select unit system for numerical population analysis ***
C
        IF (UNIT(1).EQ.'ANGSTROM') THEN
          DPOLE = ESU(1)
          QPOLE = ESU(2)
          SYSTEM = 'ESU'
        ELSE
          DPOLE = 1.0
          QPOLE = 1.0
          SYSTEM = 'AU'
        END IF
C
C     *** Calculate numerical density sums ***
C
        TOPMOM(:) = 0.0
        DO IATOM=1,NATOM
          DO LC=1,2
            DO MX=0,LC
              DO MY=0,LC-MX
                MZ = LC - MX - MY
                TOPMOM(COP(MX,MY,MZ)) = TOPMOM(COP(MX,MY,MZ)) +
     $                                  MBASIN(COP(MX,MY,MZ),IATOM)
              END DO
            END DO
          END DO
        END DO
C
C     *** Print numerical electronic dipole moments ***
C
        WRITE (PRTSTR,5000) SYSTEM
 5000   FORMAT ('NUMERICAL ELECTROSTATIC MOMENTS IN ',A4)
        CALL MESSAGE(1,STRCOMP(PRTSTR),2,1)
        WRITE (OUT,5010)
 5010   FORMAT (T2,21X,'DIPOLE MOMENTS',/
     $          T2,'ATOM',13X,'X',10X,'Y',10X,'Z')
        WRITE (OUT,'(T2,45("-"))')
        DO IATOM=1,NATOM
          WRITE (OUT,5020) IATOM,ATOMLAB(ATOMMAP(IATOM,1)),
     $                     (DPOLE*MBASIN(I,IATOM),I=2,4)
 5020     FORMAT (T2,I4,1X,A3,5X,F8.4,3X,F8.4,3X,F8.4)
        END DO
        WRITE (OUT,'(T2,45("-"))')
        WRITE (OUT,5030) (DPOLE*TOPMOM(I),I=2,4)
 5030   FORMAT (T2,"TOTAL",8X,F8.4,3X,F8.4,3X,F8.4)
C
C     *** Print numerical electronic quadrupole moments ***
C
        WRITE (OUT,5040)
 5040   FORMAT (/,T2,35X,'QUADRUPOLE MOMENTS',/
     $            T2,'  ATOM   ',7X,'XX',9X,'XY',9X,'XZ',
     $                           9X,'YY',9X,'YZ',9X,'ZZ')
        WRITE (OUT,'(T2,78("-"))')
        DO IATOM=1,NATOM
          WRITE (OUT,5050) IATOM,ATOMLAB(ATOMMAP(IATOM,1)),
     $                     QPOLE*TWONPOLE(2,0,0,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(1,1,0,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(1,0,1,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(0,2,0,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(0,1,1,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(0,0,2,MBASIN(:,IATOM))
 5050       FORMAT (T2,I4,1X,A3,5X,F8.4,5(3X,F8.4))
        END DO
        WRITE (OUT,'(T2,78("-"))')
        WRITE (OUT,5060) QPOLE*TWONPOLE(2,0,0,TOPMOM),
     $                   QPOLE*TWONPOLE(1,1,0,TOPMOM),
     $                   QPOLE*TWONPOLE(1,0,1,TOPMOM),
     $                   QPOLE*TWONPOLE(0,2,0,TOPMOM),
     $                   QPOLE*TWONPOLE(0,1,1,TOPMOM),
     $                   QPOLE*TWONPOLE(0,0,2,TOPMOM)
 5060   FORMAT (T2,'TOTAL',8X,F8.4,5(3X,F8.4))
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE POPGRD ***
C
      END
