      SUBROUTINE FFIELDINI
C
C     Purpose: Initialize the force field parameters.
C
C     History: - Creation (04.10.09, AG) 
C                         (06.12.16, TM)
C
C     ******************************************************************
C
C     List of local variables:
C
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'mmscale.h'
      INCLUDE 'mp_demon.h'
C
      LOGICAL PRMEND,PRMERR
      INTEGER I,IPOS,IVALUE,J,LINES,LINEN0,NTOKEN,STREXT,STRLEN
      INTEGER CHR2ASC(500)
      CHARACTER*500 LINE,LTU,STRING
      CHARACTER*80 DTS,FFNAME,PRTSTR,STRCOMP
      CHARACTER*80 TOKEN(80)
      CHARACTER COMMA*1,STRTYP*7
C
C     ------------------------------------------------------------------
C
C     Set default control parameters for the potential energy terms used
C     in this force field
C     Selected for AMOEBA force field terms.
C     Other force fields have similar parameters or new ones. 
C
CTODO These param are set to true but I would set them to False
CTODO and turn them to True once the related terms are read in .prm
C
      DOBOND = .FALSE.
      DOBANG = .FALSE.
      DOUREY = .FALSE.
      DOIMPTORS = .FALSE.
      DOTORS = .FALSE.
      DOCHARGE = .FALSE.
      DOVDW = .FALSE.
C
      COMMA = ','
C
      VDWINDX = 'CLASS'
C
C     *** Call initialize all MM variables (OPLS-AA, AMOEBA) ***
C          TO BE WRITTEN

      CALL INITFF
C
C     *** Check that mm file deMon.prm exists ***
C
      REWIND (PRM)
      REWIND (IOFFD)

      LINES = 0
      LINEN0 = 0

  500 CONTINUE
C
C     *** Create the PRM file with keywords and parameters only ***
C
      LINEN0 = LINEN0 + 1
C
      PRMEND = .FALSE.
      PRMERR = .FALSE.
      IF (MPMASTER) THEN
        READ (PRM,'(A)',ERR=200,END=100) STRING
        GO TO 300
  100   CONTINUE
        PRMEND = .TRUE.
        GO TO 300
  200   CONTINUE
        PRMERR = .TRUE.
  300   CONTINUE
      END IF
C
      CALL MP_SEND_ALL(PRMERR,1,MASTER,MP_LOG)
      CALL MP_SEND_ALL(PRMEND,1,MASTER,MP_LOG)
C
      IF (PRMERR) GO TO 99990
      IF (PRMEND) GO TO 600
C
      IF (MPMASTER) STRLEN = LEN_TRIM(STRING)
      CALL MP_SEND_ALL(STRLEN,1,MASTER,MP_INT)
C
      IF (STRLEN.NE.0) THEN
        IF (MPP) THEN
          IF (MPMASTER) THEN
            DO IPOS=1,STRLEN
              CHR2ASC(IPOS) = ICHAR(STRING(IPOS:IPOS))
            END DO
          END IF
          CALL MP_SEND_ALL(CHR2ASC,STRLEN,MASTER,MP_INT)
          STRING = ' '
          DO IPOS=1,STRLEN
            STRING(IPOS:IPOS) = CHAR(CHR2ASC(IPOS))
          END DO
        END IF
        LINE = LTU(STRING)
        IF ((LINE(1:1).EQ.COMMENT).OR.(LINE(7:7).EQ.COMMENT)) GO TO 500
        J = 0
        DO I=1,STREXT(STRING)
          IF (LINE(I:I).EQ.COMMA) THEN
            J = J + 1
          END IF
        END DO
        IF (J.GE.1) GO TO 500
        LINES = LINES + 1
        WRITE (IOFFD,'(A)') STRING(1:STREXT(STRING))
C
        CALL DEFLUSH(IOFFD)
      END IF
C
      GO TO 500
C
  600 CONTINUE
C
      REWIND (IOFFD)
C
C     *** Initialize type lines
C
      KEYWLINES = 0
      ATOMLINES = 0
      VDWLINES = 0
      BONDLINES = 0
      ANGLELINES = 0
      UREYLINES = 0
      IMPTORLINES = 0
      TORSIONLINES = 0
      CHARGELINES = 0
      BIOTYPLINES = 0
CCC ADLL LA
      LINKATLINES = 0
CCC ADLL LA
C
C     *** Find the keywords lines ***                    
C
  900 CONTINUE
      READ (IOFFD,'(A80)',ERR=99990,END=800) LINE
      KEYWLINES = KEYWLINES + 1
      STRING = STRCOMP(LTU(DTS(LINE)))
      CALL STRTOK(STRING,NTOKEN,TOKEN)
      IF (TOKEN(1).EQ.'ATOM') THEN
        KEYWLINES = KEYWLINES - 1
        GO TO 700
      ELSE
        GO TO 900
      END IF
C
  700 CONTINUE
C
C     *** Read the necessary keywords for this force field ***
C
      REWIND (IOFFD)

      LINEN0 = 0
      DO I=1,KEYWLINES
        READ (IOFFD,'(A80)',ERR=99990,END=800) LINE
        LINEN0 = LINEN0 + 1
        STRING = STRCOMP(LTU(DTS(LINE)))
        CALL STRTOK(STRING,NTOKEN,TOKEN)
C
        IF (TOKEN(1).EQ.'FORCEFIELD') THEN
          FFNAME = TOKEN(2)
          IF (FFNAME(1:4).NE.FFIELD(1:4)) THEN
            CALL MESSAGE(1,'ERROR FORCEFIELD NAME KEYWORD',1,0)
            GO TO 99970
          END IF
        END IF
C
C     *** ONLY Specific keywords for OPLSAA and AMOEBA are used ***
C         EXHAUSTIVE LIST IN TINKER 5.0  prmkey.f
C
C     *** Keywords read at the beginning of the FORCEFIELD FILE ***
C         Some keywords are not read but set in initff.f deMon file
C
        IF (TOKEN(1).EQ.'VDWINDX') THEN
          VDWINDX = TOKEN(2)
        END IF
        IF (TOKEN(1).EQ.'VDWTYPE') THEN
          VDWTYPE = TOKEN(2)
        END IF
        IF (TOKEN(1).EQ.'RADIUSRULE') THEN
          RADRULE = TOKEN(2)
        END IF
        IF (TOKEN(1).EQ.'RADIUSTYPE') THEN
          RADTYPE = TOKEN(2)
        END IF
        IF (TOKEN(1).EQ.'RADIUSSIZE') THEN
          RADSIZ = TOKEN(2)
        END IF
        IF (TOKEN(1).EQ.'EPSILONRULE') THEN
          EPSRULE = TOKEN(2)
        END IF
        IF (TOKEN(1).EQ.'TORSIONUNIT') THEN
          CALL GETNUM(TOKEN(2),STRTYP,TORSUNIT,IVALUE)
        END IF
C
        IF (TOKEN(1).EQ.'VDW-12-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,V2SCALE,IVALUE)
          IF (V2SCALE.GT.1.0) V2SCALE = 1.0/V2SCALE
        END IF
        IF (TOKEN(1).EQ.'VDW-13-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,V3SCALE,IVALUE)
          IF (V3SCALE.GT.1.0) V3SCALE = 1.0/V3SCALE
        END IF
        IF (TOKEN(1).EQ.'VDW-14-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,V4SCALE,IVALUE)
          IF (V4SCALE.GT.1.0) V4SCALE = 1.0/V4SCALE
        END IF
C
        IF (TOKEN(1).EQ.'CHG-12-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,C2SCALE,IVALUE)
          IF (C2SCALE.GT.1.0) C2SCALE = 1.0/C2SCALE
        END IF
        IF (TOKEN(1).EQ.'CHG-13-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,C3SCALE,IVALUE)
          IF (C3SCALE.GT.1.0) C3SCALE = 1.0/C3SCALE
        END IF
        IF (TOKEN(1).EQ.'CHG-14-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,C4SCALE,IVALUE)
          IF (C4SCALE.GT.1.0) C4SCALE = 1.0/C4SCALE
        END IF
CCC ADLL LA  ! Scaling parameters to reduce charges at the QM/MM border
        IF (TOKEN(1).EQ.'LAT-12-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,L2SCALE,IVALUE)
          IF (L2SCALE.GT.1.0) L2SCALE = 1.0/L2SCALE
        END IF
        IF (TOKEN(1).EQ.'LAT-13-SCALE') THEN
          CALL GETNUM(TOKEN(2),STRTYP,L3SCALE,IVALUE)
          IF (L3SCALE.GT.1.0) L3SCALE = 1.0/L3SCALE
        END IF
CCC ADLL LA

C
        IF (TOKEN(1).EQ.'DIELECTRIC') THEN
          CALL GETNUM(TOKEN(2),STRTYP,DIELEC,IVALUE)
        END IF
C
      END DO
C
  800 CONTINUE
      READ (IOFFD,'(A80)',ERR=99990,END=1000) LINE
      STRING = STRCOMP(LTU(DTS(LINE)))
      CALL STRTOK(STRING,NTOKEN,TOKEN)
      IF (TOKEN(1).EQ.'ATOM') THEN
        ATOMLINES = ATOMLINES + 1
      ELSE IF (TOKEN(1)(1:3).EQ.'VDW') THEN
        VDWLINES = VDWLINES + 1
CCC ADLL LA
      ELSE IF (TOKEN(1).EQ.'LINKAT') THEN
        LINKATLINES = LINKATLINES + 1
CCC ADLL LA
      ELSE IF (TOKEN(1).EQ.'BOND') THEN
        BONDLINES = BONDLINES + 1
      ELSE IF (TOKEN(1).EQ.'ANGLE') THEN
        ANGLELINES = ANGLELINES + 1
      ELSE IF (TOKEN(1).EQ.'UREYBRAD') THEN
        UREYLINES = UREYLINES + 1
      ELSE IF (TOKEN(1).EQ.'IMPTORS') THEN
        IMPTORLINES = IMPTORLINES + 1
      ELSE IF (TOKEN(1).EQ.'TORSION') THEN
        TORSIONLINES = TORSIONLINES + 1
      ELSE IF (TOKEN(1).EQ.'CHARGE') THEN
        CHARGELINES = CHARGELINES + 1
      ELSE IF (TOKEN(1).EQ.'BIOTYPE') THEN
        BIOTYPLINES = BIOTYPLINES + 1
      END IF
      GO TO 800
C
 1000 CONTINUE
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
99970 CONTINUE
      CALL ERRMSG('FFIELDINI','FORCE FIELD DEFINITION FILE PRMS DOES '//
     $            'NOT EXIST',1)
99990 CONTINUE
      WRITE (PRTSTR,'("ERROR READING PRM/FFD LINE ",I4)') LINEN0
      CALL MESSAGE(0,'--> '//TRIM(STRING),1,0)
      CALL ERRMSG('INPUT',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE FFIELDINI ***
C
      END
