      SUBROUTINE INITFF
C
C     Purpose: Initialize molecular mechanics force field variables.
CTODO
CTODO Unify file and subroutine name.
CTODO Avoid scale variables by new loop structure
CTODO
C
C     History: - Creation (24.03.09, AG) 
C                         (13.11.14, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C     POLEPS  : induced dipole convergence criterion
C     POLTYP  : Type of polarization : direct or mutual
C     V2SCALE : Van der Waals 1-2 scale factor for energy evaluations 
C     V3SCALE : Van der Waals 1-3 scale factor for energy evaluations
C     V4SCALE : Van der Waals 1-4 scale factor for energy evaluations 
C     C2SCALE : Charge 1-2 scale factor for energy evaluations 
C     C3SCALE : Charge 1-3 scale factor for energy evaluations
C     C4SCALE : Charge 1-4 scale factor for energy evaluations 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'mmscale.h'
      INCLUDE 'prm.h'     
C
      REAL NOVAL
      PARAMETER (NOVAL=99999.9)
C
C     ------------------------------------------------------------------
C
C     Set default parameters for geometry
C
C     BOND-UNIT and ANGLE-UNIT are not used.
C
C     BANGTYPE is set to 'HARMONIC' in readparam.f (other type is
C     Fourier designed by anglef in the force field: not programmed yet)
C
C     All angles are treated with HARMONIC. But if there are opbend
C     (out of plane bending) the angtype (HARMONIC) is then turned
C     to 'IN-PLANE' : The opbend sites (trigonal) are recognized and
C     their angtypes are put IN-PLANE, so that the angular potential
C     uses projected in plane values : not programmed yet.
C
C     All values in the force-fields are Angstroms, radians
C     kcal/mol/A**2 or Kcal/mol/rd**2, Energies in Kcal/
C     All MM energies are calculated in kcal/mol and the converted in h
C     
C     *** Parameters for bonds ***
C
      BONDTYPE = 'HARMONIC'
      MMBOND(:,:) = 0
      KBND(:) = 0.0
      BND0(:) = 0.0
      KBOND(:) = NOVAL
      BOND0(:) = 0.0
      CUBBND = 0.0
      QUABND = 0.0
C
C     *** Parameters for bond angles ***
C
      MMBANG(:,:) = 0
      KBANG(:) = 0.0
      BANG0(:,:) = 0.0
      KANGLE(:) = NOVAL
      ANGLE0(:) = 0.0
      CUBBANG = 0.0
      QUABANG = 0.0
      PENBANG = 0.0
      SEXBANG = 0.0
C
C     *** Parameters for Urey-Bradley to be initialized ***
C
      MMUREY(:,:) = 0
      KUREY(:) = 0.0
      UREY0(:) = 0.0
      CUBUREY = 0.0
      QUAUREY = 0.0
C
C     *** Parameters for Torsions to be initialized ***
C
      MMTANG(:,:) = 0
      TANG1(:,:) = 0.0
      TANG2(:,:) = 0.0
      TANG3(:,:) = 0.0
      TORS1(:,1) = NOVAL
      TORS2(:,1) = NOVAL
      TORS3(:,1) = NOVAL
      TORS1(:,2) = 0.0
      TORS2(:,2) = 0.0
      TORS3(:,2) = 0.0
      TORSUNIT = 1.0
C
C     *** Paramaters for improper torsions to be initialized ***
C
      IIMTORS(:,:) = 0
      IMPTANG1(:,:) = 0.0
      IMPTANG2(:,:) = 0.0
      IMPTANG3(:,:) = 0.0
      IMPTORS1(:,1) = NOVAL
      IMPTORS2(:,1) = NOVAL
      IMPTORS3(:,1) = NOVAL
      IMPTORS1(:,2) = 0.0
      IMPTORS2(:,2) = 0.0
      IMPTORS3(:,2) = 0.0
      IMPTORSUNIT = 1.0
C
C     *** Set default parameters for Charge interations ***
C
C     *** DIELEC : value of the dielectric constant ***
C
      DIELEC = 1.0
      C2SCALE = 0.0
      C3SCALE = 0.0
      C4SCALE = 1.0
C
C     *** Set default parameters for VDW Interactions ***
C
      VDWTYPE = 'LENNARD-JONES'
      RADRULE = 'ARITHMETIC'
      RADTYPE = 'R-MIN'
      RADSIZ = 'RADIUS'
      EPSRULE = 'GEOMETRIC'
      GAUSSTYPE = 'NONE'
      ABUCK = 0.0
      BBUCK = 0.0
      CBUCK = 0.0
      VRAD(:) = 0.0
      VEPS(:) = 0.0
      REDUC(:) = 0.0
C
      V2SCALE = 0.0
      V3SCALE = 0.0
      V4SCALE = 1.0
C
C     *** Set default control parameters for induced dipoles ***
C
      POLEPS = 0.000001
C
C     *** Initialize scaling factors ***
C     
CCC ADLL LA
C     ** Set default control parameters for link atoms ***
C
      L2SCALE = 1.0
      L3SCALE = 0.0
CCC ADLL LA

C     *** Cutoff distances for pair energies ***
C
      VDWCUT = 1.0E10
      VDWSWITCH = 0.9
      VDWSWITCH = VDWSWITCH*VDWCUT
      CHGCUT = 1.0E10
      CHGSWITCH = 0.65
      CHGSWITCH = CHGSWITCH*CHGCUT
C
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE INITFF ***
C
      END
