      SUBROUTINE MYINPUT
C
C     Purpose: Generate parallel deMon input files.
C
C     History: - Creation (11.06.04, AMK)
C                         (14.08.05, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C     INTCHAR: Integer to character transformation field.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'fileio.h'
      INCLUDE 'mp_demon.h'
C
      CHARACTER*500 STRING
      CHARACTER*80 PRTSTR,STRCOMP
      INTEGER IC,LINENO,LINENOOLD,STREXT
C
      INTEGER ALLOCATION,ISPACE
      INTEGER,ALLOCATABLE :: INTCHAR(:)
C
C     ------------------------------------------------------------------
C
C     *** Quick return, if parallelization is not requested ***
C
      IF (.NOT.MPP) RETURN
C
C     *** Allocate local field ***
C
      ALLOCATE(INTCHAR(500),STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('MYINPUT','ALLOCATION FAILED',1)
      END IF
C
C     *** Rewind seed (deMon.inp) and distribute input files ***
C
      REWIND (INP)
C
      ISPACE = ICHAR(' ')
      LINENOOLD = 0
C
C     *** Read seed input lines and distribute them ***
C
  500 CONTINUE
C
        LINENO = LINENOOLD + 1
        LINENOOLD = LINENO
        IF (MPMASTER) THEN
          READ (INP,'(A)',ERR=99990,END=700) STRING
          IF (LEN_TRIM(STRING).NE.0) THEN
            DO IC=1,500
              INTCHAR(IC) = ICHAR(STRING(IC:IC))
            END DO
          END IF
        END IF
C
  600   CONTINUE
C
        CALL MP_SEND_ALL(LINENO,1,MASTER,MP_INT)
C
        IF (LINENO.NE.LINENOOLD) GO TO 800
C
        CALL MP_SEND_ALL(INTCHAR,500,MASTER,MP_INT)
C
        IF (MPSLAVE) THEN
          DO IC=1,500
            IF (INTCHAR(IC).LT.32) INTCHAR(IC) = ISPACE
            STRING(IC:IC) = CHAR(INTCHAR(IC))
          END DO
        END IF
C
        IF (ME.GT.0) THEN
          WRITE (INP,'(A)') STRING(1:STREXT(STRING))
          CALL DEFLUSH(INP)
        END IF
C
        GO TO 500
C
  700 CONTINUE
C
      LINENO = -1
C
      GO TO 600
C
  800 CONTINUE
CCC ADLL QMMM
C     *** Rewind seed (deMon.geo) and distribute input files ***
C
      REWIND (GEO)
C
      ISPACE = ICHAR(' ')
      LINENOOLD = 0
C
C     *** Read seed input lines and distribute them ***
C
  900 CONTINUE
C
        LINENO = LINENOOLD + 1
        LINENOOLD = LINENO
        IF (MPMASTER) THEN
          READ (GEO,'(A)',ERR=99990,END=1100) STRING
          IF (LEN_TRIM(STRING).NE.0) THEN
            DO IC=1,500
              INTCHAR(IC) = ICHAR(STRING(IC:IC))
            END DO
          END IF
        END IF
C
 1000   CONTINUE
C
        CALL MP_SEND_ALL(LINENO,1,MASTER,MP_INT)
C
        IF (LINENO.NE.LINENOOLD) GO TO 1200
C
        CALL MP_SEND_ALL(INTCHAR,500,MASTER,MP_INT)
C
        IF (MPSLAVE) THEN
          DO IC=1,500
            IF (INTCHAR(IC).LT.32) INTCHAR(IC) = ISPACE
            STRING(IC:IC) = CHAR(INTCHAR(IC))
          END DO
        END IF
C
        IF (ME.GT.0) THEN
          WRITE (GEO,'(A)') STRING(1:STREXT(STRING))
          CALL DEFLUSH(GEO)
        END IF
C
        GO TO 900
C
 1100 CONTINUE
C
      LINENO = -1
      GO TO 1000
C
 1200 CONTINUE
C
C     *** Deallocate local field ***
C
      DEALLOCATE(INTCHAR,STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('MYINPUT','ALLOCATION FAILED',1)
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
99990 CONTINUE
      WRITE (PRTSTR,'("ERROR READING SEED INPUT LINE ",I4)') LINENO
      CALL MESSAGE(0,'--> '//STRING(:STREXT(STRING)),1,0)
      CALL ERRMSG('MYINPUT',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MYINPUT ***
C
      END
