      SUBROUTINE PROJDE1(DE1,U,DACT,DDEG,DPRI,DVAR)
C
C     Purpose: Project out the external degrees of freedom from the
C              Cartesian gradient vector and remove constrained
C              Cartesian gradients.
C
C     History: - Creation (27.10.04, UR)
C                         (01.11.05, UR)
C                         (13.11.14, GG)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DACT: Active degrees of freedom.
C     DDEG: Number of degrees of freedom.
C     DPRI: Number of degrees of freedom.
C     DVAR: Number of degrees of freedom.
C
C     List of local variables:
C
C     DE : Cartesian gradient.
C     DE1: Cartesian gradient without external degrees of freedom.
C     U  : Projector matrix.
C
C     List of local dynamical fields:
C
C     DEALL: Cartesian gradient vector with all components.
C     PROTR: Projector matrix for translations and rotations.
C     TRAFO: Cartesian gradient transformation matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'freq.h'
      INCLUDE 'mp_demon.h' 
      INCLUDE 'opt.h'
C
      INTEGER DACT,DDEG,DPRI,DVAR
      REAL DE1(DDEG),U(DPRI,DVAR)
C
      INTEGER I
C
      INTEGER ALLOCATION,DDEG_ME
      REAL,ALLOCATABLE :: DEALL(:),PROTR(:,:),TRAFO(:,:)
C
C     ------------------------------------------------------------------
C
      IF (MPMASTER) THEN
        DDEG_ME = DDEG
      ELSE
        DDEG_ME = 1
      END IF
C
C     *** Allocate local fields ****
C
      ALLOCATE(DEALL(DDEG_ME),PROTR(DDEG,DDEG),TRAFO(DDEG_ME,DDEG_ME),
     $         STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'PROJDE1','ALLOCATION FAILED')
C
C     *** Build projector for the translations and ***
C     *** infinetesimal rotations of the molecule  ***
C
      CALL BLDTR(PROTR,DDEG)
C
      IF (MPSLAVE) GO TO 100
C
C     *** Generate energy gradient transformation matrix ***
C
      CALL SQUMAT(PROTR(1,NVIB+1),TRAFO,DDEG,DDEG,DDEG,DDEG-NVIB,-1.0,
     $            'KEEP')
      CALL SYMMAT(TRAFO,DDEG,DDEG,'LOWUP')
C
      DO I=1,DDEG
        TRAFO(I,I) = 1.0 + TRAFO(I,I) 
      END DO
C
C     *** Project out external degrees of ***
C     *** freedom from Cartesian gradient ***
C
      DEALL(:) = 0.0
      DO I=1,DDEG
        DEALL(I) = DOT_PRODUCT(TRAFO(I,1:DDEG),DE(1:DDEG,1))
      END DO
C
CCC ADLL LA
C
C     *** Project out link atom forces ***
C
C      CALL PRJLA(DEALL,DDEG_ME,'OPT')
C
CCC ADLL LA
C     *** Print Cartesian gradients ***
C
      IF (MPMASTER.AND.PRTOPT) THEN
        CALL PRIDE('CARTESIAN GRADIENTS',DEALL)
      END IF
C
C     *** Contract gradient in order to remove constrains ***
C
      DE1(:) = 0.0
      CALL MULVEC(U,DEALL,DE1,DPRI,DDEG,DACT,'TRANS','NONE',.FALSE.)
C
  100 CONTINUE
C
C     *** Deallocate local fields ****
C
      DEALLOCATE(DEALL,PROTR,TRAFO,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'PROJDE1','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE PROJDE1 ***
C
      END
