      SUBROUTINE QMMMLJ
C
C     Purpose: QM/MM interface Lennard-Jones energy calculations.
C
C     History: - Creation (02.08.10, AG)
C                         (14.04.14, AMK)
C                         (13.11.14, GG)
C
C     ******************************************************************
C
C     List of local variables:
C
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'mmnrgy.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'qmmm.h'
C
      INTEGER ALLOCATION,CNTATM,IA,IATOM,JATOM
      REAL EPS,E6,E12,RED,RMIN,RMIN6,R2,R6,VDWCUT2,XIA,RX,YIA,RY,ZIA,RZ
      REAL,ALLOCATABLE :: XR(:),YR(:),ZR(:)
C
      REAL EPSQMMM,RADQMMM
CCC ADLL LA
      LOGICAL LAPAIR
CCC ADLL LA
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(XR(NATMM),YR(NATMM),ZR(NATMM),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'QMMMLJ','ALLOCATION FAILED')
C
C     *** Initialize parameters ***
C 
      VDWCUT2 = VDWCUT*VDWCUT
C
C     *** Damping function not used if non periodic ***
C     *** Reduction Factors not used for QM coordinates ***
C
C     *** Reduction factors for MM coordinates ***
C
      DO IA=1,NATMM
        IATOM = IA + NATOM
        JATOM = IREDVDW(IA) + NATOM
        RED = REDVDW(IA)
        XR(IA) = COORD(1,IATOM,CCSET)*RED +
     $           COORD(1,JATOM,CCSET)*(1.0 - RED)
        YR(IA) = COORD(2,IATOM,CCSET)*RED +
     $           COORD(2,JATOM,CCSET)*(1.0 - RED)
        ZR(IA) = COORD(3,IATOM,CCSET)*RED +
     $           COORD(3,JATOM,CCSET)*(1.0 - RED)
      END DO
C
C     *** Convert to angstrom ***
C
      XR(:) = XR(:)/BOHR
      YR(:) = YR(:)/BOHR
      ZR(:) = ZR(:)/BOHR
C
C     *** Van der Waals energy from a double loop ***
C
      CNTATM = 0
C
      DO IATOM=1,NATOM
C
C       *** Skip calculation for QM atoms with no VDW parameter ***
C
        IF (VLJRADQM(IATOM) == 0.0) CYCLE
C
        XIA = COORD(1,IATOM,CCSET)/BOHR
        YIA = COORD(2,IATOM,CCSET)/BOHR
        ZIA = COORD(3,IATOM,CCSET)/BOHR
C
        DO JATOM=1,NATMM
C
C         *** Skip calculation for MM atoms with no VDW parameter ***
C
          IF (VLJRAD(JATOM) == 0.0) CYCLE
C
CCC ADLL LA
C        *** Skip calculation for atoms at QM/MM covelent frontier ***
C
          IF (LAPAIR(IATOM,JATOM+NATOM)) CYCLE
C
CCC ADLL LA
          CNTATM = CNTATM + 1
          IF (MOD(CNTATM,NCPU).NE.ME) CYCLE
C
          RX = XIA - XR(JATOM)
          RY = YIA - YR(JATOM)
          RZ = ZIA - ZR(JATOM)
C
          R2 = RX*RX + RY*RY + RZ*RZ
C
          IF (R2.GT.VDWCUT2) THEN
            CALL ERRMSG('QMMMLJ','DISTANCE R LARGER THAN VDW CUTOFF',1)
          END IF
C
          RMIN = RADQMMM(IATOM,JATOM)
          EPS = EPSQMMM(IATOM,JATOM)
C
          RMIN6 = RMIN**6
          R6 = R2**3
C
          E6 = RMIN6/R6
          E12 = E6*E6
C
          EQMMMLJ = EQMMMLJ + EPS*(E12 - 2.0*E6)
C
        END DO
      END DO
C
C     *** Conversion of EQMMMLJ to atomic units ***
C
      EQMMMLJ = EQMMMLJ/KCALMOL
C
      IF (MPP) CALL MP_SUM(EQMMMLJ,EQMMMLJ,1,MP_REAL)
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(XR,YR,ZR,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'QMMMLJ','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE QMMMLJ ***
C
      END
