      SUBROUTINE TYPING 
C
C     Purpose: Assign All properties from the deMon.prm file to
C              every MM atom.
C
C     History: - Creation (17.11.08, AG)
C                         (18.03.09, AG)
C                         (13.11.14, GG)
C                         (06.12.16, TM)
C
C     ******************************************************************
C
C     List of local variables:
C
C     CHGMM : MM atom charge.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'prm.h'
C
      INTEGER I,IATOM,ICHG,II,I1,I2,I3,I4,J,JATOM,JJ,K,
     $        KATOM,KK,LATOM,LL,NN
      CHARACTER*80 PRTSTR,STRCOMP
      REAL DEG,FACT,NOVAL,SIXTH
      LOGICAL PRMERR
C
      PARAMETER (NOVAL=99999.9)
C
C     ------------------------------------------------------------------
C
      SIXTH = 1.0/6.0
      FACT = 2**SIXTH
      PRMERR = .FALSE.
C
C     *** Typing of atomic properties ***
C
      MMCLASS = 0
      NVALMM = 0
      DO IATOM=1,NATMM
        DO J=1,NTYPES
          IF (TYPES(J).EQ.TYPEMM(NATOM+IATOM)) THEN
            MMCLASS(IATOM) = CLASSOFTYP1(TYPES(J))
            NVALMM(IATOM) = NNEIGHMM(TYPES(J))
          END IF
        END DO
      END DO
C
C     *** Check calculated atomic valence with respect to FF values ***
C
      DO IATOM=1,NATMM
        IF ((LINKMM(NATOM+IATOM,0).EQ.1).AND.
     $      (LINKMM(NATOM+IATOM,1).EQ.0)) CYCLE
        IF (LINKMM(NATOM+IATOM,0).NE.NVALMM(IATOM)) THEN
          WRITE (PRTSTR,5010) IATOM,LINKMM(NATOM+IATOM,0)
 5010     FORMAT(' MM ATOM ',I4,' HAS A NON CONSISTENT VALENCE',I6)
          CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
C          GO TO 99991
        END IF
      END DO
C
C     *** Typing atomic VDW reduction factors read from deMon.prm ***
C     *** For monovalent atoms                                    ***
C
C     *** VDWINDX option: vdW read either by CLASS or by TYPE ***
C
      DO I=1,NATMM
        IF (VDWINDX.EQ.'TYPE') THEN
          DO J=1,NTYPES
            IF (TYPEMM(NATOM+I).EQ.TYPES(J)) THEN
              REDVDW(I) = REDUC(J)
              VLJRAD(I) = VRAD(J)
              VLJEPS(I) = VEPS(J)
              IF ((LINKMM(NATOM+I,0).NE.1).OR.(REDVDW(I).EQ.0.0)) THEN
                IREDVDW(I) = I
              ELSE
                IREDVDW(I) = LINKMM(NATOM+I,1) - NATOM
              END IF
            END IF
          END DO
        ELSE
          DO J=1,NCLASS
            IF (MMCLASS(I).EQ.CLASS1(J)) THEN
              REDVDW(I) = REDUC(J)
              VLJRAD(I) = VRAD(J)
              VLJEPS(I) = VEPS(J)
              IF ((LINKMM(NATOM+I,0).NE.1).OR.(REDVDW(I).EQ.0.0)) THEN
                IREDVDW(I) = I
              ELSE
                IREDVDW(I) = LINKMM(NATOM+I,1) - NATOM
              END IF
            END IF
          END DO
        END IF
      END DO
C
C    *** Preparing Type VDW properties ***
C
      DO I=1,NATMM
C
        IF (RADTYPE.EQ.'SIGMA') THEN
          VLJRAD(I) = FACT * VLJRAD(I)
        END IF
C
        IF (RADSIZ.EQ.'DIAMETER') THEN
          VLJRAD(I) = 0.5 * VLJRAD(I)
        END IF
        VLJEPS(I) = ABS(VLJEPS(I))
C
      END DO
C
C     *** No combination rule applied yet for 1-4 VDW radii and pot.     ***
C     *** IF used in CHARM22 VDW14 keword the VRAD14PAIR and VEPS14PAIRa ***
C     *** should be set up. Use of them in LJNRGY should also be added.  ***
C
C     *** See routines kvdw.f and elj.f in Tinker ***
C
C     *** Typing bonds ***
C
      DO I=1,NBOND
        IATOM = CBOND(I,1)
        JATOM = CBOND(I,2)
        II = MMCLASS(IATOM - NATOM)
        JJ = MMCLASS(JATOM - NATOM)
        DO K=1,NBND
          I1 = MMBOND(K,1)
          I2 = MMBOND(K,2)
          IF ((II.EQ.I1.AND.JJ.EQ.I2).OR.(II.EQ.I2.AND.JJ.EQ.I1)) THEN
            KBOND(I) = KBND(K)
            BOND0(I) = BND0(K)
          END IF
        END DO
C
C       *** Check for undefined parameter ***
C
        IF ((KBOND(I).EQ.NOVAL).OR.(KBOND(I).EQ.0.0)) THEN
          WRITE (OUT,5020) IATOM,JATOM,II,JJ
 5020     FORMAT('BOND',T9,I5,',',I5,
     $           T33,'(CLASS ',I3,',',I3,')',
     $           T57,'IS NOT DEFINED IN THE FF')
          PRMERR = .TRUE.
        END IF
C
      END DO
C
C     *** Typing Bond Angles ***
C
      DO I=1,NBANGS
        IATOM = BANGS(I,1)
        JATOM = BANGS(I,2)
        KATOM = BANGS(I,3)
        II = MMCLASS(IATOM - NATOM)
        JJ = MMCLASS(JATOM - NATOM)
        KK = MMCLASS(KATOM - NATOM)
        DO K=1,NANG
          I1 = MMBANG(K,1)
          I2 = MMBANG(K,2)
          I3 = MMBANG(K,3)
          IF (JJ.EQ.I2) THEN
            IF ((II.EQ.I1.AND.KK.EQ.I3).OR.(II.EQ.I3.AND.KK.EQ.I1)) THEN
              KANGLE(I) = KBANG(K)
              ANGLE0(I) = BANG0(K,1)
            END IF
          END IF
        END DO
        BANGTYPE(I) = 'HARMONIC'
C
C       *** Check for undefined parameter ***
C
        IF ((KANGLE(I).EQ.NOVAL).OR.(KANGLE(I).EQ.0.0)) THEN
          WRITE (OUT,5030) IATOM,JATOM,KATOM,II,JJ,KK
 5030     FORMAT('ANGLE',T9,I5,',',I5,',',I5,
     $           T33,'(CLASS ',I3,',',I3,',',I3,')',
     $           T57,'IS NOT DEFINED IN THE FF')
          PRMERR = .TRUE.
        END IF
C
      END DO
C
C     *** Typing Urey - Bradley 1-3 Interactions ***
C
      NUREYB = 0
      DO I=1,NBANGS
        IATOM = BANGS(I,1)
        JATOM = BANGS(I,2)
        KATOM = BANGS(I,3)
        II = MMCLASS(IATOM - NATOM)
        JJ = MMCLASS(JATOM - NATOM)
        KK = MMCLASS(KATOM - NATOM)
        DO K=1,NUREY
          I1 = MMUREY(K,1)
          I2 = MMUREY(K,2)
          I3 = MMUREY(K,3)
           IF (JJ.EQ.I2) THEN
            IF ((II.EQ.I1.AND.KK.EQ.I3).OR.(II.EQ.I3.AND.KK.EQ.I1)) THEN
              NUREYB = NUREYB + 1
              UREY(NUREYB,1) = IATOM
              UREY(NUREYB,2) = KATOM
              KUREYB(NUREYB) = KUREY(K)
              UREYB0(NUREYB) = UREY0(K)
            END IF
          END IF
        END DO
      END DO
C
C     *** Typing torsion angles ***
C
      DO I=1,NTANGS
        IATOM = TANGS(I,1)
        JATOM = TANGS(I,2)
        KATOM = TANGS(I,3)
        LATOM = TANGS(I,4)
        II = MMCLASS(IATOM - NATOM)
        JJ = MMCLASS(JATOM - NATOM)
        KK = MMCLASS(KATOM - NATOM)
        LL = MMCLASS(LATOM - NATOM)
        DO K=1,NTANG
          I1 = MMTANG(K,1)
          I2 = MMTANG(K,2)
          I3 = MMTANG(K,3)
          I4 = MMTANG(K,4)
          IF (((II == I1).AND.(JJ == I2).AND.
     $         (KK == I3).AND.(LL == I4)).OR.
     $        ((II == I4).AND.(JJ == I3).AND.
     $         (KK == I2).AND.(LL == I1))) THEN
            TORS1(I,1) = TANG1(K,1)
            TORS1(I,2) = TANG1(K,2)
            TORS2(I,1) = TANG2(K,1)
            TORS2(I,2) = TANG2(K,2)
            TORS3(I,1) = TANG3(K,1)
            TORS3(I,2) = TANG3(K,2)
            TORS4(I,1) = TANG4(K,1)
            TORS4(I,2) = TANG4(K,2)
          END IF
        END DO
C
C       *** Check for undefined parameter ***
C
        IF ((TORS1(I,1).EQ.NOVAL).AND.(TORS2(I,1).EQ.NOVAL).AND.
     $      (TORS3(I,1).EQ.NOVAL).AND.(TORS4(I,1).EQ.NOVAL)) THEN
          WRITE (OUT,5040) IATOM,JATOM,KATOM,LATOM,II,JJ,KK,LL
 5040     FORMAT('TORSION',I6,',',I5,',',I5,',',I5,
     $           ' (CLASS ',I3,',',I3,',',I3,',',I3,
     $           ') IS NOT DEFINED IN THE FF')
          PRMERR = .TRUE.
        END IF
C
      END DO
C
C     *** Typing improper torsions angles ***
C
      NIMPTORS = 0
      DO I=1,NIMPTS
        IATOM = IMPT(I,1)
        JATOM = IMPT(I,2)
        KATOM = IMPT(I,3)
        LATOM = IMPT(I,4)
        II = MMCLASS(IATOM - NATOM)
        JJ = MMCLASS(JATOM - NATOM)
        KK = MMCLASS(KATOM - NATOM)
        LL = MMCLASS(LATOM - NATOM)
        DEG = 1.0
        IF ((II.EQ.JJ).OR.(II.EQ.LL).OR.(JJ.EQ.LL)) DEG = 2.0
        IF ((II.EQ.JJ).AND.(II.EQ.LL).AND.(JJ.EQ.LL)) DEG = 6.0
        DO K=1,NIMTORS
          I1 = MMIMPTANG(K,1)
          I2 = MMIMPTANG(K,2)
          I3 = MMIMPTANG(K,3)
          I4 = MMIMPTANG(K,4)
          IF (KK.EQ.I3) THEN
            IF ((II.EQ.I1).AND.(JJ.EQ.I2).AND.(LL.EQ.I4)) THEN
              NIMPTORS = NIMPTORS + 1
              IIMTORS(NIMPTORS,1) = IATOM
              IIMTORS(NIMPTORS,2) = JATOM
              IIMTORS(NIMPTORS,3) = KATOM
              IIMTORS(NIMPTORS,4) = LATOM
              IMPTORS1(NIMPTORS,1) = IMPTANG1(K,1)/DEG
              IMPTORS1(NIMPTORS,2) = IMPTANG1(K,2)
              IMPTORS2(NIMPTORS,1) = IMPTANG2(K,1)/DEG
              IMPTORS2(NIMPTORS,2) = IMPTANG2(K,2)
              IMPTORS3(NIMPTORS,1) = IMPTANG3(K,1)/DEG
              IMPTORS3(NIMPTORS,2) = IMPTANG3(K,2)
              J = NIMPTORS
C
C             *** Check for undefined parameter ***
C
              IF ((IMPTORS1(NIMPTORS,1).EQ.NOVAL).AND.
     $            (IMPTORS2(NIMPTORS,1).EQ.NOVAL).AND.
     $            (IMPTORS3(NIMPTORS,1).EQ.NOVAL)) THEN
                WRITE (OUT,5050) IATOM,JATOM,KATOM,LATOM,II,JJ,KK,LL
 5050           FORMAT('IMPTORS',I6,',',I5,',',I5,',',I5,
     $                 ' (CLASS ',I3,',',I3,',',I3,',',I3,
     $                 ') IS NOT DEFINED IN THE FF')
                PRMERR = .TRUE.
              END IF
C
            END IF
          END IF
        END DO
      END DO
C
C     *** Exit if undefined parameter were found ***
C
      IF (PRMERR) CALL ERRMSG('TYPING','UNDEFIENED MM PARAMETER',1)
C
C    *** Typing electrostatic atomic Properties ***
C
      DO IATOM=1,NATMM
        NN = 0
C
        IF (NCHARGE.GT.0) THEN
C
          DO ICHG=1,NCHARGE
            IF (TYPEMM(NATOM+IATOM).EQ.MMCHG(ICHG)) THEN
              NN = 1
              CHGMM(IATOM) = CHG(ICHG)
            END IF
          END DO
C
        END IF
C
        IF (NN.EQ.0) THEN
          WRITE (PRTSTR,5000) IATOM
 5000     FORMAT(' MM ATOM ',I4,' HAS NO DEFINED TYPE')
          CALL MESSAGE(1,STRCOMP(PRTSTR),1,0)
          GO TO 99990
        END IF
      END DO
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
99990 CONTINUE
      WRITE (PRTSTR,'("ERROR IN MM ATOMS FILE LINE: ",I10)') IATOM
      CALL ERRMSG('TYPING',STRCOMP(PRTSTR),1)
      RETURN
99991 CONTINUE
      WRITE (PRTSTR,'("ERROR IN deMon.mm VERSUS VALENCE IN deMon.prm")')
      CALL ERRMSG('TYPING',STRCOMP(PRTSTR),1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TYPING ***
C
      END
