      SUBROUTINE CHKQMMMBRD
C
C     Purpose: Check if QM/MM partition cuts covalent bond.
C
C     History:
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
      INCLUDE 'qmmm.h'
C
      INTEGER IATOM,ILA,ILAI,ILINK,LATOM,NLACONNEC
      REAL DLQ,DMQ,SP
      REAL RLQ(3),RMQ(3)

C
C     -----------------------------------------------------------------
C
      LINKAT = .FALSE.
C
C     *** Add link atoms to atom list if no ***
C
      IF (NLATOM.EQ.0) THEN
        LATR(:) = 0.0
        LATQM(:,:) = 0
C
        DO 10 IATOM=1,NATOM_ALL
          DO 20 ILINK=1,LINKMM(IATOM,0)
            IF (LINKMM(IATOM,ILINK).EQ.0) CYCLE
            IF (TYPEMM(LINKMM(IATOM,ILINK)).LT.0) THEN
              NLATOM = NLATOM + 1
              IF (NLATOM.GT.MAXLNKAT) GO TO 99950 
              LATQM(NLATOM,1) = LINKMM(IATOM,ILINK)
              LATQM(NLATOM,2) = IATOM
              LATQM(NLATOM,3) = NATOM_ALL + NLATOM
              CYCLE
            END IF
 20       CONTINUE 
 10     CONTINUE
C
C     *** if LA declared in input geometry or if already defined ***
C
      ELSE 
C
        IF (LATQM(1,1).EQ.0) THEN
C
C     *** Check compatibility between input LA and QM/MM links *** 
C
          NLACONNEC = 0
          DO 30 IATOM=1,NATOM_ALL
            DO 40 ILINK=1,LINKMM(IATOM,0)
              IF (LINKMM(IATOM,ILINK).EQ.0) CYCLE
              IF (TYPEMM(LINKMM(IATOM,ILINK)).LT.0) THEN
                NLACONNEC = NLACONNEC + 1
                CYCLE
              END IF
 40         CONTINUE
 30       CONTINUE
C
          IF (NLACONNEC.NE.NLATOM) GO TO 99960
C
C     *** Associate LA declared in input geometry to QM/MM links ***
C
          ILA = 0
          DO 50 IATOM=1,NATOM_ALL
            DO 60 ILINK=1,LINKMM(IATOM,0)
              IF (LINKMM(IATOM,ILINK).EQ.0) CYCLE
              IF (TYPEMM(LINKMM(IATOM,ILINK)).LT.0) THEN
                ILA = ILA + 1
                LATQM(ILA,1) = LINKMM(IATOM,ILINK)
                LATQM(ILA,2) = IATOM
                RMQ(:) = COORD(:,IATOM,CCSET) -
     $                   COORD(:,LATQM(ILA,1),CCSET)
                DMQ = SQRT(DOT_PRODUCT(RMQ,RMQ))
                RMQ(:) = RMQ(:)/DMQ
C
                DO 70 ILAI=1,NLATOM
                  LATOM = LINKLA(ILAI,0)
                  RLQ(:) = COORD(:,LATOM,CCSET) - 
     $                     COORD(:,LATQM(ILA,1),CCSET)
                  DLQ = SQRT(DOT_PRODUCT(RLQ,RLQ))
                  RLQ(:) = RLQ(:)/DLQ
                  SP = DOT_PRODUCT(RMQ,RLQ)
                  IF ((SP.GT.0.95).AND.(DMQ.GT.DLQ)) THEN 
                    LATQM(ILA,3) = LATOM
                    GO TO 50 
                  END IF
 70             CONTINUE
                CYCLE
              END IF
 60         CONTINUE
 50       CONTINUE
C
C     *** Update atom numbering with current geometry ***
C
        ELSE
C
          ILA = 0
          DO 80 IATOM=1,NATOM_ALL
            IF (INDEX('HLFL',ATOMLAB(ATOMMAP(IATOM,1))(1:2)).NE.0) THEN
              ILA = ILA + 1
              LINKLA(ILA,0) = IATOM
            END IF
 80       CONTINUE 
C
          ILA = 0
          DO 90 IATOM=1,NATOM_ALL
            DO 100 ILINK=1,LINKMM(IATOM,0)
              IF (LINKMM(IATOM,ILINK).EQ.0) CYCLE
              IF (TYPEMM(LINKMM(IATOM,ILINK)).LT.0) THEN
                ILA = ILA + 1
                LATQM(ILA,1) = LINKMM(IATOM,ILINK)
                LATQM(ILA,2) = IATOM
                RMQ(:) = COORD(:,IATOM,CCSET) -
     $                   COORD(:,LATQM(ILA,1),CCSET)
                DMQ = SQRT(DOT_PRODUCT(RMQ,RMQ))
                RMQ(:) = RMQ(:)/DMQ
C
                DO 110 ILAI=1,NLATOM
                  LATOM = LINKLA(ILAI,0)
                  RLQ(:) = COORD(:,LATOM,CCSET) -
     $                     COORD(:,LATQM(ILA,1),CCSET)
                  DLQ = SQRT(DOT_PRODUCT(RLQ,RLQ))
                  RLQ(:) = RLQ(:)/DLQ
                  SP =   DOT_PRODUCT(RMQ,RLQ)
                  IF ((SP.GT.0.95).AND.(DMQ.GT.DLQ)) THEN
                    LATQM(ILA,3) = LATOM
                    CYCLE
                  END IF
110             CONTINUE
                CYCLE
              END IF
100         CONTINUE
 90       CONTINUE
        END IF
      END IF
C   
      IF (NLATOM.NE.0)  LINKAT = .TRUE.
C
      RETURN
C
C     *** Error handling ***
C
99950 CONTINUE
      CALL ERRMSG('CHKQMMMBRD','NUMBER OF LINK ATOMS EXCEED MACLNKAT',1)
C
99960 CONTINUE
      WRITE (OUT,99965) 
99965 FORMAT (/,/,T2,'************************************************',
     $          /,T2,'***  MISMATCH BETWEEN LINK ATOMS PRESENT IN  ***',
     $          /,T2,'***  GEOMETRY AND NUMBER OF DECLARED LINK    ***',
     $          /,T2,'***      ATOMS BASED ON CONNCECTIVITY        ***',
     $          /,T2,'************************************************')
C
      CALL ERRMSG('CHKQMMMBRD','ERROR WITH LINK ATOMS',1)
C
99970 CONTINUE
      WRITE (OUT,99975) LATQM(ILA,1),LATQM(ILA,2)
99975 FORMAT (/,/,T2,'************************************************',
     $          /,T2,'***  NO CONSISTENT LINK ATOM POSITION FOR    ***',
     $          /,T2,'***  BORDER BETWEEN QM ATOM ',I6,'      AND  ***',
     $          /,T2,'***    MM ATOM ',I6,'                        ***',
     $          /,T2,'************************************************')
C
      CALL ERRMSG('CHKQMMMBRD','ERROR WITH LINK ATOMS COORDINATES',1)
C
      STOP
C
C     -----------------------------------------------------------------_
C   
C     *** End of SUBROUTINE chkqmmmbrd ***
C 
      END
