      SUBROUTINE DE1QMMMFRT(GRADIENT,DXYZ)
C
C     Purpose: Molecular mechanics energy 1st derivatives involving QM
C              atoms at the QM/MM frontier.
C
C     History: - Creation (27.01.18, ADLL) 
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DXYZ: Dimension of x, y or z degrees of freedom.
C
C     List of local variables:
C
C     D1BOND  : Local coordinate system bond energy derivative.
C     D1R12   : Derivative of R12 with respect to atomic coordinates.
C     GRADIENT: Gradient vector.
C     KF      : Bond force constant.
C     R0      : Equilibrium bond length.
C     R12     : Bond distance.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'math.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
CCC ADLL
      INCLUDE 'qmmm.h'
CCC ADLL
C
      INTEGER DXYZ
      REAL GRADIENT(3,DXYZ)
C
      INTEGER IATOM,IBANG,IBOND,ITORS,JATOM
      REAL ANG,A0,D1,D2,D3,D1BOND,DELTA,DELTA2,DELTA3,DELTA4,DER,EXPO
      REAL PHI,KF,R0,R12
C
      INTEGER AT(4)
      REAL D1ANG(3,4),D1PHI(3,4),D1R12(3),GAM(3)
      REAL V(3),X(4),Y(4),Z(4)
C
C     ------------------------------------------------------------------
C
      IF (MPSLAVE) RETURN
C
C     *** Loop over all MM bonds ***
C
      DO IBOND=1,NLATOM
C
C     *** Load force field parameters ***
C
        KF = KLBOND(IBOND)
        R0 = LBOND0(IBOND)
C
C     *** Load atoms of the current bond ***
C
        IATOM = LBOND(IBOND,1)
        JATOM = LBOND(IBOND,2)
C
C     *** Calculate bond length 1st derivative ***
C
        D1R12(:) = COORD(:,IATOM,CCSET) - COORD(:,JATOM,CCSET)
        R12 = SQRT(D1R12(1)**2 + D1R12(2)**2 + D1R12(3)**2)
        D1R12(:) = D1R12(:)/R12
        DELTA = R12/BOHR - R0
C
C     *** Calculate harmonic potential 1st derivative ***
C
        IF (BONDTYPE.EQ.'HARMONIC') THEN
C
          D1BOND = 2.0*KF*DELTA
C
C     *** Calculate Morse potential 1st derivative ***
C
        ELSE IF (BONDTYPE.EQ.'MORSE') THEN
C
          EXPO = EXP(-2.0*DELTA)
          D1BOND = KF*EXPO*(1.0 - EXPO)
C
        END IF
C
C     *** Convert bond energy derivative to atomic units ***
C
        D1BOND = D1BOND/(BOHR*KCALMOL)
C
C     *** Add bond energy derivative to gradient vector ***
C
        GRADIENT(:,IATOM) = GRADIENT(:,IATOM) + D1BOND*D1R12(:)
        GRADIENT(:,JATOM) = GRADIENT(:,JATOM) - D1BOND*D1R12(:)
C
      END DO
C
      DO IBANG=1,NLBANG
C
        AT(1:3) = LBANGS(IBANG,1:3)
        X(1:3) = COORD(1,AT(1:3),CCSET)
        Y(1:3) = COORD(2,AT(1:3),CCSET)
        Z(1:3) = COORD(3,AT(1:3),CCSET)
C
        KF = KLANG(IBANG)
        A0 = LANG0(IBANG)*DEGRAD
C
        CALL D1ANGLE(1,2,3,X(1:3),Y(1:3),Z(1:3),ANG,D1ANG)
C
        DELTA = ANG - A0
C
        DELTA2 = DELTA*DELTA
        DELTA3 = DELTA2*DELTA
        DELTA4 = DELTA3*DELTA
        DER = KF*DELTA*(2.0 + 3.0*CUBBANG*DELTA + 4.0*QUABANG*DELTA2 +
     $                        5.0*PENBANG*DELTA3 + 6.0*SEXBANG*DELTA4)
C
C     *** Convert to atomic units ***
C
        DER = DER/KCALMOL
C
C     *** Gradient of EBANG ***
C
        GRADIENT(:,AT(1:3)) = GRADIENT(:,AT(1:3)) + DER*D1ANG(:,1:3)
      END DO
C
      DO ITORS=1,NLTANG
C
        AT(:) = LTANGS(ITORS,:)
        X(:) = COORD(1,AT(:),CCSET)
        Y(:) = COORD(2,AT(:),CCSET)
        Z(:) = COORD(3,AT(:),CCSET)
        V(1) = LTORS1(ITORS,1)
        V(2) = LTORS2(ITORS,1)
        V(3) = LTORS3(ITORS,1)
        GAM(1) = LTORS1(ITORS,2)*DEGRAD
        GAM(2) = LTORS2(ITORS,2)*DEGRAD
        GAM(3) = LTORS3(ITORS,2)*DEGRAD
C
        CALL D1TORSION(1,2,3,4,X,Y,Z,PHI,D1PHI)
C
        D1 = -SIN(PHI)*COS(GAM(1)) + COS(PHI)*SIN(GAM(1))
        D2 = 2.0*(-SIN(2.0*PHI)*COS(GAM(2)) + COS(2.0*PHI)*SIN(GAM(2)))
        D3 = 3.0*(-SIN(3.0*PHI)*COS(GAM(3)) + COS(3.0*PHI)*SIN(GAM(3)))
        DER = TORSUNIT*(V(1)*D1 + V(2)*D2 + V(3)*D3)
C
C     *** Convert to atomic units ***
C
        DER = DER/KCALMOL
C
C     *** Gradient of ETORS ***
C
        GRADIENT(:,AT(:)) = GRADIENT(:,AT(:)) + DER*D1PHI(:,:)
C
      END DO
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DE1BONDLA ***
C
      END
