      SUBROUTINE EQMMMFRT 
C
C     Purpose: Molecular mechanics terms involving QM atoms at the QM/MM
C              frontier. 
C
C     History: - Creation (27.07.18, ADLL) 
C
C     ******************************************************************
C
C     List of local variables:
C
C     KF   : Force constant.
C     R0   : Reference values 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'math.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'qmmm.h'
      INCLUDE 'physic.h'
C
      INTEGER  AT(4),IATOM,IBANG,IBOND,ITANG,JATOM
      REAL E21(3),E23(3),GAM(3),V(3),X(4),Y(4),Z(4)
      REAL A0,ANG,DELTA,DELTA2,DELTA3,DELTA4,EMMLA,EXPO,KF,PHI,Q1,Q2,Q3
      REAL R0,R12,R23
C
C     ------------------------------------------------------------------
C
      IF (MPSLAVE) RETURN
C
      EMMLA = 0.0
C
C     *** Loop over all MM bonds bond at QM/MM frontier *** 
C
      DO IBOND=1,NLATOM
C
        KF = KLBOND(IBOND)
        R0 = LBOND0(IBOND)
C
        IATOM = LBOND(IBOND,1)
        JATOM = LBOND(IBOND,2)
C
        DELTA = SUM((COORD(:,IATOM,CCSET) - COORD(:,JATOM,CCSET))**2)
        DELTA = SQRT(DELTA)/BOHR - R0
C
C     *** Calculate harmonic potential bond energy ***
C
        IF (BONDTYPE.EQ.'HARMONIC') THEN
C
          EMMLA = EMMLA + KF*DELTA**2
C
C     *** Calculate Morse potential bond energy ***
C
        ELSE IF (BONDTYPE.EQ.'MORSE') THEN
C
          EXPO = EXP(-2.0*DELTA)
          EMMLA = EMMLA + 0.25*KF*(1.0 - EXPO)**2
C
        END IF
C
      END DO
C
C     *** Conversion of EBOND to atomic units ***
C
      EBOND = EBOND + EMMLA/KCALMOL
C
C     *** Loop over all MM bond angle accross link atoms ***
C
      EMMLA = 0.0
C
      DO IBANG=1,NLBANG
C
        AT(1:3) = LBANGS(IBANG,1:3)
        X(1:3) = COORD(1,AT(1:3),CCSET)
        Y(1:3) = COORD(2,AT(1:3),CCSET)
        Z(1:3) = COORD(3,AT(1:3),CCSET)
C
        KF = KLANG(IBANG)
        A0 = LANG0(IBANG)*DEGRAD
C
        CALL GETBENDS(1,2,3,X(1:3),Y(1:3),Z(1:3),E21,E23,R12,R23,ANG)
C
        DELTA = ANG - A0
C
        DELTA2 = DELTA*DELTA
        DELTA3 = DELTA2*DELTA
        DELTA4 = DELTA3*DELTA
        EMMLA = EMMLA + KF*DELTA2*(1.0 + CUBBANG*DELTA +
     $                                   QUABANG*DELTA2 +
     $                                   PENBANG*DELTA3 +
     $                                   SEXBANG*DELTA4)
C
      END DO
C
      EBANG = EBANG + EMMLA/KCALMOL
C
C     *** Loop over all MM torsion angle at QM/MM frontier atoms ***

      EMMLA = 0.0
      DO ITANG=1,NLTANG
C
        V(1) = LTORS1(ITANG,1)
        V(2) = LTORS2(ITANG,1)
        V(3) = LTORS3(ITANG,1)
C
        IF (ALL(V(:).EQ.0.0)) CYCLE
C
        AT(:) = LTANGS(ITANG,:)
C
        X(:) = COORD(1,AT(:),CCSET)
        Y(:) = COORD(2,AT(:),CCSET)
        Z(:) = COORD(3,AT(:),CCSET)
C
        GAM(1) = LTORS1(ITANG,2)*DEGRAD
        GAM(2) = LTORS2(ITANG,2)*DEGRAD
        GAM(3) = LTORS3(ITANG,2)*DEGRAD
C
        CALL GETDIEDER(1,2,3,4,X,Y,Z,PHI)
C
        Q1 = 1.0 + COS(PHI     - GAM(1))
        Q2 = 1.0 + COS(2.0*PHI - GAM(2))
        Q3 = 1.0 + COS(3.0*PHI - GAM(3))
C
        EMMLA = EMMLA + TORSUNIT*(V(1)*Q1 + V(2)*Q2 + V(3)*Q3)
C
      END DO
C
C    *** Conversion of ETORS to A.U. ***
C
      ETORS = ETORS + EMMLA/KCALMOL
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE BONDLA ***
C
      END
