      SUBROUTINE NEWCART(LINE,CSET,TAPE)
C
C     Purpose: Write new Cartesian coordinates.
C
C     History: - Creation (23.05.05, AMK)
C                         (30.10.06, UR)
C                         (13.11.14, GG)
C                         (10.11.16, AMK)
C                         (27.01.17, TM)
C
C     ******************************************************************
C
C     List of local variables:
C
C     CSET: Cartesian coordinate set.
C     LINE: Cartesian coordinates comment line.
C     TAPE: File tape for coordinate writing.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'dynamics.h'
      INCLUDE 'flags.h'
      INCLUDE 'mm.h'
      INCLUDE 'molecule.h'
C
      CHARACTER LINE*(*)
C
      CHARACTER*80 PRTSTR
      CHARACTER ELNUM*10,LABLEN*2,LFMT*33
      INTEGER CSET,LLBEN,LLDIH,NATOMS,TAPE
      INTEGER IATOM,INEIGH,IVAR,JATOM,LINELEN,NDIG
      LOGICAL NEWCON,WCONST
      INTEGER STREXT
C
      INTEGER ALLOCATION
CCC ADLL
      INTEGER ZEROI
CCC ADLL
      INTEGER,ALLOCATABLE :: Z(:)
      CHARACTER*10,ALLOCATABLE :: LABEL(:)
      CHARACTER*13,ALLOCATABLE :: VECTOR(:)
C
C     ------------------------------------------------------------------
C
C     *** Initialization ***
C
      NEWCON = .FALSE.
      WCONST = .FALSE.
C
      NATOMS = NATOM_ALL + NDUMMY
CCC ADLL
      ZEROI = 0
CCC ADLL
C
C     *** Allocate local field ***
C
      ALLOCATE(LABEL(NATOMS),VECTOR(NATOMS),Z(NATOMS),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('NEWCART','ALLOCATION FAILED',1)
      END IF
C
C     *** Change atomic labels for deMon.new output ***
C
      LABEL(1:NATOMS) = ATOMLAB(1:NATOMS)
C
      DO IATOM=1,NATOMS
        IF (ATOMLAB(IATOM)(1:2).EQ.'XX') THEN
          ATOMLAB(IATOM) = ELNUM('X',NATOMS)
        END IF
      END DO
C
C     *** Maximum character length of atom labels ***
C
      WRITE (LABLEN,'(I2.2)') MAXVAL(LEN_TRIM(ATOMLAB(1:NATOMS)))
C
C     *** Calculate nuclear charges ***
C
      Z(:) = 0
      IF (ECPS.OR.MCPS) THEN
        Z(1:NATOMS) = ABS(ZATOM(1:NATOMS)) + ZCORE(1:NATOMS)
      ELSE
        Z(1:NATOMS) = ABS(ZATOM(1:NATOMS))
      END IF
C
C     *** Print Cartesian coordinates comment and header lines ***
C
      LINELEN = STREXT(LINE)
      WRITE (TAPE,'(A)') COMMENT
      WRITE (PRTSTR,5000) COMMENT,LINE(1:LINELEN)
      WRITE (TAPE,'(A)') PRTSTR
      WRITE (TAPE,'(A)') COMMENT
C
C     *** Write Cartesian coordinates ***
C 
      WRITE (TAPE,5010) UNIT(1)
      DO IATOM=1,NATOMS
        IF (ZATOM(IATOM).GE.0) THEN
          LFMT = '(A'//LABLEN//',1X,3F12.6,2X,I3,1X,F10.6)'
          WRITE (TAPE,LFMT) ATOMLAB(ATOMMAP(IATOM,1)),
     $                      ANGBOR(1)*COORD(:,IATOM,CSET),
     $                      Z(IATOM),MATOM(IATOM)
        ELSE
CCC ADLL LA CHARGE FORMAT I4--> I5
          LFMT = '(A'//LABLEN//',1X,3F12.6,1X,I5,1X,F10.6)'
CCC ADLL LA
          WRITE (TAPE,LFMT,ADVANCE='NO') ATOMLAB(ATOMMAP(IATOM,1)),
     $                                   ANGBOR(1)*COORD(:,IATOM,CSET),
     $                                   -TYPEMM(IATOM),MATOM(IATOM)
          IF ((LINKMM(IATOM,0).EQ.1).AND.(LINKMM(IATOM,1).EQ.0)) THEN
            WRITE (TAPE,'(" 0")')
          ELSE
            LFMT = '(I  )'
            DO INEIGH=1,LINKMM(IATOM,0)
CCC ADLL
              IF (LINKMM(IATOM,INEIGH).EQ.0) THEN
                WRITE (TAPE,LFMT,ADVANCE='NO') ZEROI 
              ELSE
                NDIG = INT(LOG10(REAL(LINKMM(IATOM,INEIGH)))) + 2
                WRITE (LFMT(3:4),'(I2.2)') NDIG
                WRITE (TAPE,LFMT,ADVANCE='NO') LINKMM(IATOM,INEIGH)
              END IF
            END DO
            WRITE (TAPE,'()')
          END IF
        END IF
      END DO
C
C     *** Write Cartesian constants ***
C
      DO IATOM=1,NATOMS
        IF (OPTXYZ(IATOM).NE.' ') WCONST = .TRUE.
      END DO 
C
      IF (WCONST) THEN 
        WRITE (TAPE,5020)
        LFMT = '(A'//LABLEN//',3X,A3)'
        VECTOR(1:NATOMS) = ATOMLAB(1:NATOMS)//OPTXYZ(1:NATOMS)
        DO IATOM=1,NATOMS
          IF (OPTXYZ(IATOM).NE.' ') THEN
            NEWCON = .TRUE.
            DO JATOM=1,IATOM-1
              IF (VECTOR(IATOM).EQ.VECTOR(JATOM)) NEWCON = .FALSE.
            END DO
            IF (NEWCON) WRITE (TAPE,LFMT(1:11)) ATOMLAB(IATOM),
     $                                          OPTXYZ(IATOM)
          END IF
        END DO
      END IF
C
C     *** Write BOMD constraints ***
C
      IF (NFIXVAR(0).GT.0) THEN
        IF (WCONST) THEN
          PRTSTR = 'CONSTANT/CONSTRAINT DEFINITION CONFLICT'
          CALL ERRMSG('NEWCART',PRTSTR,1)
        END IF
        WRITE (TAPE,5030)
        LLBEN = NFIXVAR(1)
        LLDIH = NFIXVAR(1) + NFIXVAR(2)
        DO IVAR=1,NFIXVAR(1)
          LFMT = '(I5,2X,I5)'
          WRITE (TAPE,LFMT(1:10)) ATOMMAP(FIXVAR(IVAR,1),2),
     $                            ATOMMAP(FIXVAR(IVAR,2),2)
        END DO
        DO IVAR=1,NFIXVAR(2)
          LFMT = '(I5,2X,I5,2X,I5)'
          WRITE (TAPE,LFMT(1:16)) ATOMMAP(FIXVAR(LLBEN+IVAR,1),2),
     $                            ATOMMAP(FIXVAR(LLBEN+IVAR,2),2),
     $                            ATOMMAP(FIXVAR(LLBEN+IVAR,3),2)
        END DO 
        DO IVAR=1,NFIXVAR(3)
          LFMT = '(I5,2X,I5,2X,I5,2X,I5)'
          WRITE (TAPE,LFMT(1:22)) ATOMMAP(FIXVAR(LLDIH+IVAR,1),2),
     $                            ATOMMAP(FIXVAR(LLDIH+IVAR,2),2),
     $                            ATOMMAP(FIXVAR(LLDIH+IVAR,3),2),
     $                            ATOMMAP(FIXVAR(LLDIH+IVAR,4),2)
        END DO
      END IF
C
C     *** Restore original atomic labels ***
C
      ATOMLAB(1:NATOMS) = LABEL(1:NATOMS)
C
C     *** Deallocate local field ***
C
      DEALLOCATE(LABEL,VECTOR,Z,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('NEWCART','DEALLOCATION FAILED',1)
      END IF
C      
C     *** Flush output tape file ***
C
      CALL DEFLUSH(TAPE)
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Format statments ***
C
 5000 FORMAT (A,1X,A)
 5010 FORMAT ('GEOMETRY CARTESIAN',1X,A)
 5020 FORMAT ('CONSTANTS')
 5030 FORMAT ('CONSTRAINTS')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NEWCART ***
C
      END
