      SUBROUTINE PRTMM 
C
C     Purpose: Print summary of MM energy terms in deMon.top file.
C
C     History: - Creation (16.11.18, ADLL)
C
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'qmmm.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'strings.h'
C
      INTEGER I,IATOM,ITYP,J,JATOM
      INTEGER QMATYP(MAXATOM)
      REAL RVALUE
      CHARACTER*80 ATOMTYPE*2
      CHARACTER*10 LTU,SYMBOL
C
C     ------------------------------------------------------------------
C
      IF (MPSLAVE.OR.QMONLY) RETURN
C
      OPEN (FFTOP,FILE='deMon.top',STATUS='UNKNOWN',
     $      POSITION='REWIND',ACCESS='SEQUENTIAL')
C
C     *** Determine the MM class for the QM atoms ***
C
      DO IATOM=1,NATOM
        DO ITYP=1,NTYPQM
          SYMBOL = TRIM(LABTYPQM(ITYP))
          IF (SYMBOL.EQ.LTU(ATOMLAB(ATOMMAP(IATOM,1)))) THEN
            QMCLASS(IATOM) = CLASSOFTYP2(TYPEQM(ITYP))
            QMATYP(IATOM) = TYPEQM(ITYP)
          ELSE IF (SYMBOL.EQ.ADJUSTL(LTU(ELSYM(ATOMNO(IATOM))))) THEN
            QMCLASS(IATOM) = CLASSOFTYP2(TYPEQM(ITYP))
            QMATYP(IATOM) = TYPEQM(ITYP)
          END IF
        END DO
      END DO
C
C     *** Write list of atoms ***
C
      WRITE(FFTOP,4000)
 4000 FORMAT (T1,'ABS NO',4X,'NO',2X,'LABEL',7X,'TYPE',2X,'CLASS',
     $        8X,'CHARGE',7X,'APOL',7X,'LJ RAD',7X,'LJ EPS',
     $        2X,'ATOM TYPE')
C
      IF (QMMM) THEN
        RVALUE = 0.0
        DO IATOM=1,NATOM
          WRITE (FFTOP,5000) IATOM,IATOM,ATOMLAB(ATOMMAP(IATOM,1)),
     $                      QMATYP(IATOM),QMCLASS(IATOM),
     $                      REAL(ZATOM(IATOM)),RVALUE,VLJRADQM(IATOM),
     $                      VLJEPSQM(IATOM),ATOMTYPE(IATOM)
        END DO
      END IF
      DO IATOM=1,NATMM
        JATOM = NATOM+IATOM 
        WRITE (FFTOP,5000) JATOM,IATOM,ATOMLAB(ATOMMAP(JATOM,1)),
     $                    TYPEMM(JATOM),MMCLASS(IATOM),CHGMM(IATOM),
     $                    APOLMM(IATOM)/(BOHR**3),VLJRAD(IATOM),
     $                    VLJEPS(IATOM),ATOMTYPE(JATOM)
      END DO
C
 5000 FORMAT (T1,2(I5,1X),5X,A7,2X,2(I4,3X),4(F11.6,2X),2X,A2)
C
C     *** Print bonding terms ***
C
      IF (DOBOND) THEN
C
        WRITE (FFTOP,5005) NBOND
 5005   FORMAT (//,T1,'NUMBER OF BONDS:',1X,I6,/,
     $           T1,'BOND NO FOLLOWED BY BONDED ATOMS (ABS NO)')
        J = 1
        DO WHILE (J+2.LE.NBOND)
          WRITE(FFTOP,5010) (I,CBOND(I,1),CBOND(I,2),I=J,J+2)
          J = J+3
 5010     FORMAT (T1,3(I5,2X,I5,2X,I5,2X,I5,2X))
        END DO
        I = NBOND - J + 1 
        IF (I.EQ.1) THEN
          WRITE(FFTOP,5015) J+1,CBOND(J+1,1),CBOND(J+1,2)
 5015     FORMAT (T1,I5,2X,I5,2X,I5,2X,I5,2X)
        ELSE IF (I.EQ.2) THEN
          WRITE(FFTOP,5020) (I,CBOND(I,1),CBOND(I,2),I=J,J+1) 
 5020     FORMAT (T1,2(I5,2X,I5,2X,I5,2X,I5,2X))
        END IF
      END IF
C
C     *** Print bonding terms ***
C
      IF (DOBANG) THEN
C
        WRITE (FFTOP,5025) NBANGS 
 5025   FORMAT (//,T1,'NUMBER OF BOND ANGLES:',1X,I6,/,
     $          T1,'BOND ANGLE NO FOLLOWED BY ATOMS TRIPLET (ABS NO)')
        J = 1
        DO WHILE (J+3.LE.NBANGS)
          WRITE(FFTOP,5030) (I,BANGS(I,1),BANGS(I,2),BANGS(I,3),I=J,J+3)
          J = J+4
 5030     FORMAT (T1,4(I5,1X,I5,1X,I5,1X,I5,1X,I5,1X))
        END DO
        I = NBANGS - J + 1
        IF (I.EQ.1) THEN
          WRITE(FFTOP,5035) J+1,BANGS(J+1,1),BANGS(J+1,2),BANGS(J+1,3)
 5035     FORMAT (T1,I5,1X,I5,1X,I5,1X,I5,1X,I5,1X)
        ELSE IF (I.EQ.2) THEN
          WRITE(FFTOP,5040) (I,BANGS(I,1),BANGS(I,2),BANGS(I,3),I=J,J+1)
 5040     FORMAT (T1,2(I5,1X,I5,1X,I5,1X,I5,1X,I5,1X))
        ELSE IF (I.EQ.3) THEN
          WRITE(FFTOP,5045) (I,BANGS(I,1),BANGS(I,2),BANGS(I,3),I=J,J+2)
 5045     FORMAT (T1,3(I5,1X,I5,1X,I5,1X,I5,1X,I5,1X))
        END IF
      END IF
C
C     *** Print Torsion terms ***
C
      IF (DOTORS) THEN
C
        WRITE (FFTOP,5050) NTANGS 
 5050   FORMAT (//,T1,'NUMBER OF TORSION ANGLES:',1X,I6,/,
     $          T1,'TOSION NO FOLLOWED BY ATOMS QUADRUPLET (ABS NO)')
        J = 1
        DO WHILE (J+1.LE.NTANGS)
          WRITE(FFTOP,5055) (I,TANGS(I,1),TANGS(I,2),
     $                       TANGS(I,3),TANGS(I,4),I=J,J+1)
          J = J+4
 5055     FORMAT (T1,2(I5,1X,I5,1X,I5,1X,I5,1X,I5,1X))
        END DO
        I = NTANGS - J + 1
        IF (I.EQ.1) THEN
          WRITE(FFTOP,5060) J+1,TANGS(J+1,1),TANGS(J+1,2),TANGS(J+1,3),
     $                      TANGS(J+1,4)
 5060     FORMAT (T1,I5,1X,I5,1X,I5,1X,I5,1X)
        END IF
      END IF
C

C
      CLOSE (FFTOP)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE PRTMM ***
C
      END


