      SUBROUTINE READFFMOD
C
C     Purpose: User defined modification of MM energy terms. 
C
C     History: - Creation (16.11.18, ADLL)
C
C     ******************************************************************
C
C     List of local variables :
C
C     NBNDD   : Number of bond terms to be deleted.
C     NBNDA   : Number of bond terms to be added.
C     NBAND   : umber of bond terms to be deleted.
C     NBANA   : Number of bond angle terms to be added.
C     NTORD   : Number of bond angle terms to be deleted.
C     NTORA   : Number of torsion angle terms to be added.
C     NIMPD   : Number of torsion angle terms to be deleted.
C     NIMPA   : Number of improper angle terms to be added.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'embed.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'prm.h'
C
      INTEGER MAXMOD 
      PARAMETER (MAXMOD = 20)
C
      LOGICAL ERROR,EX,MODQ,MODLJ
      INTEGER I,IEMBED,IVALUE,J,JJ,JLIM,K,KK,NTOKEN,NUMBER,PREV,STREXT
      INTEGER I1,I2,I3,I4
      INTEGER TMP(4)
      INTEGER NBNDD,NBNDA,NBAND,NBANA,NTORD,NTORA,NIMPD,NIMPA
      INTEGER ADDANG(MAXMOD,3),ADDBND(MAXMOD,2),ADDTOR(MAXMOD,4)
      INTEGER ADDIMP(MAXMOD,4),DELBND(MAXMOD),DELANG(MAXMOD)
      INTEGER DELTOR(MAXMOD),DELIMP(MAXMOD)
      REAL ADDBNDK(MAXMOD),ADDBNDR(MAXMOD),ADDANGK(MAXMOD)
      REAL ADDANGR(MAXMOD),ADDTORB(MAXMOD,4),ADDTORG(MAXMOD,4)
      REAL ADDTORP(MAXMOD,4),ADDIMPB(MAXMOD,4),ADDIMPG(MAXMOD,4)
      REAL ADDIMPP(MAXMOD,4)
      REAL FACT,RVALUE,SIXTH
C
      CHARACTER*80 DTS,LINE,LTU,PRTSTR,STRCOMP,STRING,STRTYP*7
      CHARACTER*80 TOKEN(500)
C
C     ------------------------------------------------------------------
C
C     *** Check that modification file deMon.ffm exist ***
C
      INQUIRE(FILE='deMon.ffm',ERR=99970,EXIST=EX)
      IF (.NOT.EX) GO TO 99970
C
C     *** Open  file deMon.mff ***
C
      OPEN (FFM,FILE='deMon.ffm',STATUS='UNKNOWN',ACCESS='SEQUENTIAL',
     $      FORM='FORMATTED')
 
C     *** Read Force Field modifications from FFM file ***
C
      SIXTH = 1.0/6.0
      FACT = 2**SIXTH
C
  500 CONTINUE
C
      READ (FFM,'(A80)',ERR=99980,END=600) LINE
      STRING = STRCOMP(LTU(DTS(LINE)))
C
C     *** Modify MM atom charges ***
C
      IF (INDEX(STRING,'ATOMQ').NE.0) THEN
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        CALL GETNUM(TOKEN(2),STRTYP,RVALUE,I)
        CALL GETNUM(TOKEN(3),STRTYP,CHGMM(I-NATOM),IVALUE)
C
C     *** Modify MM atom polarizabilities ***
C
      ELSE IF (INDEX(STRING,'ATOMPOL').NE.0) THEN
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        CALL GETNUM(TOKEN(2),STRTYP,RVALUE,I)
        CALL GETNUM(TOKEN(3),STRTYP,IVALUE)
        
C
C     *** Modify MM atom Lennard Jones parameters ***
C
      ELSE IF (INDEX(STRING,'ATOMLJ').NE.0) THEN
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        CALL GETNUM(TOKEN(2),STRTYP,RVALUE,I)
        CALL GETNUM(TOKEN(3),STRTYP,VLJRAD(I),IVALUE)    
        CALL GETNUM(TOKEN(4),STRTYP,VLJEPS(I),IVALUE)
        IF (NTOKEN.GT.4) CALL GETNUM(TOKEN(4),STRTYP,REDUC(I),IVALUE)
C
        IF (RADTYPE.EQ.'SIGMA') THEN
          VLJRAD(I) = FACT * VLJRAD(I)
        END IF
C
        IF (RADSIZ.EQ.'DIAMETER') THEN
          VLJRAD(I) = 0.5 * VLJRAD(I)
        END IF
        VLJEPS(I) = ABS(VLJEPS(I))
C
      END IF
      GO TO 500
C
  600 CONTINUE
C
C      IF (MPP) THEN
C        CALL MP_SYNC
C        CALL MP_SEND_ALL(CHGMM(1:NATMM),NATMM,MASTER,MP_REAL)
C      END IF
C      IF (MPP) THEN
C        CALL MP_SEND_ALL(VLJRAD(1:NATMM),NATMM,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(VLJEPS(1:NATMM),NATMM,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(REDUC(1:NATMM),NATMM,MASTER,MP_REAL)
C      END IF
C
C     *** Add or delete bonded energy terms ***
C  
C   
C      IF (MPSLAVE) GO TO 800 
C
      REWIND (FFM)
C
      NBNDD = 0
      NBNDA = 0
      NBAND = 0
      NBANA = 0
      NTORD = 0
      NTORA = 0
      NIMPD = 0
      NIMPA = 0
C
  700 CONTINUE
      READ (FFM,'(A80)',ERR=99980,END=800) LINE
      STRING = STRCOMP(LTU(DTS(LINE)))
C
C       *** bond energy terms ***
C
      IF (INDEX(STRING,'BOND').NE.0) THEN
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        IF (INDEX(TOKEN(2),'DELET').NE.0) THEN
          NBNDD = NBNDD + 1
          IF (NBNDD.GT.MAXMOD) GO TO 99990
          CALL GETNUM(TOKEN(3),STRTYP,RVALUE,DELBND(NBNDD))
        ELSE IF (INDEX(TOKEN(2),'ADD').NE.0) THEN
          NBNDA = NBNDA + 1
          IF (NBNDA.GT.MAXMOD) GO TO 99990
          CALL GETNUM(TOKEN(3),STRTYP,RVALUE,ADDBND(NBNDA,1))
          CALL GETNUM(TOKEN(4),STRTYP,RVALUE,ADDBND(NBNDA,2))
          CALL GETNUM(TOKEN(5),STRTYP,ADDBNDK(NBNDA),IVALUE)
          CALL GETNUM(TOKEN(6),STRTYP,ADDBNDR(NBNDA),IVALUE)
        END IF
C
C       *** bond angles energy terms ***
C
      ELSE IF (INDEX(STRING,'ANGLE').NE.0) THEN
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        IF (INDEX(TOKEN(2),'DELET').NE.0) THEN
          NBAND = NBAND + 1
          IF (NBAND.GT.MAXMOD) GO TO 99990
          CALL GETNUM(TOKEN(3),STRTYP,RVALUE,DELANG(NBAND))
        ELSE IF (INDEX(TOKEN(2),'ADD').NE.0) THEN
          NBANA = NBANA + 1
          IF (NBANA.GT.MAXMOD) GO TO 99990
          CALL GETNUM(TOKEN(3),STRTYP,RVALUE,ADDANG(NBANA,1))
          CALL GETNUM(TOKEN(4),STRTYP,RVALUE,ADDANG(NBANA,2))
          CALL GETNUM(TOKEN(5),STRTYP,RVALUE,ADDANG(NBANA,3))
          CALL GETNUM(TOKEN(6),STRTYP,ADDANGK(NBANA),IVALUE)
          CALL GETNUM(TOKEN(7),STRTYP,ADDANGR(NBANA),IVALUE)
        END IF
      
C       *** Torsion angles energy terms ***
C
      ELSE IF (INDEX(STRING,'TORSION').NE.0) THEN
C
        CALL STRTOK(STRING,NTOKEN,TOKEN)
        IF (INDEX(TOKEN(2),'DELET').NE.0) THEN
          NTORD = NTORD + 1
          IF (NTORD.GT.MAXMOD) GO TO 99990
          CALL GETNUM(TOKEN(3),STRTYP,RVALUE,DELTOR(NTORD))
        ELSE IF (INDEX(TOKEN(2),'ADD').NE.0) THEN
          NTORA = NTORA + 1
          IF (NTORA.GT.MAXMOD) GO TO 99990
          CALL GETNUM(TOKEN(3),STRTYP,RVALUE,ADDTOR(NTORA,1))
          CALL GETNUM(TOKEN(4),STRTYP,RVALUE,ADDTOR(NTORA,2))
          CALL GETNUM(TOKEN(5),STRTYP,RVALUE,ADDTOR(NTORA,3))
          CALL GETNUM(TOKEN(6),STRTYP,RVALUE,ADDTOR(NTORA,4))
          JLIM =  NTOKEN - 6
          JJ = 1
          K = JLIM/3
          DO J=1,K
            CALL GETNUM(TOKEN(6+JJ),STRTYP,ADDTORB(NTORA,J),IVALUE)
            CALL GETNUM(TOKEN(6+JJ+1),STRTYP,ADDTORG(NTORA,J),IVALUE)
            CALL GETNUM(TOKEN(6+JJ+2),STRTYP,ADDTORP(NTORA,J),IVALUE)
            JJ = JJ + 3
          END DO
          CALL TFOLD(ADDTORB(NTORA,:),ADDTORG(NTORA,:),ADDTORP(NTORA,:))
        END IF
      END IF
      GO TO 700
C
 800  CONTINUE
C
C      IF (MPP) THEN
C        CALL MP_SEND_ALL(NBNDD,1,MASTER,MP_INT)
C        CALL MP_SEND_ALL(NBNDA,1,MASTER,MP_INT)
C        CALL MP_SEND_ALL(NBAND,1,MASTER,MP_INT)
C        CALL MP_SEND_ALL(NBANA,1,MASTER,MP_INT)
C        CALL MP_SEND_ALL(NTORD,1,MASTER,MP_INT)
C        CALL MP_SEND_ALL(NTORA,1,MASTER,MP_INT)
CC
C        CALL MP_SEND_ALL(DELBND,MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(DELANG,MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(DELTOR,MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(DELIMP,MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(ADDBND,2*MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(ADDANG,3*MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(ADDTOR,4*MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(ADDIMP,4*MAXMOD,MASTER,MP_INT)
C        CALL MP_SEND_ALL(ADDBNDK,MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDBNDR,MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDANGK,MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDANGR,MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDTORB,4*MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDTORG,4*MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDTORP,4*MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDIMPB,4*MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDIMPG,4*MAXMOD,MASTER,MP_REAL)
C        CALL MP_SEND_ALL(ADDIMPP,4*MAXMOD,MASTER,MP_REAL)
C      END IF
C
C     *** Delete energy terms ***
C
      IF (NBNDD.EQ.0) GO TO 850
C
      PREV = 999999
      DO I=1,NBNDD
        KK = 0
        DO J=1,NBNDD
          IF ((DELBND(J).GT.KK).AND.(DELBND(J).LT.PREV)) THEN
            KK = DELBND(J)
          END IF
        END DO 
        PREV = KK
        DO J=KK,NBOND-1
          KBOND(J) = KBOND(J+1)
          BOND0(J) = BND0(J+1)
          CBOND(J,1) = CBOND(J+1,1) 
          CBOND(J,2) = CBOND(J+1,2)
        END DO
      END DO      
      NBOND = NBOND - NBNDD
C
 850  CONTINUE
C
      IF (NBAND.EQ.0) GO TO 950
C
      PREV = 999999
      DO I=1,NBAND
        KK = 0
        DO J=1,NBAND
          IF ((DELANG(J).GT.KK).AND.(DELANG(J).LT.PREV)) THEN
            KK = DELANG(J)
          END IF
        END DO
        PREV = KK
        DO J=KK,NBANGS-1
          KANGLE(J) = KANGLE(J+1)
          ANGLE0(J) = ANGLE0(J+1)
          BANGS(J,1) = BANGS(J+1,1)
          BANGS(J,2) = BANGS(J+1,2)
          BANGS(J,3) = BANGS(J+1,3)
        END DO
      END DO
      NBANGS = NBANGS - NBAND 
C
 950  CONTINUE
C
      IF (NTORD.EQ.0) GO TO 1000
C
      PREV = 999999
      DO I=1,NTORD
        KK = 0
        DO J=1,NTORD
          IF ((DELTOR(J).GT.KK).AND.(DELTOR(J).LT.PREV)) THEN
            KK = DELTOR(J) 
          END IF
        END DO
        PREV = KK
        DO J=KK,NTANGS-1
          TORS1(J,1) = TORS1(J+1 ,1)
          TORS1(J,2) = TORS1(J+1 ,2)
          TORS2(J,1) = TORS2(J+1 ,1)
          TORS2(J,2) = TORS2(J+1 ,2)
          TORS3(J,1) = TORS3(J+1 ,1)
          TORS3(J,2) = TORS3(J+1 ,2)
          TORS4(J,1) = TORS4(J+1 ,1)
          TORS4(J,2) = TORS4(J+1 ,2)
          TANGS(J,1) = TANGS(J+1,1)
          TANGS(J,2) = TANGS(J+1,2)
          TANGS(J,3) = TANGS(J+1,3)
          TANGS(J,4) = TANGS(J+1,4)
        END DO
      END DO
      NTANGS = NTANGS - NTORD
C
 1000 CONTINUE
C
C     *** Add energy terms ***
C
      IF (NBNDA.EQ.0) GO TO 1050
C
      DO J=1,NBNDA
        KBOND(NBOND+J) = ADDBNDK(J) 
        BOND0(NBOND+J) = ADDBNDR(J)
        CBOND(NBOND+J,1) = ADDBND(J,1)
        CBOND(NBOND+J,2) = ADDBND(J,1)
      END DO
      NBOND = NBOND + NBNDA 
C
 1050 CONTINUE
C
      IF (NBANA.EQ.0) GO TO 1100
C
      DO J=1,NBANA
        KBOND(NBANGS+J) = ADDANGK(J) 
        BOND0(NBANGS+J) = ADDANGR(J)
        BANGS(NBANGS+J,1) = ADDANG(J,1)   
        BANGS(NBANGS+J,2) = ADDANG(J,2)   
        BANGS(NBANGS+J,3) = ADDANG(J,3)   
      END DO
      NBANGS = NBANGS + NBANA 
C 
 1100 CONTINUE
C
      IF (NTORA.EQ.0) GO TO 1150
C
      DO J=1,NTORA
        TORS1(NTANGS+J,1) = ADDTORB(J,1) 
        TORS1(NTANGS+J,2) = ADDTORG(J,1) 
        TORS2(NTANGS+J,1) = ADDTORB(J,2)
        TORS2(NTANGS+J,2) = ADDTORG(J,2)
        TORS3(NTANGS+J,1) = ADDTORB(J,3)
        TORS3(NTANGS+J,2) = ADDTORG(J,3)
        TORS4(NTANGS+J,1) = ADDTORB(J,4)
        TORS4(NTANGS+J,2) = ADDTORG(J,4)
        TANGS(NTANGS+J,1) = ADDTOR(J,1)
        TANGS(NTANGS+J,2) = ADDTOR(J,2)
        TANGS(NTANGS+J,3) = ADDTOR(J,3)
        TANGS(NTANGS+J,4) = ADDTOR(J,4)
      END DO
      NTANGS = NTANGS + NTORA 
C
 1150 CONTINUE
C
C     *** Restore bond angle order ***

C     *** Sort double 1-3 interactions at the end ***
C
      IF ((NBAND.NE.0).OR.(NBANA.NE.0)) THEN
C
        MBANGS = NBANGS
        I = 1
        DO WHILE (I.LE.(MBANGS - 1))
          I1 = MIN(BANGS(I,1),BANGS(I,3))
          I2 = MAX(BANGS(I,1),BANGS(I,3))
          J = I + 1
          DO WHILE (J.LE.MBANGS)
            I3 = MIN(BANGS(J,1),BANGS(J,3))
            I4 = MAX(BANGS(J,1),BANGS(J,3))
            IF ((I1.EQ.I3).AND.(I2.EQ.I4)) THEN
              TMP(1:3) = BANGS(MBANGS,1:3)
              BANGS(MBANGS,1:3) = BANGS(J,1:3)
              BANGS(J,1:3) = TMP(1:3)
              MBANGS = MBANGS - 1
              CYCLE
            END IF
            J = J + 1
          END DO
          I = I + 1
        END DO
C
C     *** Sort 1-3 / 1-2 doubles at the end ***
C
        DO I=1,NBOND
          I1 = MIN(CBOND(I,1),CBOND(I,2))
          I2 = MAX(CBOND(I,1),CBOND(I,2))
          J = 1
          DO WHILE (J.LE.MBANGS)
            I3 = MIN(BANGS(J,1),BANGS(J,3))
            I4 = MAX(BANGS(J,1),BANGS(J,3))
            IF ((I1.EQ.I3).AND.(I2.EQ.I4)) THEN
              TMP(1:3) = BANGS(MBANGS,1:3)
              BANGS(MBANGS,1:3) = BANGS(J,1:3)
              BANGS(J,1:3) = TMP(1:3)
              MBANGS = MBANGS - 1
              CYCLE
            END IF
            J = J + 1
          END DO
        END DO
      END IF
C
C
C     *** Sort double 1-4 interactions at the end ***
C
      IF ((NTORD.NE.0).OR.(NTORA.NE.0))  THEN
C
        MTANGS = NTANGS
        I = 1
        DO WHILE (I.LE.(MTANGS - 1))
          I1 = MIN(TANGS(I,1),TANGS(I,4))
          I2 = MAX(TANGS(I,1),TANGS(I,4))
          J = I + 1
          DO WHILE (J.LE.MTANGS)
            I3 = MIN(TANGS(J,1),TANGS(J,4))
            I4 = MAX(TANGS(J,1),TANGS(J,4))
            IF ((I1.EQ.I3).AND.(I2.EQ.I4)) THEN
              TMP(1:4) = TANGS(MTANGS,1:4)
              TANGS(MTANGS,1:4) = TANGS(J,1:4)
              TANGS(J,1:4) = TMP(1:4)
              MTANGS = MTANGS - 1
              CYCLE
            END IF
            J = J + 1
          END DO
          I = I + 1
        END DO
C
C       *** Sort 1-4 / 1-2 doubles at the end ***
C
        DO I=1,NBOND
          I1 = MIN(CBOND(I,1),CBOND(I,2))
          I2 = MAX(CBOND(I,1),CBOND(I,2))
          J = 1
          DO WHILE (J.LE.MTANGS)
            I3 = MIN(TANGS(J,1),TANGS(J,4))
            I4 = MAX(TANGS(J,1),TANGS(J,4))
            IF ((I1.EQ.I3).AND.(I2.EQ.I4)) THEN
              TMP(1:4) = TANGS(MTANGS,1:4)
              TANGS(MTANGS,1:4) = TANGS(J,1:4)
              TANGS(J,1:4) = TMP(1:4)
              MTANGS = MTANGS - 1
              CYCLE
            END IF
            J = J + 1
          END DO
        END DO
C
C       *** Sort 1-4 / 1-3 doubles at the end ***
C
        DO I=1,MBANGS
          I1 = MIN(BANGS(I,1),BANGS(I,3))
          I2 = MAX(BANGS(I,1),BANGS(I,3))
          J = 1
          DO WHILE (J.LE.MTANGS)
            I3 = MIN(TANGS(J,1),TANGS(J,4))
            I4 = MAX(TANGS(J,1),TANGS(J,4))
            IF ((I1.EQ.I3).AND.(I2.EQ.I4)) THEN
              TMP(1:4) = TANGS(MTANGS,1:4)
              TANGS(MTANGS,1:4) = TANGS(J,1:4)
              TANGS(J,1:4) = TMP(1:4)
              MTANGS = MTANGS - 1
              CYCLE
            END IF
            J = J + 1
          END DO
        END DO
      END IF
C
 1200 CONTINUE
C

C     *** Write modified MM system topology ***
C
      IF (PRTMMTOP) CALL PRTMM
C
C     *** Close force field modification file deMon.ffm ***
C
      CLOSE (UNIT=FFM,STATUS='KEEP')
C
      RETURN
C     ------------------------------------------------------------------
C
C     *** Handling of read errors ***
C
99970 CONTINUE
      CALL ERRMSG('READFFMOD','FF MODIFICATON FILE deMon.ffm DOES NOT
     $ EXIST',1)
C
99980 CONTINUE
      WRITE (PRTSTR,'("ERROR IN deMon.ffm LINE: ",I10)') NEMBED
      CALL MESSAGE(0,'--> '//TRIM(LINE),1,0)
      CALL ERRMSG('READFFMOD',STRCOMP(PRTSTR),1)
C
99990 CONTINUE
      CALL ERRMSG('READFFMOD','TOO MANY REQUESTED MODIFICATIONS ',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE READFFMOD ***
C
      END





