      SUBROUTINE DOTDPLOT(PAR,PAI,PBR,PBI,DSTO)
C
C     Purpose: Plot time-dependent molecular fields.
C
C     History: - Creation (11.03.13, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'plot.h'
      INCLUDE 'numint.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      include 'dynamics.h'
      include 'iteration.h'
      INCLUDE 'symmetry.h'
      INCLUDE 'scfvec.h'
C
      INTEGER DSTO
      REAL PAR(DSTO,DSTO),PAI(DSTO,DSTO),PBR(DSTO,DSTO),PBI(DSTO,DSTO)
C
      INTEGER ALLOCATION,ANACYC,LLVAR_ME,NARUN,ULVAR_ME,REDCYC
      INTEGER CCSET_SAVE,IMO
      REAL RDUMMY(1),SCALFAC
      COMPLEX (KIND=8) EYE
      CHARACTER*30 STRCOM,CHAIN,PLOTFILE
      CHARACTER*80 TIMSTR 
      LOGICAL MDFLUSH
C
      REAL,ALLOCATABLE :: NCDA(:),NCDB(:)
      REAL,ALLOCATABLE :: CI(:,:),CR(:,:)
      COMPLEX (KIND=8) ,ALLOCATABLE :: F(:,:)
C
      EXTERNAL WCAP,WELF,WESP,WDENJ,WDIVJ,WPSIZ,WRHO,WTDELF
C
C     ------------------------------------------------------------------
C
C     *** Quick return if no function plotting requested ***
C
      IF ((PLOT.EQ.'OFF').OR.(LATTYP.NE.'STRUCTURED')) RETURN
C
C     *** Initialization ***
C
      IF (MPP) THEN
        TIMSTR = 'MPI PLOT FUNCTION GENERATION'
      ELSE
        TIMSTR = 'PLOT FUNCTION GENERATION'
      END IF
C
      CALL TIMER('START',TIMSTR)
C
C     *** Allocate matrices ***
C
      ALLOCATE(CR(DSTO,DSTO),CI(DSTO,DSTO),F(DSTO,DSTO),STAT=ALLOCATION)
      ALLOCATE(NCDA(MAXAUX),NCDB(MAXAUX),STAT=ALLOCATION)
C
      IF (ADDRAD) CALL IONRADINI 
C
C     *** Set-up parallel workload ***
C
      NARUN = MIN(RTMAX+FGRTCYC,RTAMAX) - MAX(RTAMIN,FGRTCYC)
      IF (FGRTCYC.EQ.0) NARUN = NARUN + 1
C
      CALL MPLIMITS(NCPU,ME,1,0,NARUN,LLVAR_ME,ULVAR_ME,(/0/),0)
C
C     *** Select MOs for plotting ***
C
      CALL GETOMOS(EIGVALA,HFOMOA,NPLOTFUN,PLOTMOA,PLOTMOSA)
      IF (OKS) CALL GETOMOS(EIGVALB,HFOMOB,NPLOTFUN,PLOTMOB,PLOTMOSB)
C
      CALL CHKPLOT('ORBITALS')
C
      IF (MPP) THEN
        IF (MPMASTER) CALL SCFIO(CI,NSTO,'READ','OVERLAP')
        CALL MP_SEND_ALL(CI,DSTO**2,MASTER,MP_REAL)
        CALL SCFIO(CI,NSTO,'WRITE','OVERLAP')
      END IF
      REWIND(IORTP)
C
C     *** Loop over simulation propagation steps *** 
C
      DO 10 RTCYC=1,RTMAX
C
        ANACYC = RTCYC + FGRTCYC
        REDCYC = RTCYC 
        IF ((ANACYC.LT.RTAMIN).OR.(ANACYC.GT.RTAMAX)) GO TO 10 
        IF ((ANACYC.NE.1).AND.(.NOT.(MDFLUSH(ANACYC,RTPRO)))) GO TO 10 
        IF ((REDCYC.LT.LLVAR_ME).OR.(REDCYC.GT.ULVAR_ME)) GO TO 10 
C
C     *** Open binary plot file ***
C
         WRITE (CHAIN,*) RTCYC + FGRTCYC 
         RTT = RTSTEP*(RTCYC + FGRTCYC)
         IF (PLOTFNC.EQ.'ELF') PLOTFILE='ELF_'//ADJUSTL(CHAIN)
         IF (PLOTFNC.EQ.'RHO') PLOTFILE='RHO_'//ADJUSTL(CHAIN)
         IF (PLOTFNC.EQ.'DENJ') PLOTFILE='DENJ_'//ADJUSTL(CHAIN)
         IF (PLOTFNC.EQ.'DIVJ') PLOTFILE='DIVJ_'//ADJUSTL(CHAIN)
         IF (PLOTFNC.EQ.'ABS') PLOTFILE='ABS_'//ADJUSTL(CHAIN)
         IF (PLOTFNC.EQ.'PSI') PLOTFILE='PSI_'//ADJUSTL(CHAIN)
C
         PLOTNAME(0) = TRIM(ADJUSTL(STRCOM(ADJUSTL(PLOTFILE))))
         CLOSE (PIE,STATUS='KEEP')
         IF (PLOT.EQ.'CUBE') THEN
C
           PLOTFILE=ADJUSTL(ADJUSTR(PLOTFILE)//'.cube')
           OPEN (PIE,FILE=PLOTFILE,STATUS='UNKNOWN',
     $     POSITION='REWIND',ACCESS='SEQUENTIAL',
     $     FORM='FORMATTED')
C
         ELSE
C
           PLOTNAME(0) = ADJUSTL(ADJUSTR(PLOTNAME(0))//'.bin') 
           CALL BINARY(RDUMMY,1,1,1,IOPLOT,PLOTNAME(0),'OPEN')
C
           IF (PLOT.EQ.'ASCII') THEN
             PLOTFILE=ADJUSTL(ADJUSTR(PLOTFILE)//'.pie')
             OPEN (PIE,FILE=PLOTFILE,STATUS='UNKNOWN',
     $       POSITION='REWIND',ACCESS='SEQUENTIAL',
     $       FORM='FORMATTED')
           END IF
C
        END IF
C
C     *** Handle rotation to standard orientation ***
C
        COORD(1:3,1:NATOM,5) = COORD(1:3,1:NATOM,CCSET)
        IF ((CCSET.NE.2).AND.(PLOT.EQ.'CUBE')) THEN 
          CALL TRABOX('SYMMETRY')
          CCSET_SAVE = CCSET
          CCSET = 2
        END IF
C
C    *** Recover density matrix from tape *** 
C
       CALL RTPIO(PAR,NCDA,DSTO,NAUX,'READ')
       CALL STOPMAT(PAR,PAR,PAI,DSTO,'EXPAND')
C
C    *** Build density matrix for unbound elecrons ***
C
      IF ((PLOTMOA.EQ.'POSE').OR.(PLOTMOA.EQ.'POSE')) THEN
        MOCC = .TRUE.
        CALL SCALMAT(PAR,DSTO,DSTO,DSTO,'TOTWD',0.5)
        CALL SCFIO(CR,DSTO,'READ','ORTHO')
        DO IMO=1,NORTH
           CI(1:DSTO,IMO) = CR(1:DSTO,IMO)*EGIS(IMO)
        END DO
        CALL GMULMAT(CI,PAR,CR,DSTO,DSTO,DSTO,DSTO,DSTO,DSTO,'TRANSA')
        CALL GMULMAT(CR,CI,PAR,DSTO,DSTO,DSTO,DSTO,DSTO,DSTO,'NORMAL')
        CALL SCFIO(CR,DSTO,'READ','COLDALPHA')
C
        DO IMO=1,DSTO
          CALL GMULMAT(PAR,CR(:,IMO),CI(:,1),DSTO,DSTO,DSTO,
     $         DSTO,DSTO,1,'NORMAL')
          OCCNA(IMO) = DOT_PRODUCT(CR(:,IMO),CI(:,1))
          IF (EIGVALA(IMO).LT.0.0) OCCNA(IMO) = 0.0
        END DO
C
        CI(:,:) = 0.0
        CALL BLDCOMMAT(CR,CI,PAR,PAI,DSTO,OCCNA,DSTO,DSTO)
      END IF
C
C    *** Perform Choleski decomposition, rotate MOS and rebuild P ***
C
      IF (PLOT.EQ.'CUBE') THEN
        IF (OKS) THEN
          SCALFAC = 1.0
        ELSE
          SCALFAC = 0.5
        END IF

        CALL SCALMAT(PAR,DSTO,DSTO,DSTO,'M = N',SCALFAC)
        CALL SCALMAT(PAR,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
        CALL SCALMAT(PAI,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
        EYE = CMPLX(0,1)
        F(:,:) = PAR(:,:) + EYE*PAI(:,:)
        CALL CHLMOSZ(F,DSTO,NOMOA,'OCC') 
        CR(:,:) = REAL(F(:,:))
        CI(:,:) = AIMAG(F(:,:))
        CALL MODMOS(CR,PAR,INPTOSTD,NSTO,'ROTATE')
        CALL MODMOS(CI,PAI,INPTOSTD,NSTO,'ROTATE')
C        
        CALL BLDCOMMAT(CR,CI,PAR,PAI,DSTO,OCCNA,LFOMOA,HFOMOA)
        IF (OKS) THEN
          CALL RTPIO(CR,NCDA,DSTO,NAUX,'READ')
          CALL STOPMAT(CR,PBR,PBI,DSTO,'EXPAND')
          IF ((CCSET.NE.2).AND.(PLOT.EQ.'CUBE')) THEN
            CALL SCALMAT(PBR,DSTO,DSTO,DSTO,'M = N',SCALFAC)
            CALL SCALMAT(PBR,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
            CALL SCALMAT(PBI,DSTO,DSTO,DSTO,'TOTWD',SCALFAC/2)
            EYE = CMPLX(0,1)
            F(:,:) = PBR(:,:) + EYE*PBI(:,:)
            CALL CHLMOSZ(F,DSTO,NOMOB,'OCC')
            PBR(:,:) = REAL(F(:,:))
            PBI(:,:) = AIMAG(F(:,:))
            CALL MODMOS(CR,PAR,INPTOSTD,NSTO,'ROTATE')
            CALL MODMOS(CI,PAI,INPTOSTD,NSTO,'ROTATE')
            CALL BLDCOMMAT(CR,CI,PBR,PBI,DSTO,OCCNB,LFOMOB,HFOMOB)
          END IF
          NCDA(:) = NCDA(:) + NCDB(:)
        END IF
      END IF
C
C     *** Plot Electron Localization Function ***
C
        IF (PLOTFNC.EQ.'ELF') THEN
C
          NPLOTFUN = 5
          CALL SCALMAT(PAR,DSTO,NSTO,NSTO,'TOTWD',0.5)
          CALL SCALMAT(PAI,DSTO,NSTO,NSTO,'TOTWD',0.5)
          IF (OKS) THEN
            CALL PLOTTER(PAR,PAI,PBR,PBI,NCDA,DSTO,WTDELF)
          ELSE
            CALL PLOTTER(PAR,PAI,PAR,PAI,NCDA,DSTO,WTDELF)
          END IF

C     *** Plot Electron density ***

        ELSE IF (PLOTFNC.EQ.'RHO') THEN

          IF (.NOT.TOPCD) THEN
            CALL SCALMAT(PAR,DSTO,NSTO,NSTO,'TOTWD',0.5)
          END IF

          CALL PLOTTER(PAR,PAI,PAR,PAI,NCDA,DSTO,WRHO)
C
C     *** Plot current density vectors ***
C
        ELSE IF (PLOTFNC.EQ.'DENJ') THEN
C
          CALL SCALMAT(PAR,DSTO,NSTO,NSTO,'TOTWD',0.5)
          CALL SCALMAT(PAI,DSTO,NSTO,NSTO,'TOTWD',0.5)
          IF (OKS) THEN
            CALL PLOTTER(PAR,PAI,PBR,PBI,NCDA,DSTO,WDENJ)
          ELSE
            CALL PLOTTER(PAR,PAI,PAR,PAI,NCDA,DSTO,WDENJ)
          END IF
C
C     *** Plot divergence of current density ***
C
        ELSE IF (PLOTFNC.EQ.'DIVJ') THEN
C
          CALL SCALMAT(PAR,DSTO,NSTO,NSTO,'TOTWD',0.5)
          CALL SCALMAT(PAI,DSTO,NSTO,NSTO,'TOTWD',0.5)
          IF (OKS) THEN
            CALL PLOTTER(PAR,PAI,PBR,PBI,NCDA,DSTO,WDIVJ)
          ELSE
            CALL PLOTTER(PAR,PAI,PAR,PAI,NCDA,DSTO,WDIVJ)
          END IF
C
C     *** Plot Molecular Orbitals ***
C
      ELSE IF (PLOTFNC.EQ.'PSI') THEN
C
        CALL PLOTTER(CR,CR,CI,CI,NCDA,DSTO,WPSIZ)
C
C     *** Plot secondary electron spectrum *** 
C
      ELSE IF (PLOTFNC.EQ.'ABS') THEN
C
        ELSPEC(:,:) = 0.0
        CALL SCALMAT(PAR,DSTO,NSTO,NSTO,'TOTWD',0.5)
        CALL GRDDEF(CAPGSN,'LOAD')
C
        CALL PLOTTER(CR,CI,PAR,PAI,NCDA,DSTO,WCAP)
        END IF
C
  10  CONTINUE 
      DEALLOCATE(CR,CI,F,STAT=ALLOCATION)
C
      CCSET = CCSET_SAVE
      PLOT = 'OFF'
      CALL TIMER('STOP',TIMSTR)
C
      DEALLOCATE(NCDA,NCDB,STAT=ALLOCATION)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DOTDPLOT ***
C
      END
