      SUBROUTINE RTANA(PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
C
C     Purpose: Carry out on-the-fly electron density analyses in RTDFT. 
C
C     History: (Creation, 25.07.16 ADLL)
C
C     *****************************************************************
C
C     List of local dimensions:
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables:
C
C     DPA: Difference alpha density matrix.
C     DPB: Difference beta density matrix.
C     PA : Alpha density matrix.
C     PB : Beta density matrix.
C
C     -----------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'  
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'opt.h'
      INCLUDE 'plot.h'
      INCLUDE 'property.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'tolerance.h'
C
      INTEGER HAS,DSTO,DSTO_ME,I,NEED
      REAL PAR(DSTO_ME,DSTO_ME),PAI(DSTO_ME,DSTO_ME)
      REAL PBR(DSTO_ME,DSTO_ME),PBI(DSTO_ME,DSTO_ME)
C    
      INTEGER  IATOM,ICOOR
      INTEGER ALLOCATION,ANACYC,LLVAR_ME,ULVAR_ME,NARUN,NGPTOT_SAVE
      REAL,ALLOCATABLE :: PA(:,:),PB(:,:),SFCDA(:),SFCDB(:),WORK(:,:)
      COMPLEX(KIND=8), ALLOCATABLE :: PAC(:,:),PBC(:,:)
C
      COMPLEX(KIND=8) EYE
      REAL SCALFAC,GRDTOL_SAVE,TIME
      CHARACTER*80 STRCOMP,STRING,TIMSTR
      CHARACTER*3 MYFILE
      LOGICAL DOSAVE,MDFLUSH
      EXTERNAL WEFG,WEFI,WESP,WD1ESP,WD2ESP
C
C     -----------------------------------------------------------------
C
C     *** Return if no analysis for the current propagation step ***
C
      IF (RTAMAX.EQ.9999999) RTAMAX = FGRTCYC+RTMAX
C
      ANACYC = RTCYC + FGRTCYC
      NARUN = MIN(RTMAX+FGRTCYC,RTAMAX) - MAX(RTAMIN,FGRTCYC)
C
      IF (FGRTCYC.EQ.0) NARUN = NARUN + 1 
      IF ((ANACYC.LT.RTAMIN).OR.(ANACYC.GT.RTAMAX)) RETURN
      IF ((ANACYC.NE.1).AND.(.NOT.(MDFLUSH(ANACYC,RTPRO)))) RETURN
C
C     *** Start timer for electronic structure analysis ***
C
      IF (MPP) THEN
        TIMSTR = 'MPI ELECTRON DYNAMICS ANALYSIS'
      ELSE
        TIMSTR = 'ELECTRON DYNAMICS ANALYSIS'
      END IF
C
      CALL TIMER('START',TIMSTR)
C
C     *** Allocate local fields ***
C
      ALLOCATE(PA(NSTO,NSTO),STAT=ALLOCATION)
      IF (OKS) THEN
        ALLOCATE(PB(NSTO,NSTO),STAT=ALLOCATION)
      END IF
C
C     *** Send auxiliary density coefficients to other processors ***
C
      IF (MPP) THEN
        CALL MP_SEND_ALL(FCDA,NAUX,MASTER,MP_REAL)
        CALL MP_SEND_ALL(FCDB,NAUX,MASTER,MP_REAL)
      END IF
C  
C     *** Save auxiliary density coefficients ***
C  
      IF (TOPCD) THEN
        ALLOCATE(SFCDA(NAUX),SFCDB(NAUX),STAT=ALLOCATION)
        SFCDA(1:NAUX) = FCDA(1:NAUX)
        SFCDB(1:NAUX) = FCDB(1:NAUX)
      END IF
C
C     *** Perform population analysis ***
C
      IF (.NOT.ESA) GO TO 200
C     
C     *** Prepare population analysis with fitted density ***
C
      IF (MPP) THEN
        CALL MP_SEND_ALL(NELA,1,MASTER,MP_REAL)
        CALL MP_SEND_ALL(NELB,1,MASTER,MP_REAL)
      END IF
C
      IF (TOPCD) THEN
C
        CALL NORMCDC(FCDA,FCDB,NAUX)
        IF (DEFORM) THEN
          IF (OKS) THEN
            FCDA(:) = FCDA(:) - MFCDA(:)
            FCDB(:) = FCDB(:) - MFCDB(:)
          ELSE
            FCDA(:) = FCDA(:) - MFCDA(:)
          END IF
        END IF
C
      ELSE
C
C     *** Prepare population analysis with KS density ***
C
        IF (MPMASTER) THEN
          IF (DEFORM) THEN
            IF (OKS) THEN
              CALL SCFIO(PA,DSTO,'READ','P0ALPHA')
              CALL SCFIO(PB,DSTO,'READ','P0BETA') 
              PA(:,:) = -PA(:,:) + PAR(:,:)
              PB(:,:) = -PB(:,:) + PBR(:,:)
            ELSE
              CALL SCFIO(PA,DSTO,'READ','P0ALPHA')
              PA(:,:) = -PA(:,:) + PAR(:,:)
            END IF
          ELSE 
            PA(:,:) = PAR(:,:)
            IF (OKS) PB(:,:) = PBR(:,:)
          END IF
        END IF
C
C     *** Send density matrices to other processors ***
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(PA,NSTO**2,MASTER,MP_REAL)
          IF (OKS) CALL MP_SEND_ALL(PB,NSTO**2,MASTER,MP_REAL)
        END IF
C
      END IF
C
C     *** Perform population analysis on grid ***
C
      IF (OKS) THEN
        CALL POPGRD_RT(PA,PB,NSTO)
      ELSE
        CALL POPGRD_RT(PA,PA,NSTO)
      END IF
C
C     *** Print population analysis results to RTP unit ***
C
      IF (MPMASTER) THEN
        CALL RTPOPOUT
        IF (NMOL.GT.0) CALL DIPCOR(RTCYC)
      END IF
C
 200  CONTINUE
C
      IF (MLRT) CALL RTMLOUT
C
C     *** Perform time-dependent MO occupation numbers analysis ***
C
      IF (.NOT.TDOCC) GO TO 300
C
C     *** Prepare density matrices for TD MO occupation numbers analysis ***
C
      IF (MPMASTER) THEN
        PA(:,:) = PAR(:,:)
        IF (OKS) PB(:,:) = PBR(:,:)
      END IF
      IF (MPP) THEN
        CALL MP_SEND_ALL(PA,NSTO**2,MASTER,MP_REAL)
        IF (OKS) CALL MP_SEND_ALL(PB,NSTO**2,MASTER,MP_REAL)
      END IF
C
C     *** Determine time-dependent MO occupation numbers ***
C
      CALL TDOCCN(PA,OCCNA,HFOMOA,DSTO,DSTO_ME,'ALPHA','PRINT')
      IF (OKS) THEN
        CALL TDOCCN(PB,OCCNB,HFOMOB,DSTO,DSTO_ME,'BETA','PRINT')
      END IF
C
 300  CONTINUE 
C
C     *** Deallocate work fields ***
C
      DEALLOCATE(PA,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'RTANA','ALPHA DEALLOCATION FAILED')
      IF (OKS) THEN
        DEALLOCATE(PB,STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'RTANA','BETA DEALLOCATION FAILED')
      END IF
C
C     *** Calculate approximate nuclear forces ***
C
      IF (.NOT.TDGRD) GO TO 400
      MOCC = .TRUE.
      CALL GRADDRV(DE(1:NDEG,1),NDEG)
      IF (MPMASTER) CALL RTGRDOUT(DE)
C
 400  CONTINUE
C
C     *** Calculate fields on Lattice points  ***
C
      IF ((PLOT.EQ.'OFF').OR.(LATTYP.NE.'POINTS')) GO TO 600
C
      CALL PIEDRV('PIEFILE')
      CALL PIEDRV('PIENAME')
      CALL FILEIO('OPEN PLOT FILES')
C
      IF (MPSLAVE) GO TO 590 
      ALLOCATE(PA(NSTO,NSTO),STAT=ALLOCATION)
      IF (OKS) ALLOCATE(PB(NSTO,NSTO),STAT=ALLOCATION)
C
      IF (USEDD) THEN
        CALL SCFIO(PA,DSTO,'READ','P0ALPHA')
        PA(:,:) = -PA(:,:) + PAR(:,:)
        IF (OKS) THEN
          CALL SCFIO(PB,DSTO,'READ','P0BETA')
          PB(:,:) = -PB(:,:) + PBR(:,:)
        END IF
      ELSE
        PA(:,:) = PAR(:,:)
        IF (OKS) PB(:,:) = PBR(:,:)
      END IF
C
      IF (PLOTFNC.EQ.'ESP') THEN
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTDER.EQ.0) THEN
          CALL PLOTTER(PA,PA,PA,PA,FCDA,NSTO,WESP)
        ELSE IF (PLOTDER.EQ.1) THEN
          CALL PLOTTER(PA,PA,PA,PA,FCDA,NSTO,WD1ESP)
        ELSE IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(PA,PA,PA,PA,FCDA,NSTO,WD2ESP)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID ESP DERIVATIVE',1)
        END IF
C
      ELSE IF (PLOTFNC.EQ.'EFI') THEN
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTDER.EQ.1) THEN
          CALL PLOTTER(PA,PA,PA,PA,FCDA,NSTO,WEFI)
        ELSE IF (PLOTDER.EQ.2) THEN
          CALL PLOTTER(PA,PA,PA,PA,FCDA,NSTO,WEFG)
        ELSE
          CALL ERRMSG('PLOTSCF','INVALID ELECTRIC FIELD DERIVATIVE',1)
        END IF
      END IF
C
 590  CONTINUE
C
      CALL FILEIO('CLOSE PLOT FILES')
C
 600  CONTINUE
C
C     *** Save density matrix for later analysis *** 
C
      IF ((LATTYP.EQ.'POINTS').AND.(.NOT.TDWFN)) GO TO 700
C
      IF (MPMASTER) THEN
        ALLOCATE(PA(NSTO,NSTO),STAT=ALLOCATION)
        IF (OKS) ALLOCATE(PB(NSTO,NSTO),STAT=ALLOCATION) 
      END IF
C
      IF (MPMASTER.AND.USEDD) THEN
        CALL SCFIO(PA,DSTO,'READ','P0ALPHA')
        PA(:,:) = -PA(:,:) + PAR(:,:)
        IF (OKS) THEN
          CALL SCFIO(PB,DSTO,'READ','P0BETA')
          PB(:,:) = -PB(:,:) + PBR(:,:)
        END IF
      END IF
C
      IF (MPMASTER) THEN
        CALL STOPMAT(PA,PAR,PAI,DSTO_ME,'REDUCE')
        IF (OKS) CALL STOPMAT(PB,PBR,PBI,DSTO_ME,'REDUCE')
      END IF
C
      CALL MPLIMITS(NCPU,ME,1,0,NARUN,LLVAR_ME,ULVAR_ME,
     $              (/0/),0)
C
      HAS = 0
      IF ((RTCYC.GE.LLVAR_ME).AND.(RTCYC.LE.ULVAR_ME)) THEN
        IF ((ANACYC.EQ.1).OR.
     $      (MDFLUSH(ANACYC,RTPRO))) HAS = ME 
      END IF
C
      CALL MP_SUM(HAS,HAS,1,MP_INT)
     
      IF ((ME.NE.HAS).AND.(ME.NE.MASTER)) GO TO 690
C
      IF (HAS.EQ.MASTER) GO TO 640
C
      ALLOCATE(PA(NSTO,NSTO),STAT=ALLOCATION)
      IF (OKS) ALLOCATE(PB(NSTO,NSTO),STAT=ALLOCATION)

      IF (MPMASTER) THEN
        CALL MP_SEND(PA,DSTO**2,HAS,MP_REAL)
      ELSE IF (ME.EQ.HAS) THEN
        CALL MP_GET(PA,DSTO**2,MASTER,MP_REAL)
      END IF
C
      IF (OKS) THEN
        IF (MPMASTER) THEN
          CALL MP_SEND(PB,DSTO**2,HAS,MP_REAL)
        ELSE IF (ME.EQ.HAS) THEN
          CALL MP_GET(PB,DSTO**2,MASTER,MP_REAL)
        END IF
      END IF
C
 640  CONTINUE
C
      IF (ME.NE.HAS) GO TO 650
C
      CALL RTPIO(PA,FCDA,DSTO,NAUX,'WRITE')
      IF (OKS) CALL RTPIO(PB,FCDB,DSTO,NAUX,'WRITE') 
C
 650  CONTINUE
C
      DEALLOCATE (PA)
      IF (OKS) DEALLOCATE (PB)
C
 690  CONTINUE
C
 700  CONTINUE
C
C     *** Restore fitted density coefficients ***
C
      IF (TOPCD) THEN
        FCDA(1:NAUX) = SFCDA(1:NAUX)
        FCDB(1:NAUX) = SFCDB(1:NAUX)
        DEALLOCATE(SFCDA,SFCDB,STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'RTANA','ALLOCATION FAILED')
      END IF
C
      CALL TIMER('STOP',TIMSTR)
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE RTANA ***
C  
      END 

