      SUBROUTINE RTOUT(PAR,PBR,DSTO_ME,OPTION)
C
C     Purpose:  RT-TDDFT output
C
C     History: - Creation (14.03.16, XJ)
C
C     ******************************************************************
C     List of local variables: 
C
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     PAR    : Real part of alpha density matrix.
C     PBR    : Real Part of beta density matrix.
c 
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'embed.h'
      INCLUDE 'flags.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mm.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'mmnrgy.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'projectile.h'
      INCLUDE 'property.h'
      INCLUDE 'qmmm.h'
      INCLUDE 'scf.h'
      INCLUDE 'xc.h'
      INCLUDE 'fileio.h'
C
      INTEGER DSTO_ME
      REAL PAR(DSTO_ME,DSTO_ME),PBR(DSTO_ME,DSTO_ME)
      CHARACTER*(*) OPTION
C 
      REAL EQMMMM,EQM,EPOLA,ETOTRT,ETOTRT_OLD,EQM_OLD,DIFFE,NUCREP
      REAL ERAD_OLD
      REAL NELP,NELAUX,NUCEMBED
      REAL TIME
C
      REAL, ALLOCATABLE :: DIST(:)
      CHARACTER*120 PRTSTR,STRCOMP,STRING*25
      CHARACTER*29 FORM3
      CHARACTER*30 FORM4
      INTEGER OFFSET,I,IATOM,IRAD
      REAL RADMASS,RADCHG,VEL
      INTEGER OUTFILE
      SAVE ETOTRT_OLD,EQM_OLD,ERAD_OLD
C
C     ------------------------------------------------------------------
C
C     *** Write propagation header ***
C
      IF (OPTION.EQ.'HEADER') THEN
C
        IF (.NOT.PRTMD) THEN
          CALL MESSAGE(1,'INFORMATION PROVIDED IN deMon.rtp FILE:',1,1)
C
C         *** Energy contribution ***
C
          IF (QMMM.AND.ADDRAD) THEN
            WRITE (PRTSTR,10) 
10          FORMAT('- TIME [AS], TOTAL ENERGY, QM ENERGY, MM ENERGY,',
     $             ' QM/MM INT ENERGY, ')
          ELSE IF (QMMM) THEN
            WRITE (PRTSTR,15) 
15          FORMAT('- TIME [AS], TOTAL ENERGY, QM ENERGY, MM ENERGY,',
     $             ' QM/MM INT ENERGY, ALL IN Ha')

          ELSE IF (ADDRAD) THEN
            WRITE (PRTSTR,20) 
20          FORMAT('- TIME [AS], TOTAL ENERGY, QM ENERGY' 
     $             ,', MOL/PROJ ENERGY, ALL IN Ha')
          ELSE
            WRITE (PRTSTR,25)
25          FORMAT('- TIME [AS], TOTAL ENERGY, IN Ha')  
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
C
          IF (QMMM) THEN
            WRITE (PRTSTR,30)
30          FORMAT('- TIME [AS], EMBEDDING ENERGIES, TOTAL',
     $      'ALL IN Ha')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
C         *** Population analysis printings ***
C
          IF (TDEKIN.AND.OKS) THEN
            WRITE (PRTSTR,100)
100       FORMAT('- TIME [AS], ATOMIC PROPERTIES, NUMBER, LABEL,' 
     $            ,' CHARGE, SPIN CHARGE, K [Ha], G[Ha], L[Ha]')  
          ELSE IF (TDEKIN) THEN
            WRITE (PRTSTR,150)
150       FORMAT('- TIME [AS], ATOMIC PROPERTIES, NUMBER, LABEL,'
     $            ,' CHARGE, K [Ha], G[Ha], L[Ha]')
          ELSE IF (OKS) THEN
            WRITE (PRTSTR,200)
200       FORMAT('- TIME [AS], ATOMIC PROPERTIES, NUMBER, LABEL,'
     $            ,' CHARGE, SPIN CHARGE')
          ELSE 
            WRITE (PRTSTR,250)
250        FORMAT('- TIME [AS], ATOMIC PROPERTIES, NUMBER, LABEL'
     $            ,' CHARGE')
          END IF
          CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          IF (FULLPA) THEN
            IF (UNIT(1).EQ.'ANGSTROM') THEN
              WRITE (PRTSTR,300)
300           FORMAT('- TIME [AS], ATOMIC DIPOLE, NUMBER, LABEL',
     $               ' DIPX, DIPY, DIPZ ALL IN [ESU]')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              WRITE (PRTSTR,350)
350           FORMAT('- TIME [AS], QUADRUPOLE MOMENT, AT NU, AT LAB',
     $               ' QXX, QXY, QXZ ,QYY, QYZ, QZZ ALL IN [ESU]')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            ELSE 
              WRITE (PRTSTR,400)
400           FORMAT('- TIME [AS], ATOMIC DIPOLE, AT NUM, AT LA', 
     $               ' DIPX, DIPY, DIPZ ALL IN [ESU]')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
              WRITE (PRTSTR,450)
450           FORMAT('- TIME [AS], QUADRUPOLE MOMENT, AT NU, AT LAB',
     $               ' QXX, QXY, QXZ ,QYY, QYZ, QZZ ALL IN [AU]')
              CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
            END IF
          END IF
C
C       *** PROJECTILE options ***
C
          IF (ADDRAD) THEN
            WRITE (PRTSTR,460) 
460         FORMAT('- TIME [AS], INCIDENT PARTICLE, VELOCITY [AU],',
     $             ' DISTANCE FROM QM [ANG], POSITION X, Y, Z IN ANG')
            CALL MESSAGE(0,STRCOMP(PRTSTR),0,0)
          END IF
C
           WRITE (OUT,5000)
C
        END IF
C
C     *** Write step information ***
C
      ELSE IF (OPTION.EQ.'STEP') THEN
C
        IF (MPSLAVE) GO TO 500
C
C     *** Set time at energy evaluation ***
C
          IF (PROPAGA.EQ.'EULER') THEN
            TIME = 1.E3*RTT/FSEC
          ELSE IF (PROPAGA.EQ.'MAGNUS') THEN
            IF (MAGPC) THEN
              TIME = 1.E3*(RTT-RTSTEP/2.0)/FSEC
            ELSE IF (MAGIT) THEN
              TIME = 1.E3*RTT/FSEC
            END IF
          END IF
C
C     *** Count electron number for propagation stability check ***
C
        CALL COUNTEL(PAR,PBR,DSTO_ME,NELP,NELAUX)
C
C     *** Calculate dipole moment ***
C     
        IF (OKS) THEN
          PAR(:,:) = PAR(:,:) + PBR(:,:)
          CALL MOMENT(PAR,1,DSTO_ME)
          PAR(:,:) = PAR(:,:) - PBR(:,:)
        ELSE 
          CALL MOMENT(PAR,1,DSTO_ME)
        END IF 
C 
C     *** Print data on output file ***
C
 500    CONTINUE 
        EMM = 0.0
        EQMMMM = 0.0
        EPOLA = 0.0
        EQM = EUPPER + EXC + EEXX + EVDW + NUCREP() 
        ETOTRT = EQM 
        IF (QMMM) THEN
          CALL MMDRV
          CALL EQMMM
          EQM = EQM - EPCQM 
          EQM = EQM - NUCEMBED()
          EQMMMM = EPCQM + EQMMMLJ + NUCEMBED()
        END IF
        ETOTRT = EQM + EMM + EQMMMM + ERADIA(1) + ERADIA(2) 
        DEPERT = EQM + EMM + EQMMMM - EQM_OLD 
        EQM_OLD = EQM + EMM + EQMMMM
        DERAD = ERADIA(1) + ERADIA(2) - ERAD_OLD
        ERAD_OLD = ERADIA(1)  + ERADIA(2)

        IF (MPSLAVE) GO TO 600 

        MDDIP(0) = SQRT(DOT_PRODUCT(MDDIP(1:3),MDDIP(1:3)))
C
        OUTFILE = OUT
        WRITE (OUTFILE,5400)  TIME,ETOTRT,
     $                  MDDIP(1)*ESU(1),MDDIP(2)*ESU(1),MDDIP(3)*ESU(1),
     $                  MDDIP(0)*ESU(1),NELP
C
C     *** Print energy contrinutions in deMon.rtp file ***
C
        IF (QMMM.AND.ADDRAD) THEN
          WRITE(RTP,5450)TIME,ETOTRT,EQM,EMM,EQMMMM,ERADIA(1)+ERADIA(2)
        ELSE IF (QMMM) THEN
          WRITE(RTP,5500) TIME,ETOTRT,EQM,EMM,EQMMMM
        ELSE IF (ADDRAD) THEN
          WRITE(RTP,5500) TIME,ETOTRT,EQM,ERADIA(1)+ERADIA(2)
        ELSE
          WRITE(RTP,5600) TIME,ETOTRT
        END IF
        IF (QMMM) THEN
          WRITE (RTP,5700) TIME,EPCQM,EQMMMM
        END IF 
C
 600  CONTINUE
C
C     *** Adapt imaginary propagation time step *** 
C
        IF (IMTPR.AND.ADRTSTEP) THEN
          DIFFE = ABS(ETOTRT-ETOTRT_OLD)
          IF (DIFFE.GT.1.E-1) THEN
            RTSTEP = 0.0001*FSEC
          ELSE IF (DIFFE.GT.1.E-2) THEN
            RTSTEP = 0.0005*FSEC
          ELSE IF (DIFFE.GT.1.E-3) THEN
            RTSTEP = 0.001*FSEC
           ELSE
            RTSTEP = 0.005*FSEC
          END IF
        ETOTRT_OLD = ETOTRT
        END IF
        IF (MPP) CALL MP_SEND_ALL(RTSTEP,1,MASTER,MP_REAL)
C
      END IF
C
C     *** Formats ***
C
 5000 FORMAT (//,T2,11X,'TIME [AS]',6X,'ETOT [A.U.]',6X,
     $        'DIP X [D]',7X,'DIP Y [D]',7X,
     $         'DIP Z [D]',6X,'DIP NORM [D]',5X,'NEL')

 5400 FORMAT (T2,'RTOUT: ',F10.3,1X,F20.9,1X,4(E15.7,1X),1X,F9.4)
 5450 FORMAT(T2,F12.3,1X,'ENERGY DECOMPOSITION',1X,5(F20.9,1X))
 5500 FORMAT(T2,F12.3,1X,'ENERGY DECOMPOSITION',1X,4(F20.9,1X))
 5600 FORMAT(T2,F12.3,1X,'TOTAL ENERGY',1X,F20.9,1X)
 5700 FORMAT(T2,F12.3,1X,'EMBEDDING ENERGIES',1X,2(F20.9,1X))
C    
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE OPRTOUT ***
C
      END
