       SUBROUTINE RTPOPOUT
C
C      Purpose: Print population analysis in the deMon.rtp outfile.
C
C      History: Creation (25.07.16, ADLL)
C
C      ****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
      INCLUDE 'projectile.h'
      INCLUDE 'property.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'strings.h'
C
      INTEGER I,IATOM,LC,MX,MY,MZ
      REAL DPOLE,GMOL,KMOL,QMOL,LMOL,PS2,QPOLE,SMOL,TIME,TWONPOLE
      REAL COM(3),DIP(3),R(3),TOPMOM(10)
C
      REAL,ALLOCATABLE :: DIST(:),PS(:)
      CHARACTER*32 FORM3
      CHARACTER*33 FORM4
      CHARACTER*34 FORM5
      CHARACTER*47 FORM0
      CHARACTER*48 FORM1
      CHARACTER*49 FORM2
      CHARACTER*80 STRING,SYSTEM
C
C     -----------------------------------------------------------------
C
C     *** Select unit system for topological analysis ***
C
      IF (UNIT(1).EQ.'ANGSTROM') THEN
        DPOLE = ESU(1)
        QPOLE = ESU(2)
        SYSTEM = 'ESU'
      ELSE
        DPOLE = 1.0
        QPOLE = 1.0
        SYSTEM = 'AU'
      END IF
C
C     *** Write charge analysis output ***
C
      TIME = 1.0E3*RTT/FSEC
      QMOL = 0.0
      KMOL = 0.0 
      GMOL = 0.0
      LMOL = 0.0
      DIP(:) = 0.0
      DO IATOM=1,NATOM
        IF (DEFORM) THEN
          QATOM(IATOM) = -NBASINA(IATOM) - NBASINB(IATOM)
        ELSE 
          QATOM(IATOM) = REAL(ZATOM(IATOM)) - NBASINA(IATOM) -
     $                                      NBASINB(IATOM)
        END IF
        QMOL = QMOL + QATOM(IATOM)
        KMOL = KMOL + KBASIN(IATOM)
        GMOL = GMOL + GBASIN(IATOM)
        LMOL = LMOL + LBASIN(IATOM)

        DO I=1,3
          DIP(I) = DIP(I) + QATOM(IATOM)*COORD(I,IATOM,CCSET)
        END DO 
      END DO
C
      IF (OKS) THEN
C
        SMOL = 0.0
        DO IATOM=1,NATOM
          IF (DEFORM) THEN
            QSPIN(IATOM) = -NBASINA(IATOM) + NBASINB(IATOM)
          ELSE
            QSPIN(IATOM) = NBASINA(IATOM) - NBASINB(IATOM)
          END IF
          SMOL = SMOL + QSPIN(IATOM)
        END DO
      END IF
C
      STRING = 'ATOMIC PROPERTIES'
      IF (TDEKIN.AND.OKS) THEN
        DO IATOM=1,NATOM
          WRITE (RTP,5010) TIME,STRING,IATOM,ATOMLAB(ATOMMAP(IATOM,1)),
     $                    QATOM(IATOM),QSPIN(IATOM),
     $                    KBASIN(IATOM),GBASIN(IATOM),LBASIN(IATOM)
        END DO
        WRITE (RTP,5011) TIME,'SUM OF ATOMIC PROPERTIES',
     $                   QMOL,SMOL,KMOL,GMOL,LMOL
5010    FORMAT (T2,F12.3,1X,A18,I5,1X,A3,1X,2(F8.4,1X),3(ES12.5,1X))
5011    FORMAT(T2,F12.3,1X,A,1X,2(F8.4,1X),3(ES12.5,1X))
      ELSE IF (TDEKIN) THEN
        DO IATOM=1,NATOM
          WRITE (RTP,5015) TIME,STRING,IATOM,ATOMLAB(ATOMMAP(IATOM,1)),
     $                    QATOM(IATOM),
     $                    KBASIN(IATOM),GBASIN(IATOM),LBASIN(IATOM)
        END DO
        WRITE (RTP,5016) TIME,'SUM OF ATOMIC PROPERTIES',
     $                   QMOL,KMOL,GMOL,LMOL
5015   FORMAT (T2,F12.3,1X,A18,I5,1X,A3,1X,F8.4,1X,3(ES12.5,1X))
5016   FORMAT(T2,F12.3,1X,A,1X,F8.4,1X,3(ES12.5,1X))
      ELSE IF (OKS) THEN
        DO IATOM=1,NATOM
          WRITE (RTP,5020) TIME,STRING,IATOM,ATOMLAB(ATOMMAP(IATOM,1)),
     $                    QATOM(IATOM),QSPIN(IATOM)
        END DO
        WRITE (RTP,5021) TIME,'SUM OF ATOMIC PROPERTIES',QMOL,SMOL
5020   FORMAT (T2,F12.3,1X,A18,I5,1X,A3,1X,2(F8.4,1X))
5021   FORMAT(T2,F12.3,1X,A,1X,2(F8.4,1X))
      ELSE 
        DO IATOM=1,NATOM
          WRITE (RTP,5025) TIME,STRING,IATOM,ATOMLAB(ATOMMAP(IATOM,1)),
     $                    QATOM(IATOM)
        END DO
        WRITE (RTP,5026) TIME,'SUM OF ATOMIC PROPERTIES',QMOL
5025   FORMAT (T2,F12.3,1X,A18,I5,1X,A3,1X,F8.4,1X)
5026   FORMAT(T2,F12.3,1X,A,1X,F8.4)
      END IF
C 
      WRITE (RTP,5040) TIME,(DPOLE*DIP(I),I=1,3)
 5040 FORMAT (T2,F12.3,1X,"TOTAL CHARGE DIPOLE",7X,3(E15.7,1X))
C
C     *** Write full population analysis output ***
C
      IF (FULLPA)  THEN
C
C     *** Calculate topological sums ***
C
        TOPMOM(:) = 0.0
        DO IATOM=1,NATOM
          DO LC=1,2
            DO MX=0,LC
              DO MY=0,LC-MX
                MZ = LC - MX - MY
                TOPMOM(COP(MX,MY,MZ)) = TOPMOM(COP(MX,MY,MZ)) +
     $                                MBASIN(COP(MX,MY,MZ),IATOM)
              END DO
            END DO
          END DO
        END DO
C
        STRING = 'ATOMIC DIPOLE'
        DO IATOM=1,NATOM
          WRITE (RTP,5090) TIME,STRING,IATOM,
     $                     ELSYM(ATOMNO(IATOM)),
     $                     (DPOLE*MBASIN(I,IATOM),I=2,4)
 5090     FORMAT (T2,F12.3,1X,A13,I5,A3,5X,F8.4,3X,F8.4,3X,F8.4)
        END DO
        WRITE (RTP,5100) TIME,(DPOLE*TOPMOM(I),I=2,4)
 5100   FORMAT (T2,F12.3,1X,"TOTAL DIPOLE",7X,3(E15.7,1X))
        WRITE (RTP,5110) TIME,(DPOLE*(DIP(I-1)+TOPMOM(I)),I=2,4)
 5110   FORMAT (T2,F12.3,1X,"Q+DIP DIPOLE",7X,3(E15.7,1X))
C
        STRING = 'ATOMIC QUADRUPOLE'
        DO IATOM=1,NATOM
          WRITE (RTP,5120) TIME,STRING,IATOM,
     $                     ELSYM(ATOMNO(IATOM)),
     $                     QPOLE*TWONPOLE(2,0,0,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(1,1,0,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(1,0,1,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(0,2,0,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(0,1,1,MBASIN(:,IATOM)),
     $                     QPOLE*TWONPOLE(0,0,2,MBASIN(:,IATOM))
 5120       FORMAT (T2,F12.3,1X,A17,I5,A3,5X,F8.4,5(3X,F8.4))
        END DO
        WRITE (RTP,5130) TIME,QPOLE*TWONPOLE(2,0,0,TOPMOM),
     $                   QPOLE*TWONPOLE(1,1,0,TOPMOM),
     $                   QPOLE*TWONPOLE(1,0,1,TOPMOM),
     $                   QPOLE*TWONPOLE(0,2,0,TOPMOM),
     $                   QPOLE*TWONPOLE(0,1,1,TOPMOM),
     $                   QPOLE*TWONPOLE(0,0,2,TOPMOM)
 5130   FORMAT (T2,F12.3,1X,'TOTAL QUADRUPOLE',7X,F8.4,5(3X,F8.4))
C
      END IF
C
C
C     -----------------------------------------------------------------
C
C     *** End of SUNROUTINE RTPOPOUT ***
C
      END
