      SUBROUTINE WTDELF(PAR,PAI,PBR,PBI,CDC,DSTO,FUNVEC,LLGPB,NGPB)
C
C     Purpose: Wrapper for Time dependent electron localization 
C     function (ELF).
C
C     History: - Creation (30.03.17, ADLL)
C 
C     ******************************************************************
C
C     List of wrapper arguments:
C
C     DSTO  : Dimension of Slater typ orbital matrix.
C     FUNVEC: Vector with plot function values.
C     LLGPB : Lower limit grid point of current batch.
C     NGPB  : Number of grid points in current batch.
C     PRE   : Density or spin density matrix.
C     PIM   : ??????????????????????? aai AAI
C
C     List of local variables:
C
C     TPH   : Two particle probability of the uniform electron gas.
C     TP    : Two particle probability.
C
C     List of dynamical fields:
C
C     D1STO: First derivatives of the basis functions.
C     STO  : Function values of the basis functions.
C
C     Output list:    Open shell         Closed shell
C
C     FUNVEC(IGP,1):  Alpha ELF             ELF
C     FUNVEC(IGP,2):  Alpha RHO             RHO
C     FUNVEC(IGP,3):  Beta ELF
C     FUNVEC(IGP,4):  Beta RHO
C     FUNVEC(IGP,5):  |GRAD alpha RHO|^2    |GRAD RHO|^2
C     FUNVEC(IGP,6):  |GRAD beta RHO|^2
C     FUNVEC(IGP,7):  Alpha TRHO            TRHO
C     FUNVEC(IGP,8):  Beta TRHO
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'math.h'
      INCLUDE 'plot.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'threshold.h'
C
      REAL FIVETHIRD,TWOTHIRD
      PARAMETER (FIVETHIRD = 5.0/3.0,TWOTHIRD = 2.0/3.0)
C
      INTEGER DSTO
      REAL PAR(DSTO,DSTO),PAI(DSTO,DSTO)
      REAL PBR(DSTO,DSTO),PBI(DSTO,DSTO)
C
      REAL CDC(MAXAUX),FUNVEC(MAXGPB,MAXPLOT)
C
      INTEGER IGP,IMO,LLGPB,NGPB
      REAL FERMIC,NU,TP,TPH
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: D1STO(:,:,:),STO(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      ALLOCATE(D1STO(MAXGPB,3,DSTO),STO(MAXGPB,DSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('WELF','ALLOCATION FAILED',1)
      END IF
C
C     *** Calculate factor for uniform electron gas ***
C
      FERMIC = ((6.0*PI**2)**TWOTHIRD)/FIVETHIRD 
C
C     *** Calculate basis function and derivative values ***
C
      CALL D1CHI(D1STO,STO,DSTO,LLGPB,NGPB)
C
C     *** Calculate kinetic energy density ***
C
      CALL KIN(PAR,D1STO,DSTO,FUNVEC(:,4),LLGPB,NGPB)
      IF (OKS) CALL KIN(PBR,D1STO,DSTO,FUNVEC(:,5),LLGPB,NGPB)

C     *** Calculate alpha density and density gradient norm ***
C
      CALL D1RHO(PAR,D1STO,STO,DSTO,FUNVEC(:,2),FUNVEC(:,8:10),
     $           LLGPB,NGPB)
      IF (OKS) CALL D1RHO(PBR,D1STO,STO,DSTO,FUNVEC(:,3),
     $                    FUNVEC(:,11:13),LLGPB,NGPB)
C
      FUNVEC(:,6:7) = 0.0
      DO IGP=1,NGPB
        FUNVEC(IGP,6) = FUNVEC(IGP,8)**2 + FUNVEC(IGP,9)**2 +
     $                  FUNVEC(IGP,10)**2
        IF (OKS) THEN
          FUNVEC(IGP,7) = FUNVEC(IGP,11)**2 + FUNVEC(IGP,12)**2 +
     $                    FUNVEC(IGP,13)**2
        END IF
      END DO
C
C     *** Calculate current density ***
C
      CALL JCUR(PAI,D1STO,STO,DSTO,FUNVEC(:,10:12),NGPB)
      IF (OKS) CALL JCUR(PBI,D1STO,STO,DSTO,FUNVEC(:,13:15),NGPB)
C
      FUNVEC(:,8:9) = 0.0
      DO IGP=1,NGPB
        FUNVEC(IGP,8) = FUNVEC(IGP,10)**2 + FUNVEC(IGP,11)**2 +
     $                FUNVEC(IGP,12)**2
        IF (OKS) THEN
          FUNVEC(IGP,9) = FUNVEC(IGP,13)**2 + FUNVEC(IGP,14)**2 +
     $                    FUNVEC(IGP,15)**2
        END IF
      END DO
C
C     *** Calculate scaling factors for ITELF/ELF definition ***
C
      NU = 2.0
C
C     *** Build ELF ***
C
      FUNVEC(:,1) = 0.0
      DO IGP=1,NGPB
        IF (OKS) THEN
         IF (FUNVEC(IGP,2).GT.RHOTHRESH) THEN
            TP = FUNVEC(IGP,4) - 0.125*FUNVEC(IGP,6) - 0.5*FUNVEC(IGP,8)
         END IF
         IF (FUNVEC(IGP,3).GT.RHOTHRESH) THEN
            TP = TP + FUNVEC(IGP,5) - 0.125*FUNVEC(IGP,7) 
     $                                - 0.5*FUNVEC(IGP,9)
         END IF
         IF ((FUNVEC(IGP,2)+FUNVEC(IGP,3)).GT.RHOTHRESH) THEN 
            TPH =  (2.0**FIVETHIRD)/(FERMIC*(FUNVEC(IGP,2)**FIVETHIRD 
     $                                     + FUNVEC(IGP,3)**FIVETHIRD))
           FUNVEC(IGP,1) = 1.0/(1.0 + (TP*TPH)**NU)
         END IF

        ELSE 
          IF (FUNVEC(IGP,2).GT.RHOTHRESH) THEN
            TP = FUNVEC(IGP,4) - 0.125*FUNVEC(IGP,6)/FUNVEC(IGP,2)
            TP = TP  - 0.5*FUNVEC(IGP,8)/FUNVEC(IGP,2)
            TPH =  (2.0**FIVETHIRD)/(FERMIC*FUNVEC(IGP,2)**FIVETHIRD)
            FUNVEC(IGP,1) =  1.0/(1.0 + (TP*TPH)**NU)
          END IF
C
C          IF (OKS) FUNVEC(IGP,2) = FUNVEC(IGP,2) + FUNVEC(IGP,3) 
C          FUNVEC(IGP,1) = 1.0/(1.0 + (TP*TPH)**NU)
        END IF
        FUNVEC(IGP,3) = FUNVEC(IGP,10) 
        FUNVEC(IGP,4) = FUNVEC(IGP,11) 
        FUNVEC(IGP,5) = FUNVEC(IGP,12) 

C        IF (ABS(FUNVEC(IGP,1)).LT.1.E-5) FUNVEC(IGP,1) = 0.0
        IF (ABS(FUNVEC(IGP,2)).LT.1.E-8) FUNVEC(IGP,2) = 0.0
      END DO
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(D1STO,STO,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('WTDELF','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE WTDELF ***
C
      END
