      SUBROUTINE ADDCAP(KSA,KSB,DSTO,OPTION)
C
C     Purpose: Add absorbing boundary potential to the imaginary part
C              of the Kohn-Sham matrix 
C   
C     Litt: Lopata, Govind, J. Chem. Theor. Comput. 2013, 9,4939.
C
C     History: - Creation (06.12.16, ADLL)
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO
      REAL KSA(DSTO,DSTO),KSB(DSTO,DSTO)
      CHARACTER*(*) OPTION
C
      INTEGER ALLOCATION,ISTO
      REAL FACTOR
      REAL,ALLOCATABLE :: GAM(:,:)
C 
C     -----------------------------------------------------------------
C
C     *** Quick return if no aborsbing potential is requested  ***
C
      IF (.NOT.ABSPOT) RETURN
C
C     *** Allocate dynamical arrays ***
C
      ALLOCATE(GAM(DSTO,DSTO),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('ADDABSP','ALLOCATION FAILED',1)
      END IF
C
C     *** Read CAP from tape and add to KS matrix **
C
      FACTOR = 1.0
      IF (OPTION.EQ.'REM') FACTOR = -1.0
C
      CALL SCFIO(GAM,DSTO,'READ','CAPA')
      KSA(:,:) = KSA(:,:) + FACTOR*GAM(:,:)
C
      IF (OKS) THEN
        CALL SCFIO(GAM,DSTO,'READ','CAPB')
        KSB(:,:) = KSB(:,:) + FACTOR*GAM(:,:)
      END IF
C
C     *** Dallocate dynamical arrays ***
C
      DEALLOCATE(GAM,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('ADDABSP','DEALLOCATION FAILED',1)
      END IF
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE ADDCAP ***
C
      END
