      SUBROUTINE BLDCAP(KS,EIG,DSTO,DSTO_ME,OPTION)
C
C     Purpose: Build absorbing boundary potential to remove unbound 
C              electrons in the course of RT-DFT simulations
C
C     Litt: Lopata, Govind, J. Chem. Theor. Comput. 2013, 9,4939.
C           Parise et al. J. Phys. Chem. Lett. 2018, 9, 844.
C
C     History: - Creation (06.12.16, ADLL)
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'fileio.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL EIG(DSTO),KS(DSTO,DSTO)
      CHARACTER*(*) OPTION
C
      REAL G0,KSI,ZERO
C
C     -----------------------------------------------------------------
C
C     *** Quick return if no absorbing potential is used ***
C
      IF (.NOT.ABSPOT) RETURN
C
      STATUS = 'BLDCAP'
C
C     *** Build real space absorbing potential matrix ***
C
      IF (CAPTYP.EQ.'SPACE') THEN
C
        CALL NUMCAP(KS,KS,DSTO) 

C     *** Build energy space absorbing potential matrix ***

      ELSE IF (CAPTYP.EQ.'ENERGY') THEN
C
        CALL ENERCAP(KS,EIG,DSTO,DSTO_ME,OPTION)
C
      END IF
C
      STATUS = 'RTDFT'
C
C     *** Print CAP matrix ***
C
      IF (MPMASTER.AND.PRTCAP) THEN
        IF (OPTION.EQ.'ALPHA') THEN
          CALL PRIMAT(KS,DSTO,1,DSTO,1,DSTO,3,8,OUT,
     $       'BASIS','TOTAL','ALPHA ABSORBING POTENTIAL MATRIX')
        ELSE IF (OPTION.EQ.'BETA') THEN
          CALL PRIMAT(KS,DSTO,1,DSTO,1,DSTO,3,5,OUT,
     $       'BASIS','TOTAL','BETA ABSORBING POTENTIAL MATRIX')
        END IF
      END IF
C
C     *** Save absorbing potential matrix on tape ***
C
      IF (MPMASTER.AND.(OPTION.EQ.'ALPHA')) THEN
        CALL SCFIO(KS,DSTO,'WRITE','CAPA')
      ELSE IF (MPMASTER.AND.(OPTION.EQ.'BETA')) THEN
         CALL SCFIO(KS,DSTO,'WRITE','CAPB')
      END IF
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE BLDCAP ***
C
      END
