      SUBROUTINE ENERCAP(KS,EIG,DSTO,DSTO_ME,SPIN)
C
C     Purpose: Build complex absorbing potential defined in energy
C              space.
C
C     Litterature : K Lopata, BEV Van Kuiken, M Khalil, N Govind
C                   J. Chem. Theor. Comput. 2012, 8, 3284.
C
C     History: - Creation (01.07.19, ADLL)
C
C     *****************************************************************
C
C     List of local variables:
C
C     KS: Alpha density matrix.
C     EIG: Beta density matrix.
C
C     -----------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KS(DSTO_ME,DSTO_ME),EIG(DSTO)
      CHARACTER*(*) SPIN
C
      INTEGER ALLOCATION,IMO
      REAL FACTOR,REMO,ZERO
      PARAMETER (ZERO = 0.0)
C
      REAL, ALLOCATABLE:: C(:,:),WORK(:,:)
      CHARACTER*80 FIXKEY,STRCOMP
C
C     -----------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      IF (MPSLAVE) GO TO 200
C
      ALLOCATE(C(DSTO_ME,DSTO_ME),WORK(DSTO_ME,DSTO_ME),STAT=ALLOCATION)
C
C     *** Load reference MO coefficients ***
C
      FIXKEY = 'MO'//SPIN
      FIXKEY = STRCOMP(FIXKEY)
C
      IF (MPMASTER) CALL SCFIO(C,DSTO,'READ',FIXKEY)
C
      KS(:,:) = C(:,:)
C
      CALL MESSAGE(1,'MO DEPENDENT DAMPING FACTORS',1,1)
      WRITE(OUT,4900)
      WRITE(OUT,4910)

      DO IMO=1,NORTH
C
        REMO = EIG(IMO) - P3ABS
        IF (REMO.LE.0.0) THEN
          FACTOR = 0.0
        ELSE IF (P2ABS.GT.0.0) THEN
          FACTOR = EXP(REMO*P2ABS) - 1
          FACTOR = -1*FACTOR*P1ABS
        ELSE 
          FACTOR = -1*P1ABS
        END IF
C
        IF (FACTOR.LT.P4ABS) FACTOR = P4ABS
C
        IF (FACTOR.EQ.ZERO) THEN
          WRITE(OUT,4920) IMO,EIG(IMO),FACTOR
        ELSE
          WRITE(OUT,4930) IMO,EIG(IMO),FACTOR,-1/(2*FACTOR*FSEC)
        END IF
        KS(:,IMO) = KS(:,IMO)*FACTOR  
C
      END DO
C
      WORK(:,:) = 0.0
      CALL GMULMAT(KS,C,WORK,DSTO,DSTO,DSTO,
     $             DSTO,DSTO,DSTO,'TRANSB') 
C
      KS(:,:) = WORK(:,:)
C
      DEALLOCATE(C,WORK,STAT=ALLOCATION)
C
 200  CONTINUE
C
C     *** Format handling ***
C
 4900 FORMAT(/,T2,'KS ORBITAL',2X,'ENERGY (Ha)',2X,'GAMMA (Ha)',
     $        2X,'LIFE TIME (fs)')
 4910 FORMAT(/,T2,'-------------------------------------------------'
     $,'---------------')
 4920 FORMAT (T2,I6,2X,F12.6,2X,F12.6,6X,'-')
 4930 FORMAT (T2,I6,2X,F12.6,2X,2(F15.6,2X))
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE ENERCAP ***
C
      END
