      SUBROUTINE CHEBYSHEV(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
C
C     Purpose: Matrix exponentiation using Chebyshev expansion and 
C              propagation matrix from over DT.
C
C     History: - Creation (19.02.18, ADLL)
C
C     ******************************************************************
C
C     List of local variables: 
C
C     KR :  Real part of KS matrix.
C     KI :  Imaginary part of KS matrix.
C     PR :  Real part of density matrix (--> propagated P).
C     PI :  Imaginary part of density matrix (--> propagated P).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME),DT
C    
      INTEGER ALLOCATION,I,ICHEB,ERROR,J,MAXLOOP
      REAL DTG,DTR,DTI,GAM,NORM
      REAL MAXEIGR,MAXEIGI,MINEIGR,MINEIGI
      COMPLEX EYE
      REAL, ALLOCATABLE :: ABES(:),DIAGR(:),DIAGI(:)
      REAL, ALLOCATABLE :: U2R(:,:),U2I(:,:),U1R(:,:),U1I(:,:)
      REAL, ALLOCATABLE :: U0R(:,:),U0I(:,:),UR(:,:),UI(:,:) 
      COMPLEX, ALLOCATABLE :: KC(:,:),SEIGC(:,:),EIGEN(:) 
      LOGICAL CHECKCV,MDFLUSH
C
C     -----------------------------------------------------------------
C
C     *** Start timer for Recursive Chebyshev expansion ***
C
C     *** Determine spectral range ***
C
      IF (MDFLUSH(RTCYC,RTDIAG)) THEN
C
        ALLOCATE(KC(DSTO_ME,DSTO_ME),SEIGC(DSTO_ME,DSTO_ME),
     $           EIGEN(DSTO_ME),STAT=ALLOCATION)
        IF (ALLOCATION.GT.0) THEN
          CALL ERRMSG('CHEBYSHEV','ALLOCATION FAILED',1)
        END IF

        IF (MPMASTER) THEN
          EYE = CMPLX(0,1)
          DO I=1,DSTO
             DO J=1,DSTO
               KC(I,J) = (KR(I,J) + EYE*KI(I,J))
             END DO
          END DO
        END IF
C
        CALL DOEXPMAT(KC,SEIGC,EIGEN,DSTO_ME)
C
        MAXEIGR = -99999.9999
        MAXEIGI = -99999.9999
        MINEIGR = 99999.9999
        MINEIGI = 99999.9999
        DO I=1,DSTO_ME
          IF (REAL(EIGEN(I)).GT.MAXEIGR) MAXEIGR = REAL(EIGEN(I))
          IF (AIMAG(EIGEN(I)).GT.MAXEIGI) MAXEIGI = AIMAG(EIGEN(I))
          IF (REAL(EIGEN(I)).LT.MINEIGR) MINEIGR= REAL(EIGEN(I))
          IF (AIMAG(EIGEN(I)).LT.MINEIGI) MINEIGI = AIMAG(EIGEN(I))
        END DO
        WRITE (7,*) MAXEIGR,MAXEIGI,MINEIGR,MINEIGI
        IF ((ABS(MAXEIGR-MINEIGR)).GT.(ABS(MAXEIGI-MINEIGI))) THEN
          MINEMOA = MINEIGR
          MAXEMOA = MAXEIGR
        ELSE
          MINEMOA = MINEIGI
          MAXEMOA = MAXEIGI
        END IF
C
        DEALLOCATE(KC,SEIGC,EIGEN,STAT=ALLOCATION)
        IF (ALLOCATION.GT.0) THEN
          CALL ERRMSG('CHEBYSHEV','DEALLOCATION FAILED',1)
        END IF
      END IF
C
C     *** ALLOCATION ***
C
      CHECKCV = .FALSE.
      IF (NEXPM.LT.0) THEN
        CHECKCV = .TRUE.
        MAXLOOP = ABS(NEXPM)
      ELSE
        MAXLOOP = NEXPM
      END IF
C
      ALLOCATE(U2R(DSTO_ME,DSTO_ME),
     $         U2I(DSTO_ME,DSTO_ME),
     $         U1R(DSTO_ME,DSTO_ME),
     $         U1I(DSTO_ME,DSTO_ME),
     $         U0R(DSTO_ME,DSTO_ME),
     $         U0I(DSTO_ME,DSTO_ME),
     $         UR(DSTO_ME,DSTO_ME),
     $         UI(DSTO_ME,DSTO_ME),
     $         ABES(MAXLOOP+1),DIAGR(DSTO_ME),DIAGI(DSTO_ME)
     $         ,STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('CHEBYSHEV','ALLOCATION FAILED',1)
      END IF
C
C      *** Rescale KS matrix to unit disk ***
C
      UR(:,:) = 0.0
      UI(:,:) = 0.0
      U2R(:,:) = 0.0
      U2I(:,:) = 0.0
C

      GAM=CHEBSCA/2*(MAXEMOA - MINEMOA)
      IF (MPMASTER) THEN
        DO I=1,DSTO
          U2R(I,I) = GAM - MINEMOA
        END DO
        KR(:,:) = KR(:,:) - U2R(:,:)
        KR(:,:) = KR(:,:)/GAM
        KI(:,:) = KI(:,:)/GAM
      END IF
C
C     *** Calculate Chebyshev expansion coefficients ***
C
      DTG = DT*GAM
      CALL CHEBYBESS(DTG,MAXLOOP+1,ABES,ERROR)
      ERROR = ERROR - 129
      DO I=2,MAXLOOP+1
        ABES(I) = ABES(I) + ABES(I)
      END DO
C
C     *** Calculate first two terms of expansion ***
C
      IF (MPMASTER) THEN
        DO I=1,DSTO
          U2R(I,I) = 1.0
        END DO
        UR(:,:) = ABES(1)*U2R(:,:) 
        UI(:,:) = ABES(1)*U2I(:,:)
C
        U1R(:,:) = KR(:,:) 
        U1I(:,:) = KI(:,:) 
        UR(:,:) = UR(:,:) + ABES(2)*U1I(:,:)
        UI(:,:) = UI(:,:) - ABES(2)*U1R(:,:)
      END IF
C
      ICHEB = 1

 50   CONTINUE
C
      ICHEB = ICHEB + 1
C
      U0R(:,:) = -1.0*U2R(:,:)
      U0I(:,:) = -1.0*U2I(:,:)
      U2R(:,:) = U1R(:,:)
      U2I(:,:) = U1I(:,:)
C
      CALL MULMATI(KR,KI,U1R,U1I,DSTO,DSTO_ME,'REPLACEB')
C
      U0R(:,:) = U0R(:,:) + 2.*U1R(:,:)
      U0I(:,:) = U0I(:,:) + 2.*U1I(:,:)
C
      DTR = 0.0
      DTI = 0.0
C
      IF(MOD(ICHEB+3,4).EQ.0) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (U0R(J,J)*ABES(ICHEB+1))**2
          DTR = DTR + (U0I(J,J)*ABES(ICHEB+1))**2
        END DO
        UI(:,:) = UI(:,:) - U0R(:,:)*ABES(ICHEB+1)
        UR(:,:) = UR(:,:) + U0I(:,:)*ABES(ICHEB+1)
      ELSE IF (MOD(ICHEB+3,4).EQ.1) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (U0I(J,J)*ABES(ICHEB+1))**2
          DTR = DTR + (U0R(J,J)*ABES(ICHEB+1))**2
        END DO
        UI(:,:) = UI(:,:) - U0I(:,:)*ABES(ICHEB+1)
        UR(:,:) = UR(:,:) - U0R(:,:)*ABES(ICHEB+1)
      ELSE IF (MOD(ICHEB+3,4).EQ.2) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (U0R(J,J)*ABES(ICHEB+1))**2
          DTR = DTR + (U0I(J,J)*ABES(ICHEB+1))**2
        END DO
        UI(:,:) = UI(:,:) + U0R(:,:)*ABES(ICHEB+1)
        UR(:,:) = UR(:,:) - U0I(:,:)*ABES(ICHEB+1)
      ELSE IF (MOD(ICHEB+3,4).EQ.3) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (U0I(J,J)*ABES(ICHEB+1))**2
          DTR = DTR + (U0R(J,J)*ABES(ICHEB+1))**2
        END DO
        UI(:,:) = UI(:,:) + U0I(:,:)*ABES(ICHEB+1)
        UR(:,:) = UR(:,:) + U0R(:,:)*ABES(ICHEB+1)
      END IF
C
      DTR = SQRT(DTR+DTI)
      U1R(:,:) = U0R(:,:)
      U1I(:,:) = U0I(:,:)
C
C     *** Check convergence ***
C
      WRITE(OUT,5000) ICHEB-1,ABES(ICHEB+1),DTR
5000  FORMAT(T2,'CHEBYSHEV ITERATIONS',1X,I4,1X,2(ES12.5,X))
C
      IF (CHECKCV) THEN
        IF (MPP) CALL MP_SEND_ALL(DTR,1,MASTER,MP_REAL)
        IF (DTR.LT.CHEBTOL) GO TO 100 
        IF (ICHEB.GE.MAXLOOP) CALL ERRMSG('CHEBYSHEV','UNABLE TO CONVER
     $GE CHEBYSHEV EXPANSION',1)
        GO TO 50
      ELSE IF (ICHEB.LT.MAXLOOP) THEN
        GO TO 50
      ELSE 
        GO TO 100
      END IF
C
 100   CONTINUE 
C
      CALL RMCHEBOUT(ICHEB-1)
C
C     *** Normalize expansion ***
C     
      IF (MPMASTER) THEN
        NORM=-2*(GAM + MINEMOA)*DT
        U1R(:,:) = UR(:,:)*COS(NORM) - UI(:,:)*SIN(NORM)
        U1I(:,:) = UR(:,:)*SIN(NORM) + UI(:,:)*COS(NORM)
        UR(:,:) = U1R(:,:)
        UI(:,:) = U1I(:,:)
      END IF
C     
C    *** P(t+dt) = U*P(t)*U+ ***
C
      CALL MPMULMAT(UR,PR,U2R,DSTO_ME,NORTH,'NORMAL')
      U2R(:,:) = -U2R(:,:)
      CALL MPMULMAT(UI,PI,U2R,DSTO_ME,NORTH,'NORMALADD')
      U2R(:,:) = -U2R(:,:)
      CALL MPMULMAT(UR,PI,U2I,DSTO_ME,NORTH,'NORMAL')
      CALL MPMULMAT(UI,PR,U2I,DSTO_ME,NORTH,'NORMALADD')
C
C    *** U*P(t)*U^-1 ***
C
      CALL MPMULMAT(U2R,UR,PR,DSTO_ME,NORTH,'TRANSB')
      CALL MPMULMAT(U2I,UI,PR,DSTO_ME,NORTH,'TRANSBADD')
      CALL MPMULMAT(U2R,UI,PI,DSTO_ME,NORTH,'TRANSB')
      PI(:,:) = -PI(:,:)
      CALL MPMULMAT(U2I,UR,PI,DSTO_ME,NORTH,'TRANSBADD')
C
      DEALLOCATE(UR,UI,U0R,U0I,U1R,U1I,U2R,U2I,
     $           ABES,DIAGR,DIAGI,STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('CHEBYSHEV','DEALLOCATION FAILED',1)
      END IF             
C
      RETURN
C
C     *** Error handling ***
C
9980  CONTINUE
      CALL ERRMSG('CHEBYSHEV','ERROR CALCULATING BESSEL FUNCTION FOR 
     $CHEBYSHEV EXPANSION',1)
9990  CONTINUE
      CALL ERRMSG('CHEBYSHEV','BESSEL COEFFICIENTS OUT OF MACHINE 
     $PRECISION',1)
  
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE TAYLOR ***
C
      END 
