      SUBROUTINE DIAEXP(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
C
C     History: - Creation (26.11.15, XJ)
C
C     ******************************************************************
C     List of local variables: 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME),DT
C
      COMPLEX EYE
C    
      INTEGER NRTY,ALLOCATION,I,J
      REAL, ALLOCATABLE :: PROI(:,:),PROR(:,:),SEIGI(:,:),SEIGR(:,:)
      COMPLEX, ALLOCATABLE :: KC(:,:),SEIGC(:,:),EIGEN(:)
C
C     -----------------------------------------------------------------
C
C     *** Allocation ***
C
      ALLOCATE(PROI(DSTO_ME,DSTO_ME),PROR(DSTO_ME,DSTO_ME),
     $         SEIGI(DSTO_ME,DSTO_ME),SEIGR(DSTO_ME,DSTO_ME),
     $         KC(DSTO_ME,DSTO_ME),SEIGC(DSTO_ME,DSTO_ME),
     $         EIGEN(DSTO_ME),
     $         STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('DIAEXP','ALLOCATION FAILED',1)
      END IF
C
C     *** Initialization of work fields ***
C
      KC(DSTO_ME,DSTO_ME) = (0.0,0.0)
      SEIGC(DSTO_ME,DSTO_ME) = (0.0,0.0)
      SEIGR(:,:) = 0.0
      SEIGI(:,:) = 0.0
      PROI(:,:) = 0.0
      PROR(:,:) = 0.0
C
C     *** Merge and scale real and imaginary parts of the KS matrix ***
C
      IF (MPMASTER) THEN
        EYE = CMPLX(0,1)
        DO I=1,DSTO
           DO J=1,DSTO
             IF (IMTPR) THEN
               KC(I,J) = CMPLX(-KR(I,J),-KI(I,J))*DT 
             ELSE
               KC(I,J) = CMPLX(KI(I,J),-KR(I,J))*DT
             END IF
           END DO
        END DO
C
C     *** Matrix exponentiation ***
C
        CALL DOEXPMAT(KC,SEIGC,EIGEN,DSTO_ME)
C
C     *** Separation of real and imaginary parts ***
C
        DO I=1,DSTO
           DO J=1,DSTO
             SEIGR(I,J) = REAL(SEIGC(I,J))
             SEIGI(I,J) = AIMAG(SEIGC(I,J))
           END DO 
        END DO
C
      END IF
C
C    *** P(t+dt) = U*P(t)*U+***
C
      CALL MPMULMAT(SEIGR,PR,PROR,DSTO_ME,NORTH,'NORMAL')
      PROR(:,:) = -PROR(:,:)
      CALL MPMULMAT(SEIGI,PI,PROR,DSTO_ME,NORTH,'NORMALADD')
      PROR(:,:) = -PROR(:,:)
      CALL MPMULMAT(SEIGR,PI,PROI,DSTO_ME,NORTH,'NORMAL')
      CALL MPMULMAT(SEIGI,PR,PROI,DSTO_ME,NORTH,'NORMALADD') 
C
C    *** U*P(t)*U^-1***
C
      CALL MPMULMAT(PROR,SEIGR,PR,DSTO_ME,NORTH,'TRANSB')
      CALL MPMULMAT(PROI,SEIGI,PR,DSTO_ME,NORTH,'TRANSBADD')
      CALL MPMULMAT(PROR,SEIGI,PI,DSTO_ME,NORTH,'TRANSB')
      PI(:,:) = -PI(:,:)
      CALL MPMULMAT(PROI,SEIGR,PI,DSTO_ME,NORTH,'TRANSBADD')
C
C     *** Deallocation ***
C
      DEALLOCATE(PROI,PROR,SEIGI,SEIGR,SEIGC,KC,EIGEN,STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('DIAEXP','ALLOCATION FAILED',1)
      END IF             
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DIAEXP ***
C
      END
