      SUBROUTINE TAYLOR(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
C
C     Purpose:  Density matrix propagation using matrix exponentiation 
C               by Taylor expansion: exp(A)= sum_n (1/n!*A^n)
C
C     History: - Creation (26.11.15, XJ)
C
C     ******************************************************************
C
C     List of local variables: 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME),DT
C    
      INTEGER ALLOCATION
      REAL, ALLOCATABLE :: WORK1(:,:),WORK2(:,:),UI(:,:),UR(:,:)
C
C     -----------------------------------------------------------------
C
C     *** Allocation ***
C
      ALLOCATE(WORK1(DSTO_ME,DSTO_ME),WORK2(DSTO_ME,DSTO_ME),
     $         UI(DSTO_ME,DSTO_ME),UR(DSTO_ME,DSTO_ME),
     $         STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('TAYLOR','ALLOCATION FAILED',1)
      END IF
C
C     *** Initialization of work fields ***
C
      UR(:,:) = 0.0
      UI(:,:) = 0.0
      WORK1(:,:) = 0.0
      WORK2(:,:) = 0.0
C
C     *** Matrix exponentiation ***
C
      CALL TYEPM(KR,KI,UR,UI,WORK1,WORK2,DSTO,DSTO_ME,DT)
C
C    *** P(t+dt) = U*P(t)*U+ ***
C
      CALL MPMULMAT(UR,PR,WORK2,DSTO_ME,NORTH,'NORMAL')
      WORK2(:,:) = -WORK2(:,:)
      CALL MPMULMAT(UI,PI,WORK2,DSTO_ME,NORTH,'NORMALADD')
      WORK2(:,:) = -WORK2(:,:)
      CALL MPMULMAT(UR,PI,WORK1,DSTO_ME,NORTH,'NORMAL')
      CALL MPMULMAT(UI,PR,WORK1,DSTO_ME,NORTH,'NORMALADD')
C
C    *** U*P(t)*U^-1 ***
C
      CALL MPMULMAT(WORK2,UR,PR,DSTO_ME,NORTH,'TRANSB')
      CALL MPMULMAT(WORK1,UI,PR,DSTO_ME,NORTH,'TRANSBADD')
      CALL MPMULMAT(WORK2,UI,PI,DSTO_ME,NORTH,'TRANSB')
      PI(:,:) = -PI(:,:)
      CALL MPMULMAT(WORK1,UR,PI,DSTO_ME,NORTH,'TRANSBADD')
C
C     *** Deallocation ***
C
      DEALLOCATE(WORK1,WORK2,UI,UR,STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('TAYLOR','ALLOCATION FAILED',1)
      END IF             
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE TAYLOR ***
C
      END 
