      SUBROUTINE TYEPM(KR,KI,UR,UI,W1,W2,DSTO,DSTO_ME,DT)
C
C     Purpose: Complex matrix exponentiation using Taylor expansion
C     History: Creation Aurelien de la LAnde, mars 2015
C
C     *****************************************************************
C
      IMPLICIT NONE
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'molecule.h'
      INCLUDE 'rtdft.h'
C
      REAL DT,DTR,DTI
      INTEGER DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL UR(DSTO_ME,DSTO_ME)
      REAL UI(DSTO_ME,DSTO_ME)
      REAL W1(DSTO_ME,DSTO_ME)
      REAL W2(DSTO_ME,DSTO_ME)
C
      INTEGER ITAY,I,J,MAXLOOP
      REAL N,M
      LOGICAL CHECKCV
C
C     -----------------------------------------------------------------
C
C     *** Initialization of output fields ***
C
      CHECKCV = .FALSE.
      IF (NEXPM.LT.0) THEN
        CHECKCV = .TRUE.
        MAXLOOP = ABS(NEXPM)
      ELSE
        MAXLOOP = NEXPM
      END IF
C
      N = 1.0
      M = 1.0
      UR(:,:) = 0.0
      UI(:,:) = 0.0
C
      DO I=1,DSTO_ME
        UR(I,I) = 1.0
        W1(I,I) = 1.0
      END DO 
      IF (MPMASTER) THEN
        UI(:,:) = -KR(:,:)*DT
        UR(:,:) = UR(:,:) + KI(:,:)*DT
        W1(:,:) = KR(:,:)
        W2(:,:) = KI(:,:)
      END IF
C
C     *** Calculation of Taylor expansion terms ***
C
      ITAY = 1
50    CONTINUE
      ITAY = ITAY + 1
    
      CALL MULMATI(KR,KI,W1,W2,DSTO,DSTO_ME,'REPLACEB')
    
      N = N*ITAY
      M = 1/N
      DTR = 0.0
      DTI = 0.0
C
      IF(MOD(ITAY+3,4).EQ.0) THEN
        DO J=1,DSTO_ME
          DTI = DTI + (W1(J,J)*(DT**ITAY)*M)**2
          DTR = DTR + (W2(J,J)*(DT**ITAY)*M)**2
        END DO
        UI(:,:) = UI(:,:) - W1(:,:)*(DT**ITAY)*M
        UR(:,:) = UR(:,:) + W2(:,:)*(DT**ITAY)*M
      ELSE IF (MOD(ITAY+3,4).EQ.1) THEN          
        DO J=1,DSTO_ME
          DTI = DTI + (W2(J,J)*(DT**ITAY)*M)**2
          DTR = DTR + (W1(J,J)*(DT**ITAY)*M)**2
        END DO
        UI(:,:) = UI(:,:) - W2(:,:)*(DT**ITAY)*M
        UR(:,:) = UR(:,:) - W1(:,:)*(DT**ITAY)*M  
      ELSE IF (MOD(ITAY+3,4).EQ.2) THEN          
        DO J=1,DSTO_ME
          DTI = DTI + (W1(J,J)*(DT**ITAY)*M)**2
          DTR = DTR + (W2(J,J)*(DT**ITAY)*M)**2
        END DO
        UI(:,:) = UI(:,:) + W1(:,:)*(DT**ITAY)*M
        UR(:,:) = UR(:,:) - W2(:,:)*(DT**ITAY)*M
      ELSE IF (MOD(ITAY+3,4).EQ.3) THEN  
        DO J=1,DSTO_ME
          DTI = DTI + (W2(J,J)*(DT**ITAY)*M)**2
          DTR = DTR + (W1(J,J)*(DT**ITAY)*M)**2
        END DO
        UI(:,:) = UI(:,:) + W2(:,:)*(DT**ITAY)*M
        UR(:,:) = UR(:,:) + W1(:,:)*(DT**ITAY)*M
      END IF
C
      DTR = SQRT(DTR+DTI)
C
C     *** Check convergence ***
C
      IF (CHECKCV) THEN
        IF(MPP) CALL MP_SEND_ALL(DTR,1,MASTER,MP_REAL)
        IF (DTR.LT.CHEBTOL) GO TO 100
        IF (ITAY.GE.MAXLOOP) CALL ERRMSG('TYEPM','UNABLE TO CONVERGE'
     $                                   //' TAYLOR EXPANSION',1)
        GO TO 50
      ELSE IF (ITAY.LT.MAXLOOP) THEN
        GO TO 50
      ELSE
        GO TO 100
      END IF
C
 100   CONTINUE
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE TYEPM ***
C
         END
