      SUBROUTINE SETPROJ(TIME,OPTION)
C
C     Purpose: Determine projectile position in inelastic
C              collision-type calculations. 
C
C     History: (Creation: 18.03.15, ADLL)
C
C     *****************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'physic.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'mm.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
C
      REAL TIME
      INTEGER IATOM,IEMBED,IRAD,I
      REAL CLAU,MC2,VELSQ,NORM,DMAX,RSQUARE
      REAL CC(3)
      CHARACTER*(*) OPTION 
C
C     -----------------------------------------------------------------
C
C     *** Set particle positions *** 
C
      MONOPOLE = .TRUE.
      MC2 = MRAD*AMU*(CLIGHT/(ABOHR*FSEC*1.E15))**2
      CLAU = CLIGHT/(ABOHR*FSEC*1.E15)
      VELSQ = CLAU**2*(1-(KINRAD/MC2 + 1)**(-2))
      VELSQ = 2*KINRAD/(MRAD*AMU)
C
      IF (STATUS.EQ.'RTDFT') THEN
        DO IRAD=1,NRADP
          QRAD(IRAD) = QIORAD
          RRAD(IRAD) = COVRAD(0)
          DO I=1,3
            CRAD(I,IRAD,CCSET) = X0IORAD(I,IRAD) + 
     $                           TIME*VIORAD(I,IRAD)*SQRT(VELSQ)
          END DO
        END DO
C
C     *** Find collision distance ***
C
        DMAX=9999.9999
        DO IATOM=1,NATOM
          DO IRAD=1,NRADP
            RSQUARE = (COORD(1,IATOM,CCSET) - CRAD(1,IRAD,CCSET))**2 +
     $                (COORD(2,IATOM,CCSET) - CRAD(2,IRAD,CCSET))**2 +
     $                (COORD(3,IATOM,CCSET) - CRAD(3,IRAD,CCSET))**2
            RSQUARE = MAX(0.0,SQRT(RSQUARE) - VDWRAD(ATOMNO(IATOM)))
            IF (RSQUARE.LE.DMAX) DMAX = RSQUARE
          END DO
        END DO

        IF (MPMASTER.AND.(OPTION.EQ.'PRTOUT')) THEN
          IF ((STATUS.EQ.'SCF').OR.(STATUS.EQ.'GRID')) THEN
            DO IRAD=1,NRADP
              WRITE(OUT,5000) QRAD(IRAD),(CRAD(I,IRAD,CCSET),
     $                        I=1,3)
            END DO
          ELSE IF (STATUS.EQ.'RTDFT') THEN
            DO IRAD=1,NRADP
              WRITE(RTP,6000) 1.E3*RTT/FSEC,SQRT(VELSQ),DMAX/BOHR,
     $                     (CRAD(I,IRAD,CCSET)/BOHR,I=1,3)
            END DO
          END IF
        END IF
C
      END IF
C
5000  FORMAT(/,T2,'FIXED CHARGED PARTICLE ADDED ',1X,4(F9.4,1X))
6000  FORMAT(T2,F12.3,1X,'INCIDENT PARTICLE',1X,5(F15.10,1X))
C
C     -----------------------------------------------------------------
C
C     *** End of SUBROUTINE SETPROJ ***
C
      END 
