      SUBROUTINE OPRT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
C
C     Purpose:  Propagate open-shell elecronic structures 
C
C     History: - Creation (21.10.15, XJ)
C
C     ******************************************************************
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     K{A/B}I    : imaginary parts of Kohn-Sham matrix.
C     K{A/B}R    : real parts of Kohn-Sham matrix.
C     P{A/B}I    : imaginary parts of density matrix.
C     P{A/B}R    : real parts of density matrix.
C 
C     ------------------------------------------------------------------
C
      use iso_c_binding
      use magma
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'numint.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'slp.h'
      INCLUDE 'tolerance.h'
C
      INTEGER DSTO,DSTO_ME
      REAL KAR(DSTO,DSTO),KAI(DSTO,DSTO)
      REAL PAI(DSTO_ME,DSTO_ME),PAR(DSTO_ME,DSTO_ME)
      REAL KBR(DSTO,DSTO),KBI(DSTO,DSTO)
      REAL PBI(DSTO_ME,DSTO_ME),PBR(DSTO_ME,DSTO_ME)
C
      LOGICAL MDFLUSH
      external blacs_get
      external blacs_gridinit
      external blacs_gridexit
      external blacs_exit
C
C     ------------------------------------------------------------------
C
C     *** Write header in output file ***
C
      CALL RTOUT(PAR,PBR,DSTO_ME,'HEADER')
C
C     *** Distribute global counters of RTDFT cycles ***
C
      CALL MP_SEND_ALL(GRTCYC,1,MASTER,MP_INT)
      CALL MP_SEND_ALL(FGRTCYC,1,MASTER,MP_INT)
C
      IF (USESCALA) THEN
        call blacs_get( 0, 0, ictxt )
        ictxt_0 = ictxt
        call blacs_gridinit( ictxt, 'All', nprow, npcol )
        call blacs_gridinit( ictxt_0, 'All', 1, 1 )
      ELSE IF (USEMAGMA) THEN
C     setting the MAGMA environment
        IF (MONOGPU) THEN
          IF (MPMASTER) THEN
            call magmaf_init()
            call magmaf_getdevice( magma_device_code )
            call magmaf_queue_create( magma_device_code, magma_queue )
          END IF
        ELSE IF (MULTIGPU) THEN
          call mpi_magma_init( DSTO, DSTO_ME)
        END IF
C       CALL EXT_MAGMAINIT('INIT')
      END IF
C
C     *** Perform electronic propagation *** 
C
      DO RTCYC=1,RTMAX
C
        GRTCYC = GRTCYC + 1
        IF (PROPAGA.EQ.'EULER') THEN
          CALL PEULER(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
        ELSE IF (PROPAGA.EQ.'MAGNUS') THEN
          IF (MAGPC) THEN
            CALL PMAGPC(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
          ELSE IF (MAGIT) THEN
            CALL PMAGIT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
          END IF
        END IF
C
        RTT = RTT + RTSTEP 
C
C     *** Print data on output file ***
C
        IF (MDFLUSH(GRTCYC,RTINT)) CALL RTOUT(PAR,PBR,DSTO_ME,'STEP')
C
C     *** Save data in restarting file ***
C
        IF (MDFLUSH(GRTCYC,RTSAV)) THEN
          CALL RTRESTART(KAR,KAI,PAR,PAI,DSTO,DSTO_ME,'SAVE','ALPHA')
          CALL RTRESTART(KBR,KBI,PBR,PBI,DSTO,DSTO_ME,'SAVE','BETA')
        END IF
C
C     *** Property calculations ***
C
        CALL RTANA(PAR,PAI,PBR,PBI,DSTO,DSTO_ME)
C
C     *** Recover XC grid type *** 

        CALL GRDDEF(XCGSN,'LOAD-TYPE')
        IF (GGFUN.EQ.'SCF') THEN
          IF (RANGRD) ROTGRD = .TRUE.
          GRDTOL = GRDTOL/10.0
        END IF
C
      END DO
C
      IF (USESCALA) THEN
        call blacs_gridexit( ictxt )
      ELSE IF (USEMAGMA) THEN
        IF (MONOGPU) THEN
          IF (MPMASTER) THEN
            call magmaf_queue_destroy( magma_queue )
            call magmaf_finalize()
          END IF
        ELSE IF (MULTIGPU) THEN
          call mpi_magma_finalize( DSTO, DSTO_ME)
        END IF
C       CALL EXT_MAGMAINIT('CLOSE')
      END IF
C
C     *** Final restarting file ***
C
      CALL RTRESTART(KAR,KAI,PAR,PAI,DSTO,DSTO_ME,'SAVE','ALPHA')
      CALL RTRESTART(KBR,KBI,PBR,PBI,DSTO,DSTO_ME,'SAVE','BETA')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE OPRT ***
C
      END
