      SUBROUTINE OPRTINI(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,
     $                   DSTO,DSTO_ME)
C
C     Purpose: Initialize RTDFT calculation for open-shell systems.
C
C     History: - Creation (09.09.15, XJ)
C                         (23.03.20, ADLL)
C
C     ******************************************************************
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables: 
C     
C     CI    : imaginary MO coefficient matrix
C     CR    : real MO coefficient matrix
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real parts of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real parts of density matrix.
C 
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'dynamics.h'
      INCLUDE 'eri.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'iteration.h'
      INCLUDE 'mmpot.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'numint.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'tolerance.h'
C
      INTEGER DSTO,DSTO_ME
      REAL PAI(DSTO_ME,DSTO_ME),PAR(DSTO_ME,DSTO_ME)
      REAL PBI(DSTO_ME,DSTO_ME),PBR(DSTO_ME,DSTO_ME)
      REAL KAR(DSTO,DSTO),KAI(DSTO,DSTO)
      REAL KBR(DSTO,DSTO),KBI(DSTO,DSTO)
C
      INTEGER ALLOCATION
      REAL TIME 
      REAL, ALLOCATABLE :: WORK(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Initialize open shell SCF ***
C
      CALL RTCINI('OPEN SHELL')
C
C     *** Restart open-shell RT-TDDFT simulation ***
C
      IF (RTRST) THEN
C
        CALL RTRESTART(KAR,KAI,PAR,PAI,DSTO,DSTO_ME,'RESTART','ALPHA')
        CALL RTRESTART(KBR,KBI,PBR,PBI,DSTO,DSTO_ME,'RESTART','BETA')
C
C     *** Distribute auxiliary vectors for direct scheme ***
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(RTT,1,MASTER,MP_REAL)
          CALL MP_SEND_ALL(GRTCYC,1,MASTER,MP_INT)
          CALL MP_SEND_ALL(FGRTCYC,1,MASTER,MP_INT)
          IF (ERITYP.EQ.'DIRECT') THEN
            CALL MP_SEND_ALL(CDCOLDA(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(CDCOLDB(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(CDJOLDA(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(CDJOLDB(1:NAUX),NAUX,MASTER,MP_REAL)
          END IF
C
          IF (TOPCD.AND.DEFORM) THEN
            CALL MP_SEND_ALL(MFCDA(1:NAUX),NAUX,MASTER,MP_REAL)
            CALL MP_SEND_ALL(MFCDB(1:NAUX),NAUX,MASTER,MP_REAL)
          END IF
C
C     *** Distribute XC grid ***
C
          CALL GRDDEF(XCGSN,'SEND_ALL')
C
        END IF
C
C     **** Initialize XC grid ***
C
        RETGRD = .TRUE.
        INITLEB = .TRUE.
        INITGRD = .TRUE.
        CALL GRDINI
        CALL GRDDEF(XCGSN,'LOAD-TYPE')
        IF (GGFUN.EQ.'SCF') THEN
          IF (RANGRD) ROTGRD = .TRUE.
          GRDTOL = GRDTOL/10.0
        END IF
        IF (.NOT.DIRGRD) CALL GRDRET
C
C     *** Save promolecular density on slave nodes ***
C
        IF (MPP) THEN
          ALLOCATE(WORK(DSTO,DSTO),STAT=ALLOCATION)
          IF (ALLOCATION.GT.0) THEN
            CALL ERRMSG('OPRTINI','ALLOCATION FAILED',1)
          END IF
          IF (MPMASTER) CALL SCFIO(WORK,DSTO,'READ','P0ALPHA')
          CALL MP_SEND_ALL(WORK,DSTO**2,MASTER,MP_REAL)
          CALL SCFIO(WORK,DSTO,'WRITE','P0ALPHA')
C
          IF (MPMASTER) CALL SCFIO(WORK,DSTO,'READ','P0BETA')
          CALL MP_SEND_ALL(WORK,DSTO**2,MASTER,MP_REAL)
          CALL SCFIO(WORK,DSTO,'WRITE','P0BETA')
C
          DEALLOCATE(WORK,STAT=ALLOCATION)
          IF (ALLOCATION.GT.0) THEN
            CALL ERRMSG('OPRTINI','DEALLOCATION FAILED',1)
          END IF
        END IF
C
      ELSE
C
C     *** Launch RT-TDDFT simulation from deMon.rst file ***
C
        CALL RSTIO(KAR,EIGVALA,PAR,DSTO,DSTO_ME,OCCNA,AOCCNA,ELCFGA,
     $             MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
        CALL RSTIO(KBR,EIGVALB,PBR,DSTO,DSTO_ME,OCCNB,AOCCNB,ELCFGB,
     $             MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
C
        KAI(:,:) = 0.0
        KBI(:,:) = 0.0
C
        IF (MPSLAVE) GO TO 100
C
C     *** Localize MOs ***
C
        IF (USERLMO) ALLOCATE(WORK(DSTO,DSTO))
C
        IF (USERLMO.AND.FBRMO) THEN
          CALL LOCALIZE(KAR,PAR,WORK,DSTO,'FB','ALL','LOOSE','ALPHA')
          CALL LOCALIZE(KBR,PBR,WORK,DSTO,'FB','ALL','LOOSE','BETA')
        ELSE IF (USERLMO.AND.PMRMO) THEN
          CALL LOCALIZE(KAR,PAR,WORK,DSTO,'PM','ALL','LOOSE','ALPHA')
          CALL LOCALIZE(KBR,PBR,WORK,DSTO,'PM','ALL','LOOSE','BETA')
        ELSE IF (USERLMO) THEN
          CALL LOCALIZE(KAR,PAR,WORK,DSTO,'CH','ALL','LOOSE','ALPHA')
          CALL LOCALIZE(KBR,PBR,WORK,DSTO,'CH','ALL','LOOSE','BETA')
        END IF
        IF (USERLMO) DEALLOCATE(WORK)
C
C     *** Exchange molecular orbitals ***
C
        CALL EXCMOS(KAR,DSTO,EIGVALA,MOEXCA,NMOEXCA)
        CALL EXCMOS(KBR,DSTO,EIGVALB,MOEXCB,NMOEXCB)
C
 100    CONTINUE
C
C     *** Save reference coefficients in orthonormal basis  ***
C
        IF (TDOCC) THEN
          IF (MPMASTER) THEN
            CALL SCFIO(KAI,DSTO,'READ','OVERLAP')
            CALL GMULMAT(KAI,KAR,PAI,DSTO,DSTO,DSTO,NSTO,NSTO,
     $                   NSTO,'NORMAL')
            CALL SCFIO(KAI,DSTO,'READ','ORTHO')
            CALL GMULMAT(KAI,PAI,PAR,DSTO,DSTO,DSTO,NSTO,NORTH,
     $                  NSTO,'TRANSA')
            KAI(:,:)  = PAR(:,:)
          END IF
C
          IF (MPP) CALL MP_SEND_ALL(KAI,NSTO**2,MASTER,MP_REAL)

          CALL SCFIO(KAI,DSTO,'WRITE','COLDALPHA')
C
          IF (MPMASTER) THEN
            CALL SCFIO(KBI,DSTO,'READ','OVERLAP')
            CALL GMULMAT(KBI,KBR,PBI,DSTO,DSTO,DSTO,NSTO,NSTO,
     $                   NOMOB,'NORMAL')
            CALL SCFIO(KBI,DSTO,'READ','ORTHO')
            CALL GMULMAT(KBI,PBI,PBR,DSTO,DSTO,DSTO,NSTO,NORTH,
     $                   NOMOB,'TRANSA')
            KBI(:,:) = PBR(:,:) 
          END IF
          IF (MPP) CALL MP_SEND_ALL(KBI,NSTO**2,MASTER,MP_REAL)
          CALL SCFIO(KBI,DSTO,'WRITE','COLDBETA')
C
          KAI(:,:) = 0.0
          PAI(:,:) = 0.0
          PAR(:,:) = 0.0
          KBI(:,:) = 0.0
          PBI(:,:) = 0.0
          PBR(:,:) = 0.0
C
        END IF
C
C       *** Build complex density matrices ***
C
        IF (MPSLAVE) GO TO 200
C
        CALL BLDCOMMAT(KAR,KAI,PAR,PAI,DSTO,OCCNA,LFOMOA,HFOMOA)
        CALL BLDCOMMAT(KBR,KBI,PBR,PBI,DSTO,OCCNB,LFOMOB,HFOMOB)
C  
        KAR(:,:) = PAR(:,:)
        KAI(:,:) = PAI(:,:)
        KBR(:,:) = PBR(:,:)
        KBI(:,:) = PBI(:,:)
C
 200    CONTINUE
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(KAR,NSTO**2,MASTER,MP_REAL)
          CALL MP_SEND_ALL(KAI,NSTO**2,MASTER,MP_REAL)
          CALL MP_SEND_ALL(KBR,NSTO**2,MASTER,MP_REAL)
          CALL MP_SEND_ALL(KBI,NSTO**2,MASTER,MP_REAL)
        END IF
C
        CALL SCFIO(KAR,DSTO,'WRITE','P0ALPHA')
        CALL SCFIO(KBR,DSTO,'WRITE','P0BETA')
C
        PAR(:,:) = PAR(:,:) + PBR(:,:)
        IF (MPMASTER) CALL MOMENT(PAR,1,DSTO)
        PAR(:,:) = PAR(:,:) - PBR(:,:)
C
C     *** Build charge density Coulomb vector ***
C
        IF (ERITYP.EQ.'DIRECT') THEN
          CALL BLDCDJ(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
        ELSE
          CALL GETCDJ(PAR,PBR,DSTO)
        END IF
C
C     *** Calculate charge density fit coefficients ***
C
        CALL GETCDC
C
C     *** Initialize deformation density analysis ***
C
        IF (DEFORM) THEN
          MFCDA(:) = FCDA(:)
          MFCDB(:) = FCDB(:)
        END IF
C
        IF (MPP) THEN
          CALL MP_SEND_ALL(MFCDA,NAUX,MASTER,MP_REAL)
          CALL MP_SEND_ALL(MFCDB,NAUX,MASTER,MP_REAL)
        END IF
C
C    *** Initialize position of projectile ***
C
        IF (ADDRAD) THEN
          TIME = 0.0
          CALL SETPROJ(TIME,'PRTOUT')
          CALL EMBEDRAD
        END IF
C
C     *** Calculation of energy bounds ***
C
        IF (MPMASTER) CALL EBOUNDS(KAR,PAR,PBR,QCDCA,QCDCB,DSTO)
C
        IF (MPMASTER) CALL MIXING(1.0,KAR,KBR,PAR,PBR,DSTO)
C
        IF (MPMASTER) THEN
          CALL STOPMAT(KAI,PAR,PAI,DSTO_ME,'REDUCE') 
          CALL STOPMAT(KBI,PBR,PBI,DSTO_ME,'REDUCE')
        END IF
C
C     *** Build Complex Absorbing Potential ***
C
        IF (ABSPOT) THEN
          CALL BLDCAP(KAR,EIGVALA(1:DSTO),DSTO,DSTO_ME,'ALPHA')
          CALL BLDCAP(KBR,EIGVALB(1:DSTO),DSTO,DSTO_ME,'BETA')
        END IF
C
        IF (EXX) THEN
          CALL FOCK(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
          IF (MPMASTER) THEN
            CALL MIXIO(KAR,DSTO,'WRITE','EXXALPHA')
            CALL MIXIO(KBR,DSTO,'WRITE','EXXBETA')
          END IF
        END IF

C     *** Build numerical exchange-correlation matrix ***
C
        CALL NUMXC(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
C
C     *** Build Kohn-Sham matrix ***
C
        CALL BLDKSMAT(KAR,KBR,PAR,PBR,DSTO,DSTO_ME)
C
C     *** Recover density matrix ***
C
        IF (MPMASTER) THEN
          CALL STOPMAT(KAI,PAR,PAI,DSTO_ME,'EXPAND')
          CALL STOPMAT(KBI,PBR,PBI,DSTO_ME,'EXPAND')
          KAI(:,:) = 0.0
          KBI(:,:) = 0.0
C
C     *** Save current KS matrix as OLD (since this is RTCYC=0) ***
C
          CALL SCFIO(KAR,DSTO,'WRITE','KOLDALPR')
          CALL SCFIO(KBR,DSTO,'WRITE','KOLDBETR')
          IF (DOIKS) THEN
            CALL SCFIO(KAI,DSTO,'WRITE','KOLDALPI')
            CALL SCFIO(KBI,DSTO,'WRITE','KOLDBETI')
          END IF
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE OPRTINI ***
C
      END
