      SUBROUTINE RTCINI(OPTION)
C
C     Purpose: initialize electron dynamics simulation. 
C
C     History: - Creation (09.09.15, XJ)
C                         (02.03.20, ADLL)
C                         
C     ******************************************************************
C
C     List of local variables:
C
C     OPTION: a) OPTION = CLOSED SHELL, closed shell RT-TDDFT.
C             b) OPTION = OPEN SHELL, open shell RT-TDDFT.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'iteration.h'
      INCLUDE 'molecule.h'
      INCLUDE 'numint.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
      INCLUDE 'threshold.h'
      INCLUDE 'tolerance.h'
      INCLUDE 'xc.h'
C
      CHARACTER OPTION*(*)
      INTEGER I
      LOGICAL STATED
C
C     ------------------------------------------------------------------
C
C     *** Initialize flags ***
C
      INITXC = .TRUE.
      CALXCE = .TRUE.
      CALXCV = .TRUE.
      IF (LAP.OR.ABSPOT.OR.EXX) THEN
        DOIKS = .TRUE.
      ELSE
        DOIKS = .FALSE.
      END IF
C
C     *** Initialize numerical values ***
C
      EXC = 0.0
      RTCYC = 0
      SCFCYC = 999999
      RTT = 0.0
C
C     *** Initialize auxiliary function vectors ***
C
      DIIS = .FALSE.
      CDFTYP = 'MIXING'
      CDCA(:) = 0.0
      CDCB(:) = 0.0 
      CDJA(:) = 0.0
      CDJB(:) = 0.0
      CDJMA(:) = 0.0
      CDJMB(:) = 0.0
      GCDCA(:) = 0.0
      GCDCB(:) = 0.0
      GFCDA(:) = 0.0
      GFCDB(:) = 0.0
      QCDCA(:) = 0.0
      QCDCB(:) = 0.0
      XCDCA(:) = 0.0
      XCDCB(:) = 0.0
C
C     *** Initialize vectors for direct SCF ***
C
      CDCOLDA(:) = 0.0
      CDCOLDB(:) = 0.0
      CDJOLDA(:) = 0.0
      CDJOLDB(:) = 0.0
C
C     *** Initialize orbital occupation ***
C
      NOMOA = 0
      NOMOB = 0
      OCCNA(:) = 0.0
      OCCNB(:) = 0.0
C
C     *** Calculate dipole moment ***
C
      CALL DIPOLES
C
      IF (OPTION.EQ.'CLOSED SHELL') THEN
        OCCNUM = 2.0
        NOMOA = NELEC/2
        NOMOB = NELEC/2
        CALL MOOCC(OCCNA,MOMODA,NMOMODA,NOMOA,LFOMOA,HFOMOA)
        MAXEMOA = EIGVALA(NSTO)
        MINEMOA = EIGVALA(1)
      ELSE IF (OPTION.EQ.'OPEN SHELL') THEN
        OCCNUM = 1.0
        NOMOA = (NELEC + MULTIP - 1)/2
        NOMOB = (NELEC - MULTIP + 1)/2
        CALL MOOCC(OCCNA,MOMODA,NMOMODA,NOMOA,LFOMOA,HFOMOA)
        CALL MOOCC(OCCNB,MOMODB,NMOMODB,NOMOB,LFOMOB,HFOMOB)
        MAXEMOA = EIGVALA(NSTO)
        MINEMOA = EIGVALA(1)
        MAXEMOB = EIGVALB(NSTO)
        MINEMOB = EIGVALB(1)
      ELSE
        CALL ERRMSG('RTCINI','UNKWNOWN OPTION',1)
      END IF

C
C     *** Initialize energy only and energy/gradient grid generation ***
C
      IF (.NOT.STATED('ATOM')) CALL GRDDEF(XCGSN,'LOAD-TYPE')
      IF (GGFUN.EQ.'SCF') THEN
        IF (RANGRD) ROTGRD = .TRUE.
        GRDTOL = GRDTOL/10.0
      END IF
      NELA = SUM(OCCNA(1:HFOMOA))
      IF (OKS) NELB = SUM(OCCNB(1:HFOMOB))
C
C     *** Initialize projectile position ***
C
      IF (ADDRAD) CALL PROJINI
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE RTCINI ***
C
      END
