      SUBROUTINE BLDCOMMAT(CR,CI,PR,PI,DSTO,OCCN,LFOMO,HFOMO)
C
C     Purpose: BuiLD density (COMplex) MATrix using the BLAS level three
C              symmetric rank k update.
C
C     History: - Creation (11.09.15, XJ)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     CR   : MO coefficient matrix for real part.
C     CI   : MO coefficient matrix contain imaginary part. 
C     HFOMO: Highest fractional occupied MO.
C     LFOMO: Lowest fractional occupied MO.
C     OCCN : Vector with the occupation numbers of the MOs.
C     PR   : Density matrix for real part.
C     PI   : Density matrix contain imaginary part.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'scf.h'
      INCLUDE 'scfvec.h'
C
      INTEGER DSTO
      REAL CR(DSTO,DSTO),PR(DSTO,DSTO)
      REAL CI(DSTO,DSTO),PI(DSTO,DSTO)
C
      REAL OCCN(*)
C
      INTEGER HFOMO,IMO,ISTO,JSTO,LFOMO
      REAL CJMO,CKMO
C
C     ------------------------------------------------------------------
C
C     *** Build density matrix for real part ***
       
      CALL BLDPMAT(CR,PR,NSTO,OCCN,LFOMO,HFOMO)
      CALL BLDPMAT(CI,PI,NSTO,OCCN,LFOMO,HFOMO)
      PR(:,:) = PR(:,:) + PI(:,:)
C
C     *** Build density matrix for imaginary part ***
C
      PI(:,:) = 0.0
      DO JSTO=1,NSTO
        DO IMO=1,HFOMO
          CJMO = 2.0*OCCN(IMO)*CR(JSTO,IMO)
          CKMO = 2.0*OCCN(IMO)*CI(JSTO,IMO)
          DO ISTO=1,JSTO-1
            PI(ISTO,JSTO) = PI(ISTO,JSTO) + CJMO*CI(ISTO,IMO)
            PI(ISTO,JSTO) = PI(ISTO,JSTO) - CKMO*CR(ISTO,IMO)
          END DO
          PI(JSTO,JSTO) = PI(JSTO,JSTO) + 0.5*CJMO*CI(JSTO,IMO)
          PI(JSTO,JSTO) = PI(JSTO,JSTO) - 0.5*CKMO*CR(JSTO,IMO)
        END DO
      END DO

      CALL SYMMAT(PI,DSTO,NSTO,'-UPLOW')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE BLDCOMMAT ***
C
      END

