      SUBROUTINE COMCPLX(AR,AI,BR,BI,DSTO,DSTO_ME,OPTION)
C
C     Purpose:  Calculate the commutator of two complex matrices. 
C
C     History: - Creation (29.10.15, XJ)
C                         (30.08.16, ADLL)
C
C     ******************************************************************
C     List of local variables: 
C
C     AI    : imaginary parts of Kohn-Sham matrix.
C     AR    : real parts of Kohn-Sham matrix.
C     BI    : imaginary parts of density matrix.
C     BR    : real parts of density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO,DSTO_ME
      REAL AR(DSTO,DSTO),AI(DSTO,DSTO)
      REAL BR(DSTO_ME,DSTO_ME),BI(DSTO_ME,DSTO_ME)
      CHARACTER*(*) OPTION
C
      INTEGER ALLOCATION
      REAL, ALLOCATABLE :: WORK1(:,:),WORK2(:,:) 
C
C     ------------------------------------------------------------------
C
      ALLOCATE(WORK1(MIN(DSTO,DSTO_ME),MIN(DSTO,DSTO_ME)),
     $         WORK2(MIN(DSTO,DSTO_ME),MIN(DSTO,DSTO_ME)),
     $         STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('COMCPLX','ALLOCATION FAILED',1)
      END IF
C
C    *** Calculate real part PROR=(AR*BR-PB*AR-AI*BI+BI*AI)***
C
      CALL MPMULMAT(BR,AR,WORK1,DSTO_ME,NORTH,'NORMAL')
      WORK1(:,:) = -WORK1(:,:)
      CALL MPMULMAT(AR,BR,WORK1,DSTO_ME,NORTH,'NORMALADD')
C
      IF (DOIKS) THEN
        WORK1(:,:) = -WORK1(:,:)
        CALL MPMULMAT(AI,BI,WORK1,DSTO_ME,NORTH,'NORMALADD')
        WORK1(:,:) = -WORK1(:,:)
        CALL MPMULMAT(BI,AI,WORK1,DSTO_ME,NORTH,'NORMALADD')
      END IF
C
C    *** Calculate imaginary part PROI=(AR*PI-PI*AR+AI*BR-BR*AI)***
C
      CALL MPMULMAT(BI,AR,WORK2,DSTO_ME,NORTH,'NORMAL')
      WORK2(:,:) = -WORK2(:,:)
      CALL MPMULMAT(AR,BI,WORK2,DSTO_ME,NORTH,'NORMALADD')
C
      IF (DOIKS) THEN
        WORK2(:,:) = -WORK2(:,:)
        CALL MPMULMAT(AI,BR,WORK2,DSTO_ME,NORTH,'NORMALADD')
        WORK2(:,:) = -WORK2(:,:)
        CALL MPMULMAT(BR,AI,WORK2,DSTO_ME,NORTH,'NORMALADD')
      END IF
C
C     *** Save results ***
C
      IF (OPTION.EQ.'REPLACEA') THEN
        AR(:,:) = WORK1(:,:)
        AI(:,:) = WORK2(:,:)
      ELSE IF (OPTION.EQ.'REPLACEB') THEN
        BR(:,:) = WORK1(:,:)
        BI(:,:) = WORK2(:,:)
      END IF
C
C     *** Deallocate dynamical arrays ***
C
      DEALLOCATE(WORK1,WORK2,STAT=ALLOCATION)
      CALL MP_SUM(ALLOCATION,ALLOCATION,1,MP_INT)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('COMCPLX','DEALLOCATION FAILED',1)
      END IF
C
C     *** End of SUBROUTINE COMCPLX ***
c
      END
