      SUBROUTINE RTPIO(P,CDC,DSTO,DAUX,OPTION)
C
C     Purpose: Driver routine for Real-Time Propagation
C              Input/Output.
C
C     History: - Creation (26.07.16, ADLL)
C
C     ******************************************************************
C
C     List of local dimension:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     P     : Compacted density matrix 
C     OPTION: a) OPTION = OPEN, open PRO tape.
C             b) OPTION = WRITE, write to PRO tape.
C             c) OPTION = READ, read from PRO tape.
C             d) OPTION = CLOSE, close PRO tape.
C             e) OPTION = DELETE, delete PRO tape.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'fileio.h'
C
      INTEGER DSTO,DAUX
      REAL CDC(DAUX)
      REAL P(DSTO,DSTO)
C
      CHARACTER*(*) OPTION
      CHARACTER*80 PRTSTR,STRCOMP
C
      INTEGER NSTO
C
C     ------------------------------------------------------------------
C
C     *** Open pro-molecular atomic density scratch I/O ***
C
      IF (OPTION.EQ.'OPEN') THEN
C
        CALL SCRSEQ(P,DSTO**2,IORTP,'OPEN')  
C
C     *** Write atomic pro-molecular density to scratch tape ***
C
      ELSE IF (OPTION.EQ.'WRITE') THEN
C
        WRITE (IORTP,ERR=99970) DSTO
        CALL SCRSEQ(P,DSTO**2,IORTP,'WRITE')
C
        CALL SCRSEQ(CDC,DAUX,IORTP,'WRITE')
C
C     *** Read atomic density from scratch tape ***
C
      ELSE IF (OPTION.EQ.'READ') THEN
C
C     *** Initialization ***
C
        P(1:DSTO,1:DSTO) = 0.0
C
C     *** Read and check atomic field dimensions ***
C
        READ (IORTP,ERR=99980,END=99990) NSTO
        IF (NSTO.NE.DSTO) THEN
          WRITE (PRTSTR,5000) NSTO,DSTO
 5000     FORMAT ('BASIS DIM. MISMATCH: NSTO = ',I5,' DSTO = ',I5)
          CALL ERRMSG('RTPIO',STRCOMP(PRTSTR),1)
        END IF
C
C     *** Read atomic pro-molecular density from scratch tape ***
C
        CALL SCRSEQ(P,DSTO**2,IORTP,'READ')
C
        CALL SCRSEQ(CDC,DAUX,IORTP,'READ')
C
C     *** Close pro-molecular atomic density scratch I/O ***
C
      ELSE IF (OPTION.EQ.'CLOSE') THEN
C
        CALL SCRSEQ(P,DSTO**2,IORTP,'CLOSE')
        NREC(IORTP) = -2
C
C     *** Delete pro-molecular atomic density scratch I/O ***
C
      ELSE IF (OPTION.EQ.'DELETE') THEN
C
        CALL SCRSEQ(P,DSTO**2,IORTP,'DELETE')
        NREC(IORTP) = -2
C
C     *** Error handling ***
C
      ELSE
C
        PRTSTR = 'UNKNOWN OPTION '//OPTION//' SPECIFIED'
        CALL ERRMSG('RTPIO',STRCOMP(PRTSTR),1)
C
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of pro-molecular density read and write errors ***
C
99970 CONTINUE
      CALL ERRMSG('RTPIO','ERROR WRITING PRO-MOLECULAR DENSITY',1)
C
99980 CONTINUE
      CALL ERRMSG('RTPIO','ERROR READING PRO-MOLECULAR DENSITY',1)
C
99990 CONTINUE
      CALL ERRMSG('RTPIO','UNEXPECTED END OF SCRATCH FILE',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE RTPIO ***
C
      END
