      SUBROUTINE TAYLORSCALA(KR,KI,PR,PI,DSTO,DSTO_ME,DT)
C
C     History: - Creation (24.01.19, KHA)
C
C     ******************************************************************
C     List of local variables: 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'slp.h'

      INTEGER(kind=4) DSTO,DSTO_ME
      REAL KR(DSTO,DSTO),KI(DSTO,DSTO)
      REAL PI(DSTO_ME,DSTO_ME),PR(DSTO_ME,DSTO_ME),DT
      INTEGER ALLOCATION

C     MPI and BLACS variables 
      integer(kind=4)               :: nb_procs,rank
C     local row and column index for the blacs grid
C     the process row and column parameters over which the first row of
C     the matrix is distributed
      integer(kind=4)               :: rsrc,csrc
C     blacs descriptor for the local array (9 parameters)
      integer(kind=4), dimension(9) :: desc_0, desc_loc
C     the leading dimension of the local array storing the local blocks
C     of the distributed matrix
      integer(kind=4)               :: lld
C     Global row and column indexes which points to the beginning of the
C     submatrices
      integer(kind=4)               :: i_kc, j_kc
      integer(kind=4)               :: i_pc, j_pc
      integer(kind=4)               :: i_seigc, j_seigc
      integer(kind=4)               :: i_zn, j_zn
C     dimension of the local sub-blocs
      integer(kind=4)               :: dim_row, dim_col
C     factor for the pdgemm subroutine such as C = alpha * A * B + beta
C     * C
      complex(kind=8)               :: alpha,beta
C     exit error info 
      integer(kind=4)               :: info
      integer(kind=4)               :: m,n
      complex(kind=8)               :: In 
  
      real, allocatable, dimension(:,:)            :: KR_grid, KI_grid
      real, allocatable, dimension(:,:)            :: PR_grid, PI_grid
      complex(kind=8), allocatable, dimension(:,:) :: KC_grid 
      complex(kind=8), allocatable, dimension(:,:) :: PC_grid
      complex(kind=8), allocatable, dimension(:,:) :: ID, ID_grid 
      complex(kind=8), allocatable, dimension(:,:) :: Zn_grid
      complex(kind=8), allocatable, dimension(:,:) :: SEIGC_grid
      complex(kind=8), allocatable, dimension(:,:) :: TEMP_grid

      external blacs_pinfo
      external blacs_gridinfo
      external pzgemr2d
      external pzgemm
      integer, external              :: numroc

C     setting the BLACS and PBLAS variables
      rsrc = 0
      csrc = 0
      i_kc = 1
      j_kc = 1
      i_pc = 1
      j_pc = 1
      i_seigc = 1
      j_seigc = 1
      i_zn = 1
      j_zn = 1
      alpha = 1.0
      beta  = 0.0
    
C     getting information about the grid definition and local processes
      call blacs_pinfo(rank, nb_procs)
      call blacs_gridinfo( ictxt, nprow, npcol, myrow, mycol )

C     calculation of lld and the dimension of the local sub matrix
      lld = numroc(DSTO, mbsub, myrow, rsrc, nprow )
      dim_row = numroc(DSTO, mbsub, myrow, rsrc, nprow )
      dim_col = numroc(DSTO, nbsub, mycol, rsrc, npcol )

C     initialization of the local descriptor
      call descinit(desc_loc, DSTO, DSTO, mbsub, nbsub,
     $     rsrc, csrc, ictxt, lld, info)

C     initialization of the descriptor for the global matrix
      desc_0(:) = 0
      desc_0(2) = -1
C     allocation and initialization of the indentity matrix by the
C     master processus
      if(rank == 0) then
C     initialization of the descriptor for the global matrix
         call descinit(desc_0,
     $        DSTO, DSTO, DSTO, DSTO,
     $        rsrc, csrc, ictxt_0, DSTO, info)
C     allocation and initialization of the global matrix ID
         allocate(ID(DSTO,DSTO))
         ID(:,:) = 0
         do m=1,DSTO
            ID(m,m) = 1.0
         enddo
      end if

C
C     *** Allocation ***
C
      allocate(KR_grid(dim_row,dim_col))
      allocate(KI_grid(dim_row,dim_col))
      allocate(PR_grid(dim_row,dim_col))
      allocate(PI_grid(dim_row,dim_col))
      allocate(KC_grid(dim_row,dim_col))
      allocate(PC_grid(dim_row,dim_col))
      allocate(ID_grid(dim_row,dim_col))
      allocate(Zn_grid(dim_row,dim_col))
      allocate(SEIGC_grid(dim_row,dim_col))
      allocate(TEMP_grid(dim_row,dim_col))

C     distribute the global matrices PR, PI, KR, KI and ID
C     in a 2D Block Cyclic Distribution way to their local matrices
      call pdgemr2d(DSTO, DSTO, KR, 1, 1, desc_0,
     $     KR_grid, 1, 1, desc_loc, ictxt)
      call pdgemr2d(DSTO, DSTO, KI, 1, 1, desc_0,
     $     KI_grid, 1, 1, desc_loc, ictxt)
      call pdgemr2d(DSTO, DSTO, PR, 1, 1, desc_0,
     $     PR_grid, 1, 1, desc_loc, ictxt)
      call pdgemr2d(DSTO, DSTO, PI, 1, 1, desc_0,
     $     PI_grid, 1, 1, desc_loc, ictxt)
      call pzgemr2d(DSTO, DSTO, ID, 1, 1, desc_0,
     $     ID_grid, 1, 1, desc_loc, ictxt)

      IF (IMTPR) THEN
        KC_grid(:,:) = cmplx(-KR_grid(:,:),-KI_grid(:,:))*DT
      ELSE
        KC_grid(:,:) = cmplx(KI_grid(:,:),-KR_grid(:,:))*DT
      END IF

C     Merge and scale real and imaginary parts of the KS matrix 
C      KC_grid(:,:) = cmplx(KI_grid(:,:),-KR_grid(:,:))*DT
C     Merge real and imaginary parts of the propagator matrix 
      PC_grid(:,:) = cmplx(PR_grid(:,:),PI_grid(:,:))

C
C     *** Matrix exponentiation ***
C

      In = 1
      SEIGC_grid(:,:) = ID_grid(:,:)
      TEMP_grid(:,:) = ID_grid(:,:)
      do n=1,NEXPM
         In = In * n
         Zn_grid(:,:) = TEMP_grid(:,:)
         call pzgemm('N','N', DSTO, DSTO, DSTO,
     $        alpha, KC_grid, i_kc, j_kc, desc_loc, Zn_grid,
     $        i_zn, j_zn, desc_loc, beta, TEMP_grid,
     $        i_zn, j_zn, desc_loc)
         SEIGC_grid(:,:) =
     $        SEIGC_grid(:,:) + TEMP_grid(:,:) / In
      enddo

C      
C     *** parallel matrix multiplication P(t+dt) = U*P(t)*U+ ***
C
      call pzgemm('N','C',DSTO, DSTO, DSTO,
     $     alpha, PC_grid, i_pc, j_pc, desc_loc, SEIGC_grid,
     $     i_seigc, j_seigc, desc_loc, beta, TEMP_grid,
     $     i_pc, j_pc, desc_loc)

      call pzgemm('N','N',DSTO, DSTO, DSTO,
     $     alpha, SEIGC_grid, i_seigc, j_seigc, desc_loc, TEMP_grid,
     $     i_pc, j_pc, desc_loc, beta, PC_grid,
     $     i_pc, j_pc, desc_loc)
 
      PR_grid(:,:) = REAL(PC_grid(:,:))
      PI_grid(:,:) = AIMAG(PC_grid(:,:))

C    distribute the local sub matrices PC with a 2D Block Cyclic
C    Distribution to the global matrix PC 
      call pdgemr2d(DSTO, DSTO, PR_grid,
     $     1, 1, desc_loc, PR, 1, 1, desc_0, ictxt)

      call pdgemr2d(DSTO, DSTO, PI_grid,
     $     1, 1, desc_loc, PI, 1, 1, desc_0, ictxt)

C     *** Deallocation ***
C
      if(rank == 0) then 
         deallocate(ID)
      end if

      DEALLOCATE(KR_grid,KI_grid,
     $           PR_grid,PI_grid,
     $           KC_grid,
     $           PC_grid,
     $           ID_grid,
     $           Zn_grid,
     $           TEMP_grid,
     $           SEIGC_grid,STAT=ALLOCATION)
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('EXPSCALA','ALLOCATION FAILED',1)
      END IF             

C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TAYLORSCALA ***
C
      END

