      SUBROUTINE SDIAGRD(PA,PB,OVEDD,DSTO,DOVER)
C
C     Purpose: Numerical calculation of the diagonal elements of the
C              overlap matrix for the adaptive
C              grid generation.
C
C     History: - Creation (23.04.20, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater type orbital matrix.
C     DOVER: Dimension of overlap vector.
C
C     List of local variables:
C
C     LLBATCH: Lower limit of a grid point batch.
C     LLGPB  : Lower limit grid point of a batch.
C     NGPB   : Number of grid points in the current batch.
C     ULBATCH: Upper limit of a grid point batch.
C     ULGPB  : Upper limit grid point of a batch.
C     OVEDD  : Electron density integrated o&n grid points. 
C
C     List of local dynamical fields:
C
C     D1HTO: First derivatives of Hermite typ orbitals.
C     D1STO: First derivatives of the basis functions.
C     HTO  : Function values of Hermite typ orbitals.
C     STO  : Function values of the basis functions.
C
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'xc.h'
C
      INTEGER DSTO,DOVER
C
      REAL OVEDD(DOVER)
      REAL PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      INTEGER IBATCH,LLBATCH,LLGPB,NGPB,ULBATCH,ULGPB
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: CDA(:),CDB(:) 
      REAL,ALLOCATABLE :: HTO(:,:),STO(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields for numerical integration ***
C
      IF (TOPCD) THEN
        ALLOCATE(CDA(MAXAUX),CDB(MAXAUX),HTO(MAXGPB,DOVER),
     $           STAT=ALLOCATION)
      ELSE 
        ALLOCATE(STO(MAXGPB,DOVER),STAT=ALLOCATION)
      END IF
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('SDIAGRD','ALLOCATION FAILED',1)
      END IF
C
      IF (TOPCD) THEN
        CALL BLDCDC(CDA,'BUILD ALPHA')
        IF (OKS) THEN 
          CALL BLDCDC(CDB,'BUILD BETA')
          CDA(:) = CDA(:) + CDB(:) 
        END IF
      END IF
C
C     *** Initialize ***
C
      OVEDD(:) = 0.0
C
C     *** Define lower and upper limit of  ***
C     *** the current batch of grid points ***
C
      LLBATCH = 1
      ULBATCH = (NGPTOT - 1)/MAXGPB + 1
C
C     *** Loop over all grid point batches in grid buffer ***
C
      DO 10 IBATCH=LLBATCH,ULBATCH
C
        IF (IBATCH.EQ.LLBATCH) THEN
          LLGPB = 1
          ULGPB = MIN(NGPTOT,MAXGPB)
        ELSE IF (IBATCH.EQ.ULBATCH) THEN
          LLGPB = (IBATCH - 1)*MAXGPB + 1
          ULGPB = NGPTOT
        ELSE
          ULGPB = IBATCH*MAXGPB
          LLGPB = ULGPB - MAXGPB + 1
        END IF
C
        NGPB = MIN(MAXGPB,ULGPB - LLGPB + 1)
C
C     *** Get chi or tau values and build density ***
C
        IF (TOPCD) THEN
          CALL TAU(HTO,DOVER,LLGPB,NGPB)
          CALL CDSDD(CDA,OVEDD,HTO,DOVER,LLGPB,NGPB)
        ELSE
          CALL CHI(STO,DOVER,LLGPB,NGPB)
          CALL OVERDIA(PA,OVEDD,STO,DOVER,LLGPB,NGPB)
        END IF
C
   10 CONTINUE
C
C     *** Deallocate local fields ***
C
      IF (TOPCD) THEN
        DEALLOCATE(CDA,CDB,HTO,STAT=ALLOCATION)
      ELSE
        DEALLOCATE(STO,STAT=ALLOCATION)
      END IF
C
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('SDIAGRD','DEALLOCATION FAILED',1)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE SDIAGRD ***
C
      END
