      SUBROUTINE SETTING
C
C     Purpose: Standard settings.
C
C     History: - Creation (16.01.96, AMK)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of status variables:
C
C     STATUS: Program status.
C
C     List of character variables:
C
C     ATOMLAB: Atomic label.
C     OPTVAR : Z-Matrix optimization variable and constant names.
C
C     List of numerical variables:
C
C     CFGCYC  : Number of SCF cycles for atomic configuration.
C     EDIIS   : Error of DIIS vector.
C     ELOWER  : Lower SCF energy bound.
C     ENERCYC : Number of energy evaluation cycles.
C     ERRCDC  : Error in charge density fitting coefficients.
C     ERRSCF  : Error in SCF energy.
C     ETHERMO : Energy transfer to the thermostat.
C     EUPPER  : Upper SCF energy bound.
C     EVDW    : Empirical van der Waals energy.
C     FFPTOL  : Finite field perturbation SCF tolerance.
C     FFSTEP  : Finite field step.
C     FFRSTEP : Finite field step for Raman calculation.
C     FIXCYC  : Start SCF cycle for fixing of MOs.
C     FIXTOL  : Tolerance for fixing of MOs.
C     FRICTION: Friction coefficient for temperature control.
C     FSCYC   : First level Shift CYCle.
C     GRADCYC : Number of gradient evaluation cycles.
C     ISOTOL  : Tolerance for isosurface reduction.
C     MDMAX   : Maximum number of molecular dynamics step.
C     MDMIN   : Minimum number of molecular dynamics step.
C     NFIX    : Number of fixed auxiliary functions.
C     NHITS   : Number of hits with confinement sphere.
C     NNULLMOS: Number of MOs with zero vector as eigenvector.
C     NRADGRD : User defined radial shells for SCF grid.
C     NRADTOP : User defined radial shells for topological grid.
C     NVARRST : Number of processed variables in frequency analysis.
C     OPTCYC  : Number of optimization cycle.
C     POLORD  : Polarizability order.
C     PRECYC  : Number of previous optimization cycles.
C     SADFAC  : Saddle optimization scaling factor.
C     TDELTA  : Step value for temperature.
C     THOOVER : Temperature scaling factor of Hoover thermostat.
C     TINIEST : Tiniest tolerance.
C     TLOWER  : Lower temperature boundary.
C     TUPPER  : Upper temperature boundary.
C     VIBSTEP : Vibrational displacement step.
C
C     List of global logical flags:
C
C     ESA     : If true, a electronic structure analysis is requested.
C     FALDA   : If true, LDA fxc kernel is used for all functionals.
C     GALDA   : If true, LDA gxc kernel is used for all functionals.
C     INITDIIS: If true, DIIS data are initialized.
C     INITGRD : If true, grid data are initialized.
C     INITLEB : If true, Lebedev grids are initialized.
C     INITOCT : If true, octree data are initialized.
C     INITXC  : If true, xc functional constants are initialized.
C     MONOPOLE: If true, use asymptotic expansion for embedding.
C     NEWINP  : If true, a new input was generated.
C     OMA     : If true, the optimal mixing approach is requested.
C     PLOTCD  : If true, plot charge density based quantities.
C     PROP    : If true, property calculation is requested.
C     RESTART : If true, the restart file will be read.
C     RETGRD  : If true, retrieve old grid for the next SCF run.
C     SCF     : If true, a single point start SCF is requested.
C     SSFG98  : If true, calculate G98 spin scaling function.
C     TOPCD   : If true, use charge density for topological analysis.
C     USEDD   : If true, use deformation density for plotting.
C     ZBUILD  : If true, Z-Matrix builder succeeded.
C
C     List of logical iteration flags:
C
C     BREAK    : If true, break iteration and stop program.
C     HOLDED   : If true, iteration is temporarily holded.
C     CONVERGED: If true, SCF is converged.
C
C     List of MOLDEN vectors:
C
C     DISMOL : Maximum and RMS displacements of optimization steps.
C     ESCFMOL: SCF energies for the first and last iteration.
C     FORMOL : Maximum and RMS forces of optimization steps.
C     SCFMOL : SCF energies of each optimization step.
C
C     List of property variables:
C
C     QATOM: Atomic net charge.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'dynamics.h'
      INCLUDE 'flags.h'
      INCLUDE 'freq.h'
      INCLUDE 'grid.h'
      INCLUDE 'iteration.h'
      INCLUDE 'molden.h'
      INCLUDE 'molecule.h'
      INCLUDE 'numint.h'
      INCLUDE 'opt.h'
      INCLUDE 'polarizability.h'
      INCLUDE 'property.h'
      INCLUDE 'scf.h'
      INCLUDE 'thermo.h'
      INCLUDE 'tolerance.h'
      INCLUDE 'xc.h'
C
C     ------------------------------------------------------------------
C
C     *** Cartesian coordinate set -> start with input coordinates ***
C
      CCSET = 1
C
C     *** Initialize program status ***
C
      STATUS = 'UNKNOWN'
C
C     *** Initialize character variables ***
C
      ATOMLAB(:) = 'UNDEFINED'
      OPTVAR(:,:) = 'UNDEFINED'
C
C     *** Initialize scan variables ***
C
      SCANVAR(:,:) = 0.0
      SCANVAR(1,0) = 5.0
C
C     *** Initialize global logical flags ***
C
      ESA = .FALSE.
      FALDA = .FALSE.
      GALDA = .FALSE.
      INITDIIS = .TRUE.
      INITGRD = .TRUE.
      INITLEB = .TRUE.
      INITOCT = .TRUE.
      INITXC = .TRUE.
      MONOPOLE = .TRUE.
      NEWINP = .FALSE.
      OMA = .FALSE.
      PLOTCD = .FALSE.
      PROP = .FALSE.
      RESTART = .FALSE.
      RETGRD = .FALSE.
      RETGRD_GST(:) = .FALSE.
      SCF = .TRUE.
      SSFG98 = .FALSE.
      TOPCD = .FALSE.
      USEDD = .FALSE.
      ZBUILD = .FALSE.
C
C     *** Set iteration flags ***
C
      BREAK = .FALSE.
      HOLDED = .FALSE.
      CONVERGED = .FALSE.
C
C     *** Set numerical tolerances ***
C
      FFPTOL = 5.0E-8
      GEOTOL = 1.0E-8
      ISOTOL = 1.0E-14
      TINIEST = 1.0E-14
C
C     *** Set SCF counters and tolerances ***
C
      CFGCYC = 0
      EDIIS = 1.0
      ELOWER = 0.0
      ERRCDC = 0.0
      ERRSCF = 0.0
      EUPPER = 0.0
      EVDW = 0.0
      FIXCYC = 0
      FIXTOL = 0.0
      FSCYC = 2
      NFIX = 0
      NNULLMOS = 0
      NRADGRD = 0
      NRADTOP = 0
C
C     *** Set optimization counters and tolerances ***
C
      ENERCYC = 0
      GRADCYC = 0
      OPTCYC = 0
      OPTNULL = 0
      PRECYC = 0
C
C     *** Set property counters and tolerances ***
C
      MDMAX = 10000
      MDMIN = 0
      NHITS = 0
      NVARRST = 0
      POLORD = 1
C
      ETHERMO = 0.0
      FFSTEP = 0.01
      FFRSTEP = 0.01
      SADFAC = 10.0
      TDELTA = 100.0
      THOOVER = 1.0
      TLOWER = 100.0
      TUPPER = 2000.0
      VIBSTEP = 1.0
C
C     *** Initialize MOLDEN vectors ***
C
      DISMOL = 0.0
      ESCFMOL = 0.0
      FORMOL = 0.0
      SCFMOL = 0.0
C
C     *** Initialize thermostat friction coefficient ***
C
      FRICTION = 0.0
C
C     *** Initialize atomic charges ***
C
      QATOM(:) = 0.0
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE SETTING ***
C
      END
