      SUBROUTINE WFNDRV(EIGVAL,OCCN,C,DSTO,OPTION)
C
C     Purpose: Driver routine for WFN file output.
C
C     History: - Creation (12.08.08, ADLL)
C                         (18.03.09, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     C     : Molecular orbital coefficients.
C     EIGVAL: Molecular orbital eigenvalues.
C     OCCN  : Vector with the MO occupation numbers.
C     OPTION: a) OPTION = HEADER, write WFN file header.
C             b) OPTION = BASIS, write basis set information.
C             c) OPTION = ALPHA, write primitive alpha SCF coefficients.
C             d) OPTION = BETA, write primitive beta SCF coefficients.
C             e) OPTION = END, write WFN file end statement.
C             f) OPTION = REWIND, rewind WFN file deMon.wfn.
C     
C     ******************************************************************
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'scfvec.h'
C
      INTEGER DSTO
      REAL C(DSTO,DSTO)
C
      REAL EIGVAL(*),OCCN(*)
C
      CHARACTER*(*) OPTION
C
C     ------------------------------------------------------------------
C
C     *** Quick return, if no WFN output is requested ***
C
      IF (WFNFILE.EQ.'NONE') RETURN
C
C     *** Write header and geometry ***
C
      IF (OPTION.EQ.'HEADER') THEN
C
        CALL WFNGEO('MOLECULE')
C
C     *** Write basis set information ***
C
      ELSE IF (OPTION.EQ.'BASIS') THEN
C
        CALL WFNBAS
C
C     *** Write primitive alpha SCF coefficients ***
C
      ELSE IF (OPTION.EQ.'ALPHA') THEN
C
        CALL WFNSCF(NOMOA,0,EIGVAL,OCCN,C,DSTO)
C
C     *** Write primitive beta SCF information ***
C
      ELSE IF (OPTION.EQ.'BETA') THEN
C
        CALL WFNSCF(NOMOB,NSTO,EIGVAL,OCCN,C,DSTO)
C
C     *** Write WFN fime end statement ***
C
      ELSE IF (OPTION.EQ.'END') THEN
C
        CALL WFNEND(0.0,0.0)
C
C     *** Rewind WFN file ***
C
      ELSE IF (OPTION.EQ.'REWIND') THEN
C
        REWIND (MOL)
C
      END IF
C
C     *** Flush buffer ***
C
      CALL DEFLUSH(MOL)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE WFNDRV ***
C
      END
