      SUBROUTINE WRTCUBE(IC,JC,KC,FUNVEC,NGPB,OPTION)
C
C     Purpose: Write cube file for plotting fields. 
C
C     History: - Creation (16.09.17, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C
C     FUNVEC    : Plot function value.
C     IC/JC/KC : Indices refering to the slice. 
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'embed.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'physic.h'
      INCLUDE 'plot.h'
      INCLUDE 'projectile.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'symmetry.h'
C
      REAL FUNVEC(MAXGPB,MAXPLOT)
      INTEGER NGPB
      CHARACTER*(*) OPTION
      LOGICAL SUCCESS
      CHARACTER*240 PRTSTR,STRCOMP,STRING1,STRING2 
      CHARACTER*240 STRCOMPL,STRINGL             
      CHARACTER*10 FORMAT1                       
      CHARACTER*11 FORMAT2                       
      CHARACTER*16 FORMAT3                       
      CHARACTER*17 FORMAT4                       
      INTEGER IC,JC,KC
      INTEGER I,J,K,IFUN,IVAL,IREC,RECLAT,TAPE,IATOM,ICOORD,ICOUNT
      INTEGER IRAD
      INTEGER ALLOCATION
      REAL CLAU,MC2,VEL,X,Y,Z
C
      REAL CE(3),FUNVAL(1),VECTOR(3)
      REAL, ALLOCATABLE:: BUFFER(:,:,:,:)
      CHARACTER*1 PSIFLG
C
C     ------------------------------------------------------------------
C
      IF (OPTION.EQ.'HEADER') THEN
C
        WRITE (PIE,'(A)') 'CUBE FILE GENERATED BY DEMON'
C
C     *** Write time value if RTTDDFT calculation is requested  ***
C
        IF (STATUS.EQ.'RTDFT') THEN
          WRITE (PIE,4999) NPLOTFUN,PLOTFNC,1.E3*RTT/FSEC
        ELSE
          WRITE (PIE,5001) NPLOTFUN,PLOTFNC
        END IF
C
        IF (ADDRAD) NATOM = NATOM + NRADP
C
C     *** Header of plot file (number of atoms and lattice interval) ***
C
        IF (NPLOTFUN.GT.1.OR.(PLOTFNC.EQ.'PSI')) THEN
          WRITE (PIE,5005) -NATOM,(LATTICE(I,0),I=1,3)
        ELSE 
          WRITE (PIE,5005) NATOM,(LATTICE(I,0),I=1,3)
        END IF
C
        IF (ADDRAD) NATOM = NATOM - NRADP
C
C     *** Write step size along lattice vectors ***
C
        DO I=1,3
          WRITE (PIE,5006) NLATTICE(I),(DSTEP(I,J),J=1,3) 
        END DO
C
C     *** Write QM atoms positions ***
C
        WRITE (PIE,5007) (ATOMNO(IATOM),
     $  (COORD(ICOORD,IATOM,CCSET),ICOORD=1,3),IATOM=1,NATOM)
        I = 2
C
C     *** Write radiation particle position ***
C
        IF (ADDRAD) THEN 
          DO IRAD=1,NRADP
            MC2 = MRAD*AMU*(CLIGHT/(ABOHR*FSEC*1.E15))**2
            CLAU = CLIGHT/(ABOHR*FSEC*1.E15)
            VEL = CLAU**2*(1-(KINRAD/MC2 + 1)**(-2))
            CE(:) = X0IORAD(:,IRAD) + RTT*VIORAD(:,IRAD)*SQRT(VEL)
            CALL TRACOO(CE,CE,1,INPTOSTD,3,SHIFT,'SHIFTROTATE')
            WRITE (PIE,5007) I, (CE(ICOORD),ICOORD=1,3)
          END DO
        END IF
C
C     *** Write number of plot functions ***
C
        IF (NPLOTFUN.GT.1.OR.(PLOTFNC.EQ.'PSI')) THEN
          STRING1 = ''
          STRING2 = ''
          STRINGL = ''
          FORMAT1 = '(Y(I4,1X))'
          FORMAT2 = '(YY(I4,1X))'
          FORMAT3 = '(Y(I4,1X,I4,1X))'
          FORMAT4 = '(YY(I4,1X,I4,1X))'
C
          IF (PLOTFNC.EQ.'PSI') THEN
C
C     *** Plot real valued MO numbers into cube file ***
C
            IF (.NOT.RTDFT) THEN
              IF (PLOTMOSA(0).LE.8) THEN
                WRITE(FORMAT1(2:2),'(I1)') PLOTMOSA(0) + 1
                WRITE(STRING1,FORMAT1) (PLOTMOSA(I),I=1,PLOTMOSA(0))
              ELSE IF (PLOTMOSA(0).LE.98) THEN
                WRITE(FORMAT2(2:3),'(I2)') PLOTMOSA(0) + 1
                WRITE(STRING1,FORMAT2) (PLOTMOSA(I),I=1,PLOTMOSA(0))
              END IF
              IF (OKS) THEN
                IF (PLOTMOSB(0).LE.8) THEN
                  WRITE(FORMAT1(2:2),'(I1)') PLOTMOSB(0) + 1
                  WRITE(STRING2,FORMAT1) (PLOTMOSB(I),I=1,PLOTMOSB(0))
                ELSE IF (PLOTMOSB(0).LE.98) THEN
                  WRITE(FORMAT2(2:3),'(I2)') PLOTMOSB(0) + 1
                  WRITE(STRING2,FORMAT2) (PLOTMOSB(I),I=1,PLOTMOSB(0))
                END IF
              END IF
C
C     *** Plot complex valued MO numbers into cube file ***
C
            ELSE
C
              IF (2*PLOTMOSA(0).LE.7) THEN
                WRITE(FORMAT3(2:2),'(I1)') 2*PLOTMOSA(0) + 2
                WRITE(STRING1,FORMAT3) (PLOTMOSA(I),PLOTMOSA(I),
     $                                I=1,PLOTMOSA(0))
              ELSE IF (2*PLOTMOSA(0).LE.97) THEN
                WRITE(FORMAT4(2:3),'(I2)') PLOTMOSA(0) + 2
                WRITE(STRING1,FORMAT4) (PLOTMOSA(I),PLOTMOSA(I),
     $                                I=1,PLOTMOSA(0))
              END IF
              IF (OKS) THEN
                IF (2*PLOTMOSB(0).LE.7) THEN
                  WRITE(FORMAT3(2:2),'(I1)') 2*PLOTMOSB(0) + 2
                  WRITE(STRING2,FORMAT3) (PLOTMOSB(I),PLOTMOSB(I),
     $                                   I=1,PLOTMOSB(0))
                ELSE IF (2*PLOTMOSB(0).LE.97) THEN
                  WRITE(FORMAT4(2:3),'(I2)') 2*PLOTMOSB(0) + 2
                  WRITE(STRING2,FORMAT4) (PLOTMOSB(I),PLOTMOSB(I),
     $                                    I=1,PLOTMOSB(0))
                END IF
              END IF
            END IF
C
C     *** Plot NPLOT function index numbers ***
C
          ELSE 
C
            IF (NPLOTFUN.LE.9) THEN
              WRITE(FORMAT3(2:2),'(I1)')  NPLOTFUN + 1
              WRITE(STRING1,FORMAT3) (I,I=1,NPLOTFUN)
            ELSE IF (NPLOTFUN.LE.99) THEN
               WRITE(FORMAT2(2:3),'(I2)') NPLOTFUN + 1
               WRITE(STRING2,FORMAT2) (I,I=1,NPLOTFUN) 
            END IF
C
          END IF
C
          WRITE(STRINGL,'(I3,X,A,X,A)') NPLOTFUN,TRIM(ADJUSTL(STRING1)),
     $                           TRIM(ADJUSTL(STRING2))
          WRITE(PIE,*) TRIM(ADJUSTL(STRCOMP(STRINGL)))
        END IF
C
C     *** Write function values by block ***
C
      ELSE IF (OPTION.EQ.'BLOCK') THEN
C
        WRITE (PIE,5009) ((FUNVEC(K,IFUN),IFUN=1,NPLOTFUN),K=1,NGPB) 
C
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Format statements ***
C
 4999 FORMAT ('MOLECULAR FIELD : ',I2,2X,A10,2X,' TIME(AS)=',2X,F12.4)
 5000 FORMAT ('PLOT COORDINATES IN ',A)
 5001 FORMAT ('MOLECULAR FIELD : ',I2,2X,A10)
 5005 FORMAT (I4,2X,3(F12.6,X))
 5006 FORMAT ((I4,2X,3(F12.6,X)))
 5007 FORMAT ((I4,2x,(3X,'0.000000',X,3(F12.6,3X))))
 5009 FORMAT ((6(E13.5)))
 5010 FORMAT (/,9X,'X',15X,'Y',15X,'Z',12X,A10)
 5020 FORMAT (4(3X,E13.6))
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE WRTCUBE ***
C
      END

